+{
   locale_version => 1.19,
   entry => <<'ENTRY', # for DUCET v8.0.0
0105      ; [.1BC3.0020.0002] # LATIN SMALL LETTER A WITH OGONEK
0061 0328 ; [.1BC3.0020.0002] # LATIN SMALL LETTER A WITH OGONEK
0104      ; [.1BC3.0020.0008] # LATIN CAPITAL LETTER A WITH OGONEK
0041 0328 ; [.1BC3.0020.0008] # LATIN CAPITAL LETTER A WITH OGONEK
0107      ; [.1BF6.0020.0002] # LATIN SMALL LETTER C WITH ACUTE
0063 0301 ; [.1BF6.0020.0002] # LATIN SMALL LETTER C WITH ACUTE
0063 0341 ; [.1BF6.0020.0002] # LATIN SMALL LETTER C WITH ACUTE
0106      ; [.1BF6.0020.0008] # LATIN CAPITAL LETTER C WITH ACUTE
0043 0301 ; [.1BF6.0020.0008] # LATIN CAPITAL LETTER C WITH ACUTE
0043 0341 ; [.1BF6.0020.0008] # LATIN CAPITAL LETTER C WITH ACUTE
0119      ; [.1C26.0020.0002] # LATIN SMALL LETTER E WITH OGONEK
0065 0328 ; [.1C26.0020.0002] # LATIN SMALL LETTER E WITH OGONEK
0118      ; [.1C26.0020.0008] # LATIN CAPITAL LETTER E WITH OGONEK
0045 0328 ; [.1C26.0020.0008] # LATIN CAPITAL LETTER E WITH OGONEK
0142      ; [.1CF3.0020.0002] # LATIN SMALL LETTER L WITH STROKE
006C 0335 ; [.1CF3.0020.0002] # LATIN SMALL LETTER L WITH STROKE
0141      ; [.1CF3.0020.0008] # LATIN CAPITAL LETTER L WITH STROKE
004C 0335 ; [.1CF3.0020.0008] # LATIN CAPITAL LETTER L WITH STROKE
0144      ; [.1D35.0020.0002] # LATIN SMALL LETTER N WITH ACUTE
006E 0301 ; [.1D35.0020.0002] # LATIN SMALL LETTER N WITH ACUTE
006E 0341 ; [.1D35.0020.0002] # LATIN SMALL LETTER N WITH ACUTE
0143      ; [.1D35.0020.0008] # LATIN CAPITAL LETTER N WITH ACUTE
004E 0301 ; [.1D35.0020.0008] # LATIN CAPITAL LETTER N WITH ACUTE
004E 0341 ; [.1D35.0020.0008] # LATIN CAPITAL LETTER N WITH ACUTE
00F3      ; [.1D59.0020.0002] # LATIN SMALL LETTER O WITH ACUTE
006F 0301 ; [.1D59.0020.0002] # LATIN SMALL LETTER O WITH ACUTE
006F 0341 ; [.1D59.0020.0002] # LATIN SMALL LETTER O WITH ACUTE
00D3      ; [.1D59.0020.0008] # LATIN CAPITAL LETTER O WITH ACUTE
004F 0301 ; [.1D59.0020.0008] # LATIN CAPITAL LETTER O WITH ACUTE
004F 0341 ; [.1D59.0020.0008] # LATIN CAPITAL LETTER O WITH ACUTE
015B      ; [.1DED.0020.0002] # LATIN SMALL LETTER S WITH ACUTE
0073 0301 ; [.1DED.0020.0002] # LATIN SMALL LETTER S WITH ACUTE
0073 0341 ; [.1DED.0020.0002] # LATIN SMALL LETTER S WITH ACUTE
015A      ; [.1DED.0020.0008] # LATIN CAPITAL LETTER S WITH ACUTE
0053 0301 ; [.1DED.0020.0008] # LATIN CAPITAL LETTER S WITH ACUTE
0053 0341 ; [.1DED.0020.0008] # LATIN CAPITAL LETTER S WITH ACUTE
017A      ; [.1E9D.0020.0002] # LATIN SMALL LETTER Z WITH ACUTE
007A 0301 ; [.1E9D.0020.0002] # LATIN SMALL LETTER Z WITH ACUTE
007A 0341 ; [.1E9D.0020.0002] # LATIN SMALL LETTER Z WITH ACUTE
0179      ; [.1E9D.0020.0008] # LATIN CAPITAL LETTER Z WITH ACUTE
005A 0301 ; [.1E9D.0020.0008] # LATIN CAPITAL LETTER Z WITH ACUTE
005A 0341 ; [.1E9D.0020.0008] # LATIN CAPITAL LETTER Z WITH ACUTE
017C      ; [.1E9E.0020.0002] # LATIN SMALL LETTER Z WITH DOT ABOVE
007A 0307 ; [.1E9E.0020.0002] # LATIN SMALL LETTER Z WITH DOT ABOVE
017B      ; [.1E9E.0020.0008] # LATIN CAPITAL LETTER Z WITH DOT ABOVE
005A 0307 ; [.1E9E.0020.0008] # LATIN CAPITAL LETTER Z WITH DOT ABOVE
ENTRY
};
