+{
   locale_version => 1.19,
   entry => <<'ENTRY', # for DUCET v8.0.0
0451      ; [.1FD6.0020.0002] # CYRILLIC SMALL LETTER IO
0435 0308 ; [.1FD6.0020.0002] # CYRILLIC SMALL LETTER IO
0401      ; [.1FD6.0020.0008] # CYRILLIC CAPITAL LETTER IO
0415 0308 ; [.1FD6.0020.0008] # CYRILLIC CAPITAL LETTER IO
04AF      ; [.20A1.0020.0002] # CYRILLIC SMALL LETTER STRAIGHT U
04AE      ; [.20A1.0020.0008] # CYRILLIC CAPITAL LETTER STRAIGHT U
0456      ; [.2112.0020.0002] # CYRILLIC SMALL LETTER BYELORUSSIAN-UKRAINIAN I
0406      ; [.2112.0020.0008] # CYRILLIC CAPITAL LETTER BYELORUSSIAN-UKRAINIAN I
ENTRY
};
