+{
   locale_version => 1.19,
   entry => <<'ENTRY', # for DUCET v8.0.0
0063 0068 ; [.1BDC.0020.0002] # <LATIN SMALL LETTER C, LATIN SMALL LETTER H>
0043 0068 ; [.1BDC.0020.0007] # <LATIN CAPITAL LETTER C, LATIN SMALL LETTER H>
0043 0048 ; [.1BDC.0020.0008] # <LATIN CAPITAL LETTER C, LATIN CAPITAL LETTER H>
0067 0062 ; [.1C70.0020.0002] # <LATIN SMALL LETTER G, LATIN SMALL LETTER B>
0047 0062 ; [.1C70.0020.0007] # <LATIN CAPITAL LETTER G, LATIN SMALL LETTER B>
0047 0042 ; [.1C70.0020.0008] # <LATIN CAPITAL LETTER G, LATIN CAPITAL LETTER B>
0067 0068 ; [.1C71.0020.0002] # <LATIN SMALL LETTER G, LATIN SMALL LETTER H>
0047 0068 ; [.1C71.0020.0007] # <LATIN CAPITAL LETTER G, LATIN SMALL LETTER H>
0047 0048 ; [.1C71.0020.0008] # <LATIN CAPITAL LETTER G, LATIN CAPITAL LETTER H>
0067 0077 ; [.1C72.0020.0002] # <LATIN SMALL LETTER G, LATIN SMALL LETTER W>
0047 0077 ; [.1C72.0020.0007] # <LATIN CAPITAL LETTER G, LATIN SMALL LETTER W>
0047 0057 ; [.1C72.0020.0008] # <LATIN CAPITAL LETTER G, LATIN CAPITAL LETTER W>
1ECB      ; [.1CAE.0020.0002] # LATIN SMALL LETTER I WITH DOT BELOW
0069 0323 ; [.1CAE.0020.0002] # LATIN SMALL LETTER I WITH DOT BELOW
1ECA      ; [.1CAE.0020.0008] # LATIN CAPITAL LETTER I WITH DOT BELOW
0049 0323 ; [.1CAE.0020.0008] # LATIN CAPITAL LETTER I WITH DOT BELOW
006B 0070 ; [.1CE1.0020.0002] # <LATIN SMALL LETTER K, LATIN SMALL LETTER P>
004B 0070 ; [.1CE1.0020.0007] # <LATIN CAPITAL LETTER K, LATIN SMALL LETTER P>
004B 0050 ; [.1CE1.0020.0008] # <LATIN CAPITAL LETTER K, LATIN CAPITAL LETTER P>
006B 0077 ; [.1CE2.0020.0002] # <LATIN SMALL LETTER K, LATIN SMALL LETTER W>
004B 0077 ; [.1CE2.0020.0007] # <LATIN CAPITAL LETTER K, LATIN SMALL LETTER W>
004B 0057 ; [.1CE2.0020.0008] # <LATIN CAPITAL LETTER K, LATIN CAPITAL LETTER W>
1E45      ; [.1D35.0020.0002] # LATIN SMALL LETTER N WITH DOT ABOVE
006E 0307 ; [.1D35.0020.0002] # LATIN SMALL LETTER N WITH DOT ABOVE
1E44      ; [.1D35.0020.0008] # LATIN CAPITAL LETTER N WITH DOT ABOVE
004E 0307 ; [.1D35.0020.0008] # LATIN CAPITAL LETTER N WITH DOT ABOVE
006E 0077 ; [.1D36.0020.0002] # <LATIN SMALL LETTER N, LATIN SMALL LETTER W>
004E 0077 ; [.1D36.0020.0007] # <LATIN CAPITAL LETTER N, LATIN SMALL LETTER W>
004E 0057 ; [.1D36.0020.0008] # <LATIN CAPITAL LETTER N, LATIN CAPITAL LETTER W>
006E 0079 ; [.1D37.0020.0002] # <LATIN SMALL LETTER N, LATIN SMALL LETTER Y>
004E 0079 ; [.1D37.0020.0007] # <LATIN CAPITAL LETTER N, LATIN SMALL LETTER Y>
004E 0059 ; [.1D37.0020.0008] # <LATIN CAPITAL LETTER N, LATIN CAPITAL LETTER Y>
1ECD      ; [.1D59.0020.0002] # LATIN SMALL LETTER O WITH DOT BELOW
006F 0323 ; [.1D59.0020.0002] # LATIN SMALL LETTER O WITH DOT BELOW
1ECC      ; [.1D59.0020.0008] # LATIN CAPITAL LETTER O WITH DOT BELOW
004F 0323 ; [.1D59.0020.0008] # LATIN CAPITAL LETTER O WITH DOT BELOW
1ED9      ; [.1D59.0020.0002][.0000.0027.0002] # LATIN SMALL LETTER O WITH CIRCUMFLEX AND DOT BELOW
1ED8      ; [.1D59.0020.0008][.0000.0027.0002] # LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND DOT BELOW
1EE3      ; [.1D59.0020.0002][.0000.003F.0002] # LATIN SMALL LETTER O WITH HORN AND DOT BELOW
1EE2      ; [.1D59.0020.0008][.0000.003F.0002] # LATIN CAPITAL LETTER O WITH HORN AND DOT BELOW
0073 0068 ; [.1DED.0020.0002] # <LATIN SMALL LETTER S, LATIN SMALL LETTER H>
0053 0068 ; [.1DED.0020.0007] # <LATIN CAPITAL LETTER S, LATIN SMALL LETTER H>
0053 0048 ; [.1DED.0020.0008] # <LATIN CAPITAL LETTER S, LATIN CAPITAL LETTER H>
1EE5      ; [.1E31.0020.0002] # LATIN SMALL LETTER U WITH DOT BELOW
0075 0323 ; [.1E31.0020.0002] # LATIN SMALL LETTER U WITH DOT BELOW
1EE4      ; [.1E31.0020.0008] # LATIN CAPITAL LETTER U WITH DOT BELOW
0055 0323 ; [.1E31.0020.0008] # LATIN CAPITAL LETTER U WITH DOT BELOW
1EF1      ; [.1E31.0020.0002][.0000.003F.0002] # LATIN SMALL LETTER U WITH HORN AND DOT BELOW
1EF0      ; [.1E31.0020.0008][.0000.003F.0002] # LATIN CAPITAL LETTER U WITH HORN AND DOT BELOW
ENTRY
};
