+{
   locale_version => 1.19,
   entry => <<'ENTRY', # for DUCET v8.0.0
006E 0067 ; [.1D35.0020.0002] # <LATIN SMALL LETTER N, LATIN SMALL LETTER G>
004E 0067 ; [.1D35.0020.0007] # <LATIN CAPITAL LETTER N, LATIN SMALL LETTER G>
004E 0047 ; [.1D35.0020.0008] # <LATIN CAPITAL LETTER N, LATIN CAPITAL LETTER G>
014B      ; [.1D35.0020.001C] # LATIN SMALL LETTER ENG
014A      ; [.1D35.0020.001D] # LATIN CAPITAL LETTER ENG
02BB      ; [.1E9D.0020.0002] # MODIFIER LETTER TURNED COMMA
02BD      ; [.1E9D.0020.0003] # MODIFIER LETTER REVERSED COMMA
00E1      ; [.1BC2.0021.0002] # LATIN SMALL LETTER A WITH ACUTE
0061 0301 ; [.1BC2.0021.0002] # LATIN SMALL LETTER A WITH ACUTE
0061 0341 ; [.1BC2.0021.0002] # LATIN SMALL LETTER A WITH ACUTE
00C1      ; [.1BC2.0021.0008] # LATIN CAPITAL LETTER A WITH ACUTE
0041 0301 ; [.1BC2.0021.0008] # LATIN CAPITAL LETTER A WITH ACUTE
0041 0341 ; [.1BC2.0021.0008] # LATIN CAPITAL LETTER A WITH ACUTE
0101      ; [.1BC2.0022.0002] # LATIN SMALL LETTER A WITH MACRON
0061 0304 ; [.1BC2.0022.0002] # LATIN SMALL LETTER A WITH MACRON
0100      ; [.1BC2.0022.0008] # LATIN CAPITAL LETTER A WITH MACRON
0041 0304 ; [.1BC2.0022.0008] # LATIN CAPITAL LETTER A WITH MACRON
00E9      ; [.1C25.0021.0002] # LATIN SMALL LETTER E WITH ACUTE
0065 0301 ; [.1C25.0021.0002] # LATIN SMALL LETTER E WITH ACUTE
0065 0341 ; [.1C25.0021.0002] # LATIN SMALL LETTER E WITH ACUTE
00C9      ; [.1C25.0021.0008] # LATIN CAPITAL LETTER E WITH ACUTE
0045 0301 ; [.1C25.0021.0008] # LATIN CAPITAL LETTER E WITH ACUTE
0045 0341 ; [.1C25.0021.0008] # LATIN CAPITAL LETTER E WITH ACUTE
0113      ; [.1C25.0022.0002] # LATIN SMALL LETTER E WITH MACRON
0065 0304 ; [.1C25.0022.0002] # LATIN SMALL LETTER E WITH MACRON
0112      ; [.1C25.0022.0008] # LATIN CAPITAL LETTER E WITH MACRON
0045 0304 ; [.1C25.0022.0008] # LATIN CAPITAL LETTER E WITH MACRON
00ED      ; [.1CAD.0021.0002] # LATIN SMALL LETTER I WITH ACUTE
0069 0301 ; [.1CAD.0021.0002] # LATIN SMALL LETTER I WITH ACUTE
0069 0341 ; [.1CAD.0021.0002] # LATIN SMALL LETTER I WITH ACUTE
00CD      ; [.1CAD.0021.0008] # LATIN CAPITAL LETTER I WITH ACUTE
0049 0301 ; [.1CAD.0021.0008] # LATIN CAPITAL LETTER I WITH ACUTE
0049 0341 ; [.1CAD.0021.0008] # LATIN CAPITAL LETTER I WITH ACUTE
012B      ; [.1CAD.0022.0002] # LATIN SMALL LETTER I WITH MACRON
0069 0304 ; [.1CAD.0022.0002] # LATIN SMALL LETTER I WITH MACRON
012A      ; [.1CAD.0022.0008] # LATIN CAPITAL LETTER I WITH MACRON
0049 0304 ; [.1CAD.0022.0008] # LATIN CAPITAL LETTER I WITH MACRON
00F3      ; [.1D58.0021.0002] # LATIN SMALL LETTER O WITH ACUTE
006F 0301 ; [.1D58.0021.0002] # LATIN SMALL LETTER O WITH ACUTE
006F 0341 ; [.1D58.0021.0002] # LATIN SMALL LETTER O WITH ACUTE
00D3      ; [.1D58.0021.0008] # LATIN CAPITAL LETTER O WITH ACUTE
004F 0301 ; [.1D58.0021.0008] # LATIN CAPITAL LETTER O WITH ACUTE
004F 0341 ; [.1D58.0021.0008] # LATIN CAPITAL LETTER O WITH ACUTE
014D      ; [.1D58.0022.0002] # LATIN SMALL LETTER O WITH MACRON
006F 0304 ; [.1D58.0022.0002] # LATIN SMALL LETTER O WITH MACRON
014C      ; [.1D58.0022.0008] # LATIN CAPITAL LETTER O WITH MACRON
004F 0304 ; [.1D58.0022.0008] # LATIN CAPITAL LETTER O WITH MACRON
00FA      ; [.1E30.0021.0002] # LATIN SMALL LETTER U WITH ACUTE
0075 0301 ; [.1E30.0021.0002] # LATIN SMALL LETTER U WITH ACUTE
0075 0341 ; [.1E30.0021.0002] # LATIN SMALL LETTER U WITH ACUTE
00DA      ; [.1E30.0021.0008] # LATIN CAPITAL LETTER U WITH ACUTE
0055 0301 ; [.1E30.0021.0008] # LATIN CAPITAL LETTER U WITH ACUTE
0055 0341 ; [.1E30.0021.0008] # LATIN CAPITAL LETTER U WITH ACUTE
016B      ; [.1E30.0022.0002] # LATIN SMALL LETTER U WITH MACRON
0075 0304 ; [.1E30.0022.0002] # LATIN SMALL LETTER U WITH MACRON
016A      ; [.1E30.0022.0008] # LATIN CAPITAL LETTER U WITH MACRON
0055 0304 ; [.1E30.0022.0008] # LATIN CAPITAL LETTER U WITH MACRON
ENTRY
};
