+{
   locale_version => 1.19,
   entry => <<'ENTRY', # for DUCET v8.0.0
00E7      ; [.1C09.0020.0002] # LATIN SMALL LETTER C WITH CEDILLA
0063 0327 ; [.1C09.0020.0002] # LATIN SMALL LETTER C WITH CEDILLA
00C7      ; [.1C09.0020.0008] # LATIN CAPITAL LETTER C WITH CEDILLA
0043 0327 ; [.1C09.0020.0008] # LATIN CAPITAL LETTER C WITH CEDILLA
0064 0068 ; [.1C24.0020.0002] # <LATIN SMALL LETTER D, LATIN SMALL LETTER H>
0064 0048 ; [.1C24.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER D, LATIN CAPITAL LETTER H>
0044 0068 ; [.1C24.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER D, LATIN SMALL LETTER H>
0044 0048 ; [.1C24.0020.0008] # <LATIN CAPITAL LETTER D, LATIN CAPITAL LETTER H>
00EB      ; [.1C5F.0020.0002] # LATIN SMALL LETTER E WITH DIAERESIS
0065 0308 ; [.1C5F.0020.0002] # LATIN SMALL LETTER E WITH DIAERESIS
00CB      ; [.1C5F.0020.0008] # LATIN CAPITAL LETTER E WITH DIAERESIS
0045 0308 ; [.1C5F.0020.0008] # LATIN CAPITAL LETTER E WITH DIAERESIS
0067 006A ; [.1C92.0020.0002] # <LATIN SMALL LETTER G, LATIN SMALL LETTER J>
0067 004A ; [.1C92.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER G, LATIN CAPITAL LETTER J>
0047 006A ; [.1C92.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER G, LATIN SMALL LETTER J>
0047 004A ; [.1C92.0020.0008] # <LATIN CAPITAL LETTER G, LATIN CAPITAL LETTER J>
006C 006C ; [.1D24.0020.0002] # <LATIN SMALL LETTER L, LATIN SMALL LETTER L>
006C 004C ; [.1D24.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER L, LATIN CAPITAL LETTER L>
004C 006C ; [.1D24.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER L, LATIN SMALL LETTER L>
004C 004C ; [.1D24.0020.0008] # <LATIN CAPITAL LETTER L, LATIN CAPITAL LETTER L>
006E 006A ; [.1D57.0020.0002] # <LATIN SMALL LETTER N, LATIN SMALL LETTER J>
006E 004A ; [.1D57.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER N, LATIN CAPITAL LETTER J>
004E 006A ; [.1D57.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER N, LATIN SMALL LETTER J>
004E 004A ; [.1D57.0020.0008] # <LATIN CAPITAL LETTER N, LATIN CAPITAL LETTER J>
0072 0072 ; [.1DEB.0020.0002] # <LATIN SMALL LETTER R, LATIN SMALL LETTER R>
0072 0052 ; [.1DEB.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER R, LATIN CAPITAL LETTER R>
0052 0072 ; [.1DEB.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER R, LATIN SMALL LETTER R>
0052 0052 ; [.1DEB.0020.0008] # <LATIN CAPITAL LETTER R, LATIN CAPITAL LETTER R>
0073 0068 ; [.1E0F.0020.0002] # <LATIN SMALL LETTER S, LATIN SMALL LETTER H>
0073 0048 ; [.1E0F.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER S, LATIN CAPITAL LETTER H>
0053 0068 ; [.1E0F.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER S, LATIN SMALL LETTER H>
0053 0048 ; [.1E0F.0020.0008] # <LATIN CAPITAL LETTER S, LATIN CAPITAL LETTER H>
0074 0068 ; [.1E2F.0020.0002] # <LATIN SMALL LETTER T, LATIN SMALL LETTER H>
0074 0048 ; [.1E2F.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER T, LATIN CAPITAL LETTER H>
0054 0068 ; [.1E2F.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER T, LATIN SMALL LETTER H>
0054 0048 ; [.1E2F.0020.0008] # <LATIN CAPITAL LETTER T, LATIN CAPITAL LETTER H>
0078 0068 ; [.1E85.0020.0002] # <LATIN SMALL LETTER X, LATIN SMALL LETTER H>
0078 0048 ; [.1E85.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER X, LATIN CAPITAL LETTER H>
0058 0068 ; [.1E85.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER X, LATIN SMALL LETTER H>
0058 0048 ; [.1E85.0020.0008] # <LATIN CAPITAL LETTER X, LATIN CAPITAL LETTER H>
007A 0068 ; [.1EB8.0020.0002] # <LATIN SMALL LETTER Z, LATIN SMALL LETTER H>
007A 0048 ; [.1EB8.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER Z, LATIN CAPITAL LETTER H>
005A 0068 ; [.1EB8.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER Z, LATIN SMALL LETTER H>
005A 0048 ; [.1EB8.0020.0008] # <LATIN CAPITAL LETTER Z, LATIN CAPITAL LETTER H>
ENTRY
};
