package #
Locale::Codes::LangExt_Retired;

# This file was automatically generated.  Any changes to this file will
# be lost the next time 'deprecate_codes' is run.
#    Generated on: Fri Nov 18 09:22:54 EST 2016

use strict;
require 5.006;
use warnings;
use utf8;

our($VERSION);
$VERSION='3.42';

$Locale::Codes::Retired{'langext'}{'alpha'}{'code'} = {
   q(yds) => q(Yiddish Sign Language),
};

$Locale::Codes::Retired{'langext'}{'alpha'}{'name'} = {
   q(hawai'i pidgin sign language) => [ q(hps), q(Hawai'i Pidgin Sign Language) ],
   q(yiddish sign language) => [ q(yds), q(Yiddish Sign Language) ],
};


1;
