/* -*- buffer-read-only: t -*-
 * !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
 * This file is built by regen/mk_invlists.pl from Unicode::UCD.
 * Any changes made here will be lost!
 */

/* See the generating file for comments */


#if defined(PERL_IN_REGCOMP_C)

static const UV Latin1_invlist[] = {
	2,	/* Number of elements */
	148565664, /* Version and data structure type */
	0,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x100
};

static const UV AboveLatin1_invlist[] = {
	2,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x100
};

#endif	/* defined(PERL_IN_REGCOMP_C) */

#if 'A' == 65 /* ASCII/Latin1 */

#if defined(PERL_IN_PERL_C)

static const UV ASCII_invlist[] = { /* for ASCII/Latin1 */
	2,	/* Number of elements */
	148565664, /* Version and data structure type */
	0,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x80
};

static const UV Assigned_invlist[] = { /* for ASCII/Latin1 */
	1276,	/* Number of elements */
	148565664, /* Version and data structure type */
	0,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x378,
	0x37A,
	0x380,
	0x384,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x530,
	0x531,
	0x557,
	0x559,
	0x560,
	0x561,
	0x588,
	0x589,
	0x58B,
	0x58D,
	0x590,
	0x591,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F5,
	0x600,
	0x61D,
	0x61E,
	0x70E,
	0x70F,
	0x74B,
	0x74D,
	0x7B2,
	0x7C0,
	0x7FB,
	0x800,
	0x82E,
	0x830,
	0x83F,
	0x840,
	0x85C,
	0x85E,
	0x85F,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x8D4,
	0x984,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BC,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CF,
	0x9D7,
	0x9D8,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E4,
	0x9E6,
	0x9FC,
	0xA01,
	0xA04,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA3C,
	0xA3D,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4E,
	0xA51,
	0xA52,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA66,
	0xA76,
	0xA81,
	0xA84,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABC,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACE,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE4,
	0xAE6,
	0xAF2,
	0xAF9,
	0xAFA,
	0xB01,
	0xB04,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3C,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4E,
	0xB56,
	0xB58,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB64,
	0xB66,
	0xB78,
	0xB82,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCE,
	0xBD0,
	0xBD1,
	0xBD7,
	0xBD8,
	0xBE6,
	0xBFB,
	0xC00,
	0xC04,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4E,
	0xC55,
	0xC57,
	0xC58,
	0xC5B,
	0xC60,
	0xC64,
	0xC66,
	0xC70,
	0xC78,
	0xC84,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBC,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCE,
	0xCD5,
	0xCD7,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE4,
	0xCE6,
	0xCF0,
	0xCF1,
	0xCF3,
	0xD01,
	0xD04,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD50,
	0xD54,
	0xD64,
	0xD66,
	0xD80,
	0xD82,
	0xD84,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xDCA,
	0xDCB,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDE6,
	0xDF0,
	0xDF2,
	0xDF5,
	0xE01,
	0xE3B,
	0xE3F,
	0xE5C,
	0xE81,
	0xE83,
	0xE84,
	0xE85,
	0xE87,
	0xE89,
	0xE8A,
	0xE8B,
	0xE8D,
	0xE8E,
	0xE94,
	0xE98,
	0xE99,
	0xEA0,
	0xEA1,
	0xEA4,
	0xEA5,
	0xEA6,
	0xEA7,
	0xEA8,
	0xEAA,
	0xEAC,
	0xEAD,
	0xEBA,
	0xEBB,
	0xEBE,
	0xEC0,
	0xEC5,
	0xEC6,
	0xEC7,
	0xEC8,
	0xECE,
	0xED0,
	0xEDA,
	0xEDC,
	0xEE0,
	0xF00,
	0xF48,
	0xF49,
	0xF6D,
	0xF71,
	0xF98,
	0xF99,
	0xFBD,
	0xFBE,
	0xFCD,
	0xFCE,
	0xFDB,
	0x1000,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x135D,
	0x137D,
	0x1380,
	0x139A,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1400,
	0x169D,
	0x16A0,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1715,
	0x1720,
	0x1737,
	0x1740,
	0x1754,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1772,
	0x1774,
	0x1780,
	0x17DE,
	0x17E0,
	0x17EA,
	0x17F0,
	0x17FA,
	0x1800,
	0x180F,
	0x1810,
	0x181A,
	0x1820,
	0x1878,
	0x1880,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1920,
	0x192C,
	0x1930,
	0x193C,
	0x1940,
	0x1941,
	0x1944,
	0x196E,
	0x1970,
	0x1975,
	0x1980,
	0x19AC,
	0x19B0,
	0x19CA,
	0x19D0,
	0x19DB,
	0x19DE,
	0x1A1C,
	0x1A1E,
	0x1A5F,
	0x1A60,
	0x1A7D,
	0x1A7F,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1AA0,
	0x1AAE,
	0x1AB0,
	0x1ABF,
	0x1B00,
	0x1B4C,
	0x1B50,
	0x1B7D,
	0x1B80,
	0x1BF4,
	0x1BFC,
	0x1C38,
	0x1C3B,
	0x1C4A,
	0x1C4D,
	0x1C89,
	0x1CC0,
	0x1CC8,
	0x1CD0,
	0x1CF7,
	0x1CF8,
	0x1CFA,
	0x1D00,
	0x1DF6,
	0x1DFB,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FC5,
	0x1FC6,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FDD,
	0x1FF0,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFF,
	0x2000,
	0x2065,
	0x2066,
	0x2072,
	0x2074,
	0x208F,
	0x2090,
	0x209D,
	0x20A0,
	0x20BF,
	0x20D0,
	0x20F1,
	0x2100,
	0x218C,
	0x2190,
	0x23FF,
	0x2400,
	0x2427,
	0x2440,
	0x244B,
	0x2460,
	0x2B74,
	0x2B76,
	0x2B96,
	0x2B98,
	0x2BBA,
	0x2BBD,
	0x2BC9,
	0x2BCA,
	0x2BD2,
	0x2BEC,
	0x2BF0,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CF4,
	0x2CF9,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D71,
	0x2D7F,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x2DE0,
	0x2E45,
	0x2E80,
	0x2E9A,
	0x2E9B,
	0x2EF4,
	0x2F00,
	0x2FD6,
	0x2FF0,
	0x2FFC,
	0x3000,
	0x3040,
	0x3041,
	0x3097,
	0x3099,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x3190,
	0x31BB,
	0x31C0,
	0x31E4,
	0x31F0,
	0x321F,
	0x3220,
	0x32FF,
	0x3300,
	0x4DB6,
	0x4DC0,
	0x9FD6,
	0xA000,
	0xA48D,
	0xA490,
	0xA4C7,
	0xA4D0,
	0xA62C,
	0xA640,
	0xA6F8,
	0xA700,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F7,
	0xA82C,
	0xA830,
	0xA83A,
	0xA840,
	0xA878,
	0xA880,
	0xA8C6,
	0xA8CE,
	0xA8DA,
	0xA8E0,
	0xA8FE,
	0xA900,
	0xA954,
	0xA95F,
	0xA97D,
	0xA980,
	0xA9CE,
	0xA9CF,
	0xA9DA,
	0xA9DE,
	0xA9FF,
	0xAA00,
	0xAA37,
	0xAA40,
	0xAA4E,
	0xAA50,
	0xAA5A,
	0xAA5C,
	0xAAC3,
	0xAADB,
	0xAAF7,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB66,
	0xAB70,
	0xABEE,
	0xABF0,
	0xABFA,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xD800,
	0xFA6E,
	0xFA70,
	0xFADA,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFBC2,
	0xFBD3,
	0xFD40,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFE,
	0xFE00,
	0xFE1A,
	0xFE20,
	0xFE53,
	0xFE54,
	0xFE67,
	0xFE68,
	0xFE6C,
	0xFE70,
	0xFE75,
	0xFE76,
	0xFEFD,
	0xFEFF,
	0xFF00,
	0xFF01,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0xFFE0,
	0xFFE7,
	0xFFE8,
	0xFFEF,
	0xFFF9,
	0xFFFE,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10100,
	0x10103,
	0x10107,
	0x10134,
	0x10137,
	0x1018F,
	0x10190,
	0x1019C,
	0x101A0,
	0x101A1,
	0x101D0,
	0x101FE,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x102E0,
	0x102FC,
	0x10300,
	0x10324,
	0x10330,
	0x1034B,
	0x10350,
	0x1037B,
	0x10380,
	0x1039E,
	0x1039F,
	0x103C4,
	0x103C8,
	0x103D6,
	0x10400,
	0x1049E,
	0x104A0,
	0x104AA,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x1056F,
	0x10570,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10857,
	0x1089F,
	0x108A7,
	0x108B0,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x108FB,
	0x1091C,
	0x1091F,
	0x1093A,
	0x1093F,
	0x10940,
	0x10980,
	0x109B8,
	0x109BC,
	0x109D0,
	0x109D2,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A38,
	0x10A3B,
	0x10A3F,
	0x10A48,
	0x10A50,
	0x10A59,
	0x10A60,
	0x10AA0,
	0x10AC0,
	0x10AE7,
	0x10AEB,
	0x10AF7,
	0x10B00,
	0x10B36,
	0x10B39,
	0x10B56,
	0x10B58,
	0x10B73,
	0x10B78,
	0x10B92,
	0x10B99,
	0x10B9D,
	0x10BA9,
	0x10BB0,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x10CFA,
	0x10D00,
	0x10E60,
	0x10E7F,
	0x11000,
	0x1104E,
	0x11052,
	0x11070,
	0x1107F,
	0x110C2,
	0x110D0,
	0x110E9,
	0x110F0,
	0x110FA,
	0x11100,
	0x11135,
	0x11136,
	0x11144,
	0x11150,
	0x11177,
	0x11180,
	0x111CE,
	0x111D0,
	0x111E0,
	0x111E1,
	0x111F5,
	0x11200,
	0x11212,
	0x11213,
	0x1123F,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112AA,
	0x112B0,
	0x112EB,
	0x112F0,
	0x112FA,
	0x11300,
	0x11304,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133C,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134E,
	0x11350,
	0x11351,
	0x11357,
	0x11358,
	0x1135D,
	0x11364,
	0x11366,
	0x1136D,
	0x11370,
	0x11375,
	0x11400,
	0x1145A,
	0x1145B,
	0x1145C,
	0x1145D,
	0x1145E,
	0x11480,
	0x114C8,
	0x114D0,
	0x114DA,
	0x11580,
	0x115B6,
	0x115B8,
	0x115DE,
	0x11600,
	0x11645,
	0x11650,
	0x1165A,
	0x11660,
	0x1166D,
	0x11680,
	0x116B8,
	0x116C0,
	0x116CA,
	0x11700,
	0x1171A,
	0x1171D,
	0x1172C,
	0x11730,
	0x11740,
	0x118A0,
	0x118F3,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C37,
	0x11C38,
	0x11C46,
	0x11C50,
	0x11C6D,
	0x11C70,
	0x11C90,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12470,
	0x12475,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16A60,
	0x16A6A,
	0x16A6E,
	0x16A70,
	0x16AD0,
	0x16AEE,
	0x16AF0,
	0x16AF6,
	0x16B00,
	0x16B46,
	0x16B50,
	0x16B5A,
	0x16B5B,
	0x16B62,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F7F,
	0x16F8F,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1BC9C,
	0x1BCA4,
	0x1D000,
	0x1D0F6,
	0x1D100,
	0x1D127,
	0x1D129,
	0x1D1E9,
	0x1D200,
	0x1D246,
	0x1D300,
	0x1D357,
	0x1D360,
	0x1D372,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D7CC,
	0x1D7CE,
	0x1DA8C,
	0x1DA9B,
	0x1DAA0,
	0x1DAA1,
	0x1DAB0,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E800,
	0x1E8C5,
	0x1E8C7,
	0x1E8D7,
	0x1E900,
	0x1E94B,
	0x1E950,
	0x1E95A,
	0x1E95E,
	0x1E960,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x1EEF0,
	0x1EEF2,
	0x1F000,
	0x1F02C,
	0x1F030,
	0x1F094,
	0x1F0A0,
	0x1F0AF,
	0x1F0B1,
	0x1F0C0,
	0x1F0C1,
	0x1F0D0,
	0x1F0D1,
	0x1F0F6,
	0x1F100,
	0x1F10D,
	0x1F110,
	0x1F12F,
	0x1F130,
	0x1F16C,
	0x1F170,
	0x1F1AD,
	0x1F1E6,
	0x1F203,
	0x1F210,
	0x1F23C,
	0x1F240,
	0x1F249,
	0x1F250,
	0x1F252,
	0x1F300,
	0x1F6D3,
	0x1F6E0,
	0x1F6ED,
	0x1F6F0,
	0x1F6F7,
	0x1F700,
	0x1F774,
	0x1F780,
	0x1F7D5,
	0x1F800,
	0x1F80C,
	0x1F810,
	0x1F848,
	0x1F850,
	0x1F85A,
	0x1F860,
	0x1F888,
	0x1F890,
	0x1F8AE,
	0x1F910,
	0x1F91F,
	0x1F920,
	0x1F928,
	0x1F930,
	0x1F931,
	0x1F933,
	0x1F93F,
	0x1F940,
	0x1F94C,
	0x1F950,
	0x1F95F,
	0x1F980,
	0x1F992,
	0x1F9C0,
	0x1F9C1,
	0x20000,
	0x2A6D7,
	0x2A700,
	0x2B735,
	0x2B740,
	0x2B81E,
	0x2B820,
	0x2CEA2,
	0x2F800,
	0x2FA1E,
	0xE0001,
	0xE0002,
	0xE0020,
	0xE0080,
	0xE0100,
	0xE01F0,
	0xF0000,
	0xFFFFE,
	0x100000,
	0x10FFFE
};

static const UV Cased_invlist[] = { /* for ASCII/Latin1 */
	271,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x41,
	0x5B,
	0x61,
	0x7B,
	0xAA,
	0xAB,
	0xB5,
	0xB6,
	0xBA,
	0xBB,
	0xC0,
	0xD7,
	0xD8,
	0xF7,
	0xF8,
	0x1BB,
	0x1BC,
	0x1C0,
	0x1C4,
	0x294,
	0x295,
	0x2B9,
	0x2C0,
	0x2C2,
	0x2E0,
	0x2E5,
	0x345,
	0x346,
	0x370,
	0x374,
	0x376,
	0x378,
	0x37A,
	0x37E,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x3F6,
	0x3F7,
	0x482,
	0x48A,
	0x530,
	0x531,
	0x557,
	0x561,
	0x588,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1C80,
	0x1C89,
	0x1D00,
	0x1DC0,
	0x1E00,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FBD,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FCD,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FE0,
	0x1FED,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFD,
	0x2071,
	0x2072,
	0x207F,
	0x2080,
	0x2090,
	0x209D,
	0x2102,
	0x2103,
	0x2107,
	0x2108,
	0x210A,
	0x2114,
	0x2115,
	0x2116,
	0x2119,
	0x211E,
	0x2124,
	0x2125,
	0x2126,
	0x2127,
	0x2128,
	0x2129,
	0x212A,
	0x212E,
	0x212F,
	0x2135,
	0x2139,
	0x213A,
	0x213C,
	0x2140,
	0x2145,
	0x214A,
	0x214E,
	0x214F,
	0x2160,
	0x2180,
	0x2183,
	0x2185,
	0x24B6,
	0x24EA,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CE5,
	0x2CEB,
	0x2CEF,
	0x2CF2,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0xA640,
	0xA66E,
	0xA680,
	0xA69E,
	0xA722,
	0xA788,
	0xA78B,
	0xA78F,
	0xA790,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F8,
	0xA7FB,
	0xAB30,
	0xAB5B,
	0xAB5C,
	0xAB66,
	0xAB70,
	0xABC0,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFF21,
	0xFF3B,
	0xFF41,
	0xFF5B,
	0x10400,
	0x10450,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x118A0,
	0x118E0,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D6C1,
	0x1D6C2,
	0x1D6DB,
	0x1D6DC,
	0x1D6FB,
	0x1D6FC,
	0x1D715,
	0x1D716,
	0x1D735,
	0x1D736,
	0x1D74F,
	0x1D750,
	0x1D76F,
	0x1D770,
	0x1D789,
	0x1D78A,
	0x1D7A9,
	0x1D7AA,
	0x1D7C3,
	0x1D7C4,
	0x1D7CC,
	0x1E900,
	0x1E944,
	0x1F130,
	0x1F14A,
	0x1F150,
	0x1F16A,
	0x1F170,
	0x1F18A
};

#endif	/* defined(PERL_IN_PERL_C) */

#if defined(PERL_IN_REGCOMP_C)

static const UV NonL1_Perl_Non_Final_Folds_invlist[] = { /* for ASCII/Latin1 */
	45,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x2BC,
	0x2BD,
	0x308,
	0x309,
	0x313,
	0x314,
	0x342,
	0x343,
	0x3AC,
	0x3AD,
	0x3AE,
	0x3AF,
	0x3B1,
	0x3B2,
	0x3B7,
	0x3B8,
	0x3B9,
	0x3BA,
	0x3C1,
	0x3C2,
	0x3C5,
	0x3C6,
	0x3C9,
	0x3CA,
	0x3CE,
	0x3CF,
	0x565,
	0x566,
	0x574,
	0x575,
	0x57E,
	0x57F,
	0x1F00,
	0x1F08,
	0x1F20,
	0x1F28,
	0x1F60,
	0x1F68,
	0x1F70,
	0x1F71,
	0x1F74,
	0x1F75,
	0x1F7C,
	0x1F7D
};

static const UV _Perl_Any_Folds_invlist[] = { /* for ASCII/Latin1 */
	253,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x41,
	0x5B,
	0x61,
	0x7B,
	0xB5,
	0xB6,
	0xC0,
	0xD7,
	0xD8,
	0xF7,
	0xF8,
	0x131,
	0x132,
	0x138,
	0x139,
	0x18D,
	0x18E,
	0x19B,
	0x19C,
	0x1AA,
	0x1AC,
	0x1BA,
	0x1BC,
	0x1BE,
	0x1BF,
	0x1C0,
	0x1C4,
	0x221,
	0x222,
	0x234,
	0x23A,
	0x255,
	0x256,
	0x258,
	0x259,
	0x25A,
	0x25B,
	0x25D,
	0x260,
	0x262,
	0x263,
	0x264,
	0x265,
	0x267,
	0x268,
	0x26D,
	0x26F,
	0x270,
	0x271,
	0x273,
	0x275,
	0x276,
	0x27D,
	0x27E,
	0x280,
	0x281,
	0x283,
	0x284,
	0x287,
	0x28D,
	0x292,
	0x293,
	0x29D,
	0x29F,
	0x2BC,
	0x2BD,
	0x2BE,
	0x2BF,
	0x300,
	0x302,
	0x307,
	0x309,
	0x30A,
	0x30B,
	0x30C,
	0x30D,
	0x313,
	0x314,
	0x331,
	0x332,
	0x342,
	0x343,
	0x345,
	0x346,
	0x370,
	0x374,
	0x376,
	0x378,
	0x37B,
	0x37E,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x3D2,
	0x3D5,
	0x3F6,
	0x3F7,
	0x3FC,
	0x3FD,
	0x482,
	0x48A,
	0x530,
	0x531,
	0x557,
	0x561,
	0x588,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1C80,
	0x1C89,
	0x1D79,
	0x1D7A,
	0x1D7D,
	0x1D7E,
	0x1E00,
	0x1E9C,
	0x1E9E,
	0x1E9F,
	0x1EA0,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FBD,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FCD,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FE0,
	0x1FED,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFD,
	0x2126,
	0x2127,
	0x212A,
	0x212C,
	0x2132,
	0x2133,
	0x214E,
	0x214F,
	0x2160,
	0x2180,
	0x2183,
	0x2185,
	0x24B6,
	0x24EA,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2C71,
	0x2C72,
	0x2C74,
	0x2C75,
	0x2C77,
	0x2C7E,
	0x2CE4,
	0x2CEB,
	0x2CEF,
	0x2CF2,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0xA640,
	0xA66E,
	0xA680,
	0xA69C,
	0xA722,
	0xA730,
	0xA732,
	0xA770,
	0xA779,
	0xA788,
	0xA78B,
	0xA78E,
	0xA790,
	0xA794,
	0xA796,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xAB53,
	0xAB54,
	0xAB70,
	0xABC0,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFF21,
	0xFF3B,
	0xFF41,
	0xFF5B,
	0x10400,
	0x10450,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x118A0,
	0x118E0,
	0x1E900,
	0x1E944
};

static const UV _Perl_Folds_To_Multi_Char_invlist[] = { /* for ASCII/Latin1 */
	59,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0xDF,
	0xE0,
	0x130,
	0x131,
	0x149,
	0x14A,
	0x1F0,
	0x1F1,
	0x390,
	0x391,
	0x3B0,
	0x3B1,
	0x587,
	0x588,
	0x1E96,
	0x1E9B,
	0x1E9E,
	0x1E9F,
	0x1F50,
	0x1F51,
	0x1F52,
	0x1F53,
	0x1F54,
	0x1F55,
	0x1F56,
	0x1F57,
	0x1F80,
	0x1FB0,
	0x1FB2,
	0x1FB5,
	0x1FB6,
	0x1FB8,
	0x1FBC,
	0x1FBD,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FC8,
	0x1FCC,
	0x1FCD,
	0x1FD2,
	0x1FD4,
	0x1FD6,
	0x1FD8,
	0x1FE2,
	0x1FE5,
	0x1FE6,
	0x1FE8,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FF8,
	0x1FFC,
	0x1FFD,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18
};

#endif	/* defined(PERL_IN_REGCOMP_C) */

#if defined(PERL_IN_PERL_C)

static const UV _Perl_GCB_invlist[] = { /* for ASCII/Latin1 */
	1615,	/* Number of elements */
	148565664, /* Version and data structure type */
	0,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0xA,
	0xB,
	0xD,
	0xE,
	0x20,
	0x7F,
	0xA0,
	0xAD,
	0xAE,
	0x300,
	0x370,
	0x483,
	0x48A,
	0x591,
	0x5BE,
	0x5BF,
	0x5C0,
	0x5C1,
	0x5C3,
	0x5C4,
	0x5C6,
	0x5C7,
	0x5C8,
	0x600,
	0x606,
	0x610,
	0x61B,
	0x61C,
	0x61D,
	0x64B,
	0x660,
	0x670,
	0x671,
	0x6D6,
	0x6DD,
	0x6DE,
	0x6DF,
	0x6E5,
	0x6E7,
	0x6E9,
	0x6EA,
	0x6EE,
	0x70F,
	0x710,
	0x711,
	0x712,
	0x730,
	0x74B,
	0x7A6,
	0x7B1,
	0x7EB,
	0x7F4,
	0x816,
	0x81A,
	0x81B,
	0x824,
	0x825,
	0x828,
	0x829,
	0x82E,
	0x859,
	0x85C,
	0x8D4,
	0x8E2,
	0x8E3,
	0x903,
	0x904,
	0x93A,
	0x93B,
	0x93C,
	0x93D,
	0x93E,
	0x941,
	0x949,
	0x94D,
	0x94E,
	0x950,
	0x951,
	0x958,
	0x962,
	0x964,
	0x981,
	0x982,
	0x984,
	0x9BC,
	0x9BD,
	0x9BE,
	0x9BF,
	0x9C1,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CD,
	0x9CE,
	0x9D7,
	0x9D8,
	0x9E2,
	0x9E4,
	0xA01,
	0xA03,
	0xA04,
	0xA3C,
	0xA3D,
	0xA3E,
	0xA41,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4E,
	0xA51,
	0xA52,
	0xA70,
	0xA72,
	0xA75,
	0xA76,
	0xA81,
	0xA83,
	0xA84,
	0xABC,
	0xABD,
	0xABE,
	0xAC1,
	0xAC6,
	0xAC7,
	0xAC9,
	0xACA,
	0xACB,
	0xACD,
	0xACE,
	0xAE2,
	0xAE4,
	0xB01,
	0xB02,
	0xB04,
	0xB3C,
	0xB3D,
	0xB3E,
	0xB40,
	0xB41,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4D,
	0xB4E,
	0xB56,
	0xB58,
	0xB62,
	0xB64,
	0xB82,
	0xB83,
	0xBBE,
	0xBBF,
	0xBC0,
	0xBC1,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCD,
	0xBCE,
	0xBD7,
	0xBD8,
	0xC00,
	0xC01,
	0xC04,
	0xC3E,
	0xC41,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4E,
	0xC55,
	0xC57,
	0xC62,
	0xC64,
	0xC81,
	0xC82,
	0xC84,
	0xCBC,
	0xCBD,
	0xCBE,
	0xCBF,
	0xCC0,
	0xCC2,
	0xCC3,
	0xCC5,
	0xCC6,
	0xCC7,
	0xCC9,
	0xCCA,
	0xCCC,
	0xCCE,
	0xCD5,
	0xCD7,
	0xCE2,
	0xCE4,
	0xD01,
	0xD02,
	0xD04,
	0xD3E,
	0xD3F,
	0xD41,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD4D,
	0xD4E,
	0xD4F,
	0xD57,
	0xD58,
	0xD62,
	0xD64,
	0xD82,
	0xD84,
	0xDCA,
	0xDCB,
	0xDCF,
	0xDD0,
	0xDD2,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDDF,
	0xDE0,
	0xDF2,
	0xDF4,
	0xE31,
	0xE32,
	0xE33,
	0xE34,
	0xE3B,
	0xE47,
	0xE4F,
	0xEB1,
	0xEB2,
	0xEB3,
	0xEB4,
	0xEBA,
	0xEBB,
	0xEBD,
	0xEC8,
	0xECE,
	0xF18,
	0xF1A,
	0xF35,
	0xF36,
	0xF37,
	0xF38,
	0xF39,
	0xF3A,
	0xF3E,
	0xF40,
	0xF71,
	0xF7F,
	0xF80,
	0xF85,
	0xF86,
	0xF88,
	0xF8D,
	0xF98,
	0xF99,
	0xFBD,
	0xFC6,
	0xFC7,
	0x102D,
	0x1031,
	0x1032,
	0x1038,
	0x1039,
	0x103B,
	0x103D,
	0x103F,
	0x1056,
	0x1058,
	0x105A,
	0x105E,
	0x1061,
	0x1071,
	0x1075,
	0x1082,
	0x1083,
	0x1084,
	0x1085,
	0x1087,
	0x108D,
	0x108E,
	0x109D,
	0x109E,
	0x1100,
	0x1160,
	0x11A8,
	0x1200,
	0x135D,
	0x1360,
	0x1712,
	0x1715,
	0x1732,
	0x1735,
	0x1752,
	0x1754,
	0x1772,
	0x1774,
	0x17B4,
	0x17B6,
	0x17B7,
	0x17BE,
	0x17C6,
	0x17C7,
	0x17C9,
	0x17D4,
	0x17DD,
	0x17DE,
	0x180B,
	0x180E,
	0x180F,
	0x1885,
	0x1887,
	0x18A9,
	0x18AA,
	0x1920,
	0x1923,
	0x1927,
	0x1929,
	0x192C,
	0x1930,
	0x1932,
	0x1933,
	0x1939,
	0x193C,
	0x1A17,
	0x1A19,
	0x1A1B,
	0x1A1C,
	0x1A55,
	0x1A56,
	0x1A57,
	0x1A58,
	0x1A5F,
	0x1A60,
	0x1A61,
	0x1A62,
	0x1A63,
	0x1A65,
	0x1A6D,
	0x1A73,
	0x1A7D,
	0x1A7F,
	0x1A80,
	0x1AB0,
	0x1ABF,
	0x1B00,
	0x1B04,
	0x1B05,
	0x1B34,
	0x1B35,
	0x1B36,
	0x1B3B,
	0x1B3C,
	0x1B3D,
	0x1B42,
	0x1B43,
	0x1B45,
	0x1B6B,
	0x1B74,
	0x1B80,
	0x1B82,
	0x1B83,
	0x1BA1,
	0x1BA2,
	0x1BA6,
	0x1BA8,
	0x1BAA,
	0x1BAB,
	0x1BAE,
	0x1BE6,
	0x1BE7,
	0x1BE8,
	0x1BEA,
	0x1BED,
	0x1BEE,
	0x1BEF,
	0x1BF2,
	0x1BF4,
	0x1C24,
	0x1C2C,
	0x1C34,
	0x1C36,
	0x1C38,
	0x1CD0,
	0x1CD3,
	0x1CD4,
	0x1CE1,
	0x1CE2,
	0x1CE9,
	0x1CED,
	0x1CEE,
	0x1CF2,
	0x1CF4,
	0x1CF5,
	0x1CF8,
	0x1CFA,
	0x1DC0,
	0x1DF6,
	0x1DFB,
	0x1E00,
	0x200B,
	0x200C,
	0x200D,
	0x200E,
	0x2010,
	0x2028,
	0x202F,
	0x2060,
	0x2070,
	0x20D0,
	0x20F1,
	0x261D,
	0x261E,
	0x26F9,
	0x26FA,
	0x270A,
	0x270E,
	0x2764,
	0x2765,
	0x2CEF,
	0x2CF2,
	0x2D7F,
	0x2D80,
	0x2DE0,
	0x2E00,
	0x302A,
	0x3030,
	0x3099,
	0x309B,
	0xA66F,
	0xA673,
	0xA674,
	0xA67E,
	0xA69E,
	0xA6A0,
	0xA6F0,
	0xA6F2,
	0xA802,
	0xA803,
	0xA806,
	0xA807,
	0xA80B,
	0xA80C,
	0xA823,
	0xA825,
	0xA827,
	0xA828,
	0xA880,
	0xA882,
	0xA8B4,
	0xA8C4,
	0xA8C6,
	0xA8E0,
	0xA8F2,
	0xA926,
	0xA92E,
	0xA947,
	0xA952,
	0xA954,
	0xA960,
	0xA97D,
	0xA980,
	0xA983,
	0xA984,
	0xA9B3,
	0xA9B4,
	0xA9B6,
	0xA9BA,
	0xA9BC,
	0xA9BD,
	0xA9C1,
	0xA9E5,
	0xA9E6,
	0xAA29,
	0xAA2F,
	0xAA31,
	0xAA33,
	0xAA35,
	0xAA37,
	0xAA43,
	0xAA44,
	0xAA4C,
	0xAA4D,
	0xAA4E,
	0xAA7C,
	0xAA7D,
	0xAAB0,
	0xAAB1,
	0xAAB2,
	0xAAB5,
	0xAAB7,
	0xAAB9,
	0xAABE,
	0xAAC0,
	0xAAC1,
	0xAAC2,
	0xAAEB,
	0xAAEC,
	0xAAEE,
	0xAAF0,
	0xAAF5,
	0xAAF6,
	0xAAF7,
	0xABE3,
	0xABE5,
	0xABE6,
	0xABE8,
	0xABE9,
	0xABEB,
	0xABEC,
	0xABED,
	0xABEE,
	0xAC00,
	0xAC01,
	0xAC1C,
	0xAC1D,
	0xAC38,
	0xAC39,
	0xAC54,
	0xAC55,
	0xAC70,
	0xAC71,
	0xAC8C,
	0xAC8D,
	0xACA8,
	0xACA9,
	0xACC4,
	0xACC5,
	0xACE0,
	0xACE1,
	0xACFC,
	0xACFD,
	0xAD18,
	0xAD19,
	0xAD34,
	0xAD35,
	0xAD50,
	0xAD51,
	0xAD6C,
	0xAD6D,
	0xAD88,
	0xAD89,
	0xADA4,
	0xADA5,
	0xADC0,
	0xADC1,
	0xADDC,
	0xADDD,
	0xADF8,
	0xADF9,
	0xAE14,
	0xAE15,
	0xAE30,
	0xAE31,
	0xAE4C,
	0xAE4D,
	0xAE68,
	0xAE69,
	0xAE84,
	0xAE85,
	0xAEA0,
	0xAEA1,
	0xAEBC,
	0xAEBD,
	0xAED8,
	0xAED9,
	0xAEF4,
	0xAEF5,
	0xAF10,
	0xAF11,
	0xAF2C,
	0xAF2D,
	0xAF48,
	0xAF49,
	0xAF64,
	0xAF65,
	0xAF80,
	0xAF81,
	0xAF9C,
	0xAF9D,
	0xAFB8,
	0xAFB9,
	0xAFD4,
	0xAFD5,
	0xAFF0,
	0xAFF1,
	0xB00C,
	0xB00D,
	0xB028,
	0xB029,
	0xB044,
	0xB045,
	0xB060,
	0xB061,
	0xB07C,
	0xB07D,
	0xB098,
	0xB099,
	0xB0B4,
	0xB0B5,
	0xB0D0,
	0xB0D1,
	0xB0EC,
	0xB0ED,
	0xB108,
	0xB109,
	0xB124,
	0xB125,
	0xB140,
	0xB141,
	0xB15C,
	0xB15D,
	0xB178,
	0xB179,
	0xB194,
	0xB195,
	0xB1B0,
	0xB1B1,
	0xB1CC,
	0xB1CD,
	0xB1E8,
	0xB1E9,
	0xB204,
	0xB205,
	0xB220,
	0xB221,
	0xB23C,
	0xB23D,
	0xB258,
	0xB259,
	0xB274,
	0xB275,
	0xB290,
	0xB291,
	0xB2AC,
	0xB2AD,
	0xB2C8,
	0xB2C9,
	0xB2E4,
	0xB2E5,
	0xB300,
	0xB301,
	0xB31C,
	0xB31D,
	0xB338,
	0xB339,
	0xB354,
	0xB355,
	0xB370,
	0xB371,
	0xB38C,
	0xB38D,
	0xB3A8,
	0xB3A9,
	0xB3C4,
	0xB3C5,
	0xB3E0,
	0xB3E1,
	0xB3FC,
	0xB3FD,
	0xB418,
	0xB419,
	0xB434,
	0xB435,
	0xB450,
	0xB451,
	0xB46C,
	0xB46D,
	0xB488,
	0xB489,
	0xB4A4,
	0xB4A5,
	0xB4C0,
	0xB4C1,
	0xB4DC,
	0xB4DD,
	0xB4F8,
	0xB4F9,
	0xB514,
	0xB515,
	0xB530,
	0xB531,
	0xB54C,
	0xB54D,
	0xB568,
	0xB569,
	0xB584,
	0xB585,
	0xB5A0,
	0xB5A1,
	0xB5BC,
	0xB5BD,
	0xB5D8,
	0xB5D9,
	0xB5F4,
	0xB5F5,
	0xB610,
	0xB611,
	0xB62C,
	0xB62D,
	0xB648,
	0xB649,
	0xB664,
	0xB665,
	0xB680,
	0xB681,
	0xB69C,
	0xB69D,
	0xB6B8,
	0xB6B9,
	0xB6D4,
	0xB6D5,
	0xB6F0,
	0xB6F1,
	0xB70C,
	0xB70D,
	0xB728,
	0xB729,
	0xB744,
	0xB745,
	0xB760,
	0xB761,
	0xB77C,
	0xB77D,
	0xB798,
	0xB799,
	0xB7B4,
	0xB7B5,
	0xB7D0,
	0xB7D1,
	0xB7EC,
	0xB7ED,
	0xB808,
	0xB809,
	0xB824,
	0xB825,
	0xB840,
	0xB841,
	0xB85C,
	0xB85D,
	0xB878,
	0xB879,
	0xB894,
	0xB895,
	0xB8B0,
	0xB8B1,
	0xB8CC,
	0xB8CD,
	0xB8E8,
	0xB8E9,
	0xB904,
	0xB905,
	0xB920,
	0xB921,
	0xB93C,
	0xB93D,
	0xB958,
	0xB959,
	0xB974,
	0xB975,
	0xB990,
	0xB991,
	0xB9AC,
	0xB9AD,
	0xB9C8,
	0xB9C9,
	0xB9E4,
	0xB9E5,
	0xBA00,
	0xBA01,
	0xBA1C,
	0xBA1D,
	0xBA38,
	0xBA39,
	0xBA54,
	0xBA55,
	0xBA70,
	0xBA71,
	0xBA8C,
	0xBA8D,
	0xBAA8,
	0xBAA9,
	0xBAC4,
	0xBAC5,
	0xBAE0,
	0xBAE1,
	0xBAFC,
	0xBAFD,
	0xBB18,
	0xBB19,
	0xBB34,
	0xBB35,
	0xBB50,
	0xBB51,
	0xBB6C,
	0xBB6D,
	0xBB88,
	0xBB89,
	0xBBA4,
	0xBBA5,
	0xBBC0,
	0xBBC1,
	0xBBDC,
	0xBBDD,
	0xBBF8,
	0xBBF9,
	0xBC14,
	0xBC15,
	0xBC30,
	0xBC31,
	0xBC4C,
	0xBC4D,
	0xBC68,
	0xBC69,
	0xBC84,
	0xBC85,
	0xBCA0,
	0xBCA1,
	0xBCBC,
	0xBCBD,
	0xBCD8,
	0xBCD9,
	0xBCF4,
	0xBCF5,
	0xBD10,
	0xBD11,
	0xBD2C,
	0xBD2D,
	0xBD48,
	0xBD49,
	0xBD64,
	0xBD65,
	0xBD80,
	0xBD81,
	0xBD9C,
	0xBD9D,
	0xBDB8,
	0xBDB9,
	0xBDD4,
	0xBDD5,
	0xBDF0,
	0xBDF1,
	0xBE0C,
	0xBE0D,
	0xBE28,
	0xBE29,
	0xBE44,
	0xBE45,
	0xBE60,
	0xBE61,
	0xBE7C,
	0xBE7D,
	0xBE98,
	0xBE99,
	0xBEB4,
	0xBEB5,
	0xBED0,
	0xBED1,
	0xBEEC,
	0xBEED,
	0xBF08,
	0xBF09,
	0xBF24,
	0xBF25,
	0xBF40,
	0xBF41,
	0xBF5C,
	0xBF5D,
	0xBF78,
	0xBF79,
	0xBF94,
	0xBF95,
	0xBFB0,
	0xBFB1,
	0xBFCC,
	0xBFCD,
	0xBFE8,
	0xBFE9,
	0xC004,
	0xC005,
	0xC020,
	0xC021,
	0xC03C,
	0xC03D,
	0xC058,
	0xC059,
	0xC074,
	0xC075,
	0xC090,
	0xC091,
	0xC0AC,
	0xC0AD,
	0xC0C8,
	0xC0C9,
	0xC0E4,
	0xC0E5,
	0xC100,
	0xC101,
	0xC11C,
	0xC11D,
	0xC138,
	0xC139,
	0xC154,
	0xC155,
	0xC170,
	0xC171,
	0xC18C,
	0xC18D,
	0xC1A8,
	0xC1A9,
	0xC1C4,
	0xC1C5,
	0xC1E0,
	0xC1E1,
	0xC1FC,
	0xC1FD,
	0xC218,
	0xC219,
	0xC234,
	0xC235,
	0xC250,
	0xC251,
	0xC26C,
	0xC26D,
	0xC288,
	0xC289,
	0xC2A4,
	0xC2A5,
	0xC2C0,
	0xC2C1,
	0xC2DC,
	0xC2DD,
	0xC2F8,
	0xC2F9,
	0xC314,
	0xC315,
	0xC330,
	0xC331,
	0xC34C,
	0xC34D,
	0xC368,
	0xC369,
	0xC384,
	0xC385,
	0xC3A0,
	0xC3A1,
	0xC3BC,
	0xC3BD,
	0xC3D8,
	0xC3D9,
	0xC3F4,
	0xC3F5,
	0xC410,
	0xC411,
	0xC42C,
	0xC42D,
	0xC448,
	0xC449,
	0xC464,
	0xC465,
	0xC480,
	0xC481,
	0xC49C,
	0xC49D,
	0xC4B8,
	0xC4B9,
	0xC4D4,
	0xC4D5,
	0xC4F0,
	0xC4F1,
	0xC50C,
	0xC50D,
	0xC528,
	0xC529,
	0xC544,
	0xC545,
	0xC560,
	0xC561,
	0xC57C,
	0xC57D,
	0xC598,
	0xC599,
	0xC5B4,
	0xC5B5,
	0xC5D0,
	0xC5D1,
	0xC5EC,
	0xC5ED,
	0xC608,
	0xC609,
	0xC624,
	0xC625,
	0xC640,
	0xC641,
	0xC65C,
	0xC65D,
	0xC678,
	0xC679,
	0xC694,
	0xC695,
	0xC6B0,
	0xC6B1,
	0xC6CC,
	0xC6CD,
	0xC6E8,
	0xC6E9,
	0xC704,
	0xC705,
	0xC720,
	0xC721,
	0xC73C,
	0xC73D,
	0xC758,
	0xC759,
	0xC774,
	0xC775,
	0xC790,
	0xC791,
	0xC7AC,
	0xC7AD,
	0xC7C8,
	0xC7C9,
	0xC7E4,
	0xC7E5,
	0xC800,
	0xC801,
	0xC81C,
	0xC81D,
	0xC838,
	0xC839,
	0xC854,
	0xC855,
	0xC870,
	0xC871,
	0xC88C,
	0xC88D,
	0xC8A8,
	0xC8A9,
	0xC8C4,
	0xC8C5,
	0xC8E0,
	0xC8E1,
	0xC8FC,
	0xC8FD,
	0xC918,
	0xC919,
	0xC934,
	0xC935,
	0xC950,
	0xC951,
	0xC96C,
	0xC96D,
	0xC988,
	0xC989,
	0xC9A4,
	0xC9A5,
	0xC9C0,
	0xC9C1,
	0xC9DC,
	0xC9DD,
	0xC9F8,
	0xC9F9,
	0xCA14,
	0xCA15,
	0xCA30,
	0xCA31,
	0xCA4C,
	0xCA4D,
	0xCA68,
	0xCA69,
	0xCA84,
	0xCA85,
	0xCAA0,
	0xCAA1,
	0xCABC,
	0xCABD,
	0xCAD8,
	0xCAD9,
	0xCAF4,
	0xCAF5,
	0xCB10,
	0xCB11,
	0xCB2C,
	0xCB2D,
	0xCB48,
	0xCB49,
	0xCB64,
	0xCB65,
	0xCB80,
	0xCB81,
	0xCB9C,
	0xCB9D,
	0xCBB8,
	0xCBB9,
	0xCBD4,
	0xCBD5,
	0xCBF0,
	0xCBF1,
	0xCC0C,
	0xCC0D,
	0xCC28,
	0xCC29,
	0xCC44,
	0xCC45,
	0xCC60,
	0xCC61,
	0xCC7C,
	0xCC7D,
	0xCC98,
	0xCC99,
	0xCCB4,
	0xCCB5,
	0xCCD0,
	0xCCD1,
	0xCCEC,
	0xCCED,
	0xCD08,
	0xCD09,
	0xCD24,
	0xCD25,
	0xCD40,
	0xCD41,
	0xCD5C,
	0xCD5D,
	0xCD78,
	0xCD79,
	0xCD94,
	0xCD95,
	0xCDB0,
	0xCDB1,
	0xCDCC,
	0xCDCD,
	0xCDE8,
	0xCDE9,
	0xCE04,
	0xCE05,
	0xCE20,
	0xCE21,
	0xCE3C,
	0xCE3D,
	0xCE58,
	0xCE59,
	0xCE74,
	0xCE75,
	0xCE90,
	0xCE91,
	0xCEAC,
	0xCEAD,
	0xCEC8,
	0xCEC9,
	0xCEE4,
	0xCEE5,
	0xCF00,
	0xCF01,
	0xCF1C,
	0xCF1D,
	0xCF38,
	0xCF39,
	0xCF54,
	0xCF55,
	0xCF70,
	0xCF71,
	0xCF8C,
	0xCF8D,
	0xCFA8,
	0xCFA9,
	0xCFC4,
	0xCFC5,
	0xCFE0,
	0xCFE1,
	0xCFFC,
	0xCFFD,
	0xD018,
	0xD019,
	0xD034,
	0xD035,
	0xD050,
	0xD051,
	0xD06C,
	0xD06D,
	0xD088,
	0xD089,
	0xD0A4,
	0xD0A5,
	0xD0C0,
	0xD0C1,
	0xD0DC,
	0xD0DD,
	0xD0F8,
	0xD0F9,
	0xD114,
	0xD115,
	0xD130,
	0xD131,
	0xD14C,
	0xD14D,
	0xD168,
	0xD169,
	0xD184,
	0xD185,
	0xD1A0,
	0xD1A1,
	0xD1BC,
	0xD1BD,
	0xD1D8,
	0xD1D9,
	0xD1F4,
	0xD1F5,
	0xD210,
	0xD211,
	0xD22C,
	0xD22D,
	0xD248,
	0xD249,
	0xD264,
	0xD265,
	0xD280,
	0xD281,
	0xD29C,
	0xD29D,
	0xD2B8,
	0xD2B9,
	0xD2D4,
	0xD2D5,
	0xD2F0,
	0xD2F1,
	0xD30C,
	0xD30D,
	0xD328,
	0xD329,
	0xD344,
	0xD345,
	0xD360,
	0xD361,
	0xD37C,
	0xD37D,
	0xD398,
	0xD399,
	0xD3B4,
	0xD3B5,
	0xD3D0,
	0xD3D1,
	0xD3EC,
	0xD3ED,
	0xD408,
	0xD409,
	0xD424,
	0xD425,
	0xD440,
	0xD441,
	0xD45C,
	0xD45D,
	0xD478,
	0xD479,
	0xD494,
	0xD495,
	0xD4B0,
	0xD4B1,
	0xD4CC,
	0xD4CD,
	0xD4E8,
	0xD4E9,
	0xD504,
	0xD505,
	0xD520,
	0xD521,
	0xD53C,
	0xD53D,
	0xD558,
	0xD559,
	0xD574,
	0xD575,
	0xD590,
	0xD591,
	0xD5AC,
	0xD5AD,
	0xD5C8,
	0xD5C9,
	0xD5E4,
	0xD5E5,
	0xD600,
	0xD601,
	0xD61C,
	0xD61D,
	0xD638,
	0xD639,
	0xD654,
	0xD655,
	0xD670,
	0xD671,
	0xD68C,
	0xD68D,
	0xD6A8,
	0xD6A9,
	0xD6C4,
	0xD6C5,
	0xD6E0,
	0xD6E1,
	0xD6FC,
	0xD6FD,
	0xD718,
	0xD719,
	0xD734,
	0xD735,
	0xD750,
	0xD751,
	0xD76C,
	0xD76D,
	0xD788,
	0xD789,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xD800,
	0xE000,
	0xFB1E,
	0xFB1F,
	0xFE00,
	0xFE10,
	0xFE20,
	0xFE30,
	0xFEFF,
	0xFF00,
	0xFF9E,
	0xFFA0,
	0xFFF0,
	0xFFFC,
	0x101FD,
	0x101FE,
	0x102E0,
	0x102E1,
	0x10376,
	0x1037B,
	0x10A01,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A10,
	0x10A38,
	0x10A3B,
	0x10A3F,
	0x10A40,
	0x10AE5,
	0x10AE7,
	0x11000,
	0x11001,
	0x11002,
	0x11003,
	0x11038,
	0x11047,
	0x1107F,
	0x11082,
	0x11083,
	0x110B0,
	0x110B3,
	0x110B7,
	0x110B9,
	0x110BB,
	0x110BD,
	0x110BE,
	0x11100,
	0x11103,
	0x11127,
	0x1112C,
	0x1112D,
	0x11135,
	0x11173,
	0x11174,
	0x11180,
	0x11182,
	0x11183,
	0x111B3,
	0x111B6,
	0x111BF,
	0x111C1,
	0x111C2,
	0x111C4,
	0x111CA,
	0x111CD,
	0x1122C,
	0x1122F,
	0x11232,
	0x11234,
	0x11235,
	0x11236,
	0x11238,
	0x1123E,
	0x1123F,
	0x112DF,
	0x112E0,
	0x112E3,
	0x112EB,
	0x11300,
	0x11302,
	0x11304,
	0x1133C,
	0x1133D,
	0x1133E,
	0x1133F,
	0x11340,
	0x11341,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134E,
	0x11357,
	0x11358,
	0x11362,
	0x11364,
	0x11366,
	0x1136D,
	0x11370,
	0x11375,
	0x11435,
	0x11438,
	0x11440,
	0x11442,
	0x11445,
	0x11446,
	0x11447,
	0x114B0,
	0x114B1,
	0x114B3,
	0x114B9,
	0x114BA,
	0x114BB,
	0x114BD,
	0x114BE,
	0x114BF,
	0x114C1,
	0x114C2,
	0x114C4,
	0x115AF,
	0x115B0,
	0x115B2,
	0x115B6,
	0x115B8,
	0x115BC,
	0x115BE,
	0x115BF,
	0x115C1,
	0x115DC,
	0x115DE,
	0x11630,
	0x11633,
	0x1163B,
	0x1163D,
	0x1163E,
	0x1163F,
	0x11641,
	0x116AB,
	0x116AC,
	0x116AD,
	0x116AE,
	0x116B0,
	0x116B6,
	0x116B7,
	0x116B8,
	0x1171D,
	0x11720,
	0x11722,
	0x11726,
	0x11727,
	0x1172C,
	0x11C2F,
	0x11C30,
	0x11C37,
	0x11C38,
	0x11C3E,
	0x11C3F,
	0x11C40,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CAA,
	0x11CB1,
	0x11CB2,
	0x11CB4,
	0x11CB5,
	0x11CB7,
	0x16AF0,
	0x16AF5,
	0x16B30,
	0x16B37,
	0x16F51,
	0x16F7F,
	0x16F8F,
	0x16F93,
	0x1BC9D,
	0x1BC9F,
	0x1BCA0,
	0x1BCA4,
	0x1D165,
	0x1D166,
	0x1D167,
	0x1D16A,
	0x1D16D,
	0x1D16E,
	0x1D173,
	0x1D17B,
	0x1D183,
	0x1D185,
	0x1D18C,
	0x1D1AA,
	0x1D1AE,
	0x1D242,
	0x1D245,
	0x1DA00,
	0x1DA37,
	0x1DA3B,
	0x1DA6D,
	0x1DA75,
	0x1DA76,
	0x1DA84,
	0x1DA85,
	0x1DA9B,
	0x1DAA0,
	0x1DAA1,
	0x1DAB0,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E8D0,
	0x1E8D7,
	0x1E944,
	0x1E94B,
	0x1F1E6,
	0x1F200,
	0x1F385,
	0x1F386,
	0x1F3C3,
	0x1F3C5,
	0x1F3CA,
	0x1F3CC,
	0x1F3FB,
	0x1F400,
	0x1F442,
	0x1F444,
	0x1F446,
	0x1F451,
	0x1F466,
	0x1F46A,
	0x1F46E,
	0x1F46F,
	0x1F470,
	0x1F479,
	0x1F47C,
	0x1F47D,
	0x1F481,
	0x1F484,
	0x1F485,
	0x1F488,
	0x1F48B,
	0x1F48C,
	0x1F4AA,
	0x1F4AB,
	0x1F575,
	0x1F576,
	0x1F57A,
	0x1F57B,
	0x1F590,
	0x1F591,
	0x1F595,
	0x1F597,
	0x1F5E8,
	0x1F5E9,
	0x1F645,
	0x1F648,
	0x1F64B,
	0x1F650,
	0x1F6A3,
	0x1F6A4,
	0x1F6B4,
	0x1F6B7,
	0x1F6C0,
	0x1F6C1,
	0x1F918,
	0x1F91F,
	0x1F926,
	0x1F927,
	0x1F930,
	0x1F931,
	0x1F933,
	0x1F93A,
	0x1F93C,
	0x1F93F,
	0xE0000,
	0xE0020,
	0xE0080,
	0xE0100,
	0xE01F0,
	0xE1000
};

#endif	/* defined(PERL_IN_PERL_C) */

#if defined(PERL_IN_REGEXEC_C)

#define GCB_ENUM_COUNT 19

typedef enum {
	GCB_Other = 0,
	GCB_CR = 1,
	GCB_Control = 2,
	GCB_E_Base = 3,
	GCB_E_Base_GAZ = 4,
	GCB_E_Modifier = 5,
	GCB_Extend = 6,
	GCB_Glue_After_Zwj = 7,
	GCB_L = 8,
	GCB_LF = 9,
	GCB_LV = 10,
	GCB_LVT = 11,
	GCB_Prepend = 12,
	GCB_Regional_Indicator = 13,
	GCB_SpacingMark = 14,
	GCB_T = 15,
	GCB_V = 16,
	GCB_ZWJ = 17,
	GCB_EDGE = 18
} GCB_enum;

static const GCB_enum _Perl_GCB_invmap[] = { /* for ASCII/Latin1 */
	GCB_Control,
	GCB_LF,
	GCB_Control,
	GCB_CR,
	GCB_Control,
	GCB_Other,
	GCB_Control,
	GCB_Other,
	GCB_Control,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Prepend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Control,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Prepend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Prepend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Prepend,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Prepend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_L,
	GCB_V,
	GCB_T,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Control,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Control,
	GCB_Extend,
	GCB_ZWJ,
	GCB_Control,
	GCB_Other,
	GCB_Control,
	GCB_Other,
	GCB_Control,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_Glue_After_Zwj,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_L,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_Other,
	GCB_V,
	GCB_Other,
	GCB_T,
	GCB_Other,
	GCB_Control,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Control,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Control,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Prepend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Prepend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Control,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Control,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Regional_Indicator,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Modifier,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base_GAZ,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_Glue_After_Zwj,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_Glue_After_Zwj,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_Control,
	GCB_Extend,
	GCB_Control,
	GCB_Extend,
	GCB_Control,
	GCB_Other
};

#endif	/* defined(PERL_IN_REGEXEC_C) */

#if defined(PERL_IN_UTF8_C)

static const UV _Perl_IDCont_invlist[] = { /* for ASCII/Latin1 */
	1367,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x30,
	0x3A,
	0x41,
	0x5B,
	0x5F,
	0x60,
	0x61,
	0x7B,
	0xAA,
	0xAB,
	0xB5,
	0xB6,
	0xBA,
	0xBB,
	0xC0,
	0xD7,
	0xD8,
	0xF7,
	0xF8,
	0x2C2,
	0x2C6,
	0x2D2,
	0x2E0,
	0x2E5,
	0x2EC,
	0x2ED,
	0x2EE,
	0x2EF,
	0x300,
	0x375,
	0x376,
	0x378,
	0x37B,
	0x37E,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x3F6,
	0x3F7,
	0x482,
	0x483,
	0x488,
	0x48A,
	0x530,
	0x531,
	0x557,
	0x559,
	0x55A,
	0x561,
	0x588,
	0x591,
	0x5BE,
	0x5BF,
	0x5C0,
	0x5C1,
	0x5C3,
	0x5C4,
	0x5C6,
	0x5C7,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F3,
	0x610,
	0x61B,
	0x620,
	0x66A,
	0x66E,
	0x6D4,
	0x6D5,
	0x6DD,
	0x6DF,
	0x6E9,
	0x6EA,
	0x6FD,
	0x6FF,
	0x700,
	0x710,
	0x74B,
	0x74D,
	0x7B2,
	0x7C0,
	0x7F6,
	0x7FA,
	0x7FB,
	0x800,
	0x82E,
	0x840,
	0x85C,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x8D4,
	0x8E2,
	0x8E3,
	0x964,
	0x966,
	0x970,
	0x971,
	0x984,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BC,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CF,
	0x9D7,
	0x9D8,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E4,
	0x9E6,
	0x9F2,
	0xA01,
	0xA04,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA3C,
	0xA3D,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4E,
	0xA51,
	0xA52,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA66,
	0xA76,
	0xA81,
	0xA84,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABC,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACE,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE4,
	0xAE6,
	0xAF0,
	0xAF9,
	0xAFA,
	0xB01,
	0xB04,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3C,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4E,
	0xB56,
	0xB58,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB64,
	0xB66,
	0xB70,
	0xB71,
	0xB72,
	0xB82,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCE,
	0xBD0,
	0xBD1,
	0xBD7,
	0xBD8,
	0xBE6,
	0xBF0,
	0xC00,
	0xC04,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4E,
	0xC55,
	0xC57,
	0xC58,
	0xC5B,
	0xC60,
	0xC64,
	0xC66,
	0xC70,
	0xC80,
	0xC84,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBC,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCE,
	0xCD5,
	0xCD7,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE4,
	0xCE6,
	0xCF0,
	0xCF1,
	0xCF3,
	0xD01,
	0xD04,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD4F,
	0xD54,
	0xD58,
	0xD5F,
	0xD64,
	0xD66,
	0xD70,
	0xD7A,
	0xD80,
	0xD82,
	0xD84,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xDCA,
	0xDCB,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDE6,
	0xDF0,
	0xDF2,
	0xDF4,
	0xE01,
	0xE3B,
	0xE40,
	0xE4F,
	0xE50,
	0xE5A,
	0xE81,
	0xE83,
	0xE84,
	0xE85,
	0xE87,
	0xE89,
	0xE8A,
	0xE8B,
	0xE8D,
	0xE8E,
	0xE94,
	0xE98,
	0xE99,
	0xEA0,
	0xEA1,
	0xEA4,
	0xEA5,
	0xEA6,
	0xEA7,
	0xEA8,
	0xEAA,
	0xEAC,
	0xEAD,
	0xEBA,
	0xEBB,
	0xEBE,
	0xEC0,
	0xEC5,
	0xEC6,
	0xEC7,
	0xEC8,
	0xECE,
	0xED0,
	0xEDA,
	0xEDC,
	0xEE0,
	0xF00,
	0xF01,
	0xF18,
	0xF1A,
	0xF20,
	0xF2A,
	0xF35,
	0xF36,
	0xF37,
	0xF38,
	0xF39,
	0xF3A,
	0xF3E,
	0xF48,
	0xF49,
	0xF6D,
	0xF71,
	0xF85,
	0xF86,
	0xF98,
	0xF99,
	0xFBD,
	0xFC6,
	0xFC7,
	0x1000,
	0x104A,
	0x1050,
	0x109E,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x10FB,
	0x10FC,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x135D,
	0x1360,
	0x1380,
	0x1390,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1401,
	0x166D,
	0x166F,
	0x1680,
	0x1681,
	0x169B,
	0x16A0,
	0x16EB,
	0x16EE,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1715,
	0x1720,
	0x1735,
	0x1740,
	0x1754,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1772,
	0x1774,
	0x1780,
	0x17D4,
	0x17D7,
	0x17D8,
	0x17DC,
	0x17DE,
	0x17E0,
	0x17EA,
	0x180B,
	0x180E,
	0x1810,
	0x181A,
	0x1820,
	0x1878,
	0x1880,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1920,
	0x192C,
	0x1930,
	0x193C,
	0x1946,
	0x196E,
	0x1970,
	0x1975,
	0x1980,
	0x19AC,
	0x19B0,
	0x19CA,
	0x19D0,
	0x19DA,
	0x1A00,
	0x1A1C,
	0x1A20,
	0x1A5F,
	0x1A60,
	0x1A7D,
	0x1A7F,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1AA7,
	0x1AA8,
	0x1AB0,
	0x1ABE,
	0x1B00,
	0x1B4C,
	0x1B50,
	0x1B5A,
	0x1B6B,
	0x1B74,
	0x1B80,
	0x1BF4,
	0x1C00,
	0x1C38,
	0x1C40,
	0x1C4A,
	0x1C4D,
	0x1C7E,
	0x1C80,
	0x1C89,
	0x1CD0,
	0x1CD3,
	0x1CD4,
	0x1CF7,
	0x1CF8,
	0x1CFA,
	0x1D00,
	0x1DF6,
	0x1DFB,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FBD,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FCD,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FE0,
	0x1FED,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFD,
	0x203F,
	0x2041,
	0x2054,
	0x2055,
	0x2071,
	0x2072,
	0x207F,
	0x2080,
	0x2090,
	0x209D,
	0x20D0,
	0x20DD,
	0x20E1,
	0x20E2,
	0x20E5,
	0x20F1,
	0x2102,
	0x2103,
	0x2107,
	0x2108,
	0x210A,
	0x2114,
	0x2115,
	0x2116,
	0x2119,
	0x211E,
	0x2124,
	0x2125,
	0x2126,
	0x2127,
	0x2128,
	0x2129,
	0x212A,
	0x212E,
	0x212F,
	0x213A,
	0x213C,
	0x2140,
	0x2145,
	0x214A,
	0x214E,
	0x214F,
	0x2160,
	0x2189,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CE5,
	0x2CEB,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D70,
	0x2D7F,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x2DE0,
	0x2E00,
	0x3005,
	0x3008,
	0x3021,
	0x3030,
	0x3031,
	0x3036,
	0x3038,
	0x303D,
	0x3041,
	0x3097,
	0x3099,
	0x309B,
	0x309D,
	0x30A0,
	0x30A1,
	0x30FB,
	0x30FC,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x31A0,
	0x31BB,
	0x31F0,
	0x3200,
	0x3400,
	0x4DB6,
	0x4E00,
	0x9FD6,
	0xA000,
	0xA48D,
	0xA4D0,
	0xA4FE,
	0xA500,
	0xA60D,
	0xA610,
	0xA62C,
	0xA640,
	0xA670,
	0xA674,
	0xA67E,
	0xA67F,
	0xA6F2,
	0xA717,
	0xA720,
	0xA722,
	0xA789,
	0xA78B,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F7,
	0xA828,
	0xA840,
	0xA874,
	0xA880,
	0xA8C6,
	0xA8D0,
	0xA8DA,
	0xA8E0,
	0xA8F8,
	0xA8FB,
	0xA8FC,
	0xA8FD,
	0xA8FE,
	0xA900,
	0xA92E,
	0xA930,
	0xA954,
	0xA960,
	0xA97D,
	0xA980,
	0xA9C1,
	0xA9CF,
	0xA9DA,
	0xA9E0,
	0xA9FF,
	0xAA00,
	0xAA37,
	0xAA40,
	0xAA4E,
	0xAA50,
	0xAA5A,
	0xAA60,
	0xAA77,
	0xAA7A,
	0xAAC3,
	0xAADB,
	0xAADE,
	0xAAE0,
	0xAAF0,
	0xAAF2,
	0xAAF7,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB5B,
	0xAB5C,
	0xAB66,
	0xAB70,
	0xABEB,
	0xABEC,
	0xABEE,
	0xABF0,
	0xABFA,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xF900,
	0xFA6E,
	0xFA70,
	0xFADA,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB29,
	0xFB2A,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFBB2,
	0xFBD3,
	0xFC5E,
	0xFC64,
	0xFD3E,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFA,
	0xFE00,
	0xFE10,
	0xFE20,
	0xFE30,
	0xFE33,
	0xFE35,
	0xFE4D,
	0xFE50,
	0xFE71,
	0xFE72,
	0xFE73,
	0xFE74,
	0xFE77,
	0xFE78,
	0xFE79,
	0xFE7A,
	0xFE7B,
	0xFE7C,
	0xFE7D,
	0xFE7E,
	0xFE7F,
	0xFEFD,
	0xFF10,
	0xFF1A,
	0xFF21,
	0xFF3B,
	0xFF3F,
	0xFF40,
	0xFF41,
	0xFF5B,
	0xFF66,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10140,
	0x10175,
	0x101FD,
	0x101FE,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x102E0,
	0x102E1,
	0x10300,
	0x10320,
	0x10330,
	0x1034B,
	0x10350,
	0x1037B,
	0x10380,
	0x1039E,
	0x103A0,
	0x103C4,
	0x103C8,
	0x103D0,
	0x103D1,
	0x103D6,
	0x10400,
	0x1049E,
	0x104A0,
	0x104AA,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10860,
	0x10877,
	0x10880,
	0x1089F,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x10900,
	0x10916,
	0x10920,
	0x1093A,
	0x10980,
	0x109B8,
	0x109BE,
	0x109C0,
	0x10A00,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A38,
	0x10A3B,
	0x10A3F,
	0x10A40,
	0x10A60,
	0x10A7D,
	0x10A80,
	0x10A9D,
	0x10AC0,
	0x10AC8,
	0x10AC9,
	0x10AE7,
	0x10B00,
	0x10B36,
	0x10B40,
	0x10B56,
	0x10B60,
	0x10B73,
	0x10B80,
	0x10B92,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x11000,
	0x11047,
	0x11066,
	0x11070,
	0x1107F,
	0x110BB,
	0x110D0,
	0x110E9,
	0x110F0,
	0x110FA,
	0x11100,
	0x11135,
	0x11136,
	0x11140,
	0x11150,
	0x11174,
	0x11176,
	0x11177,
	0x11180,
	0x111C5,
	0x111CA,
	0x111CD,
	0x111D0,
	0x111DB,
	0x111DC,
	0x111DD,
	0x11200,
	0x11212,
	0x11213,
	0x11238,
	0x1123E,
	0x1123F,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112A9,
	0x112B0,
	0x112EB,
	0x112F0,
	0x112FA,
	0x11300,
	0x11304,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133C,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134E,
	0x11350,
	0x11351,
	0x11357,
	0x11358,
	0x1135D,
	0x11364,
	0x11366,
	0x1136D,
	0x11370,
	0x11375,
	0x11400,
	0x1144B,
	0x11450,
	0x1145A,
	0x11480,
	0x114C6,
	0x114C7,
	0x114C8,
	0x114D0,
	0x114DA,
	0x11580,
	0x115B6,
	0x115B8,
	0x115C1,
	0x115D8,
	0x115DE,
	0x11600,
	0x11641,
	0x11644,
	0x11645,
	0x11650,
	0x1165A,
	0x11680,
	0x116B8,
	0x116C0,
	0x116CA,
	0x11700,
	0x1171A,
	0x1171D,
	0x1172C,
	0x11730,
	0x1173A,
	0x118A0,
	0x118EA,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C37,
	0x11C38,
	0x11C41,
	0x11C50,
	0x11C5A,
	0x11C72,
	0x11C90,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16A60,
	0x16A6A,
	0x16AD0,
	0x16AEE,
	0x16AF0,
	0x16AF5,
	0x16B00,
	0x16B37,
	0x16B40,
	0x16B44,
	0x16B50,
	0x16B5A,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F7F,
	0x16F8F,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1BC9D,
	0x1BC9F,
	0x1D165,
	0x1D16A,
	0x1D16D,
	0x1D173,
	0x1D17B,
	0x1D183,
	0x1D185,
	0x1D18C,
	0x1D1AA,
	0x1D1AE,
	0x1D242,
	0x1D245,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D6C1,
	0x1D6C2,
	0x1D6DB,
	0x1D6DC,
	0x1D6FB,
	0x1D6FC,
	0x1D715,
	0x1D716,
	0x1D735,
	0x1D736,
	0x1D74F,
	0x1D750,
	0x1D76F,
	0x1D770,
	0x1D789,
	0x1D78A,
	0x1D7A9,
	0x1D7AA,
	0x1D7C3,
	0x1D7C4,
	0x1D7CC,
	0x1D7CE,
	0x1D800,
	0x1DA00,
	0x1DA37,
	0x1DA3B,
	0x1DA6D,
	0x1DA75,
	0x1DA76,
	0x1DA84,
	0x1DA85,
	0x1DA9B,
	0x1DAA0,
	0x1DAA1,
	0x1DAB0,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E800,
	0x1E8C5,
	0x1E8D0,
	0x1E8D7,
	0x1E900,
	0x1E94B,
	0x1E950,
	0x1E95A,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x20000,
	0x2A6D7,
	0x2A700,
	0x2B735,
	0x2B740,
	0x2B81E,
	0x2B820,
	0x2CEA2,
	0x2F800,
	0x2FA1E,
	0xE0100,
	0xE01F0
};

static const UV _Perl_IDStart_invlist[] = { /* for ASCII/Latin1 */
	1161,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x41,
	0x5B,
	0x5F,
	0x60,
	0x61,
	0x7B,
	0xAA,
	0xAB,
	0xB5,
	0xB6,
	0xBA,
	0xBB,
	0xC0,
	0xD7,
	0xD8,
	0xF7,
	0xF8,
	0x2C2,
	0x2C6,
	0x2D2,
	0x2E0,
	0x2E5,
	0x2EC,
	0x2ED,
	0x2EE,
	0x2EF,
	0x370,
	0x375,
	0x376,
	0x378,
	0x37B,
	0x37E,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x3F6,
	0x3F7,
	0x482,
	0x48A,
	0x530,
	0x531,
	0x557,
	0x559,
	0x55A,
	0x561,
	0x588,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F3,
	0x620,
	0x64B,
	0x66E,
	0x670,
	0x671,
	0x6D4,
	0x6D5,
	0x6D6,
	0x6E5,
	0x6E7,
	0x6EE,
	0x6F0,
	0x6FA,
	0x6FD,
	0x6FF,
	0x700,
	0x710,
	0x711,
	0x712,
	0x730,
	0x74D,
	0x7A6,
	0x7B1,
	0x7B2,
	0x7CA,
	0x7EB,
	0x7F4,
	0x7F6,
	0x7FA,
	0x7FB,
	0x800,
	0x816,
	0x81A,
	0x81B,
	0x824,
	0x825,
	0x828,
	0x829,
	0x840,
	0x859,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x904,
	0x93A,
	0x93D,
	0x93E,
	0x950,
	0x951,
	0x958,
	0x962,
	0x971,
	0x981,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BD,
	0x9BE,
	0x9CE,
	0x9CF,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E2,
	0x9F0,
	0x9F2,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA72,
	0xA75,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABD,
	0xABE,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE2,
	0xAF9,
	0xAFA,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3D,
	0xB3E,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB62,
	0xB71,
	0xB72,
	0xB83,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBD0,
	0xBD1,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC3E,
	0xC58,
	0xC5B,
	0xC60,
	0xC62,
	0xC80,
	0xC81,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBD,
	0xCBE,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE2,
	0xCF1,
	0xCF3,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD3E,
	0xD4E,
	0xD4F,
	0xD54,
	0xD57,
	0xD5F,
	0xD62,
	0xD7A,
	0xD80,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xE01,
	0xE31,
	0xE32,
	0xE33,
	0xE40,
	0xE47,
	0xE81,
	0xE83,
	0xE84,
	0xE85,
	0xE87,
	0xE89,
	0xE8A,
	0xE8B,
	0xE8D,
	0xE8E,
	0xE94,
	0xE98,
	0xE99,
	0xEA0,
	0xEA1,
	0xEA4,
	0xEA5,
	0xEA6,
	0xEA7,
	0xEA8,
	0xEAA,
	0xEAC,
	0xEAD,
	0xEB1,
	0xEB2,
	0xEB3,
	0xEBD,
	0xEBE,
	0xEC0,
	0xEC5,
	0xEC6,
	0xEC7,
	0xEDC,
	0xEE0,
	0xF00,
	0xF01,
	0xF40,
	0xF48,
	0xF49,
	0xF6D,
	0xF88,
	0xF8D,
	0x1000,
	0x102B,
	0x103F,
	0x1040,
	0x1050,
	0x1056,
	0x105A,
	0x105E,
	0x1061,
	0x1062,
	0x1065,
	0x1067,
	0x106E,
	0x1071,
	0x1075,
	0x1082,
	0x108E,
	0x108F,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x10FB,
	0x10FC,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x1380,
	0x1390,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1401,
	0x166D,
	0x166F,
	0x1680,
	0x1681,
	0x169B,
	0x16A0,
	0x16EB,
	0x16EE,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1712,
	0x1720,
	0x1732,
	0x1740,
	0x1752,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1780,
	0x17B4,
	0x17D7,
	0x17D8,
	0x17DC,
	0x17DD,
	0x1820,
	0x1878,
	0x1880,
	0x18A9,
	0x18AA,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1950,
	0x196E,
	0x1970,
	0x1975,
	0x1980,
	0x19AC,
	0x19B0,
	0x19CA,
	0x1A00,
	0x1A17,
	0x1A20,
	0x1A55,
	0x1AA7,
	0x1AA8,
	0x1B05,
	0x1B34,
	0x1B45,
	0x1B4C,
	0x1B83,
	0x1BA1,
	0x1BAE,
	0x1BB0,
	0x1BBA,
	0x1BE6,
	0x1C00,
	0x1C24,
	0x1C4D,
	0x1C50,
	0x1C5A,
	0x1C7E,
	0x1C80,
	0x1C89,
	0x1CE9,
	0x1CED,
	0x1CEE,
	0x1CF2,
	0x1CF5,
	0x1CF7,
	0x1D00,
	0x1DC0,
	0x1E00,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FBD,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FCD,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FE0,
	0x1FED,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFD,
	0x2071,
	0x2072,
	0x207F,
	0x2080,
	0x2090,
	0x209D,
	0x2102,
	0x2103,
	0x2107,
	0x2108,
	0x210A,
	0x2114,
	0x2115,
	0x2116,
	0x2119,
	0x211E,
	0x2124,
	0x2125,
	0x2126,
	0x2127,
	0x2128,
	0x2129,
	0x212A,
	0x212E,
	0x212F,
	0x213A,
	0x213C,
	0x2140,
	0x2145,
	0x214A,
	0x214E,
	0x214F,
	0x2160,
	0x2189,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CE5,
	0x2CEB,
	0x2CEF,
	0x2CF2,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D70,
	0x2D80,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x3005,
	0x3008,
	0x3021,
	0x302A,
	0x3031,
	0x3036,
	0x3038,
	0x303D,
	0x3041,
	0x3097,
	0x309D,
	0x30A0,
	0x30A1,
	0x30FB,
	0x30FC,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x31A0,
	0x31BB,
	0x31F0,
	0x3200,
	0x3400,
	0x4DB6,
	0x4E00,
	0x9FD6,
	0xA000,
	0xA48D,
	0xA4D0,
	0xA4FE,
	0xA500,
	0xA60D,
	0xA610,
	0xA620,
	0xA62A,
	0xA62C,
	0xA640,
	0xA66F,
	0xA67F,
	0xA69E,
	0xA6A0,
	0xA6F0,
	0xA717,
	0xA720,
	0xA722,
	0xA789,
	0xA78B,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F7,
	0xA802,
	0xA803,
	0xA806,
	0xA807,
	0xA80B,
	0xA80C,
	0xA823,
	0xA840,
	0xA874,
	0xA882,
	0xA8B4,
	0xA8F2,
	0xA8F8,
	0xA8FB,
	0xA8FC,
	0xA8FD,
	0xA8FE,
	0xA90A,
	0xA926,
	0xA930,
	0xA947,
	0xA960,
	0xA97D,
	0xA984,
	0xA9B3,
	0xA9CF,
	0xA9D0,
	0xA9E0,
	0xA9E5,
	0xA9E6,
	0xA9F0,
	0xA9FA,
	0xA9FF,
	0xAA00,
	0xAA29,
	0xAA40,
	0xAA43,
	0xAA44,
	0xAA4C,
	0xAA60,
	0xAA77,
	0xAA7A,
	0xAA7B,
	0xAA7E,
	0xAAB0,
	0xAAB1,
	0xAAB2,
	0xAAB5,
	0xAAB7,
	0xAAB9,
	0xAABE,
	0xAAC0,
	0xAAC1,
	0xAAC2,
	0xAAC3,
	0xAADB,
	0xAADE,
	0xAAE0,
	0xAAEB,
	0xAAF2,
	0xAAF5,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB5B,
	0xAB5C,
	0xAB66,
	0xAB70,
	0xABE3,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xF900,
	0xFA6E,
	0xFA70,
	0xFADA,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB1E,
	0xFB1F,
	0xFB29,
	0xFB2A,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFBB2,
	0xFBD3,
	0xFC5E,
	0xFC64,
	0xFD3E,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFA,
	0xFE71,
	0xFE72,
	0xFE73,
	0xFE74,
	0xFE77,
	0xFE78,
	0xFE79,
	0xFE7A,
	0xFE7B,
	0xFE7C,
	0xFE7D,
	0xFE7E,
	0xFE7F,
	0xFEFD,
	0xFF21,
	0xFF3B,
	0xFF41,
	0xFF5B,
	0xFF66,
	0xFF9E,
	0xFFA0,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10140,
	0x10175,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x10300,
	0x10320,
	0x10330,
	0x1034B,
	0x10350,
	0x10376,
	0x10380,
	0x1039E,
	0x103A0,
	0x103C4,
	0x103C8,
	0x103D0,
	0x103D1,
	0x103D6,
	0x10400,
	0x1049E,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10860,
	0x10877,
	0x10880,
	0x1089F,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x10900,
	0x10916,
	0x10920,
	0x1093A,
	0x10980,
	0x109B8,
	0x109BE,
	0x109C0,
	0x10A00,
	0x10A01,
	0x10A10,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A60,
	0x10A7D,
	0x10A80,
	0x10A9D,
	0x10AC0,
	0x10AC8,
	0x10AC9,
	0x10AE5,
	0x10B00,
	0x10B36,
	0x10B40,
	0x10B56,
	0x10B60,
	0x10B73,
	0x10B80,
	0x10B92,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x11003,
	0x11038,
	0x11083,
	0x110B0,
	0x110D0,
	0x110E9,
	0x11103,
	0x11127,
	0x11150,
	0x11173,
	0x11176,
	0x11177,
	0x11183,
	0x111B3,
	0x111C1,
	0x111C5,
	0x111DA,
	0x111DB,
	0x111DC,
	0x111DD,
	0x11200,
	0x11212,
	0x11213,
	0x1122C,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112A9,
	0x112B0,
	0x112DF,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133D,
	0x1133E,
	0x11350,
	0x11351,
	0x1135D,
	0x11362,
	0x11400,
	0x11435,
	0x11447,
	0x1144B,
	0x11480,
	0x114B0,
	0x114C4,
	0x114C6,
	0x114C7,
	0x114C8,
	0x11580,
	0x115AF,
	0x115D8,
	0x115DC,
	0x11600,
	0x11630,
	0x11644,
	0x11645,
	0x11680,
	0x116AB,
	0x11700,
	0x1171A,
	0x118A0,
	0x118E0,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C2F,
	0x11C40,
	0x11C41,
	0x11C72,
	0x11C90,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16AD0,
	0x16AEE,
	0x16B00,
	0x16B30,
	0x16B40,
	0x16B44,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F51,
	0x16F93,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D6C1,
	0x1D6C2,
	0x1D6DB,
	0x1D6DC,
	0x1D6FB,
	0x1D6FC,
	0x1D715,
	0x1D716,
	0x1D735,
	0x1D736,
	0x1D74F,
	0x1D750,
	0x1D76F,
	0x1D770,
	0x1D789,
	0x1D78A,
	0x1D7A9,
	0x1D7AA,
	0x1D7C3,
	0x1D7C4,
	0x1D7CC,
	0x1E800,
	0x1E8C5,
	0x1E900,
	0x1E944,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x20000,
	0x2A6D7,
	0x2A700,
	0x2B735,
	0x2B740,
	0x2B81E,
	0x2B820,
	0x2CEA2,
	0x2F800,
	0x2FA1E
};

#endif	/* defined(PERL_IN_UTF8_C) */

#if defined(PERL_IN_PERL_C)

static const UV _Perl_LB_invlist[] = { /* for ASCII/Latin1 */
	2267,	/* Number of elements */
	148565664, /* Version and data structure type */
	0,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x9,
	0xA,
	0xB,
	0xD,
	0xE,
	0x20,
	0x21,
	0x22,
	0x23,
	0x24,
	0x25,
	0x26,
	0x27,
	0x28,
	0x29,
	0x2A,
	0x2B,
	0x2C,
	0x2D,
	0x2E,
	0x2F,
	0x30,
	0x3A,
	0x3C,
	0x3F,
	0x40,
	0x5B,
	0x5C,
	0x5D,
	0x5E,
	0x7B,
	0x7C,
	0x7D,
	0x7E,
	0x7F,
	0x85,
	0x86,
	0xA0,
	0xA1,
	0xA2,
	0xA3,
	0xA6,
	0xAB,
	0xAC,
	0xAD,
	0xAE,
	0xB0,
	0xB1,
	0xB2,
	0xB4,
	0xB5,
	0xBB,
	0xBC,
	0xBF,
	0xC0,
	0x2C8,
	0x2C9,
	0x2CC,
	0x2CD,
	0x2DF,
	0x2E0,
	0x300,
	0x34F,
	0x350,
	0x35C,
	0x363,
	0x370,
	0x37E,
	0x37F,
	0x483,
	0x48A,
	0x589,
	0x58A,
	0x58B,
	0x58F,
	0x590,
	0x591,
	0x5BE,
	0x5BF,
	0x5C0,
	0x5C1,
	0x5C3,
	0x5C4,
	0x5C6,
	0x5C7,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F3,
	0x609,
	0x60C,
	0x60E,
	0x610,
	0x61B,
	0x61C,
	0x61D,
	0x61E,
	0x620,
	0x64B,
	0x660,
	0x66A,
	0x66B,
	0x66D,
	0x670,
	0x671,
	0x6D4,
	0x6D5,
	0x6D6,
	0x6DD,
	0x6DF,
	0x6E5,
	0x6E7,
	0x6E9,
	0x6EA,
	0x6EE,
	0x6F0,
	0x6FA,
	0x711,
	0x712,
	0x730,
	0x74B,
	0x7A6,
	0x7B1,
	0x7C0,
	0x7CA,
	0x7EB,
	0x7F4,
	0x7F8,
	0x7F9,
	0x7FA,
	0x816,
	0x81A,
	0x81B,
	0x824,
	0x825,
	0x828,
	0x829,
	0x82E,
	0x859,
	0x85C,
	0x8D4,
	0x8E2,
	0x8E3,
	0x904,
	0x93A,
	0x93D,
	0x93E,
	0x950,
	0x951,
	0x958,
	0x962,
	0x964,
	0x966,
	0x970,
	0x981,
	0x984,
	0x9BC,
	0x9BD,
	0x9BE,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CE,
	0x9D7,
	0x9D8,
	0x9E2,
	0x9E4,
	0x9E6,
	0x9F0,
	0x9F2,
	0x9F4,
	0x9F9,
	0x9FA,
	0x9FB,
	0x9FC,
	0xA01,
	0xA04,
	0xA3C,
	0xA3D,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4E,
	0xA51,
	0xA52,
	0xA66,
	0xA70,
	0xA72,
	0xA75,
	0xA76,
	0xA81,
	0xA84,
	0xABC,
	0xABD,
	0xABE,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACE,
	0xAE2,
	0xAE4,
	0xAE6,
	0xAF0,
	0xAF1,
	0xAF2,
	0xB01,
	0xB04,
	0xB3C,
	0xB3D,
	0xB3E,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4E,
	0xB56,
	0xB58,
	0xB62,
	0xB64,
	0xB66,
	0xB70,
	0xB82,
	0xB83,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCE,
	0xBD7,
	0xBD8,
	0xBE6,
	0xBF0,
	0xBF9,
	0xBFA,
	0xC00,
	0xC04,
	0xC3E,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4E,
	0xC55,
	0xC57,
	0xC62,
	0xC64,
	0xC66,
	0xC70,
	0xC81,
	0xC84,
	0xCBC,
	0xCBD,
	0xCBE,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCE,
	0xCD5,
	0xCD7,
	0xCE2,
	0xCE4,
	0xCE6,
	0xCF0,
	0xD01,
	0xD04,
	0xD3E,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD4E,
	0xD57,
	0xD58,
	0xD62,
	0xD64,
	0xD66,
	0xD70,
	0xD79,
	0xD7A,
	0xD82,
	0xD84,
	0xDCA,
	0xDCB,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDE6,
	0xDF0,
	0xDF2,
	0xDF4,
	0xE31,
	0xE32,
	0xE34,
	0xE3B,
	0xE3F,
	0xE40,
	0xE47,
	0xE4F,
	0xE50,
	0xE5A,
	0xE5C,
	0xEB1,
	0xEB2,
	0xEB4,
	0xEBA,
	0xEBB,
	0xEBD,
	0xEC8,
	0xECE,
	0xED0,
	0xEDA,
	0xF01,
	0xF05,
	0xF06,
	0xF08,
	0xF09,
	0xF0B,
	0xF0C,
	0xF0D,
	0xF12,
	0xF13,
	0xF14,
	0xF15,
	0xF18,
	0xF1A,
	0xF20,
	0xF2A,
	0xF34,
	0xF35,
	0xF36,
	0xF37,
	0xF38,
	0xF39,
	0xF3A,
	0xF3B,
	0xF3C,
	0xF3D,
	0xF3E,
	0xF40,
	0xF71,
	0xF7F,
	0xF80,
	0xF85,
	0xF86,
	0xF88,
	0xF8D,
	0xF98,
	0xF99,
	0xFBD,
	0xFBE,
	0xFC0,
	0xFC6,
	0xFC7,
	0xFD0,
	0xFD2,
	0xFD3,
	0xFD4,
	0xFD9,
	0xFDB,
	0x102B,
	0x103F,
	0x1040,
	0x104A,
	0x104C,
	0x1056,
	0x105A,
	0x105E,
	0x1061,
	0x1062,
	0x1065,
	0x1067,
	0x106E,
	0x1071,
	0x1075,
	0x1082,
	0x108E,
	0x108F,
	0x1090,
	0x109A,
	0x109E,
	0x1100,
	0x1160,
	0x11A8,
	0x1200,
	0x135D,
	0x1360,
	0x1361,
	0x1362,
	0x1400,
	0x1401,
	0x1680,
	0x1681,
	0x169B,
	0x169C,
	0x169D,
	0x16EB,
	0x16EE,
	0x1712,
	0x1715,
	0x1732,
	0x1735,
	0x1737,
	0x1752,
	0x1754,
	0x1772,
	0x1774,
	0x17B4,
	0x17D4,
	0x17D6,
	0x17D7,
	0x17D8,
	0x17D9,
	0x17DA,
	0x17DB,
	0x17DC,
	0x17DD,
	0x17DE,
	0x17E0,
	0x17EA,
	0x1802,
	0x1804,
	0x1806,
	0x1807,
	0x1808,
	0x180A,
	0x180B,
	0x180E,
	0x180F,
	0x1810,
	0x181A,
	0x1885,
	0x1887,
	0x18A9,
	0x18AA,
	0x1920,
	0x192C,
	0x1930,
	0x193C,
	0x1944,
	0x1946,
	0x1950,
	0x19D0,
	0x19DA,
	0x1A17,
	0x1A1C,
	0x1A55,
	0x1A5F,
	0x1A60,
	0x1A7D,
	0x1A7F,
	0x1A80,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1AB0,
	0x1ABF,
	0x1B00,
	0x1B05,
	0x1B34,
	0x1B45,
	0x1B50,
	0x1B5A,
	0x1B5C,
	0x1B5D,
	0x1B61,
	0x1B6B,
	0x1B74,
	0x1B80,
	0x1B83,
	0x1BA1,
	0x1BAE,
	0x1BB0,
	0x1BBA,
	0x1BE6,
	0x1BF4,
	0x1C24,
	0x1C38,
	0x1C3B,
	0x1C40,
	0x1C4A,
	0x1C50,
	0x1C5A,
	0x1C7E,
	0x1C80,
	0x1CD0,
	0x1CD3,
	0x1CD4,
	0x1CE9,
	0x1CED,
	0x1CEE,
	0x1CF2,
	0x1CF5,
	0x1CF8,
	0x1CFA,
	0x1DC0,
	0x1DF6,
	0x1DFB,
	0x1E00,
	0x1FFD,
	0x1FFE,
	0x2000,
	0x2007,
	0x2008,
	0x200B,
	0x200C,
	0x200D,
	0x200E,
	0x2010,
	0x2011,
	0x2012,
	0x2014,
	0x2015,
	0x2018,
	0x201A,
	0x201B,
	0x201E,
	0x201F,
	0x2020,
	0x2024,
	0x2027,
	0x2028,
	0x202A,
	0x202F,
	0x2030,
	0x2038,
	0x2039,
	0x203B,
	0x203C,
	0x203E,
	0x2044,
	0x2045,
	0x2046,
	0x2047,
	0x204A,
	0x2056,
	0x2057,
	0x2058,
	0x205C,
	0x205D,
	0x2060,
	0x2061,
	0x2066,
	0x2070,
	0x207D,
	0x207E,
	0x207F,
	0x208D,
	0x208E,
	0x208F,
	0x20A0,
	0x20A7,
	0x20A8,
	0x20B6,
	0x20B7,
	0x20BB,
	0x20BC,
	0x20BE,
	0x20BF,
	0x20D0,
	0x20F1,
	0x2103,
	0x2104,
	0x2109,
	0x210A,
	0x2116,
	0x2117,
	0x2212,
	0x2214,
	0x22EF,
	0x22F0,
	0x2308,
	0x2309,
	0x230A,
	0x230B,
	0x230C,
	0x231A,
	0x231C,
	0x2329,
	0x232A,
	0x232B,
	0x23F0,
	0x23F4,
	0x2600,
	0x2604,
	0x2614,
	0x2616,
	0x2618,
	0x2619,
	0x261A,
	0x261D,
	0x261E,
	0x2620,
	0x2639,
	0x263C,
	0x2668,
	0x2669,
	0x267F,
	0x2680,
	0x26BD,
	0x26C9,
	0x26CD,
	0x26CE,
	0x26CF,
	0x26D2,
	0x26D3,
	0x26D5,
	0x26D8,
	0x26DA,
	0x26DC,
	0x26DD,
	0x26DF,
	0x26E2,
	0x26EA,
	0x26EB,
	0x26F1,
	0x26F6,
	0x26F7,
	0x26F9,
	0x26FA,
	0x26FB,
	0x26FD,
	0x2705,
	0x2708,
	0x270A,
	0x270E,
	0x275B,
	0x2761,
	0x2762,
	0x2764,
	0x2765,
	0x2768,
	0x2769,
	0x276A,
	0x276B,
	0x276C,
	0x276D,
	0x276E,
	0x276F,
	0x2770,
	0x2771,
	0x2772,
	0x2773,
	0x2774,
	0x2775,
	0x2776,
	0x27C5,
	0x27C6,
	0x27C7,
	0x27E6,
	0x27E7,
	0x27E8,
	0x27E9,
	0x27EA,
	0x27EB,
	0x27EC,
	0x27ED,
	0x27EE,
	0x27EF,
	0x27F0,
	0x2983,
	0x2984,
	0x2985,
	0x2986,
	0x2987,
	0x2988,
	0x2989,
	0x298A,
	0x298B,
	0x298C,
	0x298D,
	0x298E,
	0x298F,
	0x2990,
	0x2991,
	0x2992,
	0x2993,
	0x2994,
	0x2995,
	0x2996,
	0x2997,
	0x2998,
	0x2999,
	0x29D8,
	0x29D9,
	0x29DA,
	0x29DB,
	0x29DC,
	0x29FC,
	0x29FD,
	0x29FE,
	0x2CEF,
	0x2CF2,
	0x2CF9,
	0x2CFA,
	0x2CFD,
	0x2CFE,
	0x2CFF,
	0x2D00,
	0x2D70,
	0x2D71,
	0x2D7F,
	0x2D80,
	0x2DE0,
	0x2E00,
	0x2E0E,
	0x2E16,
	0x2E17,
	0x2E18,
	0x2E19,
	0x2E1A,
	0x2E1C,
	0x2E1E,
	0x2E20,
	0x2E22,
	0x2E23,
	0x2E24,
	0x2E25,
	0x2E26,
	0x2E27,
	0x2E28,
	0x2E29,
	0x2E2A,
	0x2E2E,
	0x2E2F,
	0x2E30,
	0x2E32,
	0x2E33,
	0x2E35,
	0x2E3A,
	0x2E3C,
	0x2E3F,
	0x2E40,
	0x2E42,
	0x2E43,
	0x2E45,
	0x2E80,
	0x2E9A,
	0x2E9B,
	0x2EF4,
	0x2F00,
	0x2FD6,
	0x2FF0,
	0x2FFC,
	0x3000,
	0x3001,
	0x3003,
	0x3005,
	0x3006,
	0x3008,
	0x3009,
	0x300A,
	0x300B,
	0x300C,
	0x300D,
	0x300E,
	0x300F,
	0x3010,
	0x3011,
	0x3012,
	0x3014,
	0x3015,
	0x3016,
	0x3017,
	0x3018,
	0x3019,
	0x301A,
	0x301B,
	0x301C,
	0x301D,
	0x301E,
	0x3020,
	0x302A,
	0x3030,
	0x3035,
	0x3036,
	0x303B,
	0x303D,
	0x3040,
	0x3041,
	0x3042,
	0x3043,
	0x3044,
	0x3045,
	0x3046,
	0x3047,
	0x3048,
	0x3049,
	0x304A,
	0x3063,
	0x3064,
	0x3083,
	0x3084,
	0x3085,
	0x3086,
	0x3087,
	0x3088,
	0x308E,
	0x308F,
	0x3095,
	0x3097,
	0x3099,
	0x309B,
	0x309F,
	0x30A0,
	0x30A2,
	0x30A3,
	0x30A4,
	0x30A5,
	0x30A6,
	0x30A7,
	0x30A8,
	0x30A9,
	0x30AA,
	0x30C3,
	0x30C4,
	0x30E3,
	0x30E4,
	0x30E5,
	0x30E6,
	0x30E7,
	0x30E8,
	0x30EE,
	0x30EF,
	0x30F5,
	0x30F7,
	0x30FB,
	0x30FF,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x3190,
	0x31BB,
	0x31C0,
	0x31E4,
	0x31F0,
	0x3200,
	0x321F,
	0x3220,
	0x3248,
	0x3250,
	0x32FF,
	0x3300,
	0x4DC0,
	0x4E00,
	0xA015,
	0xA016,
	0xA48D,
	0xA490,
	0xA4C7,
	0xA4FE,
	0xA500,
	0xA60D,
	0xA60E,
	0xA60F,
	0xA610,
	0xA620,
	0xA62A,
	0xA66F,
	0xA673,
	0xA674,
	0xA67E,
	0xA69E,
	0xA6A0,
	0xA6F0,
	0xA6F2,
	0xA6F3,
	0xA6F8,
	0xA802,
	0xA803,
	0xA806,
	0xA807,
	0xA80B,
	0xA80C,
	0xA823,
	0xA828,
	0xA838,
	0xA839,
	0xA874,
	0xA876,
	0xA878,
	0xA880,
	0xA882,
	0xA8B4,
	0xA8C6,
	0xA8CE,
	0xA8D0,
	0xA8DA,
	0xA8E0,
	0xA8F2,
	0xA8FC,
	0xA8FD,
	0xA900,
	0xA90A,
	0xA926,
	0xA92E,
	0xA930,
	0xA947,
	0xA954,
	0xA960,
	0xA97D,
	0xA980,
	0xA984,
	0xA9B3,
	0xA9C1,
	0xA9C7,
	0xA9CA,
	0xA9D0,
	0xA9DA,
	0xA9E5,
	0xA9E6,
	0xA9F0,
	0xA9FA,
	0xAA29,
	0xAA37,
	0xAA43,
	0xAA44,
	0xAA4C,
	0xAA4E,
	0xAA50,
	0xAA5A,
	0xAA5D,
	0xAA60,
	0xAA7B,
	0xAA7E,
	0xAAB0,
	0xAAB1,
	0xAAB2,
	0xAAB5,
	0xAAB7,
	0xAAB9,
	0xAABE,
	0xAAC0,
	0xAAC1,
	0xAAC2,
	0xAAEB,
	0xAAF0,
	0xAAF2,
	0xAAF5,
	0xAAF7,
	0xABE3,
	0xABEB,
	0xABEC,
	0xABEE,
	0xABF0,
	0xABFA,
	0xAC00,
	0xAC01,
	0xAC1C,
	0xAC1D,
	0xAC38,
	0xAC39,
	0xAC54,
	0xAC55,
	0xAC70,
	0xAC71,
	0xAC8C,
	0xAC8D,
	0xACA8,
	0xACA9,
	0xACC4,
	0xACC5,
	0xACE0,
	0xACE1,
	0xACFC,
	0xACFD,
	0xAD18,
	0xAD19,
	0xAD34,
	0xAD35,
	0xAD50,
	0xAD51,
	0xAD6C,
	0xAD6D,
	0xAD88,
	0xAD89,
	0xADA4,
	0xADA5,
	0xADC0,
	0xADC1,
	0xADDC,
	0xADDD,
	0xADF8,
	0xADF9,
	0xAE14,
	0xAE15,
	0xAE30,
	0xAE31,
	0xAE4C,
	0xAE4D,
	0xAE68,
	0xAE69,
	0xAE84,
	0xAE85,
	0xAEA0,
	0xAEA1,
	0xAEBC,
	0xAEBD,
	0xAED8,
	0xAED9,
	0xAEF4,
	0xAEF5,
	0xAF10,
	0xAF11,
	0xAF2C,
	0xAF2D,
	0xAF48,
	0xAF49,
	0xAF64,
	0xAF65,
	0xAF80,
	0xAF81,
	0xAF9C,
	0xAF9D,
	0xAFB8,
	0xAFB9,
	0xAFD4,
	0xAFD5,
	0xAFF0,
	0xAFF1,
	0xB00C,
	0xB00D,
	0xB028,
	0xB029,
	0xB044,
	0xB045,
	0xB060,
	0xB061,
	0xB07C,
	0xB07D,
	0xB098,
	0xB099,
	0xB0B4,
	0xB0B5,
	0xB0D0,
	0xB0D1,
	0xB0EC,
	0xB0ED,
	0xB108,
	0xB109,
	0xB124,
	0xB125,
	0xB140,
	0xB141,
	0xB15C,
	0xB15D,
	0xB178,
	0xB179,
	0xB194,
	0xB195,
	0xB1B0,
	0xB1B1,
	0xB1CC,
	0xB1CD,
	0xB1E8,
	0xB1E9,
	0xB204,
	0xB205,
	0xB220,
	0xB221,
	0xB23C,
	0xB23D,
	0xB258,
	0xB259,
	0xB274,
	0xB275,
	0xB290,
	0xB291,
	0xB2AC,
	0xB2AD,
	0xB2C8,
	0xB2C9,
	0xB2E4,
	0xB2E5,
	0xB300,
	0xB301,
	0xB31C,
	0xB31D,
	0xB338,
	0xB339,
	0xB354,
	0xB355,
	0xB370,
	0xB371,
	0xB38C,
	0xB38D,
	0xB3A8,
	0xB3A9,
	0xB3C4,
	0xB3C5,
	0xB3E0,
	0xB3E1,
	0xB3FC,
	0xB3FD,
	0xB418,
	0xB419,
	0xB434,
	0xB435,
	0xB450,
	0xB451,
	0xB46C,
	0xB46D,
	0xB488,
	0xB489,
	0xB4A4,
	0xB4A5,
	0xB4C0,
	0xB4C1,
	0xB4DC,
	0xB4DD,
	0xB4F8,
	0xB4F9,
	0xB514,
	0xB515,
	0xB530,
	0xB531,
	0xB54C,
	0xB54D,
	0xB568,
	0xB569,
	0xB584,
	0xB585,
	0xB5A0,
	0xB5A1,
	0xB5BC,
	0xB5BD,
	0xB5D8,
	0xB5D9,
	0xB5F4,
	0xB5F5,
	0xB610,
	0xB611,
	0xB62C,
	0xB62D,
	0xB648,
	0xB649,
	0xB664,
	0xB665,
	0xB680,
	0xB681,
	0xB69C,
	0xB69D,
	0xB6B8,
	0xB6B9,
	0xB6D4,
	0xB6D5,
	0xB6F0,
	0xB6F1,
	0xB70C,
	0xB70D,
	0xB728,
	0xB729,
	0xB744,
	0xB745,
	0xB760,
	0xB761,
	0xB77C,
	0xB77D,
	0xB798,
	0xB799,
	0xB7B4,
	0xB7B5,
	0xB7D0,
	0xB7D1,
	0xB7EC,
	0xB7ED,
	0xB808,
	0xB809,
	0xB824,
	0xB825,
	0xB840,
	0xB841,
	0xB85C,
	0xB85D,
	0xB878,
	0xB879,
	0xB894,
	0xB895,
	0xB8B0,
	0xB8B1,
	0xB8CC,
	0xB8CD,
	0xB8E8,
	0xB8E9,
	0xB904,
	0xB905,
	0xB920,
	0xB921,
	0xB93C,
	0xB93D,
	0xB958,
	0xB959,
	0xB974,
	0xB975,
	0xB990,
	0xB991,
	0xB9AC,
	0xB9AD,
	0xB9C8,
	0xB9C9,
	0xB9E4,
	0xB9E5,
	0xBA00,
	0xBA01,
	0xBA1C,
	0xBA1D,
	0xBA38,
	0xBA39,
	0xBA54,
	0xBA55,
	0xBA70,
	0xBA71,
	0xBA8C,
	0xBA8D,
	0xBAA8,
	0xBAA9,
	0xBAC4,
	0xBAC5,
	0xBAE0,
	0xBAE1,
	0xBAFC,
	0xBAFD,
	0xBB18,
	0xBB19,
	0xBB34,
	0xBB35,
	0xBB50,
	0xBB51,
	0xBB6C,
	0xBB6D,
	0xBB88,
	0xBB89,
	0xBBA4,
	0xBBA5,
	0xBBC0,
	0xBBC1,
	0xBBDC,
	0xBBDD,
	0xBBF8,
	0xBBF9,
	0xBC14,
	0xBC15,
	0xBC30,
	0xBC31,
	0xBC4C,
	0xBC4D,
	0xBC68,
	0xBC69,
	0xBC84,
	0xBC85,
	0xBCA0,
	0xBCA1,
	0xBCBC,
	0xBCBD,
	0xBCD8,
	0xBCD9,
	0xBCF4,
	0xBCF5,
	0xBD10,
	0xBD11,
	0xBD2C,
	0xBD2D,
	0xBD48,
	0xBD49,
	0xBD64,
	0xBD65,
	0xBD80,
	0xBD81,
	0xBD9C,
	0xBD9D,
	0xBDB8,
	0xBDB9,
	0xBDD4,
	0xBDD5,
	0xBDF0,
	0xBDF1,
	0xBE0C,
	0xBE0D,
	0xBE28,
	0xBE29,
	0xBE44,
	0xBE45,
	0xBE60,
	0xBE61,
	0xBE7C,
	0xBE7D,
	0xBE98,
	0xBE99,
	0xBEB4,
	0xBEB5,
	0xBED0,
	0xBED1,
	0xBEEC,
	0xBEED,
	0xBF08,
	0xBF09,
	0xBF24,
	0xBF25,
	0xBF40,
	0xBF41,
	0xBF5C,
	0xBF5D,
	0xBF78,
	0xBF79,
	0xBF94,
	0xBF95,
	0xBFB0,
	0xBFB1,
	0xBFCC,
	0xBFCD,
	0xBFE8,
	0xBFE9,
	0xC004,
	0xC005,
	0xC020,
	0xC021,
	0xC03C,
	0xC03D,
	0xC058,
	0xC059,
	0xC074,
	0xC075,
	0xC090,
	0xC091,
	0xC0AC,
	0xC0AD,
	0xC0C8,
	0xC0C9,
	0xC0E4,
	0xC0E5,
	0xC100,
	0xC101,
	0xC11C,
	0xC11D,
	0xC138,
	0xC139,
	0xC154,
	0xC155,
	0xC170,
	0xC171,
	0xC18C,
	0xC18D,
	0xC1A8,
	0xC1A9,
	0xC1C4,
	0xC1C5,
	0xC1E0,
	0xC1E1,
	0xC1FC,
	0xC1FD,
	0xC218,
	0xC219,
	0xC234,
	0xC235,
	0xC250,
	0xC251,
	0xC26C,
	0xC26D,
	0xC288,
	0xC289,
	0xC2A4,
	0xC2A5,
	0xC2C0,
	0xC2C1,
	0xC2DC,
	0xC2DD,
	0xC2F8,
	0xC2F9,
	0xC314,
	0xC315,
	0xC330,
	0xC331,
	0xC34C,
	0xC34D,
	0xC368,
	0xC369,
	0xC384,
	0xC385,
	0xC3A0,
	0xC3A1,
	0xC3BC,
	0xC3BD,
	0xC3D8,
	0xC3D9,
	0xC3F4,
	0xC3F5,
	0xC410,
	0xC411,
	0xC42C,
	0xC42D,
	0xC448,
	0xC449,
	0xC464,
	0xC465,
	0xC480,
	0xC481,
	0xC49C,
	0xC49D,
	0xC4B8,
	0xC4B9,
	0xC4D4,
	0xC4D5,
	0xC4F0,
	0xC4F1,
	0xC50C,
	0xC50D,
	0xC528,
	0xC529,
	0xC544,
	0xC545,
	0xC560,
	0xC561,
	0xC57C,
	0xC57D,
	0xC598,
	0xC599,
	0xC5B4,
	0xC5B5,
	0xC5D0,
	0xC5D1,
	0xC5EC,
	0xC5ED,
	0xC608,
	0xC609,
	0xC624,
	0xC625,
	0xC640,
	0xC641,
	0xC65C,
	0xC65D,
	0xC678,
	0xC679,
	0xC694,
	0xC695,
	0xC6B0,
	0xC6B1,
	0xC6CC,
	0xC6CD,
	0xC6E8,
	0xC6E9,
	0xC704,
	0xC705,
	0xC720,
	0xC721,
	0xC73C,
	0xC73D,
	0xC758,
	0xC759,
	0xC774,
	0xC775,
	0xC790,
	0xC791,
	0xC7AC,
	0xC7AD,
	0xC7C8,
	0xC7C9,
	0xC7E4,
	0xC7E5,
	0xC800,
	0xC801,
	0xC81C,
	0xC81D,
	0xC838,
	0xC839,
	0xC854,
	0xC855,
	0xC870,
	0xC871,
	0xC88C,
	0xC88D,
	0xC8A8,
	0xC8A9,
	0xC8C4,
	0xC8C5,
	0xC8E0,
	0xC8E1,
	0xC8FC,
	0xC8FD,
	0xC918,
	0xC919,
	0xC934,
	0xC935,
	0xC950,
	0xC951,
	0xC96C,
	0xC96D,
	0xC988,
	0xC989,
	0xC9A4,
	0xC9A5,
	0xC9C0,
	0xC9C1,
	0xC9DC,
	0xC9DD,
	0xC9F8,
	0xC9F9,
	0xCA14,
	0xCA15,
	0xCA30,
	0xCA31,
	0xCA4C,
	0xCA4D,
	0xCA68,
	0xCA69,
	0xCA84,
	0xCA85,
	0xCAA0,
	0xCAA1,
	0xCABC,
	0xCABD,
	0xCAD8,
	0xCAD9,
	0xCAF4,
	0xCAF5,
	0xCB10,
	0xCB11,
	0xCB2C,
	0xCB2D,
	0xCB48,
	0xCB49,
	0xCB64,
	0xCB65,
	0xCB80,
	0xCB81,
	0xCB9C,
	0xCB9D,
	0xCBB8,
	0xCBB9,
	0xCBD4,
	0xCBD5,
	0xCBF0,
	0xCBF1,
	0xCC0C,
	0xCC0D,
	0xCC28,
	0xCC29,
	0xCC44,
	0xCC45,
	0xCC60,
	0xCC61,
	0xCC7C,
	0xCC7D,
	0xCC98,
	0xCC99,
	0xCCB4,
	0xCCB5,
	0xCCD0,
	0xCCD1,
	0xCCEC,
	0xCCED,
	0xCD08,
	0xCD09,
	0xCD24,
	0xCD25,
	0xCD40,
	0xCD41,
	0xCD5C,
	0xCD5D,
	0xCD78,
	0xCD79,
	0xCD94,
	0xCD95,
	0xCDB0,
	0xCDB1,
	0xCDCC,
	0xCDCD,
	0xCDE8,
	0xCDE9,
	0xCE04,
	0xCE05,
	0xCE20,
	0xCE21,
	0xCE3C,
	0xCE3D,
	0xCE58,
	0xCE59,
	0xCE74,
	0xCE75,
	0xCE90,
	0xCE91,
	0xCEAC,
	0xCEAD,
	0xCEC8,
	0xCEC9,
	0xCEE4,
	0xCEE5,
	0xCF00,
	0xCF01,
	0xCF1C,
	0xCF1D,
	0xCF38,
	0xCF39,
	0xCF54,
	0xCF55,
	0xCF70,
	0xCF71,
	0xCF8C,
	0xCF8D,
	0xCFA8,
	0xCFA9,
	0xCFC4,
	0xCFC5,
	0xCFE0,
	0xCFE1,
	0xCFFC,
	0xCFFD,
	0xD018,
	0xD019,
	0xD034,
	0xD035,
	0xD050,
	0xD051,
	0xD06C,
	0xD06D,
	0xD088,
	0xD089,
	0xD0A4,
	0xD0A5,
	0xD0C0,
	0xD0C1,
	0xD0DC,
	0xD0DD,
	0xD0F8,
	0xD0F9,
	0xD114,
	0xD115,
	0xD130,
	0xD131,
	0xD14C,
	0xD14D,
	0xD168,
	0xD169,
	0xD184,
	0xD185,
	0xD1A0,
	0xD1A1,
	0xD1BC,
	0xD1BD,
	0xD1D8,
	0xD1D9,
	0xD1F4,
	0xD1F5,
	0xD210,
	0xD211,
	0xD22C,
	0xD22D,
	0xD248,
	0xD249,
	0xD264,
	0xD265,
	0xD280,
	0xD281,
	0xD29C,
	0xD29D,
	0xD2B8,
	0xD2B9,
	0xD2D4,
	0xD2D5,
	0xD2F0,
	0xD2F1,
	0xD30C,
	0xD30D,
	0xD328,
	0xD329,
	0xD344,
	0xD345,
	0xD360,
	0xD361,
	0xD37C,
	0xD37D,
	0xD398,
	0xD399,
	0xD3B4,
	0xD3B5,
	0xD3D0,
	0xD3D1,
	0xD3EC,
	0xD3ED,
	0xD408,
	0xD409,
	0xD424,
	0xD425,
	0xD440,
	0xD441,
	0xD45C,
	0xD45D,
	0xD478,
	0xD479,
	0xD494,
	0xD495,
	0xD4B0,
	0xD4B1,
	0xD4CC,
	0xD4CD,
	0xD4E8,
	0xD4E9,
	0xD504,
	0xD505,
	0xD520,
	0xD521,
	0xD53C,
	0xD53D,
	0xD558,
	0xD559,
	0xD574,
	0xD575,
	0xD590,
	0xD591,
	0xD5AC,
	0xD5AD,
	0xD5C8,
	0xD5C9,
	0xD5E4,
	0xD5E5,
	0xD600,
	0xD601,
	0xD61C,
	0xD61D,
	0xD638,
	0xD639,
	0xD654,
	0xD655,
	0xD670,
	0xD671,
	0xD68C,
	0xD68D,
	0xD6A8,
	0xD6A9,
	0xD6C4,
	0xD6C5,
	0xD6E0,
	0xD6E1,
	0xD6FC,
	0xD6FD,
	0xD718,
	0xD719,
	0xD734,
	0xD735,
	0xD750,
	0xD751,
	0xD76C,
	0xD76D,
	0xD788,
	0xD789,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xF900,
	0xFB00,
	0xFB1D,
	0xFB1E,
	0xFB1F,
	0xFB29,
	0xFB2A,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFB50,
	0xFD3E,
	0xFD3F,
	0xFD40,
	0xFDFC,
	0xFDFD,
	0xFE00,
	0xFE10,
	0xFE11,
	0xFE13,
	0xFE15,
	0xFE17,
	0xFE18,
	0xFE19,
	0xFE1A,
	0xFE20,
	0xFE30,
	0xFE35,
	0xFE36,
	0xFE37,
	0xFE38,
	0xFE39,
	0xFE3A,
	0xFE3B,
	0xFE3C,
	0xFE3D,
	0xFE3E,
	0xFE3F,
	0xFE40,
	0xFE41,
	0xFE42,
	0xFE43,
	0xFE44,
	0xFE45,
	0xFE47,
	0xFE48,
	0xFE49,
	0xFE50,
	0xFE51,
	0xFE52,
	0xFE53,
	0xFE54,
	0xFE56,
	0xFE58,
	0xFE59,
	0xFE5A,
	0xFE5B,
	0xFE5C,
	0xFE5D,
	0xFE5E,
	0xFE5F,
	0xFE67,
	0xFE68,
	0xFE69,
	0xFE6A,
	0xFE6B,
	0xFE6C,
	0xFEFF,
	0xFF00,
	0xFF01,
	0xFF02,
	0xFF04,
	0xFF05,
	0xFF06,
	0xFF08,
	0xFF09,
	0xFF0A,
	0xFF0C,
	0xFF0D,
	0xFF0E,
	0xFF0F,
	0xFF1A,
	0xFF1C,
	0xFF1F,
	0xFF20,
	0xFF3B,
	0xFF3C,
	0xFF3D,
	0xFF3E,
	0xFF5B,
	0xFF5C,
	0xFF5D,
	0xFF5E,
	0xFF5F,
	0xFF60,
	0xFF62,
	0xFF63,
	0xFF65,
	0xFF66,
	0xFF67,
	0xFF71,
	0xFF9E,
	0xFFA0,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0xFFE0,
	0xFFE1,
	0xFFE2,
	0xFFE5,
	0xFFE7,
	0xFFF9,
	0xFFFC,
	0xFFFD,
	0x10100,
	0x10103,
	0x101FD,
	0x101FE,
	0x102E0,
	0x102E1,
	0x10376,
	0x1037B,
	0x1039F,
	0x103A0,
	0x103D0,
	0x103D1,
	0x104A0,
	0x104AA,
	0x10857,
	0x10858,
	0x1091F,
	0x10920,
	0x10A01,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A10,
	0x10A38,
	0x10A3B,
	0x10A3F,
	0x10A40,
	0x10A50,
	0x10A58,
	0x10AE5,
	0x10AE7,
	0x10AF0,
	0x10AF6,
	0x10AF7,
	0x10B39,
	0x10B40,
	0x11000,
	0x11003,
	0x11038,
	0x11047,
	0x11049,
	0x11066,
	0x11070,
	0x1107F,
	0x11083,
	0x110B0,
	0x110BB,
	0x110BE,
	0x110C2,
	0x110F0,
	0x110FA,
	0x11100,
	0x11103,
	0x11127,
	0x11135,
	0x11136,
	0x11140,
	0x11144,
	0x11173,
	0x11174,
	0x11175,
	0x11176,
	0x11180,
	0x11183,
	0x111B3,
	0x111C1,
	0x111C5,
	0x111C7,
	0x111C8,
	0x111C9,
	0x111CA,
	0x111CD,
	0x111D0,
	0x111DA,
	0x111DB,
	0x111DC,
	0x111DD,
	0x111E0,
	0x1122C,
	0x11238,
	0x1123A,
	0x1123B,
	0x1123D,
	0x1123E,
	0x1123F,
	0x112A9,
	0x112AA,
	0x112DF,
	0x112EB,
	0x112F0,
	0x112FA,
	0x11300,
	0x11304,
	0x1133C,
	0x1133D,
	0x1133E,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134E,
	0x11357,
	0x11358,
	0x11362,
	0x11364,
	0x11366,
	0x1136D,
	0x11370,
	0x11375,
	0x11435,
	0x11447,
	0x1144B,
	0x1144F,
	0x11450,
	0x1145A,
	0x1145B,
	0x1145C,
	0x114B0,
	0x114C4,
	0x114D0,
	0x114DA,
	0x115AF,
	0x115B6,
	0x115B8,
	0x115C1,
	0x115C2,
	0x115C4,
	0x115C6,
	0x115C9,
	0x115D8,
	0x115DC,
	0x115DE,
	0x11630,
	0x11641,
	0x11643,
	0x11650,
	0x1165A,
	0x11660,
	0x1166D,
	0x116AB,
	0x116B8,
	0x116C0,
	0x116CA,
	0x1171D,
	0x1172C,
	0x11730,
	0x1173A,
	0x1173C,
	0x1173F,
	0x118E0,
	0x118EA,
	0x11C2F,
	0x11C37,
	0x11C38,
	0x11C40,
	0x11C41,
	0x11C46,
	0x11C50,
	0x11C5A,
	0x11C70,
	0x11C71,
	0x11C72,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12470,
	0x12475,
	0x13258,
	0x1325B,
	0x1325E,
	0x13282,
	0x13283,
	0x13286,
	0x13287,
	0x13288,
	0x13289,
	0x1328A,
	0x13379,
	0x1337A,
	0x1337C,
	0x145CE,
	0x145CF,
	0x145D0,
	0x16A60,
	0x16A6A,
	0x16A6E,
	0x16A70,
	0x16AF0,
	0x16AF5,
	0x16AF6,
	0x16B30,
	0x16B37,
	0x16B3A,
	0x16B44,
	0x16B45,
	0x16B50,
	0x16B5A,
	0x16F51,
	0x16F7F,
	0x16F8F,
	0x16F93,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC9D,
	0x1BC9F,
	0x1BCA0,
	0x1BCA4,
	0x1D165,
	0x1D16A,
	0x1D16D,
	0x1D183,
	0x1D185,
	0x1D18C,
	0x1D1AA,
	0x1D1AE,
	0x1D242,
	0x1D245,
	0x1D7CE,
	0x1D800,
	0x1DA00,
	0x1DA37,
	0x1DA3B,
	0x1DA6D,
	0x1DA75,
	0x1DA76,
	0x1DA84,
	0x1DA85,
	0x1DA87,
	0x1DA8B,
	0x1DA9B,
	0x1DAA0,
	0x1DAA1,
	0x1DAB0,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E8D0,
	0x1E8D7,
	0x1E944,
	0x1E94B,
	0x1E950,
	0x1E95A,
	0x1E95E,
	0x1E960,
	0x1F000,
	0x1F100,
	0x1F10D,
	0x1F110,
	0x1F12F,
	0x1F130,
	0x1F16C,
	0x1F170,
	0x1F1AD,
	0x1F1E6,
	0x1F200,
	0x1F385,
	0x1F386,
	0x1F39C,
	0x1F39E,
	0x1F3B5,
	0x1F3B7,
	0x1F3BC,
	0x1F3BD,
	0x1F3C3,
	0x1F3C5,
	0x1F3CA,
	0x1F3CC,
	0x1F3FB,
	0x1F400,
	0x1F442,
	0x1F444,
	0x1F446,
	0x1F451,
	0x1F466,
	0x1F46A,
	0x1F46E,
	0x1F46F,
	0x1F470,
	0x1F479,
	0x1F47C,
	0x1F47D,
	0x1F481,
	0x1F484,
	0x1F485,
	0x1F488,
	0x1F4A0,
	0x1F4A1,
	0x1F4A2,
	0x1F4A3,
	0x1F4A4,
	0x1F4A5,
	0x1F4AA,
	0x1F4AB,
	0x1F4AF,
	0x1F4B0,
	0x1F4B1,
	0x1F4B3,
	0x1F500,
	0x1F507,
	0x1F517,
	0x1F525,
	0x1F532,
	0x1F54A,
	0x1F575,
	0x1F576,
	0x1F57A,
	0x1F57B,
	0x1F590,
	0x1F591,
	0x1F595,
	0x1F597,
	0x1F5D4,
	0x1F5DC,
	0x1F5F4,
	0x1F5FA,
	0x1F645,
	0x1F648,
	0x1F64B,
	0x1F650,
	0x1F676,
	0x1F679,
	0x1F67C,
	0x1F680,
	0x1F6A3,
	0x1F6A4,
	0x1F6B4,
	0x1F6B7,
	0x1F6C0,
	0x1F6C1,
	0x1F700,
	0x1F774,
	0x1F780,
	0x1F7D5,
	0x1F800,
	0x1F80C,
	0x1F810,
	0x1F848,
	0x1F850,
	0x1F85A,
	0x1F860,
	0x1F888,
	0x1F890,
	0x1F8AE,
	0x1F918,
	0x1F91F,
	0x1F926,
	0x1F927,
	0x1F930,
	0x1F931,
	0x1F933,
	0x1F93A,
	0x1F93C,
	0x1F93F,
	0x1FFFE,
	0x20000,
	0x2FFFE,
	0x30000,
	0x3FFFE,
	0xE0001,
	0xE0002,
	0xE0020,
	0xE0080,
	0xE0100,
	0xE01F0
};

#endif	/* defined(PERL_IN_PERL_C) */

#if defined(PERL_IN_REGEXEC_C)

#define LB_ENUM_COUNT 39

typedef enum {
	LB_Alphabetic = 0,
	LB_Break_After = 1,
	LB_Break_Before = 2,
	LB_Break_Both = 3,
	LB_Break_Symbols = 4,
	LB_Carriage_Return = 5,
	LB_Close_Parenthesis = 6,
	LB_Close_Punctuation = 7,
	LB_Combining_Mark = 8,
	LB_Contingent_Break = 9,
	LB_E_Base = 10,
	LB_E_Modifier = 11,
	LB_Exclamation = 12,
	LB_Glue = 13,
	LB_H2 = 14,
	LB_H3 = 15,
	LB_Hebrew_Letter = 16,
	LB_Hyphen = 17,
	LB_Ideographic = 18,
	LB_Infix_Numeric = 19,
	LB_Inseparable = 20,
	LB_JL = 21,
	LB_JT = 22,
	LB_JV = 23,
	LB_Line_Feed = 24,
	LB_Mandatory_Break = 25,
	LB_Next_Line = 26,
	LB_Nonstarter = 27,
	LB_Numeric = 28,
	LB_Open_Punctuation = 29,
	LB_Postfix_Numeric = 30,
	LB_Prefix_Numeric = 31,
	LB_Quotation = 32,
	LB_Regional_Indicator = 33,
	LB_Space = 34,
	LB_Word_Joiner = 35,
	LB_ZWJ = 36,
	LB_ZWSpace = 37,
	LB_EDGE = 38
} LB_enum;

static const LB_enum _Perl_LB_invmap[] = { /* for ASCII/Latin1 */
	LB_Combining_Mark,
	LB_Break_After,
	LB_Line_Feed,
	LB_Mandatory_Break,
	LB_Carriage_Return,
	LB_Combining_Mark,
	LB_Space,
	LB_Exclamation,
	LB_Quotation,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Quotation,
	LB_Open_Punctuation,
	LB_Close_Parenthesis,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Infix_Numeric,
	LB_Hyphen,
	LB_Infix_Numeric,
	LB_Break_Symbols,
	LB_Numeric,
	LB_Infix_Numeric,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Prefix_Numeric,
	LB_Close_Parenthesis,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Break_After,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Next_Line,
	LB_Combining_Mark,
	LB_Glue,
	LB_Open_Punctuation,
	LB_Postfix_Numeric,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Quotation,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Quotation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Glue,
	LB_Combining_Mark,
	LB_Glue,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Infix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Infix_Numeric,
	LB_Break_After,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Exclamation,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Hebrew_Letter,
	LB_Alphabetic,
	LB_Hebrew_Letter,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Infix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Exclamation,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Numeric,
	LB_Postfix_Numeric,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Infix_Numeric,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Glue,
	LB_Break_Before,
	LB_Break_After,
	LB_Glue,
	LB_Exclamation,
	LB_Glue,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_After,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Break_After,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Glue,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Numeric,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_JL,
	LB_JV,
	LB_JT,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Nonstarter,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Break_After,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Glue,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Numeric,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Numeric,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Numeric,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Break_After,
	LB_Glue,
	LB_Break_After,
	LB_ZWSpace,
	LB_Combining_Mark,
	LB_ZWJ,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Glue,
	LB_Break_After,
	LB_Break_Both,
	LB_Alphabetic,
	LB_Quotation,
	LB_Open_Punctuation,
	LB_Quotation,
	LB_Open_Punctuation,
	LB_Quotation,
	LB_Alphabetic,
	LB_Inseparable,
	LB_Break_After,
	LB_Mandatory_Break,
	LB_Combining_Mark,
	LB_Glue,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Quotation,
	LB_Alphabetic,
	LB_Nonstarter,
	LB_Alphabetic,
	LB_Infix_Numeric,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Nonstarter,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Word_Joiner,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Postfix_Numeric,
	LB_Prefix_Numeric,
	LB_Postfix_Numeric,
	LB_Prefix_Numeric,
	LB_Postfix_Numeric,
	LB_Prefix_Numeric,
	LB_Postfix_Numeric,
	LB_Prefix_Numeric,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Inseparable,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_E_Base,
	LB_Alphabetic,
	LB_Quotation,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Break_After,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Quotation,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Open_Punctuation,
	LB_Break_After,
	LB_Alphabetic,
	LB_Quotation,
	LB_Alphabetic,
	LB_Quotation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Break_After,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_Both,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Open_Punctuation,
	LB_Break_After,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Break_After,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Nonstarter,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Combining_Mark,
	LB_Ideographic,
	LB_Combining_Mark,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Exclamation,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_JL,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_Alphabetic,
	LB_JV,
	LB_Alphabetic,
	LB_JT,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Hebrew_Letter,
	LB_Combining_Mark,
	LB_Hebrew_Letter,
	LB_Alphabetic,
	LB_Hebrew_Letter,
	LB_Alphabetic,
	LB_Hebrew_Letter,
	LB_Alphabetic,
	LB_Hebrew_Letter,
	LB_Alphabetic,
	LB_Hebrew_Letter,
	LB_Alphabetic,
	LB_Hebrew_Letter,
	LB_Alphabetic,
	LB_Hebrew_Letter,
	LB_Alphabetic,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Infix_Numeric,
	LB_Close_Punctuation,
	LB_Infix_Numeric,
	LB_Exclamation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Inseparable,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Ideographic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Nonstarter,
	LB_Exclamation,
	LB_Ideographic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Prefix_Numeric,
	LB_Postfix_Numeric,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Word_Joiner,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Ideographic,
	LB_Prefix_Numeric,
	LB_Postfix_Numeric,
	LB_Ideographic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Exclamation,
	LB_Ideographic,
	LB_Open_Punctuation,
	LB_Ideographic,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Open_Punctuation,
	LB_Ideographic,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Prefix_Numeric,
	LB_Ideographic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Contingent_Break,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Inseparable,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_Before,
	LB_Break_After,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Nonstarter,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Regional_Indicator,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Modifier,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Alphabetic,
	LB_Quotation,
	LB_Nonstarter,
	LB_Alphabetic,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic
};

#endif	/* defined(PERL_IN_REGEXEC_C) */

#if defined(PERL_IN_PERL_C)

static const UV _Perl_SB_invlist[] = { /* for ASCII/Latin1 */
	2957,	/* Number of elements */
	148565664, /* Version and data structure type */
	0,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x9,
	0xA,
	0xB,
	0xD,
	0xE,
	0x20,
	0x21,
	0x22,
	0x23,
	0x27,
	0x2A,
	0x2C,
	0x2E,
	0x2F,
	0x30,
	0x3A,
	0x3B,
	0x3F,
	0x40,
	0x41,
	0x5B,
	0x5C,
	0x5D,
	0x5E,
	0x61,
	0x7B,
	0x7C,
	0x7D,
	0x7E,
	0x85,
	0x86,
	0xA0,
	0xA1,
	0xAA,
	0xAB,
	0xAC,
	0xAD,
	0xAE,
	0xB5,
	0xB6,
	0xBA,
	0xBB,
	0xBC,
	0xC0,
	0xD7,
	0xD8,
	0xDF,
	0xF7,
	0xF8,
	0x100,
	0x101,
	0x102,
	0x103,
	0x104,
	0x105,
	0x106,
	0x107,
	0x108,
	0x109,
	0x10A,
	0x10B,
	0x10C,
	0x10D,
	0x10E,
	0x10F,
	0x110,
	0x111,
	0x112,
	0x113,
	0x114,
	0x115,
	0x116,
	0x117,
	0x118,
	0x119,
	0x11A,
	0x11B,
	0x11C,
	0x11D,
	0x11E,
	0x11F,
	0x120,
	0x121,
	0x122,
	0x123,
	0x124,
	0x125,
	0x126,
	0x127,
	0x128,
	0x129,
	0x12A,
	0x12B,
	0x12C,
	0x12D,
	0x12E,
	0x12F,
	0x130,
	0x131,
	0x132,
	0x133,
	0x134,
	0x135,
	0x136,
	0x137,
	0x139,
	0x13A,
	0x13B,
	0x13C,
	0x13D,
	0x13E,
	0x13F,
	0x140,
	0x141,
	0x142,
	0x143,
	0x144,
	0x145,
	0x146,
	0x147,
	0x148,
	0x14A,
	0x14B,
	0x14C,
	0x14D,
	0x14E,
	0x14F,
	0x150,
	0x151,
	0x152,
	0x153,
	0x154,
	0x155,
	0x156,
	0x157,
	0x158,
	0x159,
	0x15A,
	0x15B,
	0x15C,
	0x15D,
	0x15E,
	0x15F,
	0x160,
	0x161,
	0x162,
	0x163,
	0x164,
	0x165,
	0x166,
	0x167,
	0x168,
	0x169,
	0x16A,
	0x16B,
	0x16C,
	0x16D,
	0x16E,
	0x16F,
	0x170,
	0x171,
	0x172,
	0x173,
	0x174,
	0x175,
	0x176,
	0x177,
	0x178,
	0x17A,
	0x17B,
	0x17C,
	0x17D,
	0x17E,
	0x181,
	0x183,
	0x184,
	0x185,
	0x186,
	0x188,
	0x189,
	0x18C,
	0x18E,
	0x192,
	0x193,
	0x195,
	0x196,
	0x199,
	0x19C,
	0x19E,
	0x19F,
	0x1A1,
	0x1A2,
	0x1A3,
	0x1A4,
	0x1A5,
	0x1A6,
	0x1A8,
	0x1A9,
	0x1AA,
	0x1AC,
	0x1AD,
	0x1AE,
	0x1B0,
	0x1B1,
	0x1B4,
	0x1B5,
	0x1B6,
	0x1B7,
	0x1B9,
	0x1BB,
	0x1BC,
	0x1BD,
	0x1C0,
	0x1C4,
	0x1C6,
	0x1C7,
	0x1C9,
	0x1CA,
	0x1CC,
	0x1CD,
	0x1CE,
	0x1CF,
	0x1D0,
	0x1D1,
	0x1D2,
	0x1D3,
	0x1D4,
	0x1D5,
	0x1D6,
	0x1D7,
	0x1D8,
	0x1D9,
	0x1DA,
	0x1DB,
	0x1DC,
	0x1DE,
	0x1DF,
	0x1E0,
	0x1E1,
	0x1E2,
	0x1E3,
	0x1E4,
	0x1E5,
	0x1E6,
	0x1E7,
	0x1E8,
	0x1E9,
	0x1EA,
	0x1EB,
	0x1EC,
	0x1ED,
	0x1EE,
	0x1EF,
	0x1F1,
	0x1F3,
	0x1F4,
	0x1F5,
	0x1F6,
	0x1F9,
	0x1FA,
	0x1FB,
	0x1FC,
	0x1FD,
	0x1FE,
	0x1FF,
	0x200,
	0x201,
	0x202,
	0x203,
	0x204,
	0x205,
	0x206,
	0x207,
	0x208,
	0x209,
	0x20A,
	0x20B,
	0x20C,
	0x20D,
	0x20E,
	0x20F,
	0x210,
	0x211,
	0x212,
	0x213,
	0x214,
	0x215,
	0x216,
	0x217,
	0x218,
	0x219,
	0x21A,
	0x21B,
	0x21C,
	0x21D,
	0x21E,
	0x21F,
	0x220,
	0x221,
	0x222,
	0x223,
	0x224,
	0x225,
	0x226,
	0x227,
	0x228,
	0x229,
	0x22A,
	0x22B,
	0x22C,
	0x22D,
	0x22E,
	0x22F,
	0x230,
	0x231,
	0x232,
	0x233,
	0x23A,
	0x23C,
	0x23D,
	0x23F,
	0x241,
	0x242,
	0x243,
	0x247,
	0x248,
	0x249,
	0x24A,
	0x24B,
	0x24C,
	0x24D,
	0x24E,
	0x24F,
	0x294,
	0x295,
	0x2B9,
	0x2C0,
	0x2C2,
	0x2C6,
	0x2D2,
	0x2E0,
	0x2E5,
	0x2EC,
	0x2ED,
	0x2EE,
	0x2EF,
	0x300,
	0x370,
	0x371,
	0x372,
	0x373,
	0x374,
	0x375,
	0x376,
	0x377,
	0x378,
	0x37A,
	0x37E,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x390,
	0x391,
	0x3A2,
	0x3A3,
	0x3AC,
	0x3CF,
	0x3D0,
	0x3D2,
	0x3D5,
	0x3D8,
	0x3D9,
	0x3DA,
	0x3DB,
	0x3DC,
	0x3DD,
	0x3DE,
	0x3DF,
	0x3E0,
	0x3E1,
	0x3E2,
	0x3E3,
	0x3E4,
	0x3E5,
	0x3E6,
	0x3E7,
	0x3E8,
	0x3E9,
	0x3EA,
	0x3EB,
	0x3EC,
	0x3ED,
	0x3EE,
	0x3EF,
	0x3F4,
	0x3F5,
	0x3F6,
	0x3F7,
	0x3F8,
	0x3F9,
	0x3FB,
	0x3FD,
	0x430,
	0x460,
	0x461,
	0x462,
	0x463,
	0x464,
	0x465,
	0x466,
	0x467,
	0x468,
	0x469,
	0x46A,
	0x46B,
	0x46C,
	0x46D,
	0x46E,
	0x46F,
	0x470,
	0x471,
	0x472,
	0x473,
	0x474,
	0x475,
	0x476,
	0x477,
	0x478,
	0x479,
	0x47A,
	0x47B,
	0x47C,
	0x47D,
	0x47E,
	0x47F,
	0x480,
	0x481,
	0x482,
	0x483,
	0x48A,
	0x48B,
	0x48C,
	0x48D,
	0x48E,
	0x48F,
	0x490,
	0x491,
	0x492,
	0x493,
	0x494,
	0x495,
	0x496,
	0x497,
	0x498,
	0x499,
	0x49A,
	0x49B,
	0x49C,
	0x49D,
	0x49E,
	0x49F,
	0x4A0,
	0x4A1,
	0x4A2,
	0x4A3,
	0x4A4,
	0x4A5,
	0x4A6,
	0x4A7,
	0x4A8,
	0x4A9,
	0x4AA,
	0x4AB,
	0x4AC,
	0x4AD,
	0x4AE,
	0x4AF,
	0x4B0,
	0x4B1,
	0x4B2,
	0x4B3,
	0x4B4,
	0x4B5,
	0x4B6,
	0x4B7,
	0x4B8,
	0x4B9,
	0x4BA,
	0x4BB,
	0x4BC,
	0x4BD,
	0x4BE,
	0x4BF,
	0x4C0,
	0x4C2,
	0x4C3,
	0x4C4,
	0x4C5,
	0x4C6,
	0x4C7,
	0x4C8,
	0x4C9,
	0x4CA,
	0x4CB,
	0x4CC,
	0x4CD,
	0x4CE,
	0x4D0,
	0x4D1,
	0x4D2,
	0x4D3,
	0x4D4,
	0x4D5,
	0x4D6,
	0x4D7,
	0x4D8,
	0x4D9,
	0x4DA,
	0x4DB,
	0x4DC,
	0x4DD,
	0x4DE,
	0x4DF,
	0x4E0,
	0x4E1,
	0x4E2,
	0x4E3,
	0x4E4,
	0x4E5,
	0x4E6,
	0x4E7,
	0x4E8,
	0x4E9,
	0x4EA,
	0x4EB,
	0x4EC,
	0x4ED,
	0x4EE,
	0x4EF,
	0x4F0,
	0x4F1,
	0x4F2,
	0x4F3,
	0x4F4,
	0x4F5,
	0x4F6,
	0x4F7,
	0x4F8,
	0x4F9,
	0x4FA,
	0x4FB,
	0x4FC,
	0x4FD,
	0x4FE,
	0x4FF,
	0x500,
	0x501,
	0x502,
	0x503,
	0x504,
	0x505,
	0x506,
	0x507,
	0x508,
	0x509,
	0x50A,
	0x50B,
	0x50C,
	0x50D,
	0x50E,
	0x50F,
	0x510,
	0x511,
	0x512,
	0x513,
	0x514,
	0x515,
	0x516,
	0x517,
	0x518,
	0x519,
	0x51A,
	0x51B,
	0x51C,
	0x51D,
	0x51E,
	0x51F,
	0x520,
	0x521,
	0x522,
	0x523,
	0x524,
	0x525,
	0x526,
	0x527,
	0x528,
	0x529,
	0x52A,
	0x52B,
	0x52C,
	0x52D,
	0x52E,
	0x52F,
	0x530,
	0x531,
	0x557,
	0x559,
	0x55A,
	0x55D,
	0x55E,
	0x561,
	0x588,
	0x589,
	0x58A,
	0x591,
	0x5BE,
	0x5BF,
	0x5C0,
	0x5C1,
	0x5C3,
	0x5C4,
	0x5C6,
	0x5C7,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F4,
	0x600,
	0x606,
	0x60C,
	0x60E,
	0x610,
	0x61B,
	0x61C,
	0x61D,
	0x61F,
	0x620,
	0x64B,
	0x660,
	0x66A,
	0x66B,
	0x66D,
	0x66E,
	0x670,
	0x671,
	0x6D4,
	0x6D5,
	0x6D6,
	0x6DD,
	0x6DE,
	0x6DF,
	0x6E5,
	0x6E7,
	0x6E9,
	0x6EA,
	0x6EE,
	0x6F0,
	0x6FA,
	0x6FD,
	0x6FF,
	0x700,
	0x703,
	0x70F,
	0x710,
	0x711,
	0x712,
	0x730,
	0x74B,
	0x74D,
	0x7A6,
	0x7B1,
	0x7B2,
	0x7C0,
	0x7CA,
	0x7EB,
	0x7F4,
	0x7F6,
	0x7F8,
	0x7F9,
	0x7FA,
	0x7FB,
	0x800,
	0x816,
	0x81A,
	0x81B,
	0x824,
	0x825,
	0x828,
	0x829,
	0x82E,
	0x840,
	0x859,
	0x85C,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x8D4,
	0x8E2,
	0x8E3,
	0x904,
	0x93A,
	0x93D,
	0x93E,
	0x950,
	0x951,
	0x958,
	0x962,
	0x964,
	0x966,
	0x970,
	0x971,
	0x981,
	0x984,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BC,
	0x9BD,
	0x9BE,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CE,
	0x9CF,
	0x9D7,
	0x9D8,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E2,
	0x9E4,
	0x9E6,
	0x9F0,
	0x9F2,
	0xA01,
	0xA04,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA3C,
	0xA3D,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4E,
	0xA51,
	0xA52,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA66,
	0xA70,
	0xA72,
	0xA75,
	0xA76,
	0xA81,
	0xA84,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABC,
	0xABD,
	0xABE,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACE,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE2,
	0xAE4,
	0xAE6,
	0xAF0,
	0xAF9,
	0xAFA,
	0xB01,
	0xB04,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3C,
	0xB3D,
	0xB3E,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4E,
	0xB56,
	0xB58,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB62,
	0xB64,
	0xB66,
	0xB70,
	0xB71,
	0xB72,
	0xB82,
	0xB83,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCE,
	0xBD0,
	0xBD1,
	0xBD7,
	0xBD8,
	0xBE6,
	0xBF0,
	0xC00,
	0xC04,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC3E,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4E,
	0xC55,
	0xC57,
	0xC58,
	0xC5B,
	0xC60,
	0xC62,
	0xC64,
	0xC66,
	0xC70,
	0xC80,
	0xC81,
	0xC84,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBC,
	0xCBD,
	0xCBE,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCE,
	0xCD5,
	0xCD7,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE2,
	0xCE4,
	0xCE6,
	0xCF0,
	0xCF1,
	0xCF3,
	0xD01,
	0xD04,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD3E,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD4E,
	0xD4F,
	0xD54,
	0xD57,
	0xD58,
	0xD5F,
	0xD62,
	0xD64,
	0xD66,
	0xD70,
	0xD7A,
	0xD80,
	0xD82,
	0xD84,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xDCA,
	0xDCB,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDE6,
	0xDF0,
	0xDF2,
	0xDF4,
	0xE01,
	0xE31,
	0xE32,
	0xE34,
	0xE3B,
	0xE40,
	0xE47,
	0xE4F,
	0xE50,
	0xE5A,
	0xE81,
	0xE83,
	0xE84,
	0xE85,
	0xE87,
	0xE89,
	0xE8A,
	0xE8B,
	0xE8D,
	0xE8E,
	0xE94,
	0xE98,
	0xE99,
	0xEA0,
	0xEA1,
	0xEA4,
	0xEA5,
	0xEA6,
	0xEA7,
	0xEA8,
	0xEAA,
	0xEAC,
	0xEAD,
	0xEB1,
	0xEB2,
	0xEB4,
	0xEBA,
	0xEBB,
	0xEBD,
	0xEBE,
	0xEC0,
	0xEC5,
	0xEC6,
	0xEC7,
	0xEC8,
	0xECE,
	0xED0,
	0xEDA,
	0xEDC,
	0xEE0,
	0xF00,
	0xF01,
	0xF18,
	0xF1A,
	0xF20,
	0xF2A,
	0xF35,
	0xF36,
	0xF37,
	0xF38,
	0xF39,
	0xF3A,
	0xF3E,
	0xF40,
	0xF48,
	0xF49,
	0xF6D,
	0xF71,
	0xF85,
	0xF86,
	0xF88,
	0xF8D,
	0xF98,
	0xF99,
	0xFBD,
	0xFC6,
	0xFC7,
	0x1000,
	0x102B,
	0x103F,
	0x1040,
	0x104A,
	0x104C,
	0x1050,
	0x1056,
	0x105A,
	0x105E,
	0x1061,
	0x1062,
	0x1065,
	0x1067,
	0x106E,
	0x1071,
	0x1075,
	0x1082,
	0x108E,
	0x108F,
	0x1090,
	0x109A,
	0x109E,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x10FB,
	0x10FC,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x135D,
	0x1360,
	0x1362,
	0x1363,
	0x1367,
	0x1369,
	0x1380,
	0x1390,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1401,
	0x166D,
	0x166E,
	0x166F,
	0x1680,
	0x1681,
	0x169B,
	0x169D,
	0x16A0,
	0x16EB,
	0x16EE,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1712,
	0x1715,
	0x1720,
	0x1732,
	0x1735,
	0x1737,
	0x1740,
	0x1752,
	0x1754,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1772,
	0x1774,
	0x1780,
	0x17B4,
	0x17D4,
	0x17D7,
	0x17D8,
	0x17DC,
	0x17DD,
	0x17DE,
	0x17E0,
	0x17EA,
	0x1802,
	0x1803,
	0x1804,
	0x1808,
	0x1809,
	0x180A,
	0x180B,
	0x180E,
	0x180F,
	0x1810,
	0x181A,
	0x1820,
	0x1878,
	0x1880,
	0x1885,
	0x1887,
	0x18A9,
	0x18AA,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1920,
	0x192C,
	0x1930,
	0x193C,
	0x1944,
	0x1946,
	0x1950,
	0x196E,
	0x1970,
	0x1975,
	0x1980,
	0x19AC,
	0x19B0,
	0x19CA,
	0x19D0,
	0x19DA,
	0x1A00,
	0x1A17,
	0x1A1C,
	0x1A20,
	0x1A55,
	0x1A5F,
	0x1A60,
	0x1A7D,
	0x1A7F,
	0x1A80,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1AA7,
	0x1AA8,
	0x1AAC,
	0x1AB0,
	0x1ABF,
	0x1B00,
	0x1B05,
	0x1B34,
	0x1B45,
	0x1B4C,
	0x1B50,
	0x1B5A,
	0x1B5C,
	0x1B5E,
	0x1B60,
	0x1B6B,
	0x1B74,
	0x1B80,
	0x1B83,
	0x1BA1,
	0x1BAE,
	0x1BB0,
	0x1BBA,
	0x1BE6,
	0x1BF4,
	0x1C00,
	0x1C24,
	0x1C38,
	0x1C3B,
	0x1C3D,
	0x1C40,
	0x1C4A,
	0x1C4D,
	0x1C50,
	0x1C5A,
	0x1C7E,
	0x1C80,
	0x1C89,
	0x1CD0,
	0x1CD3,
	0x1CD4,
	0x1CE9,
	0x1CED,
	0x1CEE,
	0x1CF2,
	0x1CF5,
	0x1CF7,
	0x1CF8,
	0x1CFA,
	0x1D00,
	0x1DC0,
	0x1DF6,
	0x1DFB,
	0x1E00,
	0x1E01,
	0x1E02,
	0x1E03,
	0x1E04,
	0x1E05,
	0x1E06,
	0x1E07,
	0x1E08,
	0x1E09,
	0x1E0A,
	0x1E0B,
	0x1E0C,
	0x1E0D,
	0x1E0E,
	0x1E0F,
	0x1E10,
	0x1E11,
	0x1E12,
	0x1E13,
	0x1E14,
	0x1E15,
	0x1E16,
	0x1E17,
	0x1E18,
	0x1E19,
	0x1E1A,
	0x1E1B,
	0x1E1C,
	0x1E1D,
	0x1E1E,
	0x1E1F,
	0x1E20,
	0x1E21,
	0x1E22,
	0x1E23,
	0x1E24,
	0x1E25,
	0x1E26,
	0x1E27,
	0x1E28,
	0x1E29,
	0x1E2A,
	0x1E2B,
	0x1E2C,
	0x1E2D,
	0x1E2E,
	0x1E2F,
	0x1E30,
	0x1E31,
	0x1E32,
	0x1E33,
	0x1E34,
	0x1E35,
	0x1E36,
	0x1E37,
	0x1E38,
	0x1E39,
	0x1E3A,
	0x1E3B,
	0x1E3C,
	0x1E3D,
	0x1E3E,
	0x1E3F,
	0x1E40,
	0x1E41,
	0x1E42,
	0x1E43,
	0x1E44,
	0x1E45,
	0x1E46,
	0x1E47,
	0x1E48,
	0x1E49,
	0x1E4A,
	0x1E4B,
	0x1E4C,
	0x1E4D,
	0x1E4E,
	0x1E4F,
	0x1E50,
	0x1E51,
	0x1E52,
	0x1E53,
	0x1E54,
	0x1E55,
	0x1E56,
	0x1E57,
	0x1E58,
	0x1E59,
	0x1E5A,
	0x1E5B,
	0x1E5C,
	0x1E5D,
	0x1E5E,
	0x1E5F,
	0x1E60,
	0x1E61,
	0x1E62,
	0x1E63,
	0x1E64,
	0x1E65,
	0x1E66,
	0x1E67,
	0x1E68,
	0x1E69,
	0x1E6A,
	0x1E6B,
	0x1E6C,
	0x1E6D,
	0x1E6E,
	0x1E6F,
	0x1E70,
	0x1E71,
	0x1E72,
	0x1E73,
	0x1E74,
	0x1E75,
	0x1E76,
	0x1E77,
	0x1E78,
	0x1E79,
	0x1E7A,
	0x1E7B,
	0x1E7C,
	0x1E7D,
	0x1E7E,
	0x1E7F,
	0x1E80,
	0x1E81,
	0x1E82,
	0x1E83,
	0x1E84,
	0x1E85,
	0x1E86,
	0x1E87,
	0x1E88,
	0x1E89,
	0x1E8A,
	0x1E8B,
	0x1E8C,
	0x1E8D,
	0x1E8E,
	0x1E8F,
	0x1E90,
	0x1E91,
	0x1E92,
	0x1E93,
	0x1E94,
	0x1E95,
	0x1E9E,
	0x1E9F,
	0x1EA0,
	0x1EA1,
	0x1EA2,
	0x1EA3,
	0x1EA4,
	0x1EA5,
	0x1EA6,
	0x1EA7,
	0x1EA8,
	0x1EA9,
	0x1EAA,
	0x1EAB,
	0x1EAC,
	0x1EAD,
	0x1EAE,
	0x1EAF,
	0x1EB0,
	0x1EB1,
	0x1EB2,
	0x1EB3,
	0x1EB4,
	0x1EB5,
	0x1EB6,
	0x1EB7,
	0x1EB8,
	0x1EB9,
	0x1EBA,
	0x1EBB,
	0x1EBC,
	0x1EBD,
	0x1EBE,
	0x1EBF,
	0x1EC0,
	0x1EC1,
	0x1EC2,
	0x1EC3,
	0x1EC4,
	0x1EC5,
	0x1EC6,
	0x1EC7,
	0x1EC8,
	0x1EC9,
	0x1ECA,
	0x1ECB,
	0x1ECC,
	0x1ECD,
	0x1ECE,
	0x1ECF,
	0x1ED0,
	0x1ED1,
	0x1ED2,
	0x1ED3,
	0x1ED4,
	0x1ED5,
	0x1ED6,
	0x1ED7,
	0x1ED8,
	0x1ED9,
	0x1EDA,
	0x1EDB,
	0x1EDC,
	0x1EDD,
	0x1EDE,
	0x1EDF,
	0x1EE0,
	0x1EE1,
	0x1EE2,
	0x1EE3,
	0x1EE4,
	0x1EE5,
	0x1EE6,
	0x1EE7,
	0x1EE8,
	0x1EE9,
	0x1EEA,
	0x1EEB,
	0x1EEC,
	0x1EED,
	0x1EEE,
	0x1EEF,
	0x1EF0,
	0x1EF1,
	0x1EF2,
	0x1EF3,
	0x1EF4,
	0x1EF5,
	0x1EF6,
	0x1EF7,
	0x1EF8,
	0x1EF9,
	0x1EFA,
	0x1EFB,
	0x1EFC,
	0x1EFD,
	0x1EFE,
	0x1EFF,
	0x1F08,
	0x1F10,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F28,
	0x1F30,
	0x1F38,
	0x1F40,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F60,
	0x1F68,
	0x1F70,
	0x1F7E,
	0x1F80,
	0x1F88,
	0x1F90,
	0x1F98,
	0x1FA0,
	0x1FA8,
	0x1FB0,
	0x1FB5,
	0x1FB6,
	0x1FB8,
	0x1FBD,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FC8,
	0x1FCD,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FD8,
	0x1FDC,
	0x1FE0,
	0x1FE8,
	0x1FED,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FF8,
	0x1FFD,
	0x2000,
	0x200B,
	0x200C,
	0x200E,
	0x2010,
	0x2013,
	0x2015,
	0x2018,
	0x2020,
	0x2024,
	0x2025,
	0x2028,
	0x202A,
	0x202F,
	0x2030,
	0x2039,
	0x203B,
	0x203C,
	0x203E,
	0x2045,
	0x2047,
	0x204A,
	0x205F,
	0x2060,
	0x2065,
	0x2066,
	0x2070,
	0x2071,
	0x2072,
	0x207D,
	0x207F,
	0x2080,
	0x208D,
	0x208F,
	0x2090,
	0x209D,
	0x20D0,
	0x20F1,
	0x2102,
	0x2103,
	0x2107,
	0x2108,
	0x210A,
	0x210B,
	0x210E,
	0x2110,
	0x2113,
	0x2114,
	0x2115,
	0x2116,
	0x2119,
	0x211E,
	0x2124,
	0x2125,
	0x2126,
	0x2127,
	0x2128,
	0x2129,
	0x212A,
	0x212E,
	0x212F,
	0x2130,
	0x2134,
	0x2135,
	0x2139,
	0x213A,
	0x213C,
	0x213E,
	0x2140,
	0x2145,
	0x2146,
	0x214A,
	0x214E,
	0x214F,
	0x2160,
	0x2170,
	0x2180,
	0x2183,
	0x2184,
	0x2185,
	0x2189,
	0x2308,
	0x230C,
	0x2329,
	0x232B,
	0x24B6,
	0x24D0,
	0x24EA,
	0x275B,
	0x2761,
	0x2768,
	0x2776,
	0x27C5,
	0x27C7,
	0x27E6,
	0x27F0,
	0x2983,
	0x2999,
	0x29D8,
	0x29DC,
	0x29FC,
	0x29FE,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2C61,
	0x2C62,
	0x2C65,
	0x2C67,
	0x2C68,
	0x2C69,
	0x2C6A,
	0x2C6B,
	0x2C6C,
	0x2C6D,
	0x2C71,
	0x2C72,
	0x2C73,
	0x2C75,
	0x2C76,
	0x2C7E,
	0x2C81,
	0x2C82,
	0x2C83,
	0x2C84,
	0x2C85,
	0x2C86,
	0x2C87,
	0x2C88,
	0x2C89,
	0x2C8A,
	0x2C8B,
	0x2C8C,
	0x2C8D,
	0x2C8E,
	0x2C8F,
	0x2C90,
	0x2C91,
	0x2C92,
	0x2C93,
	0x2C94,
	0x2C95,
	0x2C96,
	0x2C97,
	0x2C98,
	0x2C99,
	0x2C9A,
	0x2C9B,
	0x2C9C,
	0x2C9D,
	0x2C9E,
	0x2C9F,
	0x2CA0,
	0x2CA1,
	0x2CA2,
	0x2CA3,
	0x2CA4,
	0x2CA5,
	0x2CA6,
	0x2CA7,
	0x2CA8,
	0x2CA9,
	0x2CAA,
	0x2CAB,
	0x2CAC,
	0x2CAD,
	0x2CAE,
	0x2CAF,
	0x2CB0,
	0x2CB1,
	0x2CB2,
	0x2CB3,
	0x2CB4,
	0x2CB5,
	0x2CB6,
	0x2CB7,
	0x2CB8,
	0x2CB9,
	0x2CBA,
	0x2CBB,
	0x2CBC,
	0x2CBD,
	0x2CBE,
	0x2CBF,
	0x2CC0,
	0x2CC1,
	0x2CC2,
	0x2CC3,
	0x2CC4,
	0x2CC5,
	0x2CC6,
	0x2CC7,
	0x2CC8,
	0x2CC9,
	0x2CCA,
	0x2CCB,
	0x2CCC,
	0x2CCD,
	0x2CCE,
	0x2CCF,
	0x2CD0,
	0x2CD1,
	0x2CD2,
	0x2CD3,
	0x2CD4,
	0x2CD5,
	0x2CD6,
	0x2CD7,
	0x2CD8,
	0x2CD9,
	0x2CDA,
	0x2CDB,
	0x2CDC,
	0x2CDD,
	0x2CDE,
	0x2CDF,
	0x2CE0,
	0x2CE1,
	0x2CE2,
	0x2CE3,
	0x2CE5,
	0x2CEB,
	0x2CEC,
	0x2CED,
	0x2CEE,
	0x2CEF,
	0x2CF2,
	0x2CF3,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D70,
	0x2D7F,
	0x2D80,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x2DE0,
	0x2E00,
	0x2E0E,
	0x2E1C,
	0x2E1E,
	0x2E20,
	0x2E2A,
	0x2E2E,
	0x2E2F,
	0x2E30,
	0x2E3C,
	0x2E3D,
	0x2E42,
	0x2E43,
	0x3000,
	0x3001,
	0x3002,
	0x3003,
	0x3005,
	0x3008,
	0x3012,
	0x3014,
	0x301C,
	0x301D,
	0x3020,
	0x3021,
	0x302A,
	0x3030,
	0x3031,
	0x3036,
	0x3038,
	0x303D,
	0x3041,
	0x3097,
	0x3099,
	0x309B,
	0x309D,
	0x30A0,
	0x30A1,
	0x30FB,
	0x30FC,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x31A0,
	0x31BB,
	0x31F0,
	0x3200,
	0x3400,
	0x4DB6,
	0x4E00,
	0x9FD6,
	0xA000,
	0xA48D,
	0xA4D0,
	0xA4FE,
	0xA4FF,
	0xA500,
	0xA60D,
	0xA60E,
	0xA610,
	0xA620,
	0xA62A,
	0xA62C,
	0xA640,
	0xA641,
	0xA642,
	0xA643,
	0xA644,
	0xA645,
	0xA646,
	0xA647,
	0xA648,
	0xA649,
	0xA64A,
	0xA64B,
	0xA64C,
	0xA64D,
	0xA64E,
	0xA64F,
	0xA650,
	0xA651,
	0xA652,
	0xA653,
	0xA654,
	0xA655,
	0xA656,
	0xA657,
	0xA658,
	0xA659,
	0xA65A,
	0xA65B,
	0xA65C,
	0xA65D,
	0xA65E,
	0xA65F,
	0xA660,
	0xA661,
	0xA662,
	0xA663,
	0xA664,
	0xA665,
	0xA666,
	0xA667,
	0xA668,
	0xA669,
	0xA66A,
	0xA66B,
	0xA66C,
	0xA66D,
	0xA66E,
	0xA66F,
	0xA673,
	0xA674,
	0xA67E,
	0xA67F,
	0xA680,
	0xA681,
	0xA682,
	0xA683,
	0xA684,
	0xA685,
	0xA686,
	0xA687,
	0xA688,
	0xA689,
	0xA68A,
	0xA68B,
	0xA68C,
	0xA68D,
	0xA68E,
	0xA68F,
	0xA690,
	0xA691,
	0xA692,
	0xA693,
	0xA694,
	0xA695,
	0xA696,
	0xA697,
	0xA698,
	0xA699,
	0xA69A,
	0xA69B,
	0xA69E,
	0xA6A0,
	0xA6F0,
	0xA6F2,
	0xA6F3,
	0xA6F4,
	0xA6F7,
	0xA6F8,
	0xA717,
	0xA720,
	0xA722,
	0xA723,
	0xA724,
	0xA725,
	0xA726,
	0xA727,
	0xA728,
	0xA729,
	0xA72A,
	0xA72B,
	0xA72C,
	0xA72D,
	0xA72E,
	0xA72F,
	0xA732,
	0xA733,
	0xA734,
	0xA735,
	0xA736,
	0xA737,
	0xA738,
	0xA739,
	0xA73A,
	0xA73B,
	0xA73C,
	0xA73D,
	0xA73E,
	0xA73F,
	0xA740,
	0xA741,
	0xA742,
	0xA743,
	0xA744,
	0xA745,
	0xA746,
	0xA747,
	0xA748,
	0xA749,
	0xA74A,
	0xA74B,
	0xA74C,
	0xA74D,
	0xA74E,
	0xA74F,
	0xA750,
	0xA751,
	0xA752,
	0xA753,
	0xA754,
	0xA755,
	0xA756,
	0xA757,
	0xA758,
	0xA759,
	0xA75A,
	0xA75B,
	0xA75C,
	0xA75D,
	0xA75E,
	0xA75F,
	0xA760,
	0xA761,
	0xA762,
	0xA763,
	0xA764,
	0xA765,
	0xA766,
	0xA767,
	0xA768,
	0xA769,
	0xA76A,
	0xA76B,
	0xA76C,
	0xA76D,
	0xA76E,
	0xA76F,
	0xA779,
	0xA77A,
	0xA77B,
	0xA77C,
	0xA77D,
	0xA77F,
	0xA780,
	0xA781,
	0xA782,
	0xA783,
	0xA784,
	0xA785,
	0xA786,
	0xA787,
	0xA788,
	0xA789,
	0xA78B,
	0xA78C,
	0xA78D,
	0xA78E,
	0xA78F,
	0xA790,
	0xA791,
	0xA792,
	0xA793,
	0xA796,
	0xA797,
	0xA798,
	0xA799,
	0xA79A,
	0xA79B,
	0xA79C,
	0xA79D,
	0xA79E,
	0xA79F,
	0xA7A0,
	0xA7A1,
	0xA7A2,
	0xA7A3,
	0xA7A4,
	0xA7A5,
	0xA7A6,
	0xA7A7,
	0xA7A8,
	0xA7A9,
	0xA7AA,
	0xA7AF,
	0xA7B0,
	0xA7B5,
	0xA7B6,
	0xA7B7,
	0xA7B8,
	0xA7F7,
	0xA7F8,
	0xA7FB,
	0xA802,
	0xA803,
	0xA806,
	0xA807,
	0xA80B,
	0xA80C,
	0xA823,
	0xA828,
	0xA840,
	0xA874,
	0xA876,
	0xA878,
	0xA880,
	0xA882,
	0xA8B4,
	0xA8C6,
	0xA8CE,
	0xA8D0,
	0xA8DA,
	0xA8E0,
	0xA8F2,
	0xA8F8,
	0xA8FB,
	0xA8FC,
	0xA8FD,
	0xA8FE,
	0xA900,
	0xA90A,
	0xA926,
	0xA92E,
	0xA92F,
	0xA930,
	0xA947,
	0xA954,
	0xA960,
	0xA97D,
	0xA980,
	0xA984,
	0xA9B3,
	0xA9C1,
	0xA9C8,
	0xA9CA,
	0xA9CF,
	0xA9D0,
	0xA9DA,
	0xA9E0,
	0xA9E5,
	0xA9E6,
	0xA9F0,
	0xA9FA,
	0xA9FF,
	0xAA00,
	0xAA29,
	0xAA37,
	0xAA40,
	0xAA43,
	0xAA44,
	0xAA4C,
	0xAA4E,
	0xAA50,
	0xAA5A,
	0xAA5D,
	0xAA60,
	0xAA77,
	0xAA7A,
	0xAA7B,
	0xAA7E,
	0xAAB0,
	0xAAB1,
	0xAAB2,
	0xAAB5,
	0xAAB7,
	0xAAB9,
	0xAABE,
	0xAAC0,
	0xAAC1,
	0xAAC2,
	0xAAC3,
	0xAADB,
	0xAADE,
	0xAAE0,
	0xAAEB,
	0xAAF0,
	0xAAF2,
	0xAAF5,
	0xAAF7,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB5B,
	0xAB5C,
	0xAB66,
	0xAB70,
	0xABC0,
	0xABE3,
	0xABEB,
	0xABEC,
	0xABEE,
	0xABF0,
	0xABFA,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xF900,
	0xFA6E,
	0xFA70,
	0xFADA,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB1E,
	0xFB1F,
	0xFB29,
	0xFB2A,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFBB2,
	0xFBD3,
	0xFD3E,
	0xFD40,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFC,
	0xFE00,
	0xFE10,
	0xFE12,
	0xFE13,
	0xFE14,
	0xFE17,
	0xFE19,
	0xFE20,
	0xFE30,
	0xFE31,
	0xFE33,
	0xFE35,
	0xFE45,
	0xFE47,
	0xFE49,
	0xFE50,
	0xFE52,
	0xFE53,
	0xFE55,
	0xFE56,
	0xFE58,
	0xFE59,
	0xFE5F,
	0xFE63,
	0xFE64,
	0xFE70,
	0xFE75,
	0xFE76,
	0xFEFD,
	0xFEFF,
	0xFF00,
	0xFF01,
	0xFF02,
	0xFF08,
	0xFF0A,
	0xFF0C,
	0xFF0E,
	0xFF0F,
	0xFF1A,
	0xFF1B,
	0xFF1F,
	0xFF20,
	0xFF21,
	0xFF3B,
	0xFF3C,
	0xFF3D,
	0xFF3E,
	0xFF41,
	0xFF5B,
	0xFF5C,
	0xFF5D,
	0xFF5E,
	0xFF5F,
	0xFF61,
	0xFF62,
	0xFF64,
	0xFF65,
	0xFF66,
	0xFF9E,
	0xFFA0,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0xFFF9,
	0xFFFC,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10140,
	0x10175,
	0x101FD,
	0x101FE,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x102E0,
	0x102E1,
	0x10300,
	0x10320,
	0x10330,
	0x1034B,
	0x10350,
	0x10376,
	0x1037B,
	0x10380,
	0x1039E,
	0x103A0,
	0x103C4,
	0x103C8,
	0x103D0,
	0x103D1,
	0x103D6,
	0x10400,
	0x10428,
	0x10450,
	0x1049E,
	0x104A0,
	0x104AA,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10860,
	0x10877,
	0x10880,
	0x1089F,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x10900,
	0x10916,
	0x10920,
	0x1093A,
	0x10980,
	0x109B8,
	0x109BE,
	0x109C0,
	0x10A00,
	0x10A01,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A10,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A38,
	0x10A3B,
	0x10A3F,
	0x10A40,
	0x10A56,
	0x10A58,
	0x10A60,
	0x10A7D,
	0x10A80,
	0x10A9D,
	0x10AC0,
	0x10AC8,
	0x10AC9,
	0x10AE5,
	0x10AE7,
	0x10B00,
	0x10B36,
	0x10B40,
	0x10B56,
	0x10B60,
	0x10B73,
	0x10B80,
	0x10B92,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x11000,
	0x11003,
	0x11038,
	0x11047,
	0x11049,
	0x11066,
	0x11070,
	0x1107F,
	0x11083,
	0x110B0,
	0x110BB,
	0x110BD,
	0x110BE,
	0x110C2,
	0x110D0,
	0x110E9,
	0x110F0,
	0x110FA,
	0x11100,
	0x11103,
	0x11127,
	0x11135,
	0x11136,
	0x11140,
	0x11141,
	0x11144,
	0x11150,
	0x11173,
	0x11174,
	0x11176,
	0x11177,
	0x11180,
	0x11183,
	0x111B3,
	0x111C1,
	0x111C5,
	0x111C7,
	0x111CA,
	0x111CD,
	0x111CE,
	0x111D0,
	0x111DA,
	0x111DB,
	0x111DC,
	0x111DD,
	0x111DE,
	0x111E0,
	0x11200,
	0x11212,
	0x11213,
	0x1122C,
	0x11238,
	0x1123A,
	0x1123B,
	0x1123D,
	0x1123E,
	0x1123F,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112A9,
	0x112AA,
	0x112B0,
	0x112DF,
	0x112EB,
	0x112F0,
	0x112FA,
	0x11300,
	0x11304,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133C,
	0x1133D,
	0x1133E,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134E,
	0x11350,
	0x11351,
	0x11357,
	0x11358,
	0x1135D,
	0x11362,
	0x11364,
	0x11366,
	0x1136D,
	0x11370,
	0x11375,
	0x11400,
	0x11435,
	0x11447,
	0x1144B,
	0x1144D,
	0x11450,
	0x1145A,
	0x11480,
	0x114B0,
	0x114C4,
	0x114C6,
	0x114C7,
	0x114C8,
	0x114D0,
	0x114DA,
	0x11580,
	0x115AF,
	0x115B6,
	0x115B8,
	0x115C1,
	0x115C2,
	0x115C4,
	0x115C9,
	0x115D8,
	0x115DC,
	0x115DE,
	0x11600,
	0x11630,
	0x11641,
	0x11643,
	0x11644,
	0x11645,
	0x11650,
	0x1165A,
	0x11680,
	0x116AB,
	0x116B8,
	0x116C0,
	0x116CA,
	0x11700,
	0x1171A,
	0x1171D,
	0x1172C,
	0x11730,
	0x1173A,
	0x1173C,
	0x1173F,
	0x118A0,
	0x118C0,
	0x118E0,
	0x118EA,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C2F,
	0x11C37,
	0x11C38,
	0x11C40,
	0x11C41,
	0x11C43,
	0x11C50,
	0x11C5A,
	0x11C72,
	0x11C90,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16A60,
	0x16A6A,
	0x16A6E,
	0x16A70,
	0x16AD0,
	0x16AEE,
	0x16AF0,
	0x16AF5,
	0x16AF6,
	0x16B00,
	0x16B30,
	0x16B37,
	0x16B39,
	0x16B40,
	0x16B44,
	0x16B45,
	0x16B50,
	0x16B5A,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F51,
	0x16F7F,
	0x16F8F,
	0x16F93,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1BC9D,
	0x1BC9F,
	0x1BCA0,
	0x1BCA4,
	0x1D165,
	0x1D16A,
	0x1D16D,
	0x1D173,
	0x1D17B,
	0x1D183,
	0x1D185,
	0x1D18C,
	0x1D1AA,
	0x1D1AE,
	0x1D242,
	0x1D245,
	0x1D400,
	0x1D41A,
	0x1D434,
	0x1D44E,
	0x1D455,
	0x1D456,
	0x1D468,
	0x1D482,
	0x1D49C,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4B6,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D4D0,
	0x1D4EA,
	0x1D504,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D538,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D56C,
	0x1D586,
	0x1D5A0,
	0x1D5BA,
	0x1D5D4,
	0x1D5EE,
	0x1D608,
	0x1D622,
	0x1D63C,
	0x1D656,
	0x1D670,
	0x1D68A,
	0x1D6A6,
	0x1D6A8,
	0x1D6C1,
	0x1D6C2,
	0x1D6DB,
	0x1D6DC,
	0x1D6E2,
	0x1D6FB,
	0x1D6FC,
	0x1D715,
	0x1D716,
	0x1D71C,
	0x1D735,
	0x1D736,
	0x1D74F,
	0x1D750,
	0x1D756,
	0x1D76F,
	0x1D770,
	0x1D789,
	0x1D78A,
	0x1D790,
	0x1D7A9,
	0x1D7AA,
	0x1D7C3,
	0x1D7C4,
	0x1D7CA,
	0x1D7CB,
	0x1D7CC,
	0x1D7CE,
	0x1D800,
	0x1DA00,
	0x1DA37,
	0x1DA3B,
	0x1DA6D,
	0x1DA75,
	0x1DA76,
	0x1DA84,
	0x1DA85,
	0x1DA88,
	0x1DA89,
	0x1DA9B,
	0x1DAA0,
	0x1DAA1,
	0x1DAB0,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E800,
	0x1E8C5,
	0x1E8D0,
	0x1E8D7,
	0x1E900,
	0x1E922,
	0x1E944,
	0x1E94B,
	0x1E950,
	0x1E95A,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x1F130,
	0x1F14A,
	0x1F150,
	0x1F16A,
	0x1F170,
	0x1F18A,
	0x1F676,
	0x1F679,
	0x20000,
	0x2A6D7,
	0x2A700,
	0x2B735,
	0x2B740,
	0x2B81E,
	0x2B820,
	0x2CEA2,
	0x2F800,
	0x2FA1E,
	0xE0001,
	0xE0002,
	0xE0020,
	0xE0080,
	0xE0100,
	0xE01F0
};

#endif	/* defined(PERL_IN_PERL_C) */

#if defined(PERL_IN_REGEXEC_C)

#define SB_ENUM_COUNT 16

typedef enum {
	SB_Other = 0,
	SB_ATerm = 1,
	SB_CR = 2,
	SB_Close = 3,
	SB_Extend = 4,
	SB_Format = 5,
	SB_LF = 6,
	SB_Lower = 7,
	SB_Numeric = 8,
	SB_OLetter = 9,
	SB_SContinue = 10,
	SB_STerm = 11,
	SB_Sep = 12,
	SB_Sp = 13,
	SB_Upper = 14,
	SB_EDGE = 15
} SB_enum;

static const SB_enum _Perl_SB_invmap[] = { /* for ASCII/Latin1 */
	SB_Other,
	SB_Sp,
	SB_LF,
	SB_Sp,
	SB_CR,
	SB_Other,
	SB_Sp,
	SB_STerm,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_SContinue,
	SB_ATerm,
	SB_Other,
	SB_Numeric,
	SB_SContinue,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Upper,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Lower,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Sep,
	SB_Other,
	SB_Sp,
	SB_Other,
	SB_Lower,
	SB_Close,
	SB_Other,
	SB_Format,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Close,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Lower,
	SB_OLetter,
	SB_Lower,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Extend,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_SContinue,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Format,
	SB_Other,
	SB_SContinue,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Format,
	SB_Other,
	SB_STerm,
	SB_OLetter,
	SB_Extend,
	SB_Numeric,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_STerm,
	SB_OLetter,
	SB_Extend,
	SB_Format,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Numeric,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_STerm,
	SB_Other,
	SB_Format,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_SContinue,
	SB_STerm,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Format,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_STerm,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Close,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Numeric,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Numeric,
	SB_Extend,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_STerm,
	SB_OLetter,
	SB_Sp,
	SB_OLetter,
	SB_Close,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_SContinue,
	SB_STerm,
	SB_Other,
	SB_SContinue,
	SB_STerm,
	SB_Other,
	SB_Extend,
	SB_Format,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_Numeric,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Numeric,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_STerm,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_STerm,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Numeric,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Numeric,
	SB_OLetter,
	SB_STerm,
	SB_Lower,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Lower,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Sp,
	SB_Format,
	SB_Extend,
	SB_Format,
	SB_Other,
	SB_SContinue,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_ATerm,
	SB_Other,
	SB_Sep,
	SB_Format,
	SB_Sp,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Close,
	SB_STerm,
	SB_Other,
	SB_Sp,
	SB_Format,
	SB_Other,
	SB_Format,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Close,
	SB_Lower,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Extend,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_STerm,
	SB_OLetter,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Sp,
	SB_SContinue,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_STerm,
	SB_OLetter,
	SB_Other,
	SB_STerm,
	SB_OLetter,
	SB_Numeric,
	SB_OLetter,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_OLetter,
	SB_Lower,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_Numeric,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Numeric,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_STerm,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_STerm,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_OLetter,
	SB_Extend,
	SB_STerm,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Close,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_SContinue,
	SB_Other,
	SB_SContinue,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_SContinue,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_SContinue,
	SB_ATerm,
	SB_Other,
	SB_SContinue,
	SB_STerm,
	SB_SContinue,
	SB_Close,
	SB_Other,
	SB_SContinue,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Format,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_SContinue,
	SB_ATerm,
	SB_Other,
	SB_SContinue,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Upper,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Lower,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_STerm,
	SB_Close,
	SB_SContinue,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Format,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_STerm,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Format,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_STerm,
	SB_Other,
	SB_Extend,
	SB_STerm,
	SB_Other,
	SB_Numeric,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_STerm,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_STerm,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_STerm,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_STerm,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_STerm,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_STerm,
	SB_Format,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Format,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Format,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other
};

#endif	/* defined(PERL_IN_REGEXEC_C) */

#if defined(PERL_IN_PERL_C)

static const UV _Perl_WB_invlist[] = { /* for ASCII/Latin1 */
	1657,	/* Number of elements */
	148565664, /* Version and data structure type */
	0,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x9,
	0xA,
	0xB,
	0xD,
	0xE,
	0x20,
	0x21,
	0x22,
	0x23,
	0x27,
	0x28,
	0x2C,
	0x2D,
	0x2E,
	0x2F,
	0x30,
	0x3A,
	0x3B,
	0x3C,
	0x41,
	0x5B,
	0x5F,
	0x60,
	0x61,
	0x7B,
	0x85,
	0x86,
	0xA0,
	0xA1,
	0xAA,
	0xAB,
	0xAD,
	0xAE,
	0xB5,
	0xB6,
	0xB7,
	0xB8,
	0xBA,
	0xBB,
	0xC0,
	0xD7,
	0xD8,
	0xF7,
	0xF8,
	0x2C2,
	0x2C6,
	0x2D2,
	0x2D7,
	0x2D8,
	0x2E0,
	0x2E5,
	0x2EC,
	0x2ED,
	0x2EE,
	0x2EF,
	0x300,
	0x370,
	0x375,
	0x376,
	0x378,
	0x37A,
	0x37E,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x3F6,
	0x3F7,
	0x482,
	0x483,
	0x48A,
	0x530,
	0x531,
	0x557,
	0x559,
	0x55A,
	0x561,
	0x588,
	0x589,
	0x58A,
	0x591,
	0x5BE,
	0x5BF,
	0x5C0,
	0x5C1,
	0x5C3,
	0x5C4,
	0x5C6,
	0x5C7,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F3,
	0x5F4,
	0x5F5,
	0x600,
	0x606,
	0x60C,
	0x60E,
	0x610,
	0x61B,
	0x61C,
	0x61D,
	0x620,
	0x64B,
	0x660,
	0x66A,
	0x66B,
	0x66C,
	0x66D,
	0x66E,
	0x670,
	0x671,
	0x6D4,
	0x6D5,
	0x6D6,
	0x6DD,
	0x6DE,
	0x6DF,
	0x6E5,
	0x6E7,
	0x6E9,
	0x6EA,
	0x6EE,
	0x6F0,
	0x6FA,
	0x6FD,
	0x6FF,
	0x700,
	0x70F,
	0x710,
	0x711,
	0x712,
	0x730,
	0x74B,
	0x74D,
	0x7A6,
	0x7B1,
	0x7B2,
	0x7C0,
	0x7CA,
	0x7EB,
	0x7F4,
	0x7F6,
	0x7F8,
	0x7F9,
	0x7FA,
	0x7FB,
	0x800,
	0x816,
	0x81A,
	0x81B,
	0x824,
	0x825,
	0x828,
	0x829,
	0x82E,
	0x840,
	0x859,
	0x85C,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x8D4,
	0x8E2,
	0x8E3,
	0x904,
	0x93A,
	0x93D,
	0x93E,
	0x950,
	0x951,
	0x958,
	0x962,
	0x964,
	0x966,
	0x970,
	0x971,
	0x981,
	0x984,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BC,
	0x9BD,
	0x9BE,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CE,
	0x9CF,
	0x9D7,
	0x9D8,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E2,
	0x9E4,
	0x9E6,
	0x9F0,
	0x9F2,
	0xA01,
	0xA04,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA3C,
	0xA3D,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4E,
	0xA51,
	0xA52,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA66,
	0xA70,
	0xA72,
	0xA75,
	0xA76,
	0xA81,
	0xA84,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABC,
	0xABD,
	0xABE,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACE,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE2,
	0xAE4,
	0xAE6,
	0xAF0,
	0xAF9,
	0xAFA,
	0xB01,
	0xB04,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3C,
	0xB3D,
	0xB3E,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4E,
	0xB56,
	0xB58,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB62,
	0xB64,
	0xB66,
	0xB70,
	0xB71,
	0xB72,
	0xB82,
	0xB83,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCE,
	0xBD0,
	0xBD1,
	0xBD7,
	0xBD8,
	0xBE6,
	0xBF0,
	0xC00,
	0xC04,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC3E,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4E,
	0xC55,
	0xC57,
	0xC58,
	0xC5B,
	0xC60,
	0xC62,
	0xC64,
	0xC66,
	0xC70,
	0xC80,
	0xC81,
	0xC84,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBC,
	0xCBD,
	0xCBE,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCE,
	0xCD5,
	0xCD7,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE2,
	0xCE4,
	0xCE6,
	0xCF0,
	0xCF1,
	0xCF3,
	0xD01,
	0xD04,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD3E,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD4E,
	0xD4F,
	0xD54,
	0xD57,
	0xD58,
	0xD5F,
	0xD62,
	0xD64,
	0xD66,
	0xD70,
	0xD7A,
	0xD80,
	0xD82,
	0xD84,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xDCA,
	0xDCB,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDE6,
	0xDF0,
	0xDF2,
	0xDF4,
	0xE31,
	0xE32,
	0xE34,
	0xE3B,
	0xE47,
	0xE4F,
	0xE50,
	0xE5A,
	0xEB1,
	0xEB2,
	0xEB4,
	0xEBA,
	0xEBB,
	0xEBD,
	0xEC8,
	0xECE,
	0xED0,
	0xEDA,
	0xF00,
	0xF01,
	0xF18,
	0xF1A,
	0xF20,
	0xF2A,
	0xF35,
	0xF36,
	0xF37,
	0xF38,
	0xF39,
	0xF3A,
	0xF3E,
	0xF40,
	0xF48,
	0xF49,
	0xF6D,
	0xF71,
	0xF85,
	0xF86,
	0xF88,
	0xF8D,
	0xF98,
	0xF99,
	0xFBD,
	0xFC6,
	0xFC7,
	0x102B,
	0x103F,
	0x1040,
	0x104A,
	0x1056,
	0x105A,
	0x105E,
	0x1061,
	0x1062,
	0x1065,
	0x1067,
	0x106E,
	0x1071,
	0x1075,
	0x1082,
	0x108E,
	0x108F,
	0x1090,
	0x109A,
	0x109E,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x10FB,
	0x10FC,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x135D,
	0x1360,
	0x1380,
	0x1390,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1401,
	0x166D,
	0x166F,
	0x1680,
	0x1681,
	0x169B,
	0x16A0,
	0x16EB,
	0x16EE,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1712,
	0x1715,
	0x1720,
	0x1732,
	0x1735,
	0x1740,
	0x1752,
	0x1754,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1772,
	0x1774,
	0x17B4,
	0x17D4,
	0x17DD,
	0x17DE,
	0x17E0,
	0x17EA,
	0x180B,
	0x180E,
	0x180F,
	0x1810,
	0x181A,
	0x1820,
	0x1878,
	0x1880,
	0x1885,
	0x1887,
	0x18A9,
	0x18AA,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1920,
	0x192C,
	0x1930,
	0x193C,
	0x1946,
	0x1950,
	0x19D0,
	0x19DA,
	0x1A00,
	0x1A17,
	0x1A1C,
	0x1A55,
	0x1A5F,
	0x1A60,
	0x1A7D,
	0x1A7F,
	0x1A80,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1AB0,
	0x1ABF,
	0x1B00,
	0x1B05,
	0x1B34,
	0x1B45,
	0x1B4C,
	0x1B50,
	0x1B5A,
	0x1B6B,
	0x1B74,
	0x1B80,
	0x1B83,
	0x1BA1,
	0x1BAE,
	0x1BB0,
	0x1BBA,
	0x1BE6,
	0x1BF4,
	0x1C00,
	0x1C24,
	0x1C38,
	0x1C40,
	0x1C4A,
	0x1C4D,
	0x1C50,
	0x1C5A,
	0x1C7E,
	0x1C80,
	0x1C89,
	0x1CD0,
	0x1CD3,
	0x1CD4,
	0x1CE9,
	0x1CED,
	0x1CEE,
	0x1CF2,
	0x1CF5,
	0x1CF7,
	0x1CF8,
	0x1CFA,
	0x1D00,
	0x1DC0,
	0x1DF6,
	0x1DFB,
	0x1E00,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FBD,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FCD,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FE0,
	0x1FED,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFD,
	0x2000,
	0x200B,
	0x200C,
	0x200D,
	0x200E,
	0x2010,
	0x2018,
	0x201A,
	0x2024,
	0x2025,
	0x2027,
	0x2028,
	0x202A,
	0x202F,
	0x2030,
	0x203F,
	0x2041,
	0x2044,
	0x2045,
	0x2054,
	0x2055,
	0x205F,
	0x2060,
	0x2065,
	0x2066,
	0x2070,
	0x2071,
	0x2072,
	0x207F,
	0x2080,
	0x2090,
	0x209D,
	0x20D0,
	0x20F1,
	0x2102,
	0x2103,
	0x2107,
	0x2108,
	0x210A,
	0x2114,
	0x2115,
	0x2116,
	0x2119,
	0x211E,
	0x2124,
	0x2125,
	0x2126,
	0x2127,
	0x2128,
	0x2129,
	0x212A,
	0x212E,
	0x212F,
	0x213A,
	0x213C,
	0x2140,
	0x2145,
	0x214A,
	0x214E,
	0x214F,
	0x2160,
	0x2189,
	0x24B6,
	0x24EA,
	0x261D,
	0x261E,
	0x26F9,
	0x26FA,
	0x270A,
	0x270E,
	0x2764,
	0x2765,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CE5,
	0x2CEB,
	0x2CEF,
	0x2CF2,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D70,
	0x2D7F,
	0x2D80,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x2DE0,
	0x2E00,
	0x2E2F,
	0x2E30,
	0x3000,
	0x3001,
	0x3005,
	0x3006,
	0x302A,
	0x3030,
	0x3031,
	0x3036,
	0x303B,
	0x303D,
	0x3099,
	0x309B,
	0x309D,
	0x30A0,
	0x30FB,
	0x30FC,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x31A0,
	0x31BB,
	0x31F0,
	0x3200,
	0x32D0,
	0x32FF,
	0x3300,
	0x3358,
	0xA000,
	0xA48D,
	0xA4D0,
	0xA4FE,
	0xA500,
	0xA60D,
	0xA610,
	0xA620,
	0xA62A,
	0xA62C,
	0xA640,
	0xA66F,
	0xA673,
	0xA674,
	0xA67E,
	0xA67F,
	0xA69E,
	0xA6A0,
	0xA6F0,
	0xA6F2,
	0xA717,
	0xA720,
	0xA722,
	0xA789,
	0xA78B,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F7,
	0xA802,
	0xA803,
	0xA806,
	0xA807,
	0xA80B,
	0xA80C,
	0xA823,
	0xA828,
	0xA840,
	0xA874,
	0xA880,
	0xA882,
	0xA8B4,
	0xA8C6,
	0xA8D0,
	0xA8DA,
	0xA8E0,
	0xA8F2,
	0xA8F8,
	0xA8FB,
	0xA8FC,
	0xA8FD,
	0xA8FE,
	0xA900,
	0xA90A,
	0xA926,
	0xA92E,
	0xA930,
	0xA947,
	0xA954,
	0xA960,
	0xA97D,
	0xA980,
	0xA984,
	0xA9B3,
	0xA9C1,
	0xA9CF,
	0xA9D0,
	0xA9DA,
	0xA9E5,
	0xA9E6,
	0xA9F0,
	0xA9FA,
	0xAA00,
	0xAA29,
	0xAA37,
	0xAA40,
	0xAA43,
	0xAA44,
	0xAA4C,
	0xAA4E,
	0xAA50,
	0xAA5A,
	0xAA7B,
	0xAA7E,
	0xAAB0,
	0xAAB1,
	0xAAB2,
	0xAAB5,
	0xAAB7,
	0xAAB9,
	0xAABE,
	0xAAC0,
	0xAAC1,
	0xAAC2,
	0xAAE0,
	0xAAEB,
	0xAAF0,
	0xAAF2,
	0xAAF5,
	0xAAF7,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB5B,
	0xAB5C,
	0xAB66,
	0xAB70,
	0xABE3,
	0xABEB,
	0xABEC,
	0xABEE,
	0xABF0,
	0xABFA,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB1E,
	0xFB1F,
	0xFB29,
	0xFB2A,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFB50,
	0xFBB2,
	0xFBD3,
	0xFD3E,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFC,
	0xFE00,
	0xFE10,
	0xFE11,
	0xFE13,
	0xFE14,
	0xFE15,
	0xFE20,
	0xFE30,
	0xFE33,
	0xFE35,
	0xFE4D,
	0xFE50,
	0xFE51,
	0xFE52,
	0xFE53,
	0xFE54,
	0xFE55,
	0xFE56,
	0xFE70,
	0xFE75,
	0xFE76,
	0xFEFD,
	0xFEFF,
	0xFF00,
	0xFF07,
	0xFF08,
	0xFF0C,
	0xFF0D,
	0xFF0E,
	0xFF0F,
	0xFF1A,
	0xFF1B,
	0xFF1C,
	0xFF21,
	0xFF3B,
	0xFF3F,
	0xFF40,
	0xFF41,
	0xFF5B,
	0xFF66,
	0xFF9E,
	0xFFA0,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0xFFF9,
	0xFFFC,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10140,
	0x10175,
	0x101FD,
	0x101FE,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x102E0,
	0x102E1,
	0x10300,
	0x10320,
	0x10330,
	0x1034B,
	0x10350,
	0x10376,
	0x1037B,
	0x10380,
	0x1039E,
	0x103A0,
	0x103C4,
	0x103C8,
	0x103D0,
	0x103D1,
	0x103D6,
	0x10400,
	0x1049E,
	0x104A0,
	0x104AA,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10860,
	0x10877,
	0x10880,
	0x1089F,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x10900,
	0x10916,
	0x10920,
	0x1093A,
	0x10980,
	0x109B8,
	0x109BE,
	0x109C0,
	0x10A00,
	0x10A01,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A10,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A38,
	0x10A3B,
	0x10A3F,
	0x10A40,
	0x10A60,
	0x10A7D,
	0x10A80,
	0x10A9D,
	0x10AC0,
	0x10AC8,
	0x10AC9,
	0x10AE5,
	0x10AE7,
	0x10B00,
	0x10B36,
	0x10B40,
	0x10B56,
	0x10B60,
	0x10B73,
	0x10B80,
	0x10B92,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x11000,
	0x11003,
	0x11038,
	0x11047,
	0x11066,
	0x11070,
	0x1107F,
	0x11083,
	0x110B0,
	0x110BB,
	0x110BD,
	0x110BE,
	0x110D0,
	0x110E9,
	0x110F0,
	0x110FA,
	0x11100,
	0x11103,
	0x11127,
	0x11135,
	0x11136,
	0x11140,
	0x11150,
	0x11173,
	0x11174,
	0x11176,
	0x11177,
	0x11180,
	0x11183,
	0x111B3,
	0x111C1,
	0x111C5,
	0x111CA,
	0x111CD,
	0x111D0,
	0x111DA,
	0x111DB,
	0x111DC,
	0x111DD,
	0x11200,
	0x11212,
	0x11213,
	0x1122C,
	0x11238,
	0x1123E,
	0x1123F,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112A9,
	0x112B0,
	0x112DF,
	0x112EB,
	0x112F0,
	0x112FA,
	0x11300,
	0x11304,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133C,
	0x1133D,
	0x1133E,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134E,
	0x11350,
	0x11351,
	0x11357,
	0x11358,
	0x1135D,
	0x11362,
	0x11364,
	0x11366,
	0x1136D,
	0x11370,
	0x11375,
	0x11400,
	0x11435,
	0x11447,
	0x1144B,
	0x11450,
	0x1145A,
	0x11480,
	0x114B0,
	0x114C4,
	0x114C6,
	0x114C7,
	0x114C8,
	0x114D0,
	0x114DA,
	0x11580,
	0x115AF,
	0x115B6,
	0x115B8,
	0x115C1,
	0x115D8,
	0x115DC,
	0x115DE,
	0x11600,
	0x11630,
	0x11641,
	0x11644,
	0x11645,
	0x11650,
	0x1165A,
	0x11680,
	0x116AB,
	0x116B8,
	0x116C0,
	0x116CA,
	0x1171D,
	0x1172C,
	0x11730,
	0x1173A,
	0x118A0,
	0x118E0,
	0x118EA,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C2F,
	0x11C37,
	0x11C38,
	0x11C40,
	0x11C41,
	0x11C50,
	0x11C5A,
	0x11C72,
	0x11C90,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16A60,
	0x16A6A,
	0x16AD0,
	0x16AEE,
	0x16AF0,
	0x16AF5,
	0x16B00,
	0x16B30,
	0x16B37,
	0x16B40,
	0x16B44,
	0x16B50,
	0x16B5A,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F51,
	0x16F7F,
	0x16F8F,
	0x16F93,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x1B000,
	0x1B001,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1BC9D,
	0x1BC9F,
	0x1BCA0,
	0x1BCA4,
	0x1D165,
	0x1D16A,
	0x1D16D,
	0x1D173,
	0x1D17B,
	0x1D183,
	0x1D185,
	0x1D18C,
	0x1D1AA,
	0x1D1AE,
	0x1D242,
	0x1D245,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D6C1,
	0x1D6C2,
	0x1D6DB,
	0x1D6DC,
	0x1D6FB,
	0x1D6FC,
	0x1D715,
	0x1D716,
	0x1D735,
	0x1D736,
	0x1D74F,
	0x1D750,
	0x1D76F,
	0x1D770,
	0x1D789,
	0x1D78A,
	0x1D7A9,
	0x1D7AA,
	0x1D7C3,
	0x1D7C4,
	0x1D7CC,
	0x1D7CE,
	0x1D800,
	0x1DA00,
	0x1DA37,
	0x1DA3B,
	0x1DA6D,
	0x1DA75,
	0x1DA76,
	0x1DA84,
	0x1DA85,
	0x1DA9B,
	0x1DAA0,
	0x1DAA1,
	0x1DAB0,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E800,
	0x1E8C5,
	0x1E8D0,
	0x1E8D7,
	0x1E900,
	0x1E944,
	0x1E94B,
	0x1E950,
	0x1E95A,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x1F130,
	0x1F14A,
	0x1F150,
	0x1F16A,
	0x1F170,
	0x1F18A,
	0x1F1E6,
	0x1F200,
	0x1F385,
	0x1F386,
	0x1F3C3,
	0x1F3C5,
	0x1F3CA,
	0x1F3CC,
	0x1F3FB,
	0x1F400,
	0x1F442,
	0x1F444,
	0x1F446,
	0x1F451,
	0x1F466,
	0x1F46A,
	0x1F46E,
	0x1F46F,
	0x1F470,
	0x1F479,
	0x1F47C,
	0x1F47D,
	0x1F481,
	0x1F484,
	0x1F485,
	0x1F488,
	0x1F48B,
	0x1F48C,
	0x1F4AA,
	0x1F4AB,
	0x1F575,
	0x1F576,
	0x1F57A,
	0x1F57B,
	0x1F590,
	0x1F591,
	0x1F595,
	0x1F597,
	0x1F5E8,
	0x1F5E9,
	0x1F645,
	0x1F648,
	0x1F64B,
	0x1F650,
	0x1F6A3,
	0x1F6A4,
	0x1F6B4,
	0x1F6B7,
	0x1F6C0,
	0x1F6C1,
	0x1F918,
	0x1F91F,
	0x1F926,
	0x1F927,
	0x1F930,
	0x1F931,
	0x1F933,
	0x1F93A,
	0x1F93C,
	0x1F93F,
	0xE0001,
	0xE0002,
	0xE0020,
	0xE0080,
	0xE0100,
	0xE01F0
};

#endif	/* defined(PERL_IN_PERL_C) */

#if defined(PERL_IN_REGEXEC_C)

#define WB_ENUM_COUNT 25

typedef enum {
	WB_Other = 0,
	WB_ALetter = 1,
	WB_CR = 2,
	WB_Double_Quote = 3,
	WB_E_Base = 4,
	WB_E_Base_GAZ = 5,
	WB_E_Modifier = 6,
	WB_Extend = 7,
	WB_ExtendNumLet = 8,
	WB_Format = 9,
	WB_Glue_After_Zwj = 10,
	WB_Hebrew_Letter = 11,
	WB_Katakana = 12,
	WB_LF = 13,
	WB_MidLetter = 14,
	WB_MidNum = 15,
	WB_MidNumLet = 16,
	WB_Newline = 17,
	WB_Numeric = 18,
	WB_Perl_Tailored_HSpace = 19,
	WB_Regional_Indicator = 20,
	WB_Single_Quote = 21,
	WB_ZWJ = 22,
	WB_EDGE = 23,
	WB_UNKNOWN = 24
} WB_enum;

static const WB_enum _Perl_WB_invmap[] = { /* for ASCII/Latin1 */
	WB_Other,
	WB_Perl_Tailored_HSpace,
	WB_LF,
	WB_Newline,
	WB_CR,
	WB_Other,
	WB_Perl_Tailored_HSpace,
	WB_Other,
	WB_Double_Quote,
	WB_Other,
	WB_Single_Quote,
	WB_Other,
	WB_MidNum,
	WB_Other,
	WB_MidNumLet,
	WB_Other,
	WB_Numeric,
	WB_MidLetter,
	WB_MidNum,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ExtendNumLet,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Newline,
	WB_Other,
	WB_Perl_Tailored_HSpace,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Format,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_MidLetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_MidLetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_MidNum,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_MidLetter,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_MidNum,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Hebrew_Letter,
	WB_Other,
	WB_Hebrew_Letter,
	WB_ALetter,
	WB_MidLetter,
	WB_Other,
	WB_Format,
	WB_Other,
	WB_MidNum,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Format,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Numeric,
	WB_Other,
	WB_Numeric,
	WB_MidNum,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Format,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Numeric,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Format,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_MidNum,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Format,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Numeric,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Perl_Tailored_HSpace,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Format,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Numeric,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Numeric,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Numeric,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Perl_Tailored_HSpace,
	WB_Other,
	WB_Extend,
	WB_ZWJ,
	WB_Format,
	WB_Other,
	WB_MidNumLet,
	WB_Other,
	WB_MidNumLet,
	WB_Other,
	WB_MidLetter,
	WB_Newline,
	WB_Format,
	WB_ExtendNumLet,
	WB_Other,
	WB_ExtendNumLet,
	WB_Other,
	WB_MidNum,
	WB_Other,
	WB_ExtendNumLet,
	WB_Other,
	WB_Perl_Tailored_HSpace,
	WB_Format,
	WB_Other,
	WB_Format,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_Glue_After_Zwj,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Perl_Tailored_HSpace,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Katakana,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Katakana,
	WB_Other,
	WB_Katakana,
	WB_Other,
	WB_Katakana,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Katakana,
	WB_Other,
	WB_Katakana,
	WB_Other,
	WB_Katakana,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Numeric,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Hebrew_Letter,
	WB_Extend,
	WB_Hebrew_Letter,
	WB_Other,
	WB_Hebrew_Letter,
	WB_Other,
	WB_Hebrew_Letter,
	WB_Other,
	WB_Hebrew_Letter,
	WB_Other,
	WB_Hebrew_Letter,
	WB_Other,
	WB_Hebrew_Letter,
	WB_Other,
	WB_Hebrew_Letter,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_MidNum,
	WB_Other,
	WB_MidLetter,
	WB_MidNum,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ExtendNumLet,
	WB_Other,
	WB_ExtendNumLet,
	WB_MidNum,
	WB_Other,
	WB_MidNumLet,
	WB_Other,
	WB_MidNum,
	WB_MidLetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Format,
	WB_Other,
	WB_MidNumLet,
	WB_Other,
	WB_MidNum,
	WB_Other,
	WB_MidNumLet,
	WB_Other,
	WB_MidLetter,
	WB_MidNum,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ExtendNumLet,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Katakana,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Format,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Format,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Katakana,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Format,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Format,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Regional_Indicator,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Modifier,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base_GAZ,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_Glue_After_Zwj,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_Glue_After_Zwj,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_Format,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other
};

#endif	/* defined(PERL_IN_REGEXEC_C) */

#if defined(PERL_IN_REGCOMP_C)

static const UV UpperLatin1_invlist[] = { /* for ASCII/Latin1 */
	3,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x80,
	0x100
};

#endif	/* defined(PERL_IN_REGCOMP_C) */

#if defined(PERL_IN_PERL_C)

static const UV VertSpace_invlist[] = { /* for ASCII/Latin1 */
	7,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0xA,
	0xE,
	0x85,
	0x86,
	0x2028,
	0x202A
};

static const UV XPerlSpace_invlist[] = { /* for ASCII/Latin1 */
	21,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x9,
	0xE,
	0x20,
	0x21,
	0x85,
	0x86,
	0xA0,
	0xA1,
	0x1680,
	0x1681,
	0x2000,
	0x200B,
	0x2028,
	0x202A,
	0x202F,
	0x2030,
	0x205F,
	0x2060,
	0x3000,
	0x3001
};

static const UV XPosixAlnum_invlist[] = { /* for ASCII/Latin1 */
	1357,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x30,
	0x3A,
	0x41,
	0x5B,
	0x61,
	0x7B,
	0xAA,
	0xAB,
	0xB5,
	0xB6,
	0xBA,
	0xBB,
	0xC0,
	0xD7,
	0xD8,
	0xF7,
	0xF8,
	0x2C2,
	0x2C6,
	0x2D2,
	0x2E0,
	0x2E5,
	0x2EC,
	0x2ED,
	0x2EE,
	0x2EF,
	0x345,
	0x346,
	0x370,
	0x375,
	0x376,
	0x378,
	0x37A,
	0x37E,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x3F6,
	0x3F7,
	0x482,
	0x48A,
	0x530,
	0x531,
	0x557,
	0x559,
	0x55A,
	0x561,
	0x588,
	0x5B0,
	0x5BE,
	0x5BF,
	0x5C0,
	0x5C1,
	0x5C3,
	0x5C4,
	0x5C6,
	0x5C7,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F3,
	0x610,
	0x61B,
	0x620,
	0x658,
	0x659,
	0x66A,
	0x66E,
	0x6D4,
	0x6D5,
	0x6DD,
	0x6E1,
	0x6E9,
	0x6ED,
	0x6FD,
	0x6FF,
	0x700,
	0x710,
	0x740,
	0x74D,
	0x7B2,
	0x7C0,
	0x7EB,
	0x7F4,
	0x7F6,
	0x7FA,
	0x7FB,
	0x800,
	0x818,
	0x81A,
	0x82D,
	0x840,
	0x859,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x8D4,
	0x8E0,
	0x8E3,
	0x8EA,
	0x8F0,
	0x93C,
	0x93D,
	0x94D,
	0x94E,
	0x951,
	0x955,
	0x964,
	0x966,
	0x970,
	0x971,
	0x984,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BD,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CD,
	0x9CE,
	0x9CF,
	0x9D7,
	0x9D8,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E4,
	0x9E6,
	0x9F2,
	0xA01,
	0xA04,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4D,
	0xA51,
	0xA52,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA66,
	0xA76,
	0xA81,
	0xA84,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABD,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACD,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE4,
	0xAE6,
	0xAF0,
	0xAF9,
	0xAFA,
	0xB01,
	0xB04,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3D,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4D,
	0xB56,
	0xB58,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB64,
	0xB66,
	0xB70,
	0xB71,
	0xB72,
	0xB82,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCD,
	0xBD0,
	0xBD1,
	0xBD7,
	0xBD8,
	0xBE6,
	0xBF0,
	0xC00,
	0xC04,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4D,
	0xC55,
	0xC57,
	0xC58,
	0xC5B,
	0xC60,
	0xC64,
	0xC66,
	0xC70,
	0xC80,
	0xC84,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBD,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCD,
	0xCD5,
	0xCD7,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE4,
	0xCE6,
	0xCF0,
	0xCF1,
	0xCF3,
	0xD01,
	0xD04,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD4D,
	0xD4E,
	0xD4F,
	0xD54,
	0xD58,
	0xD5F,
	0xD64,
	0xD66,
	0xD70,
	0xD7A,
	0xD80,
	0xD82,
	0xD84,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDE6,
	0xDF0,
	0xDF2,
	0xDF4,
	0xE01,
	0xE3B,
	0xE40,
	0xE47,
	0xE4D,
	0xE4E,
	0xE50,
	0xE5A,
	0xE81,
	0xE83,
	0xE84,
	0xE85,
	0xE87,
	0xE89,
	0xE8A,
	0xE8B,
	0xE8D,
	0xE8E,
	0xE94,
	0xE98,
	0xE99,
	0xEA0,
	0xEA1,
	0xEA4,
	0xEA5,
	0xEA6,
	0xEA7,
	0xEA8,
	0xEAA,
	0xEAC,
	0xEAD,
	0xEBA,
	0xEBB,
	0xEBE,
	0xEC0,
	0xEC5,
	0xEC6,
	0xEC7,
	0xECD,
	0xECE,
	0xED0,
	0xEDA,
	0xEDC,
	0xEE0,
	0xF00,
	0xF01,
	0xF20,
	0xF2A,
	0xF40,
	0xF48,
	0xF49,
	0xF6D,
	0xF71,
	0xF82,
	0xF88,
	0xF98,
	0xF99,
	0xFBD,
	0x1000,
	0x1037,
	0x1038,
	0x1039,
	0x103B,
	0x104A,
	0x1050,
	0x1063,
	0x1065,
	0x1069,
	0x106E,
	0x1087,
	0x108E,
	0x108F,
	0x1090,
	0x109A,
	0x109C,
	0x109E,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x10FB,
	0x10FC,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x135F,
	0x1360,
	0x1380,
	0x1390,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1401,
	0x166D,
	0x166F,
	0x1680,
	0x1681,
	0x169B,
	0x16A0,
	0x16EB,
	0x16EE,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1714,
	0x1720,
	0x1734,
	0x1740,
	0x1754,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1772,
	0x1774,
	0x1780,
	0x17B4,
	0x17B6,
	0x17C9,
	0x17D7,
	0x17D8,
	0x17DC,
	0x17DD,
	0x17E0,
	0x17EA,
	0x1810,
	0x181A,
	0x1820,
	0x1878,
	0x1880,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1920,
	0x192C,
	0x1930,
	0x1939,
	0x1946,
	0x196E,
	0x1970,
	0x1975,
	0x1980,
	0x19AC,
	0x19B0,
	0x19CA,
	0x19D0,
	0x19DA,
	0x1A00,
	0x1A1C,
	0x1A20,
	0x1A5F,
	0x1A61,
	0x1A75,
	0x1A80,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1AA7,
	0x1AA8,
	0x1B00,
	0x1B34,
	0x1B35,
	0x1B44,
	0x1B45,
	0x1B4C,
	0x1B50,
	0x1B5A,
	0x1B80,
	0x1BAA,
	0x1BAC,
	0x1BE6,
	0x1BE7,
	0x1BF2,
	0x1C00,
	0x1C36,
	0x1C40,
	0x1C4A,
	0x1C4D,
	0x1C7E,
	0x1C80,
	0x1C89,
	0x1CE9,
	0x1CED,
	0x1CEE,
	0x1CF4,
	0x1CF5,
	0x1CF7,
	0x1D00,
	0x1DC0,
	0x1DE7,
	0x1DF5,
	0x1E00,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FBD,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FCD,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FE0,
	0x1FED,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFD,
	0x2071,
	0x2072,
	0x207F,
	0x2080,
	0x2090,
	0x209D,
	0x2102,
	0x2103,
	0x2107,
	0x2108,
	0x210A,
	0x2114,
	0x2115,
	0x2116,
	0x2119,
	0x211E,
	0x2124,
	0x2125,
	0x2126,
	0x2127,
	0x2128,
	0x2129,
	0x212A,
	0x212E,
	0x212F,
	0x213A,
	0x213C,
	0x2140,
	0x2145,
	0x214A,
	0x214E,
	0x214F,
	0x2160,
	0x2189,
	0x24B6,
	0x24EA,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CE5,
	0x2CEB,
	0x2CEF,
	0x2CF2,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D70,
	0x2D80,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x2DE0,
	0x2E00,
	0x2E2F,
	0x2E30,
	0x3005,
	0x3008,
	0x3021,
	0x302A,
	0x3031,
	0x3036,
	0x3038,
	0x303D,
	0x3041,
	0x3097,
	0x309D,
	0x30A0,
	0x30A1,
	0x30FB,
	0x30FC,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x31A0,
	0x31BB,
	0x31F0,
	0x3200,
	0x3400,
	0x4DB6,
	0x4E00,
	0x9FD6,
	0xA000,
	0xA48D,
	0xA4D0,
	0xA4FE,
	0xA500,
	0xA60D,
	0xA610,
	0xA62C,
	0xA640,
	0xA66F,
	0xA674,
	0xA67C,
	0xA67F,
	0xA6F0,
	0xA717,
	0xA720,
	0xA722,
	0xA789,
	0xA78B,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F7,
	0xA802,
	0xA803,
	0xA806,
	0xA807,
	0xA80B,
	0xA80C,
	0xA828,
	0xA840,
	0xA874,
	0xA880,
	0xA8C4,
	0xA8C5,
	0xA8C6,
	0xA8D0,
	0xA8DA,
	0xA8F2,
	0xA8F8,
	0xA8FB,
	0xA8FC,
	0xA8FD,
	0xA8FE,
	0xA900,
	0xA92B,
	0xA930,
	0xA953,
	0xA960,
	0xA97D,
	0xA980,
	0xA9B3,
	0xA9B4,
	0xA9C0,
	0xA9CF,
	0xA9DA,
	0xA9E0,
	0xA9E5,
	0xA9E6,
	0xA9FF,
	0xAA00,
	0xAA37,
	0xAA40,
	0xAA4E,
	0xAA50,
	0xAA5A,
	0xAA60,
	0xAA77,
	0xAA7A,
	0xAA7B,
	0xAA7E,
	0xAABF,
	0xAAC0,
	0xAAC1,
	0xAAC2,
	0xAAC3,
	0xAADB,
	0xAADE,
	0xAAE0,
	0xAAF0,
	0xAAF2,
	0xAAF6,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB5B,
	0xAB5C,
	0xAB66,
	0xAB70,
	0xABEB,
	0xABF0,
	0xABFA,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xF900,
	0xFA6E,
	0xFA70,
	0xFADA,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB29,
	0xFB2A,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFBB2,
	0xFBD3,
	0xFD3E,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFC,
	0xFE70,
	0xFE75,
	0xFE76,
	0xFEFD,
	0xFF10,
	0xFF1A,
	0xFF21,
	0xFF3B,
	0xFF41,
	0xFF5B,
	0xFF66,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10140,
	0x10175,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x10300,
	0x10320,
	0x10330,
	0x1034B,
	0x10350,
	0x1037B,
	0x10380,
	0x1039E,
	0x103A0,
	0x103C4,
	0x103C8,
	0x103D0,
	0x103D1,
	0x103D6,
	0x10400,
	0x1049E,
	0x104A0,
	0x104AA,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10860,
	0x10877,
	0x10880,
	0x1089F,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x10900,
	0x10916,
	0x10920,
	0x1093A,
	0x10980,
	0x109B8,
	0x109BE,
	0x109C0,
	0x10A00,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A60,
	0x10A7D,
	0x10A80,
	0x10A9D,
	0x10AC0,
	0x10AC8,
	0x10AC9,
	0x10AE5,
	0x10B00,
	0x10B36,
	0x10B40,
	0x10B56,
	0x10B60,
	0x10B73,
	0x10B80,
	0x10B92,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x11000,
	0x11046,
	0x11066,
	0x11070,
	0x11082,
	0x110B9,
	0x110D0,
	0x110E9,
	0x110F0,
	0x110FA,
	0x11100,
	0x11133,
	0x11136,
	0x11140,
	0x11150,
	0x11173,
	0x11176,
	0x11177,
	0x11180,
	0x111C0,
	0x111C1,
	0x111C5,
	0x111D0,
	0x111DB,
	0x111DC,
	0x111DD,
	0x11200,
	0x11212,
	0x11213,
	0x11235,
	0x11237,
	0x11238,
	0x1123E,
	0x1123F,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112A9,
	0x112B0,
	0x112E9,
	0x112F0,
	0x112FA,
	0x11300,
	0x11304,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133D,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134D,
	0x11350,
	0x11351,
	0x11357,
	0x11358,
	0x1135D,
	0x11364,
	0x11400,
	0x11442,
	0x11443,
	0x11446,
	0x11447,
	0x1144B,
	0x11450,
	0x1145A,
	0x11480,
	0x114C2,
	0x114C4,
	0x114C6,
	0x114C7,
	0x114C8,
	0x114D0,
	0x114DA,
	0x11580,
	0x115B6,
	0x115B8,
	0x115BF,
	0x115D8,
	0x115DE,
	0x11600,
	0x1163F,
	0x11640,
	0x11641,
	0x11644,
	0x11645,
	0x11650,
	0x1165A,
	0x11680,
	0x116B6,
	0x116C0,
	0x116CA,
	0x11700,
	0x1171A,
	0x1171D,
	0x1172B,
	0x11730,
	0x1173A,
	0x118A0,
	0x118EA,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C37,
	0x11C38,
	0x11C3F,
	0x11C40,
	0x11C41,
	0x11C50,
	0x11C5A,
	0x11C72,
	0x11C90,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16A60,
	0x16A6A,
	0x16AD0,
	0x16AEE,
	0x16B00,
	0x16B37,
	0x16B40,
	0x16B44,
	0x16B50,
	0x16B5A,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F7F,
	0x16F93,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1BC9E,
	0x1BC9F,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D6C1,
	0x1D6C2,
	0x1D6DB,
	0x1D6DC,
	0x1D6FB,
	0x1D6FC,
	0x1D715,
	0x1D716,
	0x1D735,
	0x1D736,
	0x1D74F,
	0x1D750,
	0x1D76F,
	0x1D770,
	0x1D789,
	0x1D78A,
	0x1D7A9,
	0x1D7AA,
	0x1D7C3,
	0x1D7C4,
	0x1D7CC,
	0x1D7CE,
	0x1D800,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E800,
	0x1E8C5,
	0x1E900,
	0x1E944,
	0x1E947,
	0x1E948,
	0x1E950,
	0x1E95A,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x1F130,
	0x1F14A,
	0x1F150,
	0x1F16A,
	0x1F170,
	0x1F18A,
	0x20000,
	0x2A6D7,
	0x2A700,
	0x2B735,
	0x2B740,
	0x2B81E,
	0x2B820,
	0x2CEA2,
	0x2F800,
	0x2FA1E
};

static const UV XPosixAlpha_invlist[] = { /* for ASCII/Latin1 */
	1289,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x41,
	0x5B,
	0x61,
	0x7B,
	0xAA,
	0xAB,
	0xB5,
	0xB6,
	0xBA,
	0xBB,
	0xC0,
	0xD7,
	0xD8,
	0xF7,
	0xF8,
	0x2C2,
	0x2C6,
	0x2D2,
	0x2E0,
	0x2E5,
	0x2EC,
	0x2ED,
	0x2EE,
	0x2EF,
	0x345,
	0x346,
	0x370,
	0x375,
	0x376,
	0x378,
	0x37A,
	0x37E,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x3F6,
	0x3F7,
	0x482,
	0x48A,
	0x530,
	0x531,
	0x557,
	0x559,
	0x55A,
	0x561,
	0x588,
	0x5B0,
	0x5BE,
	0x5BF,
	0x5C0,
	0x5C1,
	0x5C3,
	0x5C4,
	0x5C6,
	0x5C7,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F3,
	0x610,
	0x61B,
	0x620,
	0x658,
	0x659,
	0x660,
	0x66E,
	0x6D4,
	0x6D5,
	0x6DD,
	0x6E1,
	0x6E9,
	0x6ED,
	0x6F0,
	0x6FA,
	0x6FD,
	0x6FF,
	0x700,
	0x710,
	0x740,
	0x74D,
	0x7B2,
	0x7CA,
	0x7EB,
	0x7F4,
	0x7F6,
	0x7FA,
	0x7FB,
	0x800,
	0x818,
	0x81A,
	0x82D,
	0x840,
	0x859,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x8D4,
	0x8E0,
	0x8E3,
	0x8EA,
	0x8F0,
	0x93C,
	0x93D,
	0x94D,
	0x94E,
	0x951,
	0x955,
	0x964,
	0x971,
	0x984,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BD,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CD,
	0x9CE,
	0x9CF,
	0x9D7,
	0x9D8,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E4,
	0x9F0,
	0x9F2,
	0xA01,
	0xA04,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4D,
	0xA51,
	0xA52,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA70,
	0xA76,
	0xA81,
	0xA84,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABD,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACD,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE4,
	0xAF9,
	0xAFA,
	0xB01,
	0xB04,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3D,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4D,
	0xB56,
	0xB58,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB64,
	0xB71,
	0xB72,
	0xB82,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCD,
	0xBD0,
	0xBD1,
	0xBD7,
	0xBD8,
	0xC00,
	0xC04,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4D,
	0xC55,
	0xC57,
	0xC58,
	0xC5B,
	0xC60,
	0xC64,
	0xC80,
	0xC84,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBD,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCD,
	0xCD5,
	0xCD7,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE4,
	0xCF1,
	0xCF3,
	0xD01,
	0xD04,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD4D,
	0xD4E,
	0xD4F,
	0xD54,
	0xD58,
	0xD5F,
	0xD64,
	0xD7A,
	0xD80,
	0xD82,
	0xD84,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDF2,
	0xDF4,
	0xE01,
	0xE3B,
	0xE40,
	0xE47,
	0xE4D,
	0xE4E,
	0xE81,
	0xE83,
	0xE84,
	0xE85,
	0xE87,
	0xE89,
	0xE8A,
	0xE8B,
	0xE8D,
	0xE8E,
	0xE94,
	0xE98,
	0xE99,
	0xEA0,
	0xEA1,
	0xEA4,
	0xEA5,
	0xEA6,
	0xEA7,
	0xEA8,
	0xEAA,
	0xEAC,
	0xEAD,
	0xEBA,
	0xEBB,
	0xEBE,
	0xEC0,
	0xEC5,
	0xEC6,
	0xEC7,
	0xECD,
	0xECE,
	0xEDC,
	0xEE0,
	0xF00,
	0xF01,
	0xF40,
	0xF48,
	0xF49,
	0xF6D,
	0xF71,
	0xF82,
	0xF88,
	0xF98,
	0xF99,
	0xFBD,
	0x1000,
	0x1037,
	0x1038,
	0x1039,
	0x103B,
	0x1040,
	0x1050,
	0x1063,
	0x1065,
	0x1069,
	0x106E,
	0x1087,
	0x108E,
	0x108F,
	0x109C,
	0x109E,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x10FB,
	0x10FC,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x135F,
	0x1360,
	0x1380,
	0x1390,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1401,
	0x166D,
	0x166F,
	0x1680,
	0x1681,
	0x169B,
	0x16A0,
	0x16EB,
	0x16EE,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1714,
	0x1720,
	0x1734,
	0x1740,
	0x1754,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1772,
	0x1774,
	0x1780,
	0x17B4,
	0x17B6,
	0x17C9,
	0x17D7,
	0x17D8,
	0x17DC,
	0x17DD,
	0x1820,
	0x1878,
	0x1880,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1920,
	0x192C,
	0x1930,
	0x1939,
	0x1950,
	0x196E,
	0x1970,
	0x1975,
	0x1980,
	0x19AC,
	0x19B0,
	0x19CA,
	0x1A00,
	0x1A1C,
	0x1A20,
	0x1A5F,
	0x1A61,
	0x1A75,
	0x1AA7,
	0x1AA8,
	0x1B00,
	0x1B34,
	0x1B35,
	0x1B44,
	0x1B45,
	0x1B4C,
	0x1B80,
	0x1BAA,
	0x1BAC,
	0x1BB0,
	0x1BBA,
	0x1BE6,
	0x1BE7,
	0x1BF2,
	0x1C00,
	0x1C36,
	0x1C4D,
	0x1C50,
	0x1C5A,
	0x1C7E,
	0x1C80,
	0x1C89,
	0x1CE9,
	0x1CED,
	0x1CEE,
	0x1CF4,
	0x1CF5,
	0x1CF7,
	0x1D00,
	0x1DC0,
	0x1DE7,
	0x1DF5,
	0x1E00,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FBD,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FCD,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FE0,
	0x1FED,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFD,
	0x2071,
	0x2072,
	0x207F,
	0x2080,
	0x2090,
	0x209D,
	0x2102,
	0x2103,
	0x2107,
	0x2108,
	0x210A,
	0x2114,
	0x2115,
	0x2116,
	0x2119,
	0x211E,
	0x2124,
	0x2125,
	0x2126,
	0x2127,
	0x2128,
	0x2129,
	0x212A,
	0x212E,
	0x212F,
	0x213A,
	0x213C,
	0x2140,
	0x2145,
	0x214A,
	0x214E,
	0x214F,
	0x2160,
	0x2189,
	0x24B6,
	0x24EA,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CE5,
	0x2CEB,
	0x2CEF,
	0x2CF2,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D70,
	0x2D80,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x2DE0,
	0x2E00,
	0x2E2F,
	0x2E30,
	0x3005,
	0x3008,
	0x3021,
	0x302A,
	0x3031,
	0x3036,
	0x3038,
	0x303D,
	0x3041,
	0x3097,
	0x309D,
	0x30A0,
	0x30A1,
	0x30FB,
	0x30FC,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x31A0,
	0x31BB,
	0x31F0,
	0x3200,
	0x3400,
	0x4DB6,
	0x4E00,
	0x9FD6,
	0xA000,
	0xA48D,
	0xA4D0,
	0xA4FE,
	0xA500,
	0xA60D,
	0xA610,
	0xA620,
	0xA62A,
	0xA62C,
	0xA640,
	0xA66F,
	0xA674,
	0xA67C,
	0xA67F,
	0xA6F0,
	0xA717,
	0xA720,
	0xA722,
	0xA789,
	0xA78B,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F7,
	0xA802,
	0xA803,
	0xA806,
	0xA807,
	0xA80B,
	0xA80C,
	0xA828,
	0xA840,
	0xA874,
	0xA880,
	0xA8C4,
	0xA8C5,
	0xA8C6,
	0xA8F2,
	0xA8F8,
	0xA8FB,
	0xA8FC,
	0xA8FD,
	0xA8FE,
	0xA90A,
	0xA92B,
	0xA930,
	0xA953,
	0xA960,
	0xA97D,
	0xA980,
	0xA9B3,
	0xA9B4,
	0xA9C0,
	0xA9CF,
	0xA9D0,
	0xA9E0,
	0xA9E5,
	0xA9E6,
	0xA9F0,
	0xA9FA,
	0xA9FF,
	0xAA00,
	0xAA37,
	0xAA40,
	0xAA4E,
	0xAA60,
	0xAA77,
	0xAA7A,
	0xAA7B,
	0xAA7E,
	0xAABF,
	0xAAC0,
	0xAAC1,
	0xAAC2,
	0xAAC3,
	0xAADB,
	0xAADE,
	0xAAE0,
	0xAAF0,
	0xAAF2,
	0xAAF6,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB5B,
	0xAB5C,
	0xAB66,
	0xAB70,
	0xABEB,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xF900,
	0xFA6E,
	0xFA70,
	0xFADA,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB29,
	0xFB2A,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFBB2,
	0xFBD3,
	0xFD3E,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFC,
	0xFE70,
	0xFE75,
	0xFE76,
	0xFEFD,
	0xFF21,
	0xFF3B,
	0xFF41,
	0xFF5B,
	0xFF66,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10140,
	0x10175,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x10300,
	0x10320,
	0x10330,
	0x1034B,
	0x10350,
	0x1037B,
	0x10380,
	0x1039E,
	0x103A0,
	0x103C4,
	0x103C8,
	0x103D0,
	0x103D1,
	0x103D6,
	0x10400,
	0x1049E,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10860,
	0x10877,
	0x10880,
	0x1089F,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x10900,
	0x10916,
	0x10920,
	0x1093A,
	0x10980,
	0x109B8,
	0x109BE,
	0x109C0,
	0x10A00,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A60,
	0x10A7D,
	0x10A80,
	0x10A9D,
	0x10AC0,
	0x10AC8,
	0x10AC9,
	0x10AE5,
	0x10B00,
	0x10B36,
	0x10B40,
	0x10B56,
	0x10B60,
	0x10B73,
	0x10B80,
	0x10B92,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x11000,
	0x11046,
	0x11082,
	0x110B9,
	0x110D0,
	0x110E9,
	0x11100,
	0x11133,
	0x11150,
	0x11173,
	0x11176,
	0x11177,
	0x11180,
	0x111C0,
	0x111C1,
	0x111C5,
	0x111DA,
	0x111DB,
	0x111DC,
	0x111DD,
	0x11200,
	0x11212,
	0x11213,
	0x11235,
	0x11237,
	0x11238,
	0x1123E,
	0x1123F,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112A9,
	0x112B0,
	0x112E9,
	0x11300,
	0x11304,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133D,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134D,
	0x11350,
	0x11351,
	0x11357,
	0x11358,
	0x1135D,
	0x11364,
	0x11400,
	0x11442,
	0x11443,
	0x11446,
	0x11447,
	0x1144B,
	0x11480,
	0x114C2,
	0x114C4,
	0x114C6,
	0x114C7,
	0x114C8,
	0x11580,
	0x115B6,
	0x115B8,
	0x115BF,
	0x115D8,
	0x115DE,
	0x11600,
	0x1163F,
	0x11640,
	0x11641,
	0x11644,
	0x11645,
	0x11680,
	0x116B6,
	0x11700,
	0x1171A,
	0x1171D,
	0x1172B,
	0x118A0,
	0x118E0,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C37,
	0x11C38,
	0x11C3F,
	0x11C40,
	0x11C41,
	0x11C72,
	0x11C90,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16AD0,
	0x16AEE,
	0x16B00,
	0x16B37,
	0x16B40,
	0x16B44,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F7F,
	0x16F93,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1BC9E,
	0x1BC9F,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D6C1,
	0x1D6C2,
	0x1D6DB,
	0x1D6DC,
	0x1D6FB,
	0x1D6FC,
	0x1D715,
	0x1D716,
	0x1D735,
	0x1D736,
	0x1D74F,
	0x1D750,
	0x1D76F,
	0x1D770,
	0x1D789,
	0x1D78A,
	0x1D7A9,
	0x1D7AA,
	0x1D7C3,
	0x1D7C4,
	0x1D7CC,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E800,
	0x1E8C5,
	0x1E900,
	0x1E944,
	0x1E947,
	0x1E948,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x1F130,
	0x1F14A,
	0x1F150,
	0x1F16A,
	0x1F170,
	0x1F18A,
	0x20000,
	0x2A6D7,
	0x2A700,
	0x2B735,
	0x2B740,
	0x2B81E,
	0x2B820,
	0x2CEA2,
	0x2F800,
	0x2FA1E
};

static const UV XPosixBlank_invlist[] = { /* for ASCII/Latin1 */
	17,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x9,
	0xA,
	0x20,
	0x21,
	0xA0,
	0xA1,
	0x1680,
	0x1681,
	0x2000,
	0x200B,
	0x202F,
	0x2030,
	0x205F,
	0x2060,
	0x3000,
	0x3001
};

static const UV XPosixCntrl_invlist[] = { /* for ASCII/Latin1 */
	4,	/* Number of elements */
	148565664, /* Version and data structure type */
	0,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x20,
	0x7F,
	0xA0
};

static const UV XPosixDigit_invlist[] = { /* for ASCII/Latin1 */
	109,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x30,
	0x3A,
	0x660,
	0x66A,
	0x6F0,
	0x6FA,
	0x7C0,
	0x7CA,
	0x966,
	0x970,
	0x9E6,
	0x9F0,
	0xA66,
	0xA70,
	0xAE6,
	0xAF0,
	0xB66,
	0xB70,
	0xBE6,
	0xBF0,
	0xC66,
	0xC70,
	0xCE6,
	0xCF0,
	0xD66,
	0xD70,
	0xDE6,
	0xDF0,
	0xE50,
	0xE5A,
	0xED0,
	0xEDA,
	0xF20,
	0xF2A,
	0x1040,
	0x104A,
	0x1090,
	0x109A,
	0x17E0,
	0x17EA,
	0x1810,
	0x181A,
	0x1946,
	0x1950,
	0x19D0,
	0x19DA,
	0x1A80,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1B50,
	0x1B5A,
	0x1BB0,
	0x1BBA,
	0x1C40,
	0x1C4A,
	0x1C50,
	0x1C5A,
	0xA620,
	0xA62A,
	0xA8D0,
	0xA8DA,
	0xA900,
	0xA90A,
	0xA9D0,
	0xA9DA,
	0xA9F0,
	0xA9FA,
	0xAA50,
	0xAA5A,
	0xABF0,
	0xABFA,
	0xFF10,
	0xFF1A,
	0x104A0,
	0x104AA,
	0x11066,
	0x11070,
	0x110F0,
	0x110FA,
	0x11136,
	0x11140,
	0x111D0,
	0x111DA,
	0x112F0,
	0x112FA,
	0x11450,
	0x1145A,
	0x114D0,
	0x114DA,
	0x11650,
	0x1165A,
	0x116C0,
	0x116CA,
	0x11730,
	0x1173A,
	0x118E0,
	0x118EA,
	0x11C50,
	0x11C5A,
	0x16A60,
	0x16A6A,
	0x16B50,
	0x16B5A,
	0x1D7CE,
	0x1D800,
	0x1E950,
	0x1E95A
};

static const UV XPosixGraph_invlist[] = { /* for ASCII/Latin1 */
	1287,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x21,
	0x7F,
	0xA1,
	0x378,
	0x37A,
	0x380,
	0x384,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x530,
	0x531,
	0x557,
	0x559,
	0x560,
	0x561,
	0x588,
	0x589,
	0x58B,
	0x58D,
	0x590,
	0x591,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F5,
	0x600,
	0x61D,
	0x61E,
	0x70E,
	0x70F,
	0x74B,
	0x74D,
	0x7B2,
	0x7C0,
	0x7FB,
	0x800,
	0x82E,
	0x830,
	0x83F,
	0x840,
	0x85C,
	0x85E,
	0x85F,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x8D4,
	0x984,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BC,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CF,
	0x9D7,
	0x9D8,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E4,
	0x9E6,
	0x9FC,
	0xA01,
	0xA04,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA3C,
	0xA3D,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4E,
	0xA51,
	0xA52,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA66,
	0xA76,
	0xA81,
	0xA84,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABC,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACE,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE4,
	0xAE6,
	0xAF2,
	0xAF9,
	0xAFA,
	0xB01,
	0xB04,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3C,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4E,
	0xB56,
	0xB58,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB64,
	0xB66,
	0xB78,
	0xB82,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCE,
	0xBD0,
	0xBD1,
	0xBD7,
	0xBD8,
	0xBE6,
	0xBFB,
	0xC00,
	0xC04,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4E,
	0xC55,
	0xC57,
	0xC58,
	0xC5B,
	0xC60,
	0xC64,
	0xC66,
	0xC70,
	0xC78,
	0xC84,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBC,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCE,
	0xCD5,
	0xCD7,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE4,
	0xCE6,
	0xCF0,
	0xCF1,
	0xCF3,
	0xD01,
	0xD04,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD50,
	0xD54,
	0xD64,
	0xD66,
	0xD80,
	0xD82,
	0xD84,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xDCA,
	0xDCB,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDE6,
	0xDF0,
	0xDF2,
	0xDF5,
	0xE01,
	0xE3B,
	0xE3F,
	0xE5C,
	0xE81,
	0xE83,
	0xE84,
	0xE85,
	0xE87,
	0xE89,
	0xE8A,
	0xE8B,
	0xE8D,
	0xE8E,
	0xE94,
	0xE98,
	0xE99,
	0xEA0,
	0xEA1,
	0xEA4,
	0xEA5,
	0xEA6,
	0xEA7,
	0xEA8,
	0xEAA,
	0xEAC,
	0xEAD,
	0xEBA,
	0xEBB,
	0xEBE,
	0xEC0,
	0xEC5,
	0xEC6,
	0xEC7,
	0xEC8,
	0xECE,
	0xED0,
	0xEDA,
	0xEDC,
	0xEE0,
	0xF00,
	0xF48,
	0xF49,
	0xF6D,
	0xF71,
	0xF98,
	0xF99,
	0xFBD,
	0xFBE,
	0xFCD,
	0xFCE,
	0xFDB,
	0x1000,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x135D,
	0x137D,
	0x1380,
	0x139A,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1400,
	0x1680,
	0x1681,
	0x169D,
	0x16A0,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1715,
	0x1720,
	0x1737,
	0x1740,
	0x1754,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1772,
	0x1774,
	0x1780,
	0x17DE,
	0x17E0,
	0x17EA,
	0x17F0,
	0x17FA,
	0x1800,
	0x180F,
	0x1810,
	0x181A,
	0x1820,
	0x1878,
	0x1880,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1920,
	0x192C,
	0x1930,
	0x193C,
	0x1940,
	0x1941,
	0x1944,
	0x196E,
	0x1970,
	0x1975,
	0x1980,
	0x19AC,
	0x19B0,
	0x19CA,
	0x19D0,
	0x19DB,
	0x19DE,
	0x1A1C,
	0x1A1E,
	0x1A5F,
	0x1A60,
	0x1A7D,
	0x1A7F,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1AA0,
	0x1AAE,
	0x1AB0,
	0x1ABF,
	0x1B00,
	0x1B4C,
	0x1B50,
	0x1B7D,
	0x1B80,
	0x1BF4,
	0x1BFC,
	0x1C38,
	0x1C3B,
	0x1C4A,
	0x1C4D,
	0x1C89,
	0x1CC0,
	0x1CC8,
	0x1CD0,
	0x1CF7,
	0x1CF8,
	0x1CFA,
	0x1D00,
	0x1DF6,
	0x1DFB,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FC5,
	0x1FC6,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FDD,
	0x1FF0,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFF,
	0x200B,
	0x2028,
	0x202A,
	0x202F,
	0x2030,
	0x205F,
	0x2060,
	0x2065,
	0x2066,
	0x2072,
	0x2074,
	0x208F,
	0x2090,
	0x209D,
	0x20A0,
	0x20BF,
	0x20D0,
	0x20F1,
	0x2100,
	0x218C,
	0x2190,
	0x23FF,
	0x2400,
	0x2427,
	0x2440,
	0x244B,
	0x2460,
	0x2B74,
	0x2B76,
	0x2B96,
	0x2B98,
	0x2BBA,
	0x2BBD,
	0x2BC9,
	0x2BCA,
	0x2BD2,
	0x2BEC,
	0x2BF0,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CF4,
	0x2CF9,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D71,
	0x2D7F,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x2DE0,
	0x2E45,
	0x2E80,
	0x2E9A,
	0x2E9B,
	0x2EF4,
	0x2F00,
	0x2FD6,
	0x2FF0,
	0x2FFC,
	0x3001,
	0x3040,
	0x3041,
	0x3097,
	0x3099,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x3190,
	0x31BB,
	0x31C0,
	0x31E4,
	0x31F0,
	0x321F,
	0x3220,
	0x32FF,
	0x3300,
	0x4DB6,
	0x4DC0,
	0x9FD6,
	0xA000,
	0xA48D,
	0xA490,
	0xA4C7,
	0xA4D0,
	0xA62C,
	0xA640,
	0xA6F8,
	0xA700,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F7,
	0xA82C,
	0xA830,
	0xA83A,
	0xA840,
	0xA878,
	0xA880,
	0xA8C6,
	0xA8CE,
	0xA8DA,
	0xA8E0,
	0xA8FE,
	0xA900,
	0xA954,
	0xA95F,
	0xA97D,
	0xA980,
	0xA9CE,
	0xA9CF,
	0xA9DA,
	0xA9DE,
	0xA9FF,
	0xAA00,
	0xAA37,
	0xAA40,
	0xAA4E,
	0xAA50,
	0xAA5A,
	0xAA5C,
	0xAAC3,
	0xAADB,
	0xAAF7,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB66,
	0xAB70,
	0xABEE,
	0xABF0,
	0xABFA,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xE000,
	0xFA6E,
	0xFA70,
	0xFADA,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFBC2,
	0xFBD3,
	0xFD40,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFE,
	0xFE00,
	0xFE1A,
	0xFE20,
	0xFE53,
	0xFE54,
	0xFE67,
	0xFE68,
	0xFE6C,
	0xFE70,
	0xFE75,
	0xFE76,
	0xFEFD,
	0xFEFF,
	0xFF00,
	0xFF01,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0xFFE0,
	0xFFE7,
	0xFFE8,
	0xFFEF,
	0xFFF9,
	0xFFFE,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10100,
	0x10103,
	0x10107,
	0x10134,
	0x10137,
	0x1018F,
	0x10190,
	0x1019C,
	0x101A0,
	0x101A1,
	0x101D0,
	0x101FE,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x102E0,
	0x102FC,
	0x10300,
	0x10324,
	0x10330,
	0x1034B,
	0x10350,
	0x1037B,
	0x10380,
	0x1039E,
	0x1039F,
	0x103C4,
	0x103C8,
	0x103D6,
	0x10400,
	0x1049E,
	0x104A0,
	0x104AA,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x1056F,
	0x10570,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10857,
	0x1089F,
	0x108A7,
	0x108B0,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x108FB,
	0x1091C,
	0x1091F,
	0x1093A,
	0x1093F,
	0x10940,
	0x10980,
	0x109B8,
	0x109BC,
	0x109D0,
	0x109D2,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A38,
	0x10A3B,
	0x10A3F,
	0x10A48,
	0x10A50,
	0x10A59,
	0x10A60,
	0x10AA0,
	0x10AC0,
	0x10AE7,
	0x10AEB,
	0x10AF7,
	0x10B00,
	0x10B36,
	0x10B39,
	0x10B56,
	0x10B58,
	0x10B73,
	0x10B78,
	0x10B92,
	0x10B99,
	0x10B9D,
	0x10BA9,
	0x10BB0,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x10CFA,
	0x10D00,
	0x10E60,
	0x10E7F,
	0x11000,
	0x1104E,
	0x11052,
	0x11070,
	0x1107F,
	0x110C2,
	0x110D0,
	0x110E9,
	0x110F0,
	0x110FA,
	0x11100,
	0x11135,
	0x11136,
	0x11144,
	0x11150,
	0x11177,
	0x11180,
	0x111CE,
	0x111D0,
	0x111E0,
	0x111E1,
	0x111F5,
	0x11200,
	0x11212,
	0x11213,
	0x1123F,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112AA,
	0x112B0,
	0x112EB,
	0x112F0,
	0x112FA,
	0x11300,
	0x11304,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133C,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134E,
	0x11350,
	0x11351,
	0x11357,
	0x11358,
	0x1135D,
	0x11364,
	0x11366,
	0x1136D,
	0x11370,
	0x11375,
	0x11400,
	0x1145A,
	0x1145B,
	0x1145C,
	0x1145D,
	0x1145E,
	0x11480,
	0x114C8,
	0x114D0,
	0x114DA,
	0x11580,
	0x115B6,
	0x115B8,
	0x115DE,
	0x11600,
	0x11645,
	0x11650,
	0x1165A,
	0x11660,
	0x1166D,
	0x11680,
	0x116B8,
	0x116C0,
	0x116CA,
	0x11700,
	0x1171A,
	0x1171D,
	0x1172C,
	0x11730,
	0x11740,
	0x118A0,
	0x118F3,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C37,
	0x11C38,
	0x11C46,
	0x11C50,
	0x11C6D,
	0x11C70,
	0x11C90,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12470,
	0x12475,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16A60,
	0x16A6A,
	0x16A6E,
	0x16A70,
	0x16AD0,
	0x16AEE,
	0x16AF0,
	0x16AF6,
	0x16B00,
	0x16B46,
	0x16B50,
	0x16B5A,
	0x16B5B,
	0x16B62,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F7F,
	0x16F8F,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1BC9C,
	0x1BCA4,
	0x1D000,
	0x1D0F6,
	0x1D100,
	0x1D127,
	0x1D129,
	0x1D1E9,
	0x1D200,
	0x1D246,
	0x1D300,
	0x1D357,
	0x1D360,
	0x1D372,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D7CC,
	0x1D7CE,
	0x1DA8C,
	0x1DA9B,
	0x1DAA0,
	0x1DAA1,
	0x1DAB0,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E800,
	0x1E8C5,
	0x1E8C7,
	0x1E8D7,
	0x1E900,
	0x1E94B,
	0x1E950,
	0x1E95A,
	0x1E95E,
	0x1E960,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x1EEF0,
	0x1EEF2,
	0x1F000,
	0x1F02C,
	0x1F030,
	0x1F094,
	0x1F0A0,
	0x1F0AF,
	0x1F0B1,
	0x1F0C0,
	0x1F0C1,
	0x1F0D0,
	0x1F0D1,
	0x1F0F6,
	0x1F100,
	0x1F10D,
	0x1F110,
	0x1F12F,
	0x1F130,
	0x1F16C,
	0x1F170,
	0x1F1AD,
	0x1F1E6,
	0x1F203,
	0x1F210,
	0x1F23C,
	0x1F240,
	0x1F249,
	0x1F250,
	0x1F252,
	0x1F300,
	0x1F6D3,
	0x1F6E0,
	0x1F6ED,
	0x1F6F0,
	0x1F6F7,
	0x1F700,
	0x1F774,
	0x1F780,
	0x1F7D5,
	0x1F800,
	0x1F80C,
	0x1F810,
	0x1F848,
	0x1F850,
	0x1F85A,
	0x1F860,
	0x1F888,
	0x1F890,
	0x1F8AE,
	0x1F910,
	0x1F91F,
	0x1F920,
	0x1F928,
	0x1F930,
	0x1F931,
	0x1F933,
	0x1F93F,
	0x1F940,
	0x1F94C,
	0x1F950,
	0x1F95F,
	0x1F980,
	0x1F992,
	0x1F9C0,
	0x1F9C1,
	0x20000,
	0x2A6D7,
	0x2A700,
	0x2B735,
	0x2B740,
	0x2B81E,
	0x2B820,
	0x2CEA2,
	0x2F800,
	0x2FA1E,
	0xE0001,
	0xE0002,
	0xE0020,
	0xE0080,
	0xE0100,
	0xE01F0,
	0xF0000,
	0xFFFFE,
	0x100000,
	0x10FFFE
};

static const UV XPosixLower_invlist[] = { /* for ASCII/Latin1 */
	1281,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x61,
	0x7B,
	0xAA,
	0xAB,
	0xB5,
	0xB6,
	0xBA,
	0xBB,
	0xDF,
	0xF7,
	0xF8,
	0x100,
	0x101,
	0x102,
	0x103,
	0x104,
	0x105,
	0x106,
	0x107,
	0x108,
	0x109,
	0x10A,
	0x10B,
	0x10C,
	0x10D,
	0x10E,
	0x10F,
	0x110,
	0x111,
	0x112,
	0x113,
	0x114,
	0x115,
	0x116,
	0x117,
	0x118,
	0x119,
	0x11A,
	0x11B,
	0x11C,
	0x11D,
	0x11E,
	0x11F,
	0x120,
	0x121,
	0x122,
	0x123,
	0x124,
	0x125,
	0x126,
	0x127,
	0x128,
	0x129,
	0x12A,
	0x12B,
	0x12C,
	0x12D,
	0x12E,
	0x12F,
	0x130,
	0x131,
	0x132,
	0x133,
	0x134,
	0x135,
	0x136,
	0x137,
	0x139,
	0x13A,
	0x13B,
	0x13C,
	0x13D,
	0x13E,
	0x13F,
	0x140,
	0x141,
	0x142,
	0x143,
	0x144,
	0x145,
	0x146,
	0x147,
	0x148,
	0x14A,
	0x14B,
	0x14C,
	0x14D,
	0x14E,
	0x14F,
	0x150,
	0x151,
	0x152,
	0x153,
	0x154,
	0x155,
	0x156,
	0x157,
	0x158,
	0x159,
	0x15A,
	0x15B,
	0x15C,
	0x15D,
	0x15E,
	0x15F,
	0x160,
	0x161,
	0x162,
	0x163,
	0x164,
	0x165,
	0x166,
	0x167,
	0x168,
	0x169,
	0x16A,
	0x16B,
	0x16C,
	0x16D,
	0x16E,
	0x16F,
	0x170,
	0x171,
	0x172,
	0x173,
	0x174,
	0x175,
	0x176,
	0x177,
	0x178,
	0x17A,
	0x17B,
	0x17C,
	0x17D,
	0x17E,
	0x181,
	0x183,
	0x184,
	0x185,
	0x186,
	0x188,
	0x189,
	0x18C,
	0x18E,
	0x192,
	0x193,
	0x195,
	0x196,
	0x199,
	0x19C,
	0x19E,
	0x19F,
	0x1A1,
	0x1A2,
	0x1A3,
	0x1A4,
	0x1A5,
	0x1A6,
	0x1A8,
	0x1A9,
	0x1AA,
	0x1AC,
	0x1AD,
	0x1AE,
	0x1B0,
	0x1B1,
	0x1B4,
	0x1B5,
	0x1B6,
	0x1B7,
	0x1B9,
	0x1BB,
	0x1BD,
	0x1C0,
	0x1C6,
	0x1C7,
	0x1C9,
	0x1CA,
	0x1CC,
	0x1CD,
	0x1CE,
	0x1CF,
	0x1D0,
	0x1D1,
	0x1D2,
	0x1D3,
	0x1D4,
	0x1D5,
	0x1D6,
	0x1D7,
	0x1D8,
	0x1D9,
	0x1DA,
	0x1DB,
	0x1DC,
	0x1DE,
	0x1DF,
	0x1E0,
	0x1E1,
	0x1E2,
	0x1E3,
	0x1E4,
	0x1E5,
	0x1E6,
	0x1E7,
	0x1E8,
	0x1E9,
	0x1EA,
	0x1EB,
	0x1EC,
	0x1ED,
	0x1EE,
	0x1EF,
	0x1F1,
	0x1F3,
	0x1F4,
	0x1F5,
	0x1F6,
	0x1F9,
	0x1FA,
	0x1FB,
	0x1FC,
	0x1FD,
	0x1FE,
	0x1FF,
	0x200,
	0x201,
	0x202,
	0x203,
	0x204,
	0x205,
	0x206,
	0x207,
	0x208,
	0x209,
	0x20A,
	0x20B,
	0x20C,
	0x20D,
	0x20E,
	0x20F,
	0x210,
	0x211,
	0x212,
	0x213,
	0x214,
	0x215,
	0x216,
	0x217,
	0x218,
	0x219,
	0x21A,
	0x21B,
	0x21C,
	0x21D,
	0x21E,
	0x21F,
	0x220,
	0x221,
	0x222,
	0x223,
	0x224,
	0x225,
	0x226,
	0x227,
	0x228,
	0x229,
	0x22A,
	0x22B,
	0x22C,
	0x22D,
	0x22E,
	0x22F,
	0x230,
	0x231,
	0x232,
	0x233,
	0x23A,
	0x23C,
	0x23D,
	0x23F,
	0x241,
	0x242,
	0x243,
	0x247,
	0x248,
	0x249,
	0x24A,
	0x24B,
	0x24C,
	0x24D,
	0x24E,
	0x24F,
	0x294,
	0x295,
	0x2B9,
	0x2C0,
	0x2C2,
	0x2E0,
	0x2E5,
	0x345,
	0x346,
	0x371,
	0x372,
	0x373,
	0x374,
	0x377,
	0x378,
	0x37A,
	0x37E,
	0x390,
	0x391,
	0x3AC,
	0x3CF,
	0x3D0,
	0x3D2,
	0x3D5,
	0x3D8,
	0x3D9,
	0x3DA,
	0x3DB,
	0x3DC,
	0x3DD,
	0x3DE,
	0x3DF,
	0x3E0,
	0x3E1,
	0x3E2,
	0x3E3,
	0x3E4,
	0x3E5,
	0x3E6,
	0x3E7,
	0x3E8,
	0x3E9,
	0x3EA,
	0x3EB,
	0x3EC,
	0x3ED,
	0x3EE,
	0x3EF,
	0x3F4,
	0x3F5,
	0x3F6,
	0x3F8,
	0x3F9,
	0x3FB,
	0x3FD,
	0x430,
	0x460,
	0x461,
	0x462,
	0x463,
	0x464,
	0x465,
	0x466,
	0x467,
	0x468,
	0x469,
	0x46A,
	0x46B,
	0x46C,
	0x46D,
	0x46E,
	0x46F,
	0x470,
	0x471,
	0x472,
	0x473,
	0x474,
	0x475,
	0x476,
	0x477,
	0x478,
	0x479,
	0x47A,
	0x47B,
	0x47C,
	0x47D,
	0x47E,
	0x47F,
	0x480,
	0x481,
	0x482,
	0x48B,
	0x48C,
	0x48D,
	0x48E,
	0x48F,
	0x490,
	0x491,
	0x492,
	0x493,
	0x494,
	0x495,
	0x496,
	0x497,
	0x498,
	0x499,
	0x49A,
	0x49B,
	0x49C,
	0x49D,
	0x49E,
	0x49F,
	0x4A0,
	0x4A1,
	0x4A2,
	0x4A3,
	0x4A4,
	0x4A5,
	0x4A6,
	0x4A7,
	0x4A8,
	0x4A9,
	0x4AA,
	0x4AB,
	0x4AC,
	0x4AD,
	0x4AE,
	0x4AF,
	0x4B0,
	0x4B1,
	0x4B2,
	0x4B3,
	0x4B4,
	0x4B5,
	0x4B6,
	0x4B7,
	0x4B8,
	0x4B9,
	0x4BA,
	0x4BB,
	0x4BC,
	0x4BD,
	0x4BE,
	0x4BF,
	0x4C0,
	0x4C2,
	0x4C3,
	0x4C4,
	0x4C5,
	0x4C6,
	0x4C7,
	0x4C8,
	0x4C9,
	0x4CA,
	0x4CB,
	0x4CC,
	0x4CD,
	0x4CE,
	0x4D0,
	0x4D1,
	0x4D2,
	0x4D3,
	0x4D4,
	0x4D5,
	0x4D6,
	0x4D7,
	0x4D8,
	0x4D9,
	0x4DA,
	0x4DB,
	0x4DC,
	0x4DD,
	0x4DE,
	0x4DF,
	0x4E0,
	0x4E1,
	0x4E2,
	0x4E3,
	0x4E4,
	0x4E5,
	0x4E6,
	0x4E7,
	0x4E8,
	0x4E9,
	0x4EA,
	0x4EB,
	0x4EC,
	0x4ED,
	0x4EE,
	0x4EF,
	0x4F0,
	0x4F1,
	0x4F2,
	0x4F3,
	0x4F4,
	0x4F5,
	0x4F6,
	0x4F7,
	0x4F8,
	0x4F9,
	0x4FA,
	0x4FB,
	0x4FC,
	0x4FD,
	0x4FE,
	0x4FF,
	0x500,
	0x501,
	0x502,
	0x503,
	0x504,
	0x505,
	0x506,
	0x507,
	0x508,
	0x509,
	0x50A,
	0x50B,
	0x50C,
	0x50D,
	0x50E,
	0x50F,
	0x510,
	0x511,
	0x512,
	0x513,
	0x514,
	0x515,
	0x516,
	0x517,
	0x518,
	0x519,
	0x51A,
	0x51B,
	0x51C,
	0x51D,
	0x51E,
	0x51F,
	0x520,
	0x521,
	0x522,
	0x523,
	0x524,
	0x525,
	0x526,
	0x527,
	0x528,
	0x529,
	0x52A,
	0x52B,
	0x52C,
	0x52D,
	0x52E,
	0x52F,
	0x530,
	0x561,
	0x588,
	0x13F8,
	0x13FE,
	0x1C80,
	0x1C89,
	0x1D00,
	0x1DC0,
	0x1E01,
	0x1E02,
	0x1E03,
	0x1E04,
	0x1E05,
	0x1E06,
	0x1E07,
	0x1E08,
	0x1E09,
	0x1E0A,
	0x1E0B,
	0x1E0C,
	0x1E0D,
	0x1E0E,
	0x1E0F,
	0x1E10,
	0x1E11,
	0x1E12,
	0x1E13,
	0x1E14,
	0x1E15,
	0x1E16,
	0x1E17,
	0x1E18,
	0x1E19,
	0x1E1A,
	0x1E1B,
	0x1E1C,
	0x1E1D,
	0x1E1E,
	0x1E1F,
	0x1E20,
	0x1E21,
	0x1E22,
	0x1E23,
	0x1E24,
	0x1E25,
	0x1E26,
	0x1E27,
	0x1E28,
	0x1E29,
	0x1E2A,
	0x1E2B,
	0x1E2C,
	0x1E2D,
	0x1E2E,
	0x1E2F,
	0x1E30,
	0x1E31,
	0x1E32,
	0x1E33,
	0x1E34,
	0x1E35,
	0x1E36,
	0x1E37,
	0x1E38,
	0x1E39,
	0x1E3A,
	0x1E3B,
	0x1E3C,
	0x1E3D,
	0x1E3E,
	0x1E3F,
	0x1E40,
	0x1E41,
	0x1E42,
	0x1E43,
	0x1E44,
	0x1E45,
	0x1E46,
	0x1E47,
	0x1E48,
	0x1E49,
	0x1E4A,
	0x1E4B,
	0x1E4C,
	0x1E4D,
	0x1E4E,
	0x1E4F,
	0x1E50,
	0x1E51,
	0x1E52,
	0x1E53,
	0x1E54,
	0x1E55,
	0x1E56,
	0x1E57,
	0x1E58,
	0x1E59,
	0x1E5A,
	0x1E5B,
	0x1E5C,
	0x1E5D,
	0x1E5E,
	0x1E5F,
	0x1E60,
	0x1E61,
	0x1E62,
	0x1E63,
	0x1E64,
	0x1E65,
	0x1E66,
	0x1E67,
	0x1E68,
	0x1E69,
	0x1E6A,
	0x1E6B,
	0x1E6C,
	0x1E6D,
	0x1E6E,
	0x1E6F,
	0x1E70,
	0x1E71,
	0x1E72,
	0x1E73,
	0x1E74,
	0x1E75,
	0x1E76,
	0x1E77,
	0x1E78,
	0x1E79,
	0x1E7A,
	0x1E7B,
	0x1E7C,
	0x1E7D,
	0x1E7E,
	0x1E7F,
	0x1E80,
	0x1E81,
	0x1E82,
	0x1E83,
	0x1E84,
	0x1E85,
	0x1E86,
	0x1E87,
	0x1E88,
	0x1E89,
	0x1E8A,
	0x1E8B,
	0x1E8C,
	0x1E8D,
	0x1E8E,
	0x1E8F,
	0x1E90,
	0x1E91,
	0x1E92,
	0x1E93,
	0x1E94,
	0x1E95,
	0x1E9E,
	0x1E9F,
	0x1EA0,
	0x1EA1,
	0x1EA2,
	0x1EA3,
	0x1EA4,
	0x1EA5,
	0x1EA6,
	0x1EA7,
	0x1EA8,
	0x1EA9,
	0x1EAA,
	0x1EAB,
	0x1EAC,
	0x1EAD,
	0x1EAE,
	0x1EAF,
	0x1EB0,
	0x1EB1,
	0x1EB2,
	0x1EB3,
	0x1EB4,
	0x1EB5,
	0x1EB6,
	0x1EB7,
	0x1EB8,
	0x1EB9,
	0x1EBA,
	0x1EBB,
	0x1EBC,
	0x1EBD,
	0x1EBE,
	0x1EBF,
	0x1EC0,
	0x1EC1,
	0x1EC2,
	0x1EC3,
	0x1EC4,
	0x1EC5,
	0x1EC6,
	0x1EC7,
	0x1EC8,
	0x1EC9,
	0x1ECA,
	0x1ECB,
	0x1ECC,
	0x1ECD,
	0x1ECE,
	0x1ECF,
	0x1ED0,
	0x1ED1,
	0x1ED2,
	0x1ED3,
	0x1ED4,
	0x1ED5,
	0x1ED6,
	0x1ED7,
	0x1ED8,
	0x1ED9,
	0x1EDA,
	0x1EDB,
	0x1EDC,
	0x1EDD,
	0x1EDE,
	0x1EDF,
	0x1EE0,
	0x1EE1,
	0x1EE2,
	0x1EE3,
	0x1EE4,
	0x1EE5,
	0x1EE6,
	0x1EE7,
	0x1EE8,
	0x1EE9,
	0x1EEA,
	0x1EEB,
	0x1EEC,
	0x1EED,
	0x1EEE,
	0x1EEF,
	0x1EF0,
	0x1EF1,
	0x1EF2,
	0x1EF3,
	0x1EF4,
	0x1EF5,
	0x1EF6,
	0x1EF7,
	0x1EF8,
	0x1EF9,
	0x1EFA,
	0x1EFB,
	0x1EFC,
	0x1EFD,
	0x1EFE,
	0x1EFF,
	0x1F08,
	0x1F10,
	0x1F16,
	0x1F20,
	0x1F28,
	0x1F30,
	0x1F38,
	0x1F40,
	0x1F46,
	0x1F50,
	0x1F58,
	0x1F60,
	0x1F68,
	0x1F70,
	0x1F7E,
	0x1F80,
	0x1F88,
	0x1F90,
	0x1F98,
	0x1FA0,
	0x1FA8,
	0x1FB0,
	0x1FB5,
	0x1FB6,
	0x1FB8,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FC8,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FD8,
	0x1FE0,
	0x1FE8,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FF8,
	0x2071,
	0x2072,
	0x207F,
	0x2080,
	0x2090,
	0x209D,
	0x210A,
	0x210B,
	0x210E,
	0x2110,
	0x2113,
	0x2114,
	0x212F,
	0x2130,
	0x2134,
	0x2135,
	0x2139,
	0x213A,
	0x213C,
	0x213E,
	0x2146,
	0x214A,
	0x214E,
	0x214F,
	0x2170,
	0x2180,
	0x2184,
	0x2185,
	0x24D0,
	0x24EA,
	0x2C30,
	0x2C5F,
	0x2C61,
	0x2C62,
	0x2C65,
	0x2C67,
	0x2C68,
	0x2C69,
	0x2C6A,
	0x2C6B,
	0x2C6C,
	0x2C6D,
	0x2C71,
	0x2C72,
	0x2C73,
	0x2C75,
	0x2C76,
	0x2C7E,
	0x2C81,
	0x2C82,
	0x2C83,
	0x2C84,
	0x2C85,
	0x2C86,
	0x2C87,
	0x2C88,
	0x2C89,
	0x2C8A,
	0x2C8B,
	0x2C8C,
	0x2C8D,
	0x2C8E,
	0x2C8F,
	0x2C90,
	0x2C91,
	0x2C92,
	0x2C93,
	0x2C94,
	0x2C95,
	0x2C96,
	0x2C97,
	0x2C98,
	0x2C99,
	0x2C9A,
	0x2C9B,
	0x2C9C,
	0x2C9D,
	0x2C9E,
	0x2C9F,
	0x2CA0,
	0x2CA1,
	0x2CA2,
	0x2CA3,
	0x2CA4,
	0x2CA5,
	0x2CA6,
	0x2CA7,
	0x2CA8,
	0x2CA9,
	0x2CAA,
	0x2CAB,
	0x2CAC,
	0x2CAD,
	0x2CAE,
	0x2CAF,
	0x2CB0,
	0x2CB1,
	0x2CB2,
	0x2CB3,
	0x2CB4,
	0x2CB5,
	0x2CB6,
	0x2CB7,
	0x2CB8,
	0x2CB9,
	0x2CBA,
	0x2CBB,
	0x2CBC,
	0x2CBD,
	0x2CBE,
	0x2CBF,
	0x2CC0,
	0x2CC1,
	0x2CC2,
	0x2CC3,
	0x2CC4,
	0x2CC5,
	0x2CC6,
	0x2CC7,
	0x2CC8,
	0x2CC9,
	0x2CCA,
	0x2CCB,
	0x2CCC,
	0x2CCD,
	0x2CCE,
	0x2CCF,
	0x2CD0,
	0x2CD1,
	0x2CD2,
	0x2CD3,
	0x2CD4,
	0x2CD5,
	0x2CD6,
	0x2CD7,
	0x2CD8,
	0x2CD9,
	0x2CDA,
	0x2CDB,
	0x2CDC,
	0x2CDD,
	0x2CDE,
	0x2CDF,
	0x2CE0,
	0x2CE1,
	0x2CE2,
	0x2CE3,
	0x2CE5,
	0x2CEC,
	0x2CED,
	0x2CEE,
	0x2CEF,
	0x2CF3,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0xA641,
	0xA642,
	0xA643,
	0xA644,
	0xA645,
	0xA646,
	0xA647,
	0xA648,
	0xA649,
	0xA64A,
	0xA64B,
	0xA64C,
	0xA64D,
	0xA64E,
	0xA64F,
	0xA650,
	0xA651,
	0xA652,
	0xA653,
	0xA654,
	0xA655,
	0xA656,
	0xA657,
	0xA658,
	0xA659,
	0xA65A,
	0xA65B,
	0xA65C,
	0xA65D,
	0xA65E,
	0xA65F,
	0xA660,
	0xA661,
	0xA662,
	0xA663,
	0xA664,
	0xA665,
	0xA666,
	0xA667,
	0xA668,
	0xA669,
	0xA66A,
	0xA66B,
	0xA66C,
	0xA66D,
	0xA66E,
	0xA681,
	0xA682,
	0xA683,
	0xA684,
	0xA685,
	0xA686,
	0xA687,
	0xA688,
	0xA689,
	0xA68A,
	0xA68B,
	0xA68C,
	0xA68D,
	0xA68E,
	0xA68F,
	0xA690,
	0xA691,
	0xA692,
	0xA693,
	0xA694,
	0xA695,
	0xA696,
	0xA697,
	0xA698,
	0xA699,
	0xA69A,
	0xA69B,
	0xA69E,
	0xA723,
	0xA724,
	0xA725,
	0xA726,
	0xA727,
	0xA728,
	0xA729,
	0xA72A,
	0xA72B,
	0xA72C,
	0xA72D,
	0xA72E,
	0xA72F,
	0xA732,
	0xA733,
	0xA734,
	0xA735,
	0xA736,
	0xA737,
	0xA738,
	0xA739,
	0xA73A,
	0xA73B,
	0xA73C,
	0xA73D,
	0xA73E,
	0xA73F,
	0xA740,
	0xA741,
	0xA742,
	0xA743,
	0xA744,
	0xA745,
	0xA746,
	0xA747,
	0xA748,
	0xA749,
	0xA74A,
	0xA74B,
	0xA74C,
	0xA74D,
	0xA74E,
	0xA74F,
	0xA750,
	0xA751,
	0xA752,
	0xA753,
	0xA754,
	0xA755,
	0xA756,
	0xA757,
	0xA758,
	0xA759,
	0xA75A,
	0xA75B,
	0xA75C,
	0xA75D,
	0xA75E,
	0xA75F,
	0xA760,
	0xA761,
	0xA762,
	0xA763,
	0xA764,
	0xA765,
	0xA766,
	0xA767,
	0xA768,
	0xA769,
	0xA76A,
	0xA76B,
	0xA76C,
	0xA76D,
	0xA76E,
	0xA76F,
	0xA779,
	0xA77A,
	0xA77B,
	0xA77C,
	0xA77D,
	0xA77F,
	0xA780,
	0xA781,
	0xA782,
	0xA783,
	0xA784,
	0xA785,
	0xA786,
	0xA787,
	0xA788,
	0xA78C,
	0xA78D,
	0xA78E,
	0xA78F,
	0xA791,
	0xA792,
	0xA793,
	0xA796,
	0xA797,
	0xA798,
	0xA799,
	0xA79A,
	0xA79B,
	0xA79C,
	0xA79D,
	0xA79E,
	0xA79F,
	0xA7A0,
	0xA7A1,
	0xA7A2,
	0xA7A3,
	0xA7A4,
	0xA7A5,
	0xA7A6,
	0xA7A7,
	0xA7A8,
	0xA7A9,
	0xA7AA,
	0xA7B5,
	0xA7B6,
	0xA7B7,
	0xA7B8,
	0xA7F8,
	0xA7FB,
	0xAB30,
	0xAB5B,
	0xAB5C,
	0xAB66,
	0xAB70,
	0xABC0,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFF41,
	0xFF5B,
	0x10428,
	0x10450,
	0x104D8,
	0x104FC,
	0x10CC0,
	0x10CF3,
	0x118C0,
	0x118E0,
	0x1D41A,
	0x1D434,
	0x1D44E,
	0x1D455,
	0x1D456,
	0x1D468,
	0x1D482,
	0x1D49C,
	0x1D4B6,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D4D0,
	0x1D4EA,
	0x1D504,
	0x1D51E,
	0x1D538,
	0x1D552,
	0x1D56C,
	0x1D586,
	0x1D5A0,
	0x1D5BA,
	0x1D5D4,
	0x1D5EE,
	0x1D608,
	0x1D622,
	0x1D63C,
	0x1D656,
	0x1D670,
	0x1D68A,
	0x1D6A6,
	0x1D6C2,
	0x1D6DB,
	0x1D6DC,
	0x1D6E2,
	0x1D6FC,
	0x1D715,
	0x1D716,
	0x1D71C,
	0x1D736,
	0x1D74F,
	0x1D750,
	0x1D756,
	0x1D770,
	0x1D789,
	0x1D78A,
	0x1D790,
	0x1D7AA,
	0x1D7C3,
	0x1D7C4,
	0x1D7CA,
	0x1D7CB,
	0x1D7CC,
	0x1E922,
	0x1E944
};

static const UV XPosixPrint_invlist[] = { /* for ASCII/Latin1 */
	1281,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x20,
	0x7F,
	0xA0,
	0x378,
	0x37A,
	0x380,
	0x384,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x530,
	0x531,
	0x557,
	0x559,
	0x560,
	0x561,
	0x588,
	0x589,
	0x58B,
	0x58D,
	0x590,
	0x591,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F5,
	0x600,
	0x61D,
	0x61E,
	0x70E,
	0x70F,
	0x74B,
	0x74D,
	0x7B2,
	0x7C0,
	0x7FB,
	0x800,
	0x82E,
	0x830,
	0x83F,
	0x840,
	0x85C,
	0x85E,
	0x85F,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x8D4,
	0x984,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BC,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CF,
	0x9D7,
	0x9D8,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E4,
	0x9E6,
	0x9FC,
	0xA01,
	0xA04,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA3C,
	0xA3D,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4E,
	0xA51,
	0xA52,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA66,
	0xA76,
	0xA81,
	0xA84,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABC,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACE,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE4,
	0xAE6,
	0xAF2,
	0xAF9,
	0xAFA,
	0xB01,
	0xB04,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3C,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4E,
	0xB56,
	0xB58,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB64,
	0xB66,
	0xB78,
	0xB82,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCE,
	0xBD0,
	0xBD1,
	0xBD7,
	0xBD8,
	0xBE6,
	0xBFB,
	0xC00,
	0xC04,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4E,
	0xC55,
	0xC57,
	0xC58,
	0xC5B,
	0xC60,
	0xC64,
	0xC66,
	0xC70,
	0xC78,
	0xC84,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBC,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCE,
	0xCD5,
	0xCD7,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE4,
	0xCE6,
	0xCF0,
	0xCF1,
	0xCF3,
	0xD01,
	0xD04,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD50,
	0xD54,
	0xD64,
	0xD66,
	0xD80,
	0xD82,
	0xD84,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xDCA,
	0xDCB,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDE6,
	0xDF0,
	0xDF2,
	0xDF5,
	0xE01,
	0xE3B,
	0xE3F,
	0xE5C,
	0xE81,
	0xE83,
	0xE84,
	0xE85,
	0xE87,
	0xE89,
	0xE8A,
	0xE8B,
	0xE8D,
	0xE8E,
	0xE94,
	0xE98,
	0xE99,
	0xEA0,
	0xEA1,
	0xEA4,
	0xEA5,
	0xEA6,
	0xEA7,
	0xEA8,
	0xEAA,
	0xEAC,
	0xEAD,
	0xEBA,
	0xEBB,
	0xEBE,
	0xEC0,
	0xEC5,
	0xEC6,
	0xEC7,
	0xEC8,
	0xECE,
	0xED0,
	0xEDA,
	0xEDC,
	0xEE0,
	0xF00,
	0xF48,
	0xF49,
	0xF6D,
	0xF71,
	0xF98,
	0xF99,
	0xFBD,
	0xFBE,
	0xFCD,
	0xFCE,
	0xFDB,
	0x1000,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x135D,
	0x137D,
	0x1380,
	0x139A,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1400,
	0x169D,
	0x16A0,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1715,
	0x1720,
	0x1737,
	0x1740,
	0x1754,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1772,
	0x1774,
	0x1780,
	0x17DE,
	0x17E0,
	0x17EA,
	0x17F0,
	0x17FA,
	0x1800,
	0x180F,
	0x1810,
	0x181A,
	0x1820,
	0x1878,
	0x1880,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1920,
	0x192C,
	0x1930,
	0x193C,
	0x1940,
	0x1941,
	0x1944,
	0x196E,
	0x1970,
	0x1975,
	0x1980,
	0x19AC,
	0x19B0,
	0x19CA,
	0x19D0,
	0x19DB,
	0x19DE,
	0x1A1C,
	0x1A1E,
	0x1A5F,
	0x1A60,
	0x1A7D,
	0x1A7F,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1AA0,
	0x1AAE,
	0x1AB0,
	0x1ABF,
	0x1B00,
	0x1B4C,
	0x1B50,
	0x1B7D,
	0x1B80,
	0x1BF4,
	0x1BFC,
	0x1C38,
	0x1C3B,
	0x1C4A,
	0x1C4D,
	0x1C89,
	0x1CC0,
	0x1CC8,
	0x1CD0,
	0x1CF7,
	0x1CF8,
	0x1CFA,
	0x1D00,
	0x1DF6,
	0x1DFB,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FC5,
	0x1FC6,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FDD,
	0x1FF0,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFF,
	0x2000,
	0x2028,
	0x202A,
	0x2065,
	0x2066,
	0x2072,
	0x2074,
	0x208F,
	0x2090,
	0x209D,
	0x20A0,
	0x20BF,
	0x20D0,
	0x20F1,
	0x2100,
	0x218C,
	0x2190,
	0x23FF,
	0x2400,
	0x2427,
	0x2440,
	0x244B,
	0x2460,
	0x2B74,
	0x2B76,
	0x2B96,
	0x2B98,
	0x2BBA,
	0x2BBD,
	0x2BC9,
	0x2BCA,
	0x2BD2,
	0x2BEC,
	0x2BF0,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CF4,
	0x2CF9,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D71,
	0x2D7F,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x2DE0,
	0x2E45,
	0x2E80,
	0x2E9A,
	0x2E9B,
	0x2EF4,
	0x2F00,
	0x2FD6,
	0x2FF0,
	0x2FFC,
	0x3000,
	0x3040,
	0x3041,
	0x3097,
	0x3099,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x3190,
	0x31BB,
	0x31C0,
	0x31E4,
	0x31F0,
	0x321F,
	0x3220,
	0x32FF,
	0x3300,
	0x4DB6,
	0x4DC0,
	0x9FD6,
	0xA000,
	0xA48D,
	0xA490,
	0xA4C7,
	0xA4D0,
	0xA62C,
	0xA640,
	0xA6F8,
	0xA700,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F7,
	0xA82C,
	0xA830,
	0xA83A,
	0xA840,
	0xA878,
	0xA880,
	0xA8C6,
	0xA8CE,
	0xA8DA,
	0xA8E0,
	0xA8FE,
	0xA900,
	0xA954,
	0xA95F,
	0xA97D,
	0xA980,
	0xA9CE,
	0xA9CF,
	0xA9DA,
	0xA9DE,
	0xA9FF,
	0xAA00,
	0xAA37,
	0xAA40,
	0xAA4E,
	0xAA50,
	0xAA5A,
	0xAA5C,
	0xAAC3,
	0xAADB,
	0xAAF7,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB66,
	0xAB70,
	0xABEE,
	0xABF0,
	0xABFA,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xE000,
	0xFA6E,
	0xFA70,
	0xFADA,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFBC2,
	0xFBD3,
	0xFD40,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFE,
	0xFE00,
	0xFE1A,
	0xFE20,
	0xFE53,
	0xFE54,
	0xFE67,
	0xFE68,
	0xFE6C,
	0xFE70,
	0xFE75,
	0xFE76,
	0xFEFD,
	0xFEFF,
	0xFF00,
	0xFF01,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0xFFE0,
	0xFFE7,
	0xFFE8,
	0xFFEF,
	0xFFF9,
	0xFFFE,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10100,
	0x10103,
	0x10107,
	0x10134,
	0x10137,
	0x1018F,
	0x10190,
	0x1019C,
	0x101A0,
	0x101A1,
	0x101D0,
	0x101FE,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x102E0,
	0x102FC,
	0x10300,
	0x10324,
	0x10330,
	0x1034B,
	0x10350,
	0x1037B,
	0x10380,
	0x1039E,
	0x1039F,
	0x103C4,
	0x103C8,
	0x103D6,
	0x10400,
	0x1049E,
	0x104A0,
	0x104AA,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x1056F,
	0x10570,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10857,
	0x1089F,
	0x108A7,
	0x108B0,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x108FB,
	0x1091C,
	0x1091F,
	0x1093A,
	0x1093F,
	0x10940,
	0x10980,
	0x109B8,
	0x109BC,
	0x109D0,
	0x109D2,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A38,
	0x10A3B,
	0x10A3F,
	0x10A48,
	0x10A50,
	0x10A59,
	0x10A60,
	0x10AA0,
	0x10AC0,
	0x10AE7,
	0x10AEB,
	0x10AF7,
	0x10B00,
	0x10B36,
	0x10B39,
	0x10B56,
	0x10B58,
	0x10B73,
	0x10B78,
	0x10B92,
	0x10B99,
	0x10B9D,
	0x10BA9,
	0x10BB0,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x10CFA,
	0x10D00,
	0x10E60,
	0x10E7F,
	0x11000,
	0x1104E,
	0x11052,
	0x11070,
	0x1107F,
	0x110C2,
	0x110D0,
	0x110E9,
	0x110F0,
	0x110FA,
	0x11100,
	0x11135,
	0x11136,
	0x11144,
	0x11150,
	0x11177,
	0x11180,
	0x111CE,
	0x111D0,
	0x111E0,
	0x111E1,
	0x111F5,
	0x11200,
	0x11212,
	0x11213,
	0x1123F,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112AA,
	0x112B0,
	0x112EB,
	0x112F0,
	0x112FA,
	0x11300,
	0x11304,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133C,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134E,
	0x11350,
	0x11351,
	0x11357,
	0x11358,
	0x1135D,
	0x11364,
	0x11366,
	0x1136D,
	0x11370,
	0x11375,
	0x11400,
	0x1145A,
	0x1145B,
	0x1145C,
	0x1145D,
	0x1145E,
	0x11480,
	0x114C8,
	0x114D0,
	0x114DA,
	0x11580,
	0x115B6,
	0x115B8,
	0x115DE,
	0x11600,
	0x11645,
	0x11650,
	0x1165A,
	0x11660,
	0x1166D,
	0x11680,
	0x116B8,
	0x116C0,
	0x116CA,
	0x11700,
	0x1171A,
	0x1171D,
	0x1172C,
	0x11730,
	0x11740,
	0x118A0,
	0x118F3,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C37,
	0x11C38,
	0x11C46,
	0x11C50,
	0x11C6D,
	0x11C70,
	0x11C90,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12470,
	0x12475,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16A60,
	0x16A6A,
	0x16A6E,
	0x16A70,
	0x16AD0,
	0x16AEE,
	0x16AF0,
	0x16AF6,
	0x16B00,
	0x16B46,
	0x16B50,
	0x16B5A,
	0x16B5B,
	0x16B62,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F7F,
	0x16F8F,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1BC9C,
	0x1BCA4,
	0x1D000,
	0x1D0F6,
	0x1D100,
	0x1D127,
	0x1D129,
	0x1D1E9,
	0x1D200,
	0x1D246,
	0x1D300,
	0x1D357,
	0x1D360,
	0x1D372,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D7CC,
	0x1D7CE,
	0x1DA8C,
	0x1DA9B,
	0x1DAA0,
	0x1DAA1,
	0x1DAB0,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E800,
	0x1E8C5,
	0x1E8C7,
	0x1E8D7,
	0x1E900,
	0x1E94B,
	0x1E950,
	0x1E95A,
	0x1E95E,
	0x1E960,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x1EEF0,
	0x1EEF2,
	0x1F000,
	0x1F02C,
	0x1F030,
	0x1F094,
	0x1F0A0,
	0x1F0AF,
	0x1F0B1,
	0x1F0C0,
	0x1F0C1,
	0x1F0D0,
	0x1F0D1,
	0x1F0F6,
	0x1F100,
	0x1F10D,
	0x1F110,
	0x1F12F,
	0x1F130,
	0x1F16C,
	0x1F170,
	0x1F1AD,
	0x1F1E6,
	0x1F203,
	0x1F210,
	0x1F23C,
	0x1F240,
	0x1F249,
	0x1F250,
	0x1F252,
	0x1F300,
	0x1F6D3,
	0x1F6E0,
	0x1F6ED,
	0x1F6F0,
	0x1F6F7,
	0x1F700,
	0x1F774,
	0x1F780,
	0x1F7D5,
	0x1F800,
	0x1F80C,
	0x1F810,
	0x1F848,
	0x1F850,
	0x1F85A,
	0x1F860,
	0x1F888,
	0x1F890,
	0x1F8AE,
	0x1F910,
	0x1F91F,
	0x1F920,
	0x1F928,
	0x1F930,
	0x1F931,
	0x1F933,
	0x1F93F,
	0x1F940,
	0x1F94C,
	0x1F950,
	0x1F95F,
	0x1F980,
	0x1F992,
	0x1F9C0,
	0x1F9C1,
	0x20000,
	0x2A6D7,
	0x2A700,
	0x2B735,
	0x2B740,
	0x2B81E,
	0x2B820,
	0x2CEA2,
	0x2F800,
	0x2FA1E,
	0xE0001,
	0xE0002,
	0xE0020,
	0xE0080,
	0xE0100,
	0xE01F0,
	0xF0000,
	0xFFFFE,
	0x100000,
	0x10FFFE
};

static const UV XPosixPunct_invlist[] = { /* for ASCII/Latin1 */
	327,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x21,
	0x30,
	0x3A,
	0x41,
	0x5B,
	0x61,
	0x7B,
	0x7F,
	0xA1,
	0xA2,
	0xA7,
	0xA8,
	0xAB,
	0xAC,
	0xB6,
	0xB8,
	0xBB,
	0xBC,
	0xBF,
	0xC0,
	0x37E,
	0x37F,
	0x387,
	0x388,
	0x55A,
	0x560,
	0x589,
	0x58B,
	0x5BE,
	0x5BF,
	0x5C0,
	0x5C1,
	0x5C3,
	0x5C4,
	0x5C6,
	0x5C7,
	0x5F3,
	0x5F5,
	0x609,
	0x60B,
	0x60C,
	0x60E,
	0x61B,
	0x61C,
	0x61E,
	0x620,
	0x66A,
	0x66E,
	0x6D4,
	0x6D5,
	0x700,
	0x70E,
	0x7F7,
	0x7FA,
	0x830,
	0x83F,
	0x85E,
	0x85F,
	0x964,
	0x966,
	0x970,
	0x971,
	0xAF0,
	0xAF1,
	0xDF4,
	0xDF5,
	0xE4F,
	0xE50,
	0xE5A,
	0xE5C,
	0xF04,
	0xF13,
	0xF14,
	0xF15,
	0xF3A,
	0xF3E,
	0xF85,
	0xF86,
	0xFD0,
	0xFD5,
	0xFD9,
	0xFDB,
	0x104A,
	0x1050,
	0x10FB,
	0x10FC,
	0x1360,
	0x1369,
	0x1400,
	0x1401,
	0x166D,
	0x166F,
	0x169B,
	0x169D,
	0x16EB,
	0x16EE,
	0x1735,
	0x1737,
	0x17D4,
	0x17D7,
	0x17D8,
	0x17DB,
	0x1800,
	0x180B,
	0x1944,
	0x1946,
	0x1A1E,
	0x1A20,
	0x1AA0,
	0x1AA7,
	0x1AA8,
	0x1AAE,
	0x1B5A,
	0x1B61,
	0x1BFC,
	0x1C00,
	0x1C3B,
	0x1C40,
	0x1C7E,
	0x1C80,
	0x1CC0,
	0x1CC8,
	0x1CD3,
	0x1CD4,
	0x2010,
	0x2028,
	0x2030,
	0x2044,
	0x2045,
	0x2052,
	0x2053,
	0x205F,
	0x207D,
	0x207F,
	0x208D,
	0x208F,
	0x2308,
	0x230C,
	0x2329,
	0x232B,
	0x2768,
	0x2776,
	0x27C5,
	0x27C7,
	0x27E6,
	0x27F0,
	0x2983,
	0x2999,
	0x29D8,
	0x29DC,
	0x29FC,
	0x29FE,
	0x2CF9,
	0x2CFD,
	0x2CFE,
	0x2D00,
	0x2D70,
	0x2D71,
	0x2E00,
	0x2E2F,
	0x2E30,
	0x2E45,
	0x3001,
	0x3004,
	0x3008,
	0x3012,
	0x3014,
	0x3020,
	0x3030,
	0x3031,
	0x303D,
	0x303E,
	0x30A0,
	0x30A1,
	0x30FB,
	0x30FC,
	0xA4FE,
	0xA500,
	0xA60D,
	0xA610,
	0xA673,
	0xA674,
	0xA67E,
	0xA67F,
	0xA6F2,
	0xA6F8,
	0xA874,
	0xA878,
	0xA8CE,
	0xA8D0,
	0xA8F8,
	0xA8FB,
	0xA8FC,
	0xA8FD,
	0xA92E,
	0xA930,
	0xA95F,
	0xA960,
	0xA9C1,
	0xA9CE,
	0xA9DE,
	0xA9E0,
	0xAA5C,
	0xAA60,
	0xAADE,
	0xAAE0,
	0xAAF0,
	0xAAF2,
	0xABEB,
	0xABEC,
	0xFD3E,
	0xFD40,
	0xFE10,
	0xFE1A,
	0xFE30,
	0xFE53,
	0xFE54,
	0xFE62,
	0xFE63,
	0xFE64,
	0xFE68,
	0xFE69,
	0xFE6A,
	0xFE6C,
	0xFF01,
	0xFF04,
	0xFF05,
	0xFF0B,
	0xFF0C,
	0xFF10,
	0xFF1A,
	0xFF1C,
	0xFF1F,
	0xFF21,
	0xFF3B,
	0xFF3E,
	0xFF3F,
	0xFF40,
	0xFF5B,
	0xFF5C,
	0xFF5D,
	0xFF5E,
	0xFF5F,
	0xFF66,
	0x10100,
	0x10103,
	0x1039F,
	0x103A0,
	0x103D0,
	0x103D1,
	0x1056F,
	0x10570,
	0x10857,
	0x10858,
	0x1091F,
	0x10920,
	0x1093F,
	0x10940,
	0x10A50,
	0x10A59,
	0x10A7F,
	0x10A80,
	0x10AF0,
	0x10AF7,
	0x10B39,
	0x10B40,
	0x10B99,
	0x10B9D,
	0x11047,
	0x1104E,
	0x110BB,
	0x110BD,
	0x110BE,
	0x110C2,
	0x11140,
	0x11144,
	0x11174,
	0x11176,
	0x111C5,
	0x111CA,
	0x111CD,
	0x111CE,
	0x111DB,
	0x111DC,
	0x111DD,
	0x111E0,
	0x11238,
	0x1123E,
	0x112A9,
	0x112AA,
	0x1144B,
	0x11450,
	0x1145B,
	0x1145C,
	0x1145D,
	0x1145E,
	0x114C6,
	0x114C7,
	0x115C1,
	0x115D8,
	0x11641,
	0x11644,
	0x11660,
	0x1166D,
	0x1173C,
	0x1173F,
	0x11C41,
	0x11C46,
	0x11C70,
	0x11C72,
	0x12470,
	0x12475,
	0x16A6E,
	0x16A70,
	0x16AF5,
	0x16AF6,
	0x16B37,
	0x16B3C,
	0x16B44,
	0x16B45,
	0x1BC9F,
	0x1BCA0,
	0x1DA87,
	0x1DA8C,
	0x1E95E,
	0x1E960
};

static const UV XPosixSpace_invlist[] = { /* for ASCII/Latin1 */
	21,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x9,
	0xE,
	0x20,
	0x21,
	0x85,
	0x86,
	0xA0,
	0xA1,
	0x1680,
	0x1681,
	0x2000,
	0x200B,
	0x2028,
	0x202A,
	0x202F,
	0x2030,
	0x205F,
	0x2060,
	0x3000,
	0x3001
};

static const UV XPosixUpper_invlist[] = { /* for ASCII/Latin1 */
	1265,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x41,
	0x5B,
	0xC0,
	0xD7,
	0xD8,
	0xDF,
	0x100,
	0x101,
	0x102,
	0x103,
	0x104,
	0x105,
	0x106,
	0x107,
	0x108,
	0x109,
	0x10A,
	0x10B,
	0x10C,
	0x10D,
	0x10E,
	0x10F,
	0x110,
	0x111,
	0x112,
	0x113,
	0x114,
	0x115,
	0x116,
	0x117,
	0x118,
	0x119,
	0x11A,
	0x11B,
	0x11C,
	0x11D,
	0x11E,
	0x11F,
	0x120,
	0x121,
	0x122,
	0x123,
	0x124,
	0x125,
	0x126,
	0x127,
	0x128,
	0x129,
	0x12A,
	0x12B,
	0x12C,
	0x12D,
	0x12E,
	0x12F,
	0x130,
	0x131,
	0x132,
	0x133,
	0x134,
	0x135,
	0x136,
	0x137,
	0x139,
	0x13A,
	0x13B,
	0x13C,
	0x13D,
	0x13E,
	0x13F,
	0x140,
	0x141,
	0x142,
	0x143,
	0x144,
	0x145,
	0x146,
	0x147,
	0x148,
	0x14A,
	0x14B,
	0x14C,
	0x14D,
	0x14E,
	0x14F,
	0x150,
	0x151,
	0x152,
	0x153,
	0x154,
	0x155,
	0x156,
	0x157,
	0x158,
	0x159,
	0x15A,
	0x15B,
	0x15C,
	0x15D,
	0x15E,
	0x15F,
	0x160,
	0x161,
	0x162,
	0x163,
	0x164,
	0x165,
	0x166,
	0x167,
	0x168,
	0x169,
	0x16A,
	0x16B,
	0x16C,
	0x16D,
	0x16E,
	0x16F,
	0x170,
	0x171,
	0x172,
	0x173,
	0x174,
	0x175,
	0x176,
	0x177,
	0x178,
	0x17A,
	0x17B,
	0x17C,
	0x17D,
	0x17E,
	0x181,
	0x183,
	0x184,
	0x185,
	0x186,
	0x188,
	0x189,
	0x18C,
	0x18E,
	0x192,
	0x193,
	0x195,
	0x196,
	0x199,
	0x19C,
	0x19E,
	0x19F,
	0x1A1,
	0x1A2,
	0x1A3,
	0x1A4,
	0x1A5,
	0x1A6,
	0x1A8,
	0x1A9,
	0x1AA,
	0x1AC,
	0x1AD,
	0x1AE,
	0x1B0,
	0x1B1,
	0x1B4,
	0x1B5,
	0x1B6,
	0x1B7,
	0x1B9,
	0x1BC,
	0x1BD,
	0x1C4,
	0x1C5,
	0x1C7,
	0x1C8,
	0x1CA,
	0x1CB,
	0x1CD,
	0x1CE,
	0x1CF,
	0x1D0,
	0x1D1,
	0x1D2,
	0x1D3,
	0x1D4,
	0x1D5,
	0x1D6,
	0x1D7,
	0x1D8,
	0x1D9,
	0x1DA,
	0x1DB,
	0x1DC,
	0x1DE,
	0x1DF,
	0x1E0,
	0x1E1,
	0x1E2,
	0x1E3,
	0x1E4,
	0x1E5,
	0x1E6,
	0x1E7,
	0x1E8,
	0x1E9,
	0x1EA,
	0x1EB,
	0x1EC,
	0x1ED,
	0x1EE,
	0x1EF,
	0x1F1,
	0x1F2,
	0x1F4,
	0x1F5,
	0x1F6,
	0x1F9,
	0x1FA,
	0x1FB,
	0x1FC,
	0x1FD,
	0x1FE,
	0x1FF,
	0x200,
	0x201,
	0x202,
	0x203,
	0x204,
	0x205,
	0x206,
	0x207,
	0x208,
	0x209,
	0x20A,
	0x20B,
	0x20C,
	0x20D,
	0x20E,
	0x20F,
	0x210,
	0x211,
	0x212,
	0x213,
	0x214,
	0x215,
	0x216,
	0x217,
	0x218,
	0x219,
	0x21A,
	0x21B,
	0x21C,
	0x21D,
	0x21E,
	0x21F,
	0x220,
	0x221,
	0x222,
	0x223,
	0x224,
	0x225,
	0x226,
	0x227,
	0x228,
	0x229,
	0x22A,
	0x22B,
	0x22C,
	0x22D,
	0x22E,
	0x22F,
	0x230,
	0x231,
	0x232,
	0x233,
	0x23A,
	0x23C,
	0x23D,
	0x23F,
	0x241,
	0x242,
	0x243,
	0x247,
	0x248,
	0x249,
	0x24A,
	0x24B,
	0x24C,
	0x24D,
	0x24E,
	0x24F,
	0x370,
	0x371,
	0x372,
	0x373,
	0x376,
	0x377,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x390,
	0x391,
	0x3A2,
	0x3A3,
	0x3AC,
	0x3CF,
	0x3D0,
	0x3D2,
	0x3D5,
	0x3D8,
	0x3D9,
	0x3DA,
	0x3DB,
	0x3DC,
	0x3DD,
	0x3DE,
	0x3DF,
	0x3E0,
	0x3E1,
	0x3E2,
	0x3E3,
	0x3E4,
	0x3E5,
	0x3E6,
	0x3E7,
	0x3E8,
	0x3E9,
	0x3EA,
	0x3EB,
	0x3EC,
	0x3ED,
	0x3EE,
	0x3EF,
	0x3F4,
	0x3F5,
	0x3F7,
	0x3F8,
	0x3F9,
	0x3FB,
	0x3FD,
	0x430,
	0x460,
	0x461,
	0x462,
	0x463,
	0x464,
	0x465,
	0x466,
	0x467,
	0x468,
	0x469,
	0x46A,
	0x46B,
	0x46C,
	0x46D,
	0x46E,
	0x46F,
	0x470,
	0x471,
	0x472,
	0x473,
	0x474,
	0x475,
	0x476,
	0x477,
	0x478,
	0x479,
	0x47A,
	0x47B,
	0x47C,
	0x47D,
	0x47E,
	0x47F,
	0x480,
	0x481,
	0x48A,
	0x48B,
	0x48C,
	0x48D,
	0x48E,
	0x48F,
	0x490,
	0x491,
	0x492,
	0x493,
	0x494,
	0x495,
	0x496,
	0x497,
	0x498,
	0x499,
	0x49A,
	0x49B,
	0x49C,
	0x49D,
	0x49E,
	0x49F,
	0x4A0,
	0x4A1,
	0x4A2,
	0x4A3,
	0x4A4,
	0x4A5,
	0x4A6,
	0x4A7,
	0x4A8,
	0x4A9,
	0x4AA,
	0x4AB,
	0x4AC,
	0x4AD,
	0x4AE,
	0x4AF,
	0x4B0,
	0x4B1,
	0x4B2,
	0x4B3,
	0x4B4,
	0x4B5,
	0x4B6,
	0x4B7,
	0x4B8,
	0x4B9,
	0x4BA,
	0x4BB,
	0x4BC,
	0x4BD,
	0x4BE,
	0x4BF,
	0x4C0,
	0x4C2,
	0x4C3,
	0x4C4,
	0x4C5,
	0x4C6,
	0x4C7,
	0x4C8,
	0x4C9,
	0x4CA,
	0x4CB,
	0x4CC,
	0x4CD,
	0x4CE,
	0x4D0,
	0x4D1,
	0x4D2,
	0x4D3,
	0x4D4,
	0x4D5,
	0x4D6,
	0x4D7,
	0x4D8,
	0x4D9,
	0x4DA,
	0x4DB,
	0x4DC,
	0x4DD,
	0x4DE,
	0x4DF,
	0x4E0,
	0x4E1,
	0x4E2,
	0x4E3,
	0x4E4,
	0x4E5,
	0x4E6,
	0x4E7,
	0x4E8,
	0x4E9,
	0x4EA,
	0x4EB,
	0x4EC,
	0x4ED,
	0x4EE,
	0x4EF,
	0x4F0,
	0x4F1,
	0x4F2,
	0x4F3,
	0x4F4,
	0x4F5,
	0x4F6,
	0x4F7,
	0x4F8,
	0x4F9,
	0x4FA,
	0x4FB,
	0x4FC,
	0x4FD,
	0x4FE,
	0x4FF,
	0x500,
	0x501,
	0x502,
	0x503,
	0x504,
	0x505,
	0x506,
	0x507,
	0x508,
	0x509,
	0x50A,
	0x50B,
	0x50C,
	0x50D,
	0x50E,
	0x50F,
	0x510,
	0x511,
	0x512,
	0x513,
	0x514,
	0x515,
	0x516,
	0x517,
	0x518,
	0x519,
	0x51A,
	0x51B,
	0x51C,
	0x51D,
	0x51E,
	0x51F,
	0x520,
	0x521,
	0x522,
	0x523,
	0x524,
	0x525,
	0x526,
	0x527,
	0x528,
	0x529,
	0x52A,
	0x52B,
	0x52C,
	0x52D,
	0x52E,
	0x52F,
	0x531,
	0x557,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x13A0,
	0x13F6,
	0x1E00,
	0x1E01,
	0x1E02,
	0x1E03,
	0x1E04,
	0x1E05,
	0x1E06,
	0x1E07,
	0x1E08,
	0x1E09,
	0x1E0A,
	0x1E0B,
	0x1E0C,
	0x1E0D,
	0x1E0E,
	0x1E0F,
	0x1E10,
	0x1E11,
	0x1E12,
	0x1E13,
	0x1E14,
	0x1E15,
	0x1E16,
	0x1E17,
	0x1E18,
	0x1E19,
	0x1E1A,
	0x1E1B,
	0x1E1C,
	0x1E1D,
	0x1E1E,
	0x1E1F,
	0x1E20,
	0x1E21,
	0x1E22,
	0x1E23,
	0x1E24,
	0x1E25,
	0x1E26,
	0x1E27,
	0x1E28,
	0x1E29,
	0x1E2A,
	0x1E2B,
	0x1E2C,
	0x1E2D,
	0x1E2E,
	0x1E2F,
	0x1E30,
	0x1E31,
	0x1E32,
	0x1E33,
	0x1E34,
	0x1E35,
	0x1E36,
	0x1E37,
	0x1E38,
	0x1E39,
	0x1E3A,
	0x1E3B,
	0x1E3C,
	0x1E3D,
	0x1E3E,
	0x1E3F,
	0x1E40,
	0x1E41,
	0x1E42,
	0x1E43,
	0x1E44,
	0x1E45,
	0x1E46,
	0x1E47,
	0x1E48,
	0x1E49,
	0x1E4A,
	0x1E4B,
	0x1E4C,
	0x1E4D,
	0x1E4E,
	0x1E4F,
	0x1E50,
	0x1E51,
	0x1E52,
	0x1E53,
	0x1E54,
	0x1E55,
	0x1E56,
	0x1E57,
	0x1E58,
	0x1E59,
	0x1E5A,
	0x1E5B,
	0x1E5C,
	0x1E5D,
	0x1E5E,
	0x1E5F,
	0x1E60,
	0x1E61,
	0x1E62,
	0x1E63,
	0x1E64,
	0x1E65,
	0x1E66,
	0x1E67,
	0x1E68,
	0x1E69,
	0x1E6A,
	0x1E6B,
	0x1E6C,
	0x1E6D,
	0x1E6E,
	0x1E6F,
	0x1E70,
	0x1E71,
	0x1E72,
	0x1E73,
	0x1E74,
	0x1E75,
	0x1E76,
	0x1E77,
	0x1E78,
	0x1E79,
	0x1E7A,
	0x1E7B,
	0x1E7C,
	0x1E7D,
	0x1E7E,
	0x1E7F,
	0x1E80,
	0x1E81,
	0x1E82,
	0x1E83,
	0x1E84,
	0x1E85,
	0x1E86,
	0x1E87,
	0x1E88,
	0x1E89,
	0x1E8A,
	0x1E8B,
	0x1E8C,
	0x1E8D,
	0x1E8E,
	0x1E8F,
	0x1E90,
	0x1E91,
	0x1E92,
	0x1E93,
	0x1E94,
	0x1E95,
	0x1E9E,
	0x1E9F,
	0x1EA0,
	0x1EA1,
	0x1EA2,
	0x1EA3,
	0x1EA4,
	0x1EA5,
	0x1EA6,
	0x1EA7,
	0x1EA8,
	0x1EA9,
	0x1EAA,
	0x1EAB,
	0x1EAC,
	0x1EAD,
	0x1EAE,
	0x1EAF,
	0x1EB0,
	0x1EB1,
	0x1EB2,
	0x1EB3,
	0x1EB4,
	0x1EB5,
	0x1EB6,
	0x1EB7,
	0x1EB8,
	0x1EB9,
	0x1EBA,
	0x1EBB,
	0x1EBC,
	0x1EBD,
	0x1EBE,
	0x1EBF,
	0x1EC0,
	0x1EC1,
	0x1EC2,
	0x1EC3,
	0x1EC4,
	0x1EC5,
	0x1EC6,
	0x1EC7,
	0x1EC8,
	0x1EC9,
	0x1ECA,
	0x1ECB,
	0x1ECC,
	0x1ECD,
	0x1ECE,
	0x1ECF,
	0x1ED0,
	0x1ED1,
	0x1ED2,
	0x1ED3,
	0x1ED4,
	0x1ED5,
	0x1ED6,
	0x1ED7,
	0x1ED8,
	0x1ED9,
	0x1EDA,
	0x1EDB,
	0x1EDC,
	0x1EDD,
	0x1EDE,
	0x1EDF,
	0x1EE0,
	0x1EE1,
	0x1EE2,
	0x1EE3,
	0x1EE4,
	0x1EE5,
	0x1EE6,
	0x1EE7,
	0x1EE8,
	0x1EE9,
	0x1EEA,
	0x1EEB,
	0x1EEC,
	0x1EED,
	0x1EEE,
	0x1EEF,
	0x1EF0,
	0x1EF1,
	0x1EF2,
	0x1EF3,
	0x1EF4,
	0x1EF5,
	0x1EF6,
	0x1EF7,
	0x1EF8,
	0x1EF9,
	0x1EFA,
	0x1EFB,
	0x1EFC,
	0x1EFD,
	0x1EFE,
	0x1EFF,
	0x1F08,
	0x1F10,
	0x1F18,
	0x1F1E,
	0x1F28,
	0x1F30,
	0x1F38,
	0x1F40,
	0x1F48,
	0x1F4E,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F60,
	0x1F68,
	0x1F70,
	0x1FB8,
	0x1FBC,
	0x1FC8,
	0x1FCC,
	0x1FD8,
	0x1FDC,
	0x1FE8,
	0x1FED,
	0x1FF8,
	0x1FFC,
	0x2102,
	0x2103,
	0x2107,
	0x2108,
	0x210B,
	0x210E,
	0x2110,
	0x2113,
	0x2115,
	0x2116,
	0x2119,
	0x211E,
	0x2124,
	0x2125,
	0x2126,
	0x2127,
	0x2128,
	0x2129,
	0x212A,
	0x212E,
	0x2130,
	0x2134,
	0x213E,
	0x2140,
	0x2145,
	0x2146,
	0x2160,
	0x2170,
	0x2183,
	0x2184,
	0x24B6,
	0x24D0,
	0x2C00,
	0x2C2F,
	0x2C60,
	0x2C61,
	0x2C62,
	0x2C65,
	0x2C67,
	0x2C68,
	0x2C69,
	0x2C6A,
	0x2C6B,
	0x2C6C,
	0x2C6D,
	0x2C71,
	0x2C72,
	0x2C73,
	0x2C75,
	0x2C76,
	0x2C7E,
	0x2C81,
	0x2C82,
	0x2C83,
	0x2C84,
	0x2C85,
	0x2C86,
	0x2C87,
	0x2C88,
	0x2C89,
	0x2C8A,
	0x2C8B,
	0x2C8C,
	0x2C8D,
	0x2C8E,
	0x2C8F,
	0x2C90,
	0x2C91,
	0x2C92,
	0x2C93,
	0x2C94,
	0x2C95,
	0x2C96,
	0x2C97,
	0x2C98,
	0x2C99,
	0x2C9A,
	0x2C9B,
	0x2C9C,
	0x2C9D,
	0x2C9E,
	0x2C9F,
	0x2CA0,
	0x2CA1,
	0x2CA2,
	0x2CA3,
	0x2CA4,
	0x2CA5,
	0x2CA6,
	0x2CA7,
	0x2CA8,
	0x2CA9,
	0x2CAA,
	0x2CAB,
	0x2CAC,
	0x2CAD,
	0x2CAE,
	0x2CAF,
	0x2CB0,
	0x2CB1,
	0x2CB2,
	0x2CB3,
	0x2CB4,
	0x2CB5,
	0x2CB6,
	0x2CB7,
	0x2CB8,
	0x2CB9,
	0x2CBA,
	0x2CBB,
	0x2CBC,
	0x2CBD,
	0x2CBE,
	0x2CBF,
	0x2CC0,
	0x2CC1,
	0x2CC2,
	0x2CC3,
	0x2CC4,
	0x2CC5,
	0x2CC6,
	0x2CC7,
	0x2CC8,
	0x2CC9,
	0x2CCA,
	0x2CCB,
	0x2CCC,
	0x2CCD,
	0x2CCE,
	0x2CCF,
	0x2CD0,
	0x2CD1,
	0x2CD2,
	0x2CD3,
	0x2CD4,
	0x2CD5,
	0x2CD6,
	0x2CD7,
	0x2CD8,
	0x2CD9,
	0x2CDA,
	0x2CDB,
	0x2CDC,
	0x2CDD,
	0x2CDE,
	0x2CDF,
	0x2CE0,
	0x2CE1,
	0x2CE2,
	0x2CE3,
	0x2CEB,
	0x2CEC,
	0x2CED,
	0x2CEE,
	0x2CF2,
	0x2CF3,
	0xA640,
	0xA641,
	0xA642,
	0xA643,
	0xA644,
	0xA645,
	0xA646,
	0xA647,
	0xA648,
	0xA649,
	0xA64A,
	0xA64B,
	0xA64C,
	0xA64D,
	0xA64E,
	0xA64F,
	0xA650,
	0xA651,
	0xA652,
	0xA653,
	0xA654,
	0xA655,
	0xA656,
	0xA657,
	0xA658,
	0xA659,
	0xA65A,
	0xA65B,
	0xA65C,
	0xA65D,
	0xA65E,
	0xA65F,
	0xA660,
	0xA661,
	0xA662,
	0xA663,
	0xA664,
	0xA665,
	0xA666,
	0xA667,
	0xA668,
	0xA669,
	0xA66A,
	0xA66B,
	0xA66C,
	0xA66D,
	0xA680,
	0xA681,
	0xA682,
	0xA683,
	0xA684,
	0xA685,
	0xA686,
	0xA687,
	0xA688,
	0xA689,
	0xA68A,
	0xA68B,
	0xA68C,
	0xA68D,
	0xA68E,
	0xA68F,
	0xA690,
	0xA691,
	0xA692,
	0xA693,
	0xA694,
	0xA695,
	0xA696,
	0xA697,
	0xA698,
	0xA699,
	0xA69A,
	0xA69B,
	0xA722,
	0xA723,
	0xA724,
	0xA725,
	0xA726,
	0xA727,
	0xA728,
	0xA729,
	0xA72A,
	0xA72B,
	0xA72C,
	0xA72D,
	0xA72E,
	0xA72F,
	0xA732,
	0xA733,
	0xA734,
	0xA735,
	0xA736,
	0xA737,
	0xA738,
	0xA739,
	0xA73A,
	0xA73B,
	0xA73C,
	0xA73D,
	0xA73E,
	0xA73F,
	0xA740,
	0xA741,
	0xA742,
	0xA743,
	0xA744,
	0xA745,
	0xA746,
	0xA747,
	0xA748,
	0xA749,
	0xA74A,
	0xA74B,
	0xA74C,
	0xA74D,
	0xA74E,
	0xA74F,
	0xA750,
	0xA751,
	0xA752,
	0xA753,
	0xA754,
	0xA755,
	0xA756,
	0xA757,
	0xA758,
	0xA759,
	0xA75A,
	0xA75B,
	0xA75C,
	0xA75D,
	0xA75E,
	0xA75F,
	0xA760,
	0xA761,
	0xA762,
	0xA763,
	0xA764,
	0xA765,
	0xA766,
	0xA767,
	0xA768,
	0xA769,
	0xA76A,
	0xA76B,
	0xA76C,
	0xA76D,
	0xA76E,
	0xA76F,
	0xA779,
	0xA77A,
	0xA77B,
	0xA77C,
	0xA77D,
	0xA77F,
	0xA780,
	0xA781,
	0xA782,
	0xA783,
	0xA784,
	0xA785,
	0xA786,
	0xA787,
	0xA78B,
	0xA78C,
	0xA78D,
	0xA78E,
	0xA790,
	0xA791,
	0xA792,
	0xA793,
	0xA796,
	0xA797,
	0xA798,
	0xA799,
	0xA79A,
	0xA79B,
	0xA79C,
	0xA79D,
	0xA79E,
	0xA79F,
	0xA7A0,
	0xA7A1,
	0xA7A2,
	0xA7A3,
	0xA7A4,
	0xA7A5,
	0xA7A6,
	0xA7A7,
	0xA7A8,
	0xA7A9,
	0xA7AA,
	0xA7AF,
	0xA7B0,
	0xA7B5,
	0xA7B6,
	0xA7B7,
	0xFF21,
	0xFF3B,
	0x10400,
	0x10428,
	0x104B0,
	0x104D4,
	0x10C80,
	0x10CB3,
	0x118A0,
	0x118C0,
	0x1D400,
	0x1D41A,
	0x1D434,
	0x1D44E,
	0x1D468,
	0x1D482,
	0x1D49C,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4B6,
	0x1D4D0,
	0x1D4EA,
	0x1D504,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D538,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D56C,
	0x1D586,
	0x1D5A0,
	0x1D5BA,
	0x1D5D4,
	0x1D5EE,
	0x1D608,
	0x1D622,
	0x1D63C,
	0x1D656,
	0x1D670,
	0x1D68A,
	0x1D6A8,
	0x1D6C1,
	0x1D6E2,
	0x1D6FB,
	0x1D71C,
	0x1D735,
	0x1D756,
	0x1D76F,
	0x1D790,
	0x1D7A9,
	0x1D7CA,
	0x1D7CB,
	0x1E900,
	0x1E922,
	0x1F130,
	0x1F14A,
	0x1F150,
	0x1F16A,
	0x1F170,
	0x1F18A
};

static const UV XPosixWord_invlist[] = { /* for ASCII/Latin1 */
	1361,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x30,
	0x3A,
	0x41,
	0x5B,
	0x5F,
	0x60,
	0x61,
	0x7B,
	0xAA,
	0xAB,
	0xB5,
	0xB6,
	0xBA,
	0xBB,
	0xC0,
	0xD7,
	0xD8,
	0xF7,
	0xF8,
	0x2C2,
	0x2C6,
	0x2D2,
	0x2E0,
	0x2E5,
	0x2EC,
	0x2ED,
	0x2EE,
	0x2EF,
	0x300,
	0x375,
	0x376,
	0x378,
	0x37A,
	0x37E,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x3F6,
	0x3F7,
	0x482,
	0x483,
	0x530,
	0x531,
	0x557,
	0x559,
	0x55A,
	0x561,
	0x588,
	0x591,
	0x5BE,
	0x5BF,
	0x5C0,
	0x5C1,
	0x5C3,
	0x5C4,
	0x5C6,
	0x5C7,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F3,
	0x610,
	0x61B,
	0x620,
	0x66A,
	0x66E,
	0x6D4,
	0x6D5,
	0x6DD,
	0x6DF,
	0x6E9,
	0x6EA,
	0x6FD,
	0x6FF,
	0x700,
	0x710,
	0x74B,
	0x74D,
	0x7B2,
	0x7C0,
	0x7F6,
	0x7FA,
	0x7FB,
	0x800,
	0x82E,
	0x840,
	0x85C,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x8D4,
	0x8E2,
	0x8E3,
	0x964,
	0x966,
	0x970,
	0x971,
	0x984,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BC,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CF,
	0x9D7,
	0x9D8,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E4,
	0x9E6,
	0x9F2,
	0xA01,
	0xA04,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA3C,
	0xA3D,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4E,
	0xA51,
	0xA52,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA66,
	0xA76,
	0xA81,
	0xA84,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABC,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACE,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE4,
	0xAE6,
	0xAF0,
	0xAF9,
	0xAFA,
	0xB01,
	0xB04,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3C,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4E,
	0xB56,
	0xB58,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB64,
	0xB66,
	0xB70,
	0xB71,
	0xB72,
	0xB82,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCE,
	0xBD0,
	0xBD1,
	0xBD7,
	0xBD8,
	0xBE6,
	0xBF0,
	0xC00,
	0xC04,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4E,
	0xC55,
	0xC57,
	0xC58,
	0xC5B,
	0xC60,
	0xC64,
	0xC66,
	0xC70,
	0xC80,
	0xC84,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBC,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCE,
	0xCD5,
	0xCD7,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE4,
	0xCE6,
	0xCF0,
	0xCF1,
	0xCF3,
	0xD01,
	0xD04,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD4F,
	0xD54,
	0xD58,
	0xD5F,
	0xD64,
	0xD66,
	0xD70,
	0xD7A,
	0xD80,
	0xD82,
	0xD84,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xDCA,
	0xDCB,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDE6,
	0xDF0,
	0xDF2,
	0xDF4,
	0xE01,
	0xE3B,
	0xE40,
	0xE4F,
	0xE50,
	0xE5A,
	0xE81,
	0xE83,
	0xE84,
	0xE85,
	0xE87,
	0xE89,
	0xE8A,
	0xE8B,
	0xE8D,
	0xE8E,
	0xE94,
	0xE98,
	0xE99,
	0xEA0,
	0xEA1,
	0xEA4,
	0xEA5,
	0xEA6,
	0xEA7,
	0xEA8,
	0xEAA,
	0xEAC,
	0xEAD,
	0xEBA,
	0xEBB,
	0xEBE,
	0xEC0,
	0xEC5,
	0xEC6,
	0xEC7,
	0xEC8,
	0xECE,
	0xED0,
	0xEDA,
	0xEDC,
	0xEE0,
	0xF00,
	0xF01,
	0xF18,
	0xF1A,
	0xF20,
	0xF2A,
	0xF35,
	0xF36,
	0xF37,
	0xF38,
	0xF39,
	0xF3A,
	0xF3E,
	0xF48,
	0xF49,
	0xF6D,
	0xF71,
	0xF85,
	0xF86,
	0xF98,
	0xF99,
	0xFBD,
	0xFC6,
	0xFC7,
	0x1000,
	0x104A,
	0x1050,
	0x109E,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x10FB,
	0x10FC,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x135D,
	0x1360,
	0x1380,
	0x1390,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1401,
	0x166D,
	0x166F,
	0x1680,
	0x1681,
	0x169B,
	0x16A0,
	0x16EB,
	0x16EE,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1715,
	0x1720,
	0x1735,
	0x1740,
	0x1754,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1772,
	0x1774,
	0x1780,
	0x17D4,
	0x17D7,
	0x17D8,
	0x17DC,
	0x17DE,
	0x17E0,
	0x17EA,
	0x180B,
	0x180E,
	0x1810,
	0x181A,
	0x1820,
	0x1878,
	0x1880,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1920,
	0x192C,
	0x1930,
	0x193C,
	0x1946,
	0x196E,
	0x1970,
	0x1975,
	0x1980,
	0x19AC,
	0x19B0,
	0x19CA,
	0x19D0,
	0x19DA,
	0x1A00,
	0x1A1C,
	0x1A20,
	0x1A5F,
	0x1A60,
	0x1A7D,
	0x1A7F,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1AA7,
	0x1AA8,
	0x1AB0,
	0x1ABF,
	0x1B00,
	0x1B4C,
	0x1B50,
	0x1B5A,
	0x1B6B,
	0x1B74,
	0x1B80,
	0x1BF4,
	0x1C00,
	0x1C38,
	0x1C40,
	0x1C4A,
	0x1C4D,
	0x1C7E,
	0x1C80,
	0x1C89,
	0x1CD0,
	0x1CD3,
	0x1CD4,
	0x1CF7,
	0x1CF8,
	0x1CFA,
	0x1D00,
	0x1DF6,
	0x1DFB,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FBD,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FCD,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FE0,
	0x1FED,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFD,
	0x200C,
	0x200E,
	0x203F,
	0x2041,
	0x2054,
	0x2055,
	0x2071,
	0x2072,
	0x207F,
	0x2080,
	0x2090,
	0x209D,
	0x20D0,
	0x20F1,
	0x2102,
	0x2103,
	0x2107,
	0x2108,
	0x210A,
	0x2114,
	0x2115,
	0x2116,
	0x2119,
	0x211E,
	0x2124,
	0x2125,
	0x2126,
	0x2127,
	0x2128,
	0x2129,
	0x212A,
	0x212E,
	0x212F,
	0x213A,
	0x213C,
	0x2140,
	0x2145,
	0x214A,
	0x214E,
	0x214F,
	0x2160,
	0x2189,
	0x24B6,
	0x24EA,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CE5,
	0x2CEB,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D70,
	0x2D7F,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x2DE0,
	0x2E00,
	0x2E2F,
	0x2E30,
	0x3005,
	0x3008,
	0x3021,
	0x3030,
	0x3031,
	0x3036,
	0x3038,
	0x303D,
	0x3041,
	0x3097,
	0x3099,
	0x309B,
	0x309D,
	0x30A0,
	0x30A1,
	0x30FB,
	0x30FC,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x31A0,
	0x31BB,
	0x31F0,
	0x3200,
	0x3400,
	0x4DB6,
	0x4E00,
	0x9FD6,
	0xA000,
	0xA48D,
	0xA4D0,
	0xA4FE,
	0xA500,
	0xA60D,
	0xA610,
	0xA62C,
	0xA640,
	0xA673,
	0xA674,
	0xA67E,
	0xA67F,
	0xA6F2,
	0xA717,
	0xA720,
	0xA722,
	0xA789,
	0xA78B,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F7,
	0xA828,
	0xA840,
	0xA874,
	0xA880,
	0xA8C6,
	0xA8D0,
	0xA8DA,
	0xA8E0,
	0xA8F8,
	0xA8FB,
	0xA8FC,
	0xA8FD,
	0xA8FE,
	0xA900,
	0xA92E,
	0xA930,
	0xA954,
	0xA960,
	0xA97D,
	0xA980,
	0xA9C1,
	0xA9CF,
	0xA9DA,
	0xA9E0,
	0xA9FF,
	0xAA00,
	0xAA37,
	0xAA40,
	0xAA4E,
	0xAA50,
	0xAA5A,
	0xAA60,
	0xAA77,
	0xAA7A,
	0xAAC3,
	0xAADB,
	0xAADE,
	0xAAE0,
	0xAAF0,
	0xAAF2,
	0xAAF7,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB5B,
	0xAB5C,
	0xAB66,
	0xAB70,
	0xABEB,
	0xABEC,
	0xABEE,
	0xABF0,
	0xABFA,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xF900,
	0xFA6E,
	0xFA70,
	0xFADA,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB29,
	0xFB2A,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFBB2,
	0xFBD3,
	0xFD3E,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFC,
	0xFE00,
	0xFE10,
	0xFE20,
	0xFE30,
	0xFE33,
	0xFE35,
	0xFE4D,
	0xFE50,
	0xFE70,
	0xFE75,
	0xFE76,
	0xFEFD,
	0xFF10,
	0xFF1A,
	0xFF21,
	0xFF3B,
	0xFF3F,
	0xFF40,
	0xFF41,
	0xFF5B,
	0xFF66,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10140,
	0x10175,
	0x101FD,
	0x101FE,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x102E0,
	0x102E1,
	0x10300,
	0x10320,
	0x10330,
	0x1034B,
	0x10350,
	0x1037B,
	0x10380,
	0x1039E,
	0x103A0,
	0x103C4,
	0x103C8,
	0x103D0,
	0x103D1,
	0x103D6,
	0x10400,
	0x1049E,
	0x104A0,
	0x104AA,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10860,
	0x10877,
	0x10880,
	0x1089F,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x10900,
	0x10916,
	0x10920,
	0x1093A,
	0x10980,
	0x109B8,
	0x109BE,
	0x109C0,
	0x10A00,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A38,
	0x10A3B,
	0x10A3F,
	0x10A40,
	0x10A60,
	0x10A7D,
	0x10A80,
	0x10A9D,
	0x10AC0,
	0x10AC8,
	0x10AC9,
	0x10AE7,
	0x10B00,
	0x10B36,
	0x10B40,
	0x10B56,
	0x10B60,
	0x10B73,
	0x10B80,
	0x10B92,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x11000,
	0x11047,
	0x11066,
	0x11070,
	0x1107F,
	0x110BB,
	0x110D0,
	0x110E9,
	0x110F0,
	0x110FA,
	0x11100,
	0x11135,
	0x11136,
	0x11140,
	0x11150,
	0x11174,
	0x11176,
	0x11177,
	0x11180,
	0x111C5,
	0x111CA,
	0x111CD,
	0x111D0,
	0x111DB,
	0x111DC,
	0x111DD,
	0x11200,
	0x11212,
	0x11213,
	0x11238,
	0x1123E,
	0x1123F,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112A9,
	0x112B0,
	0x112EB,
	0x112F0,
	0x112FA,
	0x11300,
	0x11304,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133C,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134E,
	0x11350,
	0x11351,
	0x11357,
	0x11358,
	0x1135D,
	0x11364,
	0x11366,
	0x1136D,
	0x11370,
	0x11375,
	0x11400,
	0x1144B,
	0x11450,
	0x1145A,
	0x11480,
	0x114C6,
	0x114C7,
	0x114C8,
	0x114D0,
	0x114DA,
	0x11580,
	0x115B6,
	0x115B8,
	0x115C1,
	0x115D8,
	0x115DE,
	0x11600,
	0x11641,
	0x11644,
	0x11645,
	0x11650,
	0x1165A,
	0x11680,
	0x116B8,
	0x116C0,
	0x116CA,
	0x11700,
	0x1171A,
	0x1171D,
	0x1172C,
	0x11730,
	0x1173A,
	0x118A0,
	0x118EA,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C37,
	0x11C38,
	0x11C41,
	0x11C50,
	0x11C5A,
	0x11C72,
	0x11C90,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16A60,
	0x16A6A,
	0x16AD0,
	0x16AEE,
	0x16AF0,
	0x16AF5,
	0x16B00,
	0x16B37,
	0x16B40,
	0x16B44,
	0x16B50,
	0x16B5A,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F7F,
	0x16F8F,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1BC9D,
	0x1BC9F,
	0x1D165,
	0x1D16A,
	0x1D16D,
	0x1D173,
	0x1D17B,
	0x1D183,
	0x1D185,
	0x1D18C,
	0x1D1AA,
	0x1D1AE,
	0x1D242,
	0x1D245,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D6C1,
	0x1D6C2,
	0x1D6DB,
	0x1D6DC,
	0x1D6FB,
	0x1D6FC,
	0x1D715,
	0x1D716,
	0x1D735,
	0x1D736,
	0x1D74F,
	0x1D750,
	0x1D76F,
	0x1D770,
	0x1D789,
	0x1D78A,
	0x1D7A9,
	0x1D7AA,
	0x1D7C3,
	0x1D7C4,
	0x1D7CC,
	0x1D7CE,
	0x1D800,
	0x1DA00,
	0x1DA37,
	0x1DA3B,
	0x1DA6D,
	0x1DA75,
	0x1DA76,
	0x1DA84,
	0x1DA85,
	0x1DA9B,
	0x1DAA0,
	0x1DAA1,
	0x1DAB0,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E800,
	0x1E8C5,
	0x1E8D0,
	0x1E8D7,
	0x1E900,
	0x1E94B,
	0x1E950,
	0x1E95A,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x1F130,
	0x1F14A,
	0x1F150,
	0x1F16A,
	0x1F170,
	0x1F18A,
	0x20000,
	0x2A6D7,
	0x2A700,
	0x2B735,
	0x2B740,
	0x2B81E,
	0x2B820,
	0x2CEA2,
	0x2F800,
	0x2FA1E,
	0xE0100,
	0xE01F0
};

static const UV XPosixXDigit_invlist[] = { /* for ASCII/Latin1 */
	13,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x30,
	0x3A,
	0x41,
	0x47,
	0x61,
	0x67,
	0xFF10,
	0xFF1A,
	0xFF21,
	0xFF27,
	0xFF41,
	0xFF47
};

#endif	/* defined(PERL_IN_PERL_C) */

#endif	/* ASCII/Latin1 */

#if 'A' == 193 /* EBCDIC 1047 */ \
     && '\\' == 224 && '[' == 173 && ']' == 189 && '{' == 192 && '}' == 208 \
     && '^' == 95 && '~' == 161 && '!' == 90 && '#' == 123 && '|' == 79 \
     && '$' == 91 && '@' == 124 && '`' == 121

#if defined(PERL_IN_PERL_C)

static const UV ASCII_invlist[] = { /* for EBCDIC 1047 */
	54,	/* Number of elements */
	148565664, /* Version and data structure type */
	0,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x4,
	0x5,
	0x6,
	0x7,
	0x8,
	0xB,
	0x14,
	0x15,
	0x17,
	0x18,
	0x1A,
	0x1C,
	0x20,
	0x26,
	0x28,
	0x2D,
	0x30,
	0x32,
	0x33,
	0x37,
	0x38,
	0x3C,
	0x3E,
	0x3F,
	0x41,
	0x4B,
	0x51,
	0x5A,
	0x62,
	0x6B,
	0x70,
	0x79,
	0x80,
	0x81,
	0x8A,
	0x91,
	0x9A,
	0xA1,
	0xAA,
	0xAD,
	0xAE,
	0xBD,
	0xBE,
	0xC0,
	0xCA,
	0xD0,
	0xDA,
	0xE0,
	0xE1,
	0xE2,
	0xEA,
	0xF0,
	0xFA
};

static const UV Assigned_invlist[] = { /* for EBCDIC 1047 */
	1276,	/* Number of elements */
	148565664, /* Version and data structure type */
	0,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x378,
	0x37A,
	0x380,
	0x384,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x530,
	0x531,
	0x557,
	0x559,
	0x560,
	0x561,
	0x588,
	0x589,
	0x58B,
	0x58D,
	0x590,
	0x591,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F5,
	0x600,
	0x61D,
	0x61E,
	0x70E,
	0x70F,
	0x74B,
	0x74D,
	0x7B2,
	0x7C0,
	0x7FB,
	0x800,
	0x82E,
	0x830,
	0x83F,
	0x840,
	0x85C,
	0x85E,
	0x85F,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x8D4,
	0x984,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BC,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CF,
	0x9D7,
	0x9D8,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E4,
	0x9E6,
	0x9FC,
	0xA01,
	0xA04,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA3C,
	0xA3D,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4E,
	0xA51,
	0xA52,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA66,
	0xA76,
	0xA81,
	0xA84,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABC,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACE,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE4,
	0xAE6,
	0xAF2,
	0xAF9,
	0xAFA,
	0xB01,
	0xB04,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3C,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4E,
	0xB56,
	0xB58,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB64,
	0xB66,
	0xB78,
	0xB82,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCE,
	0xBD0,
	0xBD1,
	0xBD7,
	0xBD8,
	0xBE6,
	0xBFB,
	0xC00,
	0xC04,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4E,
	0xC55,
	0xC57,
	0xC58,
	0xC5B,
	0xC60,
	0xC64,
	0xC66,
	0xC70,
	0xC78,
	0xC84,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBC,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCE,
	0xCD5,
	0xCD7,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE4,
	0xCE6,
	0xCF0,
	0xCF1,
	0xCF3,
	0xD01,
	0xD04,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD50,
	0xD54,
	0xD64,
	0xD66,
	0xD80,
	0xD82,
	0xD84,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xDCA,
	0xDCB,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDE6,
	0xDF0,
	0xDF2,
	0xDF5,
	0xE01,
	0xE3B,
	0xE3F,
	0xE5C,
	0xE81,
	0xE83,
	0xE84,
	0xE85,
	0xE87,
	0xE89,
	0xE8A,
	0xE8B,
	0xE8D,
	0xE8E,
	0xE94,
	0xE98,
	0xE99,
	0xEA0,
	0xEA1,
	0xEA4,
	0xEA5,
	0xEA6,
	0xEA7,
	0xEA8,
	0xEAA,
	0xEAC,
	0xEAD,
	0xEBA,
	0xEBB,
	0xEBE,
	0xEC0,
	0xEC5,
	0xEC6,
	0xEC7,
	0xEC8,
	0xECE,
	0xED0,
	0xEDA,
	0xEDC,
	0xEE0,
	0xF00,
	0xF48,
	0xF49,
	0xF6D,
	0xF71,
	0xF98,
	0xF99,
	0xFBD,
	0xFBE,
	0xFCD,
	0xFCE,
	0xFDB,
	0x1000,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x135D,
	0x137D,
	0x1380,
	0x139A,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1400,
	0x169D,
	0x16A0,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1715,
	0x1720,
	0x1737,
	0x1740,
	0x1754,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1772,
	0x1774,
	0x1780,
	0x17DE,
	0x17E0,
	0x17EA,
	0x17F0,
	0x17FA,
	0x1800,
	0x180F,
	0x1810,
	0x181A,
	0x1820,
	0x1878,
	0x1880,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1920,
	0x192C,
	0x1930,
	0x193C,
	0x1940,
	0x1941,
	0x1944,
	0x196E,
	0x1970,
	0x1975,
	0x1980,
	0x19AC,
	0x19B0,
	0x19CA,
	0x19D0,
	0x19DB,
	0x19DE,
	0x1A1C,
	0x1A1E,
	0x1A5F,
	0x1A60,
	0x1A7D,
	0x1A7F,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1AA0,
	0x1AAE,
	0x1AB0,
	0x1ABF,
	0x1B00,
	0x1B4C,
	0x1B50,
	0x1B7D,
	0x1B80,
	0x1BF4,
	0x1BFC,
	0x1C38,
	0x1C3B,
	0x1C4A,
	0x1C4D,
	0x1C89,
	0x1CC0,
	0x1CC8,
	0x1CD0,
	0x1CF7,
	0x1CF8,
	0x1CFA,
	0x1D00,
	0x1DF6,
	0x1DFB,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FC5,
	0x1FC6,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FDD,
	0x1FF0,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFF,
	0x2000,
	0x2065,
	0x2066,
	0x2072,
	0x2074,
	0x208F,
	0x2090,
	0x209D,
	0x20A0,
	0x20BF,
	0x20D0,
	0x20F1,
	0x2100,
	0x218C,
	0x2190,
	0x23FF,
	0x2400,
	0x2427,
	0x2440,
	0x244B,
	0x2460,
	0x2B74,
	0x2B76,
	0x2B96,
	0x2B98,
	0x2BBA,
	0x2BBD,
	0x2BC9,
	0x2BCA,
	0x2BD2,
	0x2BEC,
	0x2BF0,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CF4,
	0x2CF9,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D71,
	0x2D7F,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x2DE0,
	0x2E45,
	0x2E80,
	0x2E9A,
	0x2E9B,
	0x2EF4,
	0x2F00,
	0x2FD6,
	0x2FF0,
	0x2FFC,
	0x3000,
	0x3040,
	0x3041,
	0x3097,
	0x3099,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x3190,
	0x31BB,
	0x31C0,
	0x31E4,
	0x31F0,
	0x321F,
	0x3220,
	0x32FF,
	0x3300,
	0x4DB6,
	0x4DC0,
	0x9FD6,
	0xA000,
	0xA48D,
	0xA490,
	0xA4C7,
	0xA4D0,
	0xA62C,
	0xA640,
	0xA6F8,
	0xA700,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F7,
	0xA82C,
	0xA830,
	0xA83A,
	0xA840,
	0xA878,
	0xA880,
	0xA8C6,
	0xA8CE,
	0xA8DA,
	0xA8E0,
	0xA8FE,
	0xA900,
	0xA954,
	0xA95F,
	0xA97D,
	0xA980,
	0xA9CE,
	0xA9CF,
	0xA9DA,
	0xA9DE,
	0xA9FF,
	0xAA00,
	0xAA37,
	0xAA40,
	0xAA4E,
	0xAA50,
	0xAA5A,
	0xAA5C,
	0xAAC3,
	0xAADB,
	0xAAF7,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB66,
	0xAB70,
	0xABEE,
	0xABF0,
	0xABFA,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xD800,
	0xFA6E,
	0xFA70,
	0xFADA,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFBC2,
	0xFBD3,
	0xFD40,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFE,
	0xFE00,
	0xFE1A,
	0xFE20,
	0xFE53,
	0xFE54,
	0xFE67,
	0xFE68,
	0xFE6C,
	0xFE70,
	0xFE75,
	0xFE76,
	0xFEFD,
	0xFEFF,
	0xFF00,
	0xFF01,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0xFFE0,
	0xFFE7,
	0xFFE8,
	0xFFEF,
	0xFFF9,
	0xFFFE,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10100,
	0x10103,
	0x10107,
	0x10134,
	0x10137,
	0x1018F,
	0x10190,
	0x1019C,
	0x101A0,
	0x101A1,
	0x101D0,
	0x101FE,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x102E0,
	0x102FC,
	0x10300,
	0x10324,
	0x10330,
	0x1034B,
	0x10350,
	0x1037B,
	0x10380,
	0x1039E,
	0x1039F,
	0x103C4,
	0x103C8,
	0x103D6,
	0x10400,
	0x1049E,
	0x104A0,
	0x104AA,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x1056F,
	0x10570,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10857,
	0x1089F,
	0x108A7,
	0x108B0,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x108FB,
	0x1091C,
	0x1091F,
	0x1093A,
	0x1093F,
	0x10940,
	0x10980,
	0x109B8,
	0x109BC,
	0x109D0,
	0x109D2,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A38,
	0x10A3B,
	0x10A3F,
	0x10A48,
	0x10A50,
	0x10A59,
	0x10A60,
	0x10AA0,
	0x10AC0,
	0x10AE7,
	0x10AEB,
	0x10AF7,
	0x10B00,
	0x10B36,
	0x10B39,
	0x10B56,
	0x10B58,
	0x10B73,
	0x10B78,
	0x10B92,
	0x10B99,
	0x10B9D,
	0x10BA9,
	0x10BB0,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x10CFA,
	0x10D00,
	0x10E60,
	0x10E7F,
	0x11000,
	0x1104E,
	0x11052,
	0x11070,
	0x1107F,
	0x110C2,
	0x110D0,
	0x110E9,
	0x110F0,
	0x110FA,
	0x11100,
	0x11135,
	0x11136,
	0x11144,
	0x11150,
	0x11177,
	0x11180,
	0x111CE,
	0x111D0,
	0x111E0,
	0x111E1,
	0x111F5,
	0x11200,
	0x11212,
	0x11213,
	0x1123F,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112AA,
	0x112B0,
	0x112EB,
	0x112F0,
	0x112FA,
	0x11300,
	0x11304,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133C,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134E,
	0x11350,
	0x11351,
	0x11357,
	0x11358,
	0x1135D,
	0x11364,
	0x11366,
	0x1136D,
	0x11370,
	0x11375,
	0x11400,
	0x1145A,
	0x1145B,
	0x1145C,
	0x1145D,
	0x1145E,
	0x11480,
	0x114C8,
	0x114D0,
	0x114DA,
	0x11580,
	0x115B6,
	0x115B8,
	0x115DE,
	0x11600,
	0x11645,
	0x11650,
	0x1165A,
	0x11660,
	0x1166D,
	0x11680,
	0x116B8,
	0x116C0,
	0x116CA,
	0x11700,
	0x1171A,
	0x1171D,
	0x1172C,
	0x11730,
	0x11740,
	0x118A0,
	0x118F3,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C37,
	0x11C38,
	0x11C46,
	0x11C50,
	0x11C6D,
	0x11C70,
	0x11C90,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12470,
	0x12475,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16A60,
	0x16A6A,
	0x16A6E,
	0x16A70,
	0x16AD0,
	0x16AEE,
	0x16AF0,
	0x16AF6,
	0x16B00,
	0x16B46,
	0x16B50,
	0x16B5A,
	0x16B5B,
	0x16B62,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F7F,
	0x16F8F,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1BC9C,
	0x1BCA4,
	0x1D000,
	0x1D0F6,
	0x1D100,
	0x1D127,
	0x1D129,
	0x1D1E9,
	0x1D200,
	0x1D246,
	0x1D300,
	0x1D357,
	0x1D360,
	0x1D372,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D7CC,
	0x1D7CE,
	0x1DA8C,
	0x1DA9B,
	0x1DAA0,
	0x1DAA1,
	0x1DAB0,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E800,
	0x1E8C5,
	0x1E8C7,
	0x1E8D7,
	0x1E900,
	0x1E94B,
	0x1E950,
	0x1E95A,
	0x1E95E,
	0x1E960,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x1EEF0,
	0x1EEF2,
	0x1F000,
	0x1F02C,
	0x1F030,
	0x1F094,
	0x1F0A0,
	0x1F0AF,
	0x1F0B1,
	0x1F0C0,
	0x1F0C1,
	0x1F0D0,
	0x1F0D1,
	0x1F0F6,
	0x1F100,
	0x1F10D,
	0x1F110,
	0x1F12F,
	0x1F130,
	0x1F16C,
	0x1F170,
	0x1F1AD,
	0x1F1E6,
	0x1F203,
	0x1F210,
	0x1F23C,
	0x1F240,
	0x1F249,
	0x1F250,
	0x1F252,
	0x1F300,
	0x1F6D3,
	0x1F6E0,
	0x1F6ED,
	0x1F6F0,
	0x1F6F7,
	0x1F700,
	0x1F774,
	0x1F780,
	0x1F7D5,
	0x1F800,
	0x1F80C,
	0x1F810,
	0x1F848,
	0x1F850,
	0x1F85A,
	0x1F860,
	0x1F888,
	0x1F890,
	0x1F8AE,
	0x1F910,
	0x1F91F,
	0x1F920,
	0x1F928,
	0x1F930,
	0x1F931,
	0x1F933,
	0x1F93F,
	0x1F940,
	0x1F94C,
	0x1F950,
	0x1F95F,
	0x1F980,
	0x1F992,
	0x1F9C0,
	0x1F9C1,
	0x20000,
	0x2A6D7,
	0x2A700,
	0x2B735,
	0x2B740,
	0x2B81E,
	0x2B820,
	0x2CEA2,
	0x2F800,
	0x2FA1E,
	0xE0001,
	0xE0002,
	0xE0020,
	0xE0080,
	0xE0100,
	0xE01F0,
	0xF0000,
	0xFFFFE,
	0x100000,
	0x10FFFE
};

static const UV Cased_invlist[] = { /* for EBCDIC 1047 */
	297,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x42,
	0x4A,
	0x51,
	0x5A,
	0x62,
	0x6A,
	0x70,
	0x79,
	0x80,
	0x8A,
	0x8C,
	0x8F,
	0x91,
	0x9D,
	0x9E,
	0x9F,
	0xA0,
	0xA1,
	0xA2,
	0xAA,
	0xAC,
	0xAD,
	0xAE,
	0xAF,
	0xBA,
	0xBB,
	0xC1,
	0xCA,
	0xCB,
	0xD0,
	0xD1,
	0xDA,
	0xDB,
	0xE0,
	0xE2,
	0xEA,
	0xEB,
	0xF0,
	0xFB,
	0xFF,
	0x100,
	0x1BB,
	0x1BC,
	0x1C0,
	0x1C4,
	0x294,
	0x295,
	0x2B9,
	0x2C0,
	0x2C2,
	0x2E0,
	0x2E5,
	0x345,
	0x346,
	0x370,
	0x374,
	0x376,
	0x378,
	0x37A,
	0x37E,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x3F6,
	0x3F7,
	0x482,
	0x48A,
	0x530,
	0x531,
	0x557,
	0x561,
	0x588,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1C80,
	0x1C89,
	0x1D00,
	0x1DC0,
	0x1E00,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FBD,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FCD,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FE0,
	0x1FED,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFD,
	0x2071,
	0x2072,
	0x207F,
	0x2080,
	0x2090,
	0x209D,
	0x2102,
	0x2103,
	0x2107,
	0x2108,
	0x210A,
	0x2114,
	0x2115,
	0x2116,
	0x2119,
	0x211E,
	0x2124,
	0x2125,
	0x2126,
	0x2127,
	0x2128,
	0x2129,
	0x212A,
	0x212E,
	0x212F,
	0x2135,
	0x2139,
	0x213A,
	0x213C,
	0x2140,
	0x2145,
	0x214A,
	0x214E,
	0x214F,
	0x2160,
	0x2180,
	0x2183,
	0x2185,
	0x24B6,
	0x24EA,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CE5,
	0x2CEB,
	0x2CEF,
	0x2CF2,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0xA640,
	0xA66E,
	0xA680,
	0xA69E,
	0xA722,
	0xA788,
	0xA78B,
	0xA78F,
	0xA790,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F8,
	0xA7FB,
	0xAB30,
	0xAB5B,
	0xAB5C,
	0xAB66,
	0xAB70,
	0xABC0,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFF21,
	0xFF3B,
	0xFF41,
	0xFF5B,
	0x10400,
	0x10450,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x118A0,
	0x118E0,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D6C1,
	0x1D6C2,
	0x1D6DB,
	0x1D6DC,
	0x1D6FB,
	0x1D6FC,
	0x1D715,
	0x1D716,
	0x1D735,
	0x1D736,
	0x1D74F,
	0x1D750,
	0x1D76F,
	0x1D770,
	0x1D789,
	0x1D78A,
	0x1D7A9,
	0x1D7AA,
	0x1D7C3,
	0x1D7C4,
	0x1D7CC,
	0x1E900,
	0x1E944,
	0x1F130,
	0x1F14A,
	0x1F150,
	0x1F16A,
	0x1F170,
	0x1F18A
};

#endif	/* defined(PERL_IN_PERL_C) */

#if defined(PERL_IN_REGCOMP_C)

static const UV NonL1_Perl_Non_Final_Folds_invlist[] = { /* for EBCDIC 1047 */
	45,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x2BC,
	0x2BD,
	0x308,
	0x309,
	0x313,
	0x314,
	0x342,
	0x343,
	0x3AC,
	0x3AD,
	0x3AE,
	0x3AF,
	0x3B1,
	0x3B2,
	0x3B7,
	0x3B8,
	0x3B9,
	0x3BA,
	0x3C1,
	0x3C2,
	0x3C5,
	0x3C6,
	0x3C9,
	0x3CA,
	0x3CE,
	0x3CF,
	0x565,
	0x566,
	0x574,
	0x575,
	0x57E,
	0x57F,
	0x1F00,
	0x1F08,
	0x1F20,
	0x1F28,
	0x1F60,
	0x1F68,
	0x1F70,
	0x1F71,
	0x1F74,
	0x1F75,
	0x1F7C,
	0x1F7D
};

static const UV _Perl_Any_Folds_invlist[] = { /* for EBCDIC 1047 */
	285,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x42,
	0x4A,
	0x51,
	0x5A,
	0x62,
	0x6A,
	0x70,
	0x79,
	0x80,
	0x8A,
	0x8C,
	0x8F,
	0x91,
	0x9A,
	0x9C,
	0x9D,
	0x9E,
	0x9F,
	0xA0,
	0xA1,
	0xA2,
	0xAA,
	0xAC,
	0xAD,
	0xAE,
	0xAF,
	0xBA,
	0xBB,
	0xC1,
	0xCA,
	0xCB,
	0xD0,
	0xD1,
	0xDA,
	0xDB,
	0xE0,
	0xE2,
	0xEA,
	0xEB,
	0xF0,
	0xFB,
	0xFF,
	0x100,
	0x131,
	0x132,
	0x138,
	0x139,
	0x18D,
	0x18E,
	0x19B,
	0x19C,
	0x1AA,
	0x1AC,
	0x1BA,
	0x1BC,
	0x1BE,
	0x1BF,
	0x1C0,
	0x1C4,
	0x221,
	0x222,
	0x234,
	0x23A,
	0x255,
	0x256,
	0x258,
	0x259,
	0x25A,
	0x25B,
	0x25D,
	0x260,
	0x262,
	0x263,
	0x264,
	0x265,
	0x267,
	0x268,
	0x26D,
	0x26F,
	0x270,
	0x271,
	0x273,
	0x275,
	0x276,
	0x27D,
	0x27E,
	0x280,
	0x281,
	0x283,
	0x284,
	0x287,
	0x28D,
	0x292,
	0x293,
	0x29D,
	0x29F,
	0x2BC,
	0x2BD,
	0x2BE,
	0x2BF,
	0x300,
	0x302,
	0x307,
	0x309,
	0x30A,
	0x30B,
	0x30C,
	0x30D,
	0x313,
	0x314,
	0x331,
	0x332,
	0x342,
	0x343,
	0x345,
	0x346,
	0x370,
	0x374,
	0x376,
	0x378,
	0x37B,
	0x37E,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x3D2,
	0x3D5,
	0x3F6,
	0x3F7,
	0x3FC,
	0x3FD,
	0x482,
	0x48A,
	0x530,
	0x531,
	0x557,
	0x561,
	0x588,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1C80,
	0x1C89,
	0x1D79,
	0x1D7A,
	0x1D7D,
	0x1D7E,
	0x1E00,
	0x1E9C,
	0x1E9E,
	0x1E9F,
	0x1EA0,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FBD,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FCD,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FE0,
	0x1FED,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFD,
	0x2126,
	0x2127,
	0x212A,
	0x212C,
	0x2132,
	0x2133,
	0x214E,
	0x214F,
	0x2160,
	0x2180,
	0x2183,
	0x2185,
	0x24B6,
	0x24EA,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2C71,
	0x2C72,
	0x2C74,
	0x2C75,
	0x2C77,
	0x2C7E,
	0x2CE4,
	0x2CEB,
	0x2CEF,
	0x2CF2,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0xA640,
	0xA66E,
	0xA680,
	0xA69C,
	0xA722,
	0xA730,
	0xA732,
	0xA770,
	0xA779,
	0xA788,
	0xA78B,
	0xA78E,
	0xA790,
	0xA794,
	0xA796,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xAB53,
	0xAB54,
	0xAB70,
	0xABC0,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFF21,
	0xFF3B,
	0xFF41,
	0xFF5B,
	0x10400,
	0x10450,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x118A0,
	0x118E0,
	0x1E900,
	0x1E944
};

static const UV _Perl_Folds_To_Multi_Char_invlist[] = { /* for EBCDIC 1047 */
	59,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x59,
	0x5A,
	0x130,
	0x131,
	0x149,
	0x14A,
	0x1F0,
	0x1F1,
	0x390,
	0x391,
	0x3B0,
	0x3B1,
	0x587,
	0x588,
	0x1E96,
	0x1E9B,
	0x1E9E,
	0x1E9F,
	0x1F50,
	0x1F51,
	0x1F52,
	0x1F53,
	0x1F54,
	0x1F55,
	0x1F56,
	0x1F57,
	0x1F80,
	0x1FB0,
	0x1FB2,
	0x1FB5,
	0x1FB6,
	0x1FB8,
	0x1FBC,
	0x1FBD,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FC8,
	0x1FCC,
	0x1FCD,
	0x1FD2,
	0x1FD4,
	0x1FD6,
	0x1FD8,
	0x1FE2,
	0x1FE5,
	0x1FE6,
	0x1FE8,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FF8,
	0x1FFC,
	0x1FFD,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18
};

#endif	/* defined(PERL_IN_REGCOMP_C) */

#if defined(PERL_IN_PERL_C)

static const UV _Perl_GCB_invlist[] = { /* for EBCDIC 1047 */
	1615,	/* Number of elements */
	148565664, /* Version and data structure type */
	0,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0xD,
	0xE,
	0x15,
	0x16,
	0x40,
	0xCA,
	0xCB,
	0xFF,
	0x100,
	0x300,
	0x370,
	0x483,
	0x48A,
	0x591,
	0x5BE,
	0x5BF,
	0x5C0,
	0x5C1,
	0x5C3,
	0x5C4,
	0x5C6,
	0x5C7,
	0x5C8,
	0x600,
	0x606,
	0x610,
	0x61B,
	0x61C,
	0x61D,
	0x64B,
	0x660,
	0x670,
	0x671,
	0x6D6,
	0x6DD,
	0x6DE,
	0x6DF,
	0x6E5,
	0x6E7,
	0x6E9,
	0x6EA,
	0x6EE,
	0x70F,
	0x710,
	0x711,
	0x712,
	0x730,
	0x74B,
	0x7A6,
	0x7B1,
	0x7EB,
	0x7F4,
	0x816,
	0x81A,
	0x81B,
	0x824,
	0x825,
	0x828,
	0x829,
	0x82E,
	0x859,
	0x85C,
	0x8D4,
	0x8E2,
	0x8E3,
	0x903,
	0x904,
	0x93A,
	0x93B,
	0x93C,
	0x93D,
	0x93E,
	0x941,
	0x949,
	0x94D,
	0x94E,
	0x950,
	0x951,
	0x958,
	0x962,
	0x964,
	0x981,
	0x982,
	0x984,
	0x9BC,
	0x9BD,
	0x9BE,
	0x9BF,
	0x9C1,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CD,
	0x9CE,
	0x9D7,
	0x9D8,
	0x9E2,
	0x9E4,
	0xA01,
	0xA03,
	0xA04,
	0xA3C,
	0xA3D,
	0xA3E,
	0xA41,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4E,
	0xA51,
	0xA52,
	0xA70,
	0xA72,
	0xA75,
	0xA76,
	0xA81,
	0xA83,
	0xA84,
	0xABC,
	0xABD,
	0xABE,
	0xAC1,
	0xAC6,
	0xAC7,
	0xAC9,
	0xACA,
	0xACB,
	0xACD,
	0xACE,
	0xAE2,
	0xAE4,
	0xB01,
	0xB02,
	0xB04,
	0xB3C,
	0xB3D,
	0xB3E,
	0xB40,
	0xB41,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4D,
	0xB4E,
	0xB56,
	0xB58,
	0xB62,
	0xB64,
	0xB82,
	0xB83,
	0xBBE,
	0xBBF,
	0xBC0,
	0xBC1,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCD,
	0xBCE,
	0xBD7,
	0xBD8,
	0xC00,
	0xC01,
	0xC04,
	0xC3E,
	0xC41,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4E,
	0xC55,
	0xC57,
	0xC62,
	0xC64,
	0xC81,
	0xC82,
	0xC84,
	0xCBC,
	0xCBD,
	0xCBE,
	0xCBF,
	0xCC0,
	0xCC2,
	0xCC3,
	0xCC5,
	0xCC6,
	0xCC7,
	0xCC9,
	0xCCA,
	0xCCC,
	0xCCE,
	0xCD5,
	0xCD7,
	0xCE2,
	0xCE4,
	0xD01,
	0xD02,
	0xD04,
	0xD3E,
	0xD3F,
	0xD41,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD4D,
	0xD4E,
	0xD4F,
	0xD57,
	0xD58,
	0xD62,
	0xD64,
	0xD82,
	0xD84,
	0xDCA,
	0xDCB,
	0xDCF,
	0xDD0,
	0xDD2,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDDF,
	0xDE0,
	0xDF2,
	0xDF4,
	0xE31,
	0xE32,
	0xE33,
	0xE34,
	0xE3B,
	0xE47,
	0xE4F,
	0xEB1,
	0xEB2,
	0xEB3,
	0xEB4,
	0xEBA,
	0xEBB,
	0xEBD,
	0xEC8,
	0xECE,
	0xF18,
	0xF1A,
	0xF35,
	0xF36,
	0xF37,
	0xF38,
	0xF39,
	0xF3A,
	0xF3E,
	0xF40,
	0xF71,
	0xF7F,
	0xF80,
	0xF85,
	0xF86,
	0xF88,
	0xF8D,
	0xF98,
	0xF99,
	0xFBD,
	0xFC6,
	0xFC7,
	0x102D,
	0x1031,
	0x1032,
	0x1038,
	0x1039,
	0x103B,
	0x103D,
	0x103F,
	0x1056,
	0x1058,
	0x105A,
	0x105E,
	0x1061,
	0x1071,
	0x1075,
	0x1082,
	0x1083,
	0x1084,
	0x1085,
	0x1087,
	0x108D,
	0x108E,
	0x109D,
	0x109E,
	0x1100,
	0x1160,
	0x11A8,
	0x1200,
	0x135D,
	0x1360,
	0x1712,
	0x1715,
	0x1732,
	0x1735,
	0x1752,
	0x1754,
	0x1772,
	0x1774,
	0x17B4,
	0x17B6,
	0x17B7,
	0x17BE,
	0x17C6,
	0x17C7,
	0x17C9,
	0x17D4,
	0x17DD,
	0x17DE,
	0x180B,
	0x180E,
	0x180F,
	0x1885,
	0x1887,
	0x18A9,
	0x18AA,
	0x1920,
	0x1923,
	0x1927,
	0x1929,
	0x192C,
	0x1930,
	0x1932,
	0x1933,
	0x1939,
	0x193C,
	0x1A17,
	0x1A19,
	0x1A1B,
	0x1A1C,
	0x1A55,
	0x1A56,
	0x1A57,
	0x1A58,
	0x1A5F,
	0x1A60,
	0x1A61,
	0x1A62,
	0x1A63,
	0x1A65,
	0x1A6D,
	0x1A73,
	0x1A7D,
	0x1A7F,
	0x1A80,
	0x1AB0,
	0x1ABF,
	0x1B00,
	0x1B04,
	0x1B05,
	0x1B34,
	0x1B35,
	0x1B36,
	0x1B3B,
	0x1B3C,
	0x1B3D,
	0x1B42,
	0x1B43,
	0x1B45,
	0x1B6B,
	0x1B74,
	0x1B80,
	0x1B82,
	0x1B83,
	0x1BA1,
	0x1BA2,
	0x1BA6,
	0x1BA8,
	0x1BAA,
	0x1BAB,
	0x1BAE,
	0x1BE6,
	0x1BE7,
	0x1BE8,
	0x1BEA,
	0x1BED,
	0x1BEE,
	0x1BEF,
	0x1BF2,
	0x1BF4,
	0x1C24,
	0x1C2C,
	0x1C34,
	0x1C36,
	0x1C38,
	0x1CD0,
	0x1CD3,
	0x1CD4,
	0x1CE1,
	0x1CE2,
	0x1CE9,
	0x1CED,
	0x1CEE,
	0x1CF2,
	0x1CF4,
	0x1CF5,
	0x1CF8,
	0x1CFA,
	0x1DC0,
	0x1DF6,
	0x1DFB,
	0x1E00,
	0x200B,
	0x200C,
	0x200D,
	0x200E,
	0x2010,
	0x2028,
	0x202F,
	0x2060,
	0x2070,
	0x20D0,
	0x20F1,
	0x261D,
	0x261E,
	0x26F9,
	0x26FA,
	0x270A,
	0x270E,
	0x2764,
	0x2765,
	0x2CEF,
	0x2CF2,
	0x2D7F,
	0x2D80,
	0x2DE0,
	0x2E00,
	0x302A,
	0x3030,
	0x3099,
	0x309B,
	0xA66F,
	0xA673,
	0xA674,
	0xA67E,
	0xA69E,
	0xA6A0,
	0xA6F0,
	0xA6F2,
	0xA802,
	0xA803,
	0xA806,
	0xA807,
	0xA80B,
	0xA80C,
	0xA823,
	0xA825,
	0xA827,
	0xA828,
	0xA880,
	0xA882,
	0xA8B4,
	0xA8C4,
	0xA8C6,
	0xA8E0,
	0xA8F2,
	0xA926,
	0xA92E,
	0xA947,
	0xA952,
	0xA954,
	0xA960,
	0xA97D,
	0xA980,
	0xA983,
	0xA984,
	0xA9B3,
	0xA9B4,
	0xA9B6,
	0xA9BA,
	0xA9BC,
	0xA9BD,
	0xA9C1,
	0xA9E5,
	0xA9E6,
	0xAA29,
	0xAA2F,
	0xAA31,
	0xAA33,
	0xAA35,
	0xAA37,
	0xAA43,
	0xAA44,
	0xAA4C,
	0xAA4D,
	0xAA4E,
	0xAA7C,
	0xAA7D,
	0xAAB0,
	0xAAB1,
	0xAAB2,
	0xAAB5,
	0xAAB7,
	0xAAB9,
	0xAABE,
	0xAAC0,
	0xAAC1,
	0xAAC2,
	0xAAEB,
	0xAAEC,
	0xAAEE,
	0xAAF0,
	0xAAF5,
	0xAAF6,
	0xAAF7,
	0xABE3,
	0xABE5,
	0xABE6,
	0xABE8,
	0xABE9,
	0xABEB,
	0xABEC,
	0xABED,
	0xABEE,
	0xAC00,
	0xAC01,
	0xAC1C,
	0xAC1D,
	0xAC38,
	0xAC39,
	0xAC54,
	0xAC55,
	0xAC70,
	0xAC71,
	0xAC8C,
	0xAC8D,
	0xACA8,
	0xACA9,
	0xACC4,
	0xACC5,
	0xACE0,
	0xACE1,
	0xACFC,
	0xACFD,
	0xAD18,
	0xAD19,
	0xAD34,
	0xAD35,
	0xAD50,
	0xAD51,
	0xAD6C,
	0xAD6D,
	0xAD88,
	0xAD89,
	0xADA4,
	0xADA5,
	0xADC0,
	0xADC1,
	0xADDC,
	0xADDD,
	0xADF8,
	0xADF9,
	0xAE14,
	0xAE15,
	0xAE30,
	0xAE31,
	0xAE4C,
	0xAE4D,
	0xAE68,
	0xAE69,
	0xAE84,
	0xAE85,
	0xAEA0,
	0xAEA1,
	0xAEBC,
	0xAEBD,
	0xAED8,
	0xAED9,
	0xAEF4,
	0xAEF5,
	0xAF10,
	0xAF11,
	0xAF2C,
	0xAF2D,
	0xAF48,
	0xAF49,
	0xAF64,
	0xAF65,
	0xAF80,
	0xAF81,
	0xAF9C,
	0xAF9D,
	0xAFB8,
	0xAFB9,
	0xAFD4,
	0xAFD5,
	0xAFF0,
	0xAFF1,
	0xB00C,
	0xB00D,
	0xB028,
	0xB029,
	0xB044,
	0xB045,
	0xB060,
	0xB061,
	0xB07C,
	0xB07D,
	0xB098,
	0xB099,
	0xB0B4,
	0xB0B5,
	0xB0D0,
	0xB0D1,
	0xB0EC,
	0xB0ED,
	0xB108,
	0xB109,
	0xB124,
	0xB125,
	0xB140,
	0xB141,
	0xB15C,
	0xB15D,
	0xB178,
	0xB179,
	0xB194,
	0xB195,
	0xB1B0,
	0xB1B1,
	0xB1CC,
	0xB1CD,
	0xB1E8,
	0xB1E9,
	0xB204,
	0xB205,
	0xB220,
	0xB221,
	0xB23C,
	0xB23D,
	0xB258,
	0xB259,
	0xB274,
	0xB275,
	0xB290,
	0xB291,
	0xB2AC,
	0xB2AD,
	0xB2C8,
	0xB2C9,
	0xB2E4,
	0xB2E5,
	0xB300,
	0xB301,
	0xB31C,
	0xB31D,
	0xB338,
	0xB339,
	0xB354,
	0xB355,
	0xB370,
	0xB371,
	0xB38C,
	0xB38D,
	0xB3A8,
	0xB3A9,
	0xB3C4,
	0xB3C5,
	0xB3E0,
	0xB3E1,
	0xB3FC,
	0xB3FD,
	0xB418,
	0xB419,
	0xB434,
	0xB435,
	0xB450,
	0xB451,
	0xB46C,
	0xB46D,
	0xB488,
	0xB489,
	0xB4A4,
	0xB4A5,
	0xB4C0,
	0xB4C1,
	0xB4DC,
	0xB4DD,
	0xB4F8,
	0xB4F9,
	0xB514,
	0xB515,
	0xB530,
	0xB531,
	0xB54C,
	0xB54D,
	0xB568,
	0xB569,
	0xB584,
	0xB585,
	0xB5A0,
	0xB5A1,
	0xB5BC,
	0xB5BD,
	0xB5D8,
	0xB5D9,
	0xB5F4,
	0xB5F5,
	0xB610,
	0xB611,
	0xB62C,
	0xB62D,
	0xB648,
	0xB649,
	0xB664,
	0xB665,
	0xB680,
	0xB681,
	0xB69C,
	0xB69D,
	0xB6B8,
	0xB6B9,
	0xB6D4,
	0xB6D5,
	0xB6F0,
	0xB6F1,
	0xB70C,
	0xB70D,
	0xB728,
	0xB729,
	0xB744,
	0xB745,
	0xB760,
	0xB761,
	0xB77C,
	0xB77D,
	0xB798,
	0xB799,
	0xB7B4,
	0xB7B5,
	0xB7D0,
	0xB7D1,
	0xB7EC,
	0xB7ED,
	0xB808,
	0xB809,
	0xB824,
	0xB825,
	0xB840,
	0xB841,
	0xB85C,
	0xB85D,
	0xB878,
	0xB879,
	0xB894,
	0xB895,
	0xB8B0,
	0xB8B1,
	0xB8CC,
	0xB8CD,
	0xB8E8,
	0xB8E9,
	0xB904,
	0xB905,
	0xB920,
	0xB921,
	0xB93C,
	0xB93D,
	0xB958,
	0xB959,
	0xB974,
	0xB975,
	0xB990,
	0xB991,
	0xB9AC,
	0xB9AD,
	0xB9C8,
	0xB9C9,
	0xB9E4,
	0xB9E5,
	0xBA00,
	0xBA01,
	0xBA1C,
	0xBA1D,
	0xBA38,
	0xBA39,
	0xBA54,
	0xBA55,
	0xBA70,
	0xBA71,
	0xBA8C,
	0xBA8D,
	0xBAA8,
	0xBAA9,
	0xBAC4,
	0xBAC5,
	0xBAE0,
	0xBAE1,
	0xBAFC,
	0xBAFD,
	0xBB18,
	0xBB19,
	0xBB34,
	0xBB35,
	0xBB50,
	0xBB51,
	0xBB6C,
	0xBB6D,
	0xBB88,
	0xBB89,
	0xBBA4,
	0xBBA5,
	0xBBC0,
	0xBBC1,
	0xBBDC,
	0xBBDD,
	0xBBF8,
	0xBBF9,
	0xBC14,
	0xBC15,
	0xBC30,
	0xBC31,
	0xBC4C,
	0xBC4D,
	0xBC68,
	0xBC69,
	0xBC84,
	0xBC85,
	0xBCA0,
	0xBCA1,
	0xBCBC,
	0xBCBD,
	0xBCD8,
	0xBCD9,
	0xBCF4,
	0xBCF5,
	0xBD10,
	0xBD11,
	0xBD2C,
	0xBD2D,
	0xBD48,
	0xBD49,
	0xBD64,
	0xBD65,
	0xBD80,
	0xBD81,
	0xBD9C,
	0xBD9D,
	0xBDB8,
	0xBDB9,
	0xBDD4,
	0xBDD5,
	0xBDF0,
	0xBDF1,
	0xBE0C,
	0xBE0D,
	0xBE28,
	0xBE29,
	0xBE44,
	0xBE45,
	0xBE60,
	0xBE61,
	0xBE7C,
	0xBE7D,
	0xBE98,
	0xBE99,
	0xBEB4,
	0xBEB5,
	0xBED0,
	0xBED1,
	0xBEEC,
	0xBEED,
	0xBF08,
	0xBF09,
	0xBF24,
	0xBF25,
	0xBF40,
	0xBF41,
	0xBF5C,
	0xBF5D,
	0xBF78,
	0xBF79,
	0xBF94,
	0xBF95,
	0xBFB0,
	0xBFB1,
	0xBFCC,
	0xBFCD,
	0xBFE8,
	0xBFE9,
	0xC004,
	0xC005,
	0xC020,
	0xC021,
	0xC03C,
	0xC03D,
	0xC058,
	0xC059,
	0xC074,
	0xC075,
	0xC090,
	0xC091,
	0xC0AC,
	0xC0AD,
	0xC0C8,
	0xC0C9,
	0xC0E4,
	0xC0E5,
	0xC100,
	0xC101,
	0xC11C,
	0xC11D,
	0xC138,
	0xC139,
	0xC154,
	0xC155,
	0xC170,
	0xC171,
	0xC18C,
	0xC18D,
	0xC1A8,
	0xC1A9,
	0xC1C4,
	0xC1C5,
	0xC1E0,
	0xC1E1,
	0xC1FC,
	0xC1FD,
	0xC218,
	0xC219,
	0xC234,
	0xC235,
	0xC250,
	0xC251,
	0xC26C,
	0xC26D,
	0xC288,
	0xC289,
	0xC2A4,
	0xC2A5,
	0xC2C0,
	0xC2C1,
	0xC2DC,
	0xC2DD,
	0xC2F8,
	0xC2F9,
	0xC314,
	0xC315,
	0xC330,
	0xC331,
	0xC34C,
	0xC34D,
	0xC368,
	0xC369,
	0xC384,
	0xC385,
	0xC3A0,
	0xC3A1,
	0xC3BC,
	0xC3BD,
	0xC3D8,
	0xC3D9,
	0xC3F4,
	0xC3F5,
	0xC410,
	0xC411,
	0xC42C,
	0xC42D,
	0xC448,
	0xC449,
	0xC464,
	0xC465,
	0xC480,
	0xC481,
	0xC49C,
	0xC49D,
	0xC4B8,
	0xC4B9,
	0xC4D4,
	0xC4D5,
	0xC4F0,
	0xC4F1,
	0xC50C,
	0xC50D,
	0xC528,
	0xC529,
	0xC544,
	0xC545,
	0xC560,
	0xC561,
	0xC57C,
	0xC57D,
	0xC598,
	0xC599,
	0xC5B4,
	0xC5B5,
	0xC5D0,
	0xC5D1,
	0xC5EC,
	0xC5ED,
	0xC608,
	0xC609,
	0xC624,
	0xC625,
	0xC640,
	0xC641,
	0xC65C,
	0xC65D,
	0xC678,
	0xC679,
	0xC694,
	0xC695,
	0xC6B0,
	0xC6B1,
	0xC6CC,
	0xC6CD,
	0xC6E8,
	0xC6E9,
	0xC704,
	0xC705,
	0xC720,
	0xC721,
	0xC73C,
	0xC73D,
	0xC758,
	0xC759,
	0xC774,
	0xC775,
	0xC790,
	0xC791,
	0xC7AC,
	0xC7AD,
	0xC7C8,
	0xC7C9,
	0xC7E4,
	0xC7E5,
	0xC800,
	0xC801,
	0xC81C,
	0xC81D,
	0xC838,
	0xC839,
	0xC854,
	0xC855,
	0xC870,
	0xC871,
	0xC88C,
	0xC88D,
	0xC8A8,
	0xC8A9,
	0xC8C4,
	0xC8C5,
	0xC8E0,
	0xC8E1,
	0xC8FC,
	0xC8FD,
	0xC918,
	0xC919,
	0xC934,
	0xC935,
	0xC950,
	0xC951,
	0xC96C,
	0xC96D,
	0xC988,
	0xC989,
	0xC9A4,
	0xC9A5,
	0xC9C0,
	0xC9C1,
	0xC9DC,
	0xC9DD,
	0xC9F8,
	0xC9F9,
	0xCA14,
	0xCA15,
	0xCA30,
	0xCA31,
	0xCA4C,
	0xCA4D,
	0xCA68,
	0xCA69,
	0xCA84,
	0xCA85,
	0xCAA0,
	0xCAA1,
	0xCABC,
	0xCABD,
	0xCAD8,
	0xCAD9,
	0xCAF4,
	0xCAF5,
	0xCB10,
	0xCB11,
	0xCB2C,
	0xCB2D,
	0xCB48,
	0xCB49,
	0xCB64,
	0xCB65,
	0xCB80,
	0xCB81,
	0xCB9C,
	0xCB9D,
	0xCBB8,
	0xCBB9,
	0xCBD4,
	0xCBD5,
	0xCBF0,
	0xCBF1,
	0xCC0C,
	0xCC0D,
	0xCC28,
	0xCC29,
	0xCC44,
	0xCC45,
	0xCC60,
	0xCC61,
	0xCC7C,
	0xCC7D,
	0xCC98,
	0xCC99,
	0xCCB4,
	0xCCB5,
	0xCCD0,
	0xCCD1,
	0xCCEC,
	0xCCED,
	0xCD08,
	0xCD09,
	0xCD24,
	0xCD25,
	0xCD40,
	0xCD41,
	0xCD5C,
	0xCD5D,
	0xCD78,
	0xCD79,
	0xCD94,
	0xCD95,
	0xCDB0,
	0xCDB1,
	0xCDCC,
	0xCDCD,
	0xCDE8,
	0xCDE9,
	0xCE04,
	0xCE05,
	0xCE20,
	0xCE21,
	0xCE3C,
	0xCE3D,
	0xCE58,
	0xCE59,
	0xCE74,
	0xCE75,
	0xCE90,
	0xCE91,
	0xCEAC,
	0xCEAD,
	0xCEC8,
	0xCEC9,
	0xCEE4,
	0xCEE5,
	0xCF00,
	0xCF01,
	0xCF1C,
	0xCF1D,
	0xCF38,
	0xCF39,
	0xCF54,
	0xCF55,
	0xCF70,
	0xCF71,
	0xCF8C,
	0xCF8D,
	0xCFA8,
	0xCFA9,
	0xCFC4,
	0xCFC5,
	0xCFE0,
	0xCFE1,
	0xCFFC,
	0xCFFD,
	0xD018,
	0xD019,
	0xD034,
	0xD035,
	0xD050,
	0xD051,
	0xD06C,
	0xD06D,
	0xD088,
	0xD089,
	0xD0A4,
	0xD0A5,
	0xD0C0,
	0xD0C1,
	0xD0DC,
	0xD0DD,
	0xD0F8,
	0xD0F9,
	0xD114,
	0xD115,
	0xD130,
	0xD131,
	0xD14C,
	0xD14D,
	0xD168,
	0xD169,
	0xD184,
	0xD185,
	0xD1A0,
	0xD1A1,
	0xD1BC,
	0xD1BD,
	0xD1D8,
	0xD1D9,
	0xD1F4,
	0xD1F5,
	0xD210,
	0xD211,
	0xD22C,
	0xD22D,
	0xD248,
	0xD249,
	0xD264,
	0xD265,
	0xD280,
	0xD281,
	0xD29C,
	0xD29D,
	0xD2B8,
	0xD2B9,
	0xD2D4,
	0xD2D5,
	0xD2F0,
	0xD2F1,
	0xD30C,
	0xD30D,
	0xD328,
	0xD329,
	0xD344,
	0xD345,
	0xD360,
	0xD361,
	0xD37C,
	0xD37D,
	0xD398,
	0xD399,
	0xD3B4,
	0xD3B5,
	0xD3D0,
	0xD3D1,
	0xD3EC,
	0xD3ED,
	0xD408,
	0xD409,
	0xD424,
	0xD425,
	0xD440,
	0xD441,
	0xD45C,
	0xD45D,
	0xD478,
	0xD479,
	0xD494,
	0xD495,
	0xD4B0,
	0xD4B1,
	0xD4CC,
	0xD4CD,
	0xD4E8,
	0xD4E9,
	0xD504,
	0xD505,
	0xD520,
	0xD521,
	0xD53C,
	0xD53D,
	0xD558,
	0xD559,
	0xD574,
	0xD575,
	0xD590,
	0xD591,
	0xD5AC,
	0xD5AD,
	0xD5C8,
	0xD5C9,
	0xD5E4,
	0xD5E5,
	0xD600,
	0xD601,
	0xD61C,
	0xD61D,
	0xD638,
	0xD639,
	0xD654,
	0xD655,
	0xD670,
	0xD671,
	0xD68C,
	0xD68D,
	0xD6A8,
	0xD6A9,
	0xD6C4,
	0xD6C5,
	0xD6E0,
	0xD6E1,
	0xD6FC,
	0xD6FD,
	0xD718,
	0xD719,
	0xD734,
	0xD735,
	0xD750,
	0xD751,
	0xD76C,
	0xD76D,
	0xD788,
	0xD789,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xD800,
	0xE000,
	0xFB1E,
	0xFB1F,
	0xFE00,
	0xFE10,
	0xFE20,
	0xFE30,
	0xFEFF,
	0xFF00,
	0xFF9E,
	0xFFA0,
	0xFFF0,
	0xFFFC,
	0x101FD,
	0x101FE,
	0x102E0,
	0x102E1,
	0x10376,
	0x1037B,
	0x10A01,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A10,
	0x10A38,
	0x10A3B,
	0x10A3F,
	0x10A40,
	0x10AE5,
	0x10AE7,
	0x11000,
	0x11001,
	0x11002,
	0x11003,
	0x11038,
	0x11047,
	0x1107F,
	0x11082,
	0x11083,
	0x110B0,
	0x110B3,
	0x110B7,
	0x110B9,
	0x110BB,
	0x110BD,
	0x110BE,
	0x11100,
	0x11103,
	0x11127,
	0x1112C,
	0x1112D,
	0x11135,
	0x11173,
	0x11174,
	0x11180,
	0x11182,
	0x11183,
	0x111B3,
	0x111B6,
	0x111BF,
	0x111C1,
	0x111C2,
	0x111C4,
	0x111CA,
	0x111CD,
	0x1122C,
	0x1122F,
	0x11232,
	0x11234,
	0x11235,
	0x11236,
	0x11238,
	0x1123E,
	0x1123F,
	0x112DF,
	0x112E0,
	0x112E3,
	0x112EB,
	0x11300,
	0x11302,
	0x11304,
	0x1133C,
	0x1133D,
	0x1133E,
	0x1133F,
	0x11340,
	0x11341,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134E,
	0x11357,
	0x11358,
	0x11362,
	0x11364,
	0x11366,
	0x1136D,
	0x11370,
	0x11375,
	0x11435,
	0x11438,
	0x11440,
	0x11442,
	0x11445,
	0x11446,
	0x11447,
	0x114B0,
	0x114B1,
	0x114B3,
	0x114B9,
	0x114BA,
	0x114BB,
	0x114BD,
	0x114BE,
	0x114BF,
	0x114C1,
	0x114C2,
	0x114C4,
	0x115AF,
	0x115B0,
	0x115B2,
	0x115B6,
	0x115B8,
	0x115BC,
	0x115BE,
	0x115BF,
	0x115C1,
	0x115DC,
	0x115DE,
	0x11630,
	0x11633,
	0x1163B,
	0x1163D,
	0x1163E,
	0x1163F,
	0x11641,
	0x116AB,
	0x116AC,
	0x116AD,
	0x116AE,
	0x116B0,
	0x116B6,
	0x116B7,
	0x116B8,
	0x1171D,
	0x11720,
	0x11722,
	0x11726,
	0x11727,
	0x1172C,
	0x11C2F,
	0x11C30,
	0x11C37,
	0x11C38,
	0x11C3E,
	0x11C3F,
	0x11C40,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CAA,
	0x11CB1,
	0x11CB2,
	0x11CB4,
	0x11CB5,
	0x11CB7,
	0x16AF0,
	0x16AF5,
	0x16B30,
	0x16B37,
	0x16F51,
	0x16F7F,
	0x16F8F,
	0x16F93,
	0x1BC9D,
	0x1BC9F,
	0x1BCA0,
	0x1BCA4,
	0x1D165,
	0x1D166,
	0x1D167,
	0x1D16A,
	0x1D16D,
	0x1D16E,
	0x1D173,
	0x1D17B,
	0x1D183,
	0x1D185,
	0x1D18C,
	0x1D1AA,
	0x1D1AE,
	0x1D242,
	0x1D245,
	0x1DA00,
	0x1DA37,
	0x1DA3B,
	0x1DA6D,
	0x1DA75,
	0x1DA76,
	0x1DA84,
	0x1DA85,
	0x1DA9B,
	0x1DAA0,
	0x1DAA1,
	0x1DAB0,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E8D0,
	0x1E8D7,
	0x1E944,
	0x1E94B,
	0x1F1E6,
	0x1F200,
	0x1F385,
	0x1F386,
	0x1F3C3,
	0x1F3C5,
	0x1F3CA,
	0x1F3CC,
	0x1F3FB,
	0x1F400,
	0x1F442,
	0x1F444,
	0x1F446,
	0x1F451,
	0x1F466,
	0x1F46A,
	0x1F46E,
	0x1F46F,
	0x1F470,
	0x1F479,
	0x1F47C,
	0x1F47D,
	0x1F481,
	0x1F484,
	0x1F485,
	0x1F488,
	0x1F48B,
	0x1F48C,
	0x1F4AA,
	0x1F4AB,
	0x1F575,
	0x1F576,
	0x1F57A,
	0x1F57B,
	0x1F590,
	0x1F591,
	0x1F595,
	0x1F597,
	0x1F5E8,
	0x1F5E9,
	0x1F645,
	0x1F648,
	0x1F64B,
	0x1F650,
	0x1F6A3,
	0x1F6A4,
	0x1F6B4,
	0x1F6B7,
	0x1F6C0,
	0x1F6C1,
	0x1F918,
	0x1F91F,
	0x1F926,
	0x1F927,
	0x1F930,
	0x1F931,
	0x1F933,
	0x1F93A,
	0x1F93C,
	0x1F93F,
	0xE0000,
	0xE0020,
	0xE0080,
	0xE0100,
	0xE01F0,
	0xE1000
};

#endif	/* defined(PERL_IN_PERL_C) */

#if defined(PERL_IN_REGEXEC_C)

#define GCB_ENUM_COUNT 19

typedef enum {
	GCB_Other = 0,
	GCB_CR = 1,
	GCB_Control = 2,
	GCB_E_Base = 3,
	GCB_E_Base_GAZ = 4,
	GCB_E_Modifier = 5,
	GCB_Extend = 6,
	GCB_Glue_After_Zwj = 7,
	GCB_L = 8,
	GCB_LF = 9,
	GCB_LV = 10,
	GCB_LVT = 11,
	GCB_Prepend = 12,
	GCB_Regional_Indicator = 13,
	GCB_SpacingMark = 14,
	GCB_T = 15,
	GCB_V = 16,
	GCB_ZWJ = 17,
	GCB_EDGE = 18
} GCB_enum;

static const GCB_enum _Perl_GCB_invmap[] = { /* for EBCDIC 1047 */
	GCB_Control,
	GCB_CR,
	GCB_Control,
	GCB_LF,
	GCB_Control,
	GCB_Other,
	GCB_Control,
	GCB_Other,
	GCB_Control,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Prepend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Control,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Prepend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Prepend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Prepend,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Prepend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_L,
	GCB_V,
	GCB_T,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Control,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Control,
	GCB_Extend,
	GCB_ZWJ,
	GCB_Control,
	GCB_Other,
	GCB_Control,
	GCB_Other,
	GCB_Control,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_Glue_After_Zwj,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_L,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_Other,
	GCB_V,
	GCB_Other,
	GCB_T,
	GCB_Other,
	GCB_Control,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Control,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Control,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Prepend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Prepend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Control,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Control,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Regional_Indicator,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Modifier,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base_GAZ,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_Glue_After_Zwj,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_Glue_After_Zwj,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_Control,
	GCB_Extend,
	GCB_Control,
	GCB_Extend,
	GCB_Control,
	GCB_Other
};

#endif	/* defined(PERL_IN_REGEXEC_C) */

#if defined(PERL_IN_UTF8_C)

static const UV _Perl_IDCont_invlist[] = { /* for EBCDIC 1047 */
	1391,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x42,
	0x4A,
	0x51,
	0x5A,
	0x62,
	0x6A,
	0x6D,
	0x6E,
	0x70,
	0x79,
	0x80,
	0x8A,
	0x8C,
	0x8F,
	0x91,
	0x9D,
	0x9E,
	0x9F,
	0xA0,
	0xA1,
	0xA2,
	0xAA,
	0xAC,
	0xAD,
	0xAE,
	0xAF,
	0xBA,
	0xBB,
	0xC1,
	0xCA,
	0xCB,
	0xD0,
	0xD1,
	0xDA,
	0xDB,
	0xE0,
	0xE2,
	0xEA,
	0xEB,
	0xFA,
	0xFB,
	0xFF,
	0x100,
	0x2C2,
	0x2C6,
	0x2D2,
	0x2E0,
	0x2E5,
	0x2EC,
	0x2ED,
	0x2EE,
	0x2EF,
	0x300,
	0x375,
	0x376,
	0x378,
	0x37B,
	0x37E,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x3F6,
	0x3F7,
	0x482,
	0x483,
	0x488,
	0x48A,
	0x530,
	0x531,
	0x557,
	0x559,
	0x55A,
	0x561,
	0x588,
	0x591,
	0x5BE,
	0x5BF,
	0x5C0,
	0x5C1,
	0x5C3,
	0x5C4,
	0x5C6,
	0x5C7,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F3,
	0x610,
	0x61B,
	0x620,
	0x66A,
	0x66E,
	0x6D4,
	0x6D5,
	0x6DD,
	0x6DF,
	0x6E9,
	0x6EA,
	0x6FD,
	0x6FF,
	0x700,
	0x710,
	0x74B,
	0x74D,
	0x7B2,
	0x7C0,
	0x7F6,
	0x7FA,
	0x7FB,
	0x800,
	0x82E,
	0x840,
	0x85C,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x8D4,
	0x8E2,
	0x8E3,
	0x964,
	0x966,
	0x970,
	0x971,
	0x984,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BC,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CF,
	0x9D7,
	0x9D8,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E4,
	0x9E6,
	0x9F2,
	0xA01,
	0xA04,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA3C,
	0xA3D,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4E,
	0xA51,
	0xA52,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA66,
	0xA76,
	0xA81,
	0xA84,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABC,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACE,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE4,
	0xAE6,
	0xAF0,
	0xAF9,
	0xAFA,
	0xB01,
	0xB04,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3C,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4E,
	0xB56,
	0xB58,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB64,
	0xB66,
	0xB70,
	0xB71,
	0xB72,
	0xB82,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCE,
	0xBD0,
	0xBD1,
	0xBD7,
	0xBD8,
	0xBE6,
	0xBF0,
	0xC00,
	0xC04,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4E,
	0xC55,
	0xC57,
	0xC58,
	0xC5B,
	0xC60,
	0xC64,
	0xC66,
	0xC70,
	0xC80,
	0xC84,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBC,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCE,
	0xCD5,
	0xCD7,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE4,
	0xCE6,
	0xCF0,
	0xCF1,
	0xCF3,
	0xD01,
	0xD04,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD4F,
	0xD54,
	0xD58,
	0xD5F,
	0xD64,
	0xD66,
	0xD70,
	0xD7A,
	0xD80,
	0xD82,
	0xD84,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xDCA,
	0xDCB,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDE6,
	0xDF0,
	0xDF2,
	0xDF4,
	0xE01,
	0xE3B,
	0xE40,
	0xE4F,
	0xE50,
	0xE5A,
	0xE81,
	0xE83,
	0xE84,
	0xE85,
	0xE87,
	0xE89,
	0xE8A,
	0xE8B,
	0xE8D,
	0xE8E,
	0xE94,
	0xE98,
	0xE99,
	0xEA0,
	0xEA1,
	0xEA4,
	0xEA5,
	0xEA6,
	0xEA7,
	0xEA8,
	0xEAA,
	0xEAC,
	0xEAD,
	0xEBA,
	0xEBB,
	0xEBE,
	0xEC0,
	0xEC5,
	0xEC6,
	0xEC7,
	0xEC8,
	0xECE,
	0xED0,
	0xEDA,
	0xEDC,
	0xEE0,
	0xF00,
	0xF01,
	0xF18,
	0xF1A,
	0xF20,
	0xF2A,
	0xF35,
	0xF36,
	0xF37,
	0xF38,
	0xF39,
	0xF3A,
	0xF3E,
	0xF48,
	0xF49,
	0xF6D,
	0xF71,
	0xF85,
	0xF86,
	0xF98,
	0xF99,
	0xFBD,
	0xFC6,
	0xFC7,
	0x1000,
	0x104A,
	0x1050,
	0x109E,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x10FB,
	0x10FC,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x135D,
	0x1360,
	0x1380,
	0x1390,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1401,
	0x166D,
	0x166F,
	0x1680,
	0x1681,
	0x169B,
	0x16A0,
	0x16EB,
	0x16EE,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1715,
	0x1720,
	0x1735,
	0x1740,
	0x1754,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1772,
	0x1774,
	0x1780,
	0x17D4,
	0x17D7,
	0x17D8,
	0x17DC,
	0x17DE,
	0x17E0,
	0x17EA,
	0x180B,
	0x180E,
	0x1810,
	0x181A,
	0x1820,
	0x1878,
	0x1880,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1920,
	0x192C,
	0x1930,
	0x193C,
	0x1946,
	0x196E,
	0x1970,
	0x1975,
	0x1980,
	0x19AC,
	0x19B0,
	0x19CA,
	0x19D0,
	0x19DA,
	0x1A00,
	0x1A1C,
	0x1A20,
	0x1A5F,
	0x1A60,
	0x1A7D,
	0x1A7F,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1AA7,
	0x1AA8,
	0x1AB0,
	0x1ABE,
	0x1B00,
	0x1B4C,
	0x1B50,
	0x1B5A,
	0x1B6B,
	0x1B74,
	0x1B80,
	0x1BF4,
	0x1C00,
	0x1C38,
	0x1C40,
	0x1C4A,
	0x1C4D,
	0x1C7E,
	0x1C80,
	0x1C89,
	0x1CD0,
	0x1CD3,
	0x1CD4,
	0x1CF7,
	0x1CF8,
	0x1CFA,
	0x1D00,
	0x1DF6,
	0x1DFB,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FBD,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FCD,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FE0,
	0x1FED,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFD,
	0x203F,
	0x2041,
	0x2054,
	0x2055,
	0x2071,
	0x2072,
	0x207F,
	0x2080,
	0x2090,
	0x209D,
	0x20D0,
	0x20DD,
	0x20E1,
	0x20E2,
	0x20E5,
	0x20F1,
	0x2102,
	0x2103,
	0x2107,
	0x2108,
	0x210A,
	0x2114,
	0x2115,
	0x2116,
	0x2119,
	0x211E,
	0x2124,
	0x2125,
	0x2126,
	0x2127,
	0x2128,
	0x2129,
	0x212A,
	0x212E,
	0x212F,
	0x213A,
	0x213C,
	0x2140,
	0x2145,
	0x214A,
	0x214E,
	0x214F,
	0x2160,
	0x2189,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CE5,
	0x2CEB,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D70,
	0x2D7F,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x2DE0,
	0x2E00,
	0x3005,
	0x3008,
	0x3021,
	0x3030,
	0x3031,
	0x3036,
	0x3038,
	0x303D,
	0x3041,
	0x3097,
	0x3099,
	0x309B,
	0x309D,
	0x30A0,
	0x30A1,
	0x30FB,
	0x30FC,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x31A0,
	0x31BB,
	0x31F0,
	0x3200,
	0x3400,
	0x4DB6,
	0x4E00,
	0x9FD6,
	0xA000,
	0xA48D,
	0xA4D0,
	0xA4FE,
	0xA500,
	0xA60D,
	0xA610,
	0xA62C,
	0xA640,
	0xA670,
	0xA674,
	0xA67E,
	0xA67F,
	0xA6F2,
	0xA717,
	0xA720,
	0xA722,
	0xA789,
	0xA78B,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F7,
	0xA828,
	0xA840,
	0xA874,
	0xA880,
	0xA8C6,
	0xA8D0,
	0xA8DA,
	0xA8E0,
	0xA8F8,
	0xA8FB,
	0xA8FC,
	0xA8FD,
	0xA8FE,
	0xA900,
	0xA92E,
	0xA930,
	0xA954,
	0xA960,
	0xA97D,
	0xA980,
	0xA9C1,
	0xA9CF,
	0xA9DA,
	0xA9E0,
	0xA9FF,
	0xAA00,
	0xAA37,
	0xAA40,
	0xAA4E,
	0xAA50,
	0xAA5A,
	0xAA60,
	0xAA77,
	0xAA7A,
	0xAAC3,
	0xAADB,
	0xAADE,
	0xAAE0,
	0xAAF0,
	0xAAF2,
	0xAAF7,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB5B,
	0xAB5C,
	0xAB66,
	0xAB70,
	0xABEB,
	0xABEC,
	0xABEE,
	0xABF0,
	0xABFA,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xF900,
	0xFA6E,
	0xFA70,
	0xFADA,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB29,
	0xFB2A,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFBB2,
	0xFBD3,
	0xFC5E,
	0xFC64,
	0xFD3E,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFA,
	0xFE00,
	0xFE10,
	0xFE20,
	0xFE30,
	0xFE33,
	0xFE35,
	0xFE4D,
	0xFE50,
	0xFE71,
	0xFE72,
	0xFE73,
	0xFE74,
	0xFE77,
	0xFE78,
	0xFE79,
	0xFE7A,
	0xFE7B,
	0xFE7C,
	0xFE7D,
	0xFE7E,
	0xFE7F,
	0xFEFD,
	0xFF10,
	0xFF1A,
	0xFF21,
	0xFF3B,
	0xFF3F,
	0xFF40,
	0xFF41,
	0xFF5B,
	0xFF66,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10140,
	0x10175,
	0x101FD,
	0x101FE,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x102E0,
	0x102E1,
	0x10300,
	0x10320,
	0x10330,
	0x1034B,
	0x10350,
	0x1037B,
	0x10380,
	0x1039E,
	0x103A0,
	0x103C4,
	0x103C8,
	0x103D0,
	0x103D1,
	0x103D6,
	0x10400,
	0x1049E,
	0x104A0,
	0x104AA,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10860,
	0x10877,
	0x10880,
	0x1089F,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x10900,
	0x10916,
	0x10920,
	0x1093A,
	0x10980,
	0x109B8,
	0x109BE,
	0x109C0,
	0x10A00,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A38,
	0x10A3B,
	0x10A3F,
	0x10A40,
	0x10A60,
	0x10A7D,
	0x10A80,
	0x10A9D,
	0x10AC0,
	0x10AC8,
	0x10AC9,
	0x10AE7,
	0x10B00,
	0x10B36,
	0x10B40,
	0x10B56,
	0x10B60,
	0x10B73,
	0x10B80,
	0x10B92,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x11000,
	0x11047,
	0x11066,
	0x11070,
	0x1107F,
	0x110BB,
	0x110D0,
	0x110E9,
	0x110F0,
	0x110FA,
	0x11100,
	0x11135,
	0x11136,
	0x11140,
	0x11150,
	0x11174,
	0x11176,
	0x11177,
	0x11180,
	0x111C5,
	0x111CA,
	0x111CD,
	0x111D0,
	0x111DB,
	0x111DC,
	0x111DD,
	0x11200,
	0x11212,
	0x11213,
	0x11238,
	0x1123E,
	0x1123F,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112A9,
	0x112B0,
	0x112EB,
	0x112F0,
	0x112FA,
	0x11300,
	0x11304,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133C,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134E,
	0x11350,
	0x11351,
	0x11357,
	0x11358,
	0x1135D,
	0x11364,
	0x11366,
	0x1136D,
	0x11370,
	0x11375,
	0x11400,
	0x1144B,
	0x11450,
	0x1145A,
	0x11480,
	0x114C6,
	0x114C7,
	0x114C8,
	0x114D0,
	0x114DA,
	0x11580,
	0x115B6,
	0x115B8,
	0x115C1,
	0x115D8,
	0x115DE,
	0x11600,
	0x11641,
	0x11644,
	0x11645,
	0x11650,
	0x1165A,
	0x11680,
	0x116B8,
	0x116C0,
	0x116CA,
	0x11700,
	0x1171A,
	0x1171D,
	0x1172C,
	0x11730,
	0x1173A,
	0x118A0,
	0x118EA,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C37,
	0x11C38,
	0x11C41,
	0x11C50,
	0x11C5A,
	0x11C72,
	0x11C90,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16A60,
	0x16A6A,
	0x16AD0,
	0x16AEE,
	0x16AF0,
	0x16AF5,
	0x16B00,
	0x16B37,
	0x16B40,
	0x16B44,
	0x16B50,
	0x16B5A,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F7F,
	0x16F8F,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1BC9D,
	0x1BC9F,
	0x1D165,
	0x1D16A,
	0x1D16D,
	0x1D173,
	0x1D17B,
	0x1D183,
	0x1D185,
	0x1D18C,
	0x1D1AA,
	0x1D1AE,
	0x1D242,
	0x1D245,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D6C1,
	0x1D6C2,
	0x1D6DB,
	0x1D6DC,
	0x1D6FB,
	0x1D6FC,
	0x1D715,
	0x1D716,
	0x1D735,
	0x1D736,
	0x1D74F,
	0x1D750,
	0x1D76F,
	0x1D770,
	0x1D789,
	0x1D78A,
	0x1D7A9,
	0x1D7AA,
	0x1D7C3,
	0x1D7C4,
	0x1D7CC,
	0x1D7CE,
	0x1D800,
	0x1DA00,
	0x1DA37,
	0x1DA3B,
	0x1DA6D,
	0x1DA75,
	0x1DA76,
	0x1DA84,
	0x1DA85,
	0x1DA9B,
	0x1DAA0,
	0x1DAA1,
	0x1DAB0,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E800,
	0x1E8C5,
	0x1E8D0,
	0x1E8D7,
	0x1E900,
	0x1E94B,
	0x1E950,
	0x1E95A,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x20000,
	0x2A6D7,
	0x2A700,
	0x2B735,
	0x2B740,
	0x2B81E,
	0x2B820,
	0x2CEA2,
	0x2F800,
	0x2FA1E,
	0xE0100,
	0xE01F0
};

static const UV _Perl_IDStart_invlist[] = { /* for EBCDIC 1047 */
	1187,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x42,
	0x4A,
	0x51,
	0x5A,
	0x62,
	0x6A,
	0x6D,
	0x6E,
	0x70,
	0x79,
	0x80,
	0x8A,
	0x8C,
	0x8F,
	0x91,
	0x9D,
	0x9E,
	0x9F,
	0xA0,
	0xA1,
	0xA2,
	0xAA,
	0xAC,
	0xAD,
	0xAE,
	0xAF,
	0xBA,
	0xBB,
	0xC1,
	0xCA,
	0xCB,
	0xD0,
	0xD1,
	0xDA,
	0xDB,
	0xE0,
	0xE2,
	0xEA,
	0xEB,
	0xF0,
	0xFB,
	0xFF,
	0x100,
	0x2C2,
	0x2C6,
	0x2D2,
	0x2E0,
	0x2E5,
	0x2EC,
	0x2ED,
	0x2EE,
	0x2EF,
	0x370,
	0x375,
	0x376,
	0x378,
	0x37B,
	0x37E,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x3F6,
	0x3F7,
	0x482,
	0x48A,
	0x530,
	0x531,
	0x557,
	0x559,
	0x55A,
	0x561,
	0x588,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F3,
	0x620,
	0x64B,
	0x66E,
	0x670,
	0x671,
	0x6D4,
	0x6D5,
	0x6D6,
	0x6E5,
	0x6E7,
	0x6EE,
	0x6F0,
	0x6FA,
	0x6FD,
	0x6FF,
	0x700,
	0x710,
	0x711,
	0x712,
	0x730,
	0x74D,
	0x7A6,
	0x7B1,
	0x7B2,
	0x7CA,
	0x7EB,
	0x7F4,
	0x7F6,
	0x7FA,
	0x7FB,
	0x800,
	0x816,
	0x81A,
	0x81B,
	0x824,
	0x825,
	0x828,
	0x829,
	0x840,
	0x859,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x904,
	0x93A,
	0x93D,
	0x93E,
	0x950,
	0x951,
	0x958,
	0x962,
	0x971,
	0x981,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BD,
	0x9BE,
	0x9CE,
	0x9CF,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E2,
	0x9F0,
	0x9F2,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA72,
	0xA75,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABD,
	0xABE,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE2,
	0xAF9,
	0xAFA,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3D,
	0xB3E,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB62,
	0xB71,
	0xB72,
	0xB83,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBD0,
	0xBD1,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC3E,
	0xC58,
	0xC5B,
	0xC60,
	0xC62,
	0xC80,
	0xC81,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBD,
	0xCBE,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE2,
	0xCF1,
	0xCF3,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD3E,
	0xD4E,
	0xD4F,
	0xD54,
	0xD57,
	0xD5F,
	0xD62,
	0xD7A,
	0xD80,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xE01,
	0xE31,
	0xE32,
	0xE33,
	0xE40,
	0xE47,
	0xE81,
	0xE83,
	0xE84,
	0xE85,
	0xE87,
	0xE89,
	0xE8A,
	0xE8B,
	0xE8D,
	0xE8E,
	0xE94,
	0xE98,
	0xE99,
	0xEA0,
	0xEA1,
	0xEA4,
	0xEA5,
	0xEA6,
	0xEA7,
	0xEA8,
	0xEAA,
	0xEAC,
	0xEAD,
	0xEB1,
	0xEB2,
	0xEB3,
	0xEBD,
	0xEBE,
	0xEC0,
	0xEC5,
	0xEC6,
	0xEC7,
	0xEDC,
	0xEE0,
	0xF00,
	0xF01,
	0xF40,
	0xF48,
	0xF49,
	0xF6D,
	0xF88,
	0xF8D,
	0x1000,
	0x102B,
	0x103F,
	0x1040,
	0x1050,
	0x1056,
	0x105A,
	0x105E,
	0x1061,
	0x1062,
	0x1065,
	0x1067,
	0x106E,
	0x1071,
	0x1075,
	0x1082,
	0x108E,
	0x108F,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x10FB,
	0x10FC,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x1380,
	0x1390,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1401,
	0x166D,
	0x166F,
	0x1680,
	0x1681,
	0x169B,
	0x16A0,
	0x16EB,
	0x16EE,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1712,
	0x1720,
	0x1732,
	0x1740,
	0x1752,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1780,
	0x17B4,
	0x17D7,
	0x17D8,
	0x17DC,
	0x17DD,
	0x1820,
	0x1878,
	0x1880,
	0x18A9,
	0x18AA,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1950,
	0x196E,
	0x1970,
	0x1975,
	0x1980,
	0x19AC,
	0x19B0,
	0x19CA,
	0x1A00,
	0x1A17,
	0x1A20,
	0x1A55,
	0x1AA7,
	0x1AA8,
	0x1B05,
	0x1B34,
	0x1B45,
	0x1B4C,
	0x1B83,
	0x1BA1,
	0x1BAE,
	0x1BB0,
	0x1BBA,
	0x1BE6,
	0x1C00,
	0x1C24,
	0x1C4D,
	0x1C50,
	0x1C5A,
	0x1C7E,
	0x1C80,
	0x1C89,
	0x1CE9,
	0x1CED,
	0x1CEE,
	0x1CF2,
	0x1CF5,
	0x1CF7,
	0x1D00,
	0x1DC0,
	0x1E00,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FBD,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FCD,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FE0,
	0x1FED,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFD,
	0x2071,
	0x2072,
	0x207F,
	0x2080,
	0x2090,
	0x209D,
	0x2102,
	0x2103,
	0x2107,
	0x2108,
	0x210A,
	0x2114,
	0x2115,
	0x2116,
	0x2119,
	0x211E,
	0x2124,
	0x2125,
	0x2126,
	0x2127,
	0x2128,
	0x2129,
	0x212A,
	0x212E,
	0x212F,
	0x213A,
	0x213C,
	0x2140,
	0x2145,
	0x214A,
	0x214E,
	0x214F,
	0x2160,
	0x2189,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CE5,
	0x2CEB,
	0x2CEF,
	0x2CF2,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D70,
	0x2D80,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x3005,
	0x3008,
	0x3021,
	0x302A,
	0x3031,
	0x3036,
	0x3038,
	0x303D,
	0x3041,
	0x3097,
	0x309D,
	0x30A0,
	0x30A1,
	0x30FB,
	0x30FC,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x31A0,
	0x31BB,
	0x31F0,
	0x3200,
	0x3400,
	0x4DB6,
	0x4E00,
	0x9FD6,
	0xA000,
	0xA48D,
	0xA4D0,
	0xA4FE,
	0xA500,
	0xA60D,
	0xA610,
	0xA620,
	0xA62A,
	0xA62C,
	0xA640,
	0xA66F,
	0xA67F,
	0xA69E,
	0xA6A0,
	0xA6F0,
	0xA717,
	0xA720,
	0xA722,
	0xA789,
	0xA78B,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F7,
	0xA802,
	0xA803,
	0xA806,
	0xA807,
	0xA80B,
	0xA80C,
	0xA823,
	0xA840,
	0xA874,
	0xA882,
	0xA8B4,
	0xA8F2,
	0xA8F8,
	0xA8FB,
	0xA8FC,
	0xA8FD,
	0xA8FE,
	0xA90A,
	0xA926,
	0xA930,
	0xA947,
	0xA960,
	0xA97D,
	0xA984,
	0xA9B3,
	0xA9CF,
	0xA9D0,
	0xA9E0,
	0xA9E5,
	0xA9E6,
	0xA9F0,
	0xA9FA,
	0xA9FF,
	0xAA00,
	0xAA29,
	0xAA40,
	0xAA43,
	0xAA44,
	0xAA4C,
	0xAA60,
	0xAA77,
	0xAA7A,
	0xAA7B,
	0xAA7E,
	0xAAB0,
	0xAAB1,
	0xAAB2,
	0xAAB5,
	0xAAB7,
	0xAAB9,
	0xAABE,
	0xAAC0,
	0xAAC1,
	0xAAC2,
	0xAAC3,
	0xAADB,
	0xAADE,
	0xAAE0,
	0xAAEB,
	0xAAF2,
	0xAAF5,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB5B,
	0xAB5C,
	0xAB66,
	0xAB70,
	0xABE3,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xF900,
	0xFA6E,
	0xFA70,
	0xFADA,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB1E,
	0xFB1F,
	0xFB29,
	0xFB2A,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFBB2,
	0xFBD3,
	0xFC5E,
	0xFC64,
	0xFD3E,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFA,
	0xFE71,
	0xFE72,
	0xFE73,
	0xFE74,
	0xFE77,
	0xFE78,
	0xFE79,
	0xFE7A,
	0xFE7B,
	0xFE7C,
	0xFE7D,
	0xFE7E,
	0xFE7F,
	0xFEFD,
	0xFF21,
	0xFF3B,
	0xFF41,
	0xFF5B,
	0xFF66,
	0xFF9E,
	0xFFA0,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10140,
	0x10175,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x10300,
	0x10320,
	0x10330,
	0x1034B,
	0x10350,
	0x10376,
	0x10380,
	0x1039E,
	0x103A0,
	0x103C4,
	0x103C8,
	0x103D0,
	0x103D1,
	0x103D6,
	0x10400,
	0x1049E,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10860,
	0x10877,
	0x10880,
	0x1089F,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x10900,
	0x10916,
	0x10920,
	0x1093A,
	0x10980,
	0x109B8,
	0x109BE,
	0x109C0,
	0x10A00,
	0x10A01,
	0x10A10,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A60,
	0x10A7D,
	0x10A80,
	0x10A9D,
	0x10AC0,
	0x10AC8,
	0x10AC9,
	0x10AE5,
	0x10B00,
	0x10B36,
	0x10B40,
	0x10B56,
	0x10B60,
	0x10B73,
	0x10B80,
	0x10B92,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x11003,
	0x11038,
	0x11083,
	0x110B0,
	0x110D0,
	0x110E9,
	0x11103,
	0x11127,
	0x11150,
	0x11173,
	0x11176,
	0x11177,
	0x11183,
	0x111B3,
	0x111C1,
	0x111C5,
	0x111DA,
	0x111DB,
	0x111DC,
	0x111DD,
	0x11200,
	0x11212,
	0x11213,
	0x1122C,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112A9,
	0x112B0,
	0x112DF,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133D,
	0x1133E,
	0x11350,
	0x11351,
	0x1135D,
	0x11362,
	0x11400,
	0x11435,
	0x11447,
	0x1144B,
	0x11480,
	0x114B0,
	0x114C4,
	0x114C6,
	0x114C7,
	0x114C8,
	0x11580,
	0x115AF,
	0x115D8,
	0x115DC,
	0x11600,
	0x11630,
	0x11644,
	0x11645,
	0x11680,
	0x116AB,
	0x11700,
	0x1171A,
	0x118A0,
	0x118E0,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C2F,
	0x11C40,
	0x11C41,
	0x11C72,
	0x11C90,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16AD0,
	0x16AEE,
	0x16B00,
	0x16B30,
	0x16B40,
	0x16B44,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F51,
	0x16F93,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D6C1,
	0x1D6C2,
	0x1D6DB,
	0x1D6DC,
	0x1D6FB,
	0x1D6FC,
	0x1D715,
	0x1D716,
	0x1D735,
	0x1D736,
	0x1D74F,
	0x1D750,
	0x1D76F,
	0x1D770,
	0x1D789,
	0x1D78A,
	0x1D7A9,
	0x1D7AA,
	0x1D7C3,
	0x1D7C4,
	0x1D7CC,
	0x1E800,
	0x1E8C5,
	0x1E900,
	0x1E944,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x20000,
	0x2A6D7,
	0x2A700,
	0x2B735,
	0x2B740,
	0x2B81E,
	0x2B820,
	0x2CEA2,
	0x2F800,
	0x2FA1E
};

#endif	/* defined(PERL_IN_UTF8_C) */

#if defined(PERL_IN_PERL_C)

static const UV _Perl_LB_invlist[] = { /* for EBCDIC 1047 */
	2279,	/* Number of elements */
	148565664, /* Version and data structure type */
	0,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x5,
	0x6,
	0xB,
	0xD,
	0xE,
	0x15,
	0x16,
	0x25,
	0x26,
	0x40,
	0x41,
	0x42,
	0x4A,
	0x4B,
	0x4C,
	0x4D,
	0x4E,
	0x4F,
	0x50,
	0x5A,
	0x5B,
	0x5C,
	0x5D,
	0x5E,
	0x5F,
	0x60,
	0x61,
	0x62,
	0x6B,
	0x6C,
	0x6D,
	0x6F,
	0x70,
	0x7A,
	0x7B,
	0x7D,
	0x7E,
	0x7F,
	0x80,
	0x8A,
	0x8C,
	0x8F,
	0x90,
	0x91,
	0x9F,
	0xA0,
	0xAA,
	0xAC,
	0xAD,
	0xAE,
	0xB1,
	0xB3,
	0xBD,
	0xBE,
	0xBF,
	0xC0,
	0xC1,
	0xCA,
	0xCB,
	0xD0,
	0xD1,
	0xE0,
	0xE1,
	0xF0,
	0xFA,
	0xFF,
	0x100,
	0x2C8,
	0x2C9,
	0x2CC,
	0x2CD,
	0x2DF,
	0x2E0,
	0x300,
	0x34F,
	0x350,
	0x35C,
	0x363,
	0x370,
	0x37E,
	0x37F,
	0x483,
	0x48A,
	0x589,
	0x58A,
	0x58B,
	0x58F,
	0x590,
	0x591,
	0x5BE,
	0x5BF,
	0x5C0,
	0x5C1,
	0x5C3,
	0x5C4,
	0x5C6,
	0x5C7,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F3,
	0x609,
	0x60C,
	0x60E,
	0x610,
	0x61B,
	0x61C,
	0x61D,
	0x61E,
	0x620,
	0x64B,
	0x660,
	0x66A,
	0x66B,
	0x66D,
	0x670,
	0x671,
	0x6D4,
	0x6D5,
	0x6D6,
	0x6DD,
	0x6DF,
	0x6E5,
	0x6E7,
	0x6E9,
	0x6EA,
	0x6EE,
	0x6F0,
	0x6FA,
	0x711,
	0x712,
	0x730,
	0x74B,
	0x7A6,
	0x7B1,
	0x7C0,
	0x7CA,
	0x7EB,
	0x7F4,
	0x7F8,
	0x7F9,
	0x7FA,
	0x816,
	0x81A,
	0x81B,
	0x824,
	0x825,
	0x828,
	0x829,
	0x82E,
	0x859,
	0x85C,
	0x8D4,
	0x8E2,
	0x8E3,
	0x904,
	0x93A,
	0x93D,
	0x93E,
	0x950,
	0x951,
	0x958,
	0x962,
	0x964,
	0x966,
	0x970,
	0x981,
	0x984,
	0x9BC,
	0x9BD,
	0x9BE,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CE,
	0x9D7,
	0x9D8,
	0x9E2,
	0x9E4,
	0x9E6,
	0x9F0,
	0x9F2,
	0x9F4,
	0x9F9,
	0x9FA,
	0x9FB,
	0x9FC,
	0xA01,
	0xA04,
	0xA3C,
	0xA3D,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4E,
	0xA51,
	0xA52,
	0xA66,
	0xA70,
	0xA72,
	0xA75,
	0xA76,
	0xA81,
	0xA84,
	0xABC,
	0xABD,
	0xABE,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACE,
	0xAE2,
	0xAE4,
	0xAE6,
	0xAF0,
	0xAF1,
	0xAF2,
	0xB01,
	0xB04,
	0xB3C,
	0xB3D,
	0xB3E,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4E,
	0xB56,
	0xB58,
	0xB62,
	0xB64,
	0xB66,
	0xB70,
	0xB82,
	0xB83,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCE,
	0xBD7,
	0xBD8,
	0xBE6,
	0xBF0,
	0xBF9,
	0xBFA,
	0xC00,
	0xC04,
	0xC3E,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4E,
	0xC55,
	0xC57,
	0xC62,
	0xC64,
	0xC66,
	0xC70,
	0xC81,
	0xC84,
	0xCBC,
	0xCBD,
	0xCBE,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCE,
	0xCD5,
	0xCD7,
	0xCE2,
	0xCE4,
	0xCE6,
	0xCF0,
	0xD01,
	0xD04,
	0xD3E,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD4E,
	0xD57,
	0xD58,
	0xD62,
	0xD64,
	0xD66,
	0xD70,
	0xD79,
	0xD7A,
	0xD82,
	0xD84,
	0xDCA,
	0xDCB,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDE6,
	0xDF0,
	0xDF2,
	0xDF4,
	0xE31,
	0xE32,
	0xE34,
	0xE3B,
	0xE3F,
	0xE40,
	0xE47,
	0xE4F,
	0xE50,
	0xE5A,
	0xE5C,
	0xEB1,
	0xEB2,
	0xEB4,
	0xEBA,
	0xEBB,
	0xEBD,
	0xEC8,
	0xECE,
	0xED0,
	0xEDA,
	0xF01,
	0xF05,
	0xF06,
	0xF08,
	0xF09,
	0xF0B,
	0xF0C,
	0xF0D,
	0xF12,
	0xF13,
	0xF14,
	0xF15,
	0xF18,
	0xF1A,
	0xF20,
	0xF2A,
	0xF34,
	0xF35,
	0xF36,
	0xF37,
	0xF38,
	0xF39,
	0xF3A,
	0xF3B,
	0xF3C,
	0xF3D,
	0xF3E,
	0xF40,
	0xF71,
	0xF7F,
	0xF80,
	0xF85,
	0xF86,
	0xF88,
	0xF8D,
	0xF98,
	0xF99,
	0xFBD,
	0xFBE,
	0xFC0,
	0xFC6,
	0xFC7,
	0xFD0,
	0xFD2,
	0xFD3,
	0xFD4,
	0xFD9,
	0xFDB,
	0x102B,
	0x103F,
	0x1040,
	0x104A,
	0x104C,
	0x1056,
	0x105A,
	0x105E,
	0x1061,
	0x1062,
	0x1065,
	0x1067,
	0x106E,
	0x1071,
	0x1075,
	0x1082,
	0x108E,
	0x108F,
	0x1090,
	0x109A,
	0x109E,
	0x1100,
	0x1160,
	0x11A8,
	0x1200,
	0x135D,
	0x1360,
	0x1361,
	0x1362,
	0x1400,
	0x1401,
	0x1680,
	0x1681,
	0x169B,
	0x169C,
	0x169D,
	0x16EB,
	0x16EE,
	0x1712,
	0x1715,
	0x1732,
	0x1735,
	0x1737,
	0x1752,
	0x1754,
	0x1772,
	0x1774,
	0x17B4,
	0x17D4,
	0x17D6,
	0x17D7,
	0x17D8,
	0x17D9,
	0x17DA,
	0x17DB,
	0x17DC,
	0x17DD,
	0x17DE,
	0x17E0,
	0x17EA,
	0x1802,
	0x1804,
	0x1806,
	0x1807,
	0x1808,
	0x180A,
	0x180B,
	0x180E,
	0x180F,
	0x1810,
	0x181A,
	0x1885,
	0x1887,
	0x18A9,
	0x18AA,
	0x1920,
	0x192C,
	0x1930,
	0x193C,
	0x1944,
	0x1946,
	0x1950,
	0x19D0,
	0x19DA,
	0x1A17,
	0x1A1C,
	0x1A55,
	0x1A5F,
	0x1A60,
	0x1A7D,
	0x1A7F,
	0x1A80,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1AB0,
	0x1ABF,
	0x1B00,
	0x1B05,
	0x1B34,
	0x1B45,
	0x1B50,
	0x1B5A,
	0x1B5C,
	0x1B5D,
	0x1B61,
	0x1B6B,
	0x1B74,
	0x1B80,
	0x1B83,
	0x1BA1,
	0x1BAE,
	0x1BB0,
	0x1BBA,
	0x1BE6,
	0x1BF4,
	0x1C24,
	0x1C38,
	0x1C3B,
	0x1C40,
	0x1C4A,
	0x1C50,
	0x1C5A,
	0x1C7E,
	0x1C80,
	0x1CD0,
	0x1CD3,
	0x1CD4,
	0x1CE9,
	0x1CED,
	0x1CEE,
	0x1CF2,
	0x1CF5,
	0x1CF8,
	0x1CFA,
	0x1DC0,
	0x1DF6,
	0x1DFB,
	0x1E00,
	0x1FFD,
	0x1FFE,
	0x2000,
	0x2007,
	0x2008,
	0x200B,
	0x200C,
	0x200D,
	0x200E,
	0x2010,
	0x2011,
	0x2012,
	0x2014,
	0x2015,
	0x2018,
	0x201A,
	0x201B,
	0x201E,
	0x201F,
	0x2020,
	0x2024,
	0x2027,
	0x2028,
	0x202A,
	0x202F,
	0x2030,
	0x2038,
	0x2039,
	0x203B,
	0x203C,
	0x203E,
	0x2044,
	0x2045,
	0x2046,
	0x2047,
	0x204A,
	0x2056,
	0x2057,
	0x2058,
	0x205C,
	0x205D,
	0x2060,
	0x2061,
	0x2066,
	0x2070,
	0x207D,
	0x207E,
	0x207F,
	0x208D,
	0x208E,
	0x208F,
	0x20A0,
	0x20A7,
	0x20A8,
	0x20B6,
	0x20B7,
	0x20BB,
	0x20BC,
	0x20BE,
	0x20BF,
	0x20D0,
	0x20F1,
	0x2103,
	0x2104,
	0x2109,
	0x210A,
	0x2116,
	0x2117,
	0x2212,
	0x2214,
	0x22EF,
	0x22F0,
	0x2308,
	0x2309,
	0x230A,
	0x230B,
	0x230C,
	0x231A,
	0x231C,
	0x2329,
	0x232A,
	0x232B,
	0x23F0,
	0x23F4,
	0x2600,
	0x2604,
	0x2614,
	0x2616,
	0x2618,
	0x2619,
	0x261A,
	0x261D,
	0x261E,
	0x2620,
	0x2639,
	0x263C,
	0x2668,
	0x2669,
	0x267F,
	0x2680,
	0x26BD,
	0x26C9,
	0x26CD,
	0x26CE,
	0x26CF,
	0x26D2,
	0x26D3,
	0x26D5,
	0x26D8,
	0x26DA,
	0x26DC,
	0x26DD,
	0x26DF,
	0x26E2,
	0x26EA,
	0x26EB,
	0x26F1,
	0x26F6,
	0x26F7,
	0x26F9,
	0x26FA,
	0x26FB,
	0x26FD,
	0x2705,
	0x2708,
	0x270A,
	0x270E,
	0x275B,
	0x2761,
	0x2762,
	0x2764,
	0x2765,
	0x2768,
	0x2769,
	0x276A,
	0x276B,
	0x276C,
	0x276D,
	0x276E,
	0x276F,
	0x2770,
	0x2771,
	0x2772,
	0x2773,
	0x2774,
	0x2775,
	0x2776,
	0x27C5,
	0x27C6,
	0x27C7,
	0x27E6,
	0x27E7,
	0x27E8,
	0x27E9,
	0x27EA,
	0x27EB,
	0x27EC,
	0x27ED,
	0x27EE,
	0x27EF,
	0x27F0,
	0x2983,
	0x2984,
	0x2985,
	0x2986,
	0x2987,
	0x2988,
	0x2989,
	0x298A,
	0x298B,
	0x298C,
	0x298D,
	0x298E,
	0x298F,
	0x2990,
	0x2991,
	0x2992,
	0x2993,
	0x2994,
	0x2995,
	0x2996,
	0x2997,
	0x2998,
	0x2999,
	0x29D8,
	0x29D9,
	0x29DA,
	0x29DB,
	0x29DC,
	0x29FC,
	0x29FD,
	0x29FE,
	0x2CEF,
	0x2CF2,
	0x2CF9,
	0x2CFA,
	0x2CFD,
	0x2CFE,
	0x2CFF,
	0x2D00,
	0x2D70,
	0x2D71,
	0x2D7F,
	0x2D80,
	0x2DE0,
	0x2E00,
	0x2E0E,
	0x2E16,
	0x2E17,
	0x2E18,
	0x2E19,
	0x2E1A,
	0x2E1C,
	0x2E1E,
	0x2E20,
	0x2E22,
	0x2E23,
	0x2E24,
	0x2E25,
	0x2E26,
	0x2E27,
	0x2E28,
	0x2E29,
	0x2E2A,
	0x2E2E,
	0x2E2F,
	0x2E30,
	0x2E32,
	0x2E33,
	0x2E35,
	0x2E3A,
	0x2E3C,
	0x2E3F,
	0x2E40,
	0x2E42,
	0x2E43,
	0x2E45,
	0x2E80,
	0x2E9A,
	0x2E9B,
	0x2EF4,
	0x2F00,
	0x2FD6,
	0x2FF0,
	0x2FFC,
	0x3000,
	0x3001,
	0x3003,
	0x3005,
	0x3006,
	0x3008,
	0x3009,
	0x300A,
	0x300B,
	0x300C,
	0x300D,
	0x300E,
	0x300F,
	0x3010,
	0x3011,
	0x3012,
	0x3014,
	0x3015,
	0x3016,
	0x3017,
	0x3018,
	0x3019,
	0x301A,
	0x301B,
	0x301C,
	0x301D,
	0x301E,
	0x3020,
	0x302A,
	0x3030,
	0x3035,
	0x3036,
	0x303B,
	0x303D,
	0x3040,
	0x3041,
	0x3042,
	0x3043,
	0x3044,
	0x3045,
	0x3046,
	0x3047,
	0x3048,
	0x3049,
	0x304A,
	0x3063,
	0x3064,
	0x3083,
	0x3084,
	0x3085,
	0x3086,
	0x3087,
	0x3088,
	0x308E,
	0x308F,
	0x3095,
	0x3097,
	0x3099,
	0x309B,
	0x309F,
	0x30A0,
	0x30A2,
	0x30A3,
	0x30A4,
	0x30A5,
	0x30A6,
	0x30A7,
	0x30A8,
	0x30A9,
	0x30AA,
	0x30C3,
	0x30C4,
	0x30E3,
	0x30E4,
	0x30E5,
	0x30E6,
	0x30E7,
	0x30E8,
	0x30EE,
	0x30EF,
	0x30F5,
	0x30F7,
	0x30FB,
	0x30FF,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x3190,
	0x31BB,
	0x31C0,
	0x31E4,
	0x31F0,
	0x3200,
	0x321F,
	0x3220,
	0x3248,
	0x3250,
	0x32FF,
	0x3300,
	0x4DC0,
	0x4E00,
	0xA015,
	0xA016,
	0xA48D,
	0xA490,
	0xA4C7,
	0xA4FE,
	0xA500,
	0xA60D,
	0xA60E,
	0xA60F,
	0xA610,
	0xA620,
	0xA62A,
	0xA66F,
	0xA673,
	0xA674,
	0xA67E,
	0xA69E,
	0xA6A0,
	0xA6F0,
	0xA6F2,
	0xA6F3,
	0xA6F8,
	0xA802,
	0xA803,
	0xA806,
	0xA807,
	0xA80B,
	0xA80C,
	0xA823,
	0xA828,
	0xA838,
	0xA839,
	0xA874,
	0xA876,
	0xA878,
	0xA880,
	0xA882,
	0xA8B4,
	0xA8C6,
	0xA8CE,
	0xA8D0,
	0xA8DA,
	0xA8E0,
	0xA8F2,
	0xA8FC,
	0xA8FD,
	0xA900,
	0xA90A,
	0xA926,
	0xA92E,
	0xA930,
	0xA947,
	0xA954,
	0xA960,
	0xA97D,
	0xA980,
	0xA984,
	0xA9B3,
	0xA9C1,
	0xA9C7,
	0xA9CA,
	0xA9D0,
	0xA9DA,
	0xA9E5,
	0xA9E6,
	0xA9F0,
	0xA9FA,
	0xAA29,
	0xAA37,
	0xAA43,
	0xAA44,
	0xAA4C,
	0xAA4E,
	0xAA50,
	0xAA5A,
	0xAA5D,
	0xAA60,
	0xAA7B,
	0xAA7E,
	0xAAB0,
	0xAAB1,
	0xAAB2,
	0xAAB5,
	0xAAB7,
	0xAAB9,
	0xAABE,
	0xAAC0,
	0xAAC1,
	0xAAC2,
	0xAAEB,
	0xAAF0,
	0xAAF2,
	0xAAF5,
	0xAAF7,
	0xABE3,
	0xABEB,
	0xABEC,
	0xABEE,
	0xABF0,
	0xABFA,
	0xAC00,
	0xAC01,
	0xAC1C,
	0xAC1D,
	0xAC38,
	0xAC39,
	0xAC54,
	0xAC55,
	0xAC70,
	0xAC71,
	0xAC8C,
	0xAC8D,
	0xACA8,
	0xACA9,
	0xACC4,
	0xACC5,
	0xACE0,
	0xACE1,
	0xACFC,
	0xACFD,
	0xAD18,
	0xAD19,
	0xAD34,
	0xAD35,
	0xAD50,
	0xAD51,
	0xAD6C,
	0xAD6D,
	0xAD88,
	0xAD89,
	0xADA4,
	0xADA5,
	0xADC0,
	0xADC1,
	0xADDC,
	0xADDD,
	0xADF8,
	0xADF9,
	0xAE14,
	0xAE15,
	0xAE30,
	0xAE31,
	0xAE4C,
	0xAE4D,
	0xAE68,
	0xAE69,
	0xAE84,
	0xAE85,
	0xAEA0,
	0xAEA1,
	0xAEBC,
	0xAEBD,
	0xAED8,
	0xAED9,
	0xAEF4,
	0xAEF5,
	0xAF10,
	0xAF11,
	0xAF2C,
	0xAF2D,
	0xAF48,
	0xAF49,
	0xAF64,
	0xAF65,
	0xAF80,
	0xAF81,
	0xAF9C,
	0xAF9D,
	0xAFB8,
	0xAFB9,
	0xAFD4,
	0xAFD5,
	0xAFF0,
	0xAFF1,
	0xB00C,
	0xB00D,
	0xB028,
	0xB029,
	0xB044,
	0xB045,
	0xB060,
	0xB061,
	0xB07C,
	0xB07D,
	0xB098,
	0xB099,
	0xB0B4,
	0xB0B5,
	0xB0D0,
	0xB0D1,
	0xB0EC,
	0xB0ED,
	0xB108,
	0xB109,
	0xB124,
	0xB125,
	0xB140,
	0xB141,
	0xB15C,
	0xB15D,
	0xB178,
	0xB179,
	0xB194,
	0xB195,
	0xB1B0,
	0xB1B1,
	0xB1CC,
	0xB1CD,
	0xB1E8,
	0xB1E9,
	0xB204,
	0xB205,
	0xB220,
	0xB221,
	0xB23C,
	0xB23D,
	0xB258,
	0xB259,
	0xB274,
	0xB275,
	0xB290,
	0xB291,
	0xB2AC,
	0xB2AD,
	0xB2C8,
	0xB2C9,
	0xB2E4,
	0xB2E5,
	0xB300,
	0xB301,
	0xB31C,
	0xB31D,
	0xB338,
	0xB339,
	0xB354,
	0xB355,
	0xB370,
	0xB371,
	0xB38C,
	0xB38D,
	0xB3A8,
	0xB3A9,
	0xB3C4,
	0xB3C5,
	0xB3E0,
	0xB3E1,
	0xB3FC,
	0xB3FD,
	0xB418,
	0xB419,
	0xB434,
	0xB435,
	0xB450,
	0xB451,
	0xB46C,
	0xB46D,
	0xB488,
	0xB489,
	0xB4A4,
	0xB4A5,
	0xB4C0,
	0xB4C1,
	0xB4DC,
	0xB4DD,
	0xB4F8,
	0xB4F9,
	0xB514,
	0xB515,
	0xB530,
	0xB531,
	0xB54C,
	0xB54D,
	0xB568,
	0xB569,
	0xB584,
	0xB585,
	0xB5A0,
	0xB5A1,
	0xB5BC,
	0xB5BD,
	0xB5D8,
	0xB5D9,
	0xB5F4,
	0xB5F5,
	0xB610,
	0xB611,
	0xB62C,
	0xB62D,
	0xB648,
	0xB649,
	0xB664,
	0xB665,
	0xB680,
	0xB681,
	0xB69C,
	0xB69D,
	0xB6B8,
	0xB6B9,
	0xB6D4,
	0xB6D5,
	0xB6F0,
	0xB6F1,
	0xB70C,
	0xB70D,
	0xB728,
	0xB729,
	0xB744,
	0xB745,
	0xB760,
	0xB761,
	0xB77C,
	0xB77D,
	0xB798,
	0xB799,
	0xB7B4,
	0xB7B5,
	0xB7D0,
	0xB7D1,
	0xB7EC,
	0xB7ED,
	0xB808,
	0xB809,
	0xB824,
	0xB825,
	0xB840,
	0xB841,
	0xB85C,
	0xB85D,
	0xB878,
	0xB879,
	0xB894,
	0xB895,
	0xB8B0,
	0xB8B1,
	0xB8CC,
	0xB8CD,
	0xB8E8,
	0xB8E9,
	0xB904,
	0xB905,
	0xB920,
	0xB921,
	0xB93C,
	0xB93D,
	0xB958,
	0xB959,
	0xB974,
	0xB975,
	0xB990,
	0xB991,
	0xB9AC,
	0xB9AD,
	0xB9C8,
	0xB9C9,
	0xB9E4,
	0xB9E5,
	0xBA00,
	0xBA01,
	0xBA1C,
	0xBA1D,
	0xBA38,
	0xBA39,
	0xBA54,
	0xBA55,
	0xBA70,
	0xBA71,
	0xBA8C,
	0xBA8D,
	0xBAA8,
	0xBAA9,
	0xBAC4,
	0xBAC5,
	0xBAE0,
	0xBAE1,
	0xBAFC,
	0xBAFD,
	0xBB18,
	0xBB19,
	0xBB34,
	0xBB35,
	0xBB50,
	0xBB51,
	0xBB6C,
	0xBB6D,
	0xBB88,
	0xBB89,
	0xBBA4,
	0xBBA5,
	0xBBC0,
	0xBBC1,
	0xBBDC,
	0xBBDD,
	0xBBF8,
	0xBBF9,
	0xBC14,
	0xBC15,
	0xBC30,
	0xBC31,
	0xBC4C,
	0xBC4D,
	0xBC68,
	0xBC69,
	0xBC84,
	0xBC85,
	0xBCA0,
	0xBCA1,
	0xBCBC,
	0xBCBD,
	0xBCD8,
	0xBCD9,
	0xBCF4,
	0xBCF5,
	0xBD10,
	0xBD11,
	0xBD2C,
	0xBD2D,
	0xBD48,
	0xBD49,
	0xBD64,
	0xBD65,
	0xBD80,
	0xBD81,
	0xBD9C,
	0xBD9D,
	0xBDB8,
	0xBDB9,
	0xBDD4,
	0xBDD5,
	0xBDF0,
	0xBDF1,
	0xBE0C,
	0xBE0D,
	0xBE28,
	0xBE29,
	0xBE44,
	0xBE45,
	0xBE60,
	0xBE61,
	0xBE7C,
	0xBE7D,
	0xBE98,
	0xBE99,
	0xBEB4,
	0xBEB5,
	0xBED0,
	0xBED1,
	0xBEEC,
	0xBEED,
	0xBF08,
	0xBF09,
	0xBF24,
	0xBF25,
	0xBF40,
	0xBF41,
	0xBF5C,
	0xBF5D,
	0xBF78,
	0xBF79,
	0xBF94,
	0xBF95,
	0xBFB0,
	0xBFB1,
	0xBFCC,
	0xBFCD,
	0xBFE8,
	0xBFE9,
	0xC004,
	0xC005,
	0xC020,
	0xC021,
	0xC03C,
	0xC03D,
	0xC058,
	0xC059,
	0xC074,
	0xC075,
	0xC090,
	0xC091,
	0xC0AC,
	0xC0AD,
	0xC0C8,
	0xC0C9,
	0xC0E4,
	0xC0E5,
	0xC100,
	0xC101,
	0xC11C,
	0xC11D,
	0xC138,
	0xC139,
	0xC154,
	0xC155,
	0xC170,
	0xC171,
	0xC18C,
	0xC18D,
	0xC1A8,
	0xC1A9,
	0xC1C4,
	0xC1C5,
	0xC1E0,
	0xC1E1,
	0xC1FC,
	0xC1FD,
	0xC218,
	0xC219,
	0xC234,
	0xC235,
	0xC250,
	0xC251,
	0xC26C,
	0xC26D,
	0xC288,
	0xC289,
	0xC2A4,
	0xC2A5,
	0xC2C0,
	0xC2C1,
	0xC2DC,
	0xC2DD,
	0xC2F8,
	0xC2F9,
	0xC314,
	0xC315,
	0xC330,
	0xC331,
	0xC34C,
	0xC34D,
	0xC368,
	0xC369,
	0xC384,
	0xC385,
	0xC3A0,
	0xC3A1,
	0xC3BC,
	0xC3BD,
	0xC3D8,
	0xC3D9,
	0xC3F4,
	0xC3F5,
	0xC410,
	0xC411,
	0xC42C,
	0xC42D,
	0xC448,
	0xC449,
	0xC464,
	0xC465,
	0xC480,
	0xC481,
	0xC49C,
	0xC49D,
	0xC4B8,
	0xC4B9,
	0xC4D4,
	0xC4D5,
	0xC4F0,
	0xC4F1,
	0xC50C,
	0xC50D,
	0xC528,
	0xC529,
	0xC544,
	0xC545,
	0xC560,
	0xC561,
	0xC57C,
	0xC57D,
	0xC598,
	0xC599,
	0xC5B4,
	0xC5B5,
	0xC5D0,
	0xC5D1,
	0xC5EC,
	0xC5ED,
	0xC608,
	0xC609,
	0xC624,
	0xC625,
	0xC640,
	0xC641,
	0xC65C,
	0xC65D,
	0xC678,
	0xC679,
	0xC694,
	0xC695,
	0xC6B0,
	0xC6B1,
	0xC6CC,
	0xC6CD,
	0xC6E8,
	0xC6E9,
	0xC704,
	0xC705,
	0xC720,
	0xC721,
	0xC73C,
	0xC73D,
	0xC758,
	0xC759,
	0xC774,
	0xC775,
	0xC790,
	0xC791,
	0xC7AC,
	0xC7AD,
	0xC7C8,
	0xC7C9,
	0xC7E4,
	0xC7E5,
	0xC800,
	0xC801,
	0xC81C,
	0xC81D,
	0xC838,
	0xC839,
	0xC854,
	0xC855,
	0xC870,
	0xC871,
	0xC88C,
	0xC88D,
	0xC8A8,
	0xC8A9,
	0xC8C4,
	0xC8C5,
	0xC8E0,
	0xC8E1,
	0xC8FC,
	0xC8FD,
	0xC918,
	0xC919,
	0xC934,
	0xC935,
	0xC950,
	0xC951,
	0xC96C,
	0xC96D,
	0xC988,
	0xC989,
	0xC9A4,
	0xC9A5,
	0xC9C0,
	0xC9C1,
	0xC9DC,
	0xC9DD,
	0xC9F8,
	0xC9F9,
	0xCA14,
	0xCA15,
	0xCA30,
	0xCA31,
	0xCA4C,
	0xCA4D,
	0xCA68,
	0xCA69,
	0xCA84,
	0xCA85,
	0xCAA0,
	0xCAA1,
	0xCABC,
	0xCABD,
	0xCAD8,
	0xCAD9,
	0xCAF4,
	0xCAF5,
	0xCB10,
	0xCB11,
	0xCB2C,
	0xCB2D,
	0xCB48,
	0xCB49,
	0xCB64,
	0xCB65,
	0xCB80,
	0xCB81,
	0xCB9C,
	0xCB9D,
	0xCBB8,
	0xCBB9,
	0xCBD4,
	0xCBD5,
	0xCBF0,
	0xCBF1,
	0xCC0C,
	0xCC0D,
	0xCC28,
	0xCC29,
	0xCC44,
	0xCC45,
	0xCC60,
	0xCC61,
	0xCC7C,
	0xCC7D,
	0xCC98,
	0xCC99,
	0xCCB4,
	0xCCB5,
	0xCCD0,
	0xCCD1,
	0xCCEC,
	0xCCED,
	0xCD08,
	0xCD09,
	0xCD24,
	0xCD25,
	0xCD40,
	0xCD41,
	0xCD5C,
	0xCD5D,
	0xCD78,
	0xCD79,
	0xCD94,
	0xCD95,
	0xCDB0,
	0xCDB1,
	0xCDCC,
	0xCDCD,
	0xCDE8,
	0xCDE9,
	0xCE04,
	0xCE05,
	0xCE20,
	0xCE21,
	0xCE3C,
	0xCE3D,
	0xCE58,
	0xCE59,
	0xCE74,
	0xCE75,
	0xCE90,
	0xCE91,
	0xCEAC,
	0xCEAD,
	0xCEC8,
	0xCEC9,
	0xCEE4,
	0xCEE5,
	0xCF00,
	0xCF01,
	0xCF1C,
	0xCF1D,
	0xCF38,
	0xCF39,
	0xCF54,
	0xCF55,
	0xCF70,
	0xCF71,
	0xCF8C,
	0xCF8D,
	0xCFA8,
	0xCFA9,
	0xCFC4,
	0xCFC5,
	0xCFE0,
	0xCFE1,
	0xCFFC,
	0xCFFD,
	0xD018,
	0xD019,
	0xD034,
	0xD035,
	0xD050,
	0xD051,
	0xD06C,
	0xD06D,
	0xD088,
	0xD089,
	0xD0A4,
	0xD0A5,
	0xD0C0,
	0xD0C1,
	0xD0DC,
	0xD0DD,
	0xD0F8,
	0xD0F9,
	0xD114,
	0xD115,
	0xD130,
	0xD131,
	0xD14C,
	0xD14D,
	0xD168,
	0xD169,
	0xD184,
	0xD185,
	0xD1A0,
	0xD1A1,
	0xD1BC,
	0xD1BD,
	0xD1D8,
	0xD1D9,
	0xD1F4,
	0xD1F5,
	0xD210,
	0xD211,
	0xD22C,
	0xD22D,
	0xD248,
	0xD249,
	0xD264,
	0xD265,
	0xD280,
	0xD281,
	0xD29C,
	0xD29D,
	0xD2B8,
	0xD2B9,
	0xD2D4,
	0xD2D5,
	0xD2F0,
	0xD2F1,
	0xD30C,
	0xD30D,
	0xD328,
	0xD329,
	0xD344,
	0xD345,
	0xD360,
	0xD361,
	0xD37C,
	0xD37D,
	0xD398,
	0xD399,
	0xD3B4,
	0xD3B5,
	0xD3D0,
	0xD3D1,
	0xD3EC,
	0xD3ED,
	0xD408,
	0xD409,
	0xD424,
	0xD425,
	0xD440,
	0xD441,
	0xD45C,
	0xD45D,
	0xD478,
	0xD479,
	0xD494,
	0xD495,
	0xD4B0,
	0xD4B1,
	0xD4CC,
	0xD4CD,
	0xD4E8,
	0xD4E9,
	0xD504,
	0xD505,
	0xD520,
	0xD521,
	0xD53C,
	0xD53D,
	0xD558,
	0xD559,
	0xD574,
	0xD575,
	0xD590,
	0xD591,
	0xD5AC,
	0xD5AD,
	0xD5C8,
	0xD5C9,
	0xD5E4,
	0xD5E5,
	0xD600,
	0xD601,
	0xD61C,
	0xD61D,
	0xD638,
	0xD639,
	0xD654,
	0xD655,
	0xD670,
	0xD671,
	0xD68C,
	0xD68D,
	0xD6A8,
	0xD6A9,
	0xD6C4,
	0xD6C5,
	0xD6E0,
	0xD6E1,
	0xD6FC,
	0xD6FD,
	0xD718,
	0xD719,
	0xD734,
	0xD735,
	0xD750,
	0xD751,
	0xD76C,
	0xD76D,
	0xD788,
	0xD789,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xF900,
	0xFB00,
	0xFB1D,
	0xFB1E,
	0xFB1F,
	0xFB29,
	0xFB2A,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFB50,
	0xFD3E,
	0xFD3F,
	0xFD40,
	0xFDFC,
	0xFDFD,
	0xFE00,
	0xFE10,
	0xFE11,
	0xFE13,
	0xFE15,
	0xFE17,
	0xFE18,
	0xFE19,
	0xFE1A,
	0xFE20,
	0xFE30,
	0xFE35,
	0xFE36,
	0xFE37,
	0xFE38,
	0xFE39,
	0xFE3A,
	0xFE3B,
	0xFE3C,
	0xFE3D,
	0xFE3E,
	0xFE3F,
	0xFE40,
	0xFE41,
	0xFE42,
	0xFE43,
	0xFE44,
	0xFE45,
	0xFE47,
	0xFE48,
	0xFE49,
	0xFE50,
	0xFE51,
	0xFE52,
	0xFE53,
	0xFE54,
	0xFE56,
	0xFE58,
	0xFE59,
	0xFE5A,
	0xFE5B,
	0xFE5C,
	0xFE5D,
	0xFE5E,
	0xFE5F,
	0xFE67,
	0xFE68,
	0xFE69,
	0xFE6A,
	0xFE6B,
	0xFE6C,
	0xFEFF,
	0xFF00,
	0xFF01,
	0xFF02,
	0xFF04,
	0xFF05,
	0xFF06,
	0xFF08,
	0xFF09,
	0xFF0A,
	0xFF0C,
	0xFF0D,
	0xFF0E,
	0xFF0F,
	0xFF1A,
	0xFF1C,
	0xFF1F,
	0xFF20,
	0xFF3B,
	0xFF3C,
	0xFF3D,
	0xFF3E,
	0xFF5B,
	0xFF5C,
	0xFF5D,
	0xFF5E,
	0xFF5F,
	0xFF60,
	0xFF62,
	0xFF63,
	0xFF65,
	0xFF66,
	0xFF67,
	0xFF71,
	0xFF9E,
	0xFFA0,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0xFFE0,
	0xFFE1,
	0xFFE2,
	0xFFE5,
	0xFFE7,
	0xFFF9,
	0xFFFC,
	0xFFFD,
	0x10100,
	0x10103,
	0x101FD,
	0x101FE,
	0x102E0,
	0x102E1,
	0x10376,
	0x1037B,
	0x1039F,
	0x103A0,
	0x103D0,
	0x103D1,
	0x104A0,
	0x104AA,
	0x10857,
	0x10858,
	0x1091F,
	0x10920,
	0x10A01,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A10,
	0x10A38,
	0x10A3B,
	0x10A3F,
	0x10A40,
	0x10A50,
	0x10A58,
	0x10AE5,
	0x10AE7,
	0x10AF0,
	0x10AF6,
	0x10AF7,
	0x10B39,
	0x10B40,
	0x11000,
	0x11003,
	0x11038,
	0x11047,
	0x11049,
	0x11066,
	0x11070,
	0x1107F,
	0x11083,
	0x110B0,
	0x110BB,
	0x110BE,
	0x110C2,
	0x110F0,
	0x110FA,
	0x11100,
	0x11103,
	0x11127,
	0x11135,
	0x11136,
	0x11140,
	0x11144,
	0x11173,
	0x11174,
	0x11175,
	0x11176,
	0x11180,
	0x11183,
	0x111B3,
	0x111C1,
	0x111C5,
	0x111C7,
	0x111C8,
	0x111C9,
	0x111CA,
	0x111CD,
	0x111D0,
	0x111DA,
	0x111DB,
	0x111DC,
	0x111DD,
	0x111E0,
	0x1122C,
	0x11238,
	0x1123A,
	0x1123B,
	0x1123D,
	0x1123E,
	0x1123F,
	0x112A9,
	0x112AA,
	0x112DF,
	0x112EB,
	0x112F0,
	0x112FA,
	0x11300,
	0x11304,
	0x1133C,
	0x1133D,
	0x1133E,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134E,
	0x11357,
	0x11358,
	0x11362,
	0x11364,
	0x11366,
	0x1136D,
	0x11370,
	0x11375,
	0x11435,
	0x11447,
	0x1144B,
	0x1144F,
	0x11450,
	0x1145A,
	0x1145B,
	0x1145C,
	0x114B0,
	0x114C4,
	0x114D0,
	0x114DA,
	0x115AF,
	0x115B6,
	0x115B8,
	0x115C1,
	0x115C2,
	0x115C4,
	0x115C6,
	0x115C9,
	0x115D8,
	0x115DC,
	0x115DE,
	0x11630,
	0x11641,
	0x11643,
	0x11650,
	0x1165A,
	0x11660,
	0x1166D,
	0x116AB,
	0x116B8,
	0x116C0,
	0x116CA,
	0x1171D,
	0x1172C,
	0x11730,
	0x1173A,
	0x1173C,
	0x1173F,
	0x118E0,
	0x118EA,
	0x11C2F,
	0x11C37,
	0x11C38,
	0x11C40,
	0x11C41,
	0x11C46,
	0x11C50,
	0x11C5A,
	0x11C70,
	0x11C71,
	0x11C72,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12470,
	0x12475,
	0x13258,
	0x1325B,
	0x1325E,
	0x13282,
	0x13283,
	0x13286,
	0x13287,
	0x13288,
	0x13289,
	0x1328A,
	0x13379,
	0x1337A,
	0x1337C,
	0x145CE,
	0x145CF,
	0x145D0,
	0x16A60,
	0x16A6A,
	0x16A6E,
	0x16A70,
	0x16AF0,
	0x16AF5,
	0x16AF6,
	0x16B30,
	0x16B37,
	0x16B3A,
	0x16B44,
	0x16B45,
	0x16B50,
	0x16B5A,
	0x16F51,
	0x16F7F,
	0x16F8F,
	0x16F93,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC9D,
	0x1BC9F,
	0x1BCA0,
	0x1BCA4,
	0x1D165,
	0x1D16A,
	0x1D16D,
	0x1D183,
	0x1D185,
	0x1D18C,
	0x1D1AA,
	0x1D1AE,
	0x1D242,
	0x1D245,
	0x1D7CE,
	0x1D800,
	0x1DA00,
	0x1DA37,
	0x1DA3B,
	0x1DA6D,
	0x1DA75,
	0x1DA76,
	0x1DA84,
	0x1DA85,
	0x1DA87,
	0x1DA8B,
	0x1DA9B,
	0x1DAA0,
	0x1DAA1,
	0x1DAB0,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E8D0,
	0x1E8D7,
	0x1E944,
	0x1E94B,
	0x1E950,
	0x1E95A,
	0x1E95E,
	0x1E960,
	0x1F000,
	0x1F100,
	0x1F10D,
	0x1F110,
	0x1F12F,
	0x1F130,
	0x1F16C,
	0x1F170,
	0x1F1AD,
	0x1F1E6,
	0x1F200,
	0x1F385,
	0x1F386,
	0x1F39C,
	0x1F39E,
	0x1F3B5,
	0x1F3B7,
	0x1F3BC,
	0x1F3BD,
	0x1F3C3,
	0x1F3C5,
	0x1F3CA,
	0x1F3CC,
	0x1F3FB,
	0x1F400,
	0x1F442,
	0x1F444,
	0x1F446,
	0x1F451,
	0x1F466,
	0x1F46A,
	0x1F46E,
	0x1F46F,
	0x1F470,
	0x1F479,
	0x1F47C,
	0x1F47D,
	0x1F481,
	0x1F484,
	0x1F485,
	0x1F488,
	0x1F4A0,
	0x1F4A1,
	0x1F4A2,
	0x1F4A3,
	0x1F4A4,
	0x1F4A5,
	0x1F4AA,
	0x1F4AB,
	0x1F4AF,
	0x1F4B0,
	0x1F4B1,
	0x1F4B3,
	0x1F500,
	0x1F507,
	0x1F517,
	0x1F525,
	0x1F532,
	0x1F54A,
	0x1F575,
	0x1F576,
	0x1F57A,
	0x1F57B,
	0x1F590,
	0x1F591,
	0x1F595,
	0x1F597,
	0x1F5D4,
	0x1F5DC,
	0x1F5F4,
	0x1F5FA,
	0x1F645,
	0x1F648,
	0x1F64B,
	0x1F650,
	0x1F676,
	0x1F679,
	0x1F67C,
	0x1F680,
	0x1F6A3,
	0x1F6A4,
	0x1F6B4,
	0x1F6B7,
	0x1F6C0,
	0x1F6C1,
	0x1F700,
	0x1F774,
	0x1F780,
	0x1F7D5,
	0x1F800,
	0x1F80C,
	0x1F810,
	0x1F848,
	0x1F850,
	0x1F85A,
	0x1F860,
	0x1F888,
	0x1F890,
	0x1F8AE,
	0x1F918,
	0x1F91F,
	0x1F926,
	0x1F927,
	0x1F930,
	0x1F931,
	0x1F933,
	0x1F93A,
	0x1F93C,
	0x1F93F,
	0x1FFFE,
	0x20000,
	0x2FFFE,
	0x30000,
	0x3FFFE,
	0xE0001,
	0xE0002,
	0xE0020,
	0xE0080,
	0xE0100,
	0xE01F0
};

#endif	/* defined(PERL_IN_PERL_C) */

#if defined(PERL_IN_REGEXEC_C)

#define LB_ENUM_COUNT 39

typedef enum {
	LB_Alphabetic = 0,
	LB_Break_After = 1,
	LB_Break_Before = 2,
	LB_Break_Both = 3,
	LB_Break_Symbols = 4,
	LB_Carriage_Return = 5,
	LB_Close_Parenthesis = 6,
	LB_Close_Punctuation = 7,
	LB_Combining_Mark = 8,
	LB_Contingent_Break = 9,
	LB_E_Base = 10,
	LB_E_Modifier = 11,
	LB_Exclamation = 12,
	LB_Glue = 13,
	LB_H2 = 14,
	LB_H3 = 15,
	LB_Hebrew_Letter = 16,
	LB_Hyphen = 17,
	LB_Ideographic = 18,
	LB_Infix_Numeric = 19,
	LB_Inseparable = 20,
	LB_JL = 21,
	LB_JT = 22,
	LB_JV = 23,
	LB_Line_Feed = 24,
	LB_Mandatory_Break = 25,
	LB_Next_Line = 26,
	LB_Nonstarter = 27,
	LB_Numeric = 28,
	LB_Open_Punctuation = 29,
	LB_Postfix_Numeric = 30,
	LB_Prefix_Numeric = 31,
	LB_Quotation = 32,
	LB_Regional_Indicator = 33,
	LB_Space = 34,
	LB_Word_Joiner = 35,
	LB_ZWJ = 36,
	LB_ZWSpace = 37,
	LB_EDGE = 38
} LB_enum;

static const LB_enum _Perl_LB_invmap[] = { /* for EBCDIC 1047 */
	LB_Combining_Mark,
	LB_Break_After,
	LB_Combining_Mark,
	LB_Mandatory_Break,
	LB_Carriage_Return,
	LB_Combining_Mark,
	LB_Line_Feed,
	LB_Combining_Mark,
	LB_Next_Line,
	LB_Combining_Mark,
	LB_Space,
	LB_Glue,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Infix_Numeric,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Prefix_Numeric,
	LB_Break_After,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Close_Parenthesis,
	LB_Infix_Numeric,
	LB_Alphabetic,
	LB_Hyphen,
	LB_Break_Symbols,
	LB_Alphabetic,
	LB_Infix_Numeric,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Infix_Numeric,
	LB_Alphabetic,
	LB_Quotation,
	LB_Alphabetic,
	LB_Quotation,
	LB_Alphabetic,
	LB_Quotation,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Close_Parenthesis,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Glue,
	LB_Combining_Mark,
	LB_Glue,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Infix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Infix_Numeric,
	LB_Break_After,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Exclamation,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Hebrew_Letter,
	LB_Alphabetic,
	LB_Hebrew_Letter,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Infix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Exclamation,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Numeric,
	LB_Postfix_Numeric,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Infix_Numeric,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Glue,
	LB_Break_Before,
	LB_Break_After,
	LB_Glue,
	LB_Exclamation,
	LB_Glue,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_After,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Break_After,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Glue,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Numeric,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_JL,
	LB_JV,
	LB_JT,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Nonstarter,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Break_After,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Glue,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Numeric,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Numeric,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Numeric,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Break_After,
	LB_Glue,
	LB_Break_After,
	LB_ZWSpace,
	LB_Combining_Mark,
	LB_ZWJ,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Glue,
	LB_Break_After,
	LB_Break_Both,
	LB_Alphabetic,
	LB_Quotation,
	LB_Open_Punctuation,
	LB_Quotation,
	LB_Open_Punctuation,
	LB_Quotation,
	LB_Alphabetic,
	LB_Inseparable,
	LB_Break_After,
	LB_Mandatory_Break,
	LB_Combining_Mark,
	LB_Glue,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Quotation,
	LB_Alphabetic,
	LB_Nonstarter,
	LB_Alphabetic,
	LB_Infix_Numeric,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Nonstarter,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Word_Joiner,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Postfix_Numeric,
	LB_Prefix_Numeric,
	LB_Postfix_Numeric,
	LB_Prefix_Numeric,
	LB_Postfix_Numeric,
	LB_Prefix_Numeric,
	LB_Postfix_Numeric,
	LB_Prefix_Numeric,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Inseparable,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_E_Base,
	LB_Alphabetic,
	LB_Quotation,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Break_After,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Quotation,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Open_Punctuation,
	LB_Break_After,
	LB_Alphabetic,
	LB_Quotation,
	LB_Alphabetic,
	LB_Quotation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Break_After,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_Both,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Open_Punctuation,
	LB_Break_After,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Break_After,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Nonstarter,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Combining_Mark,
	LB_Ideographic,
	LB_Combining_Mark,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Exclamation,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_JL,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_Alphabetic,
	LB_JV,
	LB_Alphabetic,
	LB_JT,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Hebrew_Letter,
	LB_Combining_Mark,
	LB_Hebrew_Letter,
	LB_Alphabetic,
	LB_Hebrew_Letter,
	LB_Alphabetic,
	LB_Hebrew_Letter,
	LB_Alphabetic,
	LB_Hebrew_Letter,
	LB_Alphabetic,
	LB_Hebrew_Letter,
	LB_Alphabetic,
	LB_Hebrew_Letter,
	LB_Alphabetic,
	LB_Hebrew_Letter,
	LB_Alphabetic,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Infix_Numeric,
	LB_Close_Punctuation,
	LB_Infix_Numeric,
	LB_Exclamation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Inseparable,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Ideographic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Nonstarter,
	LB_Exclamation,
	LB_Ideographic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Prefix_Numeric,
	LB_Postfix_Numeric,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Word_Joiner,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Ideographic,
	LB_Prefix_Numeric,
	LB_Postfix_Numeric,
	LB_Ideographic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Exclamation,
	LB_Ideographic,
	LB_Open_Punctuation,
	LB_Ideographic,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Open_Punctuation,
	LB_Ideographic,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Prefix_Numeric,
	LB_Ideographic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Contingent_Break,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Inseparable,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_Before,
	LB_Break_After,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Nonstarter,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Regional_Indicator,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Modifier,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Alphabetic,
	LB_Quotation,
	LB_Nonstarter,
	LB_Alphabetic,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic
};

#endif	/* defined(PERL_IN_REGEXEC_C) */

#if defined(PERL_IN_PERL_C)

static const UV _Perl_SB_invlist[] = { /* for EBCDIC 1047 */
	2981,	/* Number of elements */
	148565664, /* Version and data structure type */
	0,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x5,
	0x6,
	0xB,
	0xD,
	0xE,
	0x15,
	0x16,
	0x25,
	0x26,
	0x40,
	0x42,
	0x4A,
	0x4B,
	0x4C,
	0x4D,
	0x4E,
	0x51,
	0x5A,
	0x5B,
	0x5D,
	0x5E,
	0x60,
	0x61,
	0x62,
	0x6A,
	0x6B,
	0x6C,
	0x6F,
	0x70,
	0x71,
	0x79,
	0x7A,
	0x7B,
	0x7D,
	0x7E,
	0x7F,
	0x80,
	0x81,
	0x8A,
	0x8C,
	0x8F,
	0x91,
	0x9D,
	0x9E,
	0x9F,
	0xA0,
	0xA1,
	0xA2,
	0xAA,
	0xAC,
	0xAD,
	0xAE,
	0xAF,
	0xBA,
	0xBB,
	0xBD,
	0xBE,
	0xC0,
	0xC1,
	0xCA,
	0xCB,
	0xD0,
	0xD1,
	0xDA,
	0xDB,
	0xE0,
	0xE2,
	0xEA,
	0xEB,
	0xF0,
	0xFA,
	0xFB,
	0xFF,
	0x100,
	0x101,
	0x102,
	0x103,
	0x104,
	0x105,
	0x106,
	0x107,
	0x108,
	0x109,
	0x10A,
	0x10B,
	0x10C,
	0x10D,
	0x10E,
	0x10F,
	0x110,
	0x111,
	0x112,
	0x113,
	0x114,
	0x115,
	0x116,
	0x117,
	0x118,
	0x119,
	0x11A,
	0x11B,
	0x11C,
	0x11D,
	0x11E,
	0x11F,
	0x120,
	0x121,
	0x122,
	0x123,
	0x124,
	0x125,
	0x126,
	0x127,
	0x128,
	0x129,
	0x12A,
	0x12B,
	0x12C,
	0x12D,
	0x12E,
	0x12F,
	0x130,
	0x131,
	0x132,
	0x133,
	0x134,
	0x135,
	0x136,
	0x137,
	0x139,
	0x13A,
	0x13B,
	0x13C,
	0x13D,
	0x13E,
	0x13F,
	0x140,
	0x141,
	0x142,
	0x143,
	0x144,
	0x145,
	0x146,
	0x147,
	0x148,
	0x14A,
	0x14B,
	0x14C,
	0x14D,
	0x14E,
	0x14F,
	0x150,
	0x151,
	0x152,
	0x153,
	0x154,
	0x155,
	0x156,
	0x157,
	0x158,
	0x159,
	0x15A,
	0x15B,
	0x15C,
	0x15D,
	0x15E,
	0x15F,
	0x160,
	0x161,
	0x162,
	0x163,
	0x164,
	0x165,
	0x166,
	0x167,
	0x168,
	0x169,
	0x16A,
	0x16B,
	0x16C,
	0x16D,
	0x16E,
	0x16F,
	0x170,
	0x171,
	0x172,
	0x173,
	0x174,
	0x175,
	0x176,
	0x177,
	0x178,
	0x17A,
	0x17B,
	0x17C,
	0x17D,
	0x17E,
	0x181,
	0x183,
	0x184,
	0x185,
	0x186,
	0x188,
	0x189,
	0x18C,
	0x18E,
	0x192,
	0x193,
	0x195,
	0x196,
	0x199,
	0x19C,
	0x19E,
	0x19F,
	0x1A1,
	0x1A2,
	0x1A3,
	0x1A4,
	0x1A5,
	0x1A6,
	0x1A8,
	0x1A9,
	0x1AA,
	0x1AC,
	0x1AD,
	0x1AE,
	0x1B0,
	0x1B1,
	0x1B4,
	0x1B5,
	0x1B6,
	0x1B7,
	0x1B9,
	0x1BB,
	0x1BC,
	0x1BD,
	0x1C0,
	0x1C4,
	0x1C6,
	0x1C7,
	0x1C9,
	0x1CA,
	0x1CC,
	0x1CD,
	0x1CE,
	0x1CF,
	0x1D0,
	0x1D1,
	0x1D2,
	0x1D3,
	0x1D4,
	0x1D5,
	0x1D6,
	0x1D7,
	0x1D8,
	0x1D9,
	0x1DA,
	0x1DB,
	0x1DC,
	0x1DE,
	0x1DF,
	0x1E0,
	0x1E1,
	0x1E2,
	0x1E3,
	0x1E4,
	0x1E5,
	0x1E6,
	0x1E7,
	0x1E8,
	0x1E9,
	0x1EA,
	0x1EB,
	0x1EC,
	0x1ED,
	0x1EE,
	0x1EF,
	0x1F1,
	0x1F3,
	0x1F4,
	0x1F5,
	0x1F6,
	0x1F9,
	0x1FA,
	0x1FB,
	0x1FC,
	0x1FD,
	0x1FE,
	0x1FF,
	0x200,
	0x201,
	0x202,
	0x203,
	0x204,
	0x205,
	0x206,
	0x207,
	0x208,
	0x209,
	0x20A,
	0x20B,
	0x20C,
	0x20D,
	0x20E,
	0x20F,
	0x210,
	0x211,
	0x212,
	0x213,
	0x214,
	0x215,
	0x216,
	0x217,
	0x218,
	0x219,
	0x21A,
	0x21B,
	0x21C,
	0x21D,
	0x21E,
	0x21F,
	0x220,
	0x221,
	0x222,
	0x223,
	0x224,
	0x225,
	0x226,
	0x227,
	0x228,
	0x229,
	0x22A,
	0x22B,
	0x22C,
	0x22D,
	0x22E,
	0x22F,
	0x230,
	0x231,
	0x232,
	0x233,
	0x23A,
	0x23C,
	0x23D,
	0x23F,
	0x241,
	0x242,
	0x243,
	0x247,
	0x248,
	0x249,
	0x24A,
	0x24B,
	0x24C,
	0x24D,
	0x24E,
	0x24F,
	0x294,
	0x295,
	0x2B9,
	0x2C0,
	0x2C2,
	0x2C6,
	0x2D2,
	0x2E0,
	0x2E5,
	0x2EC,
	0x2ED,
	0x2EE,
	0x2EF,
	0x300,
	0x370,
	0x371,
	0x372,
	0x373,
	0x374,
	0x375,
	0x376,
	0x377,
	0x378,
	0x37A,
	0x37E,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x390,
	0x391,
	0x3A2,
	0x3A3,
	0x3AC,
	0x3CF,
	0x3D0,
	0x3D2,
	0x3D5,
	0x3D8,
	0x3D9,
	0x3DA,
	0x3DB,
	0x3DC,
	0x3DD,
	0x3DE,
	0x3DF,
	0x3E0,
	0x3E1,
	0x3E2,
	0x3E3,
	0x3E4,
	0x3E5,
	0x3E6,
	0x3E7,
	0x3E8,
	0x3E9,
	0x3EA,
	0x3EB,
	0x3EC,
	0x3ED,
	0x3EE,
	0x3EF,
	0x3F4,
	0x3F5,
	0x3F6,
	0x3F7,
	0x3F8,
	0x3F9,
	0x3FB,
	0x3FD,
	0x430,
	0x460,
	0x461,
	0x462,
	0x463,
	0x464,
	0x465,
	0x466,
	0x467,
	0x468,
	0x469,
	0x46A,
	0x46B,
	0x46C,
	0x46D,
	0x46E,
	0x46F,
	0x470,
	0x471,
	0x472,
	0x473,
	0x474,
	0x475,
	0x476,
	0x477,
	0x478,
	0x479,
	0x47A,
	0x47B,
	0x47C,
	0x47D,
	0x47E,
	0x47F,
	0x480,
	0x481,
	0x482,
	0x483,
	0x48A,
	0x48B,
	0x48C,
	0x48D,
	0x48E,
	0x48F,
	0x490,
	0x491,
	0x492,
	0x493,
	0x494,
	0x495,
	0x496,
	0x497,
	0x498,
	0x499,
	0x49A,
	0x49B,
	0x49C,
	0x49D,
	0x49E,
	0x49F,
	0x4A0,
	0x4A1,
	0x4A2,
	0x4A3,
	0x4A4,
	0x4A5,
	0x4A6,
	0x4A7,
	0x4A8,
	0x4A9,
	0x4AA,
	0x4AB,
	0x4AC,
	0x4AD,
	0x4AE,
	0x4AF,
	0x4B0,
	0x4B1,
	0x4B2,
	0x4B3,
	0x4B4,
	0x4B5,
	0x4B6,
	0x4B7,
	0x4B8,
	0x4B9,
	0x4BA,
	0x4BB,
	0x4BC,
	0x4BD,
	0x4BE,
	0x4BF,
	0x4C0,
	0x4C2,
	0x4C3,
	0x4C4,
	0x4C5,
	0x4C6,
	0x4C7,
	0x4C8,
	0x4C9,
	0x4CA,
	0x4CB,
	0x4CC,
	0x4CD,
	0x4CE,
	0x4D0,
	0x4D1,
	0x4D2,
	0x4D3,
	0x4D4,
	0x4D5,
	0x4D6,
	0x4D7,
	0x4D8,
	0x4D9,
	0x4DA,
	0x4DB,
	0x4DC,
	0x4DD,
	0x4DE,
	0x4DF,
	0x4E0,
	0x4E1,
	0x4E2,
	0x4E3,
	0x4E4,
	0x4E5,
	0x4E6,
	0x4E7,
	0x4E8,
	0x4E9,
	0x4EA,
	0x4EB,
	0x4EC,
	0x4ED,
	0x4EE,
	0x4EF,
	0x4F0,
	0x4F1,
	0x4F2,
	0x4F3,
	0x4F4,
	0x4F5,
	0x4F6,
	0x4F7,
	0x4F8,
	0x4F9,
	0x4FA,
	0x4FB,
	0x4FC,
	0x4FD,
	0x4FE,
	0x4FF,
	0x500,
	0x501,
	0x502,
	0x503,
	0x504,
	0x505,
	0x506,
	0x507,
	0x508,
	0x509,
	0x50A,
	0x50B,
	0x50C,
	0x50D,
	0x50E,
	0x50F,
	0x510,
	0x511,
	0x512,
	0x513,
	0x514,
	0x515,
	0x516,
	0x517,
	0x518,
	0x519,
	0x51A,
	0x51B,
	0x51C,
	0x51D,
	0x51E,
	0x51F,
	0x520,
	0x521,
	0x522,
	0x523,
	0x524,
	0x525,
	0x526,
	0x527,
	0x528,
	0x529,
	0x52A,
	0x52B,
	0x52C,
	0x52D,
	0x52E,
	0x52F,
	0x530,
	0x531,
	0x557,
	0x559,
	0x55A,
	0x55D,
	0x55E,
	0x561,
	0x588,
	0x589,
	0x58A,
	0x591,
	0x5BE,
	0x5BF,
	0x5C0,
	0x5C1,
	0x5C3,
	0x5C4,
	0x5C6,
	0x5C7,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F4,
	0x600,
	0x606,
	0x60C,
	0x60E,
	0x610,
	0x61B,
	0x61C,
	0x61D,
	0x61F,
	0x620,
	0x64B,
	0x660,
	0x66A,
	0x66B,
	0x66D,
	0x66E,
	0x670,
	0x671,
	0x6D4,
	0x6D5,
	0x6D6,
	0x6DD,
	0x6DE,
	0x6DF,
	0x6E5,
	0x6E7,
	0x6E9,
	0x6EA,
	0x6EE,
	0x6F0,
	0x6FA,
	0x6FD,
	0x6FF,
	0x700,
	0x703,
	0x70F,
	0x710,
	0x711,
	0x712,
	0x730,
	0x74B,
	0x74D,
	0x7A6,
	0x7B1,
	0x7B2,
	0x7C0,
	0x7CA,
	0x7EB,
	0x7F4,
	0x7F6,
	0x7F8,
	0x7F9,
	0x7FA,
	0x7FB,
	0x800,
	0x816,
	0x81A,
	0x81B,
	0x824,
	0x825,
	0x828,
	0x829,
	0x82E,
	0x840,
	0x859,
	0x85C,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x8D4,
	0x8E2,
	0x8E3,
	0x904,
	0x93A,
	0x93D,
	0x93E,
	0x950,
	0x951,
	0x958,
	0x962,
	0x964,
	0x966,
	0x970,
	0x971,
	0x981,
	0x984,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BC,
	0x9BD,
	0x9BE,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CE,
	0x9CF,
	0x9D7,
	0x9D8,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E2,
	0x9E4,
	0x9E6,
	0x9F0,
	0x9F2,
	0xA01,
	0xA04,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA3C,
	0xA3D,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4E,
	0xA51,
	0xA52,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA66,
	0xA70,
	0xA72,
	0xA75,
	0xA76,
	0xA81,
	0xA84,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABC,
	0xABD,
	0xABE,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACE,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE2,
	0xAE4,
	0xAE6,
	0xAF0,
	0xAF9,
	0xAFA,
	0xB01,
	0xB04,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3C,
	0xB3D,
	0xB3E,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4E,
	0xB56,
	0xB58,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB62,
	0xB64,
	0xB66,
	0xB70,
	0xB71,
	0xB72,
	0xB82,
	0xB83,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCE,
	0xBD0,
	0xBD1,
	0xBD7,
	0xBD8,
	0xBE6,
	0xBF0,
	0xC00,
	0xC04,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC3E,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4E,
	0xC55,
	0xC57,
	0xC58,
	0xC5B,
	0xC60,
	0xC62,
	0xC64,
	0xC66,
	0xC70,
	0xC80,
	0xC81,
	0xC84,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBC,
	0xCBD,
	0xCBE,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCE,
	0xCD5,
	0xCD7,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE2,
	0xCE4,
	0xCE6,
	0xCF0,
	0xCF1,
	0xCF3,
	0xD01,
	0xD04,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD3E,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD4E,
	0xD4F,
	0xD54,
	0xD57,
	0xD58,
	0xD5F,
	0xD62,
	0xD64,
	0xD66,
	0xD70,
	0xD7A,
	0xD80,
	0xD82,
	0xD84,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xDCA,
	0xDCB,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDE6,
	0xDF0,
	0xDF2,
	0xDF4,
	0xE01,
	0xE31,
	0xE32,
	0xE34,
	0xE3B,
	0xE40,
	0xE47,
	0xE4F,
	0xE50,
	0xE5A,
	0xE81,
	0xE83,
	0xE84,
	0xE85,
	0xE87,
	0xE89,
	0xE8A,
	0xE8B,
	0xE8D,
	0xE8E,
	0xE94,
	0xE98,
	0xE99,
	0xEA0,
	0xEA1,
	0xEA4,
	0xEA5,
	0xEA6,
	0xEA7,
	0xEA8,
	0xEAA,
	0xEAC,
	0xEAD,
	0xEB1,
	0xEB2,
	0xEB4,
	0xEBA,
	0xEBB,
	0xEBD,
	0xEBE,
	0xEC0,
	0xEC5,
	0xEC6,
	0xEC7,
	0xEC8,
	0xECE,
	0xED0,
	0xEDA,
	0xEDC,
	0xEE0,
	0xF00,
	0xF01,
	0xF18,
	0xF1A,
	0xF20,
	0xF2A,
	0xF35,
	0xF36,
	0xF37,
	0xF38,
	0xF39,
	0xF3A,
	0xF3E,
	0xF40,
	0xF48,
	0xF49,
	0xF6D,
	0xF71,
	0xF85,
	0xF86,
	0xF88,
	0xF8D,
	0xF98,
	0xF99,
	0xFBD,
	0xFC6,
	0xFC7,
	0x1000,
	0x102B,
	0x103F,
	0x1040,
	0x104A,
	0x104C,
	0x1050,
	0x1056,
	0x105A,
	0x105E,
	0x1061,
	0x1062,
	0x1065,
	0x1067,
	0x106E,
	0x1071,
	0x1075,
	0x1082,
	0x108E,
	0x108F,
	0x1090,
	0x109A,
	0x109E,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x10FB,
	0x10FC,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x135D,
	0x1360,
	0x1362,
	0x1363,
	0x1367,
	0x1369,
	0x1380,
	0x1390,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1401,
	0x166D,
	0x166E,
	0x166F,
	0x1680,
	0x1681,
	0x169B,
	0x169D,
	0x16A0,
	0x16EB,
	0x16EE,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1712,
	0x1715,
	0x1720,
	0x1732,
	0x1735,
	0x1737,
	0x1740,
	0x1752,
	0x1754,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1772,
	0x1774,
	0x1780,
	0x17B4,
	0x17D4,
	0x17D7,
	0x17D8,
	0x17DC,
	0x17DD,
	0x17DE,
	0x17E0,
	0x17EA,
	0x1802,
	0x1803,
	0x1804,
	0x1808,
	0x1809,
	0x180A,
	0x180B,
	0x180E,
	0x180F,
	0x1810,
	0x181A,
	0x1820,
	0x1878,
	0x1880,
	0x1885,
	0x1887,
	0x18A9,
	0x18AA,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1920,
	0x192C,
	0x1930,
	0x193C,
	0x1944,
	0x1946,
	0x1950,
	0x196E,
	0x1970,
	0x1975,
	0x1980,
	0x19AC,
	0x19B0,
	0x19CA,
	0x19D0,
	0x19DA,
	0x1A00,
	0x1A17,
	0x1A1C,
	0x1A20,
	0x1A55,
	0x1A5F,
	0x1A60,
	0x1A7D,
	0x1A7F,
	0x1A80,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1AA7,
	0x1AA8,
	0x1AAC,
	0x1AB0,
	0x1ABF,
	0x1B00,
	0x1B05,
	0x1B34,
	0x1B45,
	0x1B4C,
	0x1B50,
	0x1B5A,
	0x1B5C,
	0x1B5E,
	0x1B60,
	0x1B6B,
	0x1B74,
	0x1B80,
	0x1B83,
	0x1BA1,
	0x1BAE,
	0x1BB0,
	0x1BBA,
	0x1BE6,
	0x1BF4,
	0x1C00,
	0x1C24,
	0x1C38,
	0x1C3B,
	0x1C3D,
	0x1C40,
	0x1C4A,
	0x1C4D,
	0x1C50,
	0x1C5A,
	0x1C7E,
	0x1C80,
	0x1C89,
	0x1CD0,
	0x1CD3,
	0x1CD4,
	0x1CE9,
	0x1CED,
	0x1CEE,
	0x1CF2,
	0x1CF5,
	0x1CF7,
	0x1CF8,
	0x1CFA,
	0x1D00,
	0x1DC0,
	0x1DF6,
	0x1DFB,
	0x1E00,
	0x1E01,
	0x1E02,
	0x1E03,
	0x1E04,
	0x1E05,
	0x1E06,
	0x1E07,
	0x1E08,
	0x1E09,
	0x1E0A,
	0x1E0B,
	0x1E0C,
	0x1E0D,
	0x1E0E,
	0x1E0F,
	0x1E10,
	0x1E11,
	0x1E12,
	0x1E13,
	0x1E14,
	0x1E15,
	0x1E16,
	0x1E17,
	0x1E18,
	0x1E19,
	0x1E1A,
	0x1E1B,
	0x1E1C,
	0x1E1D,
	0x1E1E,
	0x1E1F,
	0x1E20,
	0x1E21,
	0x1E22,
	0x1E23,
	0x1E24,
	0x1E25,
	0x1E26,
	0x1E27,
	0x1E28,
	0x1E29,
	0x1E2A,
	0x1E2B,
	0x1E2C,
	0x1E2D,
	0x1E2E,
	0x1E2F,
	0x1E30,
	0x1E31,
	0x1E32,
	0x1E33,
	0x1E34,
	0x1E35,
	0x1E36,
	0x1E37,
	0x1E38,
	0x1E39,
	0x1E3A,
	0x1E3B,
	0x1E3C,
	0x1E3D,
	0x1E3E,
	0x1E3F,
	0x1E40,
	0x1E41,
	0x1E42,
	0x1E43,
	0x1E44,
	0x1E45,
	0x1E46,
	0x1E47,
	0x1E48,
	0x1E49,
	0x1E4A,
	0x1E4B,
	0x1E4C,
	0x1E4D,
	0x1E4E,
	0x1E4F,
	0x1E50,
	0x1E51,
	0x1E52,
	0x1E53,
	0x1E54,
	0x1E55,
	0x1E56,
	0x1E57,
	0x1E58,
	0x1E59,
	0x1E5A,
	0x1E5B,
	0x1E5C,
	0x1E5D,
	0x1E5E,
	0x1E5F,
	0x1E60,
	0x1E61,
	0x1E62,
	0x1E63,
	0x1E64,
	0x1E65,
	0x1E66,
	0x1E67,
	0x1E68,
	0x1E69,
	0x1E6A,
	0x1E6B,
	0x1E6C,
	0x1E6D,
	0x1E6E,
	0x1E6F,
	0x1E70,
	0x1E71,
	0x1E72,
	0x1E73,
	0x1E74,
	0x1E75,
	0x1E76,
	0x1E77,
	0x1E78,
	0x1E79,
	0x1E7A,
	0x1E7B,
	0x1E7C,
	0x1E7D,
	0x1E7E,
	0x1E7F,
	0x1E80,
	0x1E81,
	0x1E82,
	0x1E83,
	0x1E84,
	0x1E85,
	0x1E86,
	0x1E87,
	0x1E88,
	0x1E89,
	0x1E8A,
	0x1E8B,
	0x1E8C,
	0x1E8D,
	0x1E8E,
	0x1E8F,
	0x1E90,
	0x1E91,
	0x1E92,
	0x1E93,
	0x1E94,
	0x1E95,
	0x1E9E,
	0x1E9F,
	0x1EA0,
	0x1EA1,
	0x1EA2,
	0x1EA3,
	0x1EA4,
	0x1EA5,
	0x1EA6,
	0x1EA7,
	0x1EA8,
	0x1EA9,
	0x1EAA,
	0x1EAB,
	0x1EAC,
	0x1EAD,
	0x1EAE,
	0x1EAF,
	0x1EB0,
	0x1EB1,
	0x1EB2,
	0x1EB3,
	0x1EB4,
	0x1EB5,
	0x1EB6,
	0x1EB7,
	0x1EB8,
	0x1EB9,
	0x1EBA,
	0x1EBB,
	0x1EBC,
	0x1EBD,
	0x1EBE,
	0x1EBF,
	0x1EC0,
	0x1EC1,
	0x1EC2,
	0x1EC3,
	0x1EC4,
	0x1EC5,
	0x1EC6,
	0x1EC7,
	0x1EC8,
	0x1EC9,
	0x1ECA,
	0x1ECB,
	0x1ECC,
	0x1ECD,
	0x1ECE,
	0x1ECF,
	0x1ED0,
	0x1ED1,
	0x1ED2,
	0x1ED3,
	0x1ED4,
	0x1ED5,
	0x1ED6,
	0x1ED7,
	0x1ED8,
	0x1ED9,
	0x1EDA,
	0x1EDB,
	0x1EDC,
	0x1EDD,
	0x1EDE,
	0x1EDF,
	0x1EE0,
	0x1EE1,
	0x1EE2,
	0x1EE3,
	0x1EE4,
	0x1EE5,
	0x1EE6,
	0x1EE7,
	0x1EE8,
	0x1EE9,
	0x1EEA,
	0x1EEB,
	0x1EEC,
	0x1EED,
	0x1EEE,
	0x1EEF,
	0x1EF0,
	0x1EF1,
	0x1EF2,
	0x1EF3,
	0x1EF4,
	0x1EF5,
	0x1EF6,
	0x1EF7,
	0x1EF8,
	0x1EF9,
	0x1EFA,
	0x1EFB,
	0x1EFC,
	0x1EFD,
	0x1EFE,
	0x1EFF,
	0x1F08,
	0x1F10,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F28,
	0x1F30,
	0x1F38,
	0x1F40,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F60,
	0x1F68,
	0x1F70,
	0x1F7E,
	0x1F80,
	0x1F88,
	0x1F90,
	0x1F98,
	0x1FA0,
	0x1FA8,
	0x1FB0,
	0x1FB5,
	0x1FB6,
	0x1FB8,
	0x1FBD,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FC8,
	0x1FCD,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FD8,
	0x1FDC,
	0x1FE0,
	0x1FE8,
	0x1FED,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FF8,
	0x1FFD,
	0x2000,
	0x200B,
	0x200C,
	0x200E,
	0x2010,
	0x2013,
	0x2015,
	0x2018,
	0x2020,
	0x2024,
	0x2025,
	0x2028,
	0x202A,
	0x202F,
	0x2030,
	0x2039,
	0x203B,
	0x203C,
	0x203E,
	0x2045,
	0x2047,
	0x204A,
	0x205F,
	0x2060,
	0x2065,
	0x2066,
	0x2070,
	0x2071,
	0x2072,
	0x207D,
	0x207F,
	0x2080,
	0x208D,
	0x208F,
	0x2090,
	0x209D,
	0x20D0,
	0x20F1,
	0x2102,
	0x2103,
	0x2107,
	0x2108,
	0x210A,
	0x210B,
	0x210E,
	0x2110,
	0x2113,
	0x2114,
	0x2115,
	0x2116,
	0x2119,
	0x211E,
	0x2124,
	0x2125,
	0x2126,
	0x2127,
	0x2128,
	0x2129,
	0x212A,
	0x212E,
	0x212F,
	0x2130,
	0x2134,
	0x2135,
	0x2139,
	0x213A,
	0x213C,
	0x213E,
	0x2140,
	0x2145,
	0x2146,
	0x214A,
	0x214E,
	0x214F,
	0x2160,
	0x2170,
	0x2180,
	0x2183,
	0x2184,
	0x2185,
	0x2189,
	0x2308,
	0x230C,
	0x2329,
	0x232B,
	0x24B6,
	0x24D0,
	0x24EA,
	0x275B,
	0x2761,
	0x2768,
	0x2776,
	0x27C5,
	0x27C7,
	0x27E6,
	0x27F0,
	0x2983,
	0x2999,
	0x29D8,
	0x29DC,
	0x29FC,
	0x29FE,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2C61,
	0x2C62,
	0x2C65,
	0x2C67,
	0x2C68,
	0x2C69,
	0x2C6A,
	0x2C6B,
	0x2C6C,
	0x2C6D,
	0x2C71,
	0x2C72,
	0x2C73,
	0x2C75,
	0x2C76,
	0x2C7E,
	0x2C81,
	0x2C82,
	0x2C83,
	0x2C84,
	0x2C85,
	0x2C86,
	0x2C87,
	0x2C88,
	0x2C89,
	0x2C8A,
	0x2C8B,
	0x2C8C,
	0x2C8D,
	0x2C8E,
	0x2C8F,
	0x2C90,
	0x2C91,
	0x2C92,
	0x2C93,
	0x2C94,
	0x2C95,
	0x2C96,
	0x2C97,
	0x2C98,
	0x2C99,
	0x2C9A,
	0x2C9B,
	0x2C9C,
	0x2C9D,
	0x2C9E,
	0x2C9F,
	0x2CA0,
	0x2CA1,
	0x2CA2,
	0x2CA3,
	0x2CA4,
	0x2CA5,
	0x2CA6,
	0x2CA7,
	0x2CA8,
	0x2CA9,
	0x2CAA,
	0x2CAB,
	0x2CAC,
	0x2CAD,
	0x2CAE,
	0x2CAF,
	0x2CB0,
	0x2CB1,
	0x2CB2,
	0x2CB3,
	0x2CB4,
	0x2CB5,
	0x2CB6,
	0x2CB7,
	0x2CB8,
	0x2CB9,
	0x2CBA,
	0x2CBB,
	0x2CBC,
	0x2CBD,
	0x2CBE,
	0x2CBF,
	0x2CC0,
	0x2CC1,
	0x2CC2,
	0x2CC3,
	0x2CC4,
	0x2CC5,
	0x2CC6,
	0x2CC7,
	0x2CC8,
	0x2CC9,
	0x2CCA,
	0x2CCB,
	0x2CCC,
	0x2CCD,
	0x2CCE,
	0x2CCF,
	0x2CD0,
	0x2CD1,
	0x2CD2,
	0x2CD3,
	0x2CD4,
	0x2CD5,
	0x2CD6,
	0x2CD7,
	0x2CD8,
	0x2CD9,
	0x2CDA,
	0x2CDB,
	0x2CDC,
	0x2CDD,
	0x2CDE,
	0x2CDF,
	0x2CE0,
	0x2CE1,
	0x2CE2,
	0x2CE3,
	0x2CE5,
	0x2CEB,
	0x2CEC,
	0x2CED,
	0x2CEE,
	0x2CEF,
	0x2CF2,
	0x2CF3,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D70,
	0x2D7F,
	0x2D80,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x2DE0,
	0x2E00,
	0x2E0E,
	0x2E1C,
	0x2E1E,
	0x2E20,
	0x2E2A,
	0x2E2E,
	0x2E2F,
	0x2E30,
	0x2E3C,
	0x2E3D,
	0x2E42,
	0x2E43,
	0x3000,
	0x3001,
	0x3002,
	0x3003,
	0x3005,
	0x3008,
	0x3012,
	0x3014,
	0x301C,
	0x301D,
	0x3020,
	0x3021,
	0x302A,
	0x3030,
	0x3031,
	0x3036,
	0x3038,
	0x303D,
	0x3041,
	0x3097,
	0x3099,
	0x309B,
	0x309D,
	0x30A0,
	0x30A1,
	0x30FB,
	0x30FC,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x31A0,
	0x31BB,
	0x31F0,
	0x3200,
	0x3400,
	0x4DB6,
	0x4E00,
	0x9FD6,
	0xA000,
	0xA48D,
	0xA4D0,
	0xA4FE,
	0xA4FF,
	0xA500,
	0xA60D,
	0xA60E,
	0xA610,
	0xA620,
	0xA62A,
	0xA62C,
	0xA640,
	0xA641,
	0xA642,
	0xA643,
	0xA644,
	0xA645,
	0xA646,
	0xA647,
	0xA648,
	0xA649,
	0xA64A,
	0xA64B,
	0xA64C,
	0xA64D,
	0xA64E,
	0xA64F,
	0xA650,
	0xA651,
	0xA652,
	0xA653,
	0xA654,
	0xA655,
	0xA656,
	0xA657,
	0xA658,
	0xA659,
	0xA65A,
	0xA65B,
	0xA65C,
	0xA65D,
	0xA65E,
	0xA65F,
	0xA660,
	0xA661,
	0xA662,
	0xA663,
	0xA664,
	0xA665,
	0xA666,
	0xA667,
	0xA668,
	0xA669,
	0xA66A,
	0xA66B,
	0xA66C,
	0xA66D,
	0xA66E,
	0xA66F,
	0xA673,
	0xA674,
	0xA67E,
	0xA67F,
	0xA680,
	0xA681,
	0xA682,
	0xA683,
	0xA684,
	0xA685,
	0xA686,
	0xA687,
	0xA688,
	0xA689,
	0xA68A,
	0xA68B,
	0xA68C,
	0xA68D,
	0xA68E,
	0xA68F,
	0xA690,
	0xA691,
	0xA692,
	0xA693,
	0xA694,
	0xA695,
	0xA696,
	0xA697,
	0xA698,
	0xA699,
	0xA69A,
	0xA69B,
	0xA69E,
	0xA6A0,
	0xA6F0,
	0xA6F2,
	0xA6F3,
	0xA6F4,
	0xA6F7,
	0xA6F8,
	0xA717,
	0xA720,
	0xA722,
	0xA723,
	0xA724,
	0xA725,
	0xA726,
	0xA727,
	0xA728,
	0xA729,
	0xA72A,
	0xA72B,
	0xA72C,
	0xA72D,
	0xA72E,
	0xA72F,
	0xA732,
	0xA733,
	0xA734,
	0xA735,
	0xA736,
	0xA737,
	0xA738,
	0xA739,
	0xA73A,
	0xA73B,
	0xA73C,
	0xA73D,
	0xA73E,
	0xA73F,
	0xA740,
	0xA741,
	0xA742,
	0xA743,
	0xA744,
	0xA745,
	0xA746,
	0xA747,
	0xA748,
	0xA749,
	0xA74A,
	0xA74B,
	0xA74C,
	0xA74D,
	0xA74E,
	0xA74F,
	0xA750,
	0xA751,
	0xA752,
	0xA753,
	0xA754,
	0xA755,
	0xA756,
	0xA757,
	0xA758,
	0xA759,
	0xA75A,
	0xA75B,
	0xA75C,
	0xA75D,
	0xA75E,
	0xA75F,
	0xA760,
	0xA761,
	0xA762,
	0xA763,
	0xA764,
	0xA765,
	0xA766,
	0xA767,
	0xA768,
	0xA769,
	0xA76A,
	0xA76B,
	0xA76C,
	0xA76D,
	0xA76E,
	0xA76F,
	0xA779,
	0xA77A,
	0xA77B,
	0xA77C,
	0xA77D,
	0xA77F,
	0xA780,
	0xA781,
	0xA782,
	0xA783,
	0xA784,
	0xA785,
	0xA786,
	0xA787,
	0xA788,
	0xA789,
	0xA78B,
	0xA78C,
	0xA78D,
	0xA78E,
	0xA78F,
	0xA790,
	0xA791,
	0xA792,
	0xA793,
	0xA796,
	0xA797,
	0xA798,
	0xA799,
	0xA79A,
	0xA79B,
	0xA79C,
	0xA79D,
	0xA79E,
	0xA79F,
	0xA7A0,
	0xA7A1,
	0xA7A2,
	0xA7A3,
	0xA7A4,
	0xA7A5,
	0xA7A6,
	0xA7A7,
	0xA7A8,
	0xA7A9,
	0xA7AA,
	0xA7AF,
	0xA7B0,
	0xA7B5,
	0xA7B6,
	0xA7B7,
	0xA7B8,
	0xA7F7,
	0xA7F8,
	0xA7FB,
	0xA802,
	0xA803,
	0xA806,
	0xA807,
	0xA80B,
	0xA80C,
	0xA823,
	0xA828,
	0xA840,
	0xA874,
	0xA876,
	0xA878,
	0xA880,
	0xA882,
	0xA8B4,
	0xA8C6,
	0xA8CE,
	0xA8D0,
	0xA8DA,
	0xA8E0,
	0xA8F2,
	0xA8F8,
	0xA8FB,
	0xA8FC,
	0xA8FD,
	0xA8FE,
	0xA900,
	0xA90A,
	0xA926,
	0xA92E,
	0xA92F,
	0xA930,
	0xA947,
	0xA954,
	0xA960,
	0xA97D,
	0xA980,
	0xA984,
	0xA9B3,
	0xA9C1,
	0xA9C8,
	0xA9CA,
	0xA9CF,
	0xA9D0,
	0xA9DA,
	0xA9E0,
	0xA9E5,
	0xA9E6,
	0xA9F0,
	0xA9FA,
	0xA9FF,
	0xAA00,
	0xAA29,
	0xAA37,
	0xAA40,
	0xAA43,
	0xAA44,
	0xAA4C,
	0xAA4E,
	0xAA50,
	0xAA5A,
	0xAA5D,
	0xAA60,
	0xAA77,
	0xAA7A,
	0xAA7B,
	0xAA7E,
	0xAAB0,
	0xAAB1,
	0xAAB2,
	0xAAB5,
	0xAAB7,
	0xAAB9,
	0xAABE,
	0xAAC0,
	0xAAC1,
	0xAAC2,
	0xAAC3,
	0xAADB,
	0xAADE,
	0xAAE0,
	0xAAEB,
	0xAAF0,
	0xAAF2,
	0xAAF5,
	0xAAF7,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB5B,
	0xAB5C,
	0xAB66,
	0xAB70,
	0xABC0,
	0xABE3,
	0xABEB,
	0xABEC,
	0xABEE,
	0xABF0,
	0xABFA,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xF900,
	0xFA6E,
	0xFA70,
	0xFADA,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB1E,
	0xFB1F,
	0xFB29,
	0xFB2A,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFBB2,
	0xFBD3,
	0xFD3E,
	0xFD40,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFC,
	0xFE00,
	0xFE10,
	0xFE12,
	0xFE13,
	0xFE14,
	0xFE17,
	0xFE19,
	0xFE20,
	0xFE30,
	0xFE31,
	0xFE33,
	0xFE35,
	0xFE45,
	0xFE47,
	0xFE49,
	0xFE50,
	0xFE52,
	0xFE53,
	0xFE55,
	0xFE56,
	0xFE58,
	0xFE59,
	0xFE5F,
	0xFE63,
	0xFE64,
	0xFE70,
	0xFE75,
	0xFE76,
	0xFEFD,
	0xFEFF,
	0xFF00,
	0xFF01,
	0xFF02,
	0xFF08,
	0xFF0A,
	0xFF0C,
	0xFF0E,
	0xFF0F,
	0xFF1A,
	0xFF1B,
	0xFF1F,
	0xFF20,
	0xFF21,
	0xFF3B,
	0xFF3C,
	0xFF3D,
	0xFF3E,
	0xFF41,
	0xFF5B,
	0xFF5C,
	0xFF5D,
	0xFF5E,
	0xFF5F,
	0xFF61,
	0xFF62,
	0xFF64,
	0xFF65,
	0xFF66,
	0xFF9E,
	0xFFA0,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0xFFF9,
	0xFFFC,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10140,
	0x10175,
	0x101FD,
	0x101FE,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x102E0,
	0x102E1,
	0x10300,
	0x10320,
	0x10330,
	0x1034B,
	0x10350,
	0x10376,
	0x1037B,
	0x10380,
	0x1039E,
	0x103A0,
	0x103C4,
	0x103C8,
	0x103D0,
	0x103D1,
	0x103D6,
	0x10400,
	0x10428,
	0x10450,
	0x1049E,
	0x104A0,
	0x104AA,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10860,
	0x10877,
	0x10880,
	0x1089F,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x10900,
	0x10916,
	0x10920,
	0x1093A,
	0x10980,
	0x109B8,
	0x109BE,
	0x109C0,
	0x10A00,
	0x10A01,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A10,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A38,
	0x10A3B,
	0x10A3F,
	0x10A40,
	0x10A56,
	0x10A58,
	0x10A60,
	0x10A7D,
	0x10A80,
	0x10A9D,
	0x10AC0,
	0x10AC8,
	0x10AC9,
	0x10AE5,
	0x10AE7,
	0x10B00,
	0x10B36,
	0x10B40,
	0x10B56,
	0x10B60,
	0x10B73,
	0x10B80,
	0x10B92,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x11000,
	0x11003,
	0x11038,
	0x11047,
	0x11049,
	0x11066,
	0x11070,
	0x1107F,
	0x11083,
	0x110B0,
	0x110BB,
	0x110BD,
	0x110BE,
	0x110C2,
	0x110D0,
	0x110E9,
	0x110F0,
	0x110FA,
	0x11100,
	0x11103,
	0x11127,
	0x11135,
	0x11136,
	0x11140,
	0x11141,
	0x11144,
	0x11150,
	0x11173,
	0x11174,
	0x11176,
	0x11177,
	0x11180,
	0x11183,
	0x111B3,
	0x111C1,
	0x111C5,
	0x111C7,
	0x111CA,
	0x111CD,
	0x111CE,
	0x111D0,
	0x111DA,
	0x111DB,
	0x111DC,
	0x111DD,
	0x111DE,
	0x111E0,
	0x11200,
	0x11212,
	0x11213,
	0x1122C,
	0x11238,
	0x1123A,
	0x1123B,
	0x1123D,
	0x1123E,
	0x1123F,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112A9,
	0x112AA,
	0x112B0,
	0x112DF,
	0x112EB,
	0x112F0,
	0x112FA,
	0x11300,
	0x11304,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133C,
	0x1133D,
	0x1133E,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134E,
	0x11350,
	0x11351,
	0x11357,
	0x11358,
	0x1135D,
	0x11362,
	0x11364,
	0x11366,
	0x1136D,
	0x11370,
	0x11375,
	0x11400,
	0x11435,
	0x11447,
	0x1144B,
	0x1144D,
	0x11450,
	0x1145A,
	0x11480,
	0x114B0,
	0x114C4,
	0x114C6,
	0x114C7,
	0x114C8,
	0x114D0,
	0x114DA,
	0x11580,
	0x115AF,
	0x115B6,
	0x115B8,
	0x115C1,
	0x115C2,
	0x115C4,
	0x115C9,
	0x115D8,
	0x115DC,
	0x115DE,
	0x11600,
	0x11630,
	0x11641,
	0x11643,
	0x11644,
	0x11645,
	0x11650,
	0x1165A,
	0x11680,
	0x116AB,
	0x116B8,
	0x116C0,
	0x116CA,
	0x11700,
	0x1171A,
	0x1171D,
	0x1172C,
	0x11730,
	0x1173A,
	0x1173C,
	0x1173F,
	0x118A0,
	0x118C0,
	0x118E0,
	0x118EA,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C2F,
	0x11C37,
	0x11C38,
	0x11C40,
	0x11C41,
	0x11C43,
	0x11C50,
	0x11C5A,
	0x11C72,
	0x11C90,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16A60,
	0x16A6A,
	0x16A6E,
	0x16A70,
	0x16AD0,
	0x16AEE,
	0x16AF0,
	0x16AF5,
	0x16AF6,
	0x16B00,
	0x16B30,
	0x16B37,
	0x16B39,
	0x16B40,
	0x16B44,
	0x16B45,
	0x16B50,
	0x16B5A,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F51,
	0x16F7F,
	0x16F8F,
	0x16F93,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1BC9D,
	0x1BC9F,
	0x1BCA0,
	0x1BCA4,
	0x1D165,
	0x1D16A,
	0x1D16D,
	0x1D173,
	0x1D17B,
	0x1D183,
	0x1D185,
	0x1D18C,
	0x1D1AA,
	0x1D1AE,
	0x1D242,
	0x1D245,
	0x1D400,
	0x1D41A,
	0x1D434,
	0x1D44E,
	0x1D455,
	0x1D456,
	0x1D468,
	0x1D482,
	0x1D49C,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4B6,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D4D0,
	0x1D4EA,
	0x1D504,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D538,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D56C,
	0x1D586,
	0x1D5A0,
	0x1D5BA,
	0x1D5D4,
	0x1D5EE,
	0x1D608,
	0x1D622,
	0x1D63C,
	0x1D656,
	0x1D670,
	0x1D68A,
	0x1D6A6,
	0x1D6A8,
	0x1D6C1,
	0x1D6C2,
	0x1D6DB,
	0x1D6DC,
	0x1D6E2,
	0x1D6FB,
	0x1D6FC,
	0x1D715,
	0x1D716,
	0x1D71C,
	0x1D735,
	0x1D736,
	0x1D74F,
	0x1D750,
	0x1D756,
	0x1D76F,
	0x1D770,
	0x1D789,
	0x1D78A,
	0x1D790,
	0x1D7A9,
	0x1D7AA,
	0x1D7C3,
	0x1D7C4,
	0x1D7CA,
	0x1D7CB,
	0x1D7CC,
	0x1D7CE,
	0x1D800,
	0x1DA00,
	0x1DA37,
	0x1DA3B,
	0x1DA6D,
	0x1DA75,
	0x1DA76,
	0x1DA84,
	0x1DA85,
	0x1DA88,
	0x1DA89,
	0x1DA9B,
	0x1DAA0,
	0x1DAA1,
	0x1DAB0,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E800,
	0x1E8C5,
	0x1E8D0,
	0x1E8D7,
	0x1E900,
	0x1E922,
	0x1E944,
	0x1E94B,
	0x1E950,
	0x1E95A,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x1F130,
	0x1F14A,
	0x1F150,
	0x1F16A,
	0x1F170,
	0x1F18A,
	0x1F676,
	0x1F679,
	0x20000,
	0x2A6D7,
	0x2A700,
	0x2B735,
	0x2B740,
	0x2B81E,
	0x2B820,
	0x2CEA2,
	0x2F800,
	0x2FA1E,
	0xE0001,
	0xE0002,
	0xE0020,
	0xE0080,
	0xE0100,
	0xE01F0
};

#endif	/* defined(PERL_IN_PERL_C) */

#if defined(PERL_IN_REGEXEC_C)

#define SB_ENUM_COUNT 16

typedef enum {
	SB_Other = 0,
	SB_ATerm = 1,
	SB_CR = 2,
	SB_Close = 3,
	SB_Extend = 4,
	SB_Format = 5,
	SB_LF = 6,
	SB_Lower = 7,
	SB_Numeric = 8,
	SB_OLetter = 9,
	SB_SContinue = 10,
	SB_STerm = 11,
	SB_Sep = 12,
	SB_Sp = 13,
	SB_Upper = 14,
	SB_EDGE = 15
} SB_enum;

static const SB_enum _Perl_SB_invmap[] = { /* for EBCDIC 1047 */
	SB_Other,
	SB_Sp,
	SB_Other,
	SB_Sp,
	SB_CR,
	SB_Other,
	SB_LF,
	SB_Other,
	SB_Sep,
	SB_Other,
	SB_Sp,
	SB_Lower,
	SB_Other,
	SB_ATerm,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Lower,
	SB_STerm,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_SContinue,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_SContinue,
	SB_Other,
	SB_STerm,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_SContinue,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Upper,
	SB_Lower,
	SB_Close,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Close,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Upper,
	SB_Format,
	SB_Lower,
	SB_Close,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Numeric,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Lower,
	SB_OLetter,
	SB_Lower,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Extend,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_SContinue,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Format,
	SB_Other,
	SB_SContinue,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Format,
	SB_Other,
	SB_STerm,
	SB_OLetter,
	SB_Extend,
	SB_Numeric,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_STerm,
	SB_OLetter,
	SB_Extend,
	SB_Format,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Numeric,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_STerm,
	SB_Other,
	SB_Format,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_SContinue,
	SB_STerm,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Format,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_STerm,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Close,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Numeric,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Numeric,
	SB_Extend,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_STerm,
	SB_OLetter,
	SB_Sp,
	SB_OLetter,
	SB_Close,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_SContinue,
	SB_STerm,
	SB_Other,
	SB_SContinue,
	SB_STerm,
	SB_Other,
	SB_Extend,
	SB_Format,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_Numeric,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Numeric,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_STerm,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_STerm,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Numeric,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Numeric,
	SB_OLetter,
	SB_STerm,
	SB_Lower,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Lower,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Sp,
	SB_Format,
	SB_Extend,
	SB_Format,
	SB_Other,
	SB_SContinue,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_ATerm,
	SB_Other,
	SB_Sep,
	SB_Format,
	SB_Sp,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Close,
	SB_STerm,
	SB_Other,
	SB_Sp,
	SB_Format,
	SB_Other,
	SB_Format,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Close,
	SB_Lower,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Extend,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_STerm,
	SB_OLetter,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Sp,
	SB_SContinue,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_STerm,
	SB_OLetter,
	SB_Other,
	SB_STerm,
	SB_OLetter,
	SB_Numeric,
	SB_OLetter,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_OLetter,
	SB_Lower,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_Numeric,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Numeric,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_STerm,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_STerm,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_OLetter,
	SB_Extend,
	SB_STerm,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Close,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_SContinue,
	SB_Other,
	SB_SContinue,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_SContinue,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_SContinue,
	SB_ATerm,
	SB_Other,
	SB_SContinue,
	SB_STerm,
	SB_SContinue,
	SB_Close,
	SB_Other,
	SB_SContinue,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Format,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_SContinue,
	SB_ATerm,
	SB_Other,
	SB_SContinue,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Upper,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Lower,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_STerm,
	SB_Close,
	SB_SContinue,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Format,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_STerm,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Format,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_STerm,
	SB_Other,
	SB_Extend,
	SB_STerm,
	SB_Other,
	SB_Numeric,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_STerm,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_STerm,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_STerm,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_STerm,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_STerm,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_STerm,
	SB_Format,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Format,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Format,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other
};

#endif	/* defined(PERL_IN_REGEXEC_C) */

#if defined(PERL_IN_PERL_C)

static const UV _Perl_WB_invlist[] = { /* for EBCDIC 1047 */
	1680,	/* Number of elements */
	148565664, /* Version and data structure type */
	0,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x5,
	0x6,
	0xB,
	0xD,
	0xE,
	0x15,
	0x16,
	0x25,
	0x26,
	0x40,
	0x42,
	0x4A,
	0x4B,
	0x4C,
	0x51,
	0x5A,
	0x5E,
	0x5F,
	0x62,
	0x6A,
	0x6B,
	0x6C,
	0x6D,
	0x6E,
	0x70,
	0x79,
	0x7A,
	0x7B,
	0x7D,
	0x7E,
	0x7F,
	0x80,
	0x8A,
	0x8C,
	0x8F,
	0x91,
	0x9D,
	0x9E,
	0x9F,
	0xA0,
	0xA1,
	0xA2,
	0xAA,
	0xAC,
	0xAD,
	0xAE,
	0xAF,
	0xB3,
	0xB4,
	0xBA,
	0xBB,
	0xC1,
	0xCA,
	0xCB,
	0xD0,
	0xD1,
	0xDA,
	0xDB,
	0xE0,
	0xE2,
	0xEA,
	0xEB,
	0xF0,
	0xFA,
	0xFB,
	0xFF,
	0x100,
	0x2C2,
	0x2C6,
	0x2D2,
	0x2D7,
	0x2D8,
	0x2E0,
	0x2E5,
	0x2EC,
	0x2ED,
	0x2EE,
	0x2EF,
	0x300,
	0x370,
	0x375,
	0x376,
	0x378,
	0x37A,
	0x37E,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x3F6,
	0x3F7,
	0x482,
	0x483,
	0x48A,
	0x530,
	0x531,
	0x557,
	0x559,
	0x55A,
	0x561,
	0x588,
	0x589,
	0x58A,
	0x591,
	0x5BE,
	0x5BF,
	0x5C0,
	0x5C1,
	0x5C3,
	0x5C4,
	0x5C6,
	0x5C7,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F3,
	0x5F4,
	0x5F5,
	0x600,
	0x606,
	0x60C,
	0x60E,
	0x610,
	0x61B,
	0x61C,
	0x61D,
	0x620,
	0x64B,
	0x660,
	0x66A,
	0x66B,
	0x66C,
	0x66D,
	0x66E,
	0x670,
	0x671,
	0x6D4,
	0x6D5,
	0x6D6,
	0x6DD,
	0x6DE,
	0x6DF,
	0x6E5,
	0x6E7,
	0x6E9,
	0x6EA,
	0x6EE,
	0x6F0,
	0x6FA,
	0x6FD,
	0x6FF,
	0x700,
	0x70F,
	0x710,
	0x711,
	0x712,
	0x730,
	0x74B,
	0x74D,
	0x7A6,
	0x7B1,
	0x7B2,
	0x7C0,
	0x7CA,
	0x7EB,
	0x7F4,
	0x7F6,
	0x7F8,
	0x7F9,
	0x7FA,
	0x7FB,
	0x800,
	0x816,
	0x81A,
	0x81B,
	0x824,
	0x825,
	0x828,
	0x829,
	0x82E,
	0x840,
	0x859,
	0x85C,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x8D4,
	0x8E2,
	0x8E3,
	0x904,
	0x93A,
	0x93D,
	0x93E,
	0x950,
	0x951,
	0x958,
	0x962,
	0x964,
	0x966,
	0x970,
	0x971,
	0x981,
	0x984,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BC,
	0x9BD,
	0x9BE,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CE,
	0x9CF,
	0x9D7,
	0x9D8,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E2,
	0x9E4,
	0x9E6,
	0x9F0,
	0x9F2,
	0xA01,
	0xA04,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA3C,
	0xA3D,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4E,
	0xA51,
	0xA52,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA66,
	0xA70,
	0xA72,
	0xA75,
	0xA76,
	0xA81,
	0xA84,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABC,
	0xABD,
	0xABE,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACE,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE2,
	0xAE4,
	0xAE6,
	0xAF0,
	0xAF9,
	0xAFA,
	0xB01,
	0xB04,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3C,
	0xB3D,
	0xB3E,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4E,
	0xB56,
	0xB58,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB62,
	0xB64,
	0xB66,
	0xB70,
	0xB71,
	0xB72,
	0xB82,
	0xB83,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCE,
	0xBD0,
	0xBD1,
	0xBD7,
	0xBD8,
	0xBE6,
	0xBF0,
	0xC00,
	0xC04,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC3E,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4E,
	0xC55,
	0xC57,
	0xC58,
	0xC5B,
	0xC60,
	0xC62,
	0xC64,
	0xC66,
	0xC70,
	0xC80,
	0xC81,
	0xC84,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBC,
	0xCBD,
	0xCBE,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCE,
	0xCD5,
	0xCD7,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE2,
	0xCE4,
	0xCE6,
	0xCF0,
	0xCF1,
	0xCF3,
	0xD01,
	0xD04,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD3E,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD4E,
	0xD4F,
	0xD54,
	0xD57,
	0xD58,
	0xD5F,
	0xD62,
	0xD64,
	0xD66,
	0xD70,
	0xD7A,
	0xD80,
	0xD82,
	0xD84,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xDCA,
	0xDCB,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDE6,
	0xDF0,
	0xDF2,
	0xDF4,
	0xE31,
	0xE32,
	0xE34,
	0xE3B,
	0xE47,
	0xE4F,
	0xE50,
	0xE5A,
	0xEB1,
	0xEB2,
	0xEB4,
	0xEBA,
	0xEBB,
	0xEBD,
	0xEC8,
	0xECE,
	0xED0,
	0xEDA,
	0xF00,
	0xF01,
	0xF18,
	0xF1A,
	0xF20,
	0xF2A,
	0xF35,
	0xF36,
	0xF37,
	0xF38,
	0xF39,
	0xF3A,
	0xF3E,
	0xF40,
	0xF48,
	0xF49,
	0xF6D,
	0xF71,
	0xF85,
	0xF86,
	0xF88,
	0xF8D,
	0xF98,
	0xF99,
	0xFBD,
	0xFC6,
	0xFC7,
	0x102B,
	0x103F,
	0x1040,
	0x104A,
	0x1056,
	0x105A,
	0x105E,
	0x1061,
	0x1062,
	0x1065,
	0x1067,
	0x106E,
	0x1071,
	0x1075,
	0x1082,
	0x108E,
	0x108F,
	0x1090,
	0x109A,
	0x109E,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x10FB,
	0x10FC,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x135D,
	0x1360,
	0x1380,
	0x1390,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1401,
	0x166D,
	0x166F,
	0x1680,
	0x1681,
	0x169B,
	0x16A0,
	0x16EB,
	0x16EE,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1712,
	0x1715,
	0x1720,
	0x1732,
	0x1735,
	0x1740,
	0x1752,
	0x1754,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1772,
	0x1774,
	0x17B4,
	0x17D4,
	0x17DD,
	0x17DE,
	0x17E0,
	0x17EA,
	0x180B,
	0x180E,
	0x180F,
	0x1810,
	0x181A,
	0x1820,
	0x1878,
	0x1880,
	0x1885,
	0x1887,
	0x18A9,
	0x18AA,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1920,
	0x192C,
	0x1930,
	0x193C,
	0x1946,
	0x1950,
	0x19D0,
	0x19DA,
	0x1A00,
	0x1A17,
	0x1A1C,
	0x1A55,
	0x1A5F,
	0x1A60,
	0x1A7D,
	0x1A7F,
	0x1A80,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1AB0,
	0x1ABF,
	0x1B00,
	0x1B05,
	0x1B34,
	0x1B45,
	0x1B4C,
	0x1B50,
	0x1B5A,
	0x1B6B,
	0x1B74,
	0x1B80,
	0x1B83,
	0x1BA1,
	0x1BAE,
	0x1BB0,
	0x1BBA,
	0x1BE6,
	0x1BF4,
	0x1C00,
	0x1C24,
	0x1C38,
	0x1C40,
	0x1C4A,
	0x1C4D,
	0x1C50,
	0x1C5A,
	0x1C7E,
	0x1C80,
	0x1C89,
	0x1CD0,
	0x1CD3,
	0x1CD4,
	0x1CE9,
	0x1CED,
	0x1CEE,
	0x1CF2,
	0x1CF5,
	0x1CF7,
	0x1CF8,
	0x1CFA,
	0x1D00,
	0x1DC0,
	0x1DF6,
	0x1DFB,
	0x1E00,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FBD,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FCD,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FE0,
	0x1FED,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFD,
	0x2000,
	0x200B,
	0x200C,
	0x200D,
	0x200E,
	0x2010,
	0x2018,
	0x201A,
	0x2024,
	0x2025,
	0x2027,
	0x2028,
	0x202A,
	0x202F,
	0x2030,
	0x203F,
	0x2041,
	0x2044,
	0x2045,
	0x2054,
	0x2055,
	0x205F,
	0x2060,
	0x2065,
	0x2066,
	0x2070,
	0x2071,
	0x2072,
	0x207F,
	0x2080,
	0x2090,
	0x209D,
	0x20D0,
	0x20F1,
	0x2102,
	0x2103,
	0x2107,
	0x2108,
	0x210A,
	0x2114,
	0x2115,
	0x2116,
	0x2119,
	0x211E,
	0x2124,
	0x2125,
	0x2126,
	0x2127,
	0x2128,
	0x2129,
	0x212A,
	0x212E,
	0x212F,
	0x213A,
	0x213C,
	0x2140,
	0x2145,
	0x214A,
	0x214E,
	0x214F,
	0x2160,
	0x2189,
	0x24B6,
	0x24EA,
	0x261D,
	0x261E,
	0x26F9,
	0x26FA,
	0x270A,
	0x270E,
	0x2764,
	0x2765,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CE5,
	0x2CEB,
	0x2CEF,
	0x2CF2,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D70,
	0x2D7F,
	0x2D80,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x2DE0,
	0x2E00,
	0x2E2F,
	0x2E30,
	0x3000,
	0x3001,
	0x3005,
	0x3006,
	0x302A,
	0x3030,
	0x3031,
	0x3036,
	0x303B,
	0x303D,
	0x3099,
	0x309B,
	0x309D,
	0x30A0,
	0x30FB,
	0x30FC,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x31A0,
	0x31BB,
	0x31F0,
	0x3200,
	0x32D0,
	0x32FF,
	0x3300,
	0x3358,
	0xA000,
	0xA48D,
	0xA4D0,
	0xA4FE,
	0xA500,
	0xA60D,
	0xA610,
	0xA620,
	0xA62A,
	0xA62C,
	0xA640,
	0xA66F,
	0xA673,
	0xA674,
	0xA67E,
	0xA67F,
	0xA69E,
	0xA6A0,
	0xA6F0,
	0xA6F2,
	0xA717,
	0xA720,
	0xA722,
	0xA789,
	0xA78B,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F7,
	0xA802,
	0xA803,
	0xA806,
	0xA807,
	0xA80B,
	0xA80C,
	0xA823,
	0xA828,
	0xA840,
	0xA874,
	0xA880,
	0xA882,
	0xA8B4,
	0xA8C6,
	0xA8D0,
	0xA8DA,
	0xA8E0,
	0xA8F2,
	0xA8F8,
	0xA8FB,
	0xA8FC,
	0xA8FD,
	0xA8FE,
	0xA900,
	0xA90A,
	0xA926,
	0xA92E,
	0xA930,
	0xA947,
	0xA954,
	0xA960,
	0xA97D,
	0xA980,
	0xA984,
	0xA9B3,
	0xA9C1,
	0xA9CF,
	0xA9D0,
	0xA9DA,
	0xA9E5,
	0xA9E6,
	0xA9F0,
	0xA9FA,
	0xAA00,
	0xAA29,
	0xAA37,
	0xAA40,
	0xAA43,
	0xAA44,
	0xAA4C,
	0xAA4E,
	0xAA50,
	0xAA5A,
	0xAA7B,
	0xAA7E,
	0xAAB0,
	0xAAB1,
	0xAAB2,
	0xAAB5,
	0xAAB7,
	0xAAB9,
	0xAABE,
	0xAAC0,
	0xAAC1,
	0xAAC2,
	0xAAE0,
	0xAAEB,
	0xAAF0,
	0xAAF2,
	0xAAF5,
	0xAAF7,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB5B,
	0xAB5C,
	0xAB66,
	0xAB70,
	0xABE3,
	0xABEB,
	0xABEC,
	0xABEE,
	0xABF0,
	0xABFA,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB1E,
	0xFB1F,
	0xFB29,
	0xFB2A,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFB50,
	0xFBB2,
	0xFBD3,
	0xFD3E,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFC,
	0xFE00,
	0xFE10,
	0xFE11,
	0xFE13,
	0xFE14,
	0xFE15,
	0xFE20,
	0xFE30,
	0xFE33,
	0xFE35,
	0xFE4D,
	0xFE50,
	0xFE51,
	0xFE52,
	0xFE53,
	0xFE54,
	0xFE55,
	0xFE56,
	0xFE70,
	0xFE75,
	0xFE76,
	0xFEFD,
	0xFEFF,
	0xFF00,
	0xFF07,
	0xFF08,
	0xFF0C,
	0xFF0D,
	0xFF0E,
	0xFF0F,
	0xFF1A,
	0xFF1B,
	0xFF1C,
	0xFF21,
	0xFF3B,
	0xFF3F,
	0xFF40,
	0xFF41,
	0xFF5B,
	0xFF66,
	0xFF9E,
	0xFFA0,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0xFFF9,
	0xFFFC,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10140,
	0x10175,
	0x101FD,
	0x101FE,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x102E0,
	0x102E1,
	0x10300,
	0x10320,
	0x10330,
	0x1034B,
	0x10350,
	0x10376,
	0x1037B,
	0x10380,
	0x1039E,
	0x103A0,
	0x103C4,
	0x103C8,
	0x103D0,
	0x103D1,
	0x103D6,
	0x10400,
	0x1049E,
	0x104A0,
	0x104AA,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10860,
	0x10877,
	0x10880,
	0x1089F,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x10900,
	0x10916,
	0x10920,
	0x1093A,
	0x10980,
	0x109B8,
	0x109BE,
	0x109C0,
	0x10A00,
	0x10A01,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A10,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A38,
	0x10A3B,
	0x10A3F,
	0x10A40,
	0x10A60,
	0x10A7D,
	0x10A80,
	0x10A9D,
	0x10AC0,
	0x10AC8,
	0x10AC9,
	0x10AE5,
	0x10AE7,
	0x10B00,
	0x10B36,
	0x10B40,
	0x10B56,
	0x10B60,
	0x10B73,
	0x10B80,
	0x10B92,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x11000,
	0x11003,
	0x11038,
	0x11047,
	0x11066,
	0x11070,
	0x1107F,
	0x11083,
	0x110B0,
	0x110BB,
	0x110BD,
	0x110BE,
	0x110D0,
	0x110E9,
	0x110F0,
	0x110FA,
	0x11100,
	0x11103,
	0x11127,
	0x11135,
	0x11136,
	0x11140,
	0x11150,
	0x11173,
	0x11174,
	0x11176,
	0x11177,
	0x11180,
	0x11183,
	0x111B3,
	0x111C1,
	0x111C5,
	0x111CA,
	0x111CD,
	0x111D0,
	0x111DA,
	0x111DB,
	0x111DC,
	0x111DD,
	0x11200,
	0x11212,
	0x11213,
	0x1122C,
	0x11238,
	0x1123E,
	0x1123F,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112A9,
	0x112B0,
	0x112DF,
	0x112EB,
	0x112F0,
	0x112FA,
	0x11300,
	0x11304,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133C,
	0x1133D,
	0x1133E,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134E,
	0x11350,
	0x11351,
	0x11357,
	0x11358,
	0x1135D,
	0x11362,
	0x11364,
	0x11366,
	0x1136D,
	0x11370,
	0x11375,
	0x11400,
	0x11435,
	0x11447,
	0x1144B,
	0x11450,
	0x1145A,
	0x11480,
	0x114B0,
	0x114C4,
	0x114C6,
	0x114C7,
	0x114C8,
	0x114D0,
	0x114DA,
	0x11580,
	0x115AF,
	0x115B6,
	0x115B8,
	0x115C1,
	0x115D8,
	0x115DC,
	0x115DE,
	0x11600,
	0x11630,
	0x11641,
	0x11644,
	0x11645,
	0x11650,
	0x1165A,
	0x11680,
	0x116AB,
	0x116B8,
	0x116C0,
	0x116CA,
	0x1171D,
	0x1172C,
	0x11730,
	0x1173A,
	0x118A0,
	0x118E0,
	0x118EA,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C2F,
	0x11C37,
	0x11C38,
	0x11C40,
	0x11C41,
	0x11C50,
	0x11C5A,
	0x11C72,
	0x11C90,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16A60,
	0x16A6A,
	0x16AD0,
	0x16AEE,
	0x16AF0,
	0x16AF5,
	0x16B00,
	0x16B30,
	0x16B37,
	0x16B40,
	0x16B44,
	0x16B50,
	0x16B5A,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F51,
	0x16F7F,
	0x16F8F,
	0x16F93,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x1B000,
	0x1B001,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1BC9D,
	0x1BC9F,
	0x1BCA0,
	0x1BCA4,
	0x1D165,
	0x1D16A,
	0x1D16D,
	0x1D173,
	0x1D17B,
	0x1D183,
	0x1D185,
	0x1D18C,
	0x1D1AA,
	0x1D1AE,
	0x1D242,
	0x1D245,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D6C1,
	0x1D6C2,
	0x1D6DB,
	0x1D6DC,
	0x1D6FB,
	0x1D6FC,
	0x1D715,
	0x1D716,
	0x1D735,
	0x1D736,
	0x1D74F,
	0x1D750,
	0x1D76F,
	0x1D770,
	0x1D789,
	0x1D78A,
	0x1D7A9,
	0x1D7AA,
	0x1D7C3,
	0x1D7C4,
	0x1D7CC,
	0x1D7CE,
	0x1D800,
	0x1DA00,
	0x1DA37,
	0x1DA3B,
	0x1DA6D,
	0x1DA75,
	0x1DA76,
	0x1DA84,
	0x1DA85,
	0x1DA9B,
	0x1DAA0,
	0x1DAA1,
	0x1DAB0,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E800,
	0x1E8C5,
	0x1E8D0,
	0x1E8D7,
	0x1E900,
	0x1E944,
	0x1E94B,
	0x1E950,
	0x1E95A,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x1F130,
	0x1F14A,
	0x1F150,
	0x1F16A,
	0x1F170,
	0x1F18A,
	0x1F1E6,
	0x1F200,
	0x1F385,
	0x1F386,
	0x1F3C3,
	0x1F3C5,
	0x1F3CA,
	0x1F3CC,
	0x1F3FB,
	0x1F400,
	0x1F442,
	0x1F444,
	0x1F446,
	0x1F451,
	0x1F466,
	0x1F46A,
	0x1F46E,
	0x1F46F,
	0x1F470,
	0x1F479,
	0x1F47C,
	0x1F47D,
	0x1F481,
	0x1F484,
	0x1F485,
	0x1F488,
	0x1F48B,
	0x1F48C,
	0x1F4AA,
	0x1F4AB,
	0x1F575,
	0x1F576,
	0x1F57A,
	0x1F57B,
	0x1F590,
	0x1F591,
	0x1F595,
	0x1F597,
	0x1F5E8,
	0x1F5E9,
	0x1F645,
	0x1F648,
	0x1F64B,
	0x1F650,
	0x1F6A3,
	0x1F6A4,
	0x1F6B4,
	0x1F6B7,
	0x1F6C0,
	0x1F6C1,
	0x1F918,
	0x1F91F,
	0x1F926,
	0x1F927,
	0x1F930,
	0x1F931,
	0x1F933,
	0x1F93A,
	0x1F93C,
	0x1F93F,
	0xE0001,
	0xE0002,
	0xE0020,
	0xE0080,
	0xE0100,
	0xE01F0
};

#endif	/* defined(PERL_IN_PERL_C) */

#if defined(PERL_IN_REGEXEC_C)

#define WB_ENUM_COUNT 25

typedef enum {
	WB_Other = 0,
	WB_ALetter = 1,
	WB_CR = 2,
	WB_Double_Quote = 3,
	WB_E_Base = 4,
	WB_E_Base_GAZ = 5,
	WB_E_Modifier = 6,
	WB_Extend = 7,
	WB_ExtendNumLet = 8,
	WB_Format = 9,
	WB_Glue_After_Zwj = 10,
	WB_Hebrew_Letter = 11,
	WB_Katakana = 12,
	WB_LF = 13,
	WB_MidLetter = 14,
	WB_MidNum = 15,
	WB_MidNumLet = 16,
	WB_Newline = 17,
	WB_Numeric = 18,
	WB_Perl_Tailored_HSpace = 19,
	WB_Regional_Indicator = 20,
	WB_Single_Quote = 21,
	WB_ZWJ = 22,
	WB_EDGE = 23,
	WB_UNKNOWN = 24
} WB_enum;

static const WB_enum _Perl_WB_invmap[] = { /* for EBCDIC 1047 */
	WB_Other,
	WB_Perl_Tailored_HSpace,
	WB_Other,
	WB_Newline,
	WB_CR,
	WB_Other,
	WB_LF,
	WB_Other,
	WB_Newline,
	WB_Other,
	WB_Perl_Tailored_HSpace,
	WB_ALetter,
	WB_Other,
	WB_MidNumLet,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_MidNum,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_MidNum,
	WB_Other,
	WB_ExtendNumLet,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_MidLetter,
	WB_Other,
	WB_Single_Quote,
	WB_Other,
	WB_Double_Quote,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_MidLetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Format,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_MidLetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_MidNum,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_MidLetter,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_MidNum,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Hebrew_Letter,
	WB_Other,
	WB_Hebrew_Letter,
	WB_ALetter,
	WB_MidLetter,
	WB_Other,
	WB_Format,
	WB_Other,
	WB_MidNum,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Format,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Numeric,
	WB_Other,
	WB_Numeric,
	WB_MidNum,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Format,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Numeric,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Format,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_MidNum,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Format,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Numeric,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Perl_Tailored_HSpace,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Format,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Numeric,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Numeric,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Numeric,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Perl_Tailored_HSpace,
	WB_Other,
	WB_Extend,
	WB_ZWJ,
	WB_Format,
	WB_Other,
	WB_MidNumLet,
	WB_Other,
	WB_MidNumLet,
	WB_Other,
	WB_MidLetter,
	WB_Newline,
	WB_Format,
	WB_ExtendNumLet,
	WB_Other,
	WB_ExtendNumLet,
	WB_Other,
	WB_MidNum,
	WB_Other,
	WB_ExtendNumLet,
	WB_Other,
	WB_Perl_Tailored_HSpace,
	WB_Format,
	WB_Other,
	WB_Format,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_Glue_After_Zwj,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Perl_Tailored_HSpace,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Katakana,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Katakana,
	WB_Other,
	WB_Katakana,
	WB_Other,
	WB_Katakana,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Katakana,
	WB_Other,
	WB_Katakana,
	WB_Other,
	WB_Katakana,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Numeric,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Hebrew_Letter,
	WB_Extend,
	WB_Hebrew_Letter,
	WB_Other,
	WB_Hebrew_Letter,
	WB_Other,
	WB_Hebrew_Letter,
	WB_Other,
	WB_Hebrew_Letter,
	WB_Other,
	WB_Hebrew_Letter,
	WB_Other,
	WB_Hebrew_Letter,
	WB_Other,
	WB_Hebrew_Letter,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_MidNum,
	WB_Other,
	WB_MidLetter,
	WB_MidNum,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ExtendNumLet,
	WB_Other,
	WB_ExtendNumLet,
	WB_MidNum,
	WB_Other,
	WB_MidNumLet,
	WB_Other,
	WB_MidNum,
	WB_MidLetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Format,
	WB_Other,
	WB_MidNumLet,
	WB_Other,
	WB_MidNum,
	WB_Other,
	WB_MidNumLet,
	WB_Other,
	WB_MidLetter,
	WB_MidNum,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ExtendNumLet,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Katakana,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Format,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Format,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Katakana,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Format,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Format,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Regional_Indicator,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Modifier,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base_GAZ,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_Glue_After_Zwj,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_Glue_After_Zwj,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_Format,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other
};

#endif	/* defined(PERL_IN_REGEXEC_C) */

#if defined(PERL_IN_REGCOMP_C)

static const UV UpperLatin1_invlist[] = { /* for EBCDIC 1047 */
	55,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x4,
	0x5,
	0x6,
	0x7,
	0x8,
	0xB,
	0x14,
	0x15,
	0x17,
	0x18,
	0x1A,
	0x1C,
	0x20,
	0x26,
	0x28,
	0x2D,
	0x30,
	0x32,
	0x33,
	0x37,
	0x38,
	0x3C,
	0x3E,
	0x3F,
	0x41,
	0x4B,
	0x51,
	0x5A,
	0x62,
	0x6B,
	0x70,
	0x79,
	0x80,
	0x81,
	0x8A,
	0x91,
	0x9A,
	0xA1,
	0xAA,
	0xAD,
	0xAE,
	0xBD,
	0xBE,
	0xC0,
	0xCA,
	0xD0,
	0xDA,
	0xE0,
	0xE1,
	0xE2,
	0xEA,
	0xF0,
	0xFA,
	0x100
};

#endif	/* defined(PERL_IN_REGCOMP_C) */

#if defined(PERL_IN_PERL_C)

static const UV VertSpace_invlist[] = { /* for EBCDIC 1047 */
	9,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0xB,
	0xE,
	0x15,
	0x16,
	0x25,
	0x26,
	0x2028,
	0x202A
};

static const UV XPerlSpace_invlist[] = { /* for EBCDIC 1047 */
	23,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x5,
	0x6,
	0xB,
	0xE,
	0x15,
	0x16,
	0x25,
	0x26,
	0x40,
	0x42,
	0x1680,
	0x1681,
	0x2000,
	0x200B,
	0x2028,
	0x202A,
	0x202F,
	0x2030,
	0x205F,
	0x2060,
	0x3000,
	0x3001
};

static const UV XPosixAlnum_invlist[] = { /* for EBCDIC 1047 */
	1381,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x42,
	0x4A,
	0x51,
	0x5A,
	0x62,
	0x6A,
	0x70,
	0x79,
	0x80,
	0x8A,
	0x8C,
	0x8F,
	0x91,
	0x9D,
	0x9E,
	0x9F,
	0xA0,
	0xA1,
	0xA2,
	0xAA,
	0xAC,
	0xAD,
	0xAE,
	0xAF,
	0xBA,
	0xBB,
	0xC1,
	0xCA,
	0xCB,
	0xD0,
	0xD1,
	0xDA,
	0xDB,
	0xE0,
	0xE2,
	0xEA,
	0xEB,
	0xFA,
	0xFB,
	0xFF,
	0x100,
	0x2C2,
	0x2C6,
	0x2D2,
	0x2E0,
	0x2E5,
	0x2EC,
	0x2ED,
	0x2EE,
	0x2EF,
	0x345,
	0x346,
	0x370,
	0x375,
	0x376,
	0x378,
	0x37A,
	0x37E,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x3F6,
	0x3F7,
	0x482,
	0x48A,
	0x530,
	0x531,
	0x557,
	0x559,
	0x55A,
	0x561,
	0x588,
	0x5B0,
	0x5BE,
	0x5BF,
	0x5C0,
	0x5C1,
	0x5C3,
	0x5C4,
	0x5C6,
	0x5C7,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F3,
	0x610,
	0x61B,
	0x620,
	0x658,
	0x659,
	0x66A,
	0x66E,
	0x6D4,
	0x6D5,
	0x6DD,
	0x6E1,
	0x6E9,
	0x6ED,
	0x6FD,
	0x6FF,
	0x700,
	0x710,
	0x740,
	0x74D,
	0x7B2,
	0x7C0,
	0x7EB,
	0x7F4,
	0x7F6,
	0x7FA,
	0x7FB,
	0x800,
	0x818,
	0x81A,
	0x82D,
	0x840,
	0x859,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x8D4,
	0x8E0,
	0x8E3,
	0x8EA,
	0x8F0,
	0x93C,
	0x93D,
	0x94D,
	0x94E,
	0x951,
	0x955,
	0x964,
	0x966,
	0x970,
	0x971,
	0x984,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BD,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CD,
	0x9CE,
	0x9CF,
	0x9D7,
	0x9D8,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E4,
	0x9E6,
	0x9F2,
	0xA01,
	0xA04,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4D,
	0xA51,
	0xA52,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA66,
	0xA76,
	0xA81,
	0xA84,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABD,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACD,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE4,
	0xAE6,
	0xAF0,
	0xAF9,
	0xAFA,
	0xB01,
	0xB04,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3D,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4D,
	0xB56,
	0xB58,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB64,
	0xB66,
	0xB70,
	0xB71,
	0xB72,
	0xB82,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCD,
	0xBD0,
	0xBD1,
	0xBD7,
	0xBD8,
	0xBE6,
	0xBF0,
	0xC00,
	0xC04,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4D,
	0xC55,
	0xC57,
	0xC58,
	0xC5B,
	0xC60,
	0xC64,
	0xC66,
	0xC70,
	0xC80,
	0xC84,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBD,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCD,
	0xCD5,
	0xCD7,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE4,
	0xCE6,
	0xCF0,
	0xCF1,
	0xCF3,
	0xD01,
	0xD04,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD4D,
	0xD4E,
	0xD4F,
	0xD54,
	0xD58,
	0xD5F,
	0xD64,
	0xD66,
	0xD70,
	0xD7A,
	0xD80,
	0xD82,
	0xD84,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDE6,
	0xDF0,
	0xDF2,
	0xDF4,
	0xE01,
	0xE3B,
	0xE40,
	0xE47,
	0xE4D,
	0xE4E,
	0xE50,
	0xE5A,
	0xE81,
	0xE83,
	0xE84,
	0xE85,
	0xE87,
	0xE89,
	0xE8A,
	0xE8B,
	0xE8D,
	0xE8E,
	0xE94,
	0xE98,
	0xE99,
	0xEA0,
	0xEA1,
	0xEA4,
	0xEA5,
	0xEA6,
	0xEA7,
	0xEA8,
	0xEAA,
	0xEAC,
	0xEAD,
	0xEBA,
	0xEBB,
	0xEBE,
	0xEC0,
	0xEC5,
	0xEC6,
	0xEC7,
	0xECD,
	0xECE,
	0xED0,
	0xEDA,
	0xEDC,
	0xEE0,
	0xF00,
	0xF01,
	0xF20,
	0xF2A,
	0xF40,
	0xF48,
	0xF49,
	0xF6D,
	0xF71,
	0xF82,
	0xF88,
	0xF98,
	0xF99,
	0xFBD,
	0x1000,
	0x1037,
	0x1038,
	0x1039,
	0x103B,
	0x104A,
	0x1050,
	0x1063,
	0x1065,
	0x1069,
	0x106E,
	0x1087,
	0x108E,
	0x108F,
	0x1090,
	0x109A,
	0x109C,
	0x109E,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x10FB,
	0x10FC,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x135F,
	0x1360,
	0x1380,
	0x1390,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1401,
	0x166D,
	0x166F,
	0x1680,
	0x1681,
	0x169B,
	0x16A0,
	0x16EB,
	0x16EE,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1714,
	0x1720,
	0x1734,
	0x1740,
	0x1754,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1772,
	0x1774,
	0x1780,
	0x17B4,
	0x17B6,
	0x17C9,
	0x17D7,
	0x17D8,
	0x17DC,
	0x17DD,
	0x17E0,
	0x17EA,
	0x1810,
	0x181A,
	0x1820,
	0x1878,
	0x1880,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1920,
	0x192C,
	0x1930,
	0x1939,
	0x1946,
	0x196E,
	0x1970,
	0x1975,
	0x1980,
	0x19AC,
	0x19B0,
	0x19CA,
	0x19D0,
	0x19DA,
	0x1A00,
	0x1A1C,
	0x1A20,
	0x1A5F,
	0x1A61,
	0x1A75,
	0x1A80,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1AA7,
	0x1AA8,
	0x1B00,
	0x1B34,
	0x1B35,
	0x1B44,
	0x1B45,
	0x1B4C,
	0x1B50,
	0x1B5A,
	0x1B80,
	0x1BAA,
	0x1BAC,
	0x1BE6,
	0x1BE7,
	0x1BF2,
	0x1C00,
	0x1C36,
	0x1C40,
	0x1C4A,
	0x1C4D,
	0x1C7E,
	0x1C80,
	0x1C89,
	0x1CE9,
	0x1CED,
	0x1CEE,
	0x1CF4,
	0x1CF5,
	0x1CF7,
	0x1D00,
	0x1DC0,
	0x1DE7,
	0x1DF5,
	0x1E00,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FBD,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FCD,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FE0,
	0x1FED,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFD,
	0x2071,
	0x2072,
	0x207F,
	0x2080,
	0x2090,
	0x209D,
	0x2102,
	0x2103,
	0x2107,
	0x2108,
	0x210A,
	0x2114,
	0x2115,
	0x2116,
	0x2119,
	0x211E,
	0x2124,
	0x2125,
	0x2126,
	0x2127,
	0x2128,
	0x2129,
	0x212A,
	0x212E,
	0x212F,
	0x213A,
	0x213C,
	0x2140,
	0x2145,
	0x214A,
	0x214E,
	0x214F,
	0x2160,
	0x2189,
	0x24B6,
	0x24EA,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CE5,
	0x2CEB,
	0x2CEF,
	0x2CF2,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D70,
	0x2D80,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x2DE0,
	0x2E00,
	0x2E2F,
	0x2E30,
	0x3005,
	0x3008,
	0x3021,
	0x302A,
	0x3031,
	0x3036,
	0x3038,
	0x303D,
	0x3041,
	0x3097,
	0x309D,
	0x30A0,
	0x30A1,
	0x30FB,
	0x30FC,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x31A0,
	0x31BB,
	0x31F0,
	0x3200,
	0x3400,
	0x4DB6,
	0x4E00,
	0x9FD6,
	0xA000,
	0xA48D,
	0xA4D0,
	0xA4FE,
	0xA500,
	0xA60D,
	0xA610,
	0xA62C,
	0xA640,
	0xA66F,
	0xA674,
	0xA67C,
	0xA67F,
	0xA6F0,
	0xA717,
	0xA720,
	0xA722,
	0xA789,
	0xA78B,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F7,
	0xA802,
	0xA803,
	0xA806,
	0xA807,
	0xA80B,
	0xA80C,
	0xA828,
	0xA840,
	0xA874,
	0xA880,
	0xA8C4,
	0xA8C5,
	0xA8C6,
	0xA8D0,
	0xA8DA,
	0xA8F2,
	0xA8F8,
	0xA8FB,
	0xA8FC,
	0xA8FD,
	0xA8FE,
	0xA900,
	0xA92B,
	0xA930,
	0xA953,
	0xA960,
	0xA97D,
	0xA980,
	0xA9B3,
	0xA9B4,
	0xA9C0,
	0xA9CF,
	0xA9DA,
	0xA9E0,
	0xA9E5,
	0xA9E6,
	0xA9FF,
	0xAA00,
	0xAA37,
	0xAA40,
	0xAA4E,
	0xAA50,
	0xAA5A,
	0xAA60,
	0xAA77,
	0xAA7A,
	0xAA7B,
	0xAA7E,
	0xAABF,
	0xAAC0,
	0xAAC1,
	0xAAC2,
	0xAAC3,
	0xAADB,
	0xAADE,
	0xAAE0,
	0xAAF0,
	0xAAF2,
	0xAAF6,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB5B,
	0xAB5C,
	0xAB66,
	0xAB70,
	0xABEB,
	0xABF0,
	0xABFA,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xF900,
	0xFA6E,
	0xFA70,
	0xFADA,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB29,
	0xFB2A,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFBB2,
	0xFBD3,
	0xFD3E,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFC,
	0xFE70,
	0xFE75,
	0xFE76,
	0xFEFD,
	0xFF10,
	0xFF1A,
	0xFF21,
	0xFF3B,
	0xFF41,
	0xFF5B,
	0xFF66,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10140,
	0x10175,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x10300,
	0x10320,
	0x10330,
	0x1034B,
	0x10350,
	0x1037B,
	0x10380,
	0x1039E,
	0x103A0,
	0x103C4,
	0x103C8,
	0x103D0,
	0x103D1,
	0x103D6,
	0x10400,
	0x1049E,
	0x104A0,
	0x104AA,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10860,
	0x10877,
	0x10880,
	0x1089F,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x10900,
	0x10916,
	0x10920,
	0x1093A,
	0x10980,
	0x109B8,
	0x109BE,
	0x109C0,
	0x10A00,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A60,
	0x10A7D,
	0x10A80,
	0x10A9D,
	0x10AC0,
	0x10AC8,
	0x10AC9,
	0x10AE5,
	0x10B00,
	0x10B36,
	0x10B40,
	0x10B56,
	0x10B60,
	0x10B73,
	0x10B80,
	0x10B92,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x11000,
	0x11046,
	0x11066,
	0x11070,
	0x11082,
	0x110B9,
	0x110D0,
	0x110E9,
	0x110F0,
	0x110FA,
	0x11100,
	0x11133,
	0x11136,
	0x11140,
	0x11150,
	0x11173,
	0x11176,
	0x11177,
	0x11180,
	0x111C0,
	0x111C1,
	0x111C5,
	0x111D0,
	0x111DB,
	0x111DC,
	0x111DD,
	0x11200,
	0x11212,
	0x11213,
	0x11235,
	0x11237,
	0x11238,
	0x1123E,
	0x1123F,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112A9,
	0x112B0,
	0x112E9,
	0x112F0,
	0x112FA,
	0x11300,
	0x11304,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133D,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134D,
	0x11350,
	0x11351,
	0x11357,
	0x11358,
	0x1135D,
	0x11364,
	0x11400,
	0x11442,
	0x11443,
	0x11446,
	0x11447,
	0x1144B,
	0x11450,
	0x1145A,
	0x11480,
	0x114C2,
	0x114C4,
	0x114C6,
	0x114C7,
	0x114C8,
	0x114D0,
	0x114DA,
	0x11580,
	0x115B6,
	0x115B8,
	0x115BF,
	0x115D8,
	0x115DE,
	0x11600,
	0x1163F,
	0x11640,
	0x11641,
	0x11644,
	0x11645,
	0x11650,
	0x1165A,
	0x11680,
	0x116B6,
	0x116C0,
	0x116CA,
	0x11700,
	0x1171A,
	0x1171D,
	0x1172B,
	0x11730,
	0x1173A,
	0x118A0,
	0x118EA,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C37,
	0x11C38,
	0x11C3F,
	0x11C40,
	0x11C41,
	0x11C50,
	0x11C5A,
	0x11C72,
	0x11C90,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16A60,
	0x16A6A,
	0x16AD0,
	0x16AEE,
	0x16B00,
	0x16B37,
	0x16B40,
	0x16B44,
	0x16B50,
	0x16B5A,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F7F,
	0x16F93,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1BC9E,
	0x1BC9F,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D6C1,
	0x1D6C2,
	0x1D6DB,
	0x1D6DC,
	0x1D6FB,
	0x1D6FC,
	0x1D715,
	0x1D716,
	0x1D735,
	0x1D736,
	0x1D74F,
	0x1D750,
	0x1D76F,
	0x1D770,
	0x1D789,
	0x1D78A,
	0x1D7A9,
	0x1D7AA,
	0x1D7C3,
	0x1D7C4,
	0x1D7CC,
	0x1D7CE,
	0x1D800,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E800,
	0x1E8C5,
	0x1E900,
	0x1E944,
	0x1E947,
	0x1E948,
	0x1E950,
	0x1E95A,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x1F130,
	0x1F14A,
	0x1F150,
	0x1F16A,
	0x1F170,
	0x1F18A,
	0x20000,
	0x2A6D7,
	0x2A700,
	0x2B735,
	0x2B740,
	0x2B81E,
	0x2B820,
	0x2CEA2,
	0x2F800,
	0x2FA1E
};

static const UV XPosixAlpha_invlist[] = { /* for EBCDIC 1047 */
	1315,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x42,
	0x4A,
	0x51,
	0x5A,
	0x62,
	0x6A,
	0x70,
	0x79,
	0x80,
	0x8A,
	0x8C,
	0x8F,
	0x91,
	0x9D,
	0x9E,
	0x9F,
	0xA0,
	0xA1,
	0xA2,
	0xAA,
	0xAC,
	0xAD,
	0xAE,
	0xAF,
	0xBA,
	0xBB,
	0xC1,
	0xCA,
	0xCB,
	0xD0,
	0xD1,
	0xDA,
	0xDB,
	0xE0,
	0xE2,
	0xEA,
	0xEB,
	0xF0,
	0xFB,
	0xFF,
	0x100,
	0x2C2,
	0x2C6,
	0x2D2,
	0x2E0,
	0x2E5,
	0x2EC,
	0x2ED,
	0x2EE,
	0x2EF,
	0x345,
	0x346,
	0x370,
	0x375,
	0x376,
	0x378,
	0x37A,
	0x37E,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x3F6,
	0x3F7,
	0x482,
	0x48A,
	0x530,
	0x531,
	0x557,
	0x559,
	0x55A,
	0x561,
	0x588,
	0x5B0,
	0x5BE,
	0x5BF,
	0x5C0,
	0x5C1,
	0x5C3,
	0x5C4,
	0x5C6,
	0x5C7,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F3,
	0x610,
	0x61B,
	0x620,
	0x658,
	0x659,
	0x660,
	0x66E,
	0x6D4,
	0x6D5,
	0x6DD,
	0x6E1,
	0x6E9,
	0x6ED,
	0x6F0,
	0x6FA,
	0x6FD,
	0x6FF,
	0x700,
	0x710,
	0x740,
	0x74D,
	0x7B2,
	0x7CA,
	0x7EB,
	0x7F4,
	0x7F6,
	0x7FA,
	0x7FB,
	0x800,
	0x818,
	0x81A,
	0x82D,
	0x840,
	0x859,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x8D4,
	0x8E0,
	0x8E3,
	0x8EA,
	0x8F0,
	0x93C,
	0x93D,
	0x94D,
	0x94E,
	0x951,
	0x955,
	0x964,
	0x971,
	0x984,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BD,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CD,
	0x9CE,
	0x9CF,
	0x9D7,
	0x9D8,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E4,
	0x9F0,
	0x9F2,
	0xA01,
	0xA04,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4D,
	0xA51,
	0xA52,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA70,
	0xA76,
	0xA81,
	0xA84,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABD,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACD,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE4,
	0xAF9,
	0xAFA,
	0xB01,
	0xB04,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3D,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4D,
	0xB56,
	0xB58,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB64,
	0xB71,
	0xB72,
	0xB82,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCD,
	0xBD0,
	0xBD1,
	0xBD7,
	0xBD8,
	0xC00,
	0xC04,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4D,
	0xC55,
	0xC57,
	0xC58,
	0xC5B,
	0xC60,
	0xC64,
	0xC80,
	0xC84,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBD,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCD,
	0xCD5,
	0xCD7,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE4,
	0xCF1,
	0xCF3,
	0xD01,
	0xD04,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD4D,
	0xD4E,
	0xD4F,
	0xD54,
	0xD58,
	0xD5F,
	0xD64,
	0xD7A,
	0xD80,
	0xD82,
	0xD84,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDF2,
	0xDF4,
	0xE01,
	0xE3B,
	0xE40,
	0xE47,
	0xE4D,
	0xE4E,
	0xE81,
	0xE83,
	0xE84,
	0xE85,
	0xE87,
	0xE89,
	0xE8A,
	0xE8B,
	0xE8D,
	0xE8E,
	0xE94,
	0xE98,
	0xE99,
	0xEA0,
	0xEA1,
	0xEA4,
	0xEA5,
	0xEA6,
	0xEA7,
	0xEA8,
	0xEAA,
	0xEAC,
	0xEAD,
	0xEBA,
	0xEBB,
	0xEBE,
	0xEC0,
	0xEC5,
	0xEC6,
	0xEC7,
	0xECD,
	0xECE,
	0xEDC,
	0xEE0,
	0xF00,
	0xF01,
	0xF40,
	0xF48,
	0xF49,
	0xF6D,
	0xF71,
	0xF82,
	0xF88,
	0xF98,
	0xF99,
	0xFBD,
	0x1000,
	0x1037,
	0x1038,
	0x1039,
	0x103B,
	0x1040,
	0x1050,
	0x1063,
	0x1065,
	0x1069,
	0x106E,
	0x1087,
	0x108E,
	0x108F,
	0x109C,
	0x109E,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x10FB,
	0x10FC,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x135F,
	0x1360,
	0x1380,
	0x1390,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1401,
	0x166D,
	0x166F,
	0x1680,
	0x1681,
	0x169B,
	0x16A0,
	0x16EB,
	0x16EE,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1714,
	0x1720,
	0x1734,
	0x1740,
	0x1754,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1772,
	0x1774,
	0x1780,
	0x17B4,
	0x17B6,
	0x17C9,
	0x17D7,
	0x17D8,
	0x17DC,
	0x17DD,
	0x1820,
	0x1878,
	0x1880,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1920,
	0x192C,
	0x1930,
	0x1939,
	0x1950,
	0x196E,
	0x1970,
	0x1975,
	0x1980,
	0x19AC,
	0x19B0,
	0x19CA,
	0x1A00,
	0x1A1C,
	0x1A20,
	0x1A5F,
	0x1A61,
	0x1A75,
	0x1AA7,
	0x1AA8,
	0x1B00,
	0x1B34,
	0x1B35,
	0x1B44,
	0x1B45,
	0x1B4C,
	0x1B80,
	0x1BAA,
	0x1BAC,
	0x1BB0,
	0x1BBA,
	0x1BE6,
	0x1BE7,
	0x1BF2,
	0x1C00,
	0x1C36,
	0x1C4D,
	0x1C50,
	0x1C5A,
	0x1C7E,
	0x1C80,
	0x1C89,
	0x1CE9,
	0x1CED,
	0x1CEE,
	0x1CF4,
	0x1CF5,
	0x1CF7,
	0x1D00,
	0x1DC0,
	0x1DE7,
	0x1DF5,
	0x1E00,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FBD,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FCD,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FE0,
	0x1FED,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFD,
	0x2071,
	0x2072,
	0x207F,
	0x2080,
	0x2090,
	0x209D,
	0x2102,
	0x2103,
	0x2107,
	0x2108,
	0x210A,
	0x2114,
	0x2115,
	0x2116,
	0x2119,
	0x211E,
	0x2124,
	0x2125,
	0x2126,
	0x2127,
	0x2128,
	0x2129,
	0x212A,
	0x212E,
	0x212F,
	0x213A,
	0x213C,
	0x2140,
	0x2145,
	0x214A,
	0x214E,
	0x214F,
	0x2160,
	0x2189,
	0x24B6,
	0x24EA,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CE5,
	0x2CEB,
	0x2CEF,
	0x2CF2,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D70,
	0x2D80,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x2DE0,
	0x2E00,
	0x2E2F,
	0x2E30,
	0x3005,
	0x3008,
	0x3021,
	0x302A,
	0x3031,
	0x3036,
	0x3038,
	0x303D,
	0x3041,
	0x3097,
	0x309D,
	0x30A0,
	0x30A1,
	0x30FB,
	0x30FC,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x31A0,
	0x31BB,
	0x31F0,
	0x3200,
	0x3400,
	0x4DB6,
	0x4E00,
	0x9FD6,
	0xA000,
	0xA48D,
	0xA4D0,
	0xA4FE,
	0xA500,
	0xA60D,
	0xA610,
	0xA620,
	0xA62A,
	0xA62C,
	0xA640,
	0xA66F,
	0xA674,
	0xA67C,
	0xA67F,
	0xA6F0,
	0xA717,
	0xA720,
	0xA722,
	0xA789,
	0xA78B,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F7,
	0xA802,
	0xA803,
	0xA806,
	0xA807,
	0xA80B,
	0xA80C,
	0xA828,
	0xA840,
	0xA874,
	0xA880,
	0xA8C4,
	0xA8C5,
	0xA8C6,
	0xA8F2,
	0xA8F8,
	0xA8FB,
	0xA8FC,
	0xA8FD,
	0xA8FE,
	0xA90A,
	0xA92B,
	0xA930,
	0xA953,
	0xA960,
	0xA97D,
	0xA980,
	0xA9B3,
	0xA9B4,
	0xA9C0,
	0xA9CF,
	0xA9D0,
	0xA9E0,
	0xA9E5,
	0xA9E6,
	0xA9F0,
	0xA9FA,
	0xA9FF,
	0xAA00,
	0xAA37,
	0xAA40,
	0xAA4E,
	0xAA60,
	0xAA77,
	0xAA7A,
	0xAA7B,
	0xAA7E,
	0xAABF,
	0xAAC0,
	0xAAC1,
	0xAAC2,
	0xAAC3,
	0xAADB,
	0xAADE,
	0xAAE0,
	0xAAF0,
	0xAAF2,
	0xAAF6,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB5B,
	0xAB5C,
	0xAB66,
	0xAB70,
	0xABEB,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xF900,
	0xFA6E,
	0xFA70,
	0xFADA,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB29,
	0xFB2A,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFBB2,
	0xFBD3,
	0xFD3E,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFC,
	0xFE70,
	0xFE75,
	0xFE76,
	0xFEFD,
	0xFF21,
	0xFF3B,
	0xFF41,
	0xFF5B,
	0xFF66,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10140,
	0x10175,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x10300,
	0x10320,
	0x10330,
	0x1034B,
	0x10350,
	0x1037B,
	0x10380,
	0x1039E,
	0x103A0,
	0x103C4,
	0x103C8,
	0x103D0,
	0x103D1,
	0x103D6,
	0x10400,
	0x1049E,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10860,
	0x10877,
	0x10880,
	0x1089F,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x10900,
	0x10916,
	0x10920,
	0x1093A,
	0x10980,
	0x109B8,
	0x109BE,
	0x109C0,
	0x10A00,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A60,
	0x10A7D,
	0x10A80,
	0x10A9D,
	0x10AC0,
	0x10AC8,
	0x10AC9,
	0x10AE5,
	0x10B00,
	0x10B36,
	0x10B40,
	0x10B56,
	0x10B60,
	0x10B73,
	0x10B80,
	0x10B92,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x11000,
	0x11046,
	0x11082,
	0x110B9,
	0x110D0,
	0x110E9,
	0x11100,
	0x11133,
	0x11150,
	0x11173,
	0x11176,
	0x11177,
	0x11180,
	0x111C0,
	0x111C1,
	0x111C5,
	0x111DA,
	0x111DB,
	0x111DC,
	0x111DD,
	0x11200,
	0x11212,
	0x11213,
	0x11235,
	0x11237,
	0x11238,
	0x1123E,
	0x1123F,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112A9,
	0x112B0,
	0x112E9,
	0x11300,
	0x11304,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133D,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134D,
	0x11350,
	0x11351,
	0x11357,
	0x11358,
	0x1135D,
	0x11364,
	0x11400,
	0x11442,
	0x11443,
	0x11446,
	0x11447,
	0x1144B,
	0x11480,
	0x114C2,
	0x114C4,
	0x114C6,
	0x114C7,
	0x114C8,
	0x11580,
	0x115B6,
	0x115B8,
	0x115BF,
	0x115D8,
	0x115DE,
	0x11600,
	0x1163F,
	0x11640,
	0x11641,
	0x11644,
	0x11645,
	0x11680,
	0x116B6,
	0x11700,
	0x1171A,
	0x1171D,
	0x1172B,
	0x118A0,
	0x118E0,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C37,
	0x11C38,
	0x11C3F,
	0x11C40,
	0x11C41,
	0x11C72,
	0x11C90,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16AD0,
	0x16AEE,
	0x16B00,
	0x16B37,
	0x16B40,
	0x16B44,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F7F,
	0x16F93,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1BC9E,
	0x1BC9F,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D6C1,
	0x1D6C2,
	0x1D6DB,
	0x1D6DC,
	0x1D6FB,
	0x1D6FC,
	0x1D715,
	0x1D716,
	0x1D735,
	0x1D736,
	0x1D74F,
	0x1D750,
	0x1D76F,
	0x1D770,
	0x1D789,
	0x1D78A,
	0x1D7A9,
	0x1D7AA,
	0x1D7C3,
	0x1D7C4,
	0x1D7CC,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E800,
	0x1E8C5,
	0x1E900,
	0x1E944,
	0x1E947,
	0x1E948,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x1F130,
	0x1F14A,
	0x1F150,
	0x1F16A,
	0x1F170,
	0x1F18A,
	0x20000,
	0x2A6D7,
	0x2A700,
	0x2B735,
	0x2B740,
	0x2B81E,
	0x2B820,
	0x2CEA2,
	0x2F800,
	0x2FA1E
};

static const UV XPosixBlank_invlist[] = { /* for EBCDIC 1047 */
	15,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x5,
	0x6,
	0x40,
	0x42,
	0x1680,
	0x1681,
	0x2000,
	0x200B,
	0x202F,
	0x2030,
	0x205F,
	0x2060,
	0x3000,
	0x3001
};

static const UV XPosixCntrl_invlist[] = { /* for EBCDIC 1047 */
	4,	/* Number of elements */
	148565664, /* Version and data structure type */
	0,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x40,
	0xFF,
	0x100
};

static const UV XPosixDigit_invlist[] = { /* for EBCDIC 1047 */
	109,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0xF0,
	0xFA,
	0x660,
	0x66A,
	0x6F0,
	0x6FA,
	0x7C0,
	0x7CA,
	0x966,
	0x970,
	0x9E6,
	0x9F0,
	0xA66,
	0xA70,
	0xAE6,
	0xAF0,
	0xB66,
	0xB70,
	0xBE6,
	0xBF0,
	0xC66,
	0xC70,
	0xCE6,
	0xCF0,
	0xD66,
	0xD70,
	0xDE6,
	0xDF0,
	0xE50,
	0xE5A,
	0xED0,
	0xEDA,
	0xF20,
	0xF2A,
	0x1040,
	0x104A,
	0x1090,
	0x109A,
	0x17E0,
	0x17EA,
	0x1810,
	0x181A,
	0x1946,
	0x1950,
	0x19D0,
	0x19DA,
	0x1A80,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1B50,
	0x1B5A,
	0x1BB0,
	0x1BBA,
	0x1C40,
	0x1C4A,
	0x1C50,
	0x1C5A,
	0xA620,
	0xA62A,
	0xA8D0,
	0xA8DA,
	0xA900,
	0xA90A,
	0xA9D0,
	0xA9DA,
	0xA9F0,
	0xA9FA,
	0xAA50,
	0xAA5A,
	0xABF0,
	0xABFA,
	0xFF10,
	0xFF1A,
	0x104A0,
	0x104AA,
	0x11066,
	0x11070,
	0x110F0,
	0x110FA,
	0x11136,
	0x11140,
	0x111D0,
	0x111DA,
	0x112F0,
	0x112FA,
	0x11450,
	0x1145A,
	0x114D0,
	0x114DA,
	0x11650,
	0x1165A,
	0x116C0,
	0x116CA,
	0x11730,
	0x1173A,
	0x118E0,
	0x118EA,
	0x11C50,
	0x11C5A,
	0x16A60,
	0x16A6A,
	0x16B50,
	0x16B5A,
	0x1D7CE,
	0x1D800,
	0x1E950,
	0x1E95A
};

static const UV XPosixGraph_invlist[] = { /* for EBCDIC 1047 */
	1287,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x42,
	0xFF,
	0x100,
	0x378,
	0x37A,
	0x380,
	0x384,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x530,
	0x531,
	0x557,
	0x559,
	0x560,
	0x561,
	0x588,
	0x589,
	0x58B,
	0x58D,
	0x590,
	0x591,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F5,
	0x600,
	0x61D,
	0x61E,
	0x70E,
	0x70F,
	0x74B,
	0x74D,
	0x7B2,
	0x7C0,
	0x7FB,
	0x800,
	0x82E,
	0x830,
	0x83F,
	0x840,
	0x85C,
	0x85E,
	0x85F,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x8D4,
	0x984,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BC,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CF,
	0x9D7,
	0x9D8,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E4,
	0x9E6,
	0x9FC,
	0xA01,
	0xA04,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA3C,
	0xA3D,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4E,
	0xA51,
	0xA52,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA66,
	0xA76,
	0xA81,
	0xA84,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABC,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACE,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE4,
	0xAE6,
	0xAF2,
	0xAF9,
	0xAFA,
	0xB01,
	0xB04,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3C,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4E,
	0xB56,
	0xB58,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB64,
	0xB66,
	0xB78,
	0xB82,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCE,
	0xBD0,
	0xBD1,
	0xBD7,
	0xBD8,
	0xBE6,
	0xBFB,
	0xC00,
	0xC04,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4E,
	0xC55,
	0xC57,
	0xC58,
	0xC5B,
	0xC60,
	0xC64,
	0xC66,
	0xC70,
	0xC78,
	0xC84,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBC,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCE,
	0xCD5,
	0xCD7,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE4,
	0xCE6,
	0xCF0,
	0xCF1,
	0xCF3,
	0xD01,
	0xD04,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD50,
	0xD54,
	0xD64,
	0xD66,
	0xD80,
	0xD82,
	0xD84,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xDCA,
	0xDCB,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDE6,
	0xDF0,
	0xDF2,
	0xDF5,
	0xE01,
	0xE3B,
	0xE3F,
	0xE5C,
	0xE81,
	0xE83,
	0xE84,
	0xE85,
	0xE87,
	0xE89,
	0xE8A,
	0xE8B,
	0xE8D,
	0xE8E,
	0xE94,
	0xE98,
	0xE99,
	0xEA0,
	0xEA1,
	0xEA4,
	0xEA5,
	0xEA6,
	0xEA7,
	0xEA8,
	0xEAA,
	0xEAC,
	0xEAD,
	0xEBA,
	0xEBB,
	0xEBE,
	0xEC0,
	0xEC5,
	0xEC6,
	0xEC7,
	0xEC8,
	0xECE,
	0xED0,
	0xEDA,
	0xEDC,
	0xEE0,
	0xF00,
	0xF48,
	0xF49,
	0xF6D,
	0xF71,
	0xF98,
	0xF99,
	0xFBD,
	0xFBE,
	0xFCD,
	0xFCE,
	0xFDB,
	0x1000,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x135D,
	0x137D,
	0x1380,
	0x139A,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1400,
	0x1680,
	0x1681,
	0x169D,
	0x16A0,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1715,
	0x1720,
	0x1737,
	0x1740,
	0x1754,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1772,
	0x1774,
	0x1780,
	0x17DE,
	0x17E0,
	0x17EA,
	0x17F0,
	0x17FA,
	0x1800,
	0x180F,
	0x1810,
	0x181A,
	0x1820,
	0x1878,
	0x1880,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1920,
	0x192C,
	0x1930,
	0x193C,
	0x1940,
	0x1941,
	0x1944,
	0x196E,
	0x1970,
	0x1975,
	0x1980,
	0x19AC,
	0x19B0,
	0x19CA,
	0x19D0,
	0x19DB,
	0x19DE,
	0x1A1C,
	0x1A1E,
	0x1A5F,
	0x1A60,
	0x1A7D,
	0x1A7F,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1AA0,
	0x1AAE,
	0x1AB0,
	0x1ABF,
	0x1B00,
	0x1B4C,
	0x1B50,
	0x1B7D,
	0x1B80,
	0x1BF4,
	0x1BFC,
	0x1C38,
	0x1C3B,
	0x1C4A,
	0x1C4D,
	0x1C89,
	0x1CC0,
	0x1CC8,
	0x1CD0,
	0x1CF7,
	0x1CF8,
	0x1CFA,
	0x1D00,
	0x1DF6,
	0x1DFB,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FC5,
	0x1FC6,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FDD,
	0x1FF0,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFF,
	0x200B,
	0x2028,
	0x202A,
	0x202F,
	0x2030,
	0x205F,
	0x2060,
	0x2065,
	0x2066,
	0x2072,
	0x2074,
	0x208F,
	0x2090,
	0x209D,
	0x20A0,
	0x20BF,
	0x20D0,
	0x20F1,
	0x2100,
	0x218C,
	0x2190,
	0x23FF,
	0x2400,
	0x2427,
	0x2440,
	0x244B,
	0x2460,
	0x2B74,
	0x2B76,
	0x2B96,
	0x2B98,
	0x2BBA,
	0x2BBD,
	0x2BC9,
	0x2BCA,
	0x2BD2,
	0x2BEC,
	0x2BF0,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CF4,
	0x2CF9,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D71,
	0x2D7F,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x2DE0,
	0x2E45,
	0x2E80,
	0x2E9A,
	0x2E9B,
	0x2EF4,
	0x2F00,
	0x2FD6,
	0x2FF0,
	0x2FFC,
	0x3001,
	0x3040,
	0x3041,
	0x3097,
	0x3099,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x3190,
	0x31BB,
	0x31C0,
	0x31E4,
	0x31F0,
	0x321F,
	0x3220,
	0x32FF,
	0x3300,
	0x4DB6,
	0x4DC0,
	0x9FD6,
	0xA000,
	0xA48D,
	0xA490,
	0xA4C7,
	0xA4D0,
	0xA62C,
	0xA640,
	0xA6F8,
	0xA700,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F7,
	0xA82C,
	0xA830,
	0xA83A,
	0xA840,
	0xA878,
	0xA880,
	0xA8C6,
	0xA8CE,
	0xA8DA,
	0xA8E0,
	0xA8FE,
	0xA900,
	0xA954,
	0xA95F,
	0xA97D,
	0xA980,
	0xA9CE,
	0xA9CF,
	0xA9DA,
	0xA9DE,
	0xA9FF,
	0xAA00,
	0xAA37,
	0xAA40,
	0xAA4E,
	0xAA50,
	0xAA5A,
	0xAA5C,
	0xAAC3,
	0xAADB,
	0xAAF7,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB66,
	0xAB70,
	0xABEE,
	0xABF0,
	0xABFA,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xE000,
	0xFA6E,
	0xFA70,
	0xFADA,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFBC2,
	0xFBD3,
	0xFD40,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFE,
	0xFE00,
	0xFE1A,
	0xFE20,
	0xFE53,
	0xFE54,
	0xFE67,
	0xFE68,
	0xFE6C,
	0xFE70,
	0xFE75,
	0xFE76,
	0xFEFD,
	0xFEFF,
	0xFF00,
	0xFF01,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0xFFE0,
	0xFFE7,
	0xFFE8,
	0xFFEF,
	0xFFF9,
	0xFFFE,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10100,
	0x10103,
	0x10107,
	0x10134,
	0x10137,
	0x1018F,
	0x10190,
	0x1019C,
	0x101A0,
	0x101A1,
	0x101D0,
	0x101FE,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x102E0,
	0x102FC,
	0x10300,
	0x10324,
	0x10330,
	0x1034B,
	0x10350,
	0x1037B,
	0x10380,
	0x1039E,
	0x1039F,
	0x103C4,
	0x103C8,
	0x103D6,
	0x10400,
	0x1049E,
	0x104A0,
	0x104AA,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x1056F,
	0x10570,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10857,
	0x1089F,
	0x108A7,
	0x108B0,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x108FB,
	0x1091C,
	0x1091F,
	0x1093A,
	0x1093F,
	0x10940,
	0x10980,
	0x109B8,
	0x109BC,
	0x109D0,
	0x109D2,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A38,
	0x10A3B,
	0x10A3F,
	0x10A48,
	0x10A50,
	0x10A59,
	0x10A60,
	0x10AA0,
	0x10AC0,
	0x10AE7,
	0x10AEB,
	0x10AF7,
	0x10B00,
	0x10B36,
	0x10B39,
	0x10B56,
	0x10B58,
	0x10B73,
	0x10B78,
	0x10B92,
	0x10B99,
	0x10B9D,
	0x10BA9,
	0x10BB0,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x10CFA,
	0x10D00,
	0x10E60,
	0x10E7F,
	0x11000,
	0x1104E,
	0x11052,
	0x11070,
	0x1107F,
	0x110C2,
	0x110D0,
	0x110E9,
	0x110F0,
	0x110FA,
	0x11100,
	0x11135,
	0x11136,
	0x11144,
	0x11150,
	0x11177,
	0x11180,
	0x111CE,
	0x111D0,
	0x111E0,
	0x111E1,
	0x111F5,
	0x11200,
	0x11212,
	0x11213,
	0x1123F,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112AA,
	0x112B0,
	0x112EB,
	0x112F0,
	0x112FA,
	0x11300,
	0x11304,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133C,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134E,
	0x11350,
	0x11351,
	0x11357,
	0x11358,
	0x1135D,
	0x11364,
	0x11366,
	0x1136D,
	0x11370,
	0x11375,
	0x11400,
	0x1145A,
	0x1145B,
	0x1145C,
	0x1145D,
	0x1145E,
	0x11480,
	0x114C8,
	0x114D0,
	0x114DA,
	0x11580,
	0x115B6,
	0x115B8,
	0x115DE,
	0x11600,
	0x11645,
	0x11650,
	0x1165A,
	0x11660,
	0x1166D,
	0x11680,
	0x116B8,
	0x116C0,
	0x116CA,
	0x11700,
	0x1171A,
	0x1171D,
	0x1172C,
	0x11730,
	0x11740,
	0x118A0,
	0x118F3,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C37,
	0x11C38,
	0x11C46,
	0x11C50,
	0x11C6D,
	0x11C70,
	0x11C90,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12470,
	0x12475,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16A60,
	0x16A6A,
	0x16A6E,
	0x16A70,
	0x16AD0,
	0x16AEE,
	0x16AF0,
	0x16AF6,
	0x16B00,
	0x16B46,
	0x16B50,
	0x16B5A,
	0x16B5B,
	0x16B62,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F7F,
	0x16F8F,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1BC9C,
	0x1BCA4,
	0x1D000,
	0x1D0F6,
	0x1D100,
	0x1D127,
	0x1D129,
	0x1D1E9,
	0x1D200,
	0x1D246,
	0x1D300,
	0x1D357,
	0x1D360,
	0x1D372,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D7CC,
	0x1D7CE,
	0x1DA8C,
	0x1DA9B,
	0x1DAA0,
	0x1DAA1,
	0x1DAB0,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E800,
	0x1E8C5,
	0x1E8C7,
	0x1E8D7,
	0x1E900,
	0x1E94B,
	0x1E950,
	0x1E95A,
	0x1E95E,
	0x1E960,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x1EEF0,
	0x1EEF2,
	0x1F000,
	0x1F02C,
	0x1F030,
	0x1F094,
	0x1F0A0,
	0x1F0AF,
	0x1F0B1,
	0x1F0C0,
	0x1F0C1,
	0x1F0D0,
	0x1F0D1,
	0x1F0F6,
	0x1F100,
	0x1F10D,
	0x1F110,
	0x1F12F,
	0x1F130,
	0x1F16C,
	0x1F170,
	0x1F1AD,
	0x1F1E6,
	0x1F203,
	0x1F210,
	0x1F23C,
	0x1F240,
	0x1F249,
	0x1F250,
	0x1F252,
	0x1F300,
	0x1F6D3,
	0x1F6E0,
	0x1F6ED,
	0x1F6F0,
	0x1F6F7,
	0x1F700,
	0x1F774,
	0x1F780,
	0x1F7D5,
	0x1F800,
	0x1F80C,
	0x1F810,
	0x1F848,
	0x1F850,
	0x1F85A,
	0x1F860,
	0x1F888,
	0x1F890,
	0x1F8AE,
	0x1F910,
	0x1F91F,
	0x1F920,
	0x1F928,
	0x1F930,
	0x1F931,
	0x1F933,
	0x1F93F,
	0x1F940,
	0x1F94C,
	0x1F950,
	0x1F95F,
	0x1F980,
	0x1F992,
	0x1F9C0,
	0x1F9C1,
	0x20000,
	0x2A6D7,
	0x2A700,
	0x2B735,
	0x2B740,
	0x2B81E,
	0x2B820,
	0x2CEA2,
	0x2F800,
	0x2FA1E,
	0xE0001,
	0xE0002,
	0xE0020,
	0xE0080,
	0xE0100,
	0xE01F0,
	0xF0000,
	0xFFFFE,
	0x100000,
	0x10FFFE
};

static const UV XPosixLower_invlist[] = { /* for EBCDIC 1047 */
	1289,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x42,
	0x4A,
	0x51,
	0x5A,
	0x70,
	0x71,
	0x81,
	0x8A,
	0x8C,
	0x8F,
	0x91,
	0x9D,
	0xA0,
	0xA1,
	0xA2,
	0xAA,
	0xCB,
	0xD0,
	0xDB,
	0xE0,
	0x101,
	0x102,
	0x103,
	0x104,
	0x105,
	0x106,
	0x107,
	0x108,
	0x109,
	0x10A,
	0x10B,
	0x10C,
	0x10D,
	0x10E,
	0x10F,
	0x110,
	0x111,
	0x112,
	0x113,
	0x114,
	0x115,
	0x116,
	0x117,
	0x118,
	0x119,
	0x11A,
	0x11B,
	0x11C,
	0x11D,
	0x11E,
	0x11F,
	0x120,
	0x121,
	0x122,
	0x123,
	0x124,
	0x125,
	0x126,
	0x127,
	0x128,
	0x129,
	0x12A,
	0x12B,
	0x12C,
	0x12D,
	0x12E,
	0x12F,
	0x130,
	0x131,
	0x132,
	0x133,
	0x134,
	0x135,
	0x136,
	0x137,
	0x139,
	0x13A,
	0x13B,
	0x13C,
	0x13D,
	0x13E,
	0x13F,
	0x140,
	0x141,
	0x142,
	0x143,
	0x144,
	0x145,
	0x146,
	0x147,
	0x148,
	0x14A,
	0x14B,
	0x14C,
	0x14D,
	0x14E,
	0x14F,
	0x150,
	0x151,
	0x152,
	0x153,
	0x154,
	0x155,
	0x156,
	0x157,
	0x158,
	0x159,
	0x15A,
	0x15B,
	0x15C,
	0x15D,
	0x15E,
	0x15F,
	0x160,
	0x161,
	0x162,
	0x163,
	0x164,
	0x165,
	0x166,
	0x167,
	0x168,
	0x169,
	0x16A,
	0x16B,
	0x16C,
	0x16D,
	0x16E,
	0x16F,
	0x170,
	0x171,
	0x172,
	0x173,
	0x174,
	0x175,
	0x176,
	0x177,
	0x178,
	0x17A,
	0x17B,
	0x17C,
	0x17D,
	0x17E,
	0x181,
	0x183,
	0x184,
	0x185,
	0x186,
	0x188,
	0x189,
	0x18C,
	0x18E,
	0x192,
	0x193,
	0x195,
	0x196,
	0x199,
	0x19C,
	0x19E,
	0x19F,
	0x1A1,
	0x1A2,
	0x1A3,
	0x1A4,
	0x1A5,
	0x1A6,
	0x1A8,
	0x1A9,
	0x1AA,
	0x1AC,
	0x1AD,
	0x1AE,
	0x1B0,
	0x1B1,
	0x1B4,
	0x1B5,
	0x1B6,
	0x1B7,
	0x1B9,
	0x1BB,
	0x1BD,
	0x1C0,
	0x1C6,
	0x1C7,
	0x1C9,
	0x1CA,
	0x1CC,
	0x1CD,
	0x1CE,
	0x1CF,
	0x1D0,
	0x1D1,
	0x1D2,
	0x1D3,
	0x1D4,
	0x1D5,
	0x1D6,
	0x1D7,
	0x1D8,
	0x1D9,
	0x1DA,
	0x1DB,
	0x1DC,
	0x1DE,
	0x1DF,
	0x1E0,
	0x1E1,
	0x1E2,
	0x1E3,
	0x1E4,
	0x1E5,
	0x1E6,
	0x1E7,
	0x1E8,
	0x1E9,
	0x1EA,
	0x1EB,
	0x1EC,
	0x1ED,
	0x1EE,
	0x1EF,
	0x1F1,
	0x1F3,
	0x1F4,
	0x1F5,
	0x1F6,
	0x1F9,
	0x1FA,
	0x1FB,
	0x1FC,
	0x1FD,
	0x1FE,
	0x1FF,
	0x200,
	0x201,
	0x202,
	0x203,
	0x204,
	0x205,
	0x206,
	0x207,
	0x208,
	0x209,
	0x20A,
	0x20B,
	0x20C,
	0x20D,
	0x20E,
	0x20F,
	0x210,
	0x211,
	0x212,
	0x213,
	0x214,
	0x215,
	0x216,
	0x217,
	0x218,
	0x219,
	0x21A,
	0x21B,
	0x21C,
	0x21D,
	0x21E,
	0x21F,
	0x220,
	0x221,
	0x222,
	0x223,
	0x224,
	0x225,
	0x226,
	0x227,
	0x228,
	0x229,
	0x22A,
	0x22B,
	0x22C,
	0x22D,
	0x22E,
	0x22F,
	0x230,
	0x231,
	0x232,
	0x233,
	0x23A,
	0x23C,
	0x23D,
	0x23F,
	0x241,
	0x242,
	0x243,
	0x247,
	0x248,
	0x249,
	0x24A,
	0x24B,
	0x24C,
	0x24D,
	0x24E,
	0x24F,
	0x294,
	0x295,
	0x2B9,
	0x2C0,
	0x2C2,
	0x2E0,
	0x2E5,
	0x345,
	0x346,
	0x371,
	0x372,
	0x373,
	0x374,
	0x377,
	0x378,
	0x37A,
	0x37E,
	0x390,
	0x391,
	0x3AC,
	0x3CF,
	0x3D0,
	0x3D2,
	0x3D5,
	0x3D8,
	0x3D9,
	0x3DA,
	0x3DB,
	0x3DC,
	0x3DD,
	0x3DE,
	0x3DF,
	0x3E0,
	0x3E1,
	0x3E2,
	0x3E3,
	0x3E4,
	0x3E5,
	0x3E6,
	0x3E7,
	0x3E8,
	0x3E9,
	0x3EA,
	0x3EB,
	0x3EC,
	0x3ED,
	0x3EE,
	0x3EF,
	0x3F4,
	0x3F5,
	0x3F6,
	0x3F8,
	0x3F9,
	0x3FB,
	0x3FD,
	0x430,
	0x460,
	0x461,
	0x462,
	0x463,
	0x464,
	0x465,
	0x466,
	0x467,
	0x468,
	0x469,
	0x46A,
	0x46B,
	0x46C,
	0x46D,
	0x46E,
	0x46F,
	0x470,
	0x471,
	0x472,
	0x473,
	0x474,
	0x475,
	0x476,
	0x477,
	0x478,
	0x479,
	0x47A,
	0x47B,
	0x47C,
	0x47D,
	0x47E,
	0x47F,
	0x480,
	0x481,
	0x482,
	0x48B,
	0x48C,
	0x48D,
	0x48E,
	0x48F,
	0x490,
	0x491,
	0x492,
	0x493,
	0x494,
	0x495,
	0x496,
	0x497,
	0x498,
	0x499,
	0x49A,
	0x49B,
	0x49C,
	0x49D,
	0x49E,
	0x49F,
	0x4A0,
	0x4A1,
	0x4A2,
	0x4A3,
	0x4A4,
	0x4A5,
	0x4A6,
	0x4A7,
	0x4A8,
	0x4A9,
	0x4AA,
	0x4AB,
	0x4AC,
	0x4AD,
	0x4AE,
	0x4AF,
	0x4B0,
	0x4B1,
	0x4B2,
	0x4B3,
	0x4B4,
	0x4B5,
	0x4B6,
	0x4B7,
	0x4B8,
	0x4B9,
	0x4BA,
	0x4BB,
	0x4BC,
	0x4BD,
	0x4BE,
	0x4BF,
	0x4C0,
	0x4C2,
	0x4C3,
	0x4C4,
	0x4C5,
	0x4C6,
	0x4C7,
	0x4C8,
	0x4C9,
	0x4CA,
	0x4CB,
	0x4CC,
	0x4CD,
	0x4CE,
	0x4D0,
	0x4D1,
	0x4D2,
	0x4D3,
	0x4D4,
	0x4D5,
	0x4D6,
	0x4D7,
	0x4D8,
	0x4D9,
	0x4DA,
	0x4DB,
	0x4DC,
	0x4DD,
	0x4DE,
	0x4DF,
	0x4E0,
	0x4E1,
	0x4E2,
	0x4E3,
	0x4E4,
	0x4E5,
	0x4E6,
	0x4E7,
	0x4E8,
	0x4E9,
	0x4EA,
	0x4EB,
	0x4EC,
	0x4ED,
	0x4EE,
	0x4EF,
	0x4F0,
	0x4F1,
	0x4F2,
	0x4F3,
	0x4F4,
	0x4F5,
	0x4F6,
	0x4F7,
	0x4F8,
	0x4F9,
	0x4FA,
	0x4FB,
	0x4FC,
	0x4FD,
	0x4FE,
	0x4FF,
	0x500,
	0x501,
	0x502,
	0x503,
	0x504,
	0x505,
	0x506,
	0x507,
	0x508,
	0x509,
	0x50A,
	0x50B,
	0x50C,
	0x50D,
	0x50E,
	0x50F,
	0x510,
	0x511,
	0x512,
	0x513,
	0x514,
	0x515,
	0x516,
	0x517,
	0x518,
	0x519,
	0x51A,
	0x51B,
	0x51C,
	0x51D,
	0x51E,
	0x51F,
	0x520,
	0x521,
	0x522,
	0x523,
	0x524,
	0x525,
	0x526,
	0x527,
	0x528,
	0x529,
	0x52A,
	0x52B,
	0x52C,
	0x52D,
	0x52E,
	0x52F,
	0x530,
	0x561,
	0x588,
	0x13F8,
	0x13FE,
	0x1C80,
	0x1C89,
	0x1D00,
	0x1DC0,
	0x1E01,
	0x1E02,
	0x1E03,
	0x1E04,
	0x1E05,
	0x1E06,
	0x1E07,
	0x1E08,
	0x1E09,
	0x1E0A,
	0x1E0B,
	0x1E0C,
	0x1E0D,
	0x1E0E,
	0x1E0F,
	0x1E10,
	0x1E11,
	0x1E12,
	0x1E13,
	0x1E14,
	0x1E15,
	0x1E16,
	0x1E17,
	0x1E18,
	0x1E19,
	0x1E1A,
	0x1E1B,
	0x1E1C,
	0x1E1D,
	0x1E1E,
	0x1E1F,
	0x1E20,
	0x1E21,
	0x1E22,
	0x1E23,
	0x1E24,
	0x1E25,
	0x1E26,
	0x1E27,
	0x1E28,
	0x1E29,
	0x1E2A,
	0x1E2B,
	0x1E2C,
	0x1E2D,
	0x1E2E,
	0x1E2F,
	0x1E30,
	0x1E31,
	0x1E32,
	0x1E33,
	0x1E34,
	0x1E35,
	0x1E36,
	0x1E37,
	0x1E38,
	0x1E39,
	0x1E3A,
	0x1E3B,
	0x1E3C,
	0x1E3D,
	0x1E3E,
	0x1E3F,
	0x1E40,
	0x1E41,
	0x1E42,
	0x1E43,
	0x1E44,
	0x1E45,
	0x1E46,
	0x1E47,
	0x1E48,
	0x1E49,
	0x1E4A,
	0x1E4B,
	0x1E4C,
	0x1E4D,
	0x1E4E,
	0x1E4F,
	0x1E50,
	0x1E51,
	0x1E52,
	0x1E53,
	0x1E54,
	0x1E55,
	0x1E56,
	0x1E57,
	0x1E58,
	0x1E59,
	0x1E5A,
	0x1E5B,
	0x1E5C,
	0x1E5D,
	0x1E5E,
	0x1E5F,
	0x1E60,
	0x1E61,
	0x1E62,
	0x1E63,
	0x1E64,
	0x1E65,
	0x1E66,
	0x1E67,
	0x1E68,
	0x1E69,
	0x1E6A,
	0x1E6B,
	0x1E6C,
	0x1E6D,
	0x1E6E,
	0x1E6F,
	0x1E70,
	0x1E71,
	0x1E72,
	0x1E73,
	0x1E74,
	0x1E75,
	0x1E76,
	0x1E77,
	0x1E78,
	0x1E79,
	0x1E7A,
	0x1E7B,
	0x1E7C,
	0x1E7D,
	0x1E7E,
	0x1E7F,
	0x1E80,
	0x1E81,
	0x1E82,
	0x1E83,
	0x1E84,
	0x1E85,
	0x1E86,
	0x1E87,
	0x1E88,
	0x1E89,
	0x1E8A,
	0x1E8B,
	0x1E8C,
	0x1E8D,
	0x1E8E,
	0x1E8F,
	0x1E90,
	0x1E91,
	0x1E92,
	0x1E93,
	0x1E94,
	0x1E95,
	0x1E9E,
	0x1E9F,
	0x1EA0,
	0x1EA1,
	0x1EA2,
	0x1EA3,
	0x1EA4,
	0x1EA5,
	0x1EA6,
	0x1EA7,
	0x1EA8,
	0x1EA9,
	0x1EAA,
	0x1EAB,
	0x1EAC,
	0x1EAD,
	0x1EAE,
	0x1EAF,
	0x1EB0,
	0x1EB1,
	0x1EB2,
	0x1EB3,
	0x1EB4,
	0x1EB5,
	0x1EB6,
	0x1EB7,
	0x1EB8,
	0x1EB9,
	0x1EBA,
	0x1EBB,
	0x1EBC,
	0x1EBD,
	0x1EBE,
	0x1EBF,
	0x1EC0,
	0x1EC1,
	0x1EC2,
	0x1EC3,
	0x1EC4,
	0x1EC5,
	0x1EC6,
	0x1EC7,
	0x1EC8,
	0x1EC9,
	0x1ECA,
	0x1ECB,
	0x1ECC,
	0x1ECD,
	0x1ECE,
	0x1ECF,
	0x1ED0,
	0x1ED1,
	0x1ED2,
	0x1ED3,
	0x1ED4,
	0x1ED5,
	0x1ED6,
	0x1ED7,
	0x1ED8,
	0x1ED9,
	0x1EDA,
	0x1EDB,
	0x1EDC,
	0x1EDD,
	0x1EDE,
	0x1EDF,
	0x1EE0,
	0x1EE1,
	0x1EE2,
	0x1EE3,
	0x1EE4,
	0x1EE5,
	0x1EE6,
	0x1EE7,
	0x1EE8,
	0x1EE9,
	0x1EEA,
	0x1EEB,
	0x1EEC,
	0x1EED,
	0x1EEE,
	0x1EEF,
	0x1EF0,
	0x1EF1,
	0x1EF2,
	0x1EF3,
	0x1EF4,
	0x1EF5,
	0x1EF6,
	0x1EF7,
	0x1EF8,
	0x1EF9,
	0x1EFA,
	0x1EFB,
	0x1EFC,
	0x1EFD,
	0x1EFE,
	0x1EFF,
	0x1F08,
	0x1F10,
	0x1F16,
	0x1F20,
	0x1F28,
	0x1F30,
	0x1F38,
	0x1F40,
	0x1F46,
	0x1F50,
	0x1F58,
	0x1F60,
	0x1F68,
	0x1F70,
	0x1F7E,
	0x1F80,
	0x1F88,
	0x1F90,
	0x1F98,
	0x1FA0,
	0x1FA8,
	0x1FB0,
	0x1FB5,
	0x1FB6,
	0x1FB8,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FC8,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FD8,
	0x1FE0,
	0x1FE8,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FF8,
	0x2071,
	0x2072,
	0x207F,
	0x2080,
	0x2090,
	0x209D,
	0x210A,
	0x210B,
	0x210E,
	0x2110,
	0x2113,
	0x2114,
	0x212F,
	0x2130,
	0x2134,
	0x2135,
	0x2139,
	0x213A,
	0x213C,
	0x213E,
	0x2146,
	0x214A,
	0x214E,
	0x214F,
	0x2170,
	0x2180,
	0x2184,
	0x2185,
	0x24D0,
	0x24EA,
	0x2C30,
	0x2C5F,
	0x2C61,
	0x2C62,
	0x2C65,
	0x2C67,
	0x2C68,
	0x2C69,
	0x2C6A,
	0x2C6B,
	0x2C6C,
	0x2C6D,
	0x2C71,
	0x2C72,
	0x2C73,
	0x2C75,
	0x2C76,
	0x2C7E,
	0x2C81,
	0x2C82,
	0x2C83,
	0x2C84,
	0x2C85,
	0x2C86,
	0x2C87,
	0x2C88,
	0x2C89,
	0x2C8A,
	0x2C8B,
	0x2C8C,
	0x2C8D,
	0x2C8E,
	0x2C8F,
	0x2C90,
	0x2C91,
	0x2C92,
	0x2C93,
	0x2C94,
	0x2C95,
	0x2C96,
	0x2C97,
	0x2C98,
	0x2C99,
	0x2C9A,
	0x2C9B,
	0x2C9C,
	0x2C9D,
	0x2C9E,
	0x2C9F,
	0x2CA0,
	0x2CA1,
	0x2CA2,
	0x2CA3,
	0x2CA4,
	0x2CA5,
	0x2CA6,
	0x2CA7,
	0x2CA8,
	0x2CA9,
	0x2CAA,
	0x2CAB,
	0x2CAC,
	0x2CAD,
	0x2CAE,
	0x2CAF,
	0x2CB0,
	0x2CB1,
	0x2CB2,
	0x2CB3,
	0x2CB4,
	0x2CB5,
	0x2CB6,
	0x2CB7,
	0x2CB8,
	0x2CB9,
	0x2CBA,
	0x2CBB,
	0x2CBC,
	0x2CBD,
	0x2CBE,
	0x2CBF,
	0x2CC0,
	0x2CC1,
	0x2CC2,
	0x2CC3,
	0x2CC4,
	0x2CC5,
	0x2CC6,
	0x2CC7,
	0x2CC8,
	0x2CC9,
	0x2CCA,
	0x2CCB,
	0x2CCC,
	0x2CCD,
	0x2CCE,
	0x2CCF,
	0x2CD0,
	0x2CD1,
	0x2CD2,
	0x2CD3,
	0x2CD4,
	0x2CD5,
	0x2CD6,
	0x2CD7,
	0x2CD8,
	0x2CD9,
	0x2CDA,
	0x2CDB,
	0x2CDC,
	0x2CDD,
	0x2CDE,
	0x2CDF,
	0x2CE0,
	0x2CE1,
	0x2CE2,
	0x2CE3,
	0x2CE5,
	0x2CEC,
	0x2CED,
	0x2CEE,
	0x2CEF,
	0x2CF3,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0xA641,
	0xA642,
	0xA643,
	0xA644,
	0xA645,
	0xA646,
	0xA647,
	0xA648,
	0xA649,
	0xA64A,
	0xA64B,
	0xA64C,
	0xA64D,
	0xA64E,
	0xA64F,
	0xA650,
	0xA651,
	0xA652,
	0xA653,
	0xA654,
	0xA655,
	0xA656,
	0xA657,
	0xA658,
	0xA659,
	0xA65A,
	0xA65B,
	0xA65C,
	0xA65D,
	0xA65E,
	0xA65F,
	0xA660,
	0xA661,
	0xA662,
	0xA663,
	0xA664,
	0xA665,
	0xA666,
	0xA667,
	0xA668,
	0xA669,
	0xA66A,
	0xA66B,
	0xA66C,
	0xA66D,
	0xA66E,
	0xA681,
	0xA682,
	0xA683,
	0xA684,
	0xA685,
	0xA686,
	0xA687,
	0xA688,
	0xA689,
	0xA68A,
	0xA68B,
	0xA68C,
	0xA68D,
	0xA68E,
	0xA68F,
	0xA690,
	0xA691,
	0xA692,
	0xA693,
	0xA694,
	0xA695,
	0xA696,
	0xA697,
	0xA698,
	0xA699,
	0xA69A,
	0xA69B,
	0xA69E,
	0xA723,
	0xA724,
	0xA725,
	0xA726,
	0xA727,
	0xA728,
	0xA729,
	0xA72A,
	0xA72B,
	0xA72C,
	0xA72D,
	0xA72E,
	0xA72F,
	0xA732,
	0xA733,
	0xA734,
	0xA735,
	0xA736,
	0xA737,
	0xA738,
	0xA739,
	0xA73A,
	0xA73B,
	0xA73C,
	0xA73D,
	0xA73E,
	0xA73F,
	0xA740,
	0xA741,
	0xA742,
	0xA743,
	0xA744,
	0xA745,
	0xA746,
	0xA747,
	0xA748,
	0xA749,
	0xA74A,
	0xA74B,
	0xA74C,
	0xA74D,
	0xA74E,
	0xA74F,
	0xA750,
	0xA751,
	0xA752,
	0xA753,
	0xA754,
	0xA755,
	0xA756,
	0xA757,
	0xA758,
	0xA759,
	0xA75A,
	0xA75B,
	0xA75C,
	0xA75D,
	0xA75E,
	0xA75F,
	0xA760,
	0xA761,
	0xA762,
	0xA763,
	0xA764,
	0xA765,
	0xA766,
	0xA767,
	0xA768,
	0xA769,
	0xA76A,
	0xA76B,
	0xA76C,
	0xA76D,
	0xA76E,
	0xA76F,
	0xA779,
	0xA77A,
	0xA77B,
	0xA77C,
	0xA77D,
	0xA77F,
	0xA780,
	0xA781,
	0xA782,
	0xA783,
	0xA784,
	0xA785,
	0xA786,
	0xA787,
	0xA788,
	0xA78C,
	0xA78D,
	0xA78E,
	0xA78F,
	0xA791,
	0xA792,
	0xA793,
	0xA796,
	0xA797,
	0xA798,
	0xA799,
	0xA79A,
	0xA79B,
	0xA79C,
	0xA79D,
	0xA79E,
	0xA79F,
	0xA7A0,
	0xA7A1,
	0xA7A2,
	0xA7A3,
	0xA7A4,
	0xA7A5,
	0xA7A6,
	0xA7A7,
	0xA7A8,
	0xA7A9,
	0xA7AA,
	0xA7B5,
	0xA7B6,
	0xA7B7,
	0xA7B8,
	0xA7F8,
	0xA7FB,
	0xAB30,
	0xAB5B,
	0xAB5C,
	0xAB66,
	0xAB70,
	0xABC0,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFF41,
	0xFF5B,
	0x10428,
	0x10450,
	0x104D8,
	0x104FC,
	0x10CC0,
	0x10CF3,
	0x118C0,
	0x118E0,
	0x1D41A,
	0x1D434,
	0x1D44E,
	0x1D455,
	0x1D456,
	0x1D468,
	0x1D482,
	0x1D49C,
	0x1D4B6,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D4D0,
	0x1D4EA,
	0x1D504,
	0x1D51E,
	0x1D538,
	0x1D552,
	0x1D56C,
	0x1D586,
	0x1D5A0,
	0x1D5BA,
	0x1D5D4,
	0x1D5EE,
	0x1D608,
	0x1D622,
	0x1D63C,
	0x1D656,
	0x1D670,
	0x1D68A,
	0x1D6A6,
	0x1D6C2,
	0x1D6DB,
	0x1D6DC,
	0x1D6E2,
	0x1D6FC,
	0x1D715,
	0x1D716,
	0x1D71C,
	0x1D736,
	0x1D74F,
	0x1D750,
	0x1D756,
	0x1D770,
	0x1D789,
	0x1D78A,
	0x1D790,
	0x1D7AA,
	0x1D7C3,
	0x1D7C4,
	0x1D7CA,
	0x1D7CB,
	0x1D7CC,
	0x1E922,
	0x1E944
};

static const UV XPosixPrint_invlist[] = { /* for EBCDIC 1047 */
	1281,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x40,
	0xFF,
	0x100,
	0x378,
	0x37A,
	0x380,
	0x384,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x530,
	0x531,
	0x557,
	0x559,
	0x560,
	0x561,
	0x588,
	0x589,
	0x58B,
	0x58D,
	0x590,
	0x591,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F5,
	0x600,
	0x61D,
	0x61E,
	0x70E,
	0x70F,
	0x74B,
	0x74D,
	0x7B2,
	0x7C0,
	0x7FB,
	0x800,
	0x82E,
	0x830,
	0x83F,
	0x840,
	0x85C,
	0x85E,
	0x85F,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x8D4,
	0x984,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BC,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CF,
	0x9D7,
	0x9D8,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E4,
	0x9E6,
	0x9FC,
	0xA01,
	0xA04,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA3C,
	0xA3D,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4E,
	0xA51,
	0xA52,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA66,
	0xA76,
	0xA81,
	0xA84,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABC,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACE,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE4,
	0xAE6,
	0xAF2,
	0xAF9,
	0xAFA,
	0xB01,
	0xB04,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3C,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4E,
	0xB56,
	0xB58,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB64,
	0xB66,
	0xB78,
	0xB82,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCE,
	0xBD0,
	0xBD1,
	0xBD7,
	0xBD8,
	0xBE6,
	0xBFB,
	0xC00,
	0xC04,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4E,
	0xC55,
	0xC57,
	0xC58,
	0xC5B,
	0xC60,
	0xC64,
	0xC66,
	0xC70,
	0xC78,
	0xC84,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBC,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCE,
	0xCD5,
	0xCD7,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE4,
	0xCE6,
	0xCF0,
	0xCF1,
	0xCF3,
	0xD01,
	0xD04,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD50,
	0xD54,
	0xD64,
	0xD66,
	0xD80,
	0xD82,
	0xD84,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xDCA,
	0xDCB,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDE6,
	0xDF0,
	0xDF2,
	0xDF5,
	0xE01,
	0xE3B,
	0xE3F,
	0xE5C,
	0xE81,
	0xE83,
	0xE84,
	0xE85,
	0xE87,
	0xE89,
	0xE8A,
	0xE8B,
	0xE8D,
	0xE8E,
	0xE94,
	0xE98,
	0xE99,
	0xEA0,
	0xEA1,
	0xEA4,
	0xEA5,
	0xEA6,
	0xEA7,
	0xEA8,
	0xEAA,
	0xEAC,
	0xEAD,
	0xEBA,
	0xEBB,
	0xEBE,
	0xEC0,
	0xEC5,
	0xEC6,
	0xEC7,
	0xEC8,
	0xECE,
	0xED0,
	0xEDA,
	0xEDC,
	0xEE0,
	0xF00,
	0xF48,
	0xF49,
	0xF6D,
	0xF71,
	0xF98,
	0xF99,
	0xFBD,
	0xFBE,
	0xFCD,
	0xFCE,
	0xFDB,
	0x1000,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x135D,
	0x137D,
	0x1380,
	0x139A,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1400,
	0x169D,
	0x16A0,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1715,
	0x1720,
	0x1737,
	0x1740,
	0x1754,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1772,
	0x1774,
	0x1780,
	0x17DE,
	0x17E0,
	0x17EA,
	0x17F0,
	0x17FA,
	0x1800,
	0x180F,
	0x1810,
	0x181A,
	0x1820,
	0x1878,
	0x1880,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1920,
	0x192C,
	0x1930,
	0x193C,
	0x1940,
	0x1941,
	0x1944,
	0x196E,
	0x1970,
	0x1975,
	0x1980,
	0x19AC,
	0x19B0,
	0x19CA,
	0x19D0,
	0x19DB,
	0x19DE,
	0x1A1C,
	0x1A1E,
	0x1A5F,
	0x1A60,
	0x1A7D,
	0x1A7F,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1AA0,
	0x1AAE,
	0x1AB0,
	0x1ABF,
	0x1B00,
	0x1B4C,
	0x1B50,
	0x1B7D,
	0x1B80,
	0x1BF4,
	0x1BFC,
	0x1C38,
	0x1C3B,
	0x1C4A,
	0x1C4D,
	0x1C89,
	0x1CC0,
	0x1CC8,
	0x1CD0,
	0x1CF7,
	0x1CF8,
	0x1CFA,
	0x1D00,
	0x1DF6,
	0x1DFB,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FC5,
	0x1FC6,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FDD,
	0x1FF0,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFF,
	0x2000,
	0x2028,
	0x202A,
	0x2065,
	0x2066,
	0x2072,
	0x2074,
	0x208F,
	0x2090,
	0x209D,
	0x20A0,
	0x20BF,
	0x20D0,
	0x20F1,
	0x2100,
	0x218C,
	0x2190,
	0x23FF,
	0x2400,
	0x2427,
	0x2440,
	0x244B,
	0x2460,
	0x2B74,
	0x2B76,
	0x2B96,
	0x2B98,
	0x2BBA,
	0x2BBD,
	0x2BC9,
	0x2BCA,
	0x2BD2,
	0x2BEC,
	0x2BF0,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CF4,
	0x2CF9,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D71,
	0x2D7F,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x2DE0,
	0x2E45,
	0x2E80,
	0x2E9A,
	0x2E9B,
	0x2EF4,
	0x2F00,
	0x2FD6,
	0x2FF0,
	0x2FFC,
	0x3000,
	0x3040,
	0x3041,
	0x3097,
	0x3099,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x3190,
	0x31BB,
	0x31C0,
	0x31E4,
	0x31F0,
	0x321F,
	0x3220,
	0x32FF,
	0x3300,
	0x4DB6,
	0x4DC0,
	0x9FD6,
	0xA000,
	0xA48D,
	0xA490,
	0xA4C7,
	0xA4D0,
	0xA62C,
	0xA640,
	0xA6F8,
	0xA700,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F7,
	0xA82C,
	0xA830,
	0xA83A,
	0xA840,
	0xA878,
	0xA880,
	0xA8C6,
	0xA8CE,
	0xA8DA,
	0xA8E0,
	0xA8FE,
	0xA900,
	0xA954,
	0xA95F,
	0xA97D,
	0xA980,
	0xA9CE,
	0xA9CF,
	0xA9DA,
	0xA9DE,
	0xA9FF,
	0xAA00,
	0xAA37,
	0xAA40,
	0xAA4E,
	0xAA50,
	0xAA5A,
	0xAA5C,
	0xAAC3,
	0xAADB,
	0xAAF7,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB66,
	0xAB70,
	0xABEE,
	0xABF0,
	0xABFA,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xE000,
	0xFA6E,
	0xFA70,
	0xFADA,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFBC2,
	0xFBD3,
	0xFD40,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFE,
	0xFE00,
	0xFE1A,
	0xFE20,
	0xFE53,
	0xFE54,
	0xFE67,
	0xFE68,
	0xFE6C,
	0xFE70,
	0xFE75,
	0xFE76,
	0xFEFD,
	0xFEFF,
	0xFF00,
	0xFF01,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0xFFE0,
	0xFFE7,
	0xFFE8,
	0xFFEF,
	0xFFF9,
	0xFFFE,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10100,
	0x10103,
	0x10107,
	0x10134,
	0x10137,
	0x1018F,
	0x10190,
	0x1019C,
	0x101A0,
	0x101A1,
	0x101D0,
	0x101FE,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x102E0,
	0x102FC,
	0x10300,
	0x10324,
	0x10330,
	0x1034B,
	0x10350,
	0x1037B,
	0x10380,
	0x1039E,
	0x1039F,
	0x103C4,
	0x103C8,
	0x103D6,
	0x10400,
	0x1049E,
	0x104A0,
	0x104AA,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x1056F,
	0x10570,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10857,
	0x1089F,
	0x108A7,
	0x108B0,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x108FB,
	0x1091C,
	0x1091F,
	0x1093A,
	0x1093F,
	0x10940,
	0x10980,
	0x109B8,
	0x109BC,
	0x109D0,
	0x109D2,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A38,
	0x10A3B,
	0x10A3F,
	0x10A48,
	0x10A50,
	0x10A59,
	0x10A60,
	0x10AA0,
	0x10AC0,
	0x10AE7,
	0x10AEB,
	0x10AF7,
	0x10B00,
	0x10B36,
	0x10B39,
	0x10B56,
	0x10B58,
	0x10B73,
	0x10B78,
	0x10B92,
	0x10B99,
	0x10B9D,
	0x10BA9,
	0x10BB0,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x10CFA,
	0x10D00,
	0x10E60,
	0x10E7F,
	0x11000,
	0x1104E,
	0x11052,
	0x11070,
	0x1107F,
	0x110C2,
	0x110D0,
	0x110E9,
	0x110F0,
	0x110FA,
	0x11100,
	0x11135,
	0x11136,
	0x11144,
	0x11150,
	0x11177,
	0x11180,
	0x111CE,
	0x111D0,
	0x111E0,
	0x111E1,
	0x111F5,
	0x11200,
	0x11212,
	0x11213,
	0x1123F,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112AA,
	0x112B0,
	0x112EB,
	0x112F0,
	0x112FA,
	0x11300,
	0x11304,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133C,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134E,
	0x11350,
	0x11351,
	0x11357,
	0x11358,
	0x1135D,
	0x11364,
	0x11366,
	0x1136D,
	0x11370,
	0x11375,
	0x11400,
	0x1145A,
	0x1145B,
	0x1145C,
	0x1145D,
	0x1145E,
	0x11480,
	0x114C8,
	0x114D0,
	0x114DA,
	0x11580,
	0x115B6,
	0x115B8,
	0x115DE,
	0x11600,
	0x11645,
	0x11650,
	0x1165A,
	0x11660,
	0x1166D,
	0x11680,
	0x116B8,
	0x116C0,
	0x116CA,
	0x11700,
	0x1171A,
	0x1171D,
	0x1172C,
	0x11730,
	0x11740,
	0x118A0,
	0x118F3,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C37,
	0x11C38,
	0x11C46,
	0x11C50,
	0x11C6D,
	0x11C70,
	0x11C90,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12470,
	0x12475,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16A60,
	0x16A6A,
	0x16A6E,
	0x16A70,
	0x16AD0,
	0x16AEE,
	0x16AF0,
	0x16AF6,
	0x16B00,
	0x16B46,
	0x16B50,
	0x16B5A,
	0x16B5B,
	0x16B62,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F7F,
	0x16F8F,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1BC9C,
	0x1BCA4,
	0x1D000,
	0x1D0F6,
	0x1D100,
	0x1D127,
	0x1D129,
	0x1D1E9,
	0x1D200,
	0x1D246,
	0x1D300,
	0x1D357,
	0x1D360,
	0x1D372,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D7CC,
	0x1D7CE,
	0x1DA8C,
	0x1DA9B,
	0x1DAA0,
	0x1DAA1,
	0x1DAB0,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E800,
	0x1E8C5,
	0x1E8C7,
	0x1E8D7,
	0x1E900,
	0x1E94B,
	0x1E950,
	0x1E95A,
	0x1E95E,
	0x1E960,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x1EEF0,
	0x1EEF2,
	0x1F000,
	0x1F02C,
	0x1F030,
	0x1F094,
	0x1F0A0,
	0x1F0AF,
	0x1F0B1,
	0x1F0C0,
	0x1F0C1,
	0x1F0D0,
	0x1F0D1,
	0x1F0F6,
	0x1F100,
	0x1F10D,
	0x1F110,
	0x1F12F,
	0x1F130,
	0x1F16C,
	0x1F170,
	0x1F1AD,
	0x1F1E6,
	0x1F203,
	0x1F210,
	0x1F23C,
	0x1F240,
	0x1F249,
	0x1F250,
	0x1F252,
	0x1F300,
	0x1F6D3,
	0x1F6E0,
	0x1F6ED,
	0x1F6F0,
	0x1F6F7,
	0x1F700,
	0x1F774,
	0x1F780,
	0x1F7D5,
	0x1F800,
	0x1F80C,
	0x1F810,
	0x1F848,
	0x1F850,
	0x1F85A,
	0x1F860,
	0x1F888,
	0x1F890,
	0x1F8AE,
	0x1F910,
	0x1F91F,
	0x1F920,
	0x1F928,
	0x1F930,
	0x1F931,
	0x1F933,
	0x1F93F,
	0x1F940,
	0x1F94C,
	0x1F950,
	0x1F95F,
	0x1F980,
	0x1F992,
	0x1F9C0,
	0x1F9C1,
	0x20000,
	0x2A6D7,
	0x2A700,
	0x2B735,
	0x2B740,
	0x2B81E,
	0x2B820,
	0x2CEA2,
	0x2F800,
	0x2FA1E,
	0xE0001,
	0xE0002,
	0xE0020,
	0xE0080,
	0xE0100,
	0xE01F0,
	0xF0000,
	0xFFFFE,
	0x100000,
	0x10FFFE
};

static const UV XPosixPunct_invlist[] = { /* for EBCDIC 1047 */
	335,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x4B,
	0x51,
	0x5A,
	0x62,
	0x6B,
	0x70,
	0x79,
	0x80,
	0x8A,
	0x8C,
	0xA1,
	0xA2,
	0xAA,
	0xAC,
	0xAD,
	0xAE,
	0xB3,
	0xB4,
	0xB5,
	0xB7,
	0xBD,
	0xBE,
	0xC0,
	0xC1,
	0xD0,
	0xD1,
	0xE0,
	0xE1,
	0x37E,
	0x37F,
	0x387,
	0x388,
	0x55A,
	0x560,
	0x589,
	0x58B,
	0x5BE,
	0x5BF,
	0x5C0,
	0x5C1,
	0x5C3,
	0x5C4,
	0x5C6,
	0x5C7,
	0x5F3,
	0x5F5,
	0x609,
	0x60B,
	0x60C,
	0x60E,
	0x61B,
	0x61C,
	0x61E,
	0x620,
	0x66A,
	0x66E,
	0x6D4,
	0x6D5,
	0x700,
	0x70E,
	0x7F7,
	0x7FA,
	0x830,
	0x83F,
	0x85E,
	0x85F,
	0x964,
	0x966,
	0x970,
	0x971,
	0xAF0,
	0xAF1,
	0xDF4,
	0xDF5,
	0xE4F,
	0xE50,
	0xE5A,
	0xE5C,
	0xF04,
	0xF13,
	0xF14,
	0xF15,
	0xF3A,
	0xF3E,
	0xF85,
	0xF86,
	0xFD0,
	0xFD5,
	0xFD9,
	0xFDB,
	0x104A,
	0x1050,
	0x10FB,
	0x10FC,
	0x1360,
	0x1369,
	0x1400,
	0x1401,
	0x166D,
	0x166F,
	0x169B,
	0x169D,
	0x16EB,
	0x16EE,
	0x1735,
	0x1737,
	0x17D4,
	0x17D7,
	0x17D8,
	0x17DB,
	0x1800,
	0x180B,
	0x1944,
	0x1946,
	0x1A1E,
	0x1A20,
	0x1AA0,
	0x1AA7,
	0x1AA8,
	0x1AAE,
	0x1B5A,
	0x1B61,
	0x1BFC,
	0x1C00,
	0x1C3B,
	0x1C40,
	0x1C7E,
	0x1C80,
	0x1CC0,
	0x1CC8,
	0x1CD3,
	0x1CD4,
	0x2010,
	0x2028,
	0x2030,
	0x2044,
	0x2045,
	0x2052,
	0x2053,
	0x205F,
	0x207D,
	0x207F,
	0x208D,
	0x208F,
	0x2308,
	0x230C,
	0x2329,
	0x232B,
	0x2768,
	0x2776,
	0x27C5,
	0x27C7,
	0x27E6,
	0x27F0,
	0x2983,
	0x2999,
	0x29D8,
	0x29DC,
	0x29FC,
	0x29FE,
	0x2CF9,
	0x2CFD,
	0x2CFE,
	0x2D00,
	0x2D70,
	0x2D71,
	0x2E00,
	0x2E2F,
	0x2E30,
	0x2E45,
	0x3001,
	0x3004,
	0x3008,
	0x3012,
	0x3014,
	0x3020,
	0x3030,
	0x3031,
	0x303D,
	0x303E,
	0x30A0,
	0x30A1,
	0x30FB,
	0x30FC,
	0xA4FE,
	0xA500,
	0xA60D,
	0xA610,
	0xA673,
	0xA674,
	0xA67E,
	0xA67F,
	0xA6F2,
	0xA6F8,
	0xA874,
	0xA878,
	0xA8CE,
	0xA8D0,
	0xA8F8,
	0xA8FB,
	0xA8FC,
	0xA8FD,
	0xA92E,
	0xA930,
	0xA95F,
	0xA960,
	0xA9C1,
	0xA9CE,
	0xA9DE,
	0xA9E0,
	0xAA5C,
	0xAA60,
	0xAADE,
	0xAAE0,
	0xAAF0,
	0xAAF2,
	0xABEB,
	0xABEC,
	0xFD3E,
	0xFD40,
	0xFE10,
	0xFE1A,
	0xFE30,
	0xFE53,
	0xFE54,
	0xFE62,
	0xFE63,
	0xFE64,
	0xFE68,
	0xFE69,
	0xFE6A,
	0xFE6C,
	0xFF01,
	0xFF04,
	0xFF05,
	0xFF0B,
	0xFF0C,
	0xFF10,
	0xFF1A,
	0xFF1C,
	0xFF1F,
	0xFF21,
	0xFF3B,
	0xFF3E,
	0xFF3F,
	0xFF40,
	0xFF5B,
	0xFF5C,
	0xFF5D,
	0xFF5E,
	0xFF5F,
	0xFF66,
	0x10100,
	0x10103,
	0x1039F,
	0x103A0,
	0x103D0,
	0x103D1,
	0x1056F,
	0x10570,
	0x10857,
	0x10858,
	0x1091F,
	0x10920,
	0x1093F,
	0x10940,
	0x10A50,
	0x10A59,
	0x10A7F,
	0x10A80,
	0x10AF0,
	0x10AF7,
	0x10B39,
	0x10B40,
	0x10B99,
	0x10B9D,
	0x11047,
	0x1104E,
	0x110BB,
	0x110BD,
	0x110BE,
	0x110C2,
	0x11140,
	0x11144,
	0x11174,
	0x11176,
	0x111C5,
	0x111CA,
	0x111CD,
	0x111CE,
	0x111DB,
	0x111DC,
	0x111DD,
	0x111E0,
	0x11238,
	0x1123E,
	0x112A9,
	0x112AA,
	0x1144B,
	0x11450,
	0x1145B,
	0x1145C,
	0x1145D,
	0x1145E,
	0x114C6,
	0x114C7,
	0x115C1,
	0x115D8,
	0x11641,
	0x11644,
	0x11660,
	0x1166D,
	0x1173C,
	0x1173F,
	0x11C41,
	0x11C46,
	0x11C70,
	0x11C72,
	0x12470,
	0x12475,
	0x16A6E,
	0x16A70,
	0x16AF5,
	0x16AF6,
	0x16B37,
	0x16B3C,
	0x16B44,
	0x16B45,
	0x1BC9F,
	0x1BCA0,
	0x1DA87,
	0x1DA8C,
	0x1E95E,
	0x1E960
};

static const UV XPosixSpace_invlist[] = { /* for EBCDIC 1047 */
	23,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x5,
	0x6,
	0xB,
	0xE,
	0x15,
	0x16,
	0x25,
	0x26,
	0x40,
	0x42,
	0x1680,
	0x1681,
	0x2000,
	0x200B,
	0x2028,
	0x202A,
	0x202F,
	0x2030,
	0x205F,
	0x2060,
	0x3000,
	0x3001
};

static const UV XPosixUpper_invlist[] = { /* for EBCDIC 1047 */
	1283,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x62,
	0x6A,
	0x71,
	0x79,
	0x80,
	0x81,
	0x9E,
	0x9F,
	0xAC,
	0xAD,
	0xAE,
	0xAF,
	0xBA,
	0xBB,
	0xC1,
	0xCA,
	0xD1,
	0xDA,
	0xE2,
	0xEA,
	0xEB,
	0xF0,
	0xFB,
	0xFF,
	0x100,
	0x101,
	0x102,
	0x103,
	0x104,
	0x105,
	0x106,
	0x107,
	0x108,
	0x109,
	0x10A,
	0x10B,
	0x10C,
	0x10D,
	0x10E,
	0x10F,
	0x110,
	0x111,
	0x112,
	0x113,
	0x114,
	0x115,
	0x116,
	0x117,
	0x118,
	0x119,
	0x11A,
	0x11B,
	0x11C,
	0x11D,
	0x11E,
	0x11F,
	0x120,
	0x121,
	0x122,
	0x123,
	0x124,
	0x125,
	0x126,
	0x127,
	0x128,
	0x129,
	0x12A,
	0x12B,
	0x12C,
	0x12D,
	0x12E,
	0x12F,
	0x130,
	0x131,
	0x132,
	0x133,
	0x134,
	0x135,
	0x136,
	0x137,
	0x139,
	0x13A,
	0x13B,
	0x13C,
	0x13D,
	0x13E,
	0x13F,
	0x140,
	0x141,
	0x142,
	0x143,
	0x144,
	0x145,
	0x146,
	0x147,
	0x148,
	0x14A,
	0x14B,
	0x14C,
	0x14D,
	0x14E,
	0x14F,
	0x150,
	0x151,
	0x152,
	0x153,
	0x154,
	0x155,
	0x156,
	0x157,
	0x158,
	0x159,
	0x15A,
	0x15B,
	0x15C,
	0x15D,
	0x15E,
	0x15F,
	0x160,
	0x161,
	0x162,
	0x163,
	0x164,
	0x165,
	0x166,
	0x167,
	0x168,
	0x169,
	0x16A,
	0x16B,
	0x16C,
	0x16D,
	0x16E,
	0x16F,
	0x170,
	0x171,
	0x172,
	0x173,
	0x174,
	0x175,
	0x176,
	0x177,
	0x178,
	0x17A,
	0x17B,
	0x17C,
	0x17D,
	0x17E,
	0x181,
	0x183,
	0x184,
	0x185,
	0x186,
	0x188,
	0x189,
	0x18C,
	0x18E,
	0x192,
	0x193,
	0x195,
	0x196,
	0x199,
	0x19C,
	0x19E,
	0x19F,
	0x1A1,
	0x1A2,
	0x1A3,
	0x1A4,
	0x1A5,
	0x1A6,
	0x1A8,
	0x1A9,
	0x1AA,
	0x1AC,
	0x1AD,
	0x1AE,
	0x1B0,
	0x1B1,
	0x1B4,
	0x1B5,
	0x1B6,
	0x1B7,
	0x1B9,
	0x1BC,
	0x1BD,
	0x1C4,
	0x1C5,
	0x1C7,
	0x1C8,
	0x1CA,
	0x1CB,
	0x1CD,
	0x1CE,
	0x1CF,
	0x1D0,
	0x1D1,
	0x1D2,
	0x1D3,
	0x1D4,
	0x1D5,
	0x1D6,
	0x1D7,
	0x1D8,
	0x1D9,
	0x1DA,
	0x1DB,
	0x1DC,
	0x1DE,
	0x1DF,
	0x1E0,
	0x1E1,
	0x1E2,
	0x1E3,
	0x1E4,
	0x1E5,
	0x1E6,
	0x1E7,
	0x1E8,
	0x1E9,
	0x1EA,
	0x1EB,
	0x1EC,
	0x1ED,
	0x1EE,
	0x1EF,
	0x1F1,
	0x1F2,
	0x1F4,
	0x1F5,
	0x1F6,
	0x1F9,
	0x1FA,
	0x1FB,
	0x1FC,
	0x1FD,
	0x1FE,
	0x1FF,
	0x200,
	0x201,
	0x202,
	0x203,
	0x204,
	0x205,
	0x206,
	0x207,
	0x208,
	0x209,
	0x20A,
	0x20B,
	0x20C,
	0x20D,
	0x20E,
	0x20F,
	0x210,
	0x211,
	0x212,
	0x213,
	0x214,
	0x215,
	0x216,
	0x217,
	0x218,
	0x219,
	0x21A,
	0x21B,
	0x21C,
	0x21D,
	0x21E,
	0x21F,
	0x220,
	0x221,
	0x222,
	0x223,
	0x224,
	0x225,
	0x226,
	0x227,
	0x228,
	0x229,
	0x22A,
	0x22B,
	0x22C,
	0x22D,
	0x22E,
	0x22F,
	0x230,
	0x231,
	0x232,
	0x233,
	0x23A,
	0x23C,
	0x23D,
	0x23F,
	0x241,
	0x242,
	0x243,
	0x247,
	0x248,
	0x249,
	0x24A,
	0x24B,
	0x24C,
	0x24D,
	0x24E,
	0x24F,
	0x370,
	0x371,
	0x372,
	0x373,
	0x376,
	0x377,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x390,
	0x391,
	0x3A2,
	0x3A3,
	0x3AC,
	0x3CF,
	0x3D0,
	0x3D2,
	0x3D5,
	0x3D8,
	0x3D9,
	0x3DA,
	0x3DB,
	0x3DC,
	0x3DD,
	0x3DE,
	0x3DF,
	0x3E0,
	0x3E1,
	0x3E2,
	0x3E3,
	0x3E4,
	0x3E5,
	0x3E6,
	0x3E7,
	0x3E8,
	0x3E9,
	0x3EA,
	0x3EB,
	0x3EC,
	0x3ED,
	0x3EE,
	0x3EF,
	0x3F4,
	0x3F5,
	0x3F7,
	0x3F8,
	0x3F9,
	0x3FB,
	0x3FD,
	0x430,
	0x460,
	0x461,
	0x462,
	0x463,
	0x464,
	0x465,
	0x466,
	0x467,
	0x468,
	0x469,
	0x46A,
	0x46B,
	0x46C,
	0x46D,
	0x46E,
	0x46F,
	0x470,
	0x471,
	0x472,
	0x473,
	0x474,
	0x475,
	0x476,
	0x477,
	0x478,
	0x479,
	0x47A,
	0x47B,
	0x47C,
	0x47D,
	0x47E,
	0x47F,
	0x480,
	0x481,
	0x48A,
	0x48B,
	0x48C,
	0x48D,
	0x48E,
	0x48F,
	0x490,
	0x491,
	0x492,
	0x493,
	0x494,
	0x495,
	0x496,
	0x497,
	0x498,
	0x499,
	0x49A,
	0x49B,
	0x49C,
	0x49D,
	0x49E,
	0x49F,
	0x4A0,
	0x4A1,
	0x4A2,
	0x4A3,
	0x4A4,
	0x4A5,
	0x4A6,
	0x4A7,
	0x4A8,
	0x4A9,
	0x4AA,
	0x4AB,
	0x4AC,
	0x4AD,
	0x4AE,
	0x4AF,
	0x4B0,
	0x4B1,
	0x4B2,
	0x4B3,
	0x4B4,
	0x4B5,
	0x4B6,
	0x4B7,
	0x4B8,
	0x4B9,
	0x4BA,
	0x4BB,
	0x4BC,
	0x4BD,
	0x4BE,
	0x4BF,
	0x4C0,
	0x4C2,
	0x4C3,
	0x4C4,
	0x4C5,
	0x4C6,
	0x4C7,
	0x4C8,
	0x4C9,
	0x4CA,
	0x4CB,
	0x4CC,
	0x4CD,
	0x4CE,
	0x4D0,
	0x4D1,
	0x4D2,
	0x4D3,
	0x4D4,
	0x4D5,
	0x4D6,
	0x4D7,
	0x4D8,
	0x4D9,
	0x4DA,
	0x4DB,
	0x4DC,
	0x4DD,
	0x4DE,
	0x4DF,
	0x4E0,
	0x4E1,
	0x4E2,
	0x4E3,
	0x4E4,
	0x4E5,
	0x4E6,
	0x4E7,
	0x4E8,
	0x4E9,
	0x4EA,
	0x4EB,
	0x4EC,
	0x4ED,
	0x4EE,
	0x4EF,
	0x4F0,
	0x4F1,
	0x4F2,
	0x4F3,
	0x4F4,
	0x4F5,
	0x4F6,
	0x4F7,
	0x4F8,
	0x4F9,
	0x4FA,
	0x4FB,
	0x4FC,
	0x4FD,
	0x4FE,
	0x4FF,
	0x500,
	0x501,
	0x502,
	0x503,
	0x504,
	0x505,
	0x506,
	0x507,
	0x508,
	0x509,
	0x50A,
	0x50B,
	0x50C,
	0x50D,
	0x50E,
	0x50F,
	0x510,
	0x511,
	0x512,
	0x513,
	0x514,
	0x515,
	0x516,
	0x517,
	0x518,
	0x519,
	0x51A,
	0x51B,
	0x51C,
	0x51D,
	0x51E,
	0x51F,
	0x520,
	0x521,
	0x522,
	0x523,
	0x524,
	0x525,
	0x526,
	0x527,
	0x528,
	0x529,
	0x52A,
	0x52B,
	0x52C,
	0x52D,
	0x52E,
	0x52F,
	0x531,
	0x557,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x13A0,
	0x13F6,
	0x1E00,
	0x1E01,
	0x1E02,
	0x1E03,
	0x1E04,
	0x1E05,
	0x1E06,
	0x1E07,
	0x1E08,
	0x1E09,
	0x1E0A,
	0x1E0B,
	0x1E0C,
	0x1E0D,
	0x1E0E,
	0x1E0F,
	0x1E10,
	0x1E11,
	0x1E12,
	0x1E13,
	0x1E14,
	0x1E15,
	0x1E16,
	0x1E17,
	0x1E18,
	0x1E19,
	0x1E1A,
	0x1E1B,
	0x1E1C,
	0x1E1D,
	0x1E1E,
	0x1E1F,
	0x1E20,
	0x1E21,
	0x1E22,
	0x1E23,
	0x1E24,
	0x1E25,
	0x1E26,
	0x1E27,
	0x1E28,
	0x1E29,
	0x1E2A,
	0x1E2B,
	0x1E2C,
	0x1E2D,
	0x1E2E,
	0x1E2F,
	0x1E30,
	0x1E31,
	0x1E32,
	0x1E33,
	0x1E34,
	0x1E35,
	0x1E36,
	0x1E37,
	0x1E38,
	0x1E39,
	0x1E3A,
	0x1E3B,
	0x1E3C,
	0x1E3D,
	0x1E3E,
	0x1E3F,
	0x1E40,
	0x1E41,
	0x1E42,
	0x1E43,
	0x1E44,
	0x1E45,
	0x1E46,
	0x1E47,
	0x1E48,
	0x1E49,
	0x1E4A,
	0x1E4B,
	0x1E4C,
	0x1E4D,
	0x1E4E,
	0x1E4F,
	0x1E50,
	0x1E51,
	0x1E52,
	0x1E53,
	0x1E54,
	0x1E55,
	0x1E56,
	0x1E57,
	0x1E58,
	0x1E59,
	0x1E5A,
	0x1E5B,
	0x1E5C,
	0x1E5D,
	0x1E5E,
	0x1E5F,
	0x1E60,
	0x1E61,
	0x1E62,
	0x1E63,
	0x1E64,
	0x1E65,
	0x1E66,
	0x1E67,
	0x1E68,
	0x1E69,
	0x1E6A,
	0x1E6B,
	0x1E6C,
	0x1E6D,
	0x1E6E,
	0x1E6F,
	0x1E70,
	0x1E71,
	0x1E72,
	0x1E73,
	0x1E74,
	0x1E75,
	0x1E76,
	0x1E77,
	0x1E78,
	0x1E79,
	0x1E7A,
	0x1E7B,
	0x1E7C,
	0x1E7D,
	0x1E7E,
	0x1E7F,
	0x1E80,
	0x1E81,
	0x1E82,
	0x1E83,
	0x1E84,
	0x1E85,
	0x1E86,
	0x1E87,
	0x1E88,
	0x1E89,
	0x1E8A,
	0x1E8B,
	0x1E8C,
	0x1E8D,
	0x1E8E,
	0x1E8F,
	0x1E90,
	0x1E91,
	0x1E92,
	0x1E93,
	0x1E94,
	0x1E95,
	0x1E9E,
	0x1E9F,
	0x1EA0,
	0x1EA1,
	0x1EA2,
	0x1EA3,
	0x1EA4,
	0x1EA5,
	0x1EA6,
	0x1EA7,
	0x1EA8,
	0x1EA9,
	0x1EAA,
	0x1EAB,
	0x1EAC,
	0x1EAD,
	0x1EAE,
	0x1EAF,
	0x1EB0,
	0x1EB1,
	0x1EB2,
	0x1EB3,
	0x1EB4,
	0x1EB5,
	0x1EB6,
	0x1EB7,
	0x1EB8,
	0x1EB9,
	0x1EBA,
	0x1EBB,
	0x1EBC,
	0x1EBD,
	0x1EBE,
	0x1EBF,
	0x1EC0,
	0x1EC1,
	0x1EC2,
	0x1EC3,
	0x1EC4,
	0x1EC5,
	0x1EC6,
	0x1EC7,
	0x1EC8,
	0x1EC9,
	0x1ECA,
	0x1ECB,
	0x1ECC,
	0x1ECD,
	0x1ECE,
	0x1ECF,
	0x1ED0,
	0x1ED1,
	0x1ED2,
	0x1ED3,
	0x1ED4,
	0x1ED5,
	0x1ED6,
	0x1ED7,
	0x1ED8,
	0x1ED9,
	0x1EDA,
	0x1EDB,
	0x1EDC,
	0x1EDD,
	0x1EDE,
	0x1EDF,
	0x1EE0,
	0x1EE1,
	0x1EE2,
	0x1EE3,
	0x1EE4,
	0x1EE5,
	0x1EE6,
	0x1EE7,
	0x1EE8,
	0x1EE9,
	0x1EEA,
	0x1EEB,
	0x1EEC,
	0x1EED,
	0x1EEE,
	0x1EEF,
	0x1EF0,
	0x1EF1,
	0x1EF2,
	0x1EF3,
	0x1EF4,
	0x1EF5,
	0x1EF6,
	0x1EF7,
	0x1EF8,
	0x1EF9,
	0x1EFA,
	0x1EFB,
	0x1EFC,
	0x1EFD,
	0x1EFE,
	0x1EFF,
	0x1F08,
	0x1F10,
	0x1F18,
	0x1F1E,
	0x1F28,
	0x1F30,
	0x1F38,
	0x1F40,
	0x1F48,
	0x1F4E,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F60,
	0x1F68,
	0x1F70,
	0x1FB8,
	0x1FBC,
	0x1FC8,
	0x1FCC,
	0x1FD8,
	0x1FDC,
	0x1FE8,
	0x1FED,
	0x1FF8,
	0x1FFC,
	0x2102,
	0x2103,
	0x2107,
	0x2108,
	0x210B,
	0x210E,
	0x2110,
	0x2113,
	0x2115,
	0x2116,
	0x2119,
	0x211E,
	0x2124,
	0x2125,
	0x2126,
	0x2127,
	0x2128,
	0x2129,
	0x212A,
	0x212E,
	0x2130,
	0x2134,
	0x213E,
	0x2140,
	0x2145,
	0x2146,
	0x2160,
	0x2170,
	0x2183,
	0x2184,
	0x24B6,
	0x24D0,
	0x2C00,
	0x2C2F,
	0x2C60,
	0x2C61,
	0x2C62,
	0x2C65,
	0x2C67,
	0x2C68,
	0x2C69,
	0x2C6A,
	0x2C6B,
	0x2C6C,
	0x2C6D,
	0x2C71,
	0x2C72,
	0x2C73,
	0x2C75,
	0x2C76,
	0x2C7E,
	0x2C81,
	0x2C82,
	0x2C83,
	0x2C84,
	0x2C85,
	0x2C86,
	0x2C87,
	0x2C88,
	0x2C89,
	0x2C8A,
	0x2C8B,
	0x2C8C,
	0x2C8D,
	0x2C8E,
	0x2C8F,
	0x2C90,
	0x2C91,
	0x2C92,
	0x2C93,
	0x2C94,
	0x2C95,
	0x2C96,
	0x2C97,
	0x2C98,
	0x2C99,
	0x2C9A,
	0x2C9B,
	0x2C9C,
	0x2C9D,
	0x2C9E,
	0x2C9F,
	0x2CA0,
	0x2CA1,
	0x2CA2,
	0x2CA3,
	0x2CA4,
	0x2CA5,
	0x2CA6,
	0x2CA7,
	0x2CA8,
	0x2CA9,
	0x2CAA,
	0x2CAB,
	0x2CAC,
	0x2CAD,
	0x2CAE,
	0x2CAF,
	0x2CB0,
	0x2CB1,
	0x2CB2,
	0x2CB3,
	0x2CB4,
	0x2CB5,
	0x2CB6,
	0x2CB7,
	0x2CB8,
	0x2CB9,
	0x2CBA,
	0x2CBB,
	0x2CBC,
	0x2CBD,
	0x2CBE,
	0x2CBF,
	0x2CC0,
	0x2CC1,
	0x2CC2,
	0x2CC3,
	0x2CC4,
	0x2CC5,
	0x2CC6,
	0x2CC7,
	0x2CC8,
	0x2CC9,
	0x2CCA,
	0x2CCB,
	0x2CCC,
	0x2CCD,
	0x2CCE,
	0x2CCF,
	0x2CD0,
	0x2CD1,
	0x2CD2,
	0x2CD3,
	0x2CD4,
	0x2CD5,
	0x2CD6,
	0x2CD7,
	0x2CD8,
	0x2CD9,
	0x2CDA,
	0x2CDB,
	0x2CDC,
	0x2CDD,
	0x2CDE,
	0x2CDF,
	0x2CE0,
	0x2CE1,
	0x2CE2,
	0x2CE3,
	0x2CEB,
	0x2CEC,
	0x2CED,
	0x2CEE,
	0x2CF2,
	0x2CF3,
	0xA640,
	0xA641,
	0xA642,
	0xA643,
	0xA644,
	0xA645,
	0xA646,
	0xA647,
	0xA648,
	0xA649,
	0xA64A,
	0xA64B,
	0xA64C,
	0xA64D,
	0xA64E,
	0xA64F,
	0xA650,
	0xA651,
	0xA652,
	0xA653,
	0xA654,
	0xA655,
	0xA656,
	0xA657,
	0xA658,
	0xA659,
	0xA65A,
	0xA65B,
	0xA65C,
	0xA65D,
	0xA65E,
	0xA65F,
	0xA660,
	0xA661,
	0xA662,
	0xA663,
	0xA664,
	0xA665,
	0xA666,
	0xA667,
	0xA668,
	0xA669,
	0xA66A,
	0xA66B,
	0xA66C,
	0xA66D,
	0xA680,
	0xA681,
	0xA682,
	0xA683,
	0xA684,
	0xA685,
	0xA686,
	0xA687,
	0xA688,
	0xA689,
	0xA68A,
	0xA68B,
	0xA68C,
	0xA68D,
	0xA68E,
	0xA68F,
	0xA690,
	0xA691,
	0xA692,
	0xA693,
	0xA694,
	0xA695,
	0xA696,
	0xA697,
	0xA698,
	0xA699,
	0xA69A,
	0xA69B,
	0xA722,
	0xA723,
	0xA724,
	0xA725,
	0xA726,
	0xA727,
	0xA728,
	0xA729,
	0xA72A,
	0xA72B,
	0xA72C,
	0xA72D,
	0xA72E,
	0xA72F,
	0xA732,
	0xA733,
	0xA734,
	0xA735,
	0xA736,
	0xA737,
	0xA738,
	0xA739,
	0xA73A,
	0xA73B,
	0xA73C,
	0xA73D,
	0xA73E,
	0xA73F,
	0xA740,
	0xA741,
	0xA742,
	0xA743,
	0xA744,
	0xA745,
	0xA746,
	0xA747,
	0xA748,
	0xA749,
	0xA74A,
	0xA74B,
	0xA74C,
	0xA74D,
	0xA74E,
	0xA74F,
	0xA750,
	0xA751,
	0xA752,
	0xA753,
	0xA754,
	0xA755,
	0xA756,
	0xA757,
	0xA758,
	0xA759,
	0xA75A,
	0xA75B,
	0xA75C,
	0xA75D,
	0xA75E,
	0xA75F,
	0xA760,
	0xA761,
	0xA762,
	0xA763,
	0xA764,
	0xA765,
	0xA766,
	0xA767,
	0xA768,
	0xA769,
	0xA76A,
	0xA76B,
	0xA76C,
	0xA76D,
	0xA76E,
	0xA76F,
	0xA779,
	0xA77A,
	0xA77B,
	0xA77C,
	0xA77D,
	0xA77F,
	0xA780,
	0xA781,
	0xA782,
	0xA783,
	0xA784,
	0xA785,
	0xA786,
	0xA787,
	0xA78B,
	0xA78C,
	0xA78D,
	0xA78E,
	0xA790,
	0xA791,
	0xA792,
	0xA793,
	0xA796,
	0xA797,
	0xA798,
	0xA799,
	0xA79A,
	0xA79B,
	0xA79C,
	0xA79D,
	0xA79E,
	0xA79F,
	0xA7A0,
	0xA7A1,
	0xA7A2,
	0xA7A3,
	0xA7A4,
	0xA7A5,
	0xA7A6,
	0xA7A7,
	0xA7A8,
	0xA7A9,
	0xA7AA,
	0xA7AF,
	0xA7B0,
	0xA7B5,
	0xA7B6,
	0xA7B7,
	0xFF21,
	0xFF3B,
	0x10400,
	0x10428,
	0x104B0,
	0x104D4,
	0x10C80,
	0x10CB3,
	0x118A0,
	0x118C0,
	0x1D400,
	0x1D41A,
	0x1D434,
	0x1D44E,
	0x1D468,
	0x1D482,
	0x1D49C,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4B6,
	0x1D4D0,
	0x1D4EA,
	0x1D504,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D538,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D56C,
	0x1D586,
	0x1D5A0,
	0x1D5BA,
	0x1D5D4,
	0x1D5EE,
	0x1D608,
	0x1D622,
	0x1D63C,
	0x1D656,
	0x1D670,
	0x1D68A,
	0x1D6A8,
	0x1D6C1,
	0x1D6E2,
	0x1D6FB,
	0x1D71C,
	0x1D735,
	0x1D756,
	0x1D76F,
	0x1D790,
	0x1D7A9,
	0x1D7CA,
	0x1D7CB,
	0x1E900,
	0x1E922,
	0x1F130,
	0x1F14A,
	0x1F150,
	0x1F16A,
	0x1F170,
	0x1F18A
};

static const UV XPosixWord_invlist[] = { /* for EBCDIC 1047 */
	1385,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x42,
	0x4A,
	0x51,
	0x5A,
	0x62,
	0x6A,
	0x6D,
	0x6E,
	0x70,
	0x79,
	0x80,
	0x8A,
	0x8C,
	0x8F,
	0x91,
	0x9D,
	0x9E,
	0x9F,
	0xA0,
	0xA1,
	0xA2,
	0xAA,
	0xAC,
	0xAD,
	0xAE,
	0xAF,
	0xBA,
	0xBB,
	0xC1,
	0xCA,
	0xCB,
	0xD0,
	0xD1,
	0xDA,
	0xDB,
	0xE0,
	0xE2,
	0xEA,
	0xEB,
	0xFA,
	0xFB,
	0xFF,
	0x100,
	0x2C2,
	0x2C6,
	0x2D2,
	0x2E0,
	0x2E5,
	0x2EC,
	0x2ED,
	0x2EE,
	0x2EF,
	0x300,
	0x375,
	0x376,
	0x378,
	0x37A,
	0x37E,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x3F6,
	0x3F7,
	0x482,
	0x483,
	0x530,
	0x531,
	0x557,
	0x559,
	0x55A,
	0x561,
	0x588,
	0x591,
	0x5BE,
	0x5BF,
	0x5C0,
	0x5C1,
	0x5C3,
	0x5C4,
	0x5C6,
	0x5C7,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F3,
	0x610,
	0x61B,
	0x620,
	0x66A,
	0x66E,
	0x6D4,
	0x6D5,
	0x6DD,
	0x6DF,
	0x6E9,
	0x6EA,
	0x6FD,
	0x6FF,
	0x700,
	0x710,
	0x74B,
	0x74D,
	0x7B2,
	0x7C0,
	0x7F6,
	0x7FA,
	0x7FB,
	0x800,
	0x82E,
	0x840,
	0x85C,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x8D4,
	0x8E2,
	0x8E3,
	0x964,
	0x966,
	0x970,
	0x971,
	0x984,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BC,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CF,
	0x9D7,
	0x9D8,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E4,
	0x9E6,
	0x9F2,
	0xA01,
	0xA04,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA3C,
	0xA3D,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4E,
	0xA51,
	0xA52,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA66,
	0xA76,
	0xA81,
	0xA84,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABC,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACE,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE4,
	0xAE6,
	0xAF0,
	0xAF9,
	0xAFA,
	0xB01,
	0xB04,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3C,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4E,
	0xB56,
	0xB58,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB64,
	0xB66,
	0xB70,
	0xB71,
	0xB72,
	0xB82,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCE,
	0xBD0,
	0xBD1,
	0xBD7,
	0xBD8,
	0xBE6,
	0xBF0,
	0xC00,
	0xC04,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4E,
	0xC55,
	0xC57,
	0xC58,
	0xC5B,
	0xC60,
	0xC64,
	0xC66,
	0xC70,
	0xC80,
	0xC84,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBC,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCE,
	0xCD5,
	0xCD7,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE4,
	0xCE6,
	0xCF0,
	0xCF1,
	0xCF3,
	0xD01,
	0xD04,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD4F,
	0xD54,
	0xD58,
	0xD5F,
	0xD64,
	0xD66,
	0xD70,
	0xD7A,
	0xD80,
	0xD82,
	0xD84,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xDCA,
	0xDCB,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDE6,
	0xDF0,
	0xDF2,
	0xDF4,
	0xE01,
	0xE3B,
	0xE40,
	0xE4F,
	0xE50,
	0xE5A,
	0xE81,
	0xE83,
	0xE84,
	0xE85,
	0xE87,
	0xE89,
	0xE8A,
	0xE8B,
	0xE8D,
	0xE8E,
	0xE94,
	0xE98,
	0xE99,
	0xEA0,
	0xEA1,
	0xEA4,
	0xEA5,
	0xEA6,
	0xEA7,
	0xEA8,
	0xEAA,
	0xEAC,
	0xEAD,
	0xEBA,
	0xEBB,
	0xEBE,
	0xEC0,
	0xEC5,
	0xEC6,
	0xEC7,
	0xEC8,
	0xECE,
	0xED0,
	0xEDA,
	0xEDC,
	0xEE0,
	0xF00,
	0xF01,
	0xF18,
	0xF1A,
	0xF20,
	0xF2A,
	0xF35,
	0xF36,
	0xF37,
	0xF38,
	0xF39,
	0xF3A,
	0xF3E,
	0xF48,
	0xF49,
	0xF6D,
	0xF71,
	0xF85,
	0xF86,
	0xF98,
	0xF99,
	0xFBD,
	0xFC6,
	0xFC7,
	0x1000,
	0x104A,
	0x1050,
	0x109E,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x10FB,
	0x10FC,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x135D,
	0x1360,
	0x1380,
	0x1390,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1401,
	0x166D,
	0x166F,
	0x1680,
	0x1681,
	0x169B,
	0x16A0,
	0x16EB,
	0x16EE,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1715,
	0x1720,
	0x1735,
	0x1740,
	0x1754,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1772,
	0x1774,
	0x1780,
	0x17D4,
	0x17D7,
	0x17D8,
	0x17DC,
	0x17DE,
	0x17E0,
	0x17EA,
	0x180B,
	0x180E,
	0x1810,
	0x181A,
	0x1820,
	0x1878,
	0x1880,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1920,
	0x192C,
	0x1930,
	0x193C,
	0x1946,
	0x196E,
	0x1970,
	0x1975,
	0x1980,
	0x19AC,
	0x19B0,
	0x19CA,
	0x19D0,
	0x19DA,
	0x1A00,
	0x1A1C,
	0x1A20,
	0x1A5F,
	0x1A60,
	0x1A7D,
	0x1A7F,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1AA7,
	0x1AA8,
	0x1AB0,
	0x1ABF,
	0x1B00,
	0x1B4C,
	0x1B50,
	0x1B5A,
	0x1B6B,
	0x1B74,
	0x1B80,
	0x1BF4,
	0x1C00,
	0x1C38,
	0x1C40,
	0x1C4A,
	0x1C4D,
	0x1C7E,
	0x1C80,
	0x1C89,
	0x1CD0,
	0x1CD3,
	0x1CD4,
	0x1CF7,
	0x1CF8,
	0x1CFA,
	0x1D00,
	0x1DF6,
	0x1DFB,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FBD,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FCD,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FE0,
	0x1FED,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFD,
	0x200C,
	0x200E,
	0x203F,
	0x2041,
	0x2054,
	0x2055,
	0x2071,
	0x2072,
	0x207F,
	0x2080,
	0x2090,
	0x209D,
	0x20D0,
	0x20F1,
	0x2102,
	0x2103,
	0x2107,
	0x2108,
	0x210A,
	0x2114,
	0x2115,
	0x2116,
	0x2119,
	0x211E,
	0x2124,
	0x2125,
	0x2126,
	0x2127,
	0x2128,
	0x2129,
	0x212A,
	0x212E,
	0x212F,
	0x213A,
	0x213C,
	0x2140,
	0x2145,
	0x214A,
	0x214E,
	0x214F,
	0x2160,
	0x2189,
	0x24B6,
	0x24EA,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CE5,
	0x2CEB,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D70,
	0x2D7F,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x2DE0,
	0x2E00,
	0x2E2F,
	0x2E30,
	0x3005,
	0x3008,
	0x3021,
	0x3030,
	0x3031,
	0x3036,
	0x3038,
	0x303D,
	0x3041,
	0x3097,
	0x3099,
	0x309B,
	0x309D,
	0x30A0,
	0x30A1,
	0x30FB,
	0x30FC,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x31A0,
	0x31BB,
	0x31F0,
	0x3200,
	0x3400,
	0x4DB6,
	0x4E00,
	0x9FD6,
	0xA000,
	0xA48D,
	0xA4D0,
	0xA4FE,
	0xA500,
	0xA60D,
	0xA610,
	0xA62C,
	0xA640,
	0xA673,
	0xA674,
	0xA67E,
	0xA67F,
	0xA6F2,
	0xA717,
	0xA720,
	0xA722,
	0xA789,
	0xA78B,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F7,
	0xA828,
	0xA840,
	0xA874,
	0xA880,
	0xA8C6,
	0xA8D0,
	0xA8DA,
	0xA8E0,
	0xA8F8,
	0xA8FB,
	0xA8FC,
	0xA8FD,
	0xA8FE,
	0xA900,
	0xA92E,
	0xA930,
	0xA954,
	0xA960,
	0xA97D,
	0xA980,
	0xA9C1,
	0xA9CF,
	0xA9DA,
	0xA9E0,
	0xA9FF,
	0xAA00,
	0xAA37,
	0xAA40,
	0xAA4E,
	0xAA50,
	0xAA5A,
	0xAA60,
	0xAA77,
	0xAA7A,
	0xAAC3,
	0xAADB,
	0xAADE,
	0xAAE0,
	0xAAF0,
	0xAAF2,
	0xAAF7,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB5B,
	0xAB5C,
	0xAB66,
	0xAB70,
	0xABEB,
	0xABEC,
	0xABEE,
	0xABF0,
	0xABFA,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xF900,
	0xFA6E,
	0xFA70,
	0xFADA,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB29,
	0xFB2A,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFBB2,
	0xFBD3,
	0xFD3E,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFC,
	0xFE00,
	0xFE10,
	0xFE20,
	0xFE30,
	0xFE33,
	0xFE35,
	0xFE4D,
	0xFE50,
	0xFE70,
	0xFE75,
	0xFE76,
	0xFEFD,
	0xFF10,
	0xFF1A,
	0xFF21,
	0xFF3B,
	0xFF3F,
	0xFF40,
	0xFF41,
	0xFF5B,
	0xFF66,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10140,
	0x10175,
	0x101FD,
	0x101FE,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x102E0,
	0x102E1,
	0x10300,
	0x10320,
	0x10330,
	0x1034B,
	0x10350,
	0x1037B,
	0x10380,
	0x1039E,
	0x103A0,
	0x103C4,
	0x103C8,
	0x103D0,
	0x103D1,
	0x103D6,
	0x10400,
	0x1049E,
	0x104A0,
	0x104AA,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10860,
	0x10877,
	0x10880,
	0x1089F,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x10900,
	0x10916,
	0x10920,
	0x1093A,
	0x10980,
	0x109B8,
	0x109BE,
	0x109C0,
	0x10A00,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A38,
	0x10A3B,
	0x10A3F,
	0x10A40,
	0x10A60,
	0x10A7D,
	0x10A80,
	0x10A9D,
	0x10AC0,
	0x10AC8,
	0x10AC9,
	0x10AE7,
	0x10B00,
	0x10B36,
	0x10B40,
	0x10B56,
	0x10B60,
	0x10B73,
	0x10B80,
	0x10B92,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x11000,
	0x11047,
	0x11066,
	0x11070,
	0x1107F,
	0x110BB,
	0x110D0,
	0x110E9,
	0x110F0,
	0x110FA,
	0x11100,
	0x11135,
	0x11136,
	0x11140,
	0x11150,
	0x11174,
	0x11176,
	0x11177,
	0x11180,
	0x111C5,
	0x111CA,
	0x111CD,
	0x111D0,
	0x111DB,
	0x111DC,
	0x111DD,
	0x11200,
	0x11212,
	0x11213,
	0x11238,
	0x1123E,
	0x1123F,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112A9,
	0x112B0,
	0x112EB,
	0x112F0,
	0x112FA,
	0x11300,
	0x11304,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133C,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134E,
	0x11350,
	0x11351,
	0x11357,
	0x11358,
	0x1135D,
	0x11364,
	0x11366,
	0x1136D,
	0x11370,
	0x11375,
	0x11400,
	0x1144B,
	0x11450,
	0x1145A,
	0x11480,
	0x114C6,
	0x114C7,
	0x114C8,
	0x114D0,
	0x114DA,
	0x11580,
	0x115B6,
	0x115B8,
	0x115C1,
	0x115D8,
	0x115DE,
	0x11600,
	0x11641,
	0x11644,
	0x11645,
	0x11650,
	0x1165A,
	0x11680,
	0x116B8,
	0x116C0,
	0x116CA,
	0x11700,
	0x1171A,
	0x1171D,
	0x1172C,
	0x11730,
	0x1173A,
	0x118A0,
	0x118EA,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C37,
	0x11C38,
	0x11C41,
	0x11C50,
	0x11C5A,
	0x11C72,
	0x11C90,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16A60,
	0x16A6A,
	0x16AD0,
	0x16AEE,
	0x16AF0,
	0x16AF5,
	0x16B00,
	0x16B37,
	0x16B40,
	0x16B44,
	0x16B50,
	0x16B5A,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F7F,
	0x16F8F,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1BC9D,
	0x1BC9F,
	0x1D165,
	0x1D16A,
	0x1D16D,
	0x1D173,
	0x1D17B,
	0x1D183,
	0x1D185,
	0x1D18C,
	0x1D1AA,
	0x1D1AE,
	0x1D242,
	0x1D245,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D6C1,
	0x1D6C2,
	0x1D6DB,
	0x1D6DC,
	0x1D6FB,
	0x1D6FC,
	0x1D715,
	0x1D716,
	0x1D735,
	0x1D736,
	0x1D74F,
	0x1D750,
	0x1D76F,
	0x1D770,
	0x1D789,
	0x1D78A,
	0x1D7A9,
	0x1D7AA,
	0x1D7C3,
	0x1D7C4,
	0x1D7CC,
	0x1D7CE,
	0x1D800,
	0x1DA00,
	0x1DA37,
	0x1DA3B,
	0x1DA6D,
	0x1DA75,
	0x1DA76,
	0x1DA84,
	0x1DA85,
	0x1DA9B,
	0x1DAA0,
	0x1DAA1,
	0x1DAB0,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E800,
	0x1E8C5,
	0x1E8D0,
	0x1E8D7,
	0x1E900,
	0x1E94B,
	0x1E950,
	0x1E95A,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x1F130,
	0x1F14A,
	0x1F150,
	0x1F16A,
	0x1F170,
	0x1F18A,
	0x20000,
	0x2A6D7,
	0x2A700,
	0x2B735,
	0x2B740,
	0x2B81E,
	0x2B820,
	0x2CEA2,
	0x2F800,
	0x2FA1E,
	0xE0100,
	0xE01F0
};

static const UV XPosixXDigit_invlist[] = { /* for EBCDIC 1047 */
	13,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x81,
	0x87,
	0xC1,
	0xC7,
	0xF0,
	0xFA,
	0xFF10,
	0xFF1A,
	0xFF21,
	0xFF27,
	0xFF41,
	0xFF47
};

#endif	/* defined(PERL_IN_PERL_C) */

#endif	/* EBCDIC 1047 */

#if 'A' == 193 /* EBCDIC 037 */ \
     && '\\' == 224 && '[' == 186 && ']' == 187 && '{' == 192 && '}' == 208 \
     && '^' == 176 && '~' == 161 && '!' == 90 && '#' == 123 && '|' == 79 \
     && '$' == 91 && '@' == 124 && '`' == 121

#if defined(PERL_IN_PERL_C)

static const UV ASCII_invlist[] = { /* for EBCDIC 037 */
	56,	/* Number of elements */
	148565664, /* Version and data structure type */
	0,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x4,
	0x5,
	0x6,
	0x7,
	0x8,
	0xB,
	0x14,
	0x16,
	0x17,
	0x18,
	0x1A,
	0x1C,
	0x20,
	0x25,
	0x28,
	0x2D,
	0x30,
	0x32,
	0x33,
	0x37,
	0x38,
	0x3C,
	0x3E,
	0x3F,
	0x41,
	0x4B,
	0x51,
	0x5A,
	0x5F,
	0x60,
	0x62,
	0x6B,
	0x70,
	0x79,
	0x80,
	0x81,
	0x8A,
	0x91,
	0x9A,
	0xA1,
	0xAA,
	0xB0,
	0xB1,
	0xBA,
	0xBC,
	0xC0,
	0xCA,
	0xD0,
	0xDA,
	0xE0,
	0xE1,
	0xE2,
	0xEA,
	0xF0,
	0xFA
};

static const UV Assigned_invlist[] = { /* for EBCDIC 037 */
	1276,	/* Number of elements */
	148565664, /* Version and data structure type */
	0,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x378,
	0x37A,
	0x380,
	0x384,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x530,
	0x531,
	0x557,
	0x559,
	0x560,
	0x561,
	0x588,
	0x589,
	0x58B,
	0x58D,
	0x590,
	0x591,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F5,
	0x600,
	0x61D,
	0x61E,
	0x70E,
	0x70F,
	0x74B,
	0x74D,
	0x7B2,
	0x7C0,
	0x7FB,
	0x800,
	0x82E,
	0x830,
	0x83F,
	0x840,
	0x85C,
	0x85E,
	0x85F,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x8D4,
	0x984,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BC,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CF,
	0x9D7,
	0x9D8,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E4,
	0x9E6,
	0x9FC,
	0xA01,
	0xA04,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA3C,
	0xA3D,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4E,
	0xA51,
	0xA52,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA66,
	0xA76,
	0xA81,
	0xA84,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABC,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACE,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE4,
	0xAE6,
	0xAF2,
	0xAF9,
	0xAFA,
	0xB01,
	0xB04,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3C,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4E,
	0xB56,
	0xB58,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB64,
	0xB66,
	0xB78,
	0xB82,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCE,
	0xBD0,
	0xBD1,
	0xBD7,
	0xBD8,
	0xBE6,
	0xBFB,
	0xC00,
	0xC04,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4E,
	0xC55,
	0xC57,
	0xC58,
	0xC5B,
	0xC60,
	0xC64,
	0xC66,
	0xC70,
	0xC78,
	0xC84,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBC,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCE,
	0xCD5,
	0xCD7,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE4,
	0xCE6,
	0xCF0,
	0xCF1,
	0xCF3,
	0xD01,
	0xD04,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD50,
	0xD54,
	0xD64,
	0xD66,
	0xD80,
	0xD82,
	0xD84,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xDCA,
	0xDCB,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDE6,
	0xDF0,
	0xDF2,
	0xDF5,
	0xE01,
	0xE3B,
	0xE3F,
	0xE5C,
	0xE81,
	0xE83,
	0xE84,
	0xE85,
	0xE87,
	0xE89,
	0xE8A,
	0xE8B,
	0xE8D,
	0xE8E,
	0xE94,
	0xE98,
	0xE99,
	0xEA0,
	0xEA1,
	0xEA4,
	0xEA5,
	0xEA6,
	0xEA7,
	0xEA8,
	0xEAA,
	0xEAC,
	0xEAD,
	0xEBA,
	0xEBB,
	0xEBE,
	0xEC0,
	0xEC5,
	0xEC6,
	0xEC7,
	0xEC8,
	0xECE,
	0xED0,
	0xEDA,
	0xEDC,
	0xEE0,
	0xF00,
	0xF48,
	0xF49,
	0xF6D,
	0xF71,
	0xF98,
	0xF99,
	0xFBD,
	0xFBE,
	0xFCD,
	0xFCE,
	0xFDB,
	0x1000,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x135D,
	0x137D,
	0x1380,
	0x139A,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1400,
	0x169D,
	0x16A0,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1715,
	0x1720,
	0x1737,
	0x1740,
	0x1754,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1772,
	0x1774,
	0x1780,
	0x17DE,
	0x17E0,
	0x17EA,
	0x17F0,
	0x17FA,
	0x1800,
	0x180F,
	0x1810,
	0x181A,
	0x1820,
	0x1878,
	0x1880,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1920,
	0x192C,
	0x1930,
	0x193C,
	0x1940,
	0x1941,
	0x1944,
	0x196E,
	0x1970,
	0x1975,
	0x1980,
	0x19AC,
	0x19B0,
	0x19CA,
	0x19D0,
	0x19DB,
	0x19DE,
	0x1A1C,
	0x1A1E,
	0x1A5F,
	0x1A60,
	0x1A7D,
	0x1A7F,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1AA0,
	0x1AAE,
	0x1AB0,
	0x1ABF,
	0x1B00,
	0x1B4C,
	0x1B50,
	0x1B7D,
	0x1B80,
	0x1BF4,
	0x1BFC,
	0x1C38,
	0x1C3B,
	0x1C4A,
	0x1C4D,
	0x1C89,
	0x1CC0,
	0x1CC8,
	0x1CD0,
	0x1CF7,
	0x1CF8,
	0x1CFA,
	0x1D00,
	0x1DF6,
	0x1DFB,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FC5,
	0x1FC6,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FDD,
	0x1FF0,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFF,
	0x2000,
	0x2065,
	0x2066,
	0x2072,
	0x2074,
	0x208F,
	0x2090,
	0x209D,
	0x20A0,
	0x20BF,
	0x20D0,
	0x20F1,
	0x2100,
	0x218C,
	0x2190,
	0x23FF,
	0x2400,
	0x2427,
	0x2440,
	0x244B,
	0x2460,
	0x2B74,
	0x2B76,
	0x2B96,
	0x2B98,
	0x2BBA,
	0x2BBD,
	0x2BC9,
	0x2BCA,
	0x2BD2,
	0x2BEC,
	0x2BF0,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CF4,
	0x2CF9,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D71,
	0x2D7F,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x2DE0,
	0x2E45,
	0x2E80,
	0x2E9A,
	0x2E9B,
	0x2EF4,
	0x2F00,
	0x2FD6,
	0x2FF0,
	0x2FFC,
	0x3000,
	0x3040,
	0x3041,
	0x3097,
	0x3099,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x3190,
	0x31BB,
	0x31C0,
	0x31E4,
	0x31F0,
	0x321F,
	0x3220,
	0x32FF,
	0x3300,
	0x4DB6,
	0x4DC0,
	0x9FD6,
	0xA000,
	0xA48D,
	0xA490,
	0xA4C7,
	0xA4D0,
	0xA62C,
	0xA640,
	0xA6F8,
	0xA700,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F7,
	0xA82C,
	0xA830,
	0xA83A,
	0xA840,
	0xA878,
	0xA880,
	0xA8C6,
	0xA8CE,
	0xA8DA,
	0xA8E0,
	0xA8FE,
	0xA900,
	0xA954,
	0xA95F,
	0xA97D,
	0xA980,
	0xA9CE,
	0xA9CF,
	0xA9DA,
	0xA9DE,
	0xA9FF,
	0xAA00,
	0xAA37,
	0xAA40,
	0xAA4E,
	0xAA50,
	0xAA5A,
	0xAA5C,
	0xAAC3,
	0xAADB,
	0xAAF7,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB66,
	0xAB70,
	0xABEE,
	0xABF0,
	0xABFA,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xD800,
	0xFA6E,
	0xFA70,
	0xFADA,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFBC2,
	0xFBD3,
	0xFD40,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFE,
	0xFE00,
	0xFE1A,
	0xFE20,
	0xFE53,
	0xFE54,
	0xFE67,
	0xFE68,
	0xFE6C,
	0xFE70,
	0xFE75,
	0xFE76,
	0xFEFD,
	0xFEFF,
	0xFF00,
	0xFF01,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0xFFE0,
	0xFFE7,
	0xFFE8,
	0xFFEF,
	0xFFF9,
	0xFFFE,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10100,
	0x10103,
	0x10107,
	0x10134,
	0x10137,
	0x1018F,
	0x10190,
	0x1019C,
	0x101A0,
	0x101A1,
	0x101D0,
	0x101FE,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x102E0,
	0x102FC,
	0x10300,
	0x10324,
	0x10330,
	0x1034B,
	0x10350,
	0x1037B,
	0x10380,
	0x1039E,
	0x1039F,
	0x103C4,
	0x103C8,
	0x103D6,
	0x10400,
	0x1049E,
	0x104A0,
	0x104AA,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x1056F,
	0x10570,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10857,
	0x1089F,
	0x108A7,
	0x108B0,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x108FB,
	0x1091C,
	0x1091F,
	0x1093A,
	0x1093F,
	0x10940,
	0x10980,
	0x109B8,
	0x109BC,
	0x109D0,
	0x109D2,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A38,
	0x10A3B,
	0x10A3F,
	0x10A48,
	0x10A50,
	0x10A59,
	0x10A60,
	0x10AA0,
	0x10AC0,
	0x10AE7,
	0x10AEB,
	0x10AF7,
	0x10B00,
	0x10B36,
	0x10B39,
	0x10B56,
	0x10B58,
	0x10B73,
	0x10B78,
	0x10B92,
	0x10B99,
	0x10B9D,
	0x10BA9,
	0x10BB0,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x10CFA,
	0x10D00,
	0x10E60,
	0x10E7F,
	0x11000,
	0x1104E,
	0x11052,
	0x11070,
	0x1107F,
	0x110C2,
	0x110D0,
	0x110E9,
	0x110F0,
	0x110FA,
	0x11100,
	0x11135,
	0x11136,
	0x11144,
	0x11150,
	0x11177,
	0x11180,
	0x111CE,
	0x111D0,
	0x111E0,
	0x111E1,
	0x111F5,
	0x11200,
	0x11212,
	0x11213,
	0x1123F,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112AA,
	0x112B0,
	0x112EB,
	0x112F0,
	0x112FA,
	0x11300,
	0x11304,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133C,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134E,
	0x11350,
	0x11351,
	0x11357,
	0x11358,
	0x1135D,
	0x11364,
	0x11366,
	0x1136D,
	0x11370,
	0x11375,
	0x11400,
	0x1145A,
	0x1145B,
	0x1145C,
	0x1145D,
	0x1145E,
	0x11480,
	0x114C8,
	0x114D0,
	0x114DA,
	0x11580,
	0x115B6,
	0x115B8,
	0x115DE,
	0x11600,
	0x11645,
	0x11650,
	0x1165A,
	0x11660,
	0x1166D,
	0x11680,
	0x116B8,
	0x116C0,
	0x116CA,
	0x11700,
	0x1171A,
	0x1171D,
	0x1172C,
	0x11730,
	0x11740,
	0x118A0,
	0x118F3,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C37,
	0x11C38,
	0x11C46,
	0x11C50,
	0x11C6D,
	0x11C70,
	0x11C90,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12470,
	0x12475,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16A60,
	0x16A6A,
	0x16A6E,
	0x16A70,
	0x16AD0,
	0x16AEE,
	0x16AF0,
	0x16AF6,
	0x16B00,
	0x16B46,
	0x16B50,
	0x16B5A,
	0x16B5B,
	0x16B62,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F7F,
	0x16F8F,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1BC9C,
	0x1BCA4,
	0x1D000,
	0x1D0F6,
	0x1D100,
	0x1D127,
	0x1D129,
	0x1D1E9,
	0x1D200,
	0x1D246,
	0x1D300,
	0x1D357,
	0x1D360,
	0x1D372,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D7CC,
	0x1D7CE,
	0x1DA8C,
	0x1DA9B,
	0x1DAA0,
	0x1DAA1,
	0x1DAB0,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E800,
	0x1E8C5,
	0x1E8C7,
	0x1E8D7,
	0x1E900,
	0x1E94B,
	0x1E950,
	0x1E95A,
	0x1E95E,
	0x1E960,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x1EEF0,
	0x1EEF2,
	0x1F000,
	0x1F02C,
	0x1F030,
	0x1F094,
	0x1F0A0,
	0x1F0AF,
	0x1F0B1,
	0x1F0C0,
	0x1F0C1,
	0x1F0D0,
	0x1F0D1,
	0x1F0F6,
	0x1F100,
	0x1F10D,
	0x1F110,
	0x1F12F,
	0x1F130,
	0x1F16C,
	0x1F170,
	0x1F1AD,
	0x1F1E6,
	0x1F203,
	0x1F210,
	0x1F23C,
	0x1F240,
	0x1F249,
	0x1F250,
	0x1F252,
	0x1F300,
	0x1F6D3,
	0x1F6E0,
	0x1F6ED,
	0x1F6F0,
	0x1F6F7,
	0x1F700,
	0x1F774,
	0x1F780,
	0x1F7D5,
	0x1F800,
	0x1F80C,
	0x1F810,
	0x1F848,
	0x1F850,
	0x1F85A,
	0x1F860,
	0x1F888,
	0x1F890,
	0x1F8AE,
	0x1F910,
	0x1F91F,
	0x1F920,
	0x1F928,
	0x1F930,
	0x1F931,
	0x1F933,
	0x1F93F,
	0x1F940,
	0x1F94C,
	0x1F950,
	0x1F95F,
	0x1F980,
	0x1F992,
	0x1F9C0,
	0x1F9C1,
	0x20000,
	0x2A6D7,
	0x2A700,
	0x2B735,
	0x2B740,
	0x2B81E,
	0x2B820,
	0x2CEA2,
	0x2F800,
	0x2FA1E,
	0xE0001,
	0xE0002,
	0xE0020,
	0xE0080,
	0xE0100,
	0xE01F0,
	0xF0000,
	0xFFFFE,
	0x100000,
	0x10FFFE
};

static const UV Cased_invlist[] = { /* for EBCDIC 037 */
	293,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x42,
	0x4A,
	0x51,
	0x5A,
	0x62,
	0x6A,
	0x70,
	0x79,
	0x80,
	0x8A,
	0x8C,
	0x8F,
	0x91,
	0x9D,
	0x9E,
	0x9F,
	0xA0,
	0xA1,
	0xA2,
	0xAA,
	0xAC,
	0xAF,
	0xC1,
	0xCA,
	0xCB,
	0xD0,
	0xD1,
	0xDA,
	0xDB,
	0xE0,
	0xE2,
	0xEA,
	0xEB,
	0xF0,
	0xFB,
	0xFF,
	0x100,
	0x1BB,
	0x1BC,
	0x1C0,
	0x1C4,
	0x294,
	0x295,
	0x2B9,
	0x2C0,
	0x2C2,
	0x2E0,
	0x2E5,
	0x345,
	0x346,
	0x370,
	0x374,
	0x376,
	0x378,
	0x37A,
	0x37E,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x3F6,
	0x3F7,
	0x482,
	0x48A,
	0x530,
	0x531,
	0x557,
	0x561,
	0x588,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1C80,
	0x1C89,
	0x1D00,
	0x1DC0,
	0x1E00,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FBD,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FCD,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FE0,
	0x1FED,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFD,
	0x2071,
	0x2072,
	0x207F,
	0x2080,
	0x2090,
	0x209D,
	0x2102,
	0x2103,
	0x2107,
	0x2108,
	0x210A,
	0x2114,
	0x2115,
	0x2116,
	0x2119,
	0x211E,
	0x2124,
	0x2125,
	0x2126,
	0x2127,
	0x2128,
	0x2129,
	0x212A,
	0x212E,
	0x212F,
	0x2135,
	0x2139,
	0x213A,
	0x213C,
	0x2140,
	0x2145,
	0x214A,
	0x214E,
	0x214F,
	0x2160,
	0x2180,
	0x2183,
	0x2185,
	0x24B6,
	0x24EA,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CE5,
	0x2CEB,
	0x2CEF,
	0x2CF2,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0xA640,
	0xA66E,
	0xA680,
	0xA69E,
	0xA722,
	0xA788,
	0xA78B,
	0xA78F,
	0xA790,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F8,
	0xA7FB,
	0xAB30,
	0xAB5B,
	0xAB5C,
	0xAB66,
	0xAB70,
	0xABC0,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFF21,
	0xFF3B,
	0xFF41,
	0xFF5B,
	0x10400,
	0x10450,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x118A0,
	0x118E0,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D6C1,
	0x1D6C2,
	0x1D6DB,
	0x1D6DC,
	0x1D6FB,
	0x1D6FC,
	0x1D715,
	0x1D716,
	0x1D735,
	0x1D736,
	0x1D74F,
	0x1D750,
	0x1D76F,
	0x1D770,
	0x1D789,
	0x1D78A,
	0x1D7A9,
	0x1D7AA,
	0x1D7C3,
	0x1D7C4,
	0x1D7CC,
	0x1E900,
	0x1E944,
	0x1F130,
	0x1F14A,
	0x1F150,
	0x1F16A,
	0x1F170,
	0x1F18A
};

#endif	/* defined(PERL_IN_PERL_C) */

#if defined(PERL_IN_REGCOMP_C)

static const UV NonL1_Perl_Non_Final_Folds_invlist[] = { /* for EBCDIC 037 */
	45,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x2BC,
	0x2BD,
	0x308,
	0x309,
	0x313,
	0x314,
	0x342,
	0x343,
	0x3AC,
	0x3AD,
	0x3AE,
	0x3AF,
	0x3B1,
	0x3B2,
	0x3B7,
	0x3B8,
	0x3B9,
	0x3BA,
	0x3C1,
	0x3C2,
	0x3C5,
	0x3C6,
	0x3C9,
	0x3CA,
	0x3CE,
	0x3CF,
	0x565,
	0x566,
	0x574,
	0x575,
	0x57E,
	0x57F,
	0x1F00,
	0x1F08,
	0x1F20,
	0x1F28,
	0x1F60,
	0x1F68,
	0x1F70,
	0x1F71,
	0x1F74,
	0x1F75,
	0x1F7C,
	0x1F7D
};

static const UV _Perl_Any_Folds_invlist[] = { /* for EBCDIC 037 */
	281,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x42,
	0x4A,
	0x51,
	0x5A,
	0x62,
	0x6A,
	0x70,
	0x79,
	0x80,
	0x8A,
	0x8C,
	0x8F,
	0x91,
	0x9A,
	0x9C,
	0x9D,
	0x9E,
	0x9F,
	0xA0,
	0xA1,
	0xA2,
	0xAA,
	0xAC,
	0xAF,
	0xC1,
	0xCA,
	0xCB,
	0xD0,
	0xD1,
	0xDA,
	0xDB,
	0xE0,
	0xE2,
	0xEA,
	0xEB,
	0xF0,
	0xFB,
	0xFF,
	0x100,
	0x131,
	0x132,
	0x138,
	0x139,
	0x18D,
	0x18E,
	0x19B,
	0x19C,
	0x1AA,
	0x1AC,
	0x1BA,
	0x1BC,
	0x1BE,
	0x1BF,
	0x1C0,
	0x1C4,
	0x221,
	0x222,
	0x234,
	0x23A,
	0x255,
	0x256,
	0x258,
	0x259,
	0x25A,
	0x25B,
	0x25D,
	0x260,
	0x262,
	0x263,
	0x264,
	0x265,
	0x267,
	0x268,
	0x26D,
	0x26F,
	0x270,
	0x271,
	0x273,
	0x275,
	0x276,
	0x27D,
	0x27E,
	0x280,
	0x281,
	0x283,
	0x284,
	0x287,
	0x28D,
	0x292,
	0x293,
	0x29D,
	0x29F,
	0x2BC,
	0x2BD,
	0x2BE,
	0x2BF,
	0x300,
	0x302,
	0x307,
	0x309,
	0x30A,
	0x30B,
	0x30C,
	0x30D,
	0x313,
	0x314,
	0x331,
	0x332,
	0x342,
	0x343,
	0x345,
	0x346,
	0x370,
	0x374,
	0x376,
	0x378,
	0x37B,
	0x37E,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x3D2,
	0x3D5,
	0x3F6,
	0x3F7,
	0x3FC,
	0x3FD,
	0x482,
	0x48A,
	0x530,
	0x531,
	0x557,
	0x561,
	0x588,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1C80,
	0x1C89,
	0x1D79,
	0x1D7A,
	0x1D7D,
	0x1D7E,
	0x1E00,
	0x1E9C,
	0x1E9E,
	0x1E9F,
	0x1EA0,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FBD,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FCD,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FE0,
	0x1FED,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFD,
	0x2126,
	0x2127,
	0x212A,
	0x212C,
	0x2132,
	0x2133,
	0x214E,
	0x214F,
	0x2160,
	0x2180,
	0x2183,
	0x2185,
	0x24B6,
	0x24EA,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2C71,
	0x2C72,
	0x2C74,
	0x2C75,
	0x2C77,
	0x2C7E,
	0x2CE4,
	0x2CEB,
	0x2CEF,
	0x2CF2,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0xA640,
	0xA66E,
	0xA680,
	0xA69C,
	0xA722,
	0xA730,
	0xA732,
	0xA770,
	0xA779,
	0xA788,
	0xA78B,
	0xA78E,
	0xA790,
	0xA794,
	0xA796,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xAB53,
	0xAB54,
	0xAB70,
	0xABC0,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFF21,
	0xFF3B,
	0xFF41,
	0xFF5B,
	0x10400,
	0x10450,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x118A0,
	0x118E0,
	0x1E900,
	0x1E944
};

static const UV _Perl_Folds_To_Multi_Char_invlist[] = { /* for EBCDIC 037 */
	59,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x59,
	0x5A,
	0x130,
	0x131,
	0x149,
	0x14A,
	0x1F0,
	0x1F1,
	0x390,
	0x391,
	0x3B0,
	0x3B1,
	0x587,
	0x588,
	0x1E96,
	0x1E9B,
	0x1E9E,
	0x1E9F,
	0x1F50,
	0x1F51,
	0x1F52,
	0x1F53,
	0x1F54,
	0x1F55,
	0x1F56,
	0x1F57,
	0x1F80,
	0x1FB0,
	0x1FB2,
	0x1FB5,
	0x1FB6,
	0x1FB8,
	0x1FBC,
	0x1FBD,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FC8,
	0x1FCC,
	0x1FCD,
	0x1FD2,
	0x1FD4,
	0x1FD6,
	0x1FD8,
	0x1FE2,
	0x1FE5,
	0x1FE6,
	0x1FE8,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FF8,
	0x1FFC,
	0x1FFD,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18
};

#endif	/* defined(PERL_IN_REGCOMP_C) */

#if defined(PERL_IN_PERL_C)

static const UV _Perl_GCB_invlist[] = { /* for EBCDIC 037 */
	1615,	/* Number of elements */
	148565664, /* Version and data structure type */
	0,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0xD,
	0xE,
	0x25,
	0x26,
	0x40,
	0xCA,
	0xCB,
	0xFF,
	0x100,
	0x300,
	0x370,
	0x483,
	0x48A,
	0x591,
	0x5BE,
	0x5BF,
	0x5C0,
	0x5C1,
	0x5C3,
	0x5C4,
	0x5C6,
	0x5C7,
	0x5C8,
	0x600,
	0x606,
	0x610,
	0x61B,
	0x61C,
	0x61D,
	0x64B,
	0x660,
	0x670,
	0x671,
	0x6D6,
	0x6DD,
	0x6DE,
	0x6DF,
	0x6E5,
	0x6E7,
	0x6E9,
	0x6EA,
	0x6EE,
	0x70F,
	0x710,
	0x711,
	0x712,
	0x730,
	0x74B,
	0x7A6,
	0x7B1,
	0x7EB,
	0x7F4,
	0x816,
	0x81A,
	0x81B,
	0x824,
	0x825,
	0x828,
	0x829,
	0x82E,
	0x859,
	0x85C,
	0x8D4,
	0x8E2,
	0x8E3,
	0x903,
	0x904,
	0x93A,
	0x93B,
	0x93C,
	0x93D,
	0x93E,
	0x941,
	0x949,
	0x94D,
	0x94E,
	0x950,
	0x951,
	0x958,
	0x962,
	0x964,
	0x981,
	0x982,
	0x984,
	0x9BC,
	0x9BD,
	0x9BE,
	0x9BF,
	0x9C1,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CD,
	0x9CE,
	0x9D7,
	0x9D8,
	0x9E2,
	0x9E4,
	0xA01,
	0xA03,
	0xA04,
	0xA3C,
	0xA3D,
	0xA3E,
	0xA41,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4E,
	0xA51,
	0xA52,
	0xA70,
	0xA72,
	0xA75,
	0xA76,
	0xA81,
	0xA83,
	0xA84,
	0xABC,
	0xABD,
	0xABE,
	0xAC1,
	0xAC6,
	0xAC7,
	0xAC9,
	0xACA,
	0xACB,
	0xACD,
	0xACE,
	0xAE2,
	0xAE4,
	0xB01,
	0xB02,
	0xB04,
	0xB3C,
	0xB3D,
	0xB3E,
	0xB40,
	0xB41,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4D,
	0xB4E,
	0xB56,
	0xB58,
	0xB62,
	0xB64,
	0xB82,
	0xB83,
	0xBBE,
	0xBBF,
	0xBC0,
	0xBC1,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCD,
	0xBCE,
	0xBD7,
	0xBD8,
	0xC00,
	0xC01,
	0xC04,
	0xC3E,
	0xC41,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4E,
	0xC55,
	0xC57,
	0xC62,
	0xC64,
	0xC81,
	0xC82,
	0xC84,
	0xCBC,
	0xCBD,
	0xCBE,
	0xCBF,
	0xCC0,
	0xCC2,
	0xCC3,
	0xCC5,
	0xCC6,
	0xCC7,
	0xCC9,
	0xCCA,
	0xCCC,
	0xCCE,
	0xCD5,
	0xCD7,
	0xCE2,
	0xCE4,
	0xD01,
	0xD02,
	0xD04,
	0xD3E,
	0xD3F,
	0xD41,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD4D,
	0xD4E,
	0xD4F,
	0xD57,
	0xD58,
	0xD62,
	0xD64,
	0xD82,
	0xD84,
	0xDCA,
	0xDCB,
	0xDCF,
	0xDD0,
	0xDD2,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDDF,
	0xDE0,
	0xDF2,
	0xDF4,
	0xE31,
	0xE32,
	0xE33,
	0xE34,
	0xE3B,
	0xE47,
	0xE4F,
	0xEB1,
	0xEB2,
	0xEB3,
	0xEB4,
	0xEBA,
	0xEBB,
	0xEBD,
	0xEC8,
	0xECE,
	0xF18,
	0xF1A,
	0xF35,
	0xF36,
	0xF37,
	0xF38,
	0xF39,
	0xF3A,
	0xF3E,
	0xF40,
	0xF71,
	0xF7F,
	0xF80,
	0xF85,
	0xF86,
	0xF88,
	0xF8D,
	0xF98,
	0xF99,
	0xFBD,
	0xFC6,
	0xFC7,
	0x102D,
	0x1031,
	0x1032,
	0x1038,
	0x1039,
	0x103B,
	0x103D,
	0x103F,
	0x1056,
	0x1058,
	0x105A,
	0x105E,
	0x1061,
	0x1071,
	0x1075,
	0x1082,
	0x1083,
	0x1084,
	0x1085,
	0x1087,
	0x108D,
	0x108E,
	0x109D,
	0x109E,
	0x1100,
	0x1160,
	0x11A8,
	0x1200,
	0x135D,
	0x1360,
	0x1712,
	0x1715,
	0x1732,
	0x1735,
	0x1752,
	0x1754,
	0x1772,
	0x1774,
	0x17B4,
	0x17B6,
	0x17B7,
	0x17BE,
	0x17C6,
	0x17C7,
	0x17C9,
	0x17D4,
	0x17DD,
	0x17DE,
	0x180B,
	0x180E,
	0x180F,
	0x1885,
	0x1887,
	0x18A9,
	0x18AA,
	0x1920,
	0x1923,
	0x1927,
	0x1929,
	0x192C,
	0x1930,
	0x1932,
	0x1933,
	0x1939,
	0x193C,
	0x1A17,
	0x1A19,
	0x1A1B,
	0x1A1C,
	0x1A55,
	0x1A56,
	0x1A57,
	0x1A58,
	0x1A5F,
	0x1A60,
	0x1A61,
	0x1A62,
	0x1A63,
	0x1A65,
	0x1A6D,
	0x1A73,
	0x1A7D,
	0x1A7F,
	0x1A80,
	0x1AB0,
	0x1ABF,
	0x1B00,
	0x1B04,
	0x1B05,
	0x1B34,
	0x1B35,
	0x1B36,
	0x1B3B,
	0x1B3C,
	0x1B3D,
	0x1B42,
	0x1B43,
	0x1B45,
	0x1B6B,
	0x1B74,
	0x1B80,
	0x1B82,
	0x1B83,
	0x1BA1,
	0x1BA2,
	0x1BA6,
	0x1BA8,
	0x1BAA,
	0x1BAB,
	0x1BAE,
	0x1BE6,
	0x1BE7,
	0x1BE8,
	0x1BEA,
	0x1BED,
	0x1BEE,
	0x1BEF,
	0x1BF2,
	0x1BF4,
	0x1C24,
	0x1C2C,
	0x1C34,
	0x1C36,
	0x1C38,
	0x1CD0,
	0x1CD3,
	0x1CD4,
	0x1CE1,
	0x1CE2,
	0x1CE9,
	0x1CED,
	0x1CEE,
	0x1CF2,
	0x1CF4,
	0x1CF5,
	0x1CF8,
	0x1CFA,
	0x1DC0,
	0x1DF6,
	0x1DFB,
	0x1E00,
	0x200B,
	0x200C,
	0x200D,
	0x200E,
	0x2010,
	0x2028,
	0x202F,
	0x2060,
	0x2070,
	0x20D0,
	0x20F1,
	0x261D,
	0x261E,
	0x26F9,
	0x26FA,
	0x270A,
	0x270E,
	0x2764,
	0x2765,
	0x2CEF,
	0x2CF2,
	0x2D7F,
	0x2D80,
	0x2DE0,
	0x2E00,
	0x302A,
	0x3030,
	0x3099,
	0x309B,
	0xA66F,
	0xA673,
	0xA674,
	0xA67E,
	0xA69E,
	0xA6A0,
	0xA6F0,
	0xA6F2,
	0xA802,
	0xA803,
	0xA806,
	0xA807,
	0xA80B,
	0xA80C,
	0xA823,
	0xA825,
	0xA827,
	0xA828,
	0xA880,
	0xA882,
	0xA8B4,
	0xA8C4,
	0xA8C6,
	0xA8E0,
	0xA8F2,
	0xA926,
	0xA92E,
	0xA947,
	0xA952,
	0xA954,
	0xA960,
	0xA97D,
	0xA980,
	0xA983,
	0xA984,
	0xA9B3,
	0xA9B4,
	0xA9B6,
	0xA9BA,
	0xA9BC,
	0xA9BD,
	0xA9C1,
	0xA9E5,
	0xA9E6,
	0xAA29,
	0xAA2F,
	0xAA31,
	0xAA33,
	0xAA35,
	0xAA37,
	0xAA43,
	0xAA44,
	0xAA4C,
	0xAA4D,
	0xAA4E,
	0xAA7C,
	0xAA7D,
	0xAAB0,
	0xAAB1,
	0xAAB2,
	0xAAB5,
	0xAAB7,
	0xAAB9,
	0xAABE,
	0xAAC0,
	0xAAC1,
	0xAAC2,
	0xAAEB,
	0xAAEC,
	0xAAEE,
	0xAAF0,
	0xAAF5,
	0xAAF6,
	0xAAF7,
	0xABE3,
	0xABE5,
	0xABE6,
	0xABE8,
	0xABE9,
	0xABEB,
	0xABEC,
	0xABED,
	0xABEE,
	0xAC00,
	0xAC01,
	0xAC1C,
	0xAC1D,
	0xAC38,
	0xAC39,
	0xAC54,
	0xAC55,
	0xAC70,
	0xAC71,
	0xAC8C,
	0xAC8D,
	0xACA8,
	0xACA9,
	0xACC4,
	0xACC5,
	0xACE0,
	0xACE1,
	0xACFC,
	0xACFD,
	0xAD18,
	0xAD19,
	0xAD34,
	0xAD35,
	0xAD50,
	0xAD51,
	0xAD6C,
	0xAD6D,
	0xAD88,
	0xAD89,
	0xADA4,
	0xADA5,
	0xADC0,
	0xADC1,
	0xADDC,
	0xADDD,
	0xADF8,
	0xADF9,
	0xAE14,
	0xAE15,
	0xAE30,
	0xAE31,
	0xAE4C,
	0xAE4D,
	0xAE68,
	0xAE69,
	0xAE84,
	0xAE85,
	0xAEA0,
	0xAEA1,
	0xAEBC,
	0xAEBD,
	0xAED8,
	0xAED9,
	0xAEF4,
	0xAEF5,
	0xAF10,
	0xAF11,
	0xAF2C,
	0xAF2D,
	0xAF48,
	0xAF49,
	0xAF64,
	0xAF65,
	0xAF80,
	0xAF81,
	0xAF9C,
	0xAF9D,
	0xAFB8,
	0xAFB9,
	0xAFD4,
	0xAFD5,
	0xAFF0,
	0xAFF1,
	0xB00C,
	0xB00D,
	0xB028,
	0xB029,
	0xB044,
	0xB045,
	0xB060,
	0xB061,
	0xB07C,
	0xB07D,
	0xB098,
	0xB099,
	0xB0B4,
	0xB0B5,
	0xB0D0,
	0xB0D1,
	0xB0EC,
	0xB0ED,
	0xB108,
	0xB109,
	0xB124,
	0xB125,
	0xB140,
	0xB141,
	0xB15C,
	0xB15D,
	0xB178,
	0xB179,
	0xB194,
	0xB195,
	0xB1B0,
	0xB1B1,
	0xB1CC,
	0xB1CD,
	0xB1E8,
	0xB1E9,
	0xB204,
	0xB205,
	0xB220,
	0xB221,
	0xB23C,
	0xB23D,
	0xB258,
	0xB259,
	0xB274,
	0xB275,
	0xB290,
	0xB291,
	0xB2AC,
	0xB2AD,
	0xB2C8,
	0xB2C9,
	0xB2E4,
	0xB2E5,
	0xB300,
	0xB301,
	0xB31C,
	0xB31D,
	0xB338,
	0xB339,
	0xB354,
	0xB355,
	0xB370,
	0xB371,
	0xB38C,
	0xB38D,
	0xB3A8,
	0xB3A9,
	0xB3C4,
	0xB3C5,
	0xB3E0,
	0xB3E1,
	0xB3FC,
	0xB3FD,
	0xB418,
	0xB419,
	0xB434,
	0xB435,
	0xB450,
	0xB451,
	0xB46C,
	0xB46D,
	0xB488,
	0xB489,
	0xB4A4,
	0xB4A5,
	0xB4C0,
	0xB4C1,
	0xB4DC,
	0xB4DD,
	0xB4F8,
	0xB4F9,
	0xB514,
	0xB515,
	0xB530,
	0xB531,
	0xB54C,
	0xB54D,
	0xB568,
	0xB569,
	0xB584,
	0xB585,
	0xB5A0,
	0xB5A1,
	0xB5BC,
	0xB5BD,
	0xB5D8,
	0xB5D9,
	0xB5F4,
	0xB5F5,
	0xB610,
	0xB611,
	0xB62C,
	0xB62D,
	0xB648,
	0xB649,
	0xB664,
	0xB665,
	0xB680,
	0xB681,
	0xB69C,
	0xB69D,
	0xB6B8,
	0xB6B9,
	0xB6D4,
	0xB6D5,
	0xB6F0,
	0xB6F1,
	0xB70C,
	0xB70D,
	0xB728,
	0xB729,
	0xB744,
	0xB745,
	0xB760,
	0xB761,
	0xB77C,
	0xB77D,
	0xB798,
	0xB799,
	0xB7B4,
	0xB7B5,
	0xB7D0,
	0xB7D1,
	0xB7EC,
	0xB7ED,
	0xB808,
	0xB809,
	0xB824,
	0xB825,
	0xB840,
	0xB841,
	0xB85C,
	0xB85D,
	0xB878,
	0xB879,
	0xB894,
	0xB895,
	0xB8B0,
	0xB8B1,
	0xB8CC,
	0xB8CD,
	0xB8E8,
	0xB8E9,
	0xB904,
	0xB905,
	0xB920,
	0xB921,
	0xB93C,
	0xB93D,
	0xB958,
	0xB959,
	0xB974,
	0xB975,
	0xB990,
	0xB991,
	0xB9AC,
	0xB9AD,
	0xB9C8,
	0xB9C9,
	0xB9E4,
	0xB9E5,
	0xBA00,
	0xBA01,
	0xBA1C,
	0xBA1D,
	0xBA38,
	0xBA39,
	0xBA54,
	0xBA55,
	0xBA70,
	0xBA71,
	0xBA8C,
	0xBA8D,
	0xBAA8,
	0xBAA9,
	0xBAC4,
	0xBAC5,
	0xBAE0,
	0xBAE1,
	0xBAFC,
	0xBAFD,
	0xBB18,
	0xBB19,
	0xBB34,
	0xBB35,
	0xBB50,
	0xBB51,
	0xBB6C,
	0xBB6D,
	0xBB88,
	0xBB89,
	0xBBA4,
	0xBBA5,
	0xBBC0,
	0xBBC1,
	0xBBDC,
	0xBBDD,
	0xBBF8,
	0xBBF9,
	0xBC14,
	0xBC15,
	0xBC30,
	0xBC31,
	0xBC4C,
	0xBC4D,
	0xBC68,
	0xBC69,
	0xBC84,
	0xBC85,
	0xBCA0,
	0xBCA1,
	0xBCBC,
	0xBCBD,
	0xBCD8,
	0xBCD9,
	0xBCF4,
	0xBCF5,
	0xBD10,
	0xBD11,
	0xBD2C,
	0xBD2D,
	0xBD48,
	0xBD49,
	0xBD64,
	0xBD65,
	0xBD80,
	0xBD81,
	0xBD9C,
	0xBD9D,
	0xBDB8,
	0xBDB9,
	0xBDD4,
	0xBDD5,
	0xBDF0,
	0xBDF1,
	0xBE0C,
	0xBE0D,
	0xBE28,
	0xBE29,
	0xBE44,
	0xBE45,
	0xBE60,
	0xBE61,
	0xBE7C,
	0xBE7D,
	0xBE98,
	0xBE99,
	0xBEB4,
	0xBEB5,
	0xBED0,
	0xBED1,
	0xBEEC,
	0xBEED,
	0xBF08,
	0xBF09,
	0xBF24,
	0xBF25,
	0xBF40,
	0xBF41,
	0xBF5C,
	0xBF5D,
	0xBF78,
	0xBF79,
	0xBF94,
	0xBF95,
	0xBFB0,
	0xBFB1,
	0xBFCC,
	0xBFCD,
	0xBFE8,
	0xBFE9,
	0xC004,
	0xC005,
	0xC020,
	0xC021,
	0xC03C,
	0xC03D,
	0xC058,
	0xC059,
	0xC074,
	0xC075,
	0xC090,
	0xC091,
	0xC0AC,
	0xC0AD,
	0xC0C8,
	0xC0C9,
	0xC0E4,
	0xC0E5,
	0xC100,
	0xC101,
	0xC11C,
	0xC11D,
	0xC138,
	0xC139,
	0xC154,
	0xC155,
	0xC170,
	0xC171,
	0xC18C,
	0xC18D,
	0xC1A8,
	0xC1A9,
	0xC1C4,
	0xC1C5,
	0xC1E0,
	0xC1E1,
	0xC1FC,
	0xC1FD,
	0xC218,
	0xC219,
	0xC234,
	0xC235,
	0xC250,
	0xC251,
	0xC26C,
	0xC26D,
	0xC288,
	0xC289,
	0xC2A4,
	0xC2A5,
	0xC2C0,
	0xC2C1,
	0xC2DC,
	0xC2DD,
	0xC2F8,
	0xC2F9,
	0xC314,
	0xC315,
	0xC330,
	0xC331,
	0xC34C,
	0xC34D,
	0xC368,
	0xC369,
	0xC384,
	0xC385,
	0xC3A0,
	0xC3A1,
	0xC3BC,
	0xC3BD,
	0xC3D8,
	0xC3D9,
	0xC3F4,
	0xC3F5,
	0xC410,
	0xC411,
	0xC42C,
	0xC42D,
	0xC448,
	0xC449,
	0xC464,
	0xC465,
	0xC480,
	0xC481,
	0xC49C,
	0xC49D,
	0xC4B8,
	0xC4B9,
	0xC4D4,
	0xC4D5,
	0xC4F0,
	0xC4F1,
	0xC50C,
	0xC50D,
	0xC528,
	0xC529,
	0xC544,
	0xC545,
	0xC560,
	0xC561,
	0xC57C,
	0xC57D,
	0xC598,
	0xC599,
	0xC5B4,
	0xC5B5,
	0xC5D0,
	0xC5D1,
	0xC5EC,
	0xC5ED,
	0xC608,
	0xC609,
	0xC624,
	0xC625,
	0xC640,
	0xC641,
	0xC65C,
	0xC65D,
	0xC678,
	0xC679,
	0xC694,
	0xC695,
	0xC6B0,
	0xC6B1,
	0xC6CC,
	0xC6CD,
	0xC6E8,
	0xC6E9,
	0xC704,
	0xC705,
	0xC720,
	0xC721,
	0xC73C,
	0xC73D,
	0xC758,
	0xC759,
	0xC774,
	0xC775,
	0xC790,
	0xC791,
	0xC7AC,
	0xC7AD,
	0xC7C8,
	0xC7C9,
	0xC7E4,
	0xC7E5,
	0xC800,
	0xC801,
	0xC81C,
	0xC81D,
	0xC838,
	0xC839,
	0xC854,
	0xC855,
	0xC870,
	0xC871,
	0xC88C,
	0xC88D,
	0xC8A8,
	0xC8A9,
	0xC8C4,
	0xC8C5,
	0xC8E0,
	0xC8E1,
	0xC8FC,
	0xC8FD,
	0xC918,
	0xC919,
	0xC934,
	0xC935,
	0xC950,
	0xC951,
	0xC96C,
	0xC96D,
	0xC988,
	0xC989,
	0xC9A4,
	0xC9A5,
	0xC9C0,
	0xC9C1,
	0xC9DC,
	0xC9DD,
	0xC9F8,
	0xC9F9,
	0xCA14,
	0xCA15,
	0xCA30,
	0xCA31,
	0xCA4C,
	0xCA4D,
	0xCA68,
	0xCA69,
	0xCA84,
	0xCA85,
	0xCAA0,
	0xCAA1,
	0xCABC,
	0xCABD,
	0xCAD8,
	0xCAD9,
	0xCAF4,
	0xCAF5,
	0xCB10,
	0xCB11,
	0xCB2C,
	0xCB2D,
	0xCB48,
	0xCB49,
	0xCB64,
	0xCB65,
	0xCB80,
	0xCB81,
	0xCB9C,
	0xCB9D,
	0xCBB8,
	0xCBB9,
	0xCBD4,
	0xCBD5,
	0xCBF0,
	0xCBF1,
	0xCC0C,
	0xCC0D,
	0xCC28,
	0xCC29,
	0xCC44,
	0xCC45,
	0xCC60,
	0xCC61,
	0xCC7C,
	0xCC7D,
	0xCC98,
	0xCC99,
	0xCCB4,
	0xCCB5,
	0xCCD0,
	0xCCD1,
	0xCCEC,
	0xCCED,
	0xCD08,
	0xCD09,
	0xCD24,
	0xCD25,
	0xCD40,
	0xCD41,
	0xCD5C,
	0xCD5D,
	0xCD78,
	0xCD79,
	0xCD94,
	0xCD95,
	0xCDB0,
	0xCDB1,
	0xCDCC,
	0xCDCD,
	0xCDE8,
	0xCDE9,
	0xCE04,
	0xCE05,
	0xCE20,
	0xCE21,
	0xCE3C,
	0xCE3D,
	0xCE58,
	0xCE59,
	0xCE74,
	0xCE75,
	0xCE90,
	0xCE91,
	0xCEAC,
	0xCEAD,
	0xCEC8,
	0xCEC9,
	0xCEE4,
	0xCEE5,
	0xCF00,
	0xCF01,
	0xCF1C,
	0xCF1D,
	0xCF38,
	0xCF39,
	0xCF54,
	0xCF55,
	0xCF70,
	0xCF71,
	0xCF8C,
	0xCF8D,
	0xCFA8,
	0xCFA9,
	0xCFC4,
	0xCFC5,
	0xCFE0,
	0xCFE1,
	0xCFFC,
	0xCFFD,
	0xD018,
	0xD019,
	0xD034,
	0xD035,
	0xD050,
	0xD051,
	0xD06C,
	0xD06D,
	0xD088,
	0xD089,
	0xD0A4,
	0xD0A5,
	0xD0C0,
	0xD0C1,
	0xD0DC,
	0xD0DD,
	0xD0F8,
	0xD0F9,
	0xD114,
	0xD115,
	0xD130,
	0xD131,
	0xD14C,
	0xD14D,
	0xD168,
	0xD169,
	0xD184,
	0xD185,
	0xD1A0,
	0xD1A1,
	0xD1BC,
	0xD1BD,
	0xD1D8,
	0xD1D9,
	0xD1F4,
	0xD1F5,
	0xD210,
	0xD211,
	0xD22C,
	0xD22D,
	0xD248,
	0xD249,
	0xD264,
	0xD265,
	0xD280,
	0xD281,
	0xD29C,
	0xD29D,
	0xD2B8,
	0xD2B9,
	0xD2D4,
	0xD2D5,
	0xD2F0,
	0xD2F1,
	0xD30C,
	0xD30D,
	0xD328,
	0xD329,
	0xD344,
	0xD345,
	0xD360,
	0xD361,
	0xD37C,
	0xD37D,
	0xD398,
	0xD399,
	0xD3B4,
	0xD3B5,
	0xD3D0,
	0xD3D1,
	0xD3EC,
	0xD3ED,
	0xD408,
	0xD409,
	0xD424,
	0xD425,
	0xD440,
	0xD441,
	0xD45C,
	0xD45D,
	0xD478,
	0xD479,
	0xD494,
	0xD495,
	0xD4B0,
	0xD4B1,
	0xD4CC,
	0xD4CD,
	0xD4E8,
	0xD4E9,
	0xD504,
	0xD505,
	0xD520,
	0xD521,
	0xD53C,
	0xD53D,
	0xD558,
	0xD559,
	0xD574,
	0xD575,
	0xD590,
	0xD591,
	0xD5AC,
	0xD5AD,
	0xD5C8,
	0xD5C9,
	0xD5E4,
	0xD5E5,
	0xD600,
	0xD601,
	0xD61C,
	0xD61D,
	0xD638,
	0xD639,
	0xD654,
	0xD655,
	0xD670,
	0xD671,
	0xD68C,
	0xD68D,
	0xD6A8,
	0xD6A9,
	0xD6C4,
	0xD6C5,
	0xD6E0,
	0xD6E1,
	0xD6FC,
	0xD6FD,
	0xD718,
	0xD719,
	0xD734,
	0xD735,
	0xD750,
	0xD751,
	0xD76C,
	0xD76D,
	0xD788,
	0xD789,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xD800,
	0xE000,
	0xFB1E,
	0xFB1F,
	0xFE00,
	0xFE10,
	0xFE20,
	0xFE30,
	0xFEFF,
	0xFF00,
	0xFF9E,
	0xFFA0,
	0xFFF0,
	0xFFFC,
	0x101FD,
	0x101FE,
	0x102E0,
	0x102E1,
	0x10376,
	0x1037B,
	0x10A01,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A10,
	0x10A38,
	0x10A3B,
	0x10A3F,
	0x10A40,
	0x10AE5,
	0x10AE7,
	0x11000,
	0x11001,
	0x11002,
	0x11003,
	0x11038,
	0x11047,
	0x1107F,
	0x11082,
	0x11083,
	0x110B0,
	0x110B3,
	0x110B7,
	0x110B9,
	0x110BB,
	0x110BD,
	0x110BE,
	0x11100,
	0x11103,
	0x11127,
	0x1112C,
	0x1112D,
	0x11135,
	0x11173,
	0x11174,
	0x11180,
	0x11182,
	0x11183,
	0x111B3,
	0x111B6,
	0x111BF,
	0x111C1,
	0x111C2,
	0x111C4,
	0x111CA,
	0x111CD,
	0x1122C,
	0x1122F,
	0x11232,
	0x11234,
	0x11235,
	0x11236,
	0x11238,
	0x1123E,
	0x1123F,
	0x112DF,
	0x112E0,
	0x112E3,
	0x112EB,
	0x11300,
	0x11302,
	0x11304,
	0x1133C,
	0x1133D,
	0x1133E,
	0x1133F,
	0x11340,
	0x11341,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134E,
	0x11357,
	0x11358,
	0x11362,
	0x11364,
	0x11366,
	0x1136D,
	0x11370,
	0x11375,
	0x11435,
	0x11438,
	0x11440,
	0x11442,
	0x11445,
	0x11446,
	0x11447,
	0x114B0,
	0x114B1,
	0x114B3,
	0x114B9,
	0x114BA,
	0x114BB,
	0x114BD,
	0x114BE,
	0x114BF,
	0x114C1,
	0x114C2,
	0x114C4,
	0x115AF,
	0x115B0,
	0x115B2,
	0x115B6,
	0x115B8,
	0x115BC,
	0x115BE,
	0x115BF,
	0x115C1,
	0x115DC,
	0x115DE,
	0x11630,
	0x11633,
	0x1163B,
	0x1163D,
	0x1163E,
	0x1163F,
	0x11641,
	0x116AB,
	0x116AC,
	0x116AD,
	0x116AE,
	0x116B0,
	0x116B6,
	0x116B7,
	0x116B8,
	0x1171D,
	0x11720,
	0x11722,
	0x11726,
	0x11727,
	0x1172C,
	0x11C2F,
	0x11C30,
	0x11C37,
	0x11C38,
	0x11C3E,
	0x11C3F,
	0x11C40,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CAA,
	0x11CB1,
	0x11CB2,
	0x11CB4,
	0x11CB5,
	0x11CB7,
	0x16AF0,
	0x16AF5,
	0x16B30,
	0x16B37,
	0x16F51,
	0x16F7F,
	0x16F8F,
	0x16F93,
	0x1BC9D,
	0x1BC9F,
	0x1BCA0,
	0x1BCA4,
	0x1D165,
	0x1D166,
	0x1D167,
	0x1D16A,
	0x1D16D,
	0x1D16E,
	0x1D173,
	0x1D17B,
	0x1D183,
	0x1D185,
	0x1D18C,
	0x1D1AA,
	0x1D1AE,
	0x1D242,
	0x1D245,
	0x1DA00,
	0x1DA37,
	0x1DA3B,
	0x1DA6D,
	0x1DA75,
	0x1DA76,
	0x1DA84,
	0x1DA85,
	0x1DA9B,
	0x1DAA0,
	0x1DAA1,
	0x1DAB0,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E8D0,
	0x1E8D7,
	0x1E944,
	0x1E94B,
	0x1F1E6,
	0x1F200,
	0x1F385,
	0x1F386,
	0x1F3C3,
	0x1F3C5,
	0x1F3CA,
	0x1F3CC,
	0x1F3FB,
	0x1F400,
	0x1F442,
	0x1F444,
	0x1F446,
	0x1F451,
	0x1F466,
	0x1F46A,
	0x1F46E,
	0x1F46F,
	0x1F470,
	0x1F479,
	0x1F47C,
	0x1F47D,
	0x1F481,
	0x1F484,
	0x1F485,
	0x1F488,
	0x1F48B,
	0x1F48C,
	0x1F4AA,
	0x1F4AB,
	0x1F575,
	0x1F576,
	0x1F57A,
	0x1F57B,
	0x1F590,
	0x1F591,
	0x1F595,
	0x1F597,
	0x1F5E8,
	0x1F5E9,
	0x1F645,
	0x1F648,
	0x1F64B,
	0x1F650,
	0x1F6A3,
	0x1F6A4,
	0x1F6B4,
	0x1F6B7,
	0x1F6C0,
	0x1F6C1,
	0x1F918,
	0x1F91F,
	0x1F926,
	0x1F927,
	0x1F930,
	0x1F931,
	0x1F933,
	0x1F93A,
	0x1F93C,
	0x1F93F,
	0xE0000,
	0xE0020,
	0xE0080,
	0xE0100,
	0xE01F0,
	0xE1000
};

#endif	/* defined(PERL_IN_PERL_C) */

#if defined(PERL_IN_REGEXEC_C)

#define GCB_ENUM_COUNT 19

typedef enum {
	GCB_Other = 0,
	GCB_CR = 1,
	GCB_Control = 2,
	GCB_E_Base = 3,
	GCB_E_Base_GAZ = 4,
	GCB_E_Modifier = 5,
	GCB_Extend = 6,
	GCB_Glue_After_Zwj = 7,
	GCB_L = 8,
	GCB_LF = 9,
	GCB_LV = 10,
	GCB_LVT = 11,
	GCB_Prepend = 12,
	GCB_Regional_Indicator = 13,
	GCB_SpacingMark = 14,
	GCB_T = 15,
	GCB_V = 16,
	GCB_ZWJ = 17,
	GCB_EDGE = 18
} GCB_enum;

static const GCB_enum _Perl_GCB_invmap[] = { /* for EBCDIC 037 */
	GCB_Control,
	GCB_CR,
	GCB_Control,
	GCB_LF,
	GCB_Control,
	GCB_Other,
	GCB_Control,
	GCB_Other,
	GCB_Control,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Prepend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Control,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Prepend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Prepend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Prepend,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Prepend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_L,
	GCB_V,
	GCB_T,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Control,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Control,
	GCB_Extend,
	GCB_ZWJ,
	GCB_Control,
	GCB_Other,
	GCB_Control,
	GCB_Other,
	GCB_Control,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_Glue_After_Zwj,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_L,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_LV,
	GCB_LVT,
	GCB_Other,
	GCB_V,
	GCB_Other,
	GCB_T,
	GCB_Other,
	GCB_Control,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Control,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Control,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Prepend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Prepend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Control,
	GCB_Other,
	GCB_Extend,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Other,
	GCB_SpacingMark,
	GCB_Extend,
	GCB_Control,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Extend,
	GCB_Other,
	GCB_Regional_Indicator,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Modifier,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base_GAZ,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_Glue_After_Zwj,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_Glue_After_Zwj,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_E_Base,
	GCB_Other,
	GCB_Control,
	GCB_Extend,
	GCB_Control,
	GCB_Extend,
	GCB_Control,
	GCB_Other
};

#endif	/* defined(PERL_IN_REGEXEC_C) */

#if defined(PERL_IN_UTF8_C)

static const UV _Perl_IDCont_invlist[] = { /* for EBCDIC 037 */
	1387,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x42,
	0x4A,
	0x51,
	0x5A,
	0x62,
	0x6A,
	0x6D,
	0x6E,
	0x70,
	0x79,
	0x80,
	0x8A,
	0x8C,
	0x8F,
	0x91,
	0x9D,
	0x9E,
	0x9F,
	0xA0,
	0xA1,
	0xA2,
	0xAA,
	0xAC,
	0xAF,
	0xC1,
	0xCA,
	0xCB,
	0xD0,
	0xD1,
	0xDA,
	0xDB,
	0xE0,
	0xE2,
	0xEA,
	0xEB,
	0xFA,
	0xFB,
	0xFF,
	0x100,
	0x2C2,
	0x2C6,
	0x2D2,
	0x2E0,
	0x2E5,
	0x2EC,
	0x2ED,
	0x2EE,
	0x2EF,
	0x300,
	0x375,
	0x376,
	0x378,
	0x37B,
	0x37E,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x3F6,
	0x3F7,
	0x482,
	0x483,
	0x488,
	0x48A,
	0x530,
	0x531,
	0x557,
	0x559,
	0x55A,
	0x561,
	0x588,
	0x591,
	0x5BE,
	0x5BF,
	0x5C0,
	0x5C1,
	0x5C3,
	0x5C4,
	0x5C6,
	0x5C7,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F3,
	0x610,
	0x61B,
	0x620,
	0x66A,
	0x66E,
	0x6D4,
	0x6D5,
	0x6DD,
	0x6DF,
	0x6E9,
	0x6EA,
	0x6FD,
	0x6FF,
	0x700,
	0x710,
	0x74B,
	0x74D,
	0x7B2,
	0x7C0,
	0x7F6,
	0x7FA,
	0x7FB,
	0x800,
	0x82E,
	0x840,
	0x85C,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x8D4,
	0x8E2,
	0x8E3,
	0x964,
	0x966,
	0x970,
	0x971,
	0x984,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BC,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CF,
	0x9D7,
	0x9D8,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E4,
	0x9E6,
	0x9F2,
	0xA01,
	0xA04,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA3C,
	0xA3D,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4E,
	0xA51,
	0xA52,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA66,
	0xA76,
	0xA81,
	0xA84,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABC,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACE,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE4,
	0xAE6,
	0xAF0,
	0xAF9,
	0xAFA,
	0xB01,
	0xB04,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3C,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4E,
	0xB56,
	0xB58,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB64,
	0xB66,
	0xB70,
	0xB71,
	0xB72,
	0xB82,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCE,
	0xBD0,
	0xBD1,
	0xBD7,
	0xBD8,
	0xBE6,
	0xBF0,
	0xC00,
	0xC04,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4E,
	0xC55,
	0xC57,
	0xC58,
	0xC5B,
	0xC60,
	0xC64,
	0xC66,
	0xC70,
	0xC80,
	0xC84,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBC,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCE,
	0xCD5,
	0xCD7,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE4,
	0xCE6,
	0xCF0,
	0xCF1,
	0xCF3,
	0xD01,
	0xD04,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD4F,
	0xD54,
	0xD58,
	0xD5F,
	0xD64,
	0xD66,
	0xD70,
	0xD7A,
	0xD80,
	0xD82,
	0xD84,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xDCA,
	0xDCB,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDE6,
	0xDF0,
	0xDF2,
	0xDF4,
	0xE01,
	0xE3B,
	0xE40,
	0xE4F,
	0xE50,
	0xE5A,
	0xE81,
	0xE83,
	0xE84,
	0xE85,
	0xE87,
	0xE89,
	0xE8A,
	0xE8B,
	0xE8D,
	0xE8E,
	0xE94,
	0xE98,
	0xE99,
	0xEA0,
	0xEA1,
	0xEA4,
	0xEA5,
	0xEA6,
	0xEA7,
	0xEA8,
	0xEAA,
	0xEAC,
	0xEAD,
	0xEBA,
	0xEBB,
	0xEBE,
	0xEC0,
	0xEC5,
	0xEC6,
	0xEC7,
	0xEC8,
	0xECE,
	0xED0,
	0xEDA,
	0xEDC,
	0xEE0,
	0xF00,
	0xF01,
	0xF18,
	0xF1A,
	0xF20,
	0xF2A,
	0xF35,
	0xF36,
	0xF37,
	0xF38,
	0xF39,
	0xF3A,
	0xF3E,
	0xF48,
	0xF49,
	0xF6D,
	0xF71,
	0xF85,
	0xF86,
	0xF98,
	0xF99,
	0xFBD,
	0xFC6,
	0xFC7,
	0x1000,
	0x104A,
	0x1050,
	0x109E,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x10FB,
	0x10FC,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x135D,
	0x1360,
	0x1380,
	0x1390,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1401,
	0x166D,
	0x166F,
	0x1680,
	0x1681,
	0x169B,
	0x16A0,
	0x16EB,
	0x16EE,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1715,
	0x1720,
	0x1735,
	0x1740,
	0x1754,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1772,
	0x1774,
	0x1780,
	0x17D4,
	0x17D7,
	0x17D8,
	0x17DC,
	0x17DE,
	0x17E0,
	0x17EA,
	0x180B,
	0x180E,
	0x1810,
	0x181A,
	0x1820,
	0x1878,
	0x1880,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1920,
	0x192C,
	0x1930,
	0x193C,
	0x1946,
	0x196E,
	0x1970,
	0x1975,
	0x1980,
	0x19AC,
	0x19B0,
	0x19CA,
	0x19D0,
	0x19DA,
	0x1A00,
	0x1A1C,
	0x1A20,
	0x1A5F,
	0x1A60,
	0x1A7D,
	0x1A7F,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1AA7,
	0x1AA8,
	0x1AB0,
	0x1ABE,
	0x1B00,
	0x1B4C,
	0x1B50,
	0x1B5A,
	0x1B6B,
	0x1B74,
	0x1B80,
	0x1BF4,
	0x1C00,
	0x1C38,
	0x1C40,
	0x1C4A,
	0x1C4D,
	0x1C7E,
	0x1C80,
	0x1C89,
	0x1CD0,
	0x1CD3,
	0x1CD4,
	0x1CF7,
	0x1CF8,
	0x1CFA,
	0x1D00,
	0x1DF6,
	0x1DFB,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FBD,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FCD,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FE0,
	0x1FED,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFD,
	0x203F,
	0x2041,
	0x2054,
	0x2055,
	0x2071,
	0x2072,
	0x207F,
	0x2080,
	0x2090,
	0x209D,
	0x20D0,
	0x20DD,
	0x20E1,
	0x20E2,
	0x20E5,
	0x20F1,
	0x2102,
	0x2103,
	0x2107,
	0x2108,
	0x210A,
	0x2114,
	0x2115,
	0x2116,
	0x2119,
	0x211E,
	0x2124,
	0x2125,
	0x2126,
	0x2127,
	0x2128,
	0x2129,
	0x212A,
	0x212E,
	0x212F,
	0x213A,
	0x213C,
	0x2140,
	0x2145,
	0x214A,
	0x214E,
	0x214F,
	0x2160,
	0x2189,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CE5,
	0x2CEB,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D70,
	0x2D7F,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x2DE0,
	0x2E00,
	0x3005,
	0x3008,
	0x3021,
	0x3030,
	0x3031,
	0x3036,
	0x3038,
	0x303D,
	0x3041,
	0x3097,
	0x3099,
	0x309B,
	0x309D,
	0x30A0,
	0x30A1,
	0x30FB,
	0x30FC,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x31A0,
	0x31BB,
	0x31F0,
	0x3200,
	0x3400,
	0x4DB6,
	0x4E00,
	0x9FD6,
	0xA000,
	0xA48D,
	0xA4D0,
	0xA4FE,
	0xA500,
	0xA60D,
	0xA610,
	0xA62C,
	0xA640,
	0xA670,
	0xA674,
	0xA67E,
	0xA67F,
	0xA6F2,
	0xA717,
	0xA720,
	0xA722,
	0xA789,
	0xA78B,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F7,
	0xA828,
	0xA840,
	0xA874,
	0xA880,
	0xA8C6,
	0xA8D0,
	0xA8DA,
	0xA8E0,
	0xA8F8,
	0xA8FB,
	0xA8FC,
	0xA8FD,
	0xA8FE,
	0xA900,
	0xA92E,
	0xA930,
	0xA954,
	0xA960,
	0xA97D,
	0xA980,
	0xA9C1,
	0xA9CF,
	0xA9DA,
	0xA9E0,
	0xA9FF,
	0xAA00,
	0xAA37,
	0xAA40,
	0xAA4E,
	0xAA50,
	0xAA5A,
	0xAA60,
	0xAA77,
	0xAA7A,
	0xAAC3,
	0xAADB,
	0xAADE,
	0xAAE0,
	0xAAF0,
	0xAAF2,
	0xAAF7,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB5B,
	0xAB5C,
	0xAB66,
	0xAB70,
	0xABEB,
	0xABEC,
	0xABEE,
	0xABF0,
	0xABFA,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xF900,
	0xFA6E,
	0xFA70,
	0xFADA,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB29,
	0xFB2A,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFBB2,
	0xFBD3,
	0xFC5E,
	0xFC64,
	0xFD3E,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFA,
	0xFE00,
	0xFE10,
	0xFE20,
	0xFE30,
	0xFE33,
	0xFE35,
	0xFE4D,
	0xFE50,
	0xFE71,
	0xFE72,
	0xFE73,
	0xFE74,
	0xFE77,
	0xFE78,
	0xFE79,
	0xFE7A,
	0xFE7B,
	0xFE7C,
	0xFE7D,
	0xFE7E,
	0xFE7F,
	0xFEFD,
	0xFF10,
	0xFF1A,
	0xFF21,
	0xFF3B,
	0xFF3F,
	0xFF40,
	0xFF41,
	0xFF5B,
	0xFF66,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10140,
	0x10175,
	0x101FD,
	0x101FE,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x102E0,
	0x102E1,
	0x10300,
	0x10320,
	0x10330,
	0x1034B,
	0x10350,
	0x1037B,
	0x10380,
	0x1039E,
	0x103A0,
	0x103C4,
	0x103C8,
	0x103D0,
	0x103D1,
	0x103D6,
	0x10400,
	0x1049E,
	0x104A0,
	0x104AA,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10860,
	0x10877,
	0x10880,
	0x1089F,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x10900,
	0x10916,
	0x10920,
	0x1093A,
	0x10980,
	0x109B8,
	0x109BE,
	0x109C0,
	0x10A00,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A38,
	0x10A3B,
	0x10A3F,
	0x10A40,
	0x10A60,
	0x10A7D,
	0x10A80,
	0x10A9D,
	0x10AC0,
	0x10AC8,
	0x10AC9,
	0x10AE7,
	0x10B00,
	0x10B36,
	0x10B40,
	0x10B56,
	0x10B60,
	0x10B73,
	0x10B80,
	0x10B92,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x11000,
	0x11047,
	0x11066,
	0x11070,
	0x1107F,
	0x110BB,
	0x110D0,
	0x110E9,
	0x110F0,
	0x110FA,
	0x11100,
	0x11135,
	0x11136,
	0x11140,
	0x11150,
	0x11174,
	0x11176,
	0x11177,
	0x11180,
	0x111C5,
	0x111CA,
	0x111CD,
	0x111D0,
	0x111DB,
	0x111DC,
	0x111DD,
	0x11200,
	0x11212,
	0x11213,
	0x11238,
	0x1123E,
	0x1123F,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112A9,
	0x112B0,
	0x112EB,
	0x112F0,
	0x112FA,
	0x11300,
	0x11304,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133C,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134E,
	0x11350,
	0x11351,
	0x11357,
	0x11358,
	0x1135D,
	0x11364,
	0x11366,
	0x1136D,
	0x11370,
	0x11375,
	0x11400,
	0x1144B,
	0x11450,
	0x1145A,
	0x11480,
	0x114C6,
	0x114C7,
	0x114C8,
	0x114D0,
	0x114DA,
	0x11580,
	0x115B6,
	0x115B8,
	0x115C1,
	0x115D8,
	0x115DE,
	0x11600,
	0x11641,
	0x11644,
	0x11645,
	0x11650,
	0x1165A,
	0x11680,
	0x116B8,
	0x116C0,
	0x116CA,
	0x11700,
	0x1171A,
	0x1171D,
	0x1172C,
	0x11730,
	0x1173A,
	0x118A0,
	0x118EA,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C37,
	0x11C38,
	0x11C41,
	0x11C50,
	0x11C5A,
	0x11C72,
	0x11C90,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16A60,
	0x16A6A,
	0x16AD0,
	0x16AEE,
	0x16AF0,
	0x16AF5,
	0x16B00,
	0x16B37,
	0x16B40,
	0x16B44,
	0x16B50,
	0x16B5A,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F7F,
	0x16F8F,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1BC9D,
	0x1BC9F,
	0x1D165,
	0x1D16A,
	0x1D16D,
	0x1D173,
	0x1D17B,
	0x1D183,
	0x1D185,
	0x1D18C,
	0x1D1AA,
	0x1D1AE,
	0x1D242,
	0x1D245,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D6C1,
	0x1D6C2,
	0x1D6DB,
	0x1D6DC,
	0x1D6FB,
	0x1D6FC,
	0x1D715,
	0x1D716,
	0x1D735,
	0x1D736,
	0x1D74F,
	0x1D750,
	0x1D76F,
	0x1D770,
	0x1D789,
	0x1D78A,
	0x1D7A9,
	0x1D7AA,
	0x1D7C3,
	0x1D7C4,
	0x1D7CC,
	0x1D7CE,
	0x1D800,
	0x1DA00,
	0x1DA37,
	0x1DA3B,
	0x1DA6D,
	0x1DA75,
	0x1DA76,
	0x1DA84,
	0x1DA85,
	0x1DA9B,
	0x1DAA0,
	0x1DAA1,
	0x1DAB0,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E800,
	0x1E8C5,
	0x1E8D0,
	0x1E8D7,
	0x1E900,
	0x1E94B,
	0x1E950,
	0x1E95A,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x20000,
	0x2A6D7,
	0x2A700,
	0x2B735,
	0x2B740,
	0x2B81E,
	0x2B820,
	0x2CEA2,
	0x2F800,
	0x2FA1E,
	0xE0100,
	0xE01F0
};

static const UV _Perl_IDStart_invlist[] = { /* for EBCDIC 037 */
	1183,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x42,
	0x4A,
	0x51,
	0x5A,
	0x62,
	0x6A,
	0x6D,
	0x6E,
	0x70,
	0x79,
	0x80,
	0x8A,
	0x8C,
	0x8F,
	0x91,
	0x9D,
	0x9E,
	0x9F,
	0xA0,
	0xA1,
	0xA2,
	0xAA,
	0xAC,
	0xAF,
	0xC1,
	0xCA,
	0xCB,
	0xD0,
	0xD1,
	0xDA,
	0xDB,
	0xE0,
	0xE2,
	0xEA,
	0xEB,
	0xF0,
	0xFB,
	0xFF,
	0x100,
	0x2C2,
	0x2C6,
	0x2D2,
	0x2E0,
	0x2E5,
	0x2EC,
	0x2ED,
	0x2EE,
	0x2EF,
	0x370,
	0x375,
	0x376,
	0x378,
	0x37B,
	0x37E,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x3F6,
	0x3F7,
	0x482,
	0x48A,
	0x530,
	0x531,
	0x557,
	0x559,
	0x55A,
	0x561,
	0x588,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F3,
	0x620,
	0x64B,
	0x66E,
	0x670,
	0x671,
	0x6D4,
	0x6D5,
	0x6D6,
	0x6E5,
	0x6E7,
	0x6EE,
	0x6F0,
	0x6FA,
	0x6FD,
	0x6FF,
	0x700,
	0x710,
	0x711,
	0x712,
	0x730,
	0x74D,
	0x7A6,
	0x7B1,
	0x7B2,
	0x7CA,
	0x7EB,
	0x7F4,
	0x7F6,
	0x7FA,
	0x7FB,
	0x800,
	0x816,
	0x81A,
	0x81B,
	0x824,
	0x825,
	0x828,
	0x829,
	0x840,
	0x859,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x904,
	0x93A,
	0x93D,
	0x93E,
	0x950,
	0x951,
	0x958,
	0x962,
	0x971,
	0x981,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BD,
	0x9BE,
	0x9CE,
	0x9CF,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E2,
	0x9F0,
	0x9F2,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA72,
	0xA75,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABD,
	0xABE,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE2,
	0xAF9,
	0xAFA,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3D,
	0xB3E,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB62,
	0xB71,
	0xB72,
	0xB83,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBD0,
	0xBD1,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC3E,
	0xC58,
	0xC5B,
	0xC60,
	0xC62,
	0xC80,
	0xC81,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBD,
	0xCBE,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE2,
	0xCF1,
	0xCF3,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD3E,
	0xD4E,
	0xD4F,
	0xD54,
	0xD57,
	0xD5F,
	0xD62,
	0xD7A,
	0xD80,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xE01,
	0xE31,
	0xE32,
	0xE33,
	0xE40,
	0xE47,
	0xE81,
	0xE83,
	0xE84,
	0xE85,
	0xE87,
	0xE89,
	0xE8A,
	0xE8B,
	0xE8D,
	0xE8E,
	0xE94,
	0xE98,
	0xE99,
	0xEA0,
	0xEA1,
	0xEA4,
	0xEA5,
	0xEA6,
	0xEA7,
	0xEA8,
	0xEAA,
	0xEAC,
	0xEAD,
	0xEB1,
	0xEB2,
	0xEB3,
	0xEBD,
	0xEBE,
	0xEC0,
	0xEC5,
	0xEC6,
	0xEC7,
	0xEDC,
	0xEE0,
	0xF00,
	0xF01,
	0xF40,
	0xF48,
	0xF49,
	0xF6D,
	0xF88,
	0xF8D,
	0x1000,
	0x102B,
	0x103F,
	0x1040,
	0x1050,
	0x1056,
	0x105A,
	0x105E,
	0x1061,
	0x1062,
	0x1065,
	0x1067,
	0x106E,
	0x1071,
	0x1075,
	0x1082,
	0x108E,
	0x108F,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x10FB,
	0x10FC,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x1380,
	0x1390,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1401,
	0x166D,
	0x166F,
	0x1680,
	0x1681,
	0x169B,
	0x16A0,
	0x16EB,
	0x16EE,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1712,
	0x1720,
	0x1732,
	0x1740,
	0x1752,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1780,
	0x17B4,
	0x17D7,
	0x17D8,
	0x17DC,
	0x17DD,
	0x1820,
	0x1878,
	0x1880,
	0x18A9,
	0x18AA,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1950,
	0x196E,
	0x1970,
	0x1975,
	0x1980,
	0x19AC,
	0x19B0,
	0x19CA,
	0x1A00,
	0x1A17,
	0x1A20,
	0x1A55,
	0x1AA7,
	0x1AA8,
	0x1B05,
	0x1B34,
	0x1B45,
	0x1B4C,
	0x1B83,
	0x1BA1,
	0x1BAE,
	0x1BB0,
	0x1BBA,
	0x1BE6,
	0x1C00,
	0x1C24,
	0x1C4D,
	0x1C50,
	0x1C5A,
	0x1C7E,
	0x1C80,
	0x1C89,
	0x1CE9,
	0x1CED,
	0x1CEE,
	0x1CF2,
	0x1CF5,
	0x1CF7,
	0x1D00,
	0x1DC0,
	0x1E00,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FBD,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FCD,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FE0,
	0x1FED,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFD,
	0x2071,
	0x2072,
	0x207F,
	0x2080,
	0x2090,
	0x209D,
	0x2102,
	0x2103,
	0x2107,
	0x2108,
	0x210A,
	0x2114,
	0x2115,
	0x2116,
	0x2119,
	0x211E,
	0x2124,
	0x2125,
	0x2126,
	0x2127,
	0x2128,
	0x2129,
	0x212A,
	0x212E,
	0x212F,
	0x213A,
	0x213C,
	0x2140,
	0x2145,
	0x214A,
	0x214E,
	0x214F,
	0x2160,
	0x2189,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CE5,
	0x2CEB,
	0x2CEF,
	0x2CF2,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D70,
	0x2D80,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x3005,
	0x3008,
	0x3021,
	0x302A,
	0x3031,
	0x3036,
	0x3038,
	0x303D,
	0x3041,
	0x3097,
	0x309D,
	0x30A0,
	0x30A1,
	0x30FB,
	0x30FC,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x31A0,
	0x31BB,
	0x31F0,
	0x3200,
	0x3400,
	0x4DB6,
	0x4E00,
	0x9FD6,
	0xA000,
	0xA48D,
	0xA4D0,
	0xA4FE,
	0xA500,
	0xA60D,
	0xA610,
	0xA620,
	0xA62A,
	0xA62C,
	0xA640,
	0xA66F,
	0xA67F,
	0xA69E,
	0xA6A0,
	0xA6F0,
	0xA717,
	0xA720,
	0xA722,
	0xA789,
	0xA78B,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F7,
	0xA802,
	0xA803,
	0xA806,
	0xA807,
	0xA80B,
	0xA80C,
	0xA823,
	0xA840,
	0xA874,
	0xA882,
	0xA8B4,
	0xA8F2,
	0xA8F8,
	0xA8FB,
	0xA8FC,
	0xA8FD,
	0xA8FE,
	0xA90A,
	0xA926,
	0xA930,
	0xA947,
	0xA960,
	0xA97D,
	0xA984,
	0xA9B3,
	0xA9CF,
	0xA9D0,
	0xA9E0,
	0xA9E5,
	0xA9E6,
	0xA9F0,
	0xA9FA,
	0xA9FF,
	0xAA00,
	0xAA29,
	0xAA40,
	0xAA43,
	0xAA44,
	0xAA4C,
	0xAA60,
	0xAA77,
	0xAA7A,
	0xAA7B,
	0xAA7E,
	0xAAB0,
	0xAAB1,
	0xAAB2,
	0xAAB5,
	0xAAB7,
	0xAAB9,
	0xAABE,
	0xAAC0,
	0xAAC1,
	0xAAC2,
	0xAAC3,
	0xAADB,
	0xAADE,
	0xAAE0,
	0xAAEB,
	0xAAF2,
	0xAAF5,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB5B,
	0xAB5C,
	0xAB66,
	0xAB70,
	0xABE3,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xF900,
	0xFA6E,
	0xFA70,
	0xFADA,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB1E,
	0xFB1F,
	0xFB29,
	0xFB2A,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFBB2,
	0xFBD3,
	0xFC5E,
	0xFC64,
	0xFD3E,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFA,
	0xFE71,
	0xFE72,
	0xFE73,
	0xFE74,
	0xFE77,
	0xFE78,
	0xFE79,
	0xFE7A,
	0xFE7B,
	0xFE7C,
	0xFE7D,
	0xFE7E,
	0xFE7F,
	0xFEFD,
	0xFF21,
	0xFF3B,
	0xFF41,
	0xFF5B,
	0xFF66,
	0xFF9E,
	0xFFA0,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10140,
	0x10175,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x10300,
	0x10320,
	0x10330,
	0x1034B,
	0x10350,
	0x10376,
	0x10380,
	0x1039E,
	0x103A0,
	0x103C4,
	0x103C8,
	0x103D0,
	0x103D1,
	0x103D6,
	0x10400,
	0x1049E,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10860,
	0x10877,
	0x10880,
	0x1089F,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x10900,
	0x10916,
	0x10920,
	0x1093A,
	0x10980,
	0x109B8,
	0x109BE,
	0x109C0,
	0x10A00,
	0x10A01,
	0x10A10,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A60,
	0x10A7D,
	0x10A80,
	0x10A9D,
	0x10AC0,
	0x10AC8,
	0x10AC9,
	0x10AE5,
	0x10B00,
	0x10B36,
	0x10B40,
	0x10B56,
	0x10B60,
	0x10B73,
	0x10B80,
	0x10B92,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x11003,
	0x11038,
	0x11083,
	0x110B0,
	0x110D0,
	0x110E9,
	0x11103,
	0x11127,
	0x11150,
	0x11173,
	0x11176,
	0x11177,
	0x11183,
	0x111B3,
	0x111C1,
	0x111C5,
	0x111DA,
	0x111DB,
	0x111DC,
	0x111DD,
	0x11200,
	0x11212,
	0x11213,
	0x1122C,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112A9,
	0x112B0,
	0x112DF,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133D,
	0x1133E,
	0x11350,
	0x11351,
	0x1135D,
	0x11362,
	0x11400,
	0x11435,
	0x11447,
	0x1144B,
	0x11480,
	0x114B0,
	0x114C4,
	0x114C6,
	0x114C7,
	0x114C8,
	0x11580,
	0x115AF,
	0x115D8,
	0x115DC,
	0x11600,
	0x11630,
	0x11644,
	0x11645,
	0x11680,
	0x116AB,
	0x11700,
	0x1171A,
	0x118A0,
	0x118E0,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C2F,
	0x11C40,
	0x11C41,
	0x11C72,
	0x11C90,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16AD0,
	0x16AEE,
	0x16B00,
	0x16B30,
	0x16B40,
	0x16B44,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F51,
	0x16F93,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D6C1,
	0x1D6C2,
	0x1D6DB,
	0x1D6DC,
	0x1D6FB,
	0x1D6FC,
	0x1D715,
	0x1D716,
	0x1D735,
	0x1D736,
	0x1D74F,
	0x1D750,
	0x1D76F,
	0x1D770,
	0x1D789,
	0x1D78A,
	0x1D7A9,
	0x1D7AA,
	0x1D7C3,
	0x1D7C4,
	0x1D7CC,
	0x1E800,
	0x1E8C5,
	0x1E900,
	0x1E944,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x20000,
	0x2A6D7,
	0x2A700,
	0x2B735,
	0x2B740,
	0x2B81E,
	0x2B820,
	0x2CEA2,
	0x2F800,
	0x2FA1E
};

#endif	/* defined(PERL_IN_UTF8_C) */

#if defined(PERL_IN_PERL_C)

static const UV _Perl_LB_invlist[] = { /* for EBCDIC 037 */
	2279,	/* Number of elements */
	148565664, /* Version and data structure type */
	0,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x5,
	0x6,
	0xB,
	0xD,
	0xE,
	0x15,
	0x16,
	0x25,
	0x26,
	0x40,
	0x41,
	0x42,
	0x4A,
	0x4B,
	0x4C,
	0x4D,
	0x4E,
	0x4F,
	0x50,
	0x5A,
	0x5B,
	0x5C,
	0x5D,
	0x5E,
	0x5F,
	0x60,
	0x61,
	0x62,
	0x6B,
	0x6C,
	0x6D,
	0x6F,
	0x70,
	0x7A,
	0x7B,
	0x7D,
	0x7E,
	0x7F,
	0x80,
	0x8A,
	0x8C,
	0x8F,
	0x90,
	0x91,
	0x9F,
	0xA0,
	0xAA,
	0xAC,
	0xB1,
	0xB3,
	0xBA,
	0xBB,
	0xBC,
	0xBE,
	0xBF,
	0xC0,
	0xC1,
	0xCA,
	0xCB,
	0xD0,
	0xD1,
	0xE0,
	0xE1,
	0xF0,
	0xFA,
	0xFF,
	0x100,
	0x2C8,
	0x2C9,
	0x2CC,
	0x2CD,
	0x2DF,
	0x2E0,
	0x300,
	0x34F,
	0x350,
	0x35C,
	0x363,
	0x370,
	0x37E,
	0x37F,
	0x483,
	0x48A,
	0x589,
	0x58A,
	0x58B,
	0x58F,
	0x590,
	0x591,
	0x5BE,
	0x5BF,
	0x5C0,
	0x5C1,
	0x5C3,
	0x5C4,
	0x5C6,
	0x5C7,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F3,
	0x609,
	0x60C,
	0x60E,
	0x610,
	0x61B,
	0x61C,
	0x61D,
	0x61E,
	0x620,
	0x64B,
	0x660,
	0x66A,
	0x66B,
	0x66D,
	0x670,
	0x671,
	0x6D4,
	0x6D5,
	0x6D6,
	0x6DD,
	0x6DF,
	0x6E5,
	0x6E7,
	0x6E9,
	0x6EA,
	0x6EE,
	0x6F0,
	0x6FA,
	0x711,
	0x712,
	0x730,
	0x74B,
	0x7A6,
	0x7B1,
	0x7C0,
	0x7CA,
	0x7EB,
	0x7F4,
	0x7F8,
	0x7F9,
	0x7FA,
	0x816,
	0x81A,
	0x81B,
	0x824,
	0x825,
	0x828,
	0x829,
	0x82E,
	0x859,
	0x85C,
	0x8D4,
	0x8E2,
	0x8E3,
	0x904,
	0x93A,
	0x93D,
	0x93E,
	0x950,
	0x951,
	0x958,
	0x962,
	0x964,
	0x966,
	0x970,
	0x981,
	0x984,
	0x9BC,
	0x9BD,
	0x9BE,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CE,
	0x9D7,
	0x9D8,
	0x9E2,
	0x9E4,
	0x9E6,
	0x9F0,
	0x9F2,
	0x9F4,
	0x9F9,
	0x9FA,
	0x9FB,
	0x9FC,
	0xA01,
	0xA04,
	0xA3C,
	0xA3D,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4E,
	0xA51,
	0xA52,
	0xA66,
	0xA70,
	0xA72,
	0xA75,
	0xA76,
	0xA81,
	0xA84,
	0xABC,
	0xABD,
	0xABE,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACE,
	0xAE2,
	0xAE4,
	0xAE6,
	0xAF0,
	0xAF1,
	0xAF2,
	0xB01,
	0xB04,
	0xB3C,
	0xB3D,
	0xB3E,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4E,
	0xB56,
	0xB58,
	0xB62,
	0xB64,
	0xB66,
	0xB70,
	0xB82,
	0xB83,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCE,
	0xBD7,
	0xBD8,
	0xBE6,
	0xBF0,
	0xBF9,
	0xBFA,
	0xC00,
	0xC04,
	0xC3E,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4E,
	0xC55,
	0xC57,
	0xC62,
	0xC64,
	0xC66,
	0xC70,
	0xC81,
	0xC84,
	0xCBC,
	0xCBD,
	0xCBE,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCE,
	0xCD5,
	0xCD7,
	0xCE2,
	0xCE4,
	0xCE6,
	0xCF0,
	0xD01,
	0xD04,
	0xD3E,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD4E,
	0xD57,
	0xD58,
	0xD62,
	0xD64,
	0xD66,
	0xD70,
	0xD79,
	0xD7A,
	0xD82,
	0xD84,
	0xDCA,
	0xDCB,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDE6,
	0xDF0,
	0xDF2,
	0xDF4,
	0xE31,
	0xE32,
	0xE34,
	0xE3B,
	0xE3F,
	0xE40,
	0xE47,
	0xE4F,
	0xE50,
	0xE5A,
	0xE5C,
	0xEB1,
	0xEB2,
	0xEB4,
	0xEBA,
	0xEBB,
	0xEBD,
	0xEC8,
	0xECE,
	0xED0,
	0xEDA,
	0xF01,
	0xF05,
	0xF06,
	0xF08,
	0xF09,
	0xF0B,
	0xF0C,
	0xF0D,
	0xF12,
	0xF13,
	0xF14,
	0xF15,
	0xF18,
	0xF1A,
	0xF20,
	0xF2A,
	0xF34,
	0xF35,
	0xF36,
	0xF37,
	0xF38,
	0xF39,
	0xF3A,
	0xF3B,
	0xF3C,
	0xF3D,
	0xF3E,
	0xF40,
	0xF71,
	0xF7F,
	0xF80,
	0xF85,
	0xF86,
	0xF88,
	0xF8D,
	0xF98,
	0xF99,
	0xFBD,
	0xFBE,
	0xFC0,
	0xFC6,
	0xFC7,
	0xFD0,
	0xFD2,
	0xFD3,
	0xFD4,
	0xFD9,
	0xFDB,
	0x102B,
	0x103F,
	0x1040,
	0x104A,
	0x104C,
	0x1056,
	0x105A,
	0x105E,
	0x1061,
	0x1062,
	0x1065,
	0x1067,
	0x106E,
	0x1071,
	0x1075,
	0x1082,
	0x108E,
	0x108F,
	0x1090,
	0x109A,
	0x109E,
	0x1100,
	0x1160,
	0x11A8,
	0x1200,
	0x135D,
	0x1360,
	0x1361,
	0x1362,
	0x1400,
	0x1401,
	0x1680,
	0x1681,
	0x169B,
	0x169C,
	0x169D,
	0x16EB,
	0x16EE,
	0x1712,
	0x1715,
	0x1732,
	0x1735,
	0x1737,
	0x1752,
	0x1754,
	0x1772,
	0x1774,
	0x17B4,
	0x17D4,
	0x17D6,
	0x17D7,
	0x17D8,
	0x17D9,
	0x17DA,
	0x17DB,
	0x17DC,
	0x17DD,
	0x17DE,
	0x17E0,
	0x17EA,
	0x1802,
	0x1804,
	0x1806,
	0x1807,
	0x1808,
	0x180A,
	0x180B,
	0x180E,
	0x180F,
	0x1810,
	0x181A,
	0x1885,
	0x1887,
	0x18A9,
	0x18AA,
	0x1920,
	0x192C,
	0x1930,
	0x193C,
	0x1944,
	0x1946,
	0x1950,
	0x19D0,
	0x19DA,
	0x1A17,
	0x1A1C,
	0x1A55,
	0x1A5F,
	0x1A60,
	0x1A7D,
	0x1A7F,
	0x1A80,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1AB0,
	0x1ABF,
	0x1B00,
	0x1B05,
	0x1B34,
	0x1B45,
	0x1B50,
	0x1B5A,
	0x1B5C,
	0x1B5D,
	0x1B61,
	0x1B6B,
	0x1B74,
	0x1B80,
	0x1B83,
	0x1BA1,
	0x1BAE,
	0x1BB0,
	0x1BBA,
	0x1BE6,
	0x1BF4,
	0x1C24,
	0x1C38,
	0x1C3B,
	0x1C40,
	0x1C4A,
	0x1C50,
	0x1C5A,
	0x1C7E,
	0x1C80,
	0x1CD0,
	0x1CD3,
	0x1CD4,
	0x1CE9,
	0x1CED,
	0x1CEE,
	0x1CF2,
	0x1CF5,
	0x1CF8,
	0x1CFA,
	0x1DC0,
	0x1DF6,
	0x1DFB,
	0x1E00,
	0x1FFD,
	0x1FFE,
	0x2000,
	0x2007,
	0x2008,
	0x200B,
	0x200C,
	0x200D,
	0x200E,
	0x2010,
	0x2011,
	0x2012,
	0x2014,
	0x2015,
	0x2018,
	0x201A,
	0x201B,
	0x201E,
	0x201F,
	0x2020,
	0x2024,
	0x2027,
	0x2028,
	0x202A,
	0x202F,
	0x2030,
	0x2038,
	0x2039,
	0x203B,
	0x203C,
	0x203E,
	0x2044,
	0x2045,
	0x2046,
	0x2047,
	0x204A,
	0x2056,
	0x2057,
	0x2058,
	0x205C,
	0x205D,
	0x2060,
	0x2061,
	0x2066,
	0x2070,
	0x207D,
	0x207E,
	0x207F,
	0x208D,
	0x208E,
	0x208F,
	0x20A0,
	0x20A7,
	0x20A8,
	0x20B6,
	0x20B7,
	0x20BB,
	0x20BC,
	0x20BE,
	0x20BF,
	0x20D0,
	0x20F1,
	0x2103,
	0x2104,
	0x2109,
	0x210A,
	0x2116,
	0x2117,
	0x2212,
	0x2214,
	0x22EF,
	0x22F0,
	0x2308,
	0x2309,
	0x230A,
	0x230B,
	0x230C,
	0x231A,
	0x231C,
	0x2329,
	0x232A,
	0x232B,
	0x23F0,
	0x23F4,
	0x2600,
	0x2604,
	0x2614,
	0x2616,
	0x2618,
	0x2619,
	0x261A,
	0x261D,
	0x261E,
	0x2620,
	0x2639,
	0x263C,
	0x2668,
	0x2669,
	0x267F,
	0x2680,
	0x26BD,
	0x26C9,
	0x26CD,
	0x26CE,
	0x26CF,
	0x26D2,
	0x26D3,
	0x26D5,
	0x26D8,
	0x26DA,
	0x26DC,
	0x26DD,
	0x26DF,
	0x26E2,
	0x26EA,
	0x26EB,
	0x26F1,
	0x26F6,
	0x26F7,
	0x26F9,
	0x26FA,
	0x26FB,
	0x26FD,
	0x2705,
	0x2708,
	0x270A,
	0x270E,
	0x275B,
	0x2761,
	0x2762,
	0x2764,
	0x2765,
	0x2768,
	0x2769,
	0x276A,
	0x276B,
	0x276C,
	0x276D,
	0x276E,
	0x276F,
	0x2770,
	0x2771,
	0x2772,
	0x2773,
	0x2774,
	0x2775,
	0x2776,
	0x27C5,
	0x27C6,
	0x27C7,
	0x27E6,
	0x27E7,
	0x27E8,
	0x27E9,
	0x27EA,
	0x27EB,
	0x27EC,
	0x27ED,
	0x27EE,
	0x27EF,
	0x27F0,
	0x2983,
	0x2984,
	0x2985,
	0x2986,
	0x2987,
	0x2988,
	0x2989,
	0x298A,
	0x298B,
	0x298C,
	0x298D,
	0x298E,
	0x298F,
	0x2990,
	0x2991,
	0x2992,
	0x2993,
	0x2994,
	0x2995,
	0x2996,
	0x2997,
	0x2998,
	0x2999,
	0x29D8,
	0x29D9,
	0x29DA,
	0x29DB,
	0x29DC,
	0x29FC,
	0x29FD,
	0x29FE,
	0x2CEF,
	0x2CF2,
	0x2CF9,
	0x2CFA,
	0x2CFD,
	0x2CFE,
	0x2CFF,
	0x2D00,
	0x2D70,
	0x2D71,
	0x2D7F,
	0x2D80,
	0x2DE0,
	0x2E00,
	0x2E0E,
	0x2E16,
	0x2E17,
	0x2E18,
	0x2E19,
	0x2E1A,
	0x2E1C,
	0x2E1E,
	0x2E20,
	0x2E22,
	0x2E23,
	0x2E24,
	0x2E25,
	0x2E26,
	0x2E27,
	0x2E28,
	0x2E29,
	0x2E2A,
	0x2E2E,
	0x2E2F,
	0x2E30,
	0x2E32,
	0x2E33,
	0x2E35,
	0x2E3A,
	0x2E3C,
	0x2E3F,
	0x2E40,
	0x2E42,
	0x2E43,
	0x2E45,
	0x2E80,
	0x2E9A,
	0x2E9B,
	0x2EF4,
	0x2F00,
	0x2FD6,
	0x2FF0,
	0x2FFC,
	0x3000,
	0x3001,
	0x3003,
	0x3005,
	0x3006,
	0x3008,
	0x3009,
	0x300A,
	0x300B,
	0x300C,
	0x300D,
	0x300E,
	0x300F,
	0x3010,
	0x3011,
	0x3012,
	0x3014,
	0x3015,
	0x3016,
	0x3017,
	0x3018,
	0x3019,
	0x301A,
	0x301B,
	0x301C,
	0x301D,
	0x301E,
	0x3020,
	0x302A,
	0x3030,
	0x3035,
	0x3036,
	0x303B,
	0x303D,
	0x3040,
	0x3041,
	0x3042,
	0x3043,
	0x3044,
	0x3045,
	0x3046,
	0x3047,
	0x3048,
	0x3049,
	0x304A,
	0x3063,
	0x3064,
	0x3083,
	0x3084,
	0x3085,
	0x3086,
	0x3087,
	0x3088,
	0x308E,
	0x308F,
	0x3095,
	0x3097,
	0x3099,
	0x309B,
	0x309F,
	0x30A0,
	0x30A2,
	0x30A3,
	0x30A4,
	0x30A5,
	0x30A6,
	0x30A7,
	0x30A8,
	0x30A9,
	0x30AA,
	0x30C3,
	0x30C4,
	0x30E3,
	0x30E4,
	0x30E5,
	0x30E6,
	0x30E7,
	0x30E8,
	0x30EE,
	0x30EF,
	0x30F5,
	0x30F7,
	0x30FB,
	0x30FF,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x3190,
	0x31BB,
	0x31C0,
	0x31E4,
	0x31F0,
	0x3200,
	0x321F,
	0x3220,
	0x3248,
	0x3250,
	0x32FF,
	0x3300,
	0x4DC0,
	0x4E00,
	0xA015,
	0xA016,
	0xA48D,
	0xA490,
	0xA4C7,
	0xA4FE,
	0xA500,
	0xA60D,
	0xA60E,
	0xA60F,
	0xA610,
	0xA620,
	0xA62A,
	0xA66F,
	0xA673,
	0xA674,
	0xA67E,
	0xA69E,
	0xA6A0,
	0xA6F0,
	0xA6F2,
	0xA6F3,
	0xA6F8,
	0xA802,
	0xA803,
	0xA806,
	0xA807,
	0xA80B,
	0xA80C,
	0xA823,
	0xA828,
	0xA838,
	0xA839,
	0xA874,
	0xA876,
	0xA878,
	0xA880,
	0xA882,
	0xA8B4,
	0xA8C6,
	0xA8CE,
	0xA8D0,
	0xA8DA,
	0xA8E0,
	0xA8F2,
	0xA8FC,
	0xA8FD,
	0xA900,
	0xA90A,
	0xA926,
	0xA92E,
	0xA930,
	0xA947,
	0xA954,
	0xA960,
	0xA97D,
	0xA980,
	0xA984,
	0xA9B3,
	0xA9C1,
	0xA9C7,
	0xA9CA,
	0xA9D0,
	0xA9DA,
	0xA9E5,
	0xA9E6,
	0xA9F0,
	0xA9FA,
	0xAA29,
	0xAA37,
	0xAA43,
	0xAA44,
	0xAA4C,
	0xAA4E,
	0xAA50,
	0xAA5A,
	0xAA5D,
	0xAA60,
	0xAA7B,
	0xAA7E,
	0xAAB0,
	0xAAB1,
	0xAAB2,
	0xAAB5,
	0xAAB7,
	0xAAB9,
	0xAABE,
	0xAAC0,
	0xAAC1,
	0xAAC2,
	0xAAEB,
	0xAAF0,
	0xAAF2,
	0xAAF5,
	0xAAF7,
	0xABE3,
	0xABEB,
	0xABEC,
	0xABEE,
	0xABF0,
	0xABFA,
	0xAC00,
	0xAC01,
	0xAC1C,
	0xAC1D,
	0xAC38,
	0xAC39,
	0xAC54,
	0xAC55,
	0xAC70,
	0xAC71,
	0xAC8C,
	0xAC8D,
	0xACA8,
	0xACA9,
	0xACC4,
	0xACC5,
	0xACE0,
	0xACE1,
	0xACFC,
	0xACFD,
	0xAD18,
	0xAD19,
	0xAD34,
	0xAD35,
	0xAD50,
	0xAD51,
	0xAD6C,
	0xAD6D,
	0xAD88,
	0xAD89,
	0xADA4,
	0xADA5,
	0xADC0,
	0xADC1,
	0xADDC,
	0xADDD,
	0xADF8,
	0xADF9,
	0xAE14,
	0xAE15,
	0xAE30,
	0xAE31,
	0xAE4C,
	0xAE4D,
	0xAE68,
	0xAE69,
	0xAE84,
	0xAE85,
	0xAEA0,
	0xAEA1,
	0xAEBC,
	0xAEBD,
	0xAED8,
	0xAED9,
	0xAEF4,
	0xAEF5,
	0xAF10,
	0xAF11,
	0xAF2C,
	0xAF2D,
	0xAF48,
	0xAF49,
	0xAF64,
	0xAF65,
	0xAF80,
	0xAF81,
	0xAF9C,
	0xAF9D,
	0xAFB8,
	0xAFB9,
	0xAFD4,
	0xAFD5,
	0xAFF0,
	0xAFF1,
	0xB00C,
	0xB00D,
	0xB028,
	0xB029,
	0xB044,
	0xB045,
	0xB060,
	0xB061,
	0xB07C,
	0xB07D,
	0xB098,
	0xB099,
	0xB0B4,
	0xB0B5,
	0xB0D0,
	0xB0D1,
	0xB0EC,
	0xB0ED,
	0xB108,
	0xB109,
	0xB124,
	0xB125,
	0xB140,
	0xB141,
	0xB15C,
	0xB15D,
	0xB178,
	0xB179,
	0xB194,
	0xB195,
	0xB1B0,
	0xB1B1,
	0xB1CC,
	0xB1CD,
	0xB1E8,
	0xB1E9,
	0xB204,
	0xB205,
	0xB220,
	0xB221,
	0xB23C,
	0xB23D,
	0xB258,
	0xB259,
	0xB274,
	0xB275,
	0xB290,
	0xB291,
	0xB2AC,
	0xB2AD,
	0xB2C8,
	0xB2C9,
	0xB2E4,
	0xB2E5,
	0xB300,
	0xB301,
	0xB31C,
	0xB31D,
	0xB338,
	0xB339,
	0xB354,
	0xB355,
	0xB370,
	0xB371,
	0xB38C,
	0xB38D,
	0xB3A8,
	0xB3A9,
	0xB3C4,
	0xB3C5,
	0xB3E0,
	0xB3E1,
	0xB3FC,
	0xB3FD,
	0xB418,
	0xB419,
	0xB434,
	0xB435,
	0xB450,
	0xB451,
	0xB46C,
	0xB46D,
	0xB488,
	0xB489,
	0xB4A4,
	0xB4A5,
	0xB4C0,
	0xB4C1,
	0xB4DC,
	0xB4DD,
	0xB4F8,
	0xB4F9,
	0xB514,
	0xB515,
	0xB530,
	0xB531,
	0xB54C,
	0xB54D,
	0xB568,
	0xB569,
	0xB584,
	0xB585,
	0xB5A0,
	0xB5A1,
	0xB5BC,
	0xB5BD,
	0xB5D8,
	0xB5D9,
	0xB5F4,
	0xB5F5,
	0xB610,
	0xB611,
	0xB62C,
	0xB62D,
	0xB648,
	0xB649,
	0xB664,
	0xB665,
	0xB680,
	0xB681,
	0xB69C,
	0xB69D,
	0xB6B8,
	0xB6B9,
	0xB6D4,
	0xB6D5,
	0xB6F0,
	0xB6F1,
	0xB70C,
	0xB70D,
	0xB728,
	0xB729,
	0xB744,
	0xB745,
	0xB760,
	0xB761,
	0xB77C,
	0xB77D,
	0xB798,
	0xB799,
	0xB7B4,
	0xB7B5,
	0xB7D0,
	0xB7D1,
	0xB7EC,
	0xB7ED,
	0xB808,
	0xB809,
	0xB824,
	0xB825,
	0xB840,
	0xB841,
	0xB85C,
	0xB85D,
	0xB878,
	0xB879,
	0xB894,
	0xB895,
	0xB8B0,
	0xB8B1,
	0xB8CC,
	0xB8CD,
	0xB8E8,
	0xB8E9,
	0xB904,
	0xB905,
	0xB920,
	0xB921,
	0xB93C,
	0xB93D,
	0xB958,
	0xB959,
	0xB974,
	0xB975,
	0xB990,
	0xB991,
	0xB9AC,
	0xB9AD,
	0xB9C8,
	0xB9C9,
	0xB9E4,
	0xB9E5,
	0xBA00,
	0xBA01,
	0xBA1C,
	0xBA1D,
	0xBA38,
	0xBA39,
	0xBA54,
	0xBA55,
	0xBA70,
	0xBA71,
	0xBA8C,
	0xBA8D,
	0xBAA8,
	0xBAA9,
	0xBAC4,
	0xBAC5,
	0xBAE0,
	0xBAE1,
	0xBAFC,
	0xBAFD,
	0xBB18,
	0xBB19,
	0xBB34,
	0xBB35,
	0xBB50,
	0xBB51,
	0xBB6C,
	0xBB6D,
	0xBB88,
	0xBB89,
	0xBBA4,
	0xBBA5,
	0xBBC0,
	0xBBC1,
	0xBBDC,
	0xBBDD,
	0xBBF8,
	0xBBF9,
	0xBC14,
	0xBC15,
	0xBC30,
	0xBC31,
	0xBC4C,
	0xBC4D,
	0xBC68,
	0xBC69,
	0xBC84,
	0xBC85,
	0xBCA0,
	0xBCA1,
	0xBCBC,
	0xBCBD,
	0xBCD8,
	0xBCD9,
	0xBCF4,
	0xBCF5,
	0xBD10,
	0xBD11,
	0xBD2C,
	0xBD2D,
	0xBD48,
	0xBD49,
	0xBD64,
	0xBD65,
	0xBD80,
	0xBD81,
	0xBD9C,
	0xBD9D,
	0xBDB8,
	0xBDB9,
	0xBDD4,
	0xBDD5,
	0xBDF0,
	0xBDF1,
	0xBE0C,
	0xBE0D,
	0xBE28,
	0xBE29,
	0xBE44,
	0xBE45,
	0xBE60,
	0xBE61,
	0xBE7C,
	0xBE7D,
	0xBE98,
	0xBE99,
	0xBEB4,
	0xBEB5,
	0xBED0,
	0xBED1,
	0xBEEC,
	0xBEED,
	0xBF08,
	0xBF09,
	0xBF24,
	0xBF25,
	0xBF40,
	0xBF41,
	0xBF5C,
	0xBF5D,
	0xBF78,
	0xBF79,
	0xBF94,
	0xBF95,
	0xBFB0,
	0xBFB1,
	0xBFCC,
	0xBFCD,
	0xBFE8,
	0xBFE9,
	0xC004,
	0xC005,
	0xC020,
	0xC021,
	0xC03C,
	0xC03D,
	0xC058,
	0xC059,
	0xC074,
	0xC075,
	0xC090,
	0xC091,
	0xC0AC,
	0xC0AD,
	0xC0C8,
	0xC0C9,
	0xC0E4,
	0xC0E5,
	0xC100,
	0xC101,
	0xC11C,
	0xC11D,
	0xC138,
	0xC139,
	0xC154,
	0xC155,
	0xC170,
	0xC171,
	0xC18C,
	0xC18D,
	0xC1A8,
	0xC1A9,
	0xC1C4,
	0xC1C5,
	0xC1E0,
	0xC1E1,
	0xC1FC,
	0xC1FD,
	0xC218,
	0xC219,
	0xC234,
	0xC235,
	0xC250,
	0xC251,
	0xC26C,
	0xC26D,
	0xC288,
	0xC289,
	0xC2A4,
	0xC2A5,
	0xC2C0,
	0xC2C1,
	0xC2DC,
	0xC2DD,
	0xC2F8,
	0xC2F9,
	0xC314,
	0xC315,
	0xC330,
	0xC331,
	0xC34C,
	0xC34D,
	0xC368,
	0xC369,
	0xC384,
	0xC385,
	0xC3A0,
	0xC3A1,
	0xC3BC,
	0xC3BD,
	0xC3D8,
	0xC3D9,
	0xC3F4,
	0xC3F5,
	0xC410,
	0xC411,
	0xC42C,
	0xC42D,
	0xC448,
	0xC449,
	0xC464,
	0xC465,
	0xC480,
	0xC481,
	0xC49C,
	0xC49D,
	0xC4B8,
	0xC4B9,
	0xC4D4,
	0xC4D5,
	0xC4F0,
	0xC4F1,
	0xC50C,
	0xC50D,
	0xC528,
	0xC529,
	0xC544,
	0xC545,
	0xC560,
	0xC561,
	0xC57C,
	0xC57D,
	0xC598,
	0xC599,
	0xC5B4,
	0xC5B5,
	0xC5D0,
	0xC5D1,
	0xC5EC,
	0xC5ED,
	0xC608,
	0xC609,
	0xC624,
	0xC625,
	0xC640,
	0xC641,
	0xC65C,
	0xC65D,
	0xC678,
	0xC679,
	0xC694,
	0xC695,
	0xC6B0,
	0xC6B1,
	0xC6CC,
	0xC6CD,
	0xC6E8,
	0xC6E9,
	0xC704,
	0xC705,
	0xC720,
	0xC721,
	0xC73C,
	0xC73D,
	0xC758,
	0xC759,
	0xC774,
	0xC775,
	0xC790,
	0xC791,
	0xC7AC,
	0xC7AD,
	0xC7C8,
	0xC7C9,
	0xC7E4,
	0xC7E5,
	0xC800,
	0xC801,
	0xC81C,
	0xC81D,
	0xC838,
	0xC839,
	0xC854,
	0xC855,
	0xC870,
	0xC871,
	0xC88C,
	0xC88D,
	0xC8A8,
	0xC8A9,
	0xC8C4,
	0xC8C5,
	0xC8E0,
	0xC8E1,
	0xC8FC,
	0xC8FD,
	0xC918,
	0xC919,
	0xC934,
	0xC935,
	0xC950,
	0xC951,
	0xC96C,
	0xC96D,
	0xC988,
	0xC989,
	0xC9A4,
	0xC9A5,
	0xC9C0,
	0xC9C1,
	0xC9DC,
	0xC9DD,
	0xC9F8,
	0xC9F9,
	0xCA14,
	0xCA15,
	0xCA30,
	0xCA31,
	0xCA4C,
	0xCA4D,
	0xCA68,
	0xCA69,
	0xCA84,
	0xCA85,
	0xCAA0,
	0xCAA1,
	0xCABC,
	0xCABD,
	0xCAD8,
	0xCAD9,
	0xCAF4,
	0xCAF5,
	0xCB10,
	0xCB11,
	0xCB2C,
	0xCB2D,
	0xCB48,
	0xCB49,
	0xCB64,
	0xCB65,
	0xCB80,
	0xCB81,
	0xCB9C,
	0xCB9D,
	0xCBB8,
	0xCBB9,
	0xCBD4,
	0xCBD5,
	0xCBF0,
	0xCBF1,
	0xCC0C,
	0xCC0D,
	0xCC28,
	0xCC29,
	0xCC44,
	0xCC45,
	0xCC60,
	0xCC61,
	0xCC7C,
	0xCC7D,
	0xCC98,
	0xCC99,
	0xCCB4,
	0xCCB5,
	0xCCD0,
	0xCCD1,
	0xCCEC,
	0xCCED,
	0xCD08,
	0xCD09,
	0xCD24,
	0xCD25,
	0xCD40,
	0xCD41,
	0xCD5C,
	0xCD5D,
	0xCD78,
	0xCD79,
	0xCD94,
	0xCD95,
	0xCDB0,
	0xCDB1,
	0xCDCC,
	0xCDCD,
	0xCDE8,
	0xCDE9,
	0xCE04,
	0xCE05,
	0xCE20,
	0xCE21,
	0xCE3C,
	0xCE3D,
	0xCE58,
	0xCE59,
	0xCE74,
	0xCE75,
	0xCE90,
	0xCE91,
	0xCEAC,
	0xCEAD,
	0xCEC8,
	0xCEC9,
	0xCEE4,
	0xCEE5,
	0xCF00,
	0xCF01,
	0xCF1C,
	0xCF1D,
	0xCF38,
	0xCF39,
	0xCF54,
	0xCF55,
	0xCF70,
	0xCF71,
	0xCF8C,
	0xCF8D,
	0xCFA8,
	0xCFA9,
	0xCFC4,
	0xCFC5,
	0xCFE0,
	0xCFE1,
	0xCFFC,
	0xCFFD,
	0xD018,
	0xD019,
	0xD034,
	0xD035,
	0xD050,
	0xD051,
	0xD06C,
	0xD06D,
	0xD088,
	0xD089,
	0xD0A4,
	0xD0A5,
	0xD0C0,
	0xD0C1,
	0xD0DC,
	0xD0DD,
	0xD0F8,
	0xD0F9,
	0xD114,
	0xD115,
	0xD130,
	0xD131,
	0xD14C,
	0xD14D,
	0xD168,
	0xD169,
	0xD184,
	0xD185,
	0xD1A0,
	0xD1A1,
	0xD1BC,
	0xD1BD,
	0xD1D8,
	0xD1D9,
	0xD1F4,
	0xD1F5,
	0xD210,
	0xD211,
	0xD22C,
	0xD22D,
	0xD248,
	0xD249,
	0xD264,
	0xD265,
	0xD280,
	0xD281,
	0xD29C,
	0xD29D,
	0xD2B8,
	0xD2B9,
	0xD2D4,
	0xD2D5,
	0xD2F0,
	0xD2F1,
	0xD30C,
	0xD30D,
	0xD328,
	0xD329,
	0xD344,
	0xD345,
	0xD360,
	0xD361,
	0xD37C,
	0xD37D,
	0xD398,
	0xD399,
	0xD3B4,
	0xD3B5,
	0xD3D0,
	0xD3D1,
	0xD3EC,
	0xD3ED,
	0xD408,
	0xD409,
	0xD424,
	0xD425,
	0xD440,
	0xD441,
	0xD45C,
	0xD45D,
	0xD478,
	0xD479,
	0xD494,
	0xD495,
	0xD4B0,
	0xD4B1,
	0xD4CC,
	0xD4CD,
	0xD4E8,
	0xD4E9,
	0xD504,
	0xD505,
	0xD520,
	0xD521,
	0xD53C,
	0xD53D,
	0xD558,
	0xD559,
	0xD574,
	0xD575,
	0xD590,
	0xD591,
	0xD5AC,
	0xD5AD,
	0xD5C8,
	0xD5C9,
	0xD5E4,
	0xD5E5,
	0xD600,
	0xD601,
	0xD61C,
	0xD61D,
	0xD638,
	0xD639,
	0xD654,
	0xD655,
	0xD670,
	0xD671,
	0xD68C,
	0xD68D,
	0xD6A8,
	0xD6A9,
	0xD6C4,
	0xD6C5,
	0xD6E0,
	0xD6E1,
	0xD6FC,
	0xD6FD,
	0xD718,
	0xD719,
	0xD734,
	0xD735,
	0xD750,
	0xD751,
	0xD76C,
	0xD76D,
	0xD788,
	0xD789,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xF900,
	0xFB00,
	0xFB1D,
	0xFB1E,
	0xFB1F,
	0xFB29,
	0xFB2A,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFB50,
	0xFD3E,
	0xFD3F,
	0xFD40,
	0xFDFC,
	0xFDFD,
	0xFE00,
	0xFE10,
	0xFE11,
	0xFE13,
	0xFE15,
	0xFE17,
	0xFE18,
	0xFE19,
	0xFE1A,
	0xFE20,
	0xFE30,
	0xFE35,
	0xFE36,
	0xFE37,
	0xFE38,
	0xFE39,
	0xFE3A,
	0xFE3B,
	0xFE3C,
	0xFE3D,
	0xFE3E,
	0xFE3F,
	0xFE40,
	0xFE41,
	0xFE42,
	0xFE43,
	0xFE44,
	0xFE45,
	0xFE47,
	0xFE48,
	0xFE49,
	0xFE50,
	0xFE51,
	0xFE52,
	0xFE53,
	0xFE54,
	0xFE56,
	0xFE58,
	0xFE59,
	0xFE5A,
	0xFE5B,
	0xFE5C,
	0xFE5D,
	0xFE5E,
	0xFE5F,
	0xFE67,
	0xFE68,
	0xFE69,
	0xFE6A,
	0xFE6B,
	0xFE6C,
	0xFEFF,
	0xFF00,
	0xFF01,
	0xFF02,
	0xFF04,
	0xFF05,
	0xFF06,
	0xFF08,
	0xFF09,
	0xFF0A,
	0xFF0C,
	0xFF0D,
	0xFF0E,
	0xFF0F,
	0xFF1A,
	0xFF1C,
	0xFF1F,
	0xFF20,
	0xFF3B,
	0xFF3C,
	0xFF3D,
	0xFF3E,
	0xFF5B,
	0xFF5C,
	0xFF5D,
	0xFF5E,
	0xFF5F,
	0xFF60,
	0xFF62,
	0xFF63,
	0xFF65,
	0xFF66,
	0xFF67,
	0xFF71,
	0xFF9E,
	0xFFA0,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0xFFE0,
	0xFFE1,
	0xFFE2,
	0xFFE5,
	0xFFE7,
	0xFFF9,
	0xFFFC,
	0xFFFD,
	0x10100,
	0x10103,
	0x101FD,
	0x101FE,
	0x102E0,
	0x102E1,
	0x10376,
	0x1037B,
	0x1039F,
	0x103A0,
	0x103D0,
	0x103D1,
	0x104A0,
	0x104AA,
	0x10857,
	0x10858,
	0x1091F,
	0x10920,
	0x10A01,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A10,
	0x10A38,
	0x10A3B,
	0x10A3F,
	0x10A40,
	0x10A50,
	0x10A58,
	0x10AE5,
	0x10AE7,
	0x10AF0,
	0x10AF6,
	0x10AF7,
	0x10B39,
	0x10B40,
	0x11000,
	0x11003,
	0x11038,
	0x11047,
	0x11049,
	0x11066,
	0x11070,
	0x1107F,
	0x11083,
	0x110B0,
	0x110BB,
	0x110BE,
	0x110C2,
	0x110F0,
	0x110FA,
	0x11100,
	0x11103,
	0x11127,
	0x11135,
	0x11136,
	0x11140,
	0x11144,
	0x11173,
	0x11174,
	0x11175,
	0x11176,
	0x11180,
	0x11183,
	0x111B3,
	0x111C1,
	0x111C5,
	0x111C7,
	0x111C8,
	0x111C9,
	0x111CA,
	0x111CD,
	0x111D0,
	0x111DA,
	0x111DB,
	0x111DC,
	0x111DD,
	0x111E0,
	0x1122C,
	0x11238,
	0x1123A,
	0x1123B,
	0x1123D,
	0x1123E,
	0x1123F,
	0x112A9,
	0x112AA,
	0x112DF,
	0x112EB,
	0x112F0,
	0x112FA,
	0x11300,
	0x11304,
	0x1133C,
	0x1133D,
	0x1133E,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134E,
	0x11357,
	0x11358,
	0x11362,
	0x11364,
	0x11366,
	0x1136D,
	0x11370,
	0x11375,
	0x11435,
	0x11447,
	0x1144B,
	0x1144F,
	0x11450,
	0x1145A,
	0x1145B,
	0x1145C,
	0x114B0,
	0x114C4,
	0x114D0,
	0x114DA,
	0x115AF,
	0x115B6,
	0x115B8,
	0x115C1,
	0x115C2,
	0x115C4,
	0x115C6,
	0x115C9,
	0x115D8,
	0x115DC,
	0x115DE,
	0x11630,
	0x11641,
	0x11643,
	0x11650,
	0x1165A,
	0x11660,
	0x1166D,
	0x116AB,
	0x116B8,
	0x116C0,
	0x116CA,
	0x1171D,
	0x1172C,
	0x11730,
	0x1173A,
	0x1173C,
	0x1173F,
	0x118E0,
	0x118EA,
	0x11C2F,
	0x11C37,
	0x11C38,
	0x11C40,
	0x11C41,
	0x11C46,
	0x11C50,
	0x11C5A,
	0x11C70,
	0x11C71,
	0x11C72,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12470,
	0x12475,
	0x13258,
	0x1325B,
	0x1325E,
	0x13282,
	0x13283,
	0x13286,
	0x13287,
	0x13288,
	0x13289,
	0x1328A,
	0x13379,
	0x1337A,
	0x1337C,
	0x145CE,
	0x145CF,
	0x145D0,
	0x16A60,
	0x16A6A,
	0x16A6E,
	0x16A70,
	0x16AF0,
	0x16AF5,
	0x16AF6,
	0x16B30,
	0x16B37,
	0x16B3A,
	0x16B44,
	0x16B45,
	0x16B50,
	0x16B5A,
	0x16F51,
	0x16F7F,
	0x16F8F,
	0x16F93,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC9D,
	0x1BC9F,
	0x1BCA0,
	0x1BCA4,
	0x1D165,
	0x1D16A,
	0x1D16D,
	0x1D183,
	0x1D185,
	0x1D18C,
	0x1D1AA,
	0x1D1AE,
	0x1D242,
	0x1D245,
	0x1D7CE,
	0x1D800,
	0x1DA00,
	0x1DA37,
	0x1DA3B,
	0x1DA6D,
	0x1DA75,
	0x1DA76,
	0x1DA84,
	0x1DA85,
	0x1DA87,
	0x1DA8B,
	0x1DA9B,
	0x1DAA0,
	0x1DAA1,
	0x1DAB0,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E8D0,
	0x1E8D7,
	0x1E944,
	0x1E94B,
	0x1E950,
	0x1E95A,
	0x1E95E,
	0x1E960,
	0x1F000,
	0x1F100,
	0x1F10D,
	0x1F110,
	0x1F12F,
	0x1F130,
	0x1F16C,
	0x1F170,
	0x1F1AD,
	0x1F1E6,
	0x1F200,
	0x1F385,
	0x1F386,
	0x1F39C,
	0x1F39E,
	0x1F3B5,
	0x1F3B7,
	0x1F3BC,
	0x1F3BD,
	0x1F3C3,
	0x1F3C5,
	0x1F3CA,
	0x1F3CC,
	0x1F3FB,
	0x1F400,
	0x1F442,
	0x1F444,
	0x1F446,
	0x1F451,
	0x1F466,
	0x1F46A,
	0x1F46E,
	0x1F46F,
	0x1F470,
	0x1F479,
	0x1F47C,
	0x1F47D,
	0x1F481,
	0x1F484,
	0x1F485,
	0x1F488,
	0x1F4A0,
	0x1F4A1,
	0x1F4A2,
	0x1F4A3,
	0x1F4A4,
	0x1F4A5,
	0x1F4AA,
	0x1F4AB,
	0x1F4AF,
	0x1F4B0,
	0x1F4B1,
	0x1F4B3,
	0x1F500,
	0x1F507,
	0x1F517,
	0x1F525,
	0x1F532,
	0x1F54A,
	0x1F575,
	0x1F576,
	0x1F57A,
	0x1F57B,
	0x1F590,
	0x1F591,
	0x1F595,
	0x1F597,
	0x1F5D4,
	0x1F5DC,
	0x1F5F4,
	0x1F5FA,
	0x1F645,
	0x1F648,
	0x1F64B,
	0x1F650,
	0x1F676,
	0x1F679,
	0x1F67C,
	0x1F680,
	0x1F6A3,
	0x1F6A4,
	0x1F6B4,
	0x1F6B7,
	0x1F6C0,
	0x1F6C1,
	0x1F700,
	0x1F774,
	0x1F780,
	0x1F7D5,
	0x1F800,
	0x1F80C,
	0x1F810,
	0x1F848,
	0x1F850,
	0x1F85A,
	0x1F860,
	0x1F888,
	0x1F890,
	0x1F8AE,
	0x1F918,
	0x1F91F,
	0x1F926,
	0x1F927,
	0x1F930,
	0x1F931,
	0x1F933,
	0x1F93A,
	0x1F93C,
	0x1F93F,
	0x1FFFE,
	0x20000,
	0x2FFFE,
	0x30000,
	0x3FFFE,
	0xE0001,
	0xE0002,
	0xE0020,
	0xE0080,
	0xE0100,
	0xE01F0
};

#endif	/* defined(PERL_IN_PERL_C) */

#if defined(PERL_IN_REGEXEC_C)

#define LB_ENUM_COUNT 39

typedef enum {
	LB_Alphabetic = 0,
	LB_Break_After = 1,
	LB_Break_Before = 2,
	LB_Break_Both = 3,
	LB_Break_Symbols = 4,
	LB_Carriage_Return = 5,
	LB_Close_Parenthesis = 6,
	LB_Close_Punctuation = 7,
	LB_Combining_Mark = 8,
	LB_Contingent_Break = 9,
	LB_E_Base = 10,
	LB_E_Modifier = 11,
	LB_Exclamation = 12,
	LB_Glue = 13,
	LB_H2 = 14,
	LB_H3 = 15,
	LB_Hebrew_Letter = 16,
	LB_Hyphen = 17,
	LB_Ideographic = 18,
	LB_Infix_Numeric = 19,
	LB_Inseparable = 20,
	LB_JL = 21,
	LB_JT = 22,
	LB_JV = 23,
	LB_Line_Feed = 24,
	LB_Mandatory_Break = 25,
	LB_Next_Line = 26,
	LB_Nonstarter = 27,
	LB_Numeric = 28,
	LB_Open_Punctuation = 29,
	LB_Postfix_Numeric = 30,
	LB_Prefix_Numeric = 31,
	LB_Quotation = 32,
	LB_Regional_Indicator = 33,
	LB_Space = 34,
	LB_Word_Joiner = 35,
	LB_ZWJ = 36,
	LB_ZWSpace = 37,
	LB_EDGE = 38
} LB_enum;

static const LB_enum _Perl_LB_invmap[] = { /* for EBCDIC 037 */
	LB_Combining_Mark,
	LB_Break_After,
	LB_Combining_Mark,
	LB_Mandatory_Break,
	LB_Carriage_Return,
	LB_Combining_Mark,
	LB_Next_Line,
	LB_Combining_Mark,
	LB_Line_Feed,
	LB_Combining_Mark,
	LB_Space,
	LB_Glue,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Infix_Numeric,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Prefix_Numeric,
	LB_Break_After,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Close_Parenthesis,
	LB_Infix_Numeric,
	LB_Alphabetic,
	LB_Hyphen,
	LB_Break_Symbols,
	LB_Alphabetic,
	LB_Infix_Numeric,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Infix_Numeric,
	LB_Alphabetic,
	LB_Quotation,
	LB_Alphabetic,
	LB_Quotation,
	LB_Alphabetic,
	LB_Quotation,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Parenthesis,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Glue,
	LB_Combining_Mark,
	LB_Glue,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Infix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Infix_Numeric,
	LB_Break_After,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Exclamation,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Hebrew_Letter,
	LB_Alphabetic,
	LB_Hebrew_Letter,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Infix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Exclamation,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Numeric,
	LB_Postfix_Numeric,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Infix_Numeric,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Glue,
	LB_Break_Before,
	LB_Break_After,
	LB_Glue,
	LB_Exclamation,
	LB_Glue,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_After,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Break_After,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Glue,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Numeric,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_JL,
	LB_JV,
	LB_JT,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Nonstarter,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Break_After,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Glue,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Numeric,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Numeric,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Numeric,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Break_After,
	LB_Glue,
	LB_Break_After,
	LB_ZWSpace,
	LB_Combining_Mark,
	LB_ZWJ,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Glue,
	LB_Break_After,
	LB_Break_Both,
	LB_Alphabetic,
	LB_Quotation,
	LB_Open_Punctuation,
	LB_Quotation,
	LB_Open_Punctuation,
	LB_Quotation,
	LB_Alphabetic,
	LB_Inseparable,
	LB_Break_After,
	LB_Mandatory_Break,
	LB_Combining_Mark,
	LB_Glue,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Quotation,
	LB_Alphabetic,
	LB_Nonstarter,
	LB_Alphabetic,
	LB_Infix_Numeric,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Nonstarter,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Word_Joiner,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Postfix_Numeric,
	LB_Prefix_Numeric,
	LB_Postfix_Numeric,
	LB_Prefix_Numeric,
	LB_Postfix_Numeric,
	LB_Prefix_Numeric,
	LB_Postfix_Numeric,
	LB_Prefix_Numeric,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Inseparable,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_E_Base,
	LB_Alphabetic,
	LB_Quotation,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Break_After,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Quotation,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Open_Punctuation,
	LB_Break_After,
	LB_Alphabetic,
	LB_Quotation,
	LB_Alphabetic,
	LB_Quotation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Break_After,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_Both,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Open_Punctuation,
	LB_Break_After,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Break_After,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Nonstarter,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Combining_Mark,
	LB_Ideographic,
	LB_Combining_Mark,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Exclamation,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_JL,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_H2,
	LB_H3,
	LB_Alphabetic,
	LB_JV,
	LB_Alphabetic,
	LB_JT,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Hebrew_Letter,
	LB_Combining_Mark,
	LB_Hebrew_Letter,
	LB_Alphabetic,
	LB_Hebrew_Letter,
	LB_Alphabetic,
	LB_Hebrew_Letter,
	LB_Alphabetic,
	LB_Hebrew_Letter,
	LB_Alphabetic,
	LB_Hebrew_Letter,
	LB_Alphabetic,
	LB_Hebrew_Letter,
	LB_Alphabetic,
	LB_Hebrew_Letter,
	LB_Alphabetic,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Infix_Numeric,
	LB_Close_Punctuation,
	LB_Infix_Numeric,
	LB_Exclamation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Inseparable,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Ideographic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Nonstarter,
	LB_Exclamation,
	LB_Ideographic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Prefix_Numeric,
	LB_Postfix_Numeric,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Word_Joiner,
	LB_Alphabetic,
	LB_Exclamation,
	LB_Ideographic,
	LB_Prefix_Numeric,
	LB_Postfix_Numeric,
	LB_Ideographic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Exclamation,
	LB_Ideographic,
	LB_Open_Punctuation,
	LB_Ideographic,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Open_Punctuation,
	LB_Ideographic,
	LB_Close_Punctuation,
	LB_Ideographic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Nonstarter,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Postfix_Numeric,
	LB_Prefix_Numeric,
	LB_Ideographic,
	LB_Prefix_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Contingent_Break,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Inseparable,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_Before,
	LB_Break_After,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_Before,
	LB_Exclamation,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Close_Punctuation,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Nonstarter,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Break_After,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Break_After,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Numeric,
	LB_Alphabetic,
	LB_Open_Punctuation,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Regional_Indicator,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Modifier,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Alphabetic,
	LB_Quotation,
	LB_Nonstarter,
	LB_Alphabetic,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_E_Base,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Ideographic,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic,
	LB_Combining_Mark,
	LB_Alphabetic
};

#endif	/* defined(PERL_IN_REGEXEC_C) */

#if defined(PERL_IN_PERL_C)

static const UV _Perl_SB_invlist[] = { /* for EBCDIC 037 */
	2977,	/* Number of elements */
	148565664, /* Version and data structure type */
	0,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x5,
	0x6,
	0xB,
	0xD,
	0xE,
	0x15,
	0x16,
	0x25,
	0x26,
	0x40,
	0x42,
	0x4A,
	0x4B,
	0x4C,
	0x4D,
	0x4E,
	0x51,
	0x5A,
	0x5B,
	0x5D,
	0x5E,
	0x60,
	0x61,
	0x62,
	0x6A,
	0x6B,
	0x6C,
	0x6F,
	0x70,
	0x71,
	0x79,
	0x7A,
	0x7B,
	0x7D,
	0x7E,
	0x7F,
	0x80,
	0x81,
	0x8A,
	0x8C,
	0x8F,
	0x91,
	0x9D,
	0x9E,
	0x9F,
	0xA0,
	0xA1,
	0xA2,
	0xAA,
	0xAC,
	0xAF,
	0xBA,
	0xBC,
	0xC0,
	0xC1,
	0xCA,
	0xCB,
	0xD0,
	0xD1,
	0xDA,
	0xDB,
	0xE0,
	0xE2,
	0xEA,
	0xEB,
	0xF0,
	0xFA,
	0xFB,
	0xFF,
	0x100,
	0x101,
	0x102,
	0x103,
	0x104,
	0x105,
	0x106,
	0x107,
	0x108,
	0x109,
	0x10A,
	0x10B,
	0x10C,
	0x10D,
	0x10E,
	0x10F,
	0x110,
	0x111,
	0x112,
	0x113,
	0x114,
	0x115,
	0x116,
	0x117,
	0x118,
	0x119,
	0x11A,
	0x11B,
	0x11C,
	0x11D,
	0x11E,
	0x11F,
	0x120,
	0x121,
	0x122,
	0x123,
	0x124,
	0x125,
	0x126,
	0x127,
	0x128,
	0x129,
	0x12A,
	0x12B,
	0x12C,
	0x12D,
	0x12E,
	0x12F,
	0x130,
	0x131,
	0x132,
	0x133,
	0x134,
	0x135,
	0x136,
	0x137,
	0x139,
	0x13A,
	0x13B,
	0x13C,
	0x13D,
	0x13E,
	0x13F,
	0x140,
	0x141,
	0x142,
	0x143,
	0x144,
	0x145,
	0x146,
	0x147,
	0x148,
	0x14A,
	0x14B,
	0x14C,
	0x14D,
	0x14E,
	0x14F,
	0x150,
	0x151,
	0x152,
	0x153,
	0x154,
	0x155,
	0x156,
	0x157,
	0x158,
	0x159,
	0x15A,
	0x15B,
	0x15C,
	0x15D,
	0x15E,
	0x15F,
	0x160,
	0x161,
	0x162,
	0x163,
	0x164,
	0x165,
	0x166,
	0x167,
	0x168,
	0x169,
	0x16A,
	0x16B,
	0x16C,
	0x16D,
	0x16E,
	0x16F,
	0x170,
	0x171,
	0x172,
	0x173,
	0x174,
	0x175,
	0x176,
	0x177,
	0x178,
	0x17A,
	0x17B,
	0x17C,
	0x17D,
	0x17E,
	0x181,
	0x183,
	0x184,
	0x185,
	0x186,
	0x188,
	0x189,
	0x18C,
	0x18E,
	0x192,
	0x193,
	0x195,
	0x196,
	0x199,
	0x19C,
	0x19E,
	0x19F,
	0x1A1,
	0x1A2,
	0x1A3,
	0x1A4,
	0x1A5,
	0x1A6,
	0x1A8,
	0x1A9,
	0x1AA,
	0x1AC,
	0x1AD,
	0x1AE,
	0x1B0,
	0x1B1,
	0x1B4,
	0x1B5,
	0x1B6,
	0x1B7,
	0x1B9,
	0x1BB,
	0x1BC,
	0x1BD,
	0x1C0,
	0x1C4,
	0x1C6,
	0x1C7,
	0x1C9,
	0x1CA,
	0x1CC,
	0x1CD,
	0x1CE,
	0x1CF,
	0x1D0,
	0x1D1,
	0x1D2,
	0x1D3,
	0x1D4,
	0x1D5,
	0x1D6,
	0x1D7,
	0x1D8,
	0x1D9,
	0x1DA,
	0x1DB,
	0x1DC,
	0x1DE,
	0x1DF,
	0x1E0,
	0x1E1,
	0x1E2,
	0x1E3,
	0x1E4,
	0x1E5,
	0x1E6,
	0x1E7,
	0x1E8,
	0x1E9,
	0x1EA,
	0x1EB,
	0x1EC,
	0x1ED,
	0x1EE,
	0x1EF,
	0x1F1,
	0x1F3,
	0x1F4,
	0x1F5,
	0x1F6,
	0x1F9,
	0x1FA,
	0x1FB,
	0x1FC,
	0x1FD,
	0x1FE,
	0x1FF,
	0x200,
	0x201,
	0x202,
	0x203,
	0x204,
	0x205,
	0x206,
	0x207,
	0x208,
	0x209,
	0x20A,
	0x20B,
	0x20C,
	0x20D,
	0x20E,
	0x20F,
	0x210,
	0x211,
	0x212,
	0x213,
	0x214,
	0x215,
	0x216,
	0x217,
	0x218,
	0x219,
	0x21A,
	0x21B,
	0x21C,
	0x21D,
	0x21E,
	0x21F,
	0x220,
	0x221,
	0x222,
	0x223,
	0x224,
	0x225,
	0x226,
	0x227,
	0x228,
	0x229,
	0x22A,
	0x22B,
	0x22C,
	0x22D,
	0x22E,
	0x22F,
	0x230,
	0x231,
	0x232,
	0x233,
	0x23A,
	0x23C,
	0x23D,
	0x23F,
	0x241,
	0x242,
	0x243,
	0x247,
	0x248,
	0x249,
	0x24A,
	0x24B,
	0x24C,
	0x24D,
	0x24E,
	0x24F,
	0x294,
	0x295,
	0x2B9,
	0x2C0,
	0x2C2,
	0x2C6,
	0x2D2,
	0x2E0,
	0x2E5,
	0x2EC,
	0x2ED,
	0x2EE,
	0x2EF,
	0x300,
	0x370,
	0x371,
	0x372,
	0x373,
	0x374,
	0x375,
	0x376,
	0x377,
	0x378,
	0x37A,
	0x37E,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x390,
	0x391,
	0x3A2,
	0x3A3,
	0x3AC,
	0x3CF,
	0x3D0,
	0x3D2,
	0x3D5,
	0x3D8,
	0x3D9,
	0x3DA,
	0x3DB,
	0x3DC,
	0x3DD,
	0x3DE,
	0x3DF,
	0x3E0,
	0x3E1,
	0x3E2,
	0x3E3,
	0x3E4,
	0x3E5,
	0x3E6,
	0x3E7,
	0x3E8,
	0x3E9,
	0x3EA,
	0x3EB,
	0x3EC,
	0x3ED,
	0x3EE,
	0x3EF,
	0x3F4,
	0x3F5,
	0x3F6,
	0x3F7,
	0x3F8,
	0x3F9,
	0x3FB,
	0x3FD,
	0x430,
	0x460,
	0x461,
	0x462,
	0x463,
	0x464,
	0x465,
	0x466,
	0x467,
	0x468,
	0x469,
	0x46A,
	0x46B,
	0x46C,
	0x46D,
	0x46E,
	0x46F,
	0x470,
	0x471,
	0x472,
	0x473,
	0x474,
	0x475,
	0x476,
	0x477,
	0x478,
	0x479,
	0x47A,
	0x47B,
	0x47C,
	0x47D,
	0x47E,
	0x47F,
	0x480,
	0x481,
	0x482,
	0x483,
	0x48A,
	0x48B,
	0x48C,
	0x48D,
	0x48E,
	0x48F,
	0x490,
	0x491,
	0x492,
	0x493,
	0x494,
	0x495,
	0x496,
	0x497,
	0x498,
	0x499,
	0x49A,
	0x49B,
	0x49C,
	0x49D,
	0x49E,
	0x49F,
	0x4A0,
	0x4A1,
	0x4A2,
	0x4A3,
	0x4A4,
	0x4A5,
	0x4A6,
	0x4A7,
	0x4A8,
	0x4A9,
	0x4AA,
	0x4AB,
	0x4AC,
	0x4AD,
	0x4AE,
	0x4AF,
	0x4B0,
	0x4B1,
	0x4B2,
	0x4B3,
	0x4B4,
	0x4B5,
	0x4B6,
	0x4B7,
	0x4B8,
	0x4B9,
	0x4BA,
	0x4BB,
	0x4BC,
	0x4BD,
	0x4BE,
	0x4BF,
	0x4C0,
	0x4C2,
	0x4C3,
	0x4C4,
	0x4C5,
	0x4C6,
	0x4C7,
	0x4C8,
	0x4C9,
	0x4CA,
	0x4CB,
	0x4CC,
	0x4CD,
	0x4CE,
	0x4D0,
	0x4D1,
	0x4D2,
	0x4D3,
	0x4D4,
	0x4D5,
	0x4D6,
	0x4D7,
	0x4D8,
	0x4D9,
	0x4DA,
	0x4DB,
	0x4DC,
	0x4DD,
	0x4DE,
	0x4DF,
	0x4E0,
	0x4E1,
	0x4E2,
	0x4E3,
	0x4E4,
	0x4E5,
	0x4E6,
	0x4E7,
	0x4E8,
	0x4E9,
	0x4EA,
	0x4EB,
	0x4EC,
	0x4ED,
	0x4EE,
	0x4EF,
	0x4F0,
	0x4F1,
	0x4F2,
	0x4F3,
	0x4F4,
	0x4F5,
	0x4F6,
	0x4F7,
	0x4F8,
	0x4F9,
	0x4FA,
	0x4FB,
	0x4FC,
	0x4FD,
	0x4FE,
	0x4FF,
	0x500,
	0x501,
	0x502,
	0x503,
	0x504,
	0x505,
	0x506,
	0x507,
	0x508,
	0x509,
	0x50A,
	0x50B,
	0x50C,
	0x50D,
	0x50E,
	0x50F,
	0x510,
	0x511,
	0x512,
	0x513,
	0x514,
	0x515,
	0x516,
	0x517,
	0x518,
	0x519,
	0x51A,
	0x51B,
	0x51C,
	0x51D,
	0x51E,
	0x51F,
	0x520,
	0x521,
	0x522,
	0x523,
	0x524,
	0x525,
	0x526,
	0x527,
	0x528,
	0x529,
	0x52A,
	0x52B,
	0x52C,
	0x52D,
	0x52E,
	0x52F,
	0x530,
	0x531,
	0x557,
	0x559,
	0x55A,
	0x55D,
	0x55E,
	0x561,
	0x588,
	0x589,
	0x58A,
	0x591,
	0x5BE,
	0x5BF,
	0x5C0,
	0x5C1,
	0x5C3,
	0x5C4,
	0x5C6,
	0x5C7,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F4,
	0x600,
	0x606,
	0x60C,
	0x60E,
	0x610,
	0x61B,
	0x61C,
	0x61D,
	0x61F,
	0x620,
	0x64B,
	0x660,
	0x66A,
	0x66B,
	0x66D,
	0x66E,
	0x670,
	0x671,
	0x6D4,
	0x6D5,
	0x6D6,
	0x6DD,
	0x6DE,
	0x6DF,
	0x6E5,
	0x6E7,
	0x6E9,
	0x6EA,
	0x6EE,
	0x6F0,
	0x6FA,
	0x6FD,
	0x6FF,
	0x700,
	0x703,
	0x70F,
	0x710,
	0x711,
	0x712,
	0x730,
	0x74B,
	0x74D,
	0x7A6,
	0x7B1,
	0x7B2,
	0x7C0,
	0x7CA,
	0x7EB,
	0x7F4,
	0x7F6,
	0x7F8,
	0x7F9,
	0x7FA,
	0x7FB,
	0x800,
	0x816,
	0x81A,
	0x81B,
	0x824,
	0x825,
	0x828,
	0x829,
	0x82E,
	0x840,
	0x859,
	0x85C,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x8D4,
	0x8E2,
	0x8E3,
	0x904,
	0x93A,
	0x93D,
	0x93E,
	0x950,
	0x951,
	0x958,
	0x962,
	0x964,
	0x966,
	0x970,
	0x971,
	0x981,
	0x984,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BC,
	0x9BD,
	0x9BE,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CE,
	0x9CF,
	0x9D7,
	0x9D8,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E2,
	0x9E4,
	0x9E6,
	0x9F0,
	0x9F2,
	0xA01,
	0xA04,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA3C,
	0xA3D,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4E,
	0xA51,
	0xA52,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA66,
	0xA70,
	0xA72,
	0xA75,
	0xA76,
	0xA81,
	0xA84,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABC,
	0xABD,
	0xABE,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACE,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE2,
	0xAE4,
	0xAE6,
	0xAF0,
	0xAF9,
	0xAFA,
	0xB01,
	0xB04,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3C,
	0xB3D,
	0xB3E,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4E,
	0xB56,
	0xB58,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB62,
	0xB64,
	0xB66,
	0xB70,
	0xB71,
	0xB72,
	0xB82,
	0xB83,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCE,
	0xBD0,
	0xBD1,
	0xBD7,
	0xBD8,
	0xBE6,
	0xBF0,
	0xC00,
	0xC04,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC3E,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4E,
	0xC55,
	0xC57,
	0xC58,
	0xC5B,
	0xC60,
	0xC62,
	0xC64,
	0xC66,
	0xC70,
	0xC80,
	0xC81,
	0xC84,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBC,
	0xCBD,
	0xCBE,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCE,
	0xCD5,
	0xCD7,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE2,
	0xCE4,
	0xCE6,
	0xCF0,
	0xCF1,
	0xCF3,
	0xD01,
	0xD04,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD3E,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD4E,
	0xD4F,
	0xD54,
	0xD57,
	0xD58,
	0xD5F,
	0xD62,
	0xD64,
	0xD66,
	0xD70,
	0xD7A,
	0xD80,
	0xD82,
	0xD84,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xDCA,
	0xDCB,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDE6,
	0xDF0,
	0xDF2,
	0xDF4,
	0xE01,
	0xE31,
	0xE32,
	0xE34,
	0xE3B,
	0xE40,
	0xE47,
	0xE4F,
	0xE50,
	0xE5A,
	0xE81,
	0xE83,
	0xE84,
	0xE85,
	0xE87,
	0xE89,
	0xE8A,
	0xE8B,
	0xE8D,
	0xE8E,
	0xE94,
	0xE98,
	0xE99,
	0xEA0,
	0xEA1,
	0xEA4,
	0xEA5,
	0xEA6,
	0xEA7,
	0xEA8,
	0xEAA,
	0xEAC,
	0xEAD,
	0xEB1,
	0xEB2,
	0xEB4,
	0xEBA,
	0xEBB,
	0xEBD,
	0xEBE,
	0xEC0,
	0xEC5,
	0xEC6,
	0xEC7,
	0xEC8,
	0xECE,
	0xED0,
	0xEDA,
	0xEDC,
	0xEE0,
	0xF00,
	0xF01,
	0xF18,
	0xF1A,
	0xF20,
	0xF2A,
	0xF35,
	0xF36,
	0xF37,
	0xF38,
	0xF39,
	0xF3A,
	0xF3E,
	0xF40,
	0xF48,
	0xF49,
	0xF6D,
	0xF71,
	0xF85,
	0xF86,
	0xF88,
	0xF8D,
	0xF98,
	0xF99,
	0xFBD,
	0xFC6,
	0xFC7,
	0x1000,
	0x102B,
	0x103F,
	0x1040,
	0x104A,
	0x104C,
	0x1050,
	0x1056,
	0x105A,
	0x105E,
	0x1061,
	0x1062,
	0x1065,
	0x1067,
	0x106E,
	0x1071,
	0x1075,
	0x1082,
	0x108E,
	0x108F,
	0x1090,
	0x109A,
	0x109E,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x10FB,
	0x10FC,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x135D,
	0x1360,
	0x1362,
	0x1363,
	0x1367,
	0x1369,
	0x1380,
	0x1390,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1401,
	0x166D,
	0x166E,
	0x166F,
	0x1680,
	0x1681,
	0x169B,
	0x169D,
	0x16A0,
	0x16EB,
	0x16EE,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1712,
	0x1715,
	0x1720,
	0x1732,
	0x1735,
	0x1737,
	0x1740,
	0x1752,
	0x1754,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1772,
	0x1774,
	0x1780,
	0x17B4,
	0x17D4,
	0x17D7,
	0x17D8,
	0x17DC,
	0x17DD,
	0x17DE,
	0x17E0,
	0x17EA,
	0x1802,
	0x1803,
	0x1804,
	0x1808,
	0x1809,
	0x180A,
	0x180B,
	0x180E,
	0x180F,
	0x1810,
	0x181A,
	0x1820,
	0x1878,
	0x1880,
	0x1885,
	0x1887,
	0x18A9,
	0x18AA,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1920,
	0x192C,
	0x1930,
	0x193C,
	0x1944,
	0x1946,
	0x1950,
	0x196E,
	0x1970,
	0x1975,
	0x1980,
	0x19AC,
	0x19B0,
	0x19CA,
	0x19D0,
	0x19DA,
	0x1A00,
	0x1A17,
	0x1A1C,
	0x1A20,
	0x1A55,
	0x1A5F,
	0x1A60,
	0x1A7D,
	0x1A7F,
	0x1A80,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1AA7,
	0x1AA8,
	0x1AAC,
	0x1AB0,
	0x1ABF,
	0x1B00,
	0x1B05,
	0x1B34,
	0x1B45,
	0x1B4C,
	0x1B50,
	0x1B5A,
	0x1B5C,
	0x1B5E,
	0x1B60,
	0x1B6B,
	0x1B74,
	0x1B80,
	0x1B83,
	0x1BA1,
	0x1BAE,
	0x1BB0,
	0x1BBA,
	0x1BE6,
	0x1BF4,
	0x1C00,
	0x1C24,
	0x1C38,
	0x1C3B,
	0x1C3D,
	0x1C40,
	0x1C4A,
	0x1C4D,
	0x1C50,
	0x1C5A,
	0x1C7E,
	0x1C80,
	0x1C89,
	0x1CD0,
	0x1CD3,
	0x1CD4,
	0x1CE9,
	0x1CED,
	0x1CEE,
	0x1CF2,
	0x1CF5,
	0x1CF7,
	0x1CF8,
	0x1CFA,
	0x1D00,
	0x1DC0,
	0x1DF6,
	0x1DFB,
	0x1E00,
	0x1E01,
	0x1E02,
	0x1E03,
	0x1E04,
	0x1E05,
	0x1E06,
	0x1E07,
	0x1E08,
	0x1E09,
	0x1E0A,
	0x1E0B,
	0x1E0C,
	0x1E0D,
	0x1E0E,
	0x1E0F,
	0x1E10,
	0x1E11,
	0x1E12,
	0x1E13,
	0x1E14,
	0x1E15,
	0x1E16,
	0x1E17,
	0x1E18,
	0x1E19,
	0x1E1A,
	0x1E1B,
	0x1E1C,
	0x1E1D,
	0x1E1E,
	0x1E1F,
	0x1E20,
	0x1E21,
	0x1E22,
	0x1E23,
	0x1E24,
	0x1E25,
	0x1E26,
	0x1E27,
	0x1E28,
	0x1E29,
	0x1E2A,
	0x1E2B,
	0x1E2C,
	0x1E2D,
	0x1E2E,
	0x1E2F,
	0x1E30,
	0x1E31,
	0x1E32,
	0x1E33,
	0x1E34,
	0x1E35,
	0x1E36,
	0x1E37,
	0x1E38,
	0x1E39,
	0x1E3A,
	0x1E3B,
	0x1E3C,
	0x1E3D,
	0x1E3E,
	0x1E3F,
	0x1E40,
	0x1E41,
	0x1E42,
	0x1E43,
	0x1E44,
	0x1E45,
	0x1E46,
	0x1E47,
	0x1E48,
	0x1E49,
	0x1E4A,
	0x1E4B,
	0x1E4C,
	0x1E4D,
	0x1E4E,
	0x1E4F,
	0x1E50,
	0x1E51,
	0x1E52,
	0x1E53,
	0x1E54,
	0x1E55,
	0x1E56,
	0x1E57,
	0x1E58,
	0x1E59,
	0x1E5A,
	0x1E5B,
	0x1E5C,
	0x1E5D,
	0x1E5E,
	0x1E5F,
	0x1E60,
	0x1E61,
	0x1E62,
	0x1E63,
	0x1E64,
	0x1E65,
	0x1E66,
	0x1E67,
	0x1E68,
	0x1E69,
	0x1E6A,
	0x1E6B,
	0x1E6C,
	0x1E6D,
	0x1E6E,
	0x1E6F,
	0x1E70,
	0x1E71,
	0x1E72,
	0x1E73,
	0x1E74,
	0x1E75,
	0x1E76,
	0x1E77,
	0x1E78,
	0x1E79,
	0x1E7A,
	0x1E7B,
	0x1E7C,
	0x1E7D,
	0x1E7E,
	0x1E7F,
	0x1E80,
	0x1E81,
	0x1E82,
	0x1E83,
	0x1E84,
	0x1E85,
	0x1E86,
	0x1E87,
	0x1E88,
	0x1E89,
	0x1E8A,
	0x1E8B,
	0x1E8C,
	0x1E8D,
	0x1E8E,
	0x1E8F,
	0x1E90,
	0x1E91,
	0x1E92,
	0x1E93,
	0x1E94,
	0x1E95,
	0x1E9E,
	0x1E9F,
	0x1EA0,
	0x1EA1,
	0x1EA2,
	0x1EA3,
	0x1EA4,
	0x1EA5,
	0x1EA6,
	0x1EA7,
	0x1EA8,
	0x1EA9,
	0x1EAA,
	0x1EAB,
	0x1EAC,
	0x1EAD,
	0x1EAE,
	0x1EAF,
	0x1EB0,
	0x1EB1,
	0x1EB2,
	0x1EB3,
	0x1EB4,
	0x1EB5,
	0x1EB6,
	0x1EB7,
	0x1EB8,
	0x1EB9,
	0x1EBA,
	0x1EBB,
	0x1EBC,
	0x1EBD,
	0x1EBE,
	0x1EBF,
	0x1EC0,
	0x1EC1,
	0x1EC2,
	0x1EC3,
	0x1EC4,
	0x1EC5,
	0x1EC6,
	0x1EC7,
	0x1EC8,
	0x1EC9,
	0x1ECA,
	0x1ECB,
	0x1ECC,
	0x1ECD,
	0x1ECE,
	0x1ECF,
	0x1ED0,
	0x1ED1,
	0x1ED2,
	0x1ED3,
	0x1ED4,
	0x1ED5,
	0x1ED6,
	0x1ED7,
	0x1ED8,
	0x1ED9,
	0x1EDA,
	0x1EDB,
	0x1EDC,
	0x1EDD,
	0x1EDE,
	0x1EDF,
	0x1EE0,
	0x1EE1,
	0x1EE2,
	0x1EE3,
	0x1EE4,
	0x1EE5,
	0x1EE6,
	0x1EE7,
	0x1EE8,
	0x1EE9,
	0x1EEA,
	0x1EEB,
	0x1EEC,
	0x1EED,
	0x1EEE,
	0x1EEF,
	0x1EF0,
	0x1EF1,
	0x1EF2,
	0x1EF3,
	0x1EF4,
	0x1EF5,
	0x1EF6,
	0x1EF7,
	0x1EF8,
	0x1EF9,
	0x1EFA,
	0x1EFB,
	0x1EFC,
	0x1EFD,
	0x1EFE,
	0x1EFF,
	0x1F08,
	0x1F10,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F28,
	0x1F30,
	0x1F38,
	0x1F40,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F60,
	0x1F68,
	0x1F70,
	0x1F7E,
	0x1F80,
	0x1F88,
	0x1F90,
	0x1F98,
	0x1FA0,
	0x1FA8,
	0x1FB0,
	0x1FB5,
	0x1FB6,
	0x1FB8,
	0x1FBD,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FC8,
	0x1FCD,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FD8,
	0x1FDC,
	0x1FE0,
	0x1FE8,
	0x1FED,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FF8,
	0x1FFD,
	0x2000,
	0x200B,
	0x200C,
	0x200E,
	0x2010,
	0x2013,
	0x2015,
	0x2018,
	0x2020,
	0x2024,
	0x2025,
	0x2028,
	0x202A,
	0x202F,
	0x2030,
	0x2039,
	0x203B,
	0x203C,
	0x203E,
	0x2045,
	0x2047,
	0x204A,
	0x205F,
	0x2060,
	0x2065,
	0x2066,
	0x2070,
	0x2071,
	0x2072,
	0x207D,
	0x207F,
	0x2080,
	0x208D,
	0x208F,
	0x2090,
	0x209D,
	0x20D0,
	0x20F1,
	0x2102,
	0x2103,
	0x2107,
	0x2108,
	0x210A,
	0x210B,
	0x210E,
	0x2110,
	0x2113,
	0x2114,
	0x2115,
	0x2116,
	0x2119,
	0x211E,
	0x2124,
	0x2125,
	0x2126,
	0x2127,
	0x2128,
	0x2129,
	0x212A,
	0x212E,
	0x212F,
	0x2130,
	0x2134,
	0x2135,
	0x2139,
	0x213A,
	0x213C,
	0x213E,
	0x2140,
	0x2145,
	0x2146,
	0x214A,
	0x214E,
	0x214F,
	0x2160,
	0x2170,
	0x2180,
	0x2183,
	0x2184,
	0x2185,
	0x2189,
	0x2308,
	0x230C,
	0x2329,
	0x232B,
	0x24B6,
	0x24D0,
	0x24EA,
	0x275B,
	0x2761,
	0x2768,
	0x2776,
	0x27C5,
	0x27C7,
	0x27E6,
	0x27F0,
	0x2983,
	0x2999,
	0x29D8,
	0x29DC,
	0x29FC,
	0x29FE,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2C61,
	0x2C62,
	0x2C65,
	0x2C67,
	0x2C68,
	0x2C69,
	0x2C6A,
	0x2C6B,
	0x2C6C,
	0x2C6D,
	0x2C71,
	0x2C72,
	0x2C73,
	0x2C75,
	0x2C76,
	0x2C7E,
	0x2C81,
	0x2C82,
	0x2C83,
	0x2C84,
	0x2C85,
	0x2C86,
	0x2C87,
	0x2C88,
	0x2C89,
	0x2C8A,
	0x2C8B,
	0x2C8C,
	0x2C8D,
	0x2C8E,
	0x2C8F,
	0x2C90,
	0x2C91,
	0x2C92,
	0x2C93,
	0x2C94,
	0x2C95,
	0x2C96,
	0x2C97,
	0x2C98,
	0x2C99,
	0x2C9A,
	0x2C9B,
	0x2C9C,
	0x2C9D,
	0x2C9E,
	0x2C9F,
	0x2CA0,
	0x2CA1,
	0x2CA2,
	0x2CA3,
	0x2CA4,
	0x2CA5,
	0x2CA6,
	0x2CA7,
	0x2CA8,
	0x2CA9,
	0x2CAA,
	0x2CAB,
	0x2CAC,
	0x2CAD,
	0x2CAE,
	0x2CAF,
	0x2CB0,
	0x2CB1,
	0x2CB2,
	0x2CB3,
	0x2CB4,
	0x2CB5,
	0x2CB6,
	0x2CB7,
	0x2CB8,
	0x2CB9,
	0x2CBA,
	0x2CBB,
	0x2CBC,
	0x2CBD,
	0x2CBE,
	0x2CBF,
	0x2CC0,
	0x2CC1,
	0x2CC2,
	0x2CC3,
	0x2CC4,
	0x2CC5,
	0x2CC6,
	0x2CC7,
	0x2CC8,
	0x2CC9,
	0x2CCA,
	0x2CCB,
	0x2CCC,
	0x2CCD,
	0x2CCE,
	0x2CCF,
	0x2CD0,
	0x2CD1,
	0x2CD2,
	0x2CD3,
	0x2CD4,
	0x2CD5,
	0x2CD6,
	0x2CD7,
	0x2CD8,
	0x2CD9,
	0x2CDA,
	0x2CDB,
	0x2CDC,
	0x2CDD,
	0x2CDE,
	0x2CDF,
	0x2CE0,
	0x2CE1,
	0x2CE2,
	0x2CE3,
	0x2CE5,
	0x2CEB,
	0x2CEC,
	0x2CED,
	0x2CEE,
	0x2CEF,
	0x2CF2,
	0x2CF3,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D70,
	0x2D7F,
	0x2D80,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x2DE0,
	0x2E00,
	0x2E0E,
	0x2E1C,
	0x2E1E,
	0x2E20,
	0x2E2A,
	0x2E2E,
	0x2E2F,
	0x2E30,
	0x2E3C,
	0x2E3D,
	0x2E42,
	0x2E43,
	0x3000,
	0x3001,
	0x3002,
	0x3003,
	0x3005,
	0x3008,
	0x3012,
	0x3014,
	0x301C,
	0x301D,
	0x3020,
	0x3021,
	0x302A,
	0x3030,
	0x3031,
	0x3036,
	0x3038,
	0x303D,
	0x3041,
	0x3097,
	0x3099,
	0x309B,
	0x309D,
	0x30A0,
	0x30A1,
	0x30FB,
	0x30FC,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x31A0,
	0x31BB,
	0x31F0,
	0x3200,
	0x3400,
	0x4DB6,
	0x4E00,
	0x9FD6,
	0xA000,
	0xA48D,
	0xA4D0,
	0xA4FE,
	0xA4FF,
	0xA500,
	0xA60D,
	0xA60E,
	0xA610,
	0xA620,
	0xA62A,
	0xA62C,
	0xA640,
	0xA641,
	0xA642,
	0xA643,
	0xA644,
	0xA645,
	0xA646,
	0xA647,
	0xA648,
	0xA649,
	0xA64A,
	0xA64B,
	0xA64C,
	0xA64D,
	0xA64E,
	0xA64F,
	0xA650,
	0xA651,
	0xA652,
	0xA653,
	0xA654,
	0xA655,
	0xA656,
	0xA657,
	0xA658,
	0xA659,
	0xA65A,
	0xA65B,
	0xA65C,
	0xA65D,
	0xA65E,
	0xA65F,
	0xA660,
	0xA661,
	0xA662,
	0xA663,
	0xA664,
	0xA665,
	0xA666,
	0xA667,
	0xA668,
	0xA669,
	0xA66A,
	0xA66B,
	0xA66C,
	0xA66D,
	0xA66E,
	0xA66F,
	0xA673,
	0xA674,
	0xA67E,
	0xA67F,
	0xA680,
	0xA681,
	0xA682,
	0xA683,
	0xA684,
	0xA685,
	0xA686,
	0xA687,
	0xA688,
	0xA689,
	0xA68A,
	0xA68B,
	0xA68C,
	0xA68D,
	0xA68E,
	0xA68F,
	0xA690,
	0xA691,
	0xA692,
	0xA693,
	0xA694,
	0xA695,
	0xA696,
	0xA697,
	0xA698,
	0xA699,
	0xA69A,
	0xA69B,
	0xA69E,
	0xA6A0,
	0xA6F0,
	0xA6F2,
	0xA6F3,
	0xA6F4,
	0xA6F7,
	0xA6F8,
	0xA717,
	0xA720,
	0xA722,
	0xA723,
	0xA724,
	0xA725,
	0xA726,
	0xA727,
	0xA728,
	0xA729,
	0xA72A,
	0xA72B,
	0xA72C,
	0xA72D,
	0xA72E,
	0xA72F,
	0xA732,
	0xA733,
	0xA734,
	0xA735,
	0xA736,
	0xA737,
	0xA738,
	0xA739,
	0xA73A,
	0xA73B,
	0xA73C,
	0xA73D,
	0xA73E,
	0xA73F,
	0xA740,
	0xA741,
	0xA742,
	0xA743,
	0xA744,
	0xA745,
	0xA746,
	0xA747,
	0xA748,
	0xA749,
	0xA74A,
	0xA74B,
	0xA74C,
	0xA74D,
	0xA74E,
	0xA74F,
	0xA750,
	0xA751,
	0xA752,
	0xA753,
	0xA754,
	0xA755,
	0xA756,
	0xA757,
	0xA758,
	0xA759,
	0xA75A,
	0xA75B,
	0xA75C,
	0xA75D,
	0xA75E,
	0xA75F,
	0xA760,
	0xA761,
	0xA762,
	0xA763,
	0xA764,
	0xA765,
	0xA766,
	0xA767,
	0xA768,
	0xA769,
	0xA76A,
	0xA76B,
	0xA76C,
	0xA76D,
	0xA76E,
	0xA76F,
	0xA779,
	0xA77A,
	0xA77B,
	0xA77C,
	0xA77D,
	0xA77F,
	0xA780,
	0xA781,
	0xA782,
	0xA783,
	0xA784,
	0xA785,
	0xA786,
	0xA787,
	0xA788,
	0xA789,
	0xA78B,
	0xA78C,
	0xA78D,
	0xA78E,
	0xA78F,
	0xA790,
	0xA791,
	0xA792,
	0xA793,
	0xA796,
	0xA797,
	0xA798,
	0xA799,
	0xA79A,
	0xA79B,
	0xA79C,
	0xA79D,
	0xA79E,
	0xA79F,
	0xA7A0,
	0xA7A1,
	0xA7A2,
	0xA7A3,
	0xA7A4,
	0xA7A5,
	0xA7A6,
	0xA7A7,
	0xA7A8,
	0xA7A9,
	0xA7AA,
	0xA7AF,
	0xA7B0,
	0xA7B5,
	0xA7B6,
	0xA7B7,
	0xA7B8,
	0xA7F7,
	0xA7F8,
	0xA7FB,
	0xA802,
	0xA803,
	0xA806,
	0xA807,
	0xA80B,
	0xA80C,
	0xA823,
	0xA828,
	0xA840,
	0xA874,
	0xA876,
	0xA878,
	0xA880,
	0xA882,
	0xA8B4,
	0xA8C6,
	0xA8CE,
	0xA8D0,
	0xA8DA,
	0xA8E0,
	0xA8F2,
	0xA8F8,
	0xA8FB,
	0xA8FC,
	0xA8FD,
	0xA8FE,
	0xA900,
	0xA90A,
	0xA926,
	0xA92E,
	0xA92F,
	0xA930,
	0xA947,
	0xA954,
	0xA960,
	0xA97D,
	0xA980,
	0xA984,
	0xA9B3,
	0xA9C1,
	0xA9C8,
	0xA9CA,
	0xA9CF,
	0xA9D0,
	0xA9DA,
	0xA9E0,
	0xA9E5,
	0xA9E6,
	0xA9F0,
	0xA9FA,
	0xA9FF,
	0xAA00,
	0xAA29,
	0xAA37,
	0xAA40,
	0xAA43,
	0xAA44,
	0xAA4C,
	0xAA4E,
	0xAA50,
	0xAA5A,
	0xAA5D,
	0xAA60,
	0xAA77,
	0xAA7A,
	0xAA7B,
	0xAA7E,
	0xAAB0,
	0xAAB1,
	0xAAB2,
	0xAAB5,
	0xAAB7,
	0xAAB9,
	0xAABE,
	0xAAC0,
	0xAAC1,
	0xAAC2,
	0xAAC3,
	0xAADB,
	0xAADE,
	0xAAE0,
	0xAAEB,
	0xAAF0,
	0xAAF2,
	0xAAF5,
	0xAAF7,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB5B,
	0xAB5C,
	0xAB66,
	0xAB70,
	0xABC0,
	0xABE3,
	0xABEB,
	0xABEC,
	0xABEE,
	0xABF0,
	0xABFA,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xF900,
	0xFA6E,
	0xFA70,
	0xFADA,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB1E,
	0xFB1F,
	0xFB29,
	0xFB2A,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFBB2,
	0xFBD3,
	0xFD3E,
	0xFD40,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFC,
	0xFE00,
	0xFE10,
	0xFE12,
	0xFE13,
	0xFE14,
	0xFE17,
	0xFE19,
	0xFE20,
	0xFE30,
	0xFE31,
	0xFE33,
	0xFE35,
	0xFE45,
	0xFE47,
	0xFE49,
	0xFE50,
	0xFE52,
	0xFE53,
	0xFE55,
	0xFE56,
	0xFE58,
	0xFE59,
	0xFE5F,
	0xFE63,
	0xFE64,
	0xFE70,
	0xFE75,
	0xFE76,
	0xFEFD,
	0xFEFF,
	0xFF00,
	0xFF01,
	0xFF02,
	0xFF08,
	0xFF0A,
	0xFF0C,
	0xFF0E,
	0xFF0F,
	0xFF1A,
	0xFF1B,
	0xFF1F,
	0xFF20,
	0xFF21,
	0xFF3B,
	0xFF3C,
	0xFF3D,
	0xFF3E,
	0xFF41,
	0xFF5B,
	0xFF5C,
	0xFF5D,
	0xFF5E,
	0xFF5F,
	0xFF61,
	0xFF62,
	0xFF64,
	0xFF65,
	0xFF66,
	0xFF9E,
	0xFFA0,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0xFFF9,
	0xFFFC,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10140,
	0x10175,
	0x101FD,
	0x101FE,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x102E0,
	0x102E1,
	0x10300,
	0x10320,
	0x10330,
	0x1034B,
	0x10350,
	0x10376,
	0x1037B,
	0x10380,
	0x1039E,
	0x103A0,
	0x103C4,
	0x103C8,
	0x103D0,
	0x103D1,
	0x103D6,
	0x10400,
	0x10428,
	0x10450,
	0x1049E,
	0x104A0,
	0x104AA,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10860,
	0x10877,
	0x10880,
	0x1089F,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x10900,
	0x10916,
	0x10920,
	0x1093A,
	0x10980,
	0x109B8,
	0x109BE,
	0x109C0,
	0x10A00,
	0x10A01,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A10,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A38,
	0x10A3B,
	0x10A3F,
	0x10A40,
	0x10A56,
	0x10A58,
	0x10A60,
	0x10A7D,
	0x10A80,
	0x10A9D,
	0x10AC0,
	0x10AC8,
	0x10AC9,
	0x10AE5,
	0x10AE7,
	0x10B00,
	0x10B36,
	0x10B40,
	0x10B56,
	0x10B60,
	0x10B73,
	0x10B80,
	0x10B92,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x11000,
	0x11003,
	0x11038,
	0x11047,
	0x11049,
	0x11066,
	0x11070,
	0x1107F,
	0x11083,
	0x110B0,
	0x110BB,
	0x110BD,
	0x110BE,
	0x110C2,
	0x110D0,
	0x110E9,
	0x110F0,
	0x110FA,
	0x11100,
	0x11103,
	0x11127,
	0x11135,
	0x11136,
	0x11140,
	0x11141,
	0x11144,
	0x11150,
	0x11173,
	0x11174,
	0x11176,
	0x11177,
	0x11180,
	0x11183,
	0x111B3,
	0x111C1,
	0x111C5,
	0x111C7,
	0x111CA,
	0x111CD,
	0x111CE,
	0x111D0,
	0x111DA,
	0x111DB,
	0x111DC,
	0x111DD,
	0x111DE,
	0x111E0,
	0x11200,
	0x11212,
	0x11213,
	0x1122C,
	0x11238,
	0x1123A,
	0x1123B,
	0x1123D,
	0x1123E,
	0x1123F,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112A9,
	0x112AA,
	0x112B0,
	0x112DF,
	0x112EB,
	0x112F0,
	0x112FA,
	0x11300,
	0x11304,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133C,
	0x1133D,
	0x1133E,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134E,
	0x11350,
	0x11351,
	0x11357,
	0x11358,
	0x1135D,
	0x11362,
	0x11364,
	0x11366,
	0x1136D,
	0x11370,
	0x11375,
	0x11400,
	0x11435,
	0x11447,
	0x1144B,
	0x1144D,
	0x11450,
	0x1145A,
	0x11480,
	0x114B0,
	0x114C4,
	0x114C6,
	0x114C7,
	0x114C8,
	0x114D0,
	0x114DA,
	0x11580,
	0x115AF,
	0x115B6,
	0x115B8,
	0x115C1,
	0x115C2,
	0x115C4,
	0x115C9,
	0x115D8,
	0x115DC,
	0x115DE,
	0x11600,
	0x11630,
	0x11641,
	0x11643,
	0x11644,
	0x11645,
	0x11650,
	0x1165A,
	0x11680,
	0x116AB,
	0x116B8,
	0x116C0,
	0x116CA,
	0x11700,
	0x1171A,
	0x1171D,
	0x1172C,
	0x11730,
	0x1173A,
	0x1173C,
	0x1173F,
	0x118A0,
	0x118C0,
	0x118E0,
	0x118EA,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C2F,
	0x11C37,
	0x11C38,
	0x11C40,
	0x11C41,
	0x11C43,
	0x11C50,
	0x11C5A,
	0x11C72,
	0x11C90,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16A60,
	0x16A6A,
	0x16A6E,
	0x16A70,
	0x16AD0,
	0x16AEE,
	0x16AF0,
	0x16AF5,
	0x16AF6,
	0x16B00,
	0x16B30,
	0x16B37,
	0x16B39,
	0x16B40,
	0x16B44,
	0x16B45,
	0x16B50,
	0x16B5A,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F51,
	0x16F7F,
	0x16F8F,
	0x16F93,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1BC9D,
	0x1BC9F,
	0x1BCA0,
	0x1BCA4,
	0x1D165,
	0x1D16A,
	0x1D16D,
	0x1D173,
	0x1D17B,
	0x1D183,
	0x1D185,
	0x1D18C,
	0x1D1AA,
	0x1D1AE,
	0x1D242,
	0x1D245,
	0x1D400,
	0x1D41A,
	0x1D434,
	0x1D44E,
	0x1D455,
	0x1D456,
	0x1D468,
	0x1D482,
	0x1D49C,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4B6,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D4D0,
	0x1D4EA,
	0x1D504,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D538,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D56C,
	0x1D586,
	0x1D5A0,
	0x1D5BA,
	0x1D5D4,
	0x1D5EE,
	0x1D608,
	0x1D622,
	0x1D63C,
	0x1D656,
	0x1D670,
	0x1D68A,
	0x1D6A6,
	0x1D6A8,
	0x1D6C1,
	0x1D6C2,
	0x1D6DB,
	0x1D6DC,
	0x1D6E2,
	0x1D6FB,
	0x1D6FC,
	0x1D715,
	0x1D716,
	0x1D71C,
	0x1D735,
	0x1D736,
	0x1D74F,
	0x1D750,
	0x1D756,
	0x1D76F,
	0x1D770,
	0x1D789,
	0x1D78A,
	0x1D790,
	0x1D7A9,
	0x1D7AA,
	0x1D7C3,
	0x1D7C4,
	0x1D7CA,
	0x1D7CB,
	0x1D7CC,
	0x1D7CE,
	0x1D800,
	0x1DA00,
	0x1DA37,
	0x1DA3B,
	0x1DA6D,
	0x1DA75,
	0x1DA76,
	0x1DA84,
	0x1DA85,
	0x1DA88,
	0x1DA89,
	0x1DA9B,
	0x1DAA0,
	0x1DAA1,
	0x1DAB0,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E800,
	0x1E8C5,
	0x1E8D0,
	0x1E8D7,
	0x1E900,
	0x1E922,
	0x1E944,
	0x1E94B,
	0x1E950,
	0x1E95A,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x1F130,
	0x1F14A,
	0x1F150,
	0x1F16A,
	0x1F170,
	0x1F18A,
	0x1F676,
	0x1F679,
	0x20000,
	0x2A6D7,
	0x2A700,
	0x2B735,
	0x2B740,
	0x2B81E,
	0x2B820,
	0x2CEA2,
	0x2F800,
	0x2FA1E,
	0xE0001,
	0xE0002,
	0xE0020,
	0xE0080,
	0xE0100,
	0xE01F0
};

#endif	/* defined(PERL_IN_PERL_C) */

#if defined(PERL_IN_REGEXEC_C)

#define SB_ENUM_COUNT 16

typedef enum {
	SB_Other = 0,
	SB_ATerm = 1,
	SB_CR = 2,
	SB_Close = 3,
	SB_Extend = 4,
	SB_Format = 5,
	SB_LF = 6,
	SB_Lower = 7,
	SB_Numeric = 8,
	SB_OLetter = 9,
	SB_SContinue = 10,
	SB_STerm = 11,
	SB_Sep = 12,
	SB_Sp = 13,
	SB_Upper = 14,
	SB_EDGE = 15
} SB_enum;

static const SB_enum _Perl_SB_invmap[] = { /* for EBCDIC 037 */
	SB_Other,
	SB_Sp,
	SB_Other,
	SB_Sp,
	SB_CR,
	SB_Other,
	SB_Sep,
	SB_Other,
	SB_LF,
	SB_Other,
	SB_Sp,
	SB_Lower,
	SB_Other,
	SB_ATerm,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Lower,
	SB_STerm,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_SContinue,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_SContinue,
	SB_Other,
	SB_STerm,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_SContinue,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Upper,
	SB_Lower,
	SB_Close,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Upper,
	SB_Format,
	SB_Lower,
	SB_Close,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Numeric,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Lower,
	SB_OLetter,
	SB_Lower,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Extend,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_SContinue,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Format,
	SB_Other,
	SB_SContinue,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Format,
	SB_Other,
	SB_STerm,
	SB_OLetter,
	SB_Extend,
	SB_Numeric,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_STerm,
	SB_OLetter,
	SB_Extend,
	SB_Format,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Numeric,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_STerm,
	SB_Other,
	SB_Format,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_SContinue,
	SB_STerm,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Format,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_STerm,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Close,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Numeric,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Numeric,
	SB_Extend,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_STerm,
	SB_OLetter,
	SB_Sp,
	SB_OLetter,
	SB_Close,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_SContinue,
	SB_STerm,
	SB_Other,
	SB_SContinue,
	SB_STerm,
	SB_Other,
	SB_Extend,
	SB_Format,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_Numeric,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Numeric,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_STerm,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_STerm,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Numeric,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Numeric,
	SB_OLetter,
	SB_STerm,
	SB_Lower,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Lower,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Sp,
	SB_Format,
	SB_Extend,
	SB_Format,
	SB_Other,
	SB_SContinue,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_ATerm,
	SB_Other,
	SB_Sep,
	SB_Format,
	SB_Sp,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Close,
	SB_STerm,
	SB_Other,
	SB_Sp,
	SB_Format,
	SB_Other,
	SB_Format,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Close,
	SB_Lower,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Extend,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_STerm,
	SB_OLetter,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Sp,
	SB_SContinue,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_STerm,
	SB_OLetter,
	SB_Other,
	SB_STerm,
	SB_OLetter,
	SB_Numeric,
	SB_OLetter,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_OLetter,
	SB_Lower,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_Numeric,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Numeric,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_STerm,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_STerm,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_OLetter,
	SB_Extend,
	SB_STerm,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Close,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_SContinue,
	SB_Other,
	SB_SContinue,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_SContinue,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_SContinue,
	SB_ATerm,
	SB_Other,
	SB_SContinue,
	SB_STerm,
	SB_SContinue,
	SB_Close,
	SB_Other,
	SB_SContinue,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Format,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_SContinue,
	SB_ATerm,
	SB_Other,
	SB_SContinue,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Upper,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Lower,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_Close,
	SB_STerm,
	SB_Close,
	SB_SContinue,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Format,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_STerm,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Format,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_STerm,
	SB_Other,
	SB_Extend,
	SB_STerm,
	SB_Other,
	SB_Numeric,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_STerm,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_STerm,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_STerm,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_STerm,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_STerm,
	SB_Other,
	SB_OLetter,
	SB_STerm,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_STerm,
	SB_Format,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Format,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Other,
	SB_Lower,
	SB_Other,
	SB_Lower,
	SB_Upper,
	SB_Lower,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_STerm,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Upper,
	SB_Lower,
	SB_Extend,
	SB_Other,
	SB_Numeric,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Upper,
	SB_Other,
	SB_Close,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_OLetter,
	SB_Other,
	SB_Format,
	SB_Other,
	SB_Extend,
	SB_Other,
	SB_Extend,
	SB_Other
};

#endif	/* defined(PERL_IN_REGEXEC_C) */

#if defined(PERL_IN_PERL_C)

static const UV _Perl_WB_invlist[] = { /* for EBCDIC 037 */
	1676,	/* Number of elements */
	148565664, /* Version and data structure type */
	0,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x5,
	0x6,
	0xB,
	0xD,
	0xE,
	0x15,
	0x16,
	0x25,
	0x26,
	0x40,
	0x42,
	0x4A,
	0x4B,
	0x4C,
	0x51,
	0x5A,
	0x5E,
	0x5F,
	0x62,
	0x6A,
	0x6B,
	0x6C,
	0x6D,
	0x6E,
	0x70,
	0x79,
	0x7A,
	0x7B,
	0x7D,
	0x7E,
	0x7F,
	0x80,
	0x8A,
	0x8C,
	0x8F,
	0x91,
	0x9D,
	0x9E,
	0x9F,
	0xA0,
	0xA1,
	0xA2,
	0xAA,
	0xAC,
	0xAF,
	0xB3,
	0xB4,
	0xC1,
	0xCA,
	0xCB,
	0xD0,
	0xD1,
	0xDA,
	0xDB,
	0xE0,
	0xE2,
	0xEA,
	0xEB,
	0xF0,
	0xFA,
	0xFB,
	0xFF,
	0x100,
	0x2C2,
	0x2C6,
	0x2D2,
	0x2D7,
	0x2D8,
	0x2E0,
	0x2E5,
	0x2EC,
	0x2ED,
	0x2EE,
	0x2EF,
	0x300,
	0x370,
	0x375,
	0x376,
	0x378,
	0x37A,
	0x37E,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x3F6,
	0x3F7,
	0x482,
	0x483,
	0x48A,
	0x530,
	0x531,
	0x557,
	0x559,
	0x55A,
	0x561,
	0x588,
	0x589,
	0x58A,
	0x591,
	0x5BE,
	0x5BF,
	0x5C0,
	0x5C1,
	0x5C3,
	0x5C4,
	0x5C6,
	0x5C7,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F3,
	0x5F4,
	0x5F5,
	0x600,
	0x606,
	0x60C,
	0x60E,
	0x610,
	0x61B,
	0x61C,
	0x61D,
	0x620,
	0x64B,
	0x660,
	0x66A,
	0x66B,
	0x66C,
	0x66D,
	0x66E,
	0x670,
	0x671,
	0x6D4,
	0x6D5,
	0x6D6,
	0x6DD,
	0x6DE,
	0x6DF,
	0x6E5,
	0x6E7,
	0x6E9,
	0x6EA,
	0x6EE,
	0x6F0,
	0x6FA,
	0x6FD,
	0x6FF,
	0x700,
	0x70F,
	0x710,
	0x711,
	0x712,
	0x730,
	0x74B,
	0x74D,
	0x7A6,
	0x7B1,
	0x7B2,
	0x7C0,
	0x7CA,
	0x7EB,
	0x7F4,
	0x7F6,
	0x7F8,
	0x7F9,
	0x7FA,
	0x7FB,
	0x800,
	0x816,
	0x81A,
	0x81B,
	0x824,
	0x825,
	0x828,
	0x829,
	0x82E,
	0x840,
	0x859,
	0x85C,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x8D4,
	0x8E2,
	0x8E3,
	0x904,
	0x93A,
	0x93D,
	0x93E,
	0x950,
	0x951,
	0x958,
	0x962,
	0x964,
	0x966,
	0x970,
	0x971,
	0x981,
	0x984,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BC,
	0x9BD,
	0x9BE,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CE,
	0x9CF,
	0x9D7,
	0x9D8,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E2,
	0x9E4,
	0x9E6,
	0x9F0,
	0x9F2,
	0xA01,
	0xA04,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA3C,
	0xA3D,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4E,
	0xA51,
	0xA52,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA66,
	0xA70,
	0xA72,
	0xA75,
	0xA76,
	0xA81,
	0xA84,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABC,
	0xABD,
	0xABE,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACE,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE2,
	0xAE4,
	0xAE6,
	0xAF0,
	0xAF9,
	0xAFA,
	0xB01,
	0xB04,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3C,
	0xB3D,
	0xB3E,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4E,
	0xB56,
	0xB58,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB62,
	0xB64,
	0xB66,
	0xB70,
	0xB71,
	0xB72,
	0xB82,
	0xB83,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCE,
	0xBD0,
	0xBD1,
	0xBD7,
	0xBD8,
	0xBE6,
	0xBF0,
	0xC00,
	0xC04,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC3E,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4E,
	0xC55,
	0xC57,
	0xC58,
	0xC5B,
	0xC60,
	0xC62,
	0xC64,
	0xC66,
	0xC70,
	0xC80,
	0xC81,
	0xC84,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBC,
	0xCBD,
	0xCBE,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCE,
	0xCD5,
	0xCD7,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE2,
	0xCE4,
	0xCE6,
	0xCF0,
	0xCF1,
	0xCF3,
	0xD01,
	0xD04,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD3E,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD4E,
	0xD4F,
	0xD54,
	0xD57,
	0xD58,
	0xD5F,
	0xD62,
	0xD64,
	0xD66,
	0xD70,
	0xD7A,
	0xD80,
	0xD82,
	0xD84,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xDCA,
	0xDCB,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDE6,
	0xDF0,
	0xDF2,
	0xDF4,
	0xE31,
	0xE32,
	0xE34,
	0xE3B,
	0xE47,
	0xE4F,
	0xE50,
	0xE5A,
	0xEB1,
	0xEB2,
	0xEB4,
	0xEBA,
	0xEBB,
	0xEBD,
	0xEC8,
	0xECE,
	0xED0,
	0xEDA,
	0xF00,
	0xF01,
	0xF18,
	0xF1A,
	0xF20,
	0xF2A,
	0xF35,
	0xF36,
	0xF37,
	0xF38,
	0xF39,
	0xF3A,
	0xF3E,
	0xF40,
	0xF48,
	0xF49,
	0xF6D,
	0xF71,
	0xF85,
	0xF86,
	0xF88,
	0xF8D,
	0xF98,
	0xF99,
	0xFBD,
	0xFC6,
	0xFC7,
	0x102B,
	0x103F,
	0x1040,
	0x104A,
	0x1056,
	0x105A,
	0x105E,
	0x1061,
	0x1062,
	0x1065,
	0x1067,
	0x106E,
	0x1071,
	0x1075,
	0x1082,
	0x108E,
	0x108F,
	0x1090,
	0x109A,
	0x109E,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x10FB,
	0x10FC,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x135D,
	0x1360,
	0x1380,
	0x1390,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1401,
	0x166D,
	0x166F,
	0x1680,
	0x1681,
	0x169B,
	0x16A0,
	0x16EB,
	0x16EE,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1712,
	0x1715,
	0x1720,
	0x1732,
	0x1735,
	0x1740,
	0x1752,
	0x1754,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1772,
	0x1774,
	0x17B4,
	0x17D4,
	0x17DD,
	0x17DE,
	0x17E0,
	0x17EA,
	0x180B,
	0x180E,
	0x180F,
	0x1810,
	0x181A,
	0x1820,
	0x1878,
	0x1880,
	0x1885,
	0x1887,
	0x18A9,
	0x18AA,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1920,
	0x192C,
	0x1930,
	0x193C,
	0x1946,
	0x1950,
	0x19D0,
	0x19DA,
	0x1A00,
	0x1A17,
	0x1A1C,
	0x1A55,
	0x1A5F,
	0x1A60,
	0x1A7D,
	0x1A7F,
	0x1A80,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1AB0,
	0x1ABF,
	0x1B00,
	0x1B05,
	0x1B34,
	0x1B45,
	0x1B4C,
	0x1B50,
	0x1B5A,
	0x1B6B,
	0x1B74,
	0x1B80,
	0x1B83,
	0x1BA1,
	0x1BAE,
	0x1BB0,
	0x1BBA,
	0x1BE6,
	0x1BF4,
	0x1C00,
	0x1C24,
	0x1C38,
	0x1C40,
	0x1C4A,
	0x1C4D,
	0x1C50,
	0x1C5A,
	0x1C7E,
	0x1C80,
	0x1C89,
	0x1CD0,
	0x1CD3,
	0x1CD4,
	0x1CE9,
	0x1CED,
	0x1CEE,
	0x1CF2,
	0x1CF5,
	0x1CF7,
	0x1CF8,
	0x1CFA,
	0x1D00,
	0x1DC0,
	0x1DF6,
	0x1DFB,
	0x1E00,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FBD,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FCD,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FE0,
	0x1FED,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFD,
	0x2000,
	0x200B,
	0x200C,
	0x200D,
	0x200E,
	0x2010,
	0x2018,
	0x201A,
	0x2024,
	0x2025,
	0x2027,
	0x2028,
	0x202A,
	0x202F,
	0x2030,
	0x203F,
	0x2041,
	0x2044,
	0x2045,
	0x2054,
	0x2055,
	0x205F,
	0x2060,
	0x2065,
	0x2066,
	0x2070,
	0x2071,
	0x2072,
	0x207F,
	0x2080,
	0x2090,
	0x209D,
	0x20D0,
	0x20F1,
	0x2102,
	0x2103,
	0x2107,
	0x2108,
	0x210A,
	0x2114,
	0x2115,
	0x2116,
	0x2119,
	0x211E,
	0x2124,
	0x2125,
	0x2126,
	0x2127,
	0x2128,
	0x2129,
	0x212A,
	0x212E,
	0x212F,
	0x213A,
	0x213C,
	0x2140,
	0x2145,
	0x214A,
	0x214E,
	0x214F,
	0x2160,
	0x2189,
	0x24B6,
	0x24EA,
	0x261D,
	0x261E,
	0x26F9,
	0x26FA,
	0x270A,
	0x270E,
	0x2764,
	0x2765,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CE5,
	0x2CEB,
	0x2CEF,
	0x2CF2,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D70,
	0x2D7F,
	0x2D80,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x2DE0,
	0x2E00,
	0x2E2F,
	0x2E30,
	0x3000,
	0x3001,
	0x3005,
	0x3006,
	0x302A,
	0x3030,
	0x3031,
	0x3036,
	0x303B,
	0x303D,
	0x3099,
	0x309B,
	0x309D,
	0x30A0,
	0x30FB,
	0x30FC,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x31A0,
	0x31BB,
	0x31F0,
	0x3200,
	0x32D0,
	0x32FF,
	0x3300,
	0x3358,
	0xA000,
	0xA48D,
	0xA4D0,
	0xA4FE,
	0xA500,
	0xA60D,
	0xA610,
	0xA620,
	0xA62A,
	0xA62C,
	0xA640,
	0xA66F,
	0xA673,
	0xA674,
	0xA67E,
	0xA67F,
	0xA69E,
	0xA6A0,
	0xA6F0,
	0xA6F2,
	0xA717,
	0xA720,
	0xA722,
	0xA789,
	0xA78B,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F7,
	0xA802,
	0xA803,
	0xA806,
	0xA807,
	0xA80B,
	0xA80C,
	0xA823,
	0xA828,
	0xA840,
	0xA874,
	0xA880,
	0xA882,
	0xA8B4,
	0xA8C6,
	0xA8D0,
	0xA8DA,
	0xA8E0,
	0xA8F2,
	0xA8F8,
	0xA8FB,
	0xA8FC,
	0xA8FD,
	0xA8FE,
	0xA900,
	0xA90A,
	0xA926,
	0xA92E,
	0xA930,
	0xA947,
	0xA954,
	0xA960,
	0xA97D,
	0xA980,
	0xA984,
	0xA9B3,
	0xA9C1,
	0xA9CF,
	0xA9D0,
	0xA9DA,
	0xA9E5,
	0xA9E6,
	0xA9F0,
	0xA9FA,
	0xAA00,
	0xAA29,
	0xAA37,
	0xAA40,
	0xAA43,
	0xAA44,
	0xAA4C,
	0xAA4E,
	0xAA50,
	0xAA5A,
	0xAA7B,
	0xAA7E,
	0xAAB0,
	0xAAB1,
	0xAAB2,
	0xAAB5,
	0xAAB7,
	0xAAB9,
	0xAABE,
	0xAAC0,
	0xAAC1,
	0xAAC2,
	0xAAE0,
	0xAAEB,
	0xAAF0,
	0xAAF2,
	0xAAF5,
	0xAAF7,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB5B,
	0xAB5C,
	0xAB66,
	0xAB70,
	0xABE3,
	0xABEB,
	0xABEC,
	0xABEE,
	0xABF0,
	0xABFA,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB1E,
	0xFB1F,
	0xFB29,
	0xFB2A,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFB50,
	0xFBB2,
	0xFBD3,
	0xFD3E,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFC,
	0xFE00,
	0xFE10,
	0xFE11,
	0xFE13,
	0xFE14,
	0xFE15,
	0xFE20,
	0xFE30,
	0xFE33,
	0xFE35,
	0xFE4D,
	0xFE50,
	0xFE51,
	0xFE52,
	0xFE53,
	0xFE54,
	0xFE55,
	0xFE56,
	0xFE70,
	0xFE75,
	0xFE76,
	0xFEFD,
	0xFEFF,
	0xFF00,
	0xFF07,
	0xFF08,
	0xFF0C,
	0xFF0D,
	0xFF0E,
	0xFF0F,
	0xFF1A,
	0xFF1B,
	0xFF1C,
	0xFF21,
	0xFF3B,
	0xFF3F,
	0xFF40,
	0xFF41,
	0xFF5B,
	0xFF66,
	0xFF9E,
	0xFFA0,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0xFFF9,
	0xFFFC,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10140,
	0x10175,
	0x101FD,
	0x101FE,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x102E0,
	0x102E1,
	0x10300,
	0x10320,
	0x10330,
	0x1034B,
	0x10350,
	0x10376,
	0x1037B,
	0x10380,
	0x1039E,
	0x103A0,
	0x103C4,
	0x103C8,
	0x103D0,
	0x103D1,
	0x103D6,
	0x10400,
	0x1049E,
	0x104A0,
	0x104AA,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10860,
	0x10877,
	0x10880,
	0x1089F,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x10900,
	0x10916,
	0x10920,
	0x1093A,
	0x10980,
	0x109B8,
	0x109BE,
	0x109C0,
	0x10A00,
	0x10A01,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A10,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A38,
	0x10A3B,
	0x10A3F,
	0x10A40,
	0x10A60,
	0x10A7D,
	0x10A80,
	0x10A9D,
	0x10AC0,
	0x10AC8,
	0x10AC9,
	0x10AE5,
	0x10AE7,
	0x10B00,
	0x10B36,
	0x10B40,
	0x10B56,
	0x10B60,
	0x10B73,
	0x10B80,
	0x10B92,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x11000,
	0x11003,
	0x11038,
	0x11047,
	0x11066,
	0x11070,
	0x1107F,
	0x11083,
	0x110B0,
	0x110BB,
	0x110BD,
	0x110BE,
	0x110D0,
	0x110E9,
	0x110F0,
	0x110FA,
	0x11100,
	0x11103,
	0x11127,
	0x11135,
	0x11136,
	0x11140,
	0x11150,
	0x11173,
	0x11174,
	0x11176,
	0x11177,
	0x11180,
	0x11183,
	0x111B3,
	0x111C1,
	0x111C5,
	0x111CA,
	0x111CD,
	0x111D0,
	0x111DA,
	0x111DB,
	0x111DC,
	0x111DD,
	0x11200,
	0x11212,
	0x11213,
	0x1122C,
	0x11238,
	0x1123E,
	0x1123F,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112A9,
	0x112B0,
	0x112DF,
	0x112EB,
	0x112F0,
	0x112FA,
	0x11300,
	0x11304,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133C,
	0x1133D,
	0x1133E,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134E,
	0x11350,
	0x11351,
	0x11357,
	0x11358,
	0x1135D,
	0x11362,
	0x11364,
	0x11366,
	0x1136D,
	0x11370,
	0x11375,
	0x11400,
	0x11435,
	0x11447,
	0x1144B,
	0x11450,
	0x1145A,
	0x11480,
	0x114B0,
	0x114C4,
	0x114C6,
	0x114C7,
	0x114C8,
	0x114D0,
	0x114DA,
	0x11580,
	0x115AF,
	0x115B6,
	0x115B8,
	0x115C1,
	0x115D8,
	0x115DC,
	0x115DE,
	0x11600,
	0x11630,
	0x11641,
	0x11644,
	0x11645,
	0x11650,
	0x1165A,
	0x11680,
	0x116AB,
	0x116B8,
	0x116C0,
	0x116CA,
	0x1171D,
	0x1172C,
	0x11730,
	0x1173A,
	0x118A0,
	0x118E0,
	0x118EA,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C2F,
	0x11C37,
	0x11C38,
	0x11C40,
	0x11C41,
	0x11C50,
	0x11C5A,
	0x11C72,
	0x11C90,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16A60,
	0x16A6A,
	0x16AD0,
	0x16AEE,
	0x16AF0,
	0x16AF5,
	0x16B00,
	0x16B30,
	0x16B37,
	0x16B40,
	0x16B44,
	0x16B50,
	0x16B5A,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F51,
	0x16F7F,
	0x16F8F,
	0x16F93,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x1B000,
	0x1B001,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1BC9D,
	0x1BC9F,
	0x1BCA0,
	0x1BCA4,
	0x1D165,
	0x1D16A,
	0x1D16D,
	0x1D173,
	0x1D17B,
	0x1D183,
	0x1D185,
	0x1D18C,
	0x1D1AA,
	0x1D1AE,
	0x1D242,
	0x1D245,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D6C1,
	0x1D6C2,
	0x1D6DB,
	0x1D6DC,
	0x1D6FB,
	0x1D6FC,
	0x1D715,
	0x1D716,
	0x1D735,
	0x1D736,
	0x1D74F,
	0x1D750,
	0x1D76F,
	0x1D770,
	0x1D789,
	0x1D78A,
	0x1D7A9,
	0x1D7AA,
	0x1D7C3,
	0x1D7C4,
	0x1D7CC,
	0x1D7CE,
	0x1D800,
	0x1DA00,
	0x1DA37,
	0x1DA3B,
	0x1DA6D,
	0x1DA75,
	0x1DA76,
	0x1DA84,
	0x1DA85,
	0x1DA9B,
	0x1DAA0,
	0x1DAA1,
	0x1DAB0,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E800,
	0x1E8C5,
	0x1E8D0,
	0x1E8D7,
	0x1E900,
	0x1E944,
	0x1E94B,
	0x1E950,
	0x1E95A,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x1F130,
	0x1F14A,
	0x1F150,
	0x1F16A,
	0x1F170,
	0x1F18A,
	0x1F1E6,
	0x1F200,
	0x1F385,
	0x1F386,
	0x1F3C3,
	0x1F3C5,
	0x1F3CA,
	0x1F3CC,
	0x1F3FB,
	0x1F400,
	0x1F442,
	0x1F444,
	0x1F446,
	0x1F451,
	0x1F466,
	0x1F46A,
	0x1F46E,
	0x1F46F,
	0x1F470,
	0x1F479,
	0x1F47C,
	0x1F47D,
	0x1F481,
	0x1F484,
	0x1F485,
	0x1F488,
	0x1F48B,
	0x1F48C,
	0x1F4AA,
	0x1F4AB,
	0x1F575,
	0x1F576,
	0x1F57A,
	0x1F57B,
	0x1F590,
	0x1F591,
	0x1F595,
	0x1F597,
	0x1F5E8,
	0x1F5E9,
	0x1F645,
	0x1F648,
	0x1F64B,
	0x1F650,
	0x1F6A3,
	0x1F6A4,
	0x1F6B4,
	0x1F6B7,
	0x1F6C0,
	0x1F6C1,
	0x1F918,
	0x1F91F,
	0x1F926,
	0x1F927,
	0x1F930,
	0x1F931,
	0x1F933,
	0x1F93A,
	0x1F93C,
	0x1F93F,
	0xE0001,
	0xE0002,
	0xE0020,
	0xE0080,
	0xE0100,
	0xE01F0
};

#endif	/* defined(PERL_IN_PERL_C) */

#if defined(PERL_IN_REGEXEC_C)

#define WB_ENUM_COUNT 25

typedef enum {
	WB_Other = 0,
	WB_ALetter = 1,
	WB_CR = 2,
	WB_Double_Quote = 3,
	WB_E_Base = 4,
	WB_E_Base_GAZ = 5,
	WB_E_Modifier = 6,
	WB_Extend = 7,
	WB_ExtendNumLet = 8,
	WB_Format = 9,
	WB_Glue_After_Zwj = 10,
	WB_Hebrew_Letter = 11,
	WB_Katakana = 12,
	WB_LF = 13,
	WB_MidLetter = 14,
	WB_MidNum = 15,
	WB_MidNumLet = 16,
	WB_Newline = 17,
	WB_Numeric = 18,
	WB_Perl_Tailored_HSpace = 19,
	WB_Regional_Indicator = 20,
	WB_Single_Quote = 21,
	WB_ZWJ = 22,
	WB_EDGE = 23,
	WB_UNKNOWN = 24
} WB_enum;

static const WB_enum _Perl_WB_invmap[] = { /* for EBCDIC 037 */
	WB_Other,
	WB_Perl_Tailored_HSpace,
	WB_Other,
	WB_Newline,
	WB_CR,
	WB_Other,
	WB_Newline,
	WB_Other,
	WB_LF,
	WB_Other,
	WB_Perl_Tailored_HSpace,
	WB_ALetter,
	WB_Other,
	WB_MidNumLet,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_MidNum,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_MidNum,
	WB_Other,
	WB_ExtendNumLet,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_MidLetter,
	WB_Other,
	WB_Single_Quote,
	WB_Other,
	WB_Double_Quote,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_MidLetter,
	WB_Other,
	WB_ALetter,
	WB_Format,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_MidLetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_MidNum,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_MidLetter,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_MidNum,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Hebrew_Letter,
	WB_Other,
	WB_Hebrew_Letter,
	WB_ALetter,
	WB_MidLetter,
	WB_Other,
	WB_Format,
	WB_Other,
	WB_MidNum,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Format,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Numeric,
	WB_Other,
	WB_Numeric,
	WB_MidNum,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Format,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Numeric,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Format,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_MidNum,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Format,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Numeric,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Perl_Tailored_HSpace,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Format,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Numeric,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Numeric,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Numeric,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Perl_Tailored_HSpace,
	WB_Other,
	WB_Extend,
	WB_ZWJ,
	WB_Format,
	WB_Other,
	WB_MidNumLet,
	WB_Other,
	WB_MidNumLet,
	WB_Other,
	WB_MidLetter,
	WB_Newline,
	WB_Format,
	WB_ExtendNumLet,
	WB_Other,
	WB_ExtendNumLet,
	WB_Other,
	WB_MidNum,
	WB_Other,
	WB_ExtendNumLet,
	WB_Other,
	WB_Perl_Tailored_HSpace,
	WB_Format,
	WB_Other,
	WB_Format,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_Glue_After_Zwj,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Perl_Tailored_HSpace,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Katakana,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Katakana,
	WB_Other,
	WB_Katakana,
	WB_Other,
	WB_Katakana,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Katakana,
	WB_Other,
	WB_Katakana,
	WB_Other,
	WB_Katakana,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Numeric,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Hebrew_Letter,
	WB_Extend,
	WB_Hebrew_Letter,
	WB_Other,
	WB_Hebrew_Letter,
	WB_Other,
	WB_Hebrew_Letter,
	WB_Other,
	WB_Hebrew_Letter,
	WB_Other,
	WB_Hebrew_Letter,
	WB_Other,
	WB_Hebrew_Letter,
	WB_Other,
	WB_Hebrew_Letter,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_MidNum,
	WB_Other,
	WB_MidLetter,
	WB_MidNum,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ExtendNumLet,
	WB_Other,
	WB_ExtendNumLet,
	WB_MidNum,
	WB_Other,
	WB_MidNumLet,
	WB_Other,
	WB_MidNum,
	WB_MidLetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Format,
	WB_Other,
	WB_MidNumLet,
	WB_Other,
	WB_MidNum,
	WB_Other,
	WB_MidNumLet,
	WB_Other,
	WB_MidLetter,
	WB_MidNum,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ExtendNumLet,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Katakana,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Format,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Format,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Katakana,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Format,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Format,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_ALetter,
	WB_Extend,
	WB_Other,
	WB_Numeric,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_ALetter,
	WB_Other,
	WB_Regional_Indicator,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Modifier,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base_GAZ,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_Glue_After_Zwj,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_Glue_After_Zwj,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_E_Base,
	WB_Other,
	WB_Format,
	WB_Other,
	WB_Extend,
	WB_Other,
	WB_Extend,
	WB_Other
};

#endif	/* defined(PERL_IN_REGEXEC_C) */

#if defined(PERL_IN_REGCOMP_C)

static const UV UpperLatin1_invlist[] = { /* for EBCDIC 037 */
	57,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x4,
	0x5,
	0x6,
	0x7,
	0x8,
	0xB,
	0x14,
	0x16,
	0x17,
	0x18,
	0x1A,
	0x1C,
	0x20,
	0x25,
	0x28,
	0x2D,
	0x30,
	0x32,
	0x33,
	0x37,
	0x38,
	0x3C,
	0x3E,
	0x3F,
	0x41,
	0x4B,
	0x51,
	0x5A,
	0x5F,
	0x60,
	0x62,
	0x6B,
	0x70,
	0x79,
	0x80,
	0x81,
	0x8A,
	0x91,
	0x9A,
	0xA1,
	0xAA,
	0xB0,
	0xB1,
	0xBA,
	0xBC,
	0xC0,
	0xCA,
	0xD0,
	0xDA,
	0xE0,
	0xE1,
	0xE2,
	0xEA,
	0xF0,
	0xFA,
	0x100
};

#endif	/* defined(PERL_IN_REGCOMP_C) */

#if defined(PERL_IN_PERL_C)

static const UV VertSpace_invlist[] = { /* for EBCDIC 037 */
	9,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0xB,
	0xE,
	0x15,
	0x16,
	0x25,
	0x26,
	0x2028,
	0x202A
};

static const UV XPerlSpace_invlist[] = { /* for EBCDIC 037 */
	23,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x5,
	0x6,
	0xB,
	0xE,
	0x15,
	0x16,
	0x25,
	0x26,
	0x40,
	0x42,
	0x1680,
	0x1681,
	0x2000,
	0x200B,
	0x2028,
	0x202A,
	0x202F,
	0x2030,
	0x205F,
	0x2060,
	0x3000,
	0x3001
};

static const UV XPosixAlnum_invlist[] = { /* for EBCDIC 037 */
	1377,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x42,
	0x4A,
	0x51,
	0x5A,
	0x62,
	0x6A,
	0x70,
	0x79,
	0x80,
	0x8A,
	0x8C,
	0x8F,
	0x91,
	0x9D,
	0x9E,
	0x9F,
	0xA0,
	0xA1,
	0xA2,
	0xAA,
	0xAC,
	0xAF,
	0xC1,
	0xCA,
	0xCB,
	0xD0,
	0xD1,
	0xDA,
	0xDB,
	0xE0,
	0xE2,
	0xEA,
	0xEB,
	0xFA,
	0xFB,
	0xFF,
	0x100,
	0x2C2,
	0x2C6,
	0x2D2,
	0x2E0,
	0x2E5,
	0x2EC,
	0x2ED,
	0x2EE,
	0x2EF,
	0x345,
	0x346,
	0x370,
	0x375,
	0x376,
	0x378,
	0x37A,
	0x37E,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x3F6,
	0x3F7,
	0x482,
	0x48A,
	0x530,
	0x531,
	0x557,
	0x559,
	0x55A,
	0x561,
	0x588,
	0x5B0,
	0x5BE,
	0x5BF,
	0x5C0,
	0x5C1,
	0x5C3,
	0x5C4,
	0x5C6,
	0x5C7,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F3,
	0x610,
	0x61B,
	0x620,
	0x658,
	0x659,
	0x66A,
	0x66E,
	0x6D4,
	0x6D5,
	0x6DD,
	0x6E1,
	0x6E9,
	0x6ED,
	0x6FD,
	0x6FF,
	0x700,
	0x710,
	0x740,
	0x74D,
	0x7B2,
	0x7C0,
	0x7EB,
	0x7F4,
	0x7F6,
	0x7FA,
	0x7FB,
	0x800,
	0x818,
	0x81A,
	0x82D,
	0x840,
	0x859,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x8D4,
	0x8E0,
	0x8E3,
	0x8EA,
	0x8F0,
	0x93C,
	0x93D,
	0x94D,
	0x94E,
	0x951,
	0x955,
	0x964,
	0x966,
	0x970,
	0x971,
	0x984,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BD,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CD,
	0x9CE,
	0x9CF,
	0x9D7,
	0x9D8,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E4,
	0x9E6,
	0x9F2,
	0xA01,
	0xA04,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4D,
	0xA51,
	0xA52,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA66,
	0xA76,
	0xA81,
	0xA84,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABD,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACD,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE4,
	0xAE6,
	0xAF0,
	0xAF9,
	0xAFA,
	0xB01,
	0xB04,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3D,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4D,
	0xB56,
	0xB58,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB64,
	0xB66,
	0xB70,
	0xB71,
	0xB72,
	0xB82,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCD,
	0xBD0,
	0xBD1,
	0xBD7,
	0xBD8,
	0xBE6,
	0xBF0,
	0xC00,
	0xC04,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4D,
	0xC55,
	0xC57,
	0xC58,
	0xC5B,
	0xC60,
	0xC64,
	0xC66,
	0xC70,
	0xC80,
	0xC84,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBD,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCD,
	0xCD5,
	0xCD7,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE4,
	0xCE6,
	0xCF0,
	0xCF1,
	0xCF3,
	0xD01,
	0xD04,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD4D,
	0xD4E,
	0xD4F,
	0xD54,
	0xD58,
	0xD5F,
	0xD64,
	0xD66,
	0xD70,
	0xD7A,
	0xD80,
	0xD82,
	0xD84,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDE6,
	0xDF0,
	0xDF2,
	0xDF4,
	0xE01,
	0xE3B,
	0xE40,
	0xE47,
	0xE4D,
	0xE4E,
	0xE50,
	0xE5A,
	0xE81,
	0xE83,
	0xE84,
	0xE85,
	0xE87,
	0xE89,
	0xE8A,
	0xE8B,
	0xE8D,
	0xE8E,
	0xE94,
	0xE98,
	0xE99,
	0xEA0,
	0xEA1,
	0xEA4,
	0xEA5,
	0xEA6,
	0xEA7,
	0xEA8,
	0xEAA,
	0xEAC,
	0xEAD,
	0xEBA,
	0xEBB,
	0xEBE,
	0xEC0,
	0xEC5,
	0xEC6,
	0xEC7,
	0xECD,
	0xECE,
	0xED0,
	0xEDA,
	0xEDC,
	0xEE0,
	0xF00,
	0xF01,
	0xF20,
	0xF2A,
	0xF40,
	0xF48,
	0xF49,
	0xF6D,
	0xF71,
	0xF82,
	0xF88,
	0xF98,
	0xF99,
	0xFBD,
	0x1000,
	0x1037,
	0x1038,
	0x1039,
	0x103B,
	0x104A,
	0x1050,
	0x1063,
	0x1065,
	0x1069,
	0x106E,
	0x1087,
	0x108E,
	0x108F,
	0x1090,
	0x109A,
	0x109C,
	0x109E,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x10FB,
	0x10FC,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x135F,
	0x1360,
	0x1380,
	0x1390,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1401,
	0x166D,
	0x166F,
	0x1680,
	0x1681,
	0x169B,
	0x16A0,
	0x16EB,
	0x16EE,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1714,
	0x1720,
	0x1734,
	0x1740,
	0x1754,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1772,
	0x1774,
	0x1780,
	0x17B4,
	0x17B6,
	0x17C9,
	0x17D7,
	0x17D8,
	0x17DC,
	0x17DD,
	0x17E0,
	0x17EA,
	0x1810,
	0x181A,
	0x1820,
	0x1878,
	0x1880,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1920,
	0x192C,
	0x1930,
	0x1939,
	0x1946,
	0x196E,
	0x1970,
	0x1975,
	0x1980,
	0x19AC,
	0x19B0,
	0x19CA,
	0x19D0,
	0x19DA,
	0x1A00,
	0x1A1C,
	0x1A20,
	0x1A5F,
	0x1A61,
	0x1A75,
	0x1A80,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1AA7,
	0x1AA8,
	0x1B00,
	0x1B34,
	0x1B35,
	0x1B44,
	0x1B45,
	0x1B4C,
	0x1B50,
	0x1B5A,
	0x1B80,
	0x1BAA,
	0x1BAC,
	0x1BE6,
	0x1BE7,
	0x1BF2,
	0x1C00,
	0x1C36,
	0x1C40,
	0x1C4A,
	0x1C4D,
	0x1C7E,
	0x1C80,
	0x1C89,
	0x1CE9,
	0x1CED,
	0x1CEE,
	0x1CF4,
	0x1CF5,
	0x1CF7,
	0x1D00,
	0x1DC0,
	0x1DE7,
	0x1DF5,
	0x1E00,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FBD,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FCD,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FE0,
	0x1FED,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFD,
	0x2071,
	0x2072,
	0x207F,
	0x2080,
	0x2090,
	0x209D,
	0x2102,
	0x2103,
	0x2107,
	0x2108,
	0x210A,
	0x2114,
	0x2115,
	0x2116,
	0x2119,
	0x211E,
	0x2124,
	0x2125,
	0x2126,
	0x2127,
	0x2128,
	0x2129,
	0x212A,
	0x212E,
	0x212F,
	0x213A,
	0x213C,
	0x2140,
	0x2145,
	0x214A,
	0x214E,
	0x214F,
	0x2160,
	0x2189,
	0x24B6,
	0x24EA,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CE5,
	0x2CEB,
	0x2CEF,
	0x2CF2,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D70,
	0x2D80,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x2DE0,
	0x2E00,
	0x2E2F,
	0x2E30,
	0x3005,
	0x3008,
	0x3021,
	0x302A,
	0x3031,
	0x3036,
	0x3038,
	0x303D,
	0x3041,
	0x3097,
	0x309D,
	0x30A0,
	0x30A1,
	0x30FB,
	0x30FC,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x31A0,
	0x31BB,
	0x31F0,
	0x3200,
	0x3400,
	0x4DB6,
	0x4E00,
	0x9FD6,
	0xA000,
	0xA48D,
	0xA4D0,
	0xA4FE,
	0xA500,
	0xA60D,
	0xA610,
	0xA62C,
	0xA640,
	0xA66F,
	0xA674,
	0xA67C,
	0xA67F,
	0xA6F0,
	0xA717,
	0xA720,
	0xA722,
	0xA789,
	0xA78B,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F7,
	0xA802,
	0xA803,
	0xA806,
	0xA807,
	0xA80B,
	0xA80C,
	0xA828,
	0xA840,
	0xA874,
	0xA880,
	0xA8C4,
	0xA8C5,
	0xA8C6,
	0xA8D0,
	0xA8DA,
	0xA8F2,
	0xA8F8,
	0xA8FB,
	0xA8FC,
	0xA8FD,
	0xA8FE,
	0xA900,
	0xA92B,
	0xA930,
	0xA953,
	0xA960,
	0xA97D,
	0xA980,
	0xA9B3,
	0xA9B4,
	0xA9C0,
	0xA9CF,
	0xA9DA,
	0xA9E0,
	0xA9E5,
	0xA9E6,
	0xA9FF,
	0xAA00,
	0xAA37,
	0xAA40,
	0xAA4E,
	0xAA50,
	0xAA5A,
	0xAA60,
	0xAA77,
	0xAA7A,
	0xAA7B,
	0xAA7E,
	0xAABF,
	0xAAC0,
	0xAAC1,
	0xAAC2,
	0xAAC3,
	0xAADB,
	0xAADE,
	0xAAE0,
	0xAAF0,
	0xAAF2,
	0xAAF6,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB5B,
	0xAB5C,
	0xAB66,
	0xAB70,
	0xABEB,
	0xABF0,
	0xABFA,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xF900,
	0xFA6E,
	0xFA70,
	0xFADA,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB29,
	0xFB2A,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFBB2,
	0xFBD3,
	0xFD3E,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFC,
	0xFE70,
	0xFE75,
	0xFE76,
	0xFEFD,
	0xFF10,
	0xFF1A,
	0xFF21,
	0xFF3B,
	0xFF41,
	0xFF5B,
	0xFF66,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10140,
	0x10175,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x10300,
	0x10320,
	0x10330,
	0x1034B,
	0x10350,
	0x1037B,
	0x10380,
	0x1039E,
	0x103A0,
	0x103C4,
	0x103C8,
	0x103D0,
	0x103D1,
	0x103D6,
	0x10400,
	0x1049E,
	0x104A0,
	0x104AA,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10860,
	0x10877,
	0x10880,
	0x1089F,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x10900,
	0x10916,
	0x10920,
	0x1093A,
	0x10980,
	0x109B8,
	0x109BE,
	0x109C0,
	0x10A00,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A60,
	0x10A7D,
	0x10A80,
	0x10A9D,
	0x10AC0,
	0x10AC8,
	0x10AC9,
	0x10AE5,
	0x10B00,
	0x10B36,
	0x10B40,
	0x10B56,
	0x10B60,
	0x10B73,
	0x10B80,
	0x10B92,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x11000,
	0x11046,
	0x11066,
	0x11070,
	0x11082,
	0x110B9,
	0x110D0,
	0x110E9,
	0x110F0,
	0x110FA,
	0x11100,
	0x11133,
	0x11136,
	0x11140,
	0x11150,
	0x11173,
	0x11176,
	0x11177,
	0x11180,
	0x111C0,
	0x111C1,
	0x111C5,
	0x111D0,
	0x111DB,
	0x111DC,
	0x111DD,
	0x11200,
	0x11212,
	0x11213,
	0x11235,
	0x11237,
	0x11238,
	0x1123E,
	0x1123F,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112A9,
	0x112B0,
	0x112E9,
	0x112F0,
	0x112FA,
	0x11300,
	0x11304,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133D,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134D,
	0x11350,
	0x11351,
	0x11357,
	0x11358,
	0x1135D,
	0x11364,
	0x11400,
	0x11442,
	0x11443,
	0x11446,
	0x11447,
	0x1144B,
	0x11450,
	0x1145A,
	0x11480,
	0x114C2,
	0x114C4,
	0x114C6,
	0x114C7,
	0x114C8,
	0x114D0,
	0x114DA,
	0x11580,
	0x115B6,
	0x115B8,
	0x115BF,
	0x115D8,
	0x115DE,
	0x11600,
	0x1163F,
	0x11640,
	0x11641,
	0x11644,
	0x11645,
	0x11650,
	0x1165A,
	0x11680,
	0x116B6,
	0x116C0,
	0x116CA,
	0x11700,
	0x1171A,
	0x1171D,
	0x1172B,
	0x11730,
	0x1173A,
	0x118A0,
	0x118EA,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C37,
	0x11C38,
	0x11C3F,
	0x11C40,
	0x11C41,
	0x11C50,
	0x11C5A,
	0x11C72,
	0x11C90,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16A60,
	0x16A6A,
	0x16AD0,
	0x16AEE,
	0x16B00,
	0x16B37,
	0x16B40,
	0x16B44,
	0x16B50,
	0x16B5A,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F7F,
	0x16F93,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1BC9E,
	0x1BC9F,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D6C1,
	0x1D6C2,
	0x1D6DB,
	0x1D6DC,
	0x1D6FB,
	0x1D6FC,
	0x1D715,
	0x1D716,
	0x1D735,
	0x1D736,
	0x1D74F,
	0x1D750,
	0x1D76F,
	0x1D770,
	0x1D789,
	0x1D78A,
	0x1D7A9,
	0x1D7AA,
	0x1D7C3,
	0x1D7C4,
	0x1D7CC,
	0x1D7CE,
	0x1D800,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E800,
	0x1E8C5,
	0x1E900,
	0x1E944,
	0x1E947,
	0x1E948,
	0x1E950,
	0x1E95A,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x1F130,
	0x1F14A,
	0x1F150,
	0x1F16A,
	0x1F170,
	0x1F18A,
	0x20000,
	0x2A6D7,
	0x2A700,
	0x2B735,
	0x2B740,
	0x2B81E,
	0x2B820,
	0x2CEA2,
	0x2F800,
	0x2FA1E
};

static const UV XPosixAlpha_invlist[] = { /* for EBCDIC 037 */
	1311,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x42,
	0x4A,
	0x51,
	0x5A,
	0x62,
	0x6A,
	0x70,
	0x79,
	0x80,
	0x8A,
	0x8C,
	0x8F,
	0x91,
	0x9D,
	0x9E,
	0x9F,
	0xA0,
	0xA1,
	0xA2,
	0xAA,
	0xAC,
	0xAF,
	0xC1,
	0xCA,
	0xCB,
	0xD0,
	0xD1,
	0xDA,
	0xDB,
	0xE0,
	0xE2,
	0xEA,
	0xEB,
	0xF0,
	0xFB,
	0xFF,
	0x100,
	0x2C2,
	0x2C6,
	0x2D2,
	0x2E0,
	0x2E5,
	0x2EC,
	0x2ED,
	0x2EE,
	0x2EF,
	0x345,
	0x346,
	0x370,
	0x375,
	0x376,
	0x378,
	0x37A,
	0x37E,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x3F6,
	0x3F7,
	0x482,
	0x48A,
	0x530,
	0x531,
	0x557,
	0x559,
	0x55A,
	0x561,
	0x588,
	0x5B0,
	0x5BE,
	0x5BF,
	0x5C0,
	0x5C1,
	0x5C3,
	0x5C4,
	0x5C6,
	0x5C7,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F3,
	0x610,
	0x61B,
	0x620,
	0x658,
	0x659,
	0x660,
	0x66E,
	0x6D4,
	0x6D5,
	0x6DD,
	0x6E1,
	0x6E9,
	0x6ED,
	0x6F0,
	0x6FA,
	0x6FD,
	0x6FF,
	0x700,
	0x710,
	0x740,
	0x74D,
	0x7B2,
	0x7CA,
	0x7EB,
	0x7F4,
	0x7F6,
	0x7FA,
	0x7FB,
	0x800,
	0x818,
	0x81A,
	0x82D,
	0x840,
	0x859,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x8D4,
	0x8E0,
	0x8E3,
	0x8EA,
	0x8F0,
	0x93C,
	0x93D,
	0x94D,
	0x94E,
	0x951,
	0x955,
	0x964,
	0x971,
	0x984,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BD,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CD,
	0x9CE,
	0x9CF,
	0x9D7,
	0x9D8,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E4,
	0x9F0,
	0x9F2,
	0xA01,
	0xA04,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4D,
	0xA51,
	0xA52,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA70,
	0xA76,
	0xA81,
	0xA84,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABD,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACD,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE4,
	0xAF9,
	0xAFA,
	0xB01,
	0xB04,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3D,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4D,
	0xB56,
	0xB58,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB64,
	0xB71,
	0xB72,
	0xB82,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCD,
	0xBD0,
	0xBD1,
	0xBD7,
	0xBD8,
	0xC00,
	0xC04,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4D,
	0xC55,
	0xC57,
	0xC58,
	0xC5B,
	0xC60,
	0xC64,
	0xC80,
	0xC84,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBD,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCD,
	0xCD5,
	0xCD7,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE4,
	0xCF1,
	0xCF3,
	0xD01,
	0xD04,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD4D,
	0xD4E,
	0xD4F,
	0xD54,
	0xD58,
	0xD5F,
	0xD64,
	0xD7A,
	0xD80,
	0xD82,
	0xD84,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDF2,
	0xDF4,
	0xE01,
	0xE3B,
	0xE40,
	0xE47,
	0xE4D,
	0xE4E,
	0xE81,
	0xE83,
	0xE84,
	0xE85,
	0xE87,
	0xE89,
	0xE8A,
	0xE8B,
	0xE8D,
	0xE8E,
	0xE94,
	0xE98,
	0xE99,
	0xEA0,
	0xEA1,
	0xEA4,
	0xEA5,
	0xEA6,
	0xEA7,
	0xEA8,
	0xEAA,
	0xEAC,
	0xEAD,
	0xEBA,
	0xEBB,
	0xEBE,
	0xEC0,
	0xEC5,
	0xEC6,
	0xEC7,
	0xECD,
	0xECE,
	0xEDC,
	0xEE0,
	0xF00,
	0xF01,
	0xF40,
	0xF48,
	0xF49,
	0xF6D,
	0xF71,
	0xF82,
	0xF88,
	0xF98,
	0xF99,
	0xFBD,
	0x1000,
	0x1037,
	0x1038,
	0x1039,
	0x103B,
	0x1040,
	0x1050,
	0x1063,
	0x1065,
	0x1069,
	0x106E,
	0x1087,
	0x108E,
	0x108F,
	0x109C,
	0x109E,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x10FB,
	0x10FC,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x135F,
	0x1360,
	0x1380,
	0x1390,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1401,
	0x166D,
	0x166F,
	0x1680,
	0x1681,
	0x169B,
	0x16A0,
	0x16EB,
	0x16EE,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1714,
	0x1720,
	0x1734,
	0x1740,
	0x1754,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1772,
	0x1774,
	0x1780,
	0x17B4,
	0x17B6,
	0x17C9,
	0x17D7,
	0x17D8,
	0x17DC,
	0x17DD,
	0x1820,
	0x1878,
	0x1880,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1920,
	0x192C,
	0x1930,
	0x1939,
	0x1950,
	0x196E,
	0x1970,
	0x1975,
	0x1980,
	0x19AC,
	0x19B0,
	0x19CA,
	0x1A00,
	0x1A1C,
	0x1A20,
	0x1A5F,
	0x1A61,
	0x1A75,
	0x1AA7,
	0x1AA8,
	0x1B00,
	0x1B34,
	0x1B35,
	0x1B44,
	0x1B45,
	0x1B4C,
	0x1B80,
	0x1BAA,
	0x1BAC,
	0x1BB0,
	0x1BBA,
	0x1BE6,
	0x1BE7,
	0x1BF2,
	0x1C00,
	0x1C36,
	0x1C4D,
	0x1C50,
	0x1C5A,
	0x1C7E,
	0x1C80,
	0x1C89,
	0x1CE9,
	0x1CED,
	0x1CEE,
	0x1CF4,
	0x1CF5,
	0x1CF7,
	0x1D00,
	0x1DC0,
	0x1DE7,
	0x1DF5,
	0x1E00,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FBD,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FCD,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FE0,
	0x1FED,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFD,
	0x2071,
	0x2072,
	0x207F,
	0x2080,
	0x2090,
	0x209D,
	0x2102,
	0x2103,
	0x2107,
	0x2108,
	0x210A,
	0x2114,
	0x2115,
	0x2116,
	0x2119,
	0x211E,
	0x2124,
	0x2125,
	0x2126,
	0x2127,
	0x2128,
	0x2129,
	0x212A,
	0x212E,
	0x212F,
	0x213A,
	0x213C,
	0x2140,
	0x2145,
	0x214A,
	0x214E,
	0x214F,
	0x2160,
	0x2189,
	0x24B6,
	0x24EA,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CE5,
	0x2CEB,
	0x2CEF,
	0x2CF2,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D70,
	0x2D80,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x2DE0,
	0x2E00,
	0x2E2F,
	0x2E30,
	0x3005,
	0x3008,
	0x3021,
	0x302A,
	0x3031,
	0x3036,
	0x3038,
	0x303D,
	0x3041,
	0x3097,
	0x309D,
	0x30A0,
	0x30A1,
	0x30FB,
	0x30FC,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x31A0,
	0x31BB,
	0x31F0,
	0x3200,
	0x3400,
	0x4DB6,
	0x4E00,
	0x9FD6,
	0xA000,
	0xA48D,
	0xA4D0,
	0xA4FE,
	0xA500,
	0xA60D,
	0xA610,
	0xA620,
	0xA62A,
	0xA62C,
	0xA640,
	0xA66F,
	0xA674,
	0xA67C,
	0xA67F,
	0xA6F0,
	0xA717,
	0xA720,
	0xA722,
	0xA789,
	0xA78B,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F7,
	0xA802,
	0xA803,
	0xA806,
	0xA807,
	0xA80B,
	0xA80C,
	0xA828,
	0xA840,
	0xA874,
	0xA880,
	0xA8C4,
	0xA8C5,
	0xA8C6,
	0xA8F2,
	0xA8F8,
	0xA8FB,
	0xA8FC,
	0xA8FD,
	0xA8FE,
	0xA90A,
	0xA92B,
	0xA930,
	0xA953,
	0xA960,
	0xA97D,
	0xA980,
	0xA9B3,
	0xA9B4,
	0xA9C0,
	0xA9CF,
	0xA9D0,
	0xA9E0,
	0xA9E5,
	0xA9E6,
	0xA9F0,
	0xA9FA,
	0xA9FF,
	0xAA00,
	0xAA37,
	0xAA40,
	0xAA4E,
	0xAA60,
	0xAA77,
	0xAA7A,
	0xAA7B,
	0xAA7E,
	0xAABF,
	0xAAC0,
	0xAAC1,
	0xAAC2,
	0xAAC3,
	0xAADB,
	0xAADE,
	0xAAE0,
	0xAAF0,
	0xAAF2,
	0xAAF6,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB5B,
	0xAB5C,
	0xAB66,
	0xAB70,
	0xABEB,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xF900,
	0xFA6E,
	0xFA70,
	0xFADA,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB29,
	0xFB2A,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFBB2,
	0xFBD3,
	0xFD3E,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFC,
	0xFE70,
	0xFE75,
	0xFE76,
	0xFEFD,
	0xFF21,
	0xFF3B,
	0xFF41,
	0xFF5B,
	0xFF66,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10140,
	0x10175,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x10300,
	0x10320,
	0x10330,
	0x1034B,
	0x10350,
	0x1037B,
	0x10380,
	0x1039E,
	0x103A0,
	0x103C4,
	0x103C8,
	0x103D0,
	0x103D1,
	0x103D6,
	0x10400,
	0x1049E,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10860,
	0x10877,
	0x10880,
	0x1089F,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x10900,
	0x10916,
	0x10920,
	0x1093A,
	0x10980,
	0x109B8,
	0x109BE,
	0x109C0,
	0x10A00,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A60,
	0x10A7D,
	0x10A80,
	0x10A9D,
	0x10AC0,
	0x10AC8,
	0x10AC9,
	0x10AE5,
	0x10B00,
	0x10B36,
	0x10B40,
	0x10B56,
	0x10B60,
	0x10B73,
	0x10B80,
	0x10B92,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x11000,
	0x11046,
	0x11082,
	0x110B9,
	0x110D0,
	0x110E9,
	0x11100,
	0x11133,
	0x11150,
	0x11173,
	0x11176,
	0x11177,
	0x11180,
	0x111C0,
	0x111C1,
	0x111C5,
	0x111DA,
	0x111DB,
	0x111DC,
	0x111DD,
	0x11200,
	0x11212,
	0x11213,
	0x11235,
	0x11237,
	0x11238,
	0x1123E,
	0x1123F,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112A9,
	0x112B0,
	0x112E9,
	0x11300,
	0x11304,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133D,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134D,
	0x11350,
	0x11351,
	0x11357,
	0x11358,
	0x1135D,
	0x11364,
	0x11400,
	0x11442,
	0x11443,
	0x11446,
	0x11447,
	0x1144B,
	0x11480,
	0x114C2,
	0x114C4,
	0x114C6,
	0x114C7,
	0x114C8,
	0x11580,
	0x115B6,
	0x115B8,
	0x115BF,
	0x115D8,
	0x115DE,
	0x11600,
	0x1163F,
	0x11640,
	0x11641,
	0x11644,
	0x11645,
	0x11680,
	0x116B6,
	0x11700,
	0x1171A,
	0x1171D,
	0x1172B,
	0x118A0,
	0x118E0,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C37,
	0x11C38,
	0x11C3F,
	0x11C40,
	0x11C41,
	0x11C72,
	0x11C90,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16AD0,
	0x16AEE,
	0x16B00,
	0x16B37,
	0x16B40,
	0x16B44,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F7F,
	0x16F93,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1BC9E,
	0x1BC9F,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D6C1,
	0x1D6C2,
	0x1D6DB,
	0x1D6DC,
	0x1D6FB,
	0x1D6FC,
	0x1D715,
	0x1D716,
	0x1D735,
	0x1D736,
	0x1D74F,
	0x1D750,
	0x1D76F,
	0x1D770,
	0x1D789,
	0x1D78A,
	0x1D7A9,
	0x1D7AA,
	0x1D7C3,
	0x1D7C4,
	0x1D7CC,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E800,
	0x1E8C5,
	0x1E900,
	0x1E944,
	0x1E947,
	0x1E948,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x1F130,
	0x1F14A,
	0x1F150,
	0x1F16A,
	0x1F170,
	0x1F18A,
	0x20000,
	0x2A6D7,
	0x2A700,
	0x2B735,
	0x2B740,
	0x2B81E,
	0x2B820,
	0x2CEA2,
	0x2F800,
	0x2FA1E
};

static const UV XPosixBlank_invlist[] = { /* for EBCDIC 037 */
	15,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x5,
	0x6,
	0x40,
	0x42,
	0x1680,
	0x1681,
	0x2000,
	0x200B,
	0x202F,
	0x2030,
	0x205F,
	0x2060,
	0x3000,
	0x3001
};

static const UV XPosixCntrl_invlist[] = { /* for EBCDIC 037 */
	4,	/* Number of elements */
	148565664, /* Version and data structure type */
	0,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x40,
	0xFF,
	0x100
};

static const UV XPosixDigit_invlist[] = { /* for EBCDIC 037 */
	109,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0xF0,
	0xFA,
	0x660,
	0x66A,
	0x6F0,
	0x6FA,
	0x7C0,
	0x7CA,
	0x966,
	0x970,
	0x9E6,
	0x9F0,
	0xA66,
	0xA70,
	0xAE6,
	0xAF0,
	0xB66,
	0xB70,
	0xBE6,
	0xBF0,
	0xC66,
	0xC70,
	0xCE6,
	0xCF0,
	0xD66,
	0xD70,
	0xDE6,
	0xDF0,
	0xE50,
	0xE5A,
	0xED0,
	0xEDA,
	0xF20,
	0xF2A,
	0x1040,
	0x104A,
	0x1090,
	0x109A,
	0x17E0,
	0x17EA,
	0x1810,
	0x181A,
	0x1946,
	0x1950,
	0x19D0,
	0x19DA,
	0x1A80,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1B50,
	0x1B5A,
	0x1BB0,
	0x1BBA,
	0x1C40,
	0x1C4A,
	0x1C50,
	0x1C5A,
	0xA620,
	0xA62A,
	0xA8D0,
	0xA8DA,
	0xA900,
	0xA90A,
	0xA9D0,
	0xA9DA,
	0xA9F0,
	0xA9FA,
	0xAA50,
	0xAA5A,
	0xABF0,
	0xABFA,
	0xFF10,
	0xFF1A,
	0x104A0,
	0x104AA,
	0x11066,
	0x11070,
	0x110F0,
	0x110FA,
	0x11136,
	0x11140,
	0x111D0,
	0x111DA,
	0x112F0,
	0x112FA,
	0x11450,
	0x1145A,
	0x114D0,
	0x114DA,
	0x11650,
	0x1165A,
	0x116C0,
	0x116CA,
	0x11730,
	0x1173A,
	0x118E0,
	0x118EA,
	0x11C50,
	0x11C5A,
	0x16A60,
	0x16A6A,
	0x16B50,
	0x16B5A,
	0x1D7CE,
	0x1D800,
	0x1E950,
	0x1E95A
};

static const UV XPosixGraph_invlist[] = { /* for EBCDIC 037 */
	1287,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x42,
	0xFF,
	0x100,
	0x378,
	0x37A,
	0x380,
	0x384,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x530,
	0x531,
	0x557,
	0x559,
	0x560,
	0x561,
	0x588,
	0x589,
	0x58B,
	0x58D,
	0x590,
	0x591,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F5,
	0x600,
	0x61D,
	0x61E,
	0x70E,
	0x70F,
	0x74B,
	0x74D,
	0x7B2,
	0x7C0,
	0x7FB,
	0x800,
	0x82E,
	0x830,
	0x83F,
	0x840,
	0x85C,
	0x85E,
	0x85F,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x8D4,
	0x984,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BC,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CF,
	0x9D7,
	0x9D8,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E4,
	0x9E6,
	0x9FC,
	0xA01,
	0xA04,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA3C,
	0xA3D,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4E,
	0xA51,
	0xA52,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA66,
	0xA76,
	0xA81,
	0xA84,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABC,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACE,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE4,
	0xAE6,
	0xAF2,
	0xAF9,
	0xAFA,
	0xB01,
	0xB04,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3C,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4E,
	0xB56,
	0xB58,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB64,
	0xB66,
	0xB78,
	0xB82,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCE,
	0xBD0,
	0xBD1,
	0xBD7,
	0xBD8,
	0xBE6,
	0xBFB,
	0xC00,
	0xC04,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4E,
	0xC55,
	0xC57,
	0xC58,
	0xC5B,
	0xC60,
	0xC64,
	0xC66,
	0xC70,
	0xC78,
	0xC84,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBC,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCE,
	0xCD5,
	0xCD7,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE4,
	0xCE6,
	0xCF0,
	0xCF1,
	0xCF3,
	0xD01,
	0xD04,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD50,
	0xD54,
	0xD64,
	0xD66,
	0xD80,
	0xD82,
	0xD84,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xDCA,
	0xDCB,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDE6,
	0xDF0,
	0xDF2,
	0xDF5,
	0xE01,
	0xE3B,
	0xE3F,
	0xE5C,
	0xE81,
	0xE83,
	0xE84,
	0xE85,
	0xE87,
	0xE89,
	0xE8A,
	0xE8B,
	0xE8D,
	0xE8E,
	0xE94,
	0xE98,
	0xE99,
	0xEA0,
	0xEA1,
	0xEA4,
	0xEA5,
	0xEA6,
	0xEA7,
	0xEA8,
	0xEAA,
	0xEAC,
	0xEAD,
	0xEBA,
	0xEBB,
	0xEBE,
	0xEC0,
	0xEC5,
	0xEC6,
	0xEC7,
	0xEC8,
	0xECE,
	0xED0,
	0xEDA,
	0xEDC,
	0xEE0,
	0xF00,
	0xF48,
	0xF49,
	0xF6D,
	0xF71,
	0xF98,
	0xF99,
	0xFBD,
	0xFBE,
	0xFCD,
	0xFCE,
	0xFDB,
	0x1000,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x135D,
	0x137D,
	0x1380,
	0x139A,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1400,
	0x1680,
	0x1681,
	0x169D,
	0x16A0,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1715,
	0x1720,
	0x1737,
	0x1740,
	0x1754,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1772,
	0x1774,
	0x1780,
	0x17DE,
	0x17E0,
	0x17EA,
	0x17F0,
	0x17FA,
	0x1800,
	0x180F,
	0x1810,
	0x181A,
	0x1820,
	0x1878,
	0x1880,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1920,
	0x192C,
	0x1930,
	0x193C,
	0x1940,
	0x1941,
	0x1944,
	0x196E,
	0x1970,
	0x1975,
	0x1980,
	0x19AC,
	0x19B0,
	0x19CA,
	0x19D0,
	0x19DB,
	0x19DE,
	0x1A1C,
	0x1A1E,
	0x1A5F,
	0x1A60,
	0x1A7D,
	0x1A7F,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1AA0,
	0x1AAE,
	0x1AB0,
	0x1ABF,
	0x1B00,
	0x1B4C,
	0x1B50,
	0x1B7D,
	0x1B80,
	0x1BF4,
	0x1BFC,
	0x1C38,
	0x1C3B,
	0x1C4A,
	0x1C4D,
	0x1C89,
	0x1CC0,
	0x1CC8,
	0x1CD0,
	0x1CF7,
	0x1CF8,
	0x1CFA,
	0x1D00,
	0x1DF6,
	0x1DFB,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FC5,
	0x1FC6,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FDD,
	0x1FF0,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFF,
	0x200B,
	0x2028,
	0x202A,
	0x202F,
	0x2030,
	0x205F,
	0x2060,
	0x2065,
	0x2066,
	0x2072,
	0x2074,
	0x208F,
	0x2090,
	0x209D,
	0x20A0,
	0x20BF,
	0x20D0,
	0x20F1,
	0x2100,
	0x218C,
	0x2190,
	0x23FF,
	0x2400,
	0x2427,
	0x2440,
	0x244B,
	0x2460,
	0x2B74,
	0x2B76,
	0x2B96,
	0x2B98,
	0x2BBA,
	0x2BBD,
	0x2BC9,
	0x2BCA,
	0x2BD2,
	0x2BEC,
	0x2BF0,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CF4,
	0x2CF9,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D71,
	0x2D7F,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x2DE0,
	0x2E45,
	0x2E80,
	0x2E9A,
	0x2E9B,
	0x2EF4,
	0x2F00,
	0x2FD6,
	0x2FF0,
	0x2FFC,
	0x3001,
	0x3040,
	0x3041,
	0x3097,
	0x3099,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x3190,
	0x31BB,
	0x31C0,
	0x31E4,
	0x31F0,
	0x321F,
	0x3220,
	0x32FF,
	0x3300,
	0x4DB6,
	0x4DC0,
	0x9FD6,
	0xA000,
	0xA48D,
	0xA490,
	0xA4C7,
	0xA4D0,
	0xA62C,
	0xA640,
	0xA6F8,
	0xA700,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F7,
	0xA82C,
	0xA830,
	0xA83A,
	0xA840,
	0xA878,
	0xA880,
	0xA8C6,
	0xA8CE,
	0xA8DA,
	0xA8E0,
	0xA8FE,
	0xA900,
	0xA954,
	0xA95F,
	0xA97D,
	0xA980,
	0xA9CE,
	0xA9CF,
	0xA9DA,
	0xA9DE,
	0xA9FF,
	0xAA00,
	0xAA37,
	0xAA40,
	0xAA4E,
	0xAA50,
	0xAA5A,
	0xAA5C,
	0xAAC3,
	0xAADB,
	0xAAF7,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB66,
	0xAB70,
	0xABEE,
	0xABF0,
	0xABFA,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xE000,
	0xFA6E,
	0xFA70,
	0xFADA,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFBC2,
	0xFBD3,
	0xFD40,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFE,
	0xFE00,
	0xFE1A,
	0xFE20,
	0xFE53,
	0xFE54,
	0xFE67,
	0xFE68,
	0xFE6C,
	0xFE70,
	0xFE75,
	0xFE76,
	0xFEFD,
	0xFEFF,
	0xFF00,
	0xFF01,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0xFFE0,
	0xFFE7,
	0xFFE8,
	0xFFEF,
	0xFFF9,
	0xFFFE,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10100,
	0x10103,
	0x10107,
	0x10134,
	0x10137,
	0x1018F,
	0x10190,
	0x1019C,
	0x101A0,
	0x101A1,
	0x101D0,
	0x101FE,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x102E0,
	0x102FC,
	0x10300,
	0x10324,
	0x10330,
	0x1034B,
	0x10350,
	0x1037B,
	0x10380,
	0x1039E,
	0x1039F,
	0x103C4,
	0x103C8,
	0x103D6,
	0x10400,
	0x1049E,
	0x104A0,
	0x104AA,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x1056F,
	0x10570,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10857,
	0x1089F,
	0x108A7,
	0x108B0,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x108FB,
	0x1091C,
	0x1091F,
	0x1093A,
	0x1093F,
	0x10940,
	0x10980,
	0x109B8,
	0x109BC,
	0x109D0,
	0x109D2,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A38,
	0x10A3B,
	0x10A3F,
	0x10A48,
	0x10A50,
	0x10A59,
	0x10A60,
	0x10AA0,
	0x10AC0,
	0x10AE7,
	0x10AEB,
	0x10AF7,
	0x10B00,
	0x10B36,
	0x10B39,
	0x10B56,
	0x10B58,
	0x10B73,
	0x10B78,
	0x10B92,
	0x10B99,
	0x10B9D,
	0x10BA9,
	0x10BB0,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x10CFA,
	0x10D00,
	0x10E60,
	0x10E7F,
	0x11000,
	0x1104E,
	0x11052,
	0x11070,
	0x1107F,
	0x110C2,
	0x110D0,
	0x110E9,
	0x110F0,
	0x110FA,
	0x11100,
	0x11135,
	0x11136,
	0x11144,
	0x11150,
	0x11177,
	0x11180,
	0x111CE,
	0x111D0,
	0x111E0,
	0x111E1,
	0x111F5,
	0x11200,
	0x11212,
	0x11213,
	0x1123F,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112AA,
	0x112B0,
	0x112EB,
	0x112F0,
	0x112FA,
	0x11300,
	0x11304,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133C,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134E,
	0x11350,
	0x11351,
	0x11357,
	0x11358,
	0x1135D,
	0x11364,
	0x11366,
	0x1136D,
	0x11370,
	0x11375,
	0x11400,
	0x1145A,
	0x1145B,
	0x1145C,
	0x1145D,
	0x1145E,
	0x11480,
	0x114C8,
	0x114D0,
	0x114DA,
	0x11580,
	0x115B6,
	0x115B8,
	0x115DE,
	0x11600,
	0x11645,
	0x11650,
	0x1165A,
	0x11660,
	0x1166D,
	0x11680,
	0x116B8,
	0x116C0,
	0x116CA,
	0x11700,
	0x1171A,
	0x1171D,
	0x1172C,
	0x11730,
	0x11740,
	0x118A0,
	0x118F3,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C37,
	0x11C38,
	0x11C46,
	0x11C50,
	0x11C6D,
	0x11C70,
	0x11C90,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12470,
	0x12475,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16A60,
	0x16A6A,
	0x16A6E,
	0x16A70,
	0x16AD0,
	0x16AEE,
	0x16AF0,
	0x16AF6,
	0x16B00,
	0x16B46,
	0x16B50,
	0x16B5A,
	0x16B5B,
	0x16B62,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F7F,
	0x16F8F,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1BC9C,
	0x1BCA4,
	0x1D000,
	0x1D0F6,
	0x1D100,
	0x1D127,
	0x1D129,
	0x1D1E9,
	0x1D200,
	0x1D246,
	0x1D300,
	0x1D357,
	0x1D360,
	0x1D372,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D7CC,
	0x1D7CE,
	0x1DA8C,
	0x1DA9B,
	0x1DAA0,
	0x1DAA1,
	0x1DAB0,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E800,
	0x1E8C5,
	0x1E8C7,
	0x1E8D7,
	0x1E900,
	0x1E94B,
	0x1E950,
	0x1E95A,
	0x1E95E,
	0x1E960,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x1EEF0,
	0x1EEF2,
	0x1F000,
	0x1F02C,
	0x1F030,
	0x1F094,
	0x1F0A0,
	0x1F0AF,
	0x1F0B1,
	0x1F0C0,
	0x1F0C1,
	0x1F0D0,
	0x1F0D1,
	0x1F0F6,
	0x1F100,
	0x1F10D,
	0x1F110,
	0x1F12F,
	0x1F130,
	0x1F16C,
	0x1F170,
	0x1F1AD,
	0x1F1E6,
	0x1F203,
	0x1F210,
	0x1F23C,
	0x1F240,
	0x1F249,
	0x1F250,
	0x1F252,
	0x1F300,
	0x1F6D3,
	0x1F6E0,
	0x1F6ED,
	0x1F6F0,
	0x1F6F7,
	0x1F700,
	0x1F774,
	0x1F780,
	0x1F7D5,
	0x1F800,
	0x1F80C,
	0x1F810,
	0x1F848,
	0x1F850,
	0x1F85A,
	0x1F860,
	0x1F888,
	0x1F890,
	0x1F8AE,
	0x1F910,
	0x1F91F,
	0x1F920,
	0x1F928,
	0x1F930,
	0x1F931,
	0x1F933,
	0x1F93F,
	0x1F940,
	0x1F94C,
	0x1F950,
	0x1F95F,
	0x1F980,
	0x1F992,
	0x1F9C0,
	0x1F9C1,
	0x20000,
	0x2A6D7,
	0x2A700,
	0x2B735,
	0x2B740,
	0x2B81E,
	0x2B820,
	0x2CEA2,
	0x2F800,
	0x2FA1E,
	0xE0001,
	0xE0002,
	0xE0020,
	0xE0080,
	0xE0100,
	0xE01F0,
	0xF0000,
	0xFFFFE,
	0x100000,
	0x10FFFE
};

static const UV XPosixLower_invlist[] = { /* for EBCDIC 037 */
	1289,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x42,
	0x4A,
	0x51,
	0x5A,
	0x70,
	0x71,
	0x81,
	0x8A,
	0x8C,
	0x8F,
	0x91,
	0x9D,
	0xA0,
	0xA1,
	0xA2,
	0xAA,
	0xCB,
	0xD0,
	0xDB,
	0xE0,
	0x101,
	0x102,
	0x103,
	0x104,
	0x105,
	0x106,
	0x107,
	0x108,
	0x109,
	0x10A,
	0x10B,
	0x10C,
	0x10D,
	0x10E,
	0x10F,
	0x110,
	0x111,
	0x112,
	0x113,
	0x114,
	0x115,
	0x116,
	0x117,
	0x118,
	0x119,
	0x11A,
	0x11B,
	0x11C,
	0x11D,
	0x11E,
	0x11F,
	0x120,
	0x121,
	0x122,
	0x123,
	0x124,
	0x125,
	0x126,
	0x127,
	0x128,
	0x129,
	0x12A,
	0x12B,
	0x12C,
	0x12D,
	0x12E,
	0x12F,
	0x130,
	0x131,
	0x132,
	0x133,
	0x134,
	0x135,
	0x136,
	0x137,
	0x139,
	0x13A,
	0x13B,
	0x13C,
	0x13D,
	0x13E,
	0x13F,
	0x140,
	0x141,
	0x142,
	0x143,
	0x144,
	0x145,
	0x146,
	0x147,
	0x148,
	0x14A,
	0x14B,
	0x14C,
	0x14D,
	0x14E,
	0x14F,
	0x150,
	0x151,
	0x152,
	0x153,
	0x154,
	0x155,
	0x156,
	0x157,
	0x158,
	0x159,
	0x15A,
	0x15B,
	0x15C,
	0x15D,
	0x15E,
	0x15F,
	0x160,
	0x161,
	0x162,
	0x163,
	0x164,
	0x165,
	0x166,
	0x167,
	0x168,
	0x169,
	0x16A,
	0x16B,
	0x16C,
	0x16D,
	0x16E,
	0x16F,
	0x170,
	0x171,
	0x172,
	0x173,
	0x174,
	0x175,
	0x176,
	0x177,
	0x178,
	0x17A,
	0x17B,
	0x17C,
	0x17D,
	0x17E,
	0x181,
	0x183,
	0x184,
	0x185,
	0x186,
	0x188,
	0x189,
	0x18C,
	0x18E,
	0x192,
	0x193,
	0x195,
	0x196,
	0x199,
	0x19C,
	0x19E,
	0x19F,
	0x1A1,
	0x1A2,
	0x1A3,
	0x1A4,
	0x1A5,
	0x1A6,
	0x1A8,
	0x1A9,
	0x1AA,
	0x1AC,
	0x1AD,
	0x1AE,
	0x1B0,
	0x1B1,
	0x1B4,
	0x1B5,
	0x1B6,
	0x1B7,
	0x1B9,
	0x1BB,
	0x1BD,
	0x1C0,
	0x1C6,
	0x1C7,
	0x1C9,
	0x1CA,
	0x1CC,
	0x1CD,
	0x1CE,
	0x1CF,
	0x1D0,
	0x1D1,
	0x1D2,
	0x1D3,
	0x1D4,
	0x1D5,
	0x1D6,
	0x1D7,
	0x1D8,
	0x1D9,
	0x1DA,
	0x1DB,
	0x1DC,
	0x1DE,
	0x1DF,
	0x1E0,
	0x1E1,
	0x1E2,
	0x1E3,
	0x1E4,
	0x1E5,
	0x1E6,
	0x1E7,
	0x1E8,
	0x1E9,
	0x1EA,
	0x1EB,
	0x1EC,
	0x1ED,
	0x1EE,
	0x1EF,
	0x1F1,
	0x1F3,
	0x1F4,
	0x1F5,
	0x1F6,
	0x1F9,
	0x1FA,
	0x1FB,
	0x1FC,
	0x1FD,
	0x1FE,
	0x1FF,
	0x200,
	0x201,
	0x202,
	0x203,
	0x204,
	0x205,
	0x206,
	0x207,
	0x208,
	0x209,
	0x20A,
	0x20B,
	0x20C,
	0x20D,
	0x20E,
	0x20F,
	0x210,
	0x211,
	0x212,
	0x213,
	0x214,
	0x215,
	0x216,
	0x217,
	0x218,
	0x219,
	0x21A,
	0x21B,
	0x21C,
	0x21D,
	0x21E,
	0x21F,
	0x220,
	0x221,
	0x222,
	0x223,
	0x224,
	0x225,
	0x226,
	0x227,
	0x228,
	0x229,
	0x22A,
	0x22B,
	0x22C,
	0x22D,
	0x22E,
	0x22F,
	0x230,
	0x231,
	0x232,
	0x233,
	0x23A,
	0x23C,
	0x23D,
	0x23F,
	0x241,
	0x242,
	0x243,
	0x247,
	0x248,
	0x249,
	0x24A,
	0x24B,
	0x24C,
	0x24D,
	0x24E,
	0x24F,
	0x294,
	0x295,
	0x2B9,
	0x2C0,
	0x2C2,
	0x2E0,
	0x2E5,
	0x345,
	0x346,
	0x371,
	0x372,
	0x373,
	0x374,
	0x377,
	0x378,
	0x37A,
	0x37E,
	0x390,
	0x391,
	0x3AC,
	0x3CF,
	0x3D0,
	0x3D2,
	0x3D5,
	0x3D8,
	0x3D9,
	0x3DA,
	0x3DB,
	0x3DC,
	0x3DD,
	0x3DE,
	0x3DF,
	0x3E0,
	0x3E1,
	0x3E2,
	0x3E3,
	0x3E4,
	0x3E5,
	0x3E6,
	0x3E7,
	0x3E8,
	0x3E9,
	0x3EA,
	0x3EB,
	0x3EC,
	0x3ED,
	0x3EE,
	0x3EF,
	0x3F4,
	0x3F5,
	0x3F6,
	0x3F8,
	0x3F9,
	0x3FB,
	0x3FD,
	0x430,
	0x460,
	0x461,
	0x462,
	0x463,
	0x464,
	0x465,
	0x466,
	0x467,
	0x468,
	0x469,
	0x46A,
	0x46B,
	0x46C,
	0x46D,
	0x46E,
	0x46F,
	0x470,
	0x471,
	0x472,
	0x473,
	0x474,
	0x475,
	0x476,
	0x477,
	0x478,
	0x479,
	0x47A,
	0x47B,
	0x47C,
	0x47D,
	0x47E,
	0x47F,
	0x480,
	0x481,
	0x482,
	0x48B,
	0x48C,
	0x48D,
	0x48E,
	0x48F,
	0x490,
	0x491,
	0x492,
	0x493,
	0x494,
	0x495,
	0x496,
	0x497,
	0x498,
	0x499,
	0x49A,
	0x49B,
	0x49C,
	0x49D,
	0x49E,
	0x49F,
	0x4A0,
	0x4A1,
	0x4A2,
	0x4A3,
	0x4A4,
	0x4A5,
	0x4A6,
	0x4A7,
	0x4A8,
	0x4A9,
	0x4AA,
	0x4AB,
	0x4AC,
	0x4AD,
	0x4AE,
	0x4AF,
	0x4B0,
	0x4B1,
	0x4B2,
	0x4B3,
	0x4B4,
	0x4B5,
	0x4B6,
	0x4B7,
	0x4B8,
	0x4B9,
	0x4BA,
	0x4BB,
	0x4BC,
	0x4BD,
	0x4BE,
	0x4BF,
	0x4C0,
	0x4C2,
	0x4C3,
	0x4C4,
	0x4C5,
	0x4C6,
	0x4C7,
	0x4C8,
	0x4C9,
	0x4CA,
	0x4CB,
	0x4CC,
	0x4CD,
	0x4CE,
	0x4D0,
	0x4D1,
	0x4D2,
	0x4D3,
	0x4D4,
	0x4D5,
	0x4D6,
	0x4D7,
	0x4D8,
	0x4D9,
	0x4DA,
	0x4DB,
	0x4DC,
	0x4DD,
	0x4DE,
	0x4DF,
	0x4E0,
	0x4E1,
	0x4E2,
	0x4E3,
	0x4E4,
	0x4E5,
	0x4E6,
	0x4E7,
	0x4E8,
	0x4E9,
	0x4EA,
	0x4EB,
	0x4EC,
	0x4ED,
	0x4EE,
	0x4EF,
	0x4F0,
	0x4F1,
	0x4F2,
	0x4F3,
	0x4F4,
	0x4F5,
	0x4F6,
	0x4F7,
	0x4F8,
	0x4F9,
	0x4FA,
	0x4FB,
	0x4FC,
	0x4FD,
	0x4FE,
	0x4FF,
	0x500,
	0x501,
	0x502,
	0x503,
	0x504,
	0x505,
	0x506,
	0x507,
	0x508,
	0x509,
	0x50A,
	0x50B,
	0x50C,
	0x50D,
	0x50E,
	0x50F,
	0x510,
	0x511,
	0x512,
	0x513,
	0x514,
	0x515,
	0x516,
	0x517,
	0x518,
	0x519,
	0x51A,
	0x51B,
	0x51C,
	0x51D,
	0x51E,
	0x51F,
	0x520,
	0x521,
	0x522,
	0x523,
	0x524,
	0x525,
	0x526,
	0x527,
	0x528,
	0x529,
	0x52A,
	0x52B,
	0x52C,
	0x52D,
	0x52E,
	0x52F,
	0x530,
	0x561,
	0x588,
	0x13F8,
	0x13FE,
	0x1C80,
	0x1C89,
	0x1D00,
	0x1DC0,
	0x1E01,
	0x1E02,
	0x1E03,
	0x1E04,
	0x1E05,
	0x1E06,
	0x1E07,
	0x1E08,
	0x1E09,
	0x1E0A,
	0x1E0B,
	0x1E0C,
	0x1E0D,
	0x1E0E,
	0x1E0F,
	0x1E10,
	0x1E11,
	0x1E12,
	0x1E13,
	0x1E14,
	0x1E15,
	0x1E16,
	0x1E17,
	0x1E18,
	0x1E19,
	0x1E1A,
	0x1E1B,
	0x1E1C,
	0x1E1D,
	0x1E1E,
	0x1E1F,
	0x1E20,
	0x1E21,
	0x1E22,
	0x1E23,
	0x1E24,
	0x1E25,
	0x1E26,
	0x1E27,
	0x1E28,
	0x1E29,
	0x1E2A,
	0x1E2B,
	0x1E2C,
	0x1E2D,
	0x1E2E,
	0x1E2F,
	0x1E30,
	0x1E31,
	0x1E32,
	0x1E33,
	0x1E34,
	0x1E35,
	0x1E36,
	0x1E37,
	0x1E38,
	0x1E39,
	0x1E3A,
	0x1E3B,
	0x1E3C,
	0x1E3D,
	0x1E3E,
	0x1E3F,
	0x1E40,
	0x1E41,
	0x1E42,
	0x1E43,
	0x1E44,
	0x1E45,
	0x1E46,
	0x1E47,
	0x1E48,
	0x1E49,
	0x1E4A,
	0x1E4B,
	0x1E4C,
	0x1E4D,
	0x1E4E,
	0x1E4F,
	0x1E50,
	0x1E51,
	0x1E52,
	0x1E53,
	0x1E54,
	0x1E55,
	0x1E56,
	0x1E57,
	0x1E58,
	0x1E59,
	0x1E5A,
	0x1E5B,
	0x1E5C,
	0x1E5D,
	0x1E5E,
	0x1E5F,
	0x1E60,
	0x1E61,
	0x1E62,
	0x1E63,
	0x1E64,
	0x1E65,
	0x1E66,
	0x1E67,
	0x1E68,
	0x1E69,
	0x1E6A,
	0x1E6B,
	0x1E6C,
	0x1E6D,
	0x1E6E,
	0x1E6F,
	0x1E70,
	0x1E71,
	0x1E72,
	0x1E73,
	0x1E74,
	0x1E75,
	0x1E76,
	0x1E77,
	0x1E78,
	0x1E79,
	0x1E7A,
	0x1E7B,
	0x1E7C,
	0x1E7D,
	0x1E7E,
	0x1E7F,
	0x1E80,
	0x1E81,
	0x1E82,
	0x1E83,
	0x1E84,
	0x1E85,
	0x1E86,
	0x1E87,
	0x1E88,
	0x1E89,
	0x1E8A,
	0x1E8B,
	0x1E8C,
	0x1E8D,
	0x1E8E,
	0x1E8F,
	0x1E90,
	0x1E91,
	0x1E92,
	0x1E93,
	0x1E94,
	0x1E95,
	0x1E9E,
	0x1E9F,
	0x1EA0,
	0x1EA1,
	0x1EA2,
	0x1EA3,
	0x1EA4,
	0x1EA5,
	0x1EA6,
	0x1EA7,
	0x1EA8,
	0x1EA9,
	0x1EAA,
	0x1EAB,
	0x1EAC,
	0x1EAD,
	0x1EAE,
	0x1EAF,
	0x1EB0,
	0x1EB1,
	0x1EB2,
	0x1EB3,
	0x1EB4,
	0x1EB5,
	0x1EB6,
	0x1EB7,
	0x1EB8,
	0x1EB9,
	0x1EBA,
	0x1EBB,
	0x1EBC,
	0x1EBD,
	0x1EBE,
	0x1EBF,
	0x1EC0,
	0x1EC1,
	0x1EC2,
	0x1EC3,
	0x1EC4,
	0x1EC5,
	0x1EC6,
	0x1EC7,
	0x1EC8,
	0x1EC9,
	0x1ECA,
	0x1ECB,
	0x1ECC,
	0x1ECD,
	0x1ECE,
	0x1ECF,
	0x1ED0,
	0x1ED1,
	0x1ED2,
	0x1ED3,
	0x1ED4,
	0x1ED5,
	0x1ED6,
	0x1ED7,
	0x1ED8,
	0x1ED9,
	0x1EDA,
	0x1EDB,
	0x1EDC,
	0x1EDD,
	0x1EDE,
	0x1EDF,
	0x1EE0,
	0x1EE1,
	0x1EE2,
	0x1EE3,
	0x1EE4,
	0x1EE5,
	0x1EE6,
	0x1EE7,
	0x1EE8,
	0x1EE9,
	0x1EEA,
	0x1EEB,
	0x1EEC,
	0x1EED,
	0x1EEE,
	0x1EEF,
	0x1EF0,
	0x1EF1,
	0x1EF2,
	0x1EF3,
	0x1EF4,
	0x1EF5,
	0x1EF6,
	0x1EF7,
	0x1EF8,
	0x1EF9,
	0x1EFA,
	0x1EFB,
	0x1EFC,
	0x1EFD,
	0x1EFE,
	0x1EFF,
	0x1F08,
	0x1F10,
	0x1F16,
	0x1F20,
	0x1F28,
	0x1F30,
	0x1F38,
	0x1F40,
	0x1F46,
	0x1F50,
	0x1F58,
	0x1F60,
	0x1F68,
	0x1F70,
	0x1F7E,
	0x1F80,
	0x1F88,
	0x1F90,
	0x1F98,
	0x1FA0,
	0x1FA8,
	0x1FB0,
	0x1FB5,
	0x1FB6,
	0x1FB8,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FC8,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FD8,
	0x1FE0,
	0x1FE8,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FF8,
	0x2071,
	0x2072,
	0x207F,
	0x2080,
	0x2090,
	0x209D,
	0x210A,
	0x210B,
	0x210E,
	0x2110,
	0x2113,
	0x2114,
	0x212F,
	0x2130,
	0x2134,
	0x2135,
	0x2139,
	0x213A,
	0x213C,
	0x213E,
	0x2146,
	0x214A,
	0x214E,
	0x214F,
	0x2170,
	0x2180,
	0x2184,
	0x2185,
	0x24D0,
	0x24EA,
	0x2C30,
	0x2C5F,
	0x2C61,
	0x2C62,
	0x2C65,
	0x2C67,
	0x2C68,
	0x2C69,
	0x2C6A,
	0x2C6B,
	0x2C6C,
	0x2C6D,
	0x2C71,
	0x2C72,
	0x2C73,
	0x2C75,
	0x2C76,
	0x2C7E,
	0x2C81,
	0x2C82,
	0x2C83,
	0x2C84,
	0x2C85,
	0x2C86,
	0x2C87,
	0x2C88,
	0x2C89,
	0x2C8A,
	0x2C8B,
	0x2C8C,
	0x2C8D,
	0x2C8E,
	0x2C8F,
	0x2C90,
	0x2C91,
	0x2C92,
	0x2C93,
	0x2C94,
	0x2C95,
	0x2C96,
	0x2C97,
	0x2C98,
	0x2C99,
	0x2C9A,
	0x2C9B,
	0x2C9C,
	0x2C9D,
	0x2C9E,
	0x2C9F,
	0x2CA0,
	0x2CA1,
	0x2CA2,
	0x2CA3,
	0x2CA4,
	0x2CA5,
	0x2CA6,
	0x2CA7,
	0x2CA8,
	0x2CA9,
	0x2CAA,
	0x2CAB,
	0x2CAC,
	0x2CAD,
	0x2CAE,
	0x2CAF,
	0x2CB0,
	0x2CB1,
	0x2CB2,
	0x2CB3,
	0x2CB4,
	0x2CB5,
	0x2CB6,
	0x2CB7,
	0x2CB8,
	0x2CB9,
	0x2CBA,
	0x2CBB,
	0x2CBC,
	0x2CBD,
	0x2CBE,
	0x2CBF,
	0x2CC0,
	0x2CC1,
	0x2CC2,
	0x2CC3,
	0x2CC4,
	0x2CC5,
	0x2CC6,
	0x2CC7,
	0x2CC8,
	0x2CC9,
	0x2CCA,
	0x2CCB,
	0x2CCC,
	0x2CCD,
	0x2CCE,
	0x2CCF,
	0x2CD0,
	0x2CD1,
	0x2CD2,
	0x2CD3,
	0x2CD4,
	0x2CD5,
	0x2CD6,
	0x2CD7,
	0x2CD8,
	0x2CD9,
	0x2CDA,
	0x2CDB,
	0x2CDC,
	0x2CDD,
	0x2CDE,
	0x2CDF,
	0x2CE0,
	0x2CE1,
	0x2CE2,
	0x2CE3,
	0x2CE5,
	0x2CEC,
	0x2CED,
	0x2CEE,
	0x2CEF,
	0x2CF3,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0xA641,
	0xA642,
	0xA643,
	0xA644,
	0xA645,
	0xA646,
	0xA647,
	0xA648,
	0xA649,
	0xA64A,
	0xA64B,
	0xA64C,
	0xA64D,
	0xA64E,
	0xA64F,
	0xA650,
	0xA651,
	0xA652,
	0xA653,
	0xA654,
	0xA655,
	0xA656,
	0xA657,
	0xA658,
	0xA659,
	0xA65A,
	0xA65B,
	0xA65C,
	0xA65D,
	0xA65E,
	0xA65F,
	0xA660,
	0xA661,
	0xA662,
	0xA663,
	0xA664,
	0xA665,
	0xA666,
	0xA667,
	0xA668,
	0xA669,
	0xA66A,
	0xA66B,
	0xA66C,
	0xA66D,
	0xA66E,
	0xA681,
	0xA682,
	0xA683,
	0xA684,
	0xA685,
	0xA686,
	0xA687,
	0xA688,
	0xA689,
	0xA68A,
	0xA68B,
	0xA68C,
	0xA68D,
	0xA68E,
	0xA68F,
	0xA690,
	0xA691,
	0xA692,
	0xA693,
	0xA694,
	0xA695,
	0xA696,
	0xA697,
	0xA698,
	0xA699,
	0xA69A,
	0xA69B,
	0xA69E,
	0xA723,
	0xA724,
	0xA725,
	0xA726,
	0xA727,
	0xA728,
	0xA729,
	0xA72A,
	0xA72B,
	0xA72C,
	0xA72D,
	0xA72E,
	0xA72F,
	0xA732,
	0xA733,
	0xA734,
	0xA735,
	0xA736,
	0xA737,
	0xA738,
	0xA739,
	0xA73A,
	0xA73B,
	0xA73C,
	0xA73D,
	0xA73E,
	0xA73F,
	0xA740,
	0xA741,
	0xA742,
	0xA743,
	0xA744,
	0xA745,
	0xA746,
	0xA747,
	0xA748,
	0xA749,
	0xA74A,
	0xA74B,
	0xA74C,
	0xA74D,
	0xA74E,
	0xA74F,
	0xA750,
	0xA751,
	0xA752,
	0xA753,
	0xA754,
	0xA755,
	0xA756,
	0xA757,
	0xA758,
	0xA759,
	0xA75A,
	0xA75B,
	0xA75C,
	0xA75D,
	0xA75E,
	0xA75F,
	0xA760,
	0xA761,
	0xA762,
	0xA763,
	0xA764,
	0xA765,
	0xA766,
	0xA767,
	0xA768,
	0xA769,
	0xA76A,
	0xA76B,
	0xA76C,
	0xA76D,
	0xA76E,
	0xA76F,
	0xA779,
	0xA77A,
	0xA77B,
	0xA77C,
	0xA77D,
	0xA77F,
	0xA780,
	0xA781,
	0xA782,
	0xA783,
	0xA784,
	0xA785,
	0xA786,
	0xA787,
	0xA788,
	0xA78C,
	0xA78D,
	0xA78E,
	0xA78F,
	0xA791,
	0xA792,
	0xA793,
	0xA796,
	0xA797,
	0xA798,
	0xA799,
	0xA79A,
	0xA79B,
	0xA79C,
	0xA79D,
	0xA79E,
	0xA79F,
	0xA7A0,
	0xA7A1,
	0xA7A2,
	0xA7A3,
	0xA7A4,
	0xA7A5,
	0xA7A6,
	0xA7A7,
	0xA7A8,
	0xA7A9,
	0xA7AA,
	0xA7B5,
	0xA7B6,
	0xA7B7,
	0xA7B8,
	0xA7F8,
	0xA7FB,
	0xAB30,
	0xAB5B,
	0xAB5C,
	0xAB66,
	0xAB70,
	0xABC0,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFF41,
	0xFF5B,
	0x10428,
	0x10450,
	0x104D8,
	0x104FC,
	0x10CC0,
	0x10CF3,
	0x118C0,
	0x118E0,
	0x1D41A,
	0x1D434,
	0x1D44E,
	0x1D455,
	0x1D456,
	0x1D468,
	0x1D482,
	0x1D49C,
	0x1D4B6,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D4D0,
	0x1D4EA,
	0x1D504,
	0x1D51E,
	0x1D538,
	0x1D552,
	0x1D56C,
	0x1D586,
	0x1D5A0,
	0x1D5BA,
	0x1D5D4,
	0x1D5EE,
	0x1D608,
	0x1D622,
	0x1D63C,
	0x1D656,
	0x1D670,
	0x1D68A,
	0x1D6A6,
	0x1D6C2,
	0x1D6DB,
	0x1D6DC,
	0x1D6E2,
	0x1D6FC,
	0x1D715,
	0x1D716,
	0x1D71C,
	0x1D736,
	0x1D74F,
	0x1D750,
	0x1D756,
	0x1D770,
	0x1D789,
	0x1D78A,
	0x1D790,
	0x1D7AA,
	0x1D7C3,
	0x1D7C4,
	0x1D7CA,
	0x1D7CB,
	0x1D7CC,
	0x1E922,
	0x1E944
};

static const UV XPosixPrint_invlist[] = { /* for EBCDIC 037 */
	1281,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x40,
	0xFF,
	0x100,
	0x378,
	0x37A,
	0x380,
	0x384,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x530,
	0x531,
	0x557,
	0x559,
	0x560,
	0x561,
	0x588,
	0x589,
	0x58B,
	0x58D,
	0x590,
	0x591,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F5,
	0x600,
	0x61D,
	0x61E,
	0x70E,
	0x70F,
	0x74B,
	0x74D,
	0x7B2,
	0x7C0,
	0x7FB,
	0x800,
	0x82E,
	0x830,
	0x83F,
	0x840,
	0x85C,
	0x85E,
	0x85F,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x8D4,
	0x984,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BC,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CF,
	0x9D7,
	0x9D8,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E4,
	0x9E6,
	0x9FC,
	0xA01,
	0xA04,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA3C,
	0xA3D,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4E,
	0xA51,
	0xA52,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA66,
	0xA76,
	0xA81,
	0xA84,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABC,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACE,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE4,
	0xAE6,
	0xAF2,
	0xAF9,
	0xAFA,
	0xB01,
	0xB04,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3C,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4E,
	0xB56,
	0xB58,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB64,
	0xB66,
	0xB78,
	0xB82,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCE,
	0xBD0,
	0xBD1,
	0xBD7,
	0xBD8,
	0xBE6,
	0xBFB,
	0xC00,
	0xC04,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4E,
	0xC55,
	0xC57,
	0xC58,
	0xC5B,
	0xC60,
	0xC64,
	0xC66,
	0xC70,
	0xC78,
	0xC84,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBC,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCE,
	0xCD5,
	0xCD7,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE4,
	0xCE6,
	0xCF0,
	0xCF1,
	0xCF3,
	0xD01,
	0xD04,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD50,
	0xD54,
	0xD64,
	0xD66,
	0xD80,
	0xD82,
	0xD84,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xDCA,
	0xDCB,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDE6,
	0xDF0,
	0xDF2,
	0xDF5,
	0xE01,
	0xE3B,
	0xE3F,
	0xE5C,
	0xE81,
	0xE83,
	0xE84,
	0xE85,
	0xE87,
	0xE89,
	0xE8A,
	0xE8B,
	0xE8D,
	0xE8E,
	0xE94,
	0xE98,
	0xE99,
	0xEA0,
	0xEA1,
	0xEA4,
	0xEA5,
	0xEA6,
	0xEA7,
	0xEA8,
	0xEAA,
	0xEAC,
	0xEAD,
	0xEBA,
	0xEBB,
	0xEBE,
	0xEC0,
	0xEC5,
	0xEC6,
	0xEC7,
	0xEC8,
	0xECE,
	0xED0,
	0xEDA,
	0xEDC,
	0xEE0,
	0xF00,
	0xF48,
	0xF49,
	0xF6D,
	0xF71,
	0xF98,
	0xF99,
	0xFBD,
	0xFBE,
	0xFCD,
	0xFCE,
	0xFDB,
	0x1000,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x135D,
	0x137D,
	0x1380,
	0x139A,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1400,
	0x169D,
	0x16A0,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1715,
	0x1720,
	0x1737,
	0x1740,
	0x1754,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1772,
	0x1774,
	0x1780,
	0x17DE,
	0x17E0,
	0x17EA,
	0x17F0,
	0x17FA,
	0x1800,
	0x180F,
	0x1810,
	0x181A,
	0x1820,
	0x1878,
	0x1880,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1920,
	0x192C,
	0x1930,
	0x193C,
	0x1940,
	0x1941,
	0x1944,
	0x196E,
	0x1970,
	0x1975,
	0x1980,
	0x19AC,
	0x19B0,
	0x19CA,
	0x19D0,
	0x19DB,
	0x19DE,
	0x1A1C,
	0x1A1E,
	0x1A5F,
	0x1A60,
	0x1A7D,
	0x1A7F,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1AA0,
	0x1AAE,
	0x1AB0,
	0x1ABF,
	0x1B00,
	0x1B4C,
	0x1B50,
	0x1B7D,
	0x1B80,
	0x1BF4,
	0x1BFC,
	0x1C38,
	0x1C3B,
	0x1C4A,
	0x1C4D,
	0x1C89,
	0x1CC0,
	0x1CC8,
	0x1CD0,
	0x1CF7,
	0x1CF8,
	0x1CFA,
	0x1D00,
	0x1DF6,
	0x1DFB,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FC5,
	0x1FC6,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FDD,
	0x1FF0,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFF,
	0x2000,
	0x2028,
	0x202A,
	0x2065,
	0x2066,
	0x2072,
	0x2074,
	0x208F,
	0x2090,
	0x209D,
	0x20A0,
	0x20BF,
	0x20D0,
	0x20F1,
	0x2100,
	0x218C,
	0x2190,
	0x23FF,
	0x2400,
	0x2427,
	0x2440,
	0x244B,
	0x2460,
	0x2B74,
	0x2B76,
	0x2B96,
	0x2B98,
	0x2BBA,
	0x2BBD,
	0x2BC9,
	0x2BCA,
	0x2BD2,
	0x2BEC,
	0x2BF0,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CF4,
	0x2CF9,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D71,
	0x2D7F,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x2DE0,
	0x2E45,
	0x2E80,
	0x2E9A,
	0x2E9B,
	0x2EF4,
	0x2F00,
	0x2FD6,
	0x2FF0,
	0x2FFC,
	0x3000,
	0x3040,
	0x3041,
	0x3097,
	0x3099,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x3190,
	0x31BB,
	0x31C0,
	0x31E4,
	0x31F0,
	0x321F,
	0x3220,
	0x32FF,
	0x3300,
	0x4DB6,
	0x4DC0,
	0x9FD6,
	0xA000,
	0xA48D,
	0xA490,
	0xA4C7,
	0xA4D0,
	0xA62C,
	0xA640,
	0xA6F8,
	0xA700,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F7,
	0xA82C,
	0xA830,
	0xA83A,
	0xA840,
	0xA878,
	0xA880,
	0xA8C6,
	0xA8CE,
	0xA8DA,
	0xA8E0,
	0xA8FE,
	0xA900,
	0xA954,
	0xA95F,
	0xA97D,
	0xA980,
	0xA9CE,
	0xA9CF,
	0xA9DA,
	0xA9DE,
	0xA9FF,
	0xAA00,
	0xAA37,
	0xAA40,
	0xAA4E,
	0xAA50,
	0xAA5A,
	0xAA5C,
	0xAAC3,
	0xAADB,
	0xAAF7,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB66,
	0xAB70,
	0xABEE,
	0xABF0,
	0xABFA,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xE000,
	0xFA6E,
	0xFA70,
	0xFADA,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFBC2,
	0xFBD3,
	0xFD40,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFE,
	0xFE00,
	0xFE1A,
	0xFE20,
	0xFE53,
	0xFE54,
	0xFE67,
	0xFE68,
	0xFE6C,
	0xFE70,
	0xFE75,
	0xFE76,
	0xFEFD,
	0xFEFF,
	0xFF00,
	0xFF01,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0xFFE0,
	0xFFE7,
	0xFFE8,
	0xFFEF,
	0xFFF9,
	0xFFFE,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10100,
	0x10103,
	0x10107,
	0x10134,
	0x10137,
	0x1018F,
	0x10190,
	0x1019C,
	0x101A0,
	0x101A1,
	0x101D0,
	0x101FE,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x102E0,
	0x102FC,
	0x10300,
	0x10324,
	0x10330,
	0x1034B,
	0x10350,
	0x1037B,
	0x10380,
	0x1039E,
	0x1039F,
	0x103C4,
	0x103C8,
	0x103D6,
	0x10400,
	0x1049E,
	0x104A0,
	0x104AA,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x1056F,
	0x10570,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10857,
	0x1089F,
	0x108A7,
	0x108B0,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x108FB,
	0x1091C,
	0x1091F,
	0x1093A,
	0x1093F,
	0x10940,
	0x10980,
	0x109B8,
	0x109BC,
	0x109D0,
	0x109D2,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A38,
	0x10A3B,
	0x10A3F,
	0x10A48,
	0x10A50,
	0x10A59,
	0x10A60,
	0x10AA0,
	0x10AC0,
	0x10AE7,
	0x10AEB,
	0x10AF7,
	0x10B00,
	0x10B36,
	0x10B39,
	0x10B56,
	0x10B58,
	0x10B73,
	0x10B78,
	0x10B92,
	0x10B99,
	0x10B9D,
	0x10BA9,
	0x10BB0,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x10CFA,
	0x10D00,
	0x10E60,
	0x10E7F,
	0x11000,
	0x1104E,
	0x11052,
	0x11070,
	0x1107F,
	0x110C2,
	0x110D0,
	0x110E9,
	0x110F0,
	0x110FA,
	0x11100,
	0x11135,
	0x11136,
	0x11144,
	0x11150,
	0x11177,
	0x11180,
	0x111CE,
	0x111D0,
	0x111E0,
	0x111E1,
	0x111F5,
	0x11200,
	0x11212,
	0x11213,
	0x1123F,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112AA,
	0x112B0,
	0x112EB,
	0x112F0,
	0x112FA,
	0x11300,
	0x11304,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133C,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134E,
	0x11350,
	0x11351,
	0x11357,
	0x11358,
	0x1135D,
	0x11364,
	0x11366,
	0x1136D,
	0x11370,
	0x11375,
	0x11400,
	0x1145A,
	0x1145B,
	0x1145C,
	0x1145D,
	0x1145E,
	0x11480,
	0x114C8,
	0x114D0,
	0x114DA,
	0x11580,
	0x115B6,
	0x115B8,
	0x115DE,
	0x11600,
	0x11645,
	0x11650,
	0x1165A,
	0x11660,
	0x1166D,
	0x11680,
	0x116B8,
	0x116C0,
	0x116CA,
	0x11700,
	0x1171A,
	0x1171D,
	0x1172C,
	0x11730,
	0x11740,
	0x118A0,
	0x118F3,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C37,
	0x11C38,
	0x11C46,
	0x11C50,
	0x11C6D,
	0x11C70,
	0x11C90,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12470,
	0x12475,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16A60,
	0x16A6A,
	0x16A6E,
	0x16A70,
	0x16AD0,
	0x16AEE,
	0x16AF0,
	0x16AF6,
	0x16B00,
	0x16B46,
	0x16B50,
	0x16B5A,
	0x16B5B,
	0x16B62,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F7F,
	0x16F8F,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1BC9C,
	0x1BCA4,
	0x1D000,
	0x1D0F6,
	0x1D100,
	0x1D127,
	0x1D129,
	0x1D1E9,
	0x1D200,
	0x1D246,
	0x1D300,
	0x1D357,
	0x1D360,
	0x1D372,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D7CC,
	0x1D7CE,
	0x1DA8C,
	0x1DA9B,
	0x1DAA0,
	0x1DAA1,
	0x1DAB0,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E800,
	0x1E8C5,
	0x1E8C7,
	0x1E8D7,
	0x1E900,
	0x1E94B,
	0x1E950,
	0x1E95A,
	0x1E95E,
	0x1E960,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x1EEF0,
	0x1EEF2,
	0x1F000,
	0x1F02C,
	0x1F030,
	0x1F094,
	0x1F0A0,
	0x1F0AF,
	0x1F0B1,
	0x1F0C0,
	0x1F0C1,
	0x1F0D0,
	0x1F0D1,
	0x1F0F6,
	0x1F100,
	0x1F10D,
	0x1F110,
	0x1F12F,
	0x1F130,
	0x1F16C,
	0x1F170,
	0x1F1AD,
	0x1F1E6,
	0x1F203,
	0x1F210,
	0x1F23C,
	0x1F240,
	0x1F249,
	0x1F250,
	0x1F252,
	0x1F300,
	0x1F6D3,
	0x1F6E0,
	0x1F6ED,
	0x1F6F0,
	0x1F6F7,
	0x1F700,
	0x1F774,
	0x1F780,
	0x1F7D5,
	0x1F800,
	0x1F80C,
	0x1F810,
	0x1F848,
	0x1F850,
	0x1F85A,
	0x1F860,
	0x1F888,
	0x1F890,
	0x1F8AE,
	0x1F910,
	0x1F91F,
	0x1F920,
	0x1F928,
	0x1F930,
	0x1F931,
	0x1F933,
	0x1F93F,
	0x1F940,
	0x1F94C,
	0x1F950,
	0x1F95F,
	0x1F980,
	0x1F992,
	0x1F9C0,
	0x1F9C1,
	0x20000,
	0x2A6D7,
	0x2A700,
	0x2B735,
	0x2B740,
	0x2B81E,
	0x2B820,
	0x2CEA2,
	0x2F800,
	0x2FA1E,
	0xE0001,
	0xE0002,
	0xE0020,
	0xE0080,
	0xE0100,
	0xE01F0,
	0xF0000,
	0xFFFFE,
	0x100000,
	0x10FFFE
};

static const UV XPosixPunct_invlist[] = { /* for EBCDIC 037 */
	337,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x4B,
	0x51,
	0x5A,
	0x5F,
	0x60,
	0x62,
	0x6B,
	0x70,
	0x79,
	0x80,
	0x8A,
	0x8C,
	0xA1,
	0xA2,
	0xAA,
	0xAC,
	0xB0,
	0xB1,
	0xB3,
	0xB4,
	0xB5,
	0xB7,
	0xBA,
	0xBC,
	0xC0,
	0xC1,
	0xD0,
	0xD1,
	0xE0,
	0xE1,
	0x37E,
	0x37F,
	0x387,
	0x388,
	0x55A,
	0x560,
	0x589,
	0x58B,
	0x5BE,
	0x5BF,
	0x5C0,
	0x5C1,
	0x5C3,
	0x5C4,
	0x5C6,
	0x5C7,
	0x5F3,
	0x5F5,
	0x609,
	0x60B,
	0x60C,
	0x60E,
	0x61B,
	0x61C,
	0x61E,
	0x620,
	0x66A,
	0x66E,
	0x6D4,
	0x6D5,
	0x700,
	0x70E,
	0x7F7,
	0x7FA,
	0x830,
	0x83F,
	0x85E,
	0x85F,
	0x964,
	0x966,
	0x970,
	0x971,
	0xAF0,
	0xAF1,
	0xDF4,
	0xDF5,
	0xE4F,
	0xE50,
	0xE5A,
	0xE5C,
	0xF04,
	0xF13,
	0xF14,
	0xF15,
	0xF3A,
	0xF3E,
	0xF85,
	0xF86,
	0xFD0,
	0xFD5,
	0xFD9,
	0xFDB,
	0x104A,
	0x1050,
	0x10FB,
	0x10FC,
	0x1360,
	0x1369,
	0x1400,
	0x1401,
	0x166D,
	0x166F,
	0x169B,
	0x169D,
	0x16EB,
	0x16EE,
	0x1735,
	0x1737,
	0x17D4,
	0x17D7,
	0x17D8,
	0x17DB,
	0x1800,
	0x180B,
	0x1944,
	0x1946,
	0x1A1E,
	0x1A20,
	0x1AA0,
	0x1AA7,
	0x1AA8,
	0x1AAE,
	0x1B5A,
	0x1B61,
	0x1BFC,
	0x1C00,
	0x1C3B,
	0x1C40,
	0x1C7E,
	0x1C80,
	0x1CC0,
	0x1CC8,
	0x1CD3,
	0x1CD4,
	0x2010,
	0x2028,
	0x2030,
	0x2044,
	0x2045,
	0x2052,
	0x2053,
	0x205F,
	0x207D,
	0x207F,
	0x208D,
	0x208F,
	0x2308,
	0x230C,
	0x2329,
	0x232B,
	0x2768,
	0x2776,
	0x27C5,
	0x27C7,
	0x27E6,
	0x27F0,
	0x2983,
	0x2999,
	0x29D8,
	0x29DC,
	0x29FC,
	0x29FE,
	0x2CF9,
	0x2CFD,
	0x2CFE,
	0x2D00,
	0x2D70,
	0x2D71,
	0x2E00,
	0x2E2F,
	0x2E30,
	0x2E45,
	0x3001,
	0x3004,
	0x3008,
	0x3012,
	0x3014,
	0x3020,
	0x3030,
	0x3031,
	0x303D,
	0x303E,
	0x30A0,
	0x30A1,
	0x30FB,
	0x30FC,
	0xA4FE,
	0xA500,
	0xA60D,
	0xA610,
	0xA673,
	0xA674,
	0xA67E,
	0xA67F,
	0xA6F2,
	0xA6F8,
	0xA874,
	0xA878,
	0xA8CE,
	0xA8D0,
	0xA8F8,
	0xA8FB,
	0xA8FC,
	0xA8FD,
	0xA92E,
	0xA930,
	0xA95F,
	0xA960,
	0xA9C1,
	0xA9CE,
	0xA9DE,
	0xA9E0,
	0xAA5C,
	0xAA60,
	0xAADE,
	0xAAE0,
	0xAAF0,
	0xAAF2,
	0xABEB,
	0xABEC,
	0xFD3E,
	0xFD40,
	0xFE10,
	0xFE1A,
	0xFE30,
	0xFE53,
	0xFE54,
	0xFE62,
	0xFE63,
	0xFE64,
	0xFE68,
	0xFE69,
	0xFE6A,
	0xFE6C,
	0xFF01,
	0xFF04,
	0xFF05,
	0xFF0B,
	0xFF0C,
	0xFF10,
	0xFF1A,
	0xFF1C,
	0xFF1F,
	0xFF21,
	0xFF3B,
	0xFF3E,
	0xFF3F,
	0xFF40,
	0xFF5B,
	0xFF5C,
	0xFF5D,
	0xFF5E,
	0xFF5F,
	0xFF66,
	0x10100,
	0x10103,
	0x1039F,
	0x103A0,
	0x103D0,
	0x103D1,
	0x1056F,
	0x10570,
	0x10857,
	0x10858,
	0x1091F,
	0x10920,
	0x1093F,
	0x10940,
	0x10A50,
	0x10A59,
	0x10A7F,
	0x10A80,
	0x10AF0,
	0x10AF7,
	0x10B39,
	0x10B40,
	0x10B99,
	0x10B9D,
	0x11047,
	0x1104E,
	0x110BB,
	0x110BD,
	0x110BE,
	0x110C2,
	0x11140,
	0x11144,
	0x11174,
	0x11176,
	0x111C5,
	0x111CA,
	0x111CD,
	0x111CE,
	0x111DB,
	0x111DC,
	0x111DD,
	0x111E0,
	0x11238,
	0x1123E,
	0x112A9,
	0x112AA,
	0x1144B,
	0x11450,
	0x1145B,
	0x1145C,
	0x1145D,
	0x1145E,
	0x114C6,
	0x114C7,
	0x115C1,
	0x115D8,
	0x11641,
	0x11644,
	0x11660,
	0x1166D,
	0x1173C,
	0x1173F,
	0x11C41,
	0x11C46,
	0x11C70,
	0x11C72,
	0x12470,
	0x12475,
	0x16A6E,
	0x16A70,
	0x16AF5,
	0x16AF6,
	0x16B37,
	0x16B3C,
	0x16B44,
	0x16B45,
	0x1BC9F,
	0x1BCA0,
	0x1DA87,
	0x1DA8C,
	0x1E95E,
	0x1E960
};

static const UV XPosixSpace_invlist[] = { /* for EBCDIC 037 */
	23,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x5,
	0x6,
	0xB,
	0xE,
	0x15,
	0x16,
	0x25,
	0x26,
	0x40,
	0x42,
	0x1680,
	0x1681,
	0x2000,
	0x200B,
	0x2028,
	0x202A,
	0x202F,
	0x2030,
	0x205F,
	0x2060,
	0x3000,
	0x3001
};

static const UV XPosixUpper_invlist[] = { /* for EBCDIC 037 */
	1279,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x62,
	0x6A,
	0x71,
	0x79,
	0x80,
	0x81,
	0x9E,
	0x9F,
	0xAC,
	0xAF,
	0xC1,
	0xCA,
	0xD1,
	0xDA,
	0xE2,
	0xEA,
	0xEB,
	0xF0,
	0xFB,
	0xFF,
	0x100,
	0x101,
	0x102,
	0x103,
	0x104,
	0x105,
	0x106,
	0x107,
	0x108,
	0x109,
	0x10A,
	0x10B,
	0x10C,
	0x10D,
	0x10E,
	0x10F,
	0x110,
	0x111,
	0x112,
	0x113,
	0x114,
	0x115,
	0x116,
	0x117,
	0x118,
	0x119,
	0x11A,
	0x11B,
	0x11C,
	0x11D,
	0x11E,
	0x11F,
	0x120,
	0x121,
	0x122,
	0x123,
	0x124,
	0x125,
	0x126,
	0x127,
	0x128,
	0x129,
	0x12A,
	0x12B,
	0x12C,
	0x12D,
	0x12E,
	0x12F,
	0x130,
	0x131,
	0x132,
	0x133,
	0x134,
	0x135,
	0x136,
	0x137,
	0x139,
	0x13A,
	0x13B,
	0x13C,
	0x13D,
	0x13E,
	0x13F,
	0x140,
	0x141,
	0x142,
	0x143,
	0x144,
	0x145,
	0x146,
	0x147,
	0x148,
	0x14A,
	0x14B,
	0x14C,
	0x14D,
	0x14E,
	0x14F,
	0x150,
	0x151,
	0x152,
	0x153,
	0x154,
	0x155,
	0x156,
	0x157,
	0x158,
	0x159,
	0x15A,
	0x15B,
	0x15C,
	0x15D,
	0x15E,
	0x15F,
	0x160,
	0x161,
	0x162,
	0x163,
	0x164,
	0x165,
	0x166,
	0x167,
	0x168,
	0x169,
	0x16A,
	0x16B,
	0x16C,
	0x16D,
	0x16E,
	0x16F,
	0x170,
	0x171,
	0x172,
	0x173,
	0x174,
	0x175,
	0x176,
	0x177,
	0x178,
	0x17A,
	0x17B,
	0x17C,
	0x17D,
	0x17E,
	0x181,
	0x183,
	0x184,
	0x185,
	0x186,
	0x188,
	0x189,
	0x18C,
	0x18E,
	0x192,
	0x193,
	0x195,
	0x196,
	0x199,
	0x19C,
	0x19E,
	0x19F,
	0x1A1,
	0x1A2,
	0x1A3,
	0x1A4,
	0x1A5,
	0x1A6,
	0x1A8,
	0x1A9,
	0x1AA,
	0x1AC,
	0x1AD,
	0x1AE,
	0x1B0,
	0x1B1,
	0x1B4,
	0x1B5,
	0x1B6,
	0x1B7,
	0x1B9,
	0x1BC,
	0x1BD,
	0x1C4,
	0x1C5,
	0x1C7,
	0x1C8,
	0x1CA,
	0x1CB,
	0x1CD,
	0x1CE,
	0x1CF,
	0x1D0,
	0x1D1,
	0x1D2,
	0x1D3,
	0x1D4,
	0x1D5,
	0x1D6,
	0x1D7,
	0x1D8,
	0x1D9,
	0x1DA,
	0x1DB,
	0x1DC,
	0x1DE,
	0x1DF,
	0x1E0,
	0x1E1,
	0x1E2,
	0x1E3,
	0x1E4,
	0x1E5,
	0x1E6,
	0x1E7,
	0x1E8,
	0x1E9,
	0x1EA,
	0x1EB,
	0x1EC,
	0x1ED,
	0x1EE,
	0x1EF,
	0x1F1,
	0x1F2,
	0x1F4,
	0x1F5,
	0x1F6,
	0x1F9,
	0x1FA,
	0x1FB,
	0x1FC,
	0x1FD,
	0x1FE,
	0x1FF,
	0x200,
	0x201,
	0x202,
	0x203,
	0x204,
	0x205,
	0x206,
	0x207,
	0x208,
	0x209,
	0x20A,
	0x20B,
	0x20C,
	0x20D,
	0x20E,
	0x20F,
	0x210,
	0x211,
	0x212,
	0x213,
	0x214,
	0x215,
	0x216,
	0x217,
	0x218,
	0x219,
	0x21A,
	0x21B,
	0x21C,
	0x21D,
	0x21E,
	0x21F,
	0x220,
	0x221,
	0x222,
	0x223,
	0x224,
	0x225,
	0x226,
	0x227,
	0x228,
	0x229,
	0x22A,
	0x22B,
	0x22C,
	0x22D,
	0x22E,
	0x22F,
	0x230,
	0x231,
	0x232,
	0x233,
	0x23A,
	0x23C,
	0x23D,
	0x23F,
	0x241,
	0x242,
	0x243,
	0x247,
	0x248,
	0x249,
	0x24A,
	0x24B,
	0x24C,
	0x24D,
	0x24E,
	0x24F,
	0x370,
	0x371,
	0x372,
	0x373,
	0x376,
	0x377,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x390,
	0x391,
	0x3A2,
	0x3A3,
	0x3AC,
	0x3CF,
	0x3D0,
	0x3D2,
	0x3D5,
	0x3D8,
	0x3D9,
	0x3DA,
	0x3DB,
	0x3DC,
	0x3DD,
	0x3DE,
	0x3DF,
	0x3E0,
	0x3E1,
	0x3E2,
	0x3E3,
	0x3E4,
	0x3E5,
	0x3E6,
	0x3E7,
	0x3E8,
	0x3E9,
	0x3EA,
	0x3EB,
	0x3EC,
	0x3ED,
	0x3EE,
	0x3EF,
	0x3F4,
	0x3F5,
	0x3F7,
	0x3F8,
	0x3F9,
	0x3FB,
	0x3FD,
	0x430,
	0x460,
	0x461,
	0x462,
	0x463,
	0x464,
	0x465,
	0x466,
	0x467,
	0x468,
	0x469,
	0x46A,
	0x46B,
	0x46C,
	0x46D,
	0x46E,
	0x46F,
	0x470,
	0x471,
	0x472,
	0x473,
	0x474,
	0x475,
	0x476,
	0x477,
	0x478,
	0x479,
	0x47A,
	0x47B,
	0x47C,
	0x47D,
	0x47E,
	0x47F,
	0x480,
	0x481,
	0x48A,
	0x48B,
	0x48C,
	0x48D,
	0x48E,
	0x48F,
	0x490,
	0x491,
	0x492,
	0x493,
	0x494,
	0x495,
	0x496,
	0x497,
	0x498,
	0x499,
	0x49A,
	0x49B,
	0x49C,
	0x49D,
	0x49E,
	0x49F,
	0x4A0,
	0x4A1,
	0x4A2,
	0x4A3,
	0x4A4,
	0x4A5,
	0x4A6,
	0x4A7,
	0x4A8,
	0x4A9,
	0x4AA,
	0x4AB,
	0x4AC,
	0x4AD,
	0x4AE,
	0x4AF,
	0x4B0,
	0x4B1,
	0x4B2,
	0x4B3,
	0x4B4,
	0x4B5,
	0x4B6,
	0x4B7,
	0x4B8,
	0x4B9,
	0x4BA,
	0x4BB,
	0x4BC,
	0x4BD,
	0x4BE,
	0x4BF,
	0x4C0,
	0x4C2,
	0x4C3,
	0x4C4,
	0x4C5,
	0x4C6,
	0x4C7,
	0x4C8,
	0x4C9,
	0x4CA,
	0x4CB,
	0x4CC,
	0x4CD,
	0x4CE,
	0x4D0,
	0x4D1,
	0x4D2,
	0x4D3,
	0x4D4,
	0x4D5,
	0x4D6,
	0x4D7,
	0x4D8,
	0x4D9,
	0x4DA,
	0x4DB,
	0x4DC,
	0x4DD,
	0x4DE,
	0x4DF,
	0x4E0,
	0x4E1,
	0x4E2,
	0x4E3,
	0x4E4,
	0x4E5,
	0x4E6,
	0x4E7,
	0x4E8,
	0x4E9,
	0x4EA,
	0x4EB,
	0x4EC,
	0x4ED,
	0x4EE,
	0x4EF,
	0x4F0,
	0x4F1,
	0x4F2,
	0x4F3,
	0x4F4,
	0x4F5,
	0x4F6,
	0x4F7,
	0x4F8,
	0x4F9,
	0x4FA,
	0x4FB,
	0x4FC,
	0x4FD,
	0x4FE,
	0x4FF,
	0x500,
	0x501,
	0x502,
	0x503,
	0x504,
	0x505,
	0x506,
	0x507,
	0x508,
	0x509,
	0x50A,
	0x50B,
	0x50C,
	0x50D,
	0x50E,
	0x50F,
	0x510,
	0x511,
	0x512,
	0x513,
	0x514,
	0x515,
	0x516,
	0x517,
	0x518,
	0x519,
	0x51A,
	0x51B,
	0x51C,
	0x51D,
	0x51E,
	0x51F,
	0x520,
	0x521,
	0x522,
	0x523,
	0x524,
	0x525,
	0x526,
	0x527,
	0x528,
	0x529,
	0x52A,
	0x52B,
	0x52C,
	0x52D,
	0x52E,
	0x52F,
	0x531,
	0x557,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x13A0,
	0x13F6,
	0x1E00,
	0x1E01,
	0x1E02,
	0x1E03,
	0x1E04,
	0x1E05,
	0x1E06,
	0x1E07,
	0x1E08,
	0x1E09,
	0x1E0A,
	0x1E0B,
	0x1E0C,
	0x1E0D,
	0x1E0E,
	0x1E0F,
	0x1E10,
	0x1E11,
	0x1E12,
	0x1E13,
	0x1E14,
	0x1E15,
	0x1E16,
	0x1E17,
	0x1E18,
	0x1E19,
	0x1E1A,
	0x1E1B,
	0x1E1C,
	0x1E1D,
	0x1E1E,
	0x1E1F,
	0x1E20,
	0x1E21,
	0x1E22,
	0x1E23,
	0x1E24,
	0x1E25,
	0x1E26,
	0x1E27,
	0x1E28,
	0x1E29,
	0x1E2A,
	0x1E2B,
	0x1E2C,
	0x1E2D,
	0x1E2E,
	0x1E2F,
	0x1E30,
	0x1E31,
	0x1E32,
	0x1E33,
	0x1E34,
	0x1E35,
	0x1E36,
	0x1E37,
	0x1E38,
	0x1E39,
	0x1E3A,
	0x1E3B,
	0x1E3C,
	0x1E3D,
	0x1E3E,
	0x1E3F,
	0x1E40,
	0x1E41,
	0x1E42,
	0x1E43,
	0x1E44,
	0x1E45,
	0x1E46,
	0x1E47,
	0x1E48,
	0x1E49,
	0x1E4A,
	0x1E4B,
	0x1E4C,
	0x1E4D,
	0x1E4E,
	0x1E4F,
	0x1E50,
	0x1E51,
	0x1E52,
	0x1E53,
	0x1E54,
	0x1E55,
	0x1E56,
	0x1E57,
	0x1E58,
	0x1E59,
	0x1E5A,
	0x1E5B,
	0x1E5C,
	0x1E5D,
	0x1E5E,
	0x1E5F,
	0x1E60,
	0x1E61,
	0x1E62,
	0x1E63,
	0x1E64,
	0x1E65,
	0x1E66,
	0x1E67,
	0x1E68,
	0x1E69,
	0x1E6A,
	0x1E6B,
	0x1E6C,
	0x1E6D,
	0x1E6E,
	0x1E6F,
	0x1E70,
	0x1E71,
	0x1E72,
	0x1E73,
	0x1E74,
	0x1E75,
	0x1E76,
	0x1E77,
	0x1E78,
	0x1E79,
	0x1E7A,
	0x1E7B,
	0x1E7C,
	0x1E7D,
	0x1E7E,
	0x1E7F,
	0x1E80,
	0x1E81,
	0x1E82,
	0x1E83,
	0x1E84,
	0x1E85,
	0x1E86,
	0x1E87,
	0x1E88,
	0x1E89,
	0x1E8A,
	0x1E8B,
	0x1E8C,
	0x1E8D,
	0x1E8E,
	0x1E8F,
	0x1E90,
	0x1E91,
	0x1E92,
	0x1E93,
	0x1E94,
	0x1E95,
	0x1E9E,
	0x1E9F,
	0x1EA0,
	0x1EA1,
	0x1EA2,
	0x1EA3,
	0x1EA4,
	0x1EA5,
	0x1EA6,
	0x1EA7,
	0x1EA8,
	0x1EA9,
	0x1EAA,
	0x1EAB,
	0x1EAC,
	0x1EAD,
	0x1EAE,
	0x1EAF,
	0x1EB0,
	0x1EB1,
	0x1EB2,
	0x1EB3,
	0x1EB4,
	0x1EB5,
	0x1EB6,
	0x1EB7,
	0x1EB8,
	0x1EB9,
	0x1EBA,
	0x1EBB,
	0x1EBC,
	0x1EBD,
	0x1EBE,
	0x1EBF,
	0x1EC0,
	0x1EC1,
	0x1EC2,
	0x1EC3,
	0x1EC4,
	0x1EC5,
	0x1EC6,
	0x1EC7,
	0x1EC8,
	0x1EC9,
	0x1ECA,
	0x1ECB,
	0x1ECC,
	0x1ECD,
	0x1ECE,
	0x1ECF,
	0x1ED0,
	0x1ED1,
	0x1ED2,
	0x1ED3,
	0x1ED4,
	0x1ED5,
	0x1ED6,
	0x1ED7,
	0x1ED8,
	0x1ED9,
	0x1EDA,
	0x1EDB,
	0x1EDC,
	0x1EDD,
	0x1EDE,
	0x1EDF,
	0x1EE0,
	0x1EE1,
	0x1EE2,
	0x1EE3,
	0x1EE4,
	0x1EE5,
	0x1EE6,
	0x1EE7,
	0x1EE8,
	0x1EE9,
	0x1EEA,
	0x1EEB,
	0x1EEC,
	0x1EED,
	0x1EEE,
	0x1EEF,
	0x1EF0,
	0x1EF1,
	0x1EF2,
	0x1EF3,
	0x1EF4,
	0x1EF5,
	0x1EF6,
	0x1EF7,
	0x1EF8,
	0x1EF9,
	0x1EFA,
	0x1EFB,
	0x1EFC,
	0x1EFD,
	0x1EFE,
	0x1EFF,
	0x1F08,
	0x1F10,
	0x1F18,
	0x1F1E,
	0x1F28,
	0x1F30,
	0x1F38,
	0x1F40,
	0x1F48,
	0x1F4E,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F60,
	0x1F68,
	0x1F70,
	0x1FB8,
	0x1FBC,
	0x1FC8,
	0x1FCC,
	0x1FD8,
	0x1FDC,
	0x1FE8,
	0x1FED,
	0x1FF8,
	0x1FFC,
	0x2102,
	0x2103,
	0x2107,
	0x2108,
	0x210B,
	0x210E,
	0x2110,
	0x2113,
	0x2115,
	0x2116,
	0x2119,
	0x211E,
	0x2124,
	0x2125,
	0x2126,
	0x2127,
	0x2128,
	0x2129,
	0x212A,
	0x212E,
	0x2130,
	0x2134,
	0x213E,
	0x2140,
	0x2145,
	0x2146,
	0x2160,
	0x2170,
	0x2183,
	0x2184,
	0x24B6,
	0x24D0,
	0x2C00,
	0x2C2F,
	0x2C60,
	0x2C61,
	0x2C62,
	0x2C65,
	0x2C67,
	0x2C68,
	0x2C69,
	0x2C6A,
	0x2C6B,
	0x2C6C,
	0x2C6D,
	0x2C71,
	0x2C72,
	0x2C73,
	0x2C75,
	0x2C76,
	0x2C7E,
	0x2C81,
	0x2C82,
	0x2C83,
	0x2C84,
	0x2C85,
	0x2C86,
	0x2C87,
	0x2C88,
	0x2C89,
	0x2C8A,
	0x2C8B,
	0x2C8C,
	0x2C8D,
	0x2C8E,
	0x2C8F,
	0x2C90,
	0x2C91,
	0x2C92,
	0x2C93,
	0x2C94,
	0x2C95,
	0x2C96,
	0x2C97,
	0x2C98,
	0x2C99,
	0x2C9A,
	0x2C9B,
	0x2C9C,
	0x2C9D,
	0x2C9E,
	0x2C9F,
	0x2CA0,
	0x2CA1,
	0x2CA2,
	0x2CA3,
	0x2CA4,
	0x2CA5,
	0x2CA6,
	0x2CA7,
	0x2CA8,
	0x2CA9,
	0x2CAA,
	0x2CAB,
	0x2CAC,
	0x2CAD,
	0x2CAE,
	0x2CAF,
	0x2CB0,
	0x2CB1,
	0x2CB2,
	0x2CB3,
	0x2CB4,
	0x2CB5,
	0x2CB6,
	0x2CB7,
	0x2CB8,
	0x2CB9,
	0x2CBA,
	0x2CBB,
	0x2CBC,
	0x2CBD,
	0x2CBE,
	0x2CBF,
	0x2CC0,
	0x2CC1,
	0x2CC2,
	0x2CC3,
	0x2CC4,
	0x2CC5,
	0x2CC6,
	0x2CC7,
	0x2CC8,
	0x2CC9,
	0x2CCA,
	0x2CCB,
	0x2CCC,
	0x2CCD,
	0x2CCE,
	0x2CCF,
	0x2CD0,
	0x2CD1,
	0x2CD2,
	0x2CD3,
	0x2CD4,
	0x2CD5,
	0x2CD6,
	0x2CD7,
	0x2CD8,
	0x2CD9,
	0x2CDA,
	0x2CDB,
	0x2CDC,
	0x2CDD,
	0x2CDE,
	0x2CDF,
	0x2CE0,
	0x2CE1,
	0x2CE2,
	0x2CE3,
	0x2CEB,
	0x2CEC,
	0x2CED,
	0x2CEE,
	0x2CF2,
	0x2CF3,
	0xA640,
	0xA641,
	0xA642,
	0xA643,
	0xA644,
	0xA645,
	0xA646,
	0xA647,
	0xA648,
	0xA649,
	0xA64A,
	0xA64B,
	0xA64C,
	0xA64D,
	0xA64E,
	0xA64F,
	0xA650,
	0xA651,
	0xA652,
	0xA653,
	0xA654,
	0xA655,
	0xA656,
	0xA657,
	0xA658,
	0xA659,
	0xA65A,
	0xA65B,
	0xA65C,
	0xA65D,
	0xA65E,
	0xA65F,
	0xA660,
	0xA661,
	0xA662,
	0xA663,
	0xA664,
	0xA665,
	0xA666,
	0xA667,
	0xA668,
	0xA669,
	0xA66A,
	0xA66B,
	0xA66C,
	0xA66D,
	0xA680,
	0xA681,
	0xA682,
	0xA683,
	0xA684,
	0xA685,
	0xA686,
	0xA687,
	0xA688,
	0xA689,
	0xA68A,
	0xA68B,
	0xA68C,
	0xA68D,
	0xA68E,
	0xA68F,
	0xA690,
	0xA691,
	0xA692,
	0xA693,
	0xA694,
	0xA695,
	0xA696,
	0xA697,
	0xA698,
	0xA699,
	0xA69A,
	0xA69B,
	0xA722,
	0xA723,
	0xA724,
	0xA725,
	0xA726,
	0xA727,
	0xA728,
	0xA729,
	0xA72A,
	0xA72B,
	0xA72C,
	0xA72D,
	0xA72E,
	0xA72F,
	0xA732,
	0xA733,
	0xA734,
	0xA735,
	0xA736,
	0xA737,
	0xA738,
	0xA739,
	0xA73A,
	0xA73B,
	0xA73C,
	0xA73D,
	0xA73E,
	0xA73F,
	0xA740,
	0xA741,
	0xA742,
	0xA743,
	0xA744,
	0xA745,
	0xA746,
	0xA747,
	0xA748,
	0xA749,
	0xA74A,
	0xA74B,
	0xA74C,
	0xA74D,
	0xA74E,
	0xA74F,
	0xA750,
	0xA751,
	0xA752,
	0xA753,
	0xA754,
	0xA755,
	0xA756,
	0xA757,
	0xA758,
	0xA759,
	0xA75A,
	0xA75B,
	0xA75C,
	0xA75D,
	0xA75E,
	0xA75F,
	0xA760,
	0xA761,
	0xA762,
	0xA763,
	0xA764,
	0xA765,
	0xA766,
	0xA767,
	0xA768,
	0xA769,
	0xA76A,
	0xA76B,
	0xA76C,
	0xA76D,
	0xA76E,
	0xA76F,
	0xA779,
	0xA77A,
	0xA77B,
	0xA77C,
	0xA77D,
	0xA77F,
	0xA780,
	0xA781,
	0xA782,
	0xA783,
	0xA784,
	0xA785,
	0xA786,
	0xA787,
	0xA78B,
	0xA78C,
	0xA78D,
	0xA78E,
	0xA790,
	0xA791,
	0xA792,
	0xA793,
	0xA796,
	0xA797,
	0xA798,
	0xA799,
	0xA79A,
	0xA79B,
	0xA79C,
	0xA79D,
	0xA79E,
	0xA79F,
	0xA7A0,
	0xA7A1,
	0xA7A2,
	0xA7A3,
	0xA7A4,
	0xA7A5,
	0xA7A6,
	0xA7A7,
	0xA7A8,
	0xA7A9,
	0xA7AA,
	0xA7AF,
	0xA7B0,
	0xA7B5,
	0xA7B6,
	0xA7B7,
	0xFF21,
	0xFF3B,
	0x10400,
	0x10428,
	0x104B0,
	0x104D4,
	0x10C80,
	0x10CB3,
	0x118A0,
	0x118C0,
	0x1D400,
	0x1D41A,
	0x1D434,
	0x1D44E,
	0x1D468,
	0x1D482,
	0x1D49C,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4B6,
	0x1D4D0,
	0x1D4EA,
	0x1D504,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D538,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D56C,
	0x1D586,
	0x1D5A0,
	0x1D5BA,
	0x1D5D4,
	0x1D5EE,
	0x1D608,
	0x1D622,
	0x1D63C,
	0x1D656,
	0x1D670,
	0x1D68A,
	0x1D6A8,
	0x1D6C1,
	0x1D6E2,
	0x1D6FB,
	0x1D71C,
	0x1D735,
	0x1D756,
	0x1D76F,
	0x1D790,
	0x1D7A9,
	0x1D7CA,
	0x1D7CB,
	0x1E900,
	0x1E922,
	0x1F130,
	0x1F14A,
	0x1F150,
	0x1F16A,
	0x1F170,
	0x1F18A
};

static const UV XPosixWord_invlist[] = { /* for EBCDIC 037 */
	1381,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x42,
	0x4A,
	0x51,
	0x5A,
	0x62,
	0x6A,
	0x6D,
	0x6E,
	0x70,
	0x79,
	0x80,
	0x8A,
	0x8C,
	0x8F,
	0x91,
	0x9D,
	0x9E,
	0x9F,
	0xA0,
	0xA1,
	0xA2,
	0xAA,
	0xAC,
	0xAF,
	0xC1,
	0xCA,
	0xCB,
	0xD0,
	0xD1,
	0xDA,
	0xDB,
	0xE0,
	0xE2,
	0xEA,
	0xEB,
	0xFA,
	0xFB,
	0xFF,
	0x100,
	0x2C2,
	0x2C6,
	0x2D2,
	0x2E0,
	0x2E5,
	0x2EC,
	0x2ED,
	0x2EE,
	0x2EF,
	0x300,
	0x375,
	0x376,
	0x378,
	0x37A,
	0x37E,
	0x37F,
	0x380,
	0x386,
	0x387,
	0x388,
	0x38B,
	0x38C,
	0x38D,
	0x38E,
	0x3A2,
	0x3A3,
	0x3F6,
	0x3F7,
	0x482,
	0x483,
	0x530,
	0x531,
	0x557,
	0x559,
	0x55A,
	0x561,
	0x588,
	0x591,
	0x5BE,
	0x5BF,
	0x5C0,
	0x5C1,
	0x5C3,
	0x5C4,
	0x5C6,
	0x5C7,
	0x5C8,
	0x5D0,
	0x5EB,
	0x5F0,
	0x5F3,
	0x610,
	0x61B,
	0x620,
	0x66A,
	0x66E,
	0x6D4,
	0x6D5,
	0x6DD,
	0x6DF,
	0x6E9,
	0x6EA,
	0x6FD,
	0x6FF,
	0x700,
	0x710,
	0x74B,
	0x74D,
	0x7B2,
	0x7C0,
	0x7F6,
	0x7FA,
	0x7FB,
	0x800,
	0x82E,
	0x840,
	0x85C,
	0x8A0,
	0x8B5,
	0x8B6,
	0x8BE,
	0x8D4,
	0x8E2,
	0x8E3,
	0x964,
	0x966,
	0x970,
	0x971,
	0x984,
	0x985,
	0x98D,
	0x98F,
	0x991,
	0x993,
	0x9A9,
	0x9AA,
	0x9B1,
	0x9B2,
	0x9B3,
	0x9B6,
	0x9BA,
	0x9BC,
	0x9C5,
	0x9C7,
	0x9C9,
	0x9CB,
	0x9CF,
	0x9D7,
	0x9D8,
	0x9DC,
	0x9DE,
	0x9DF,
	0x9E4,
	0x9E6,
	0x9F2,
	0xA01,
	0xA04,
	0xA05,
	0xA0B,
	0xA0F,
	0xA11,
	0xA13,
	0xA29,
	0xA2A,
	0xA31,
	0xA32,
	0xA34,
	0xA35,
	0xA37,
	0xA38,
	0xA3A,
	0xA3C,
	0xA3D,
	0xA3E,
	0xA43,
	0xA47,
	0xA49,
	0xA4B,
	0xA4E,
	0xA51,
	0xA52,
	0xA59,
	0xA5D,
	0xA5E,
	0xA5F,
	0xA66,
	0xA76,
	0xA81,
	0xA84,
	0xA85,
	0xA8E,
	0xA8F,
	0xA92,
	0xA93,
	0xAA9,
	0xAAA,
	0xAB1,
	0xAB2,
	0xAB4,
	0xAB5,
	0xABA,
	0xABC,
	0xAC6,
	0xAC7,
	0xACA,
	0xACB,
	0xACE,
	0xAD0,
	0xAD1,
	0xAE0,
	0xAE4,
	0xAE6,
	0xAF0,
	0xAF9,
	0xAFA,
	0xB01,
	0xB04,
	0xB05,
	0xB0D,
	0xB0F,
	0xB11,
	0xB13,
	0xB29,
	0xB2A,
	0xB31,
	0xB32,
	0xB34,
	0xB35,
	0xB3A,
	0xB3C,
	0xB45,
	0xB47,
	0xB49,
	0xB4B,
	0xB4E,
	0xB56,
	0xB58,
	0xB5C,
	0xB5E,
	0xB5F,
	0xB64,
	0xB66,
	0xB70,
	0xB71,
	0xB72,
	0xB82,
	0xB84,
	0xB85,
	0xB8B,
	0xB8E,
	0xB91,
	0xB92,
	0xB96,
	0xB99,
	0xB9B,
	0xB9C,
	0xB9D,
	0xB9E,
	0xBA0,
	0xBA3,
	0xBA5,
	0xBA8,
	0xBAB,
	0xBAE,
	0xBBA,
	0xBBE,
	0xBC3,
	0xBC6,
	0xBC9,
	0xBCA,
	0xBCE,
	0xBD0,
	0xBD1,
	0xBD7,
	0xBD8,
	0xBE6,
	0xBF0,
	0xC00,
	0xC04,
	0xC05,
	0xC0D,
	0xC0E,
	0xC11,
	0xC12,
	0xC29,
	0xC2A,
	0xC3A,
	0xC3D,
	0xC45,
	0xC46,
	0xC49,
	0xC4A,
	0xC4E,
	0xC55,
	0xC57,
	0xC58,
	0xC5B,
	0xC60,
	0xC64,
	0xC66,
	0xC70,
	0xC80,
	0xC84,
	0xC85,
	0xC8D,
	0xC8E,
	0xC91,
	0xC92,
	0xCA9,
	0xCAA,
	0xCB4,
	0xCB5,
	0xCBA,
	0xCBC,
	0xCC5,
	0xCC6,
	0xCC9,
	0xCCA,
	0xCCE,
	0xCD5,
	0xCD7,
	0xCDE,
	0xCDF,
	0xCE0,
	0xCE4,
	0xCE6,
	0xCF0,
	0xCF1,
	0xCF3,
	0xD01,
	0xD04,
	0xD05,
	0xD0D,
	0xD0E,
	0xD11,
	0xD12,
	0xD3B,
	0xD3D,
	0xD45,
	0xD46,
	0xD49,
	0xD4A,
	0xD4F,
	0xD54,
	0xD58,
	0xD5F,
	0xD64,
	0xD66,
	0xD70,
	0xD7A,
	0xD80,
	0xD82,
	0xD84,
	0xD85,
	0xD97,
	0xD9A,
	0xDB2,
	0xDB3,
	0xDBC,
	0xDBD,
	0xDBE,
	0xDC0,
	0xDC7,
	0xDCA,
	0xDCB,
	0xDCF,
	0xDD5,
	0xDD6,
	0xDD7,
	0xDD8,
	0xDE0,
	0xDE6,
	0xDF0,
	0xDF2,
	0xDF4,
	0xE01,
	0xE3B,
	0xE40,
	0xE4F,
	0xE50,
	0xE5A,
	0xE81,
	0xE83,
	0xE84,
	0xE85,
	0xE87,
	0xE89,
	0xE8A,
	0xE8B,
	0xE8D,
	0xE8E,
	0xE94,
	0xE98,
	0xE99,
	0xEA0,
	0xEA1,
	0xEA4,
	0xEA5,
	0xEA6,
	0xEA7,
	0xEA8,
	0xEAA,
	0xEAC,
	0xEAD,
	0xEBA,
	0xEBB,
	0xEBE,
	0xEC0,
	0xEC5,
	0xEC6,
	0xEC7,
	0xEC8,
	0xECE,
	0xED0,
	0xEDA,
	0xEDC,
	0xEE0,
	0xF00,
	0xF01,
	0xF18,
	0xF1A,
	0xF20,
	0xF2A,
	0xF35,
	0xF36,
	0xF37,
	0xF38,
	0xF39,
	0xF3A,
	0xF3E,
	0xF48,
	0xF49,
	0xF6D,
	0xF71,
	0xF85,
	0xF86,
	0xF98,
	0xF99,
	0xFBD,
	0xFC6,
	0xFC7,
	0x1000,
	0x104A,
	0x1050,
	0x109E,
	0x10A0,
	0x10C6,
	0x10C7,
	0x10C8,
	0x10CD,
	0x10CE,
	0x10D0,
	0x10FB,
	0x10FC,
	0x1249,
	0x124A,
	0x124E,
	0x1250,
	0x1257,
	0x1258,
	0x1259,
	0x125A,
	0x125E,
	0x1260,
	0x1289,
	0x128A,
	0x128E,
	0x1290,
	0x12B1,
	0x12B2,
	0x12B6,
	0x12B8,
	0x12BF,
	0x12C0,
	0x12C1,
	0x12C2,
	0x12C6,
	0x12C8,
	0x12D7,
	0x12D8,
	0x1311,
	0x1312,
	0x1316,
	0x1318,
	0x135B,
	0x135D,
	0x1360,
	0x1380,
	0x1390,
	0x13A0,
	0x13F6,
	0x13F8,
	0x13FE,
	0x1401,
	0x166D,
	0x166F,
	0x1680,
	0x1681,
	0x169B,
	0x16A0,
	0x16EB,
	0x16EE,
	0x16F9,
	0x1700,
	0x170D,
	0x170E,
	0x1715,
	0x1720,
	0x1735,
	0x1740,
	0x1754,
	0x1760,
	0x176D,
	0x176E,
	0x1771,
	0x1772,
	0x1774,
	0x1780,
	0x17D4,
	0x17D7,
	0x17D8,
	0x17DC,
	0x17DE,
	0x17E0,
	0x17EA,
	0x180B,
	0x180E,
	0x1810,
	0x181A,
	0x1820,
	0x1878,
	0x1880,
	0x18AB,
	0x18B0,
	0x18F6,
	0x1900,
	0x191F,
	0x1920,
	0x192C,
	0x1930,
	0x193C,
	0x1946,
	0x196E,
	0x1970,
	0x1975,
	0x1980,
	0x19AC,
	0x19B0,
	0x19CA,
	0x19D0,
	0x19DA,
	0x1A00,
	0x1A1C,
	0x1A20,
	0x1A5F,
	0x1A60,
	0x1A7D,
	0x1A7F,
	0x1A8A,
	0x1A90,
	0x1A9A,
	0x1AA7,
	0x1AA8,
	0x1AB0,
	0x1ABF,
	0x1B00,
	0x1B4C,
	0x1B50,
	0x1B5A,
	0x1B6B,
	0x1B74,
	0x1B80,
	0x1BF4,
	0x1C00,
	0x1C38,
	0x1C40,
	0x1C4A,
	0x1C4D,
	0x1C7E,
	0x1C80,
	0x1C89,
	0x1CD0,
	0x1CD3,
	0x1CD4,
	0x1CF7,
	0x1CF8,
	0x1CFA,
	0x1D00,
	0x1DF6,
	0x1DFB,
	0x1F16,
	0x1F18,
	0x1F1E,
	0x1F20,
	0x1F46,
	0x1F48,
	0x1F4E,
	0x1F50,
	0x1F58,
	0x1F59,
	0x1F5A,
	0x1F5B,
	0x1F5C,
	0x1F5D,
	0x1F5E,
	0x1F5F,
	0x1F7E,
	0x1F80,
	0x1FB5,
	0x1FB6,
	0x1FBD,
	0x1FBE,
	0x1FBF,
	0x1FC2,
	0x1FC5,
	0x1FC6,
	0x1FCD,
	0x1FD0,
	0x1FD4,
	0x1FD6,
	0x1FDC,
	0x1FE0,
	0x1FED,
	0x1FF2,
	0x1FF5,
	0x1FF6,
	0x1FFD,
	0x200C,
	0x200E,
	0x203F,
	0x2041,
	0x2054,
	0x2055,
	0x2071,
	0x2072,
	0x207F,
	0x2080,
	0x2090,
	0x209D,
	0x20D0,
	0x20F1,
	0x2102,
	0x2103,
	0x2107,
	0x2108,
	0x210A,
	0x2114,
	0x2115,
	0x2116,
	0x2119,
	0x211E,
	0x2124,
	0x2125,
	0x2126,
	0x2127,
	0x2128,
	0x2129,
	0x212A,
	0x212E,
	0x212F,
	0x213A,
	0x213C,
	0x2140,
	0x2145,
	0x214A,
	0x214E,
	0x214F,
	0x2160,
	0x2189,
	0x24B6,
	0x24EA,
	0x2C00,
	0x2C2F,
	0x2C30,
	0x2C5F,
	0x2C60,
	0x2CE5,
	0x2CEB,
	0x2CF4,
	0x2D00,
	0x2D26,
	0x2D27,
	0x2D28,
	0x2D2D,
	0x2D2E,
	0x2D30,
	0x2D68,
	0x2D6F,
	0x2D70,
	0x2D7F,
	0x2D97,
	0x2DA0,
	0x2DA7,
	0x2DA8,
	0x2DAF,
	0x2DB0,
	0x2DB7,
	0x2DB8,
	0x2DBF,
	0x2DC0,
	0x2DC7,
	0x2DC8,
	0x2DCF,
	0x2DD0,
	0x2DD7,
	0x2DD8,
	0x2DDF,
	0x2DE0,
	0x2E00,
	0x2E2F,
	0x2E30,
	0x3005,
	0x3008,
	0x3021,
	0x3030,
	0x3031,
	0x3036,
	0x3038,
	0x303D,
	0x3041,
	0x3097,
	0x3099,
	0x309B,
	0x309D,
	0x30A0,
	0x30A1,
	0x30FB,
	0x30FC,
	0x3100,
	0x3105,
	0x312E,
	0x3131,
	0x318F,
	0x31A0,
	0x31BB,
	0x31F0,
	0x3200,
	0x3400,
	0x4DB6,
	0x4E00,
	0x9FD6,
	0xA000,
	0xA48D,
	0xA4D0,
	0xA4FE,
	0xA500,
	0xA60D,
	0xA610,
	0xA62C,
	0xA640,
	0xA673,
	0xA674,
	0xA67E,
	0xA67F,
	0xA6F2,
	0xA717,
	0xA720,
	0xA722,
	0xA789,
	0xA78B,
	0xA7AF,
	0xA7B0,
	0xA7B8,
	0xA7F7,
	0xA828,
	0xA840,
	0xA874,
	0xA880,
	0xA8C6,
	0xA8D0,
	0xA8DA,
	0xA8E0,
	0xA8F8,
	0xA8FB,
	0xA8FC,
	0xA8FD,
	0xA8FE,
	0xA900,
	0xA92E,
	0xA930,
	0xA954,
	0xA960,
	0xA97D,
	0xA980,
	0xA9C1,
	0xA9CF,
	0xA9DA,
	0xA9E0,
	0xA9FF,
	0xAA00,
	0xAA37,
	0xAA40,
	0xAA4E,
	0xAA50,
	0xAA5A,
	0xAA60,
	0xAA77,
	0xAA7A,
	0xAAC3,
	0xAADB,
	0xAADE,
	0xAAE0,
	0xAAF0,
	0xAAF2,
	0xAAF7,
	0xAB01,
	0xAB07,
	0xAB09,
	0xAB0F,
	0xAB11,
	0xAB17,
	0xAB20,
	0xAB27,
	0xAB28,
	0xAB2F,
	0xAB30,
	0xAB5B,
	0xAB5C,
	0xAB66,
	0xAB70,
	0xABEB,
	0xABEC,
	0xABEE,
	0xABF0,
	0xABFA,
	0xAC00,
	0xD7A4,
	0xD7B0,
	0xD7C7,
	0xD7CB,
	0xD7FC,
	0xF900,
	0xFA6E,
	0xFA70,
	0xFADA,
	0xFB00,
	0xFB07,
	0xFB13,
	0xFB18,
	0xFB1D,
	0xFB29,
	0xFB2A,
	0xFB37,
	0xFB38,
	0xFB3D,
	0xFB3E,
	0xFB3F,
	0xFB40,
	0xFB42,
	0xFB43,
	0xFB45,
	0xFB46,
	0xFBB2,
	0xFBD3,
	0xFD3E,
	0xFD50,
	0xFD90,
	0xFD92,
	0xFDC8,
	0xFDF0,
	0xFDFC,
	0xFE00,
	0xFE10,
	0xFE20,
	0xFE30,
	0xFE33,
	0xFE35,
	0xFE4D,
	0xFE50,
	0xFE70,
	0xFE75,
	0xFE76,
	0xFEFD,
	0xFF10,
	0xFF1A,
	0xFF21,
	0xFF3B,
	0xFF3F,
	0xFF40,
	0xFF41,
	0xFF5B,
	0xFF66,
	0xFFBF,
	0xFFC2,
	0xFFC8,
	0xFFCA,
	0xFFD0,
	0xFFD2,
	0xFFD8,
	0xFFDA,
	0xFFDD,
	0x10000,
	0x1000C,
	0x1000D,
	0x10027,
	0x10028,
	0x1003B,
	0x1003C,
	0x1003E,
	0x1003F,
	0x1004E,
	0x10050,
	0x1005E,
	0x10080,
	0x100FB,
	0x10140,
	0x10175,
	0x101FD,
	0x101FE,
	0x10280,
	0x1029D,
	0x102A0,
	0x102D1,
	0x102E0,
	0x102E1,
	0x10300,
	0x10320,
	0x10330,
	0x1034B,
	0x10350,
	0x1037B,
	0x10380,
	0x1039E,
	0x103A0,
	0x103C4,
	0x103C8,
	0x103D0,
	0x103D1,
	0x103D6,
	0x10400,
	0x1049E,
	0x104A0,
	0x104AA,
	0x104B0,
	0x104D4,
	0x104D8,
	0x104FC,
	0x10500,
	0x10528,
	0x10530,
	0x10564,
	0x10600,
	0x10737,
	0x10740,
	0x10756,
	0x10760,
	0x10768,
	0x10800,
	0x10806,
	0x10808,
	0x10809,
	0x1080A,
	0x10836,
	0x10837,
	0x10839,
	0x1083C,
	0x1083D,
	0x1083F,
	0x10856,
	0x10860,
	0x10877,
	0x10880,
	0x1089F,
	0x108E0,
	0x108F3,
	0x108F4,
	0x108F6,
	0x10900,
	0x10916,
	0x10920,
	0x1093A,
	0x10980,
	0x109B8,
	0x109BE,
	0x109C0,
	0x10A00,
	0x10A04,
	0x10A05,
	0x10A07,
	0x10A0C,
	0x10A14,
	0x10A15,
	0x10A18,
	0x10A19,
	0x10A34,
	0x10A38,
	0x10A3B,
	0x10A3F,
	0x10A40,
	0x10A60,
	0x10A7D,
	0x10A80,
	0x10A9D,
	0x10AC0,
	0x10AC8,
	0x10AC9,
	0x10AE7,
	0x10B00,
	0x10B36,
	0x10B40,
	0x10B56,
	0x10B60,
	0x10B73,
	0x10B80,
	0x10B92,
	0x10C00,
	0x10C49,
	0x10C80,
	0x10CB3,
	0x10CC0,
	0x10CF3,
	0x11000,
	0x11047,
	0x11066,
	0x11070,
	0x1107F,
	0x110BB,
	0x110D0,
	0x110E9,
	0x110F0,
	0x110FA,
	0x11100,
	0x11135,
	0x11136,
	0x11140,
	0x11150,
	0x11174,
	0x11176,
	0x11177,
	0x11180,
	0x111C5,
	0x111CA,
	0x111CD,
	0x111D0,
	0x111DB,
	0x111DC,
	0x111DD,
	0x11200,
	0x11212,
	0x11213,
	0x11238,
	0x1123E,
	0x1123F,
	0x11280,
	0x11287,
	0x11288,
	0x11289,
	0x1128A,
	0x1128E,
	0x1128F,
	0x1129E,
	0x1129F,
	0x112A9,
	0x112B0,
	0x112EB,
	0x112F0,
	0x112FA,
	0x11300,
	0x11304,
	0x11305,
	0x1130D,
	0x1130F,
	0x11311,
	0x11313,
	0x11329,
	0x1132A,
	0x11331,
	0x11332,
	0x11334,
	0x11335,
	0x1133A,
	0x1133C,
	0x11345,
	0x11347,
	0x11349,
	0x1134B,
	0x1134E,
	0x11350,
	0x11351,
	0x11357,
	0x11358,
	0x1135D,
	0x11364,
	0x11366,
	0x1136D,
	0x11370,
	0x11375,
	0x11400,
	0x1144B,
	0x11450,
	0x1145A,
	0x11480,
	0x114C6,
	0x114C7,
	0x114C8,
	0x114D0,
	0x114DA,
	0x11580,
	0x115B6,
	0x115B8,
	0x115C1,
	0x115D8,
	0x115DE,
	0x11600,
	0x11641,
	0x11644,
	0x11645,
	0x11650,
	0x1165A,
	0x11680,
	0x116B8,
	0x116C0,
	0x116CA,
	0x11700,
	0x1171A,
	0x1171D,
	0x1172C,
	0x11730,
	0x1173A,
	0x118A0,
	0x118EA,
	0x118FF,
	0x11900,
	0x11AC0,
	0x11AF9,
	0x11C00,
	0x11C09,
	0x11C0A,
	0x11C37,
	0x11C38,
	0x11C41,
	0x11C50,
	0x11C5A,
	0x11C72,
	0x11C90,
	0x11C92,
	0x11CA8,
	0x11CA9,
	0x11CB7,
	0x12000,
	0x1239A,
	0x12400,
	0x1246F,
	0x12480,
	0x12544,
	0x13000,
	0x1342F,
	0x14400,
	0x14647,
	0x16800,
	0x16A39,
	0x16A40,
	0x16A5F,
	0x16A60,
	0x16A6A,
	0x16AD0,
	0x16AEE,
	0x16AF0,
	0x16AF5,
	0x16B00,
	0x16B37,
	0x16B40,
	0x16B44,
	0x16B50,
	0x16B5A,
	0x16B63,
	0x16B78,
	0x16B7D,
	0x16B90,
	0x16F00,
	0x16F45,
	0x16F50,
	0x16F7F,
	0x16F8F,
	0x16FA0,
	0x16FE0,
	0x16FE1,
	0x17000,
	0x187ED,
	0x18800,
	0x18AF3,
	0x1B000,
	0x1B002,
	0x1BC00,
	0x1BC6B,
	0x1BC70,
	0x1BC7D,
	0x1BC80,
	0x1BC89,
	0x1BC90,
	0x1BC9A,
	0x1BC9D,
	0x1BC9F,
	0x1D165,
	0x1D16A,
	0x1D16D,
	0x1D173,
	0x1D17B,
	0x1D183,
	0x1D185,
	0x1D18C,
	0x1D1AA,
	0x1D1AE,
	0x1D242,
	0x1D245,
	0x1D400,
	0x1D455,
	0x1D456,
	0x1D49D,
	0x1D49E,
	0x1D4A0,
	0x1D4A2,
	0x1D4A3,
	0x1D4A5,
	0x1D4A7,
	0x1D4A9,
	0x1D4AD,
	0x1D4AE,
	0x1D4BA,
	0x1D4BB,
	0x1D4BC,
	0x1D4BD,
	0x1D4C4,
	0x1D4C5,
	0x1D506,
	0x1D507,
	0x1D50B,
	0x1D50D,
	0x1D515,
	0x1D516,
	0x1D51D,
	0x1D51E,
	0x1D53A,
	0x1D53B,
	0x1D53F,
	0x1D540,
	0x1D545,
	0x1D546,
	0x1D547,
	0x1D54A,
	0x1D551,
	0x1D552,
	0x1D6A6,
	0x1D6A8,
	0x1D6C1,
	0x1D6C2,
	0x1D6DB,
	0x1D6DC,
	0x1D6FB,
	0x1D6FC,
	0x1D715,
	0x1D716,
	0x1D735,
	0x1D736,
	0x1D74F,
	0x1D750,
	0x1D76F,
	0x1D770,
	0x1D789,
	0x1D78A,
	0x1D7A9,
	0x1D7AA,
	0x1D7C3,
	0x1D7C4,
	0x1D7CC,
	0x1D7CE,
	0x1D800,
	0x1DA00,
	0x1DA37,
	0x1DA3B,
	0x1DA6D,
	0x1DA75,
	0x1DA76,
	0x1DA84,
	0x1DA85,
	0x1DA9B,
	0x1DAA0,
	0x1DAA1,
	0x1DAB0,
	0x1E000,
	0x1E007,
	0x1E008,
	0x1E019,
	0x1E01B,
	0x1E022,
	0x1E023,
	0x1E025,
	0x1E026,
	0x1E02B,
	0x1E800,
	0x1E8C5,
	0x1E8D0,
	0x1E8D7,
	0x1E900,
	0x1E94B,
	0x1E950,
	0x1E95A,
	0x1EE00,
	0x1EE04,
	0x1EE05,
	0x1EE20,
	0x1EE21,
	0x1EE23,
	0x1EE24,
	0x1EE25,
	0x1EE27,
	0x1EE28,
	0x1EE29,
	0x1EE33,
	0x1EE34,
	0x1EE38,
	0x1EE39,
	0x1EE3A,
	0x1EE3B,
	0x1EE3C,
	0x1EE42,
	0x1EE43,
	0x1EE47,
	0x1EE48,
	0x1EE49,
	0x1EE4A,
	0x1EE4B,
	0x1EE4C,
	0x1EE4D,
	0x1EE50,
	0x1EE51,
	0x1EE53,
	0x1EE54,
	0x1EE55,
	0x1EE57,
	0x1EE58,
	0x1EE59,
	0x1EE5A,
	0x1EE5B,
	0x1EE5C,
	0x1EE5D,
	0x1EE5E,
	0x1EE5F,
	0x1EE60,
	0x1EE61,
	0x1EE63,
	0x1EE64,
	0x1EE65,
	0x1EE67,
	0x1EE6B,
	0x1EE6C,
	0x1EE73,
	0x1EE74,
	0x1EE78,
	0x1EE79,
	0x1EE7D,
	0x1EE7E,
	0x1EE7F,
	0x1EE80,
	0x1EE8A,
	0x1EE8B,
	0x1EE9C,
	0x1EEA1,
	0x1EEA4,
	0x1EEA5,
	0x1EEAA,
	0x1EEAB,
	0x1EEBC,
	0x1F130,
	0x1F14A,
	0x1F150,
	0x1F16A,
	0x1F170,
	0x1F18A,
	0x20000,
	0x2A6D7,
	0x2A700,
	0x2B735,
	0x2B740,
	0x2B81E,
	0x2B820,
	0x2CEA2,
	0x2F800,
	0x2FA1E,
	0xE0100,
	0xE01F0
};

static const UV XPosixXDigit_invlist[] = { /* for EBCDIC 037 */
	13,	/* Number of elements */
	148565664, /* Version and data structure type */
	1,	/* 0 if the list starts at 0;
		   1 if it starts at the element beyond 0 */
	0x0,
	0x81,
	0x87,
	0xC1,
	0xC7,
	0xF0,
	0xFA,
	0xFF10,
	0xFF1A,
	0xFF21,
	0xFF27,
	0xFF41,
	0xFF47
};

#endif	/* defined(PERL_IN_PERL_C) */

#endif	/* EBCDIC 037 */

#if defined(PERL_IN_REGEXEC_C)

#define GCB_NOBREAK      0
#define GCB_BREAKABLE    1
#define GCB_RI_then_RI   2
#define GCB_EX_then_EM   3

static const U8 GCB_table[19][19] = {
   /* 'edg' stands for 'EDGE' */
/*        XX CR CN EB EBG EM EX GAZ  L LF LV LVT PP RI SM  T  V ZWJ edg */
/* XX */ { 1, 1, 1, 1,  1, 1, 0,  1, 1, 1, 1,  1, 1, 1, 0, 1, 1,  0,  1 },
/* CR */ { 1, 1, 1, 1,  1, 1, 1,  1, 1, 0, 1,  1, 1, 1, 1, 1, 1,  1,  1 },
/* CN */ { 1, 1, 1, 1,  1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1, 1, 1,  1,  1 },
/* EB */ { 1, 1, 1, 1,  1, 0, 0,  1, 1, 1, 1,  1, 1, 1, 0, 1, 1,  0,  1 },
/* EBG*/ { 1, 1, 1, 1,  1, 0, 0,  1, 1, 1, 1,  1, 1, 1, 0, 1, 1,  0,  1 },
/* EM */ { 1, 1, 1, 1,  1, 1, 0,  1, 1, 1, 1,  1, 1, 1, 0, 1, 1,  0,  1 },
/* EX */ { 1, 1, 1, 1,  1, 3, 0,  1, 1, 1, 1,  1, 1, 1, 0, 1, 1,  0,  1 },
/* GAZ*/ { 1, 1, 1, 1,  1, 1, 0,  1, 1, 1, 1,  1, 1, 1, 0, 1, 1,  0,  1 },
/* L  */ { 1, 1, 1, 1,  1, 1, 0,  1, 0, 1, 0,  0, 1, 1, 0, 1, 0,  0,  1 },
/* LF */ { 1, 1, 1, 1,  1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1, 1, 1,  1,  1 },
/* LV */ { 1, 1, 1, 1,  1, 1, 0,  1, 1, 1, 1,  1, 1, 1, 0, 0, 0,  0,  1 },
/* LVT*/ { 1, 1, 1, 1,  1, 1, 0,  1, 1, 1, 1,  1, 1, 1, 0, 0, 1,  0,  1 },
/* PP */ { 0, 1, 1, 0,  0, 0, 0,  0, 0, 1, 0,  0, 0, 0, 0, 0, 0,  0,  1 },
/* RI */ { 1, 1, 1, 1,  1, 1, 0,  1, 1, 1, 1,  1, 1, 2, 0, 1, 1,  0,  1 },
/* SM */ { 1, 1, 1, 1,  1, 1, 0,  1, 1, 1, 1,  1, 1, 1, 0, 1, 1,  0,  1 },
/* T  */ { 1, 1, 1, 1,  1, 1, 0,  1, 1, 1, 1,  1, 1, 1, 0, 0, 1,  0,  1 },
/* V  */ { 1, 1, 1, 1,  1, 1, 0,  1, 1, 1, 1,  1, 1, 1, 0, 0, 0,  0,  1 },
/* ZWJ*/ { 1, 1, 1, 1,  0, 1, 0,  0, 1, 1, 1,  1, 1, 1, 0, 1, 1,  0,  1 },
/* edg*/ { 1, 1, 1, 1,  1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1, 1, 1,  1,  0 }
};

#define LB_NOBREAK                        0
#define LB_BREAKABLE                      1
#define LB_NOBREAK_EVEN_WITH_SP_BETWEEN   2
#define LB_CM_ZWJ_foo                     3
#define LB_SP_foo                         6
#define LB_PR_or_PO_then_OP_or_HY         9
#define LB_SY_or_IS_then_various         11
#define LB_HY_or_BA_then_foo             13
#define LB_RI_then_RI                    15
#define LB_various_then_PO_or_PR         32

static const U8 LB_table[39][39] = {
   /* 'edg' stands for 'EDGE' */
/*        AL BA BB B2 SY CR CP CL CM CB EB EM EX GL H2 H3 HL HY ID IS IN JL JT JV LF BK NL NS NU OP PO PR QU RI SP WJ ZWJ ZW edg */
/* AL */ { 0, 0, 1, 1, 2, 0, 2, 2, 0, 1, 1, 1, 2, 0, 1, 1, 0, 0, 1, 2, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0,  0, 0,  1 },
/* BA */ {14, 0,14,14, 2, 0, 2, 2, 0, 1,14,14, 2,14,14,14,14, 0,14, 2,14,14,14,14, 0, 0, 0, 0,14,14,14,14, 0,14, 0, 0,  0, 0,  1 },
/* BB */ { 0, 0, 0, 0, 2, 0, 2, 2, 0, 1, 0, 0, 2, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  0, 0,  1 },
/* B2 */ { 1, 0, 1, 2, 2, 0, 2, 2, 0, 1, 1, 1, 2, 0, 1, 1, 1, 0, 1, 2, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 0,  0, 0,  1 },
/* SY */ { 1, 0, 1, 1, 2, 0, 2, 2, 0, 1, 1, 1, 2, 0, 1, 1, 0, 0, 1, 2, 1, 1, 1, 1, 0, 0, 0, 0,12, 1,33,33, 0, 1, 0, 0,  0, 0,  1 },
/* CR */ { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  1, 1,  1 },
/* CP */ { 0, 0, 1, 1, 2, 0, 2, 2, 0, 1, 1, 1, 2, 0, 1, 1, 0, 0, 1, 2, 1, 1, 1, 1, 0, 0, 0, 2, 0, 1,33,33, 0, 1, 0, 0,  0, 0,  1 },
/* CL */ { 1, 0, 1, 1, 2, 0, 2, 2, 0, 1, 1, 1, 2, 0, 1, 1, 1, 0, 1, 2, 1, 1, 1, 1, 0, 0, 0, 2, 1, 1,33,33, 0, 1, 0, 0,  0, 0,  1 },
/* CM */ { 3, 3, 3, 3, 3, 0, 3, 3, 0, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 0, 0, 0, 3, 3, 3, 3, 3, 3, 3, 0, 3,  3, 0,  1 },
/* CB */ { 1, 1, 1, 1, 2, 0, 2, 2, 0, 1, 1, 1, 2, 0, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0,  0, 0,  1 },
/* EB */ { 1, 0, 1, 1, 2, 0, 2, 2, 0, 1, 1, 0, 2, 0, 1, 1, 1, 0, 1, 2, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 1, 0, 1, 0, 0,  0, 0,  1 },
/* EM */ { 1, 0, 1, 1, 2, 0, 2, 2, 0, 1, 1, 1, 2, 0, 1, 1, 1, 0, 1, 2, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 1, 0, 1, 0, 0,  0, 0,  1 },
/* EX */ { 1, 0, 1, 1, 2, 0, 2, 2, 0, 1, 1, 1, 2, 0, 1, 1, 1, 0, 1, 2, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 0,  0, 0,  1 },
/* GL */ { 0, 0, 0, 0, 2, 0, 2, 2, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  0, 0,  1 },
/* H2 */ { 1, 0, 1, 1, 2, 0, 2, 2, 0, 1, 1, 1, 2, 0, 1, 1, 1, 0, 1, 2, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1, 0, 1, 0, 0,  0, 0,  1 },
/* H3 */ { 1, 0, 1, 1, 2, 0, 2, 2, 0, 1, 1, 1, 2, 0, 1, 1, 1, 0, 1, 2, 0, 1, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1, 0, 1, 0, 0,  0, 0,  1 },
/* HL */ { 0, 0, 1, 1, 2, 0, 2, 2, 0, 1, 1, 1, 2, 0, 1, 1, 0, 0, 1, 2, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0,  0, 0,  1 },
/* HY */ {14, 0,14,14, 2, 0, 2, 2, 0, 1,14,14, 2,14,14,14,14, 0,14, 2,14,14,14,14, 0, 0, 0, 0,13,14,14,14, 0,14, 0, 0,  0, 0,  1 },
/* ID */ { 1, 0, 1, 1, 2, 0, 2, 2, 0, 1, 1, 1, 2, 0, 1, 1, 1, 0, 1, 2, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 1, 0, 1, 0, 0,  0, 0,  1 },
/* IS */ { 0, 0, 1, 1, 2, 0, 2, 2, 0, 1, 1, 1, 2, 0, 1, 1, 0, 0, 1, 2, 1, 1, 1, 1, 0, 0, 0, 0,12, 1,33,33, 0, 1, 0, 0,  0, 0,  1 },
/* IN */ { 1, 0, 1, 1, 2, 0, 2, 2, 0, 1, 1, 1, 2, 0, 1, 1, 1, 0, 1, 2, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 0,  0, 0,  1 },
/* JL */ { 1, 0, 1, 1, 2, 0, 2, 2, 0, 1, 1, 1, 2, 0, 0, 0, 1, 0, 1, 2, 0, 0, 1, 0, 0, 0, 0, 0, 1, 1, 0, 1, 0, 1, 0, 0,  0, 0,  1 },
/* JT */ { 1, 0, 1, 1, 2, 0, 2, 2, 0, 1, 1, 1, 2, 0, 1, 1, 1, 0, 1, 2, 0, 1, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1, 0, 1, 0, 0,  0, 0,  1 },
/* JV */ { 1, 0, 1, 1, 2, 0, 2, 2, 0, 1, 1, 1, 2, 0, 1, 1, 1, 0, 1, 2, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1, 0, 1, 0, 0,  0, 0,  1 },
/* LF */ { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  1, 1,  1 },
/* BK */ { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  1, 1,  1 },
/* NL */ { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  1, 1,  1 },
/* NS */ { 1, 0, 1, 1, 2, 0, 2, 2, 0, 1, 1, 1, 2, 0, 1, 1, 1, 0, 1, 2, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 0,  0, 0,  1 },
/* NU */ { 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 2, 0, 1, 1, 0, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0,  0, 0,  1 },
/* OP */ { 2, 2, 2, 2, 2, 0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0, 2, 2, 2, 2, 2, 2, 2, 0, 2,  2, 0,  1 },
/* PO */ { 0, 0, 1, 1, 2, 0, 2, 2, 0, 1, 1, 1, 2, 0, 1, 1, 0, 0, 1, 2, 1, 1, 1, 1, 0, 0, 0, 0, 0,10, 1, 1, 0, 1, 0, 0,  0, 0,  1 },
/* PR */ { 0, 0, 1, 1, 2, 0, 2, 2, 0, 1, 0, 0, 2, 0, 0, 0, 0, 0, 0, 2, 1, 0, 0, 0, 0, 0, 0, 0, 0,10, 1, 1, 0, 1, 0, 0,  0, 0,  1 },
/* QU */ { 0, 0, 0, 0, 2, 0, 2, 2, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0,  0, 0,  1 },
/* RI */ { 1, 0, 1, 1, 2, 0, 2, 2, 0, 1, 1, 1, 2, 0, 1, 1, 1, 0, 1, 2, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0,15, 0, 0,  0, 0,  1 },
/* SP */ { 7, 7, 7, 7, 8, 0, 8, 8, 7, 7, 7, 7, 8, 7, 7, 7, 7, 7, 7, 8, 7, 7, 7, 7, 0, 0, 0, 7, 7, 7, 7, 7, 7, 7, 0, 8,  7, 0,  1 },
/* WJ */ { 0, 0, 0, 0, 2, 0, 2, 2, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  0, 0,  1 },
/* ZWJ*/ { 3, 3, 3, 3, 3, 0, 3, 3, 0, 3, 0, 0, 3, 3, 3, 3, 3, 3, 0, 3, 3, 3, 3, 3, 0, 0, 0, 3, 3, 3, 3, 3, 3, 3, 0, 3,  0, 0,  1 },
/* ZW */ { 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1,  1, 0,  1 },
/* edg*/ { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  0, 0,  0 }
};

#define WB_NOBREAK                        0
#define WB_BREAKABLE                      1
#define WB_hs_then_hs                     2
#define WB_Ex_or_FO_or_ZWJ_then_foo       3
#define WB_DQ_then_HL                     4
#define WB_HL_then_DQ                     6
#define WB_LE_or_HL_then_MB_or_ML_or_SQ   8
#define WB_MB_or_ML_or_SQ_then_LE_or_HL  10
#define WB_MB_or_MN_or_SQ_then_NU        12
#define WB_NU_then_MB_or_MN_or_SQ        14
#define WB_RI_then_RI                    16

static const U8 WB_table[24][24] = {
   /* 'Ext' stands for 'Extend'; 'edg' stands for 'EDGE'; 'hs' stands for
    * 'Perl_Tailored_HSpace'; 'unk' stands for 'UNKNOWN' */
/*        XX LE CR DQ EB EBG EM Ext EX FO GAZ HL KA LF ML MN MB NL NU hs RI SQ ZWJ edg */
/* XX */ { 1, 1, 1, 1, 1,  1, 1,  0, 1, 0,  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  0,  1 },
/* LE */ { 1, 0, 1, 1, 1,  1, 1,  0, 0, 0,  1, 0, 1, 1, 9, 1, 9, 1, 0, 1, 1, 9,  0,  1 },
/* CR */ { 1, 1, 0, 1, 1,  1, 1,  1, 1, 1,  1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1,  1,  1 },
/* DQ */ { 1, 1, 1, 1, 1,  1, 1,  0, 1, 0,  1, 5, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  0,  1 },
/* EB */ { 1, 1, 1, 1, 1,  1, 0,  0, 1, 0,  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  0,  1 },
/* EBG*/ { 1, 1, 1, 1, 1,  1, 0,  0, 1, 0,  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  0,  1 },
/* EM */ { 1, 1, 1, 1, 1,  1, 1,  0, 1, 0,  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  0,  1 },
/* Ext*/ { 3, 3, 1, 3, 3,  3, 3,  0, 3, 0,  3, 3, 3, 1, 3, 3, 3, 1, 3, 1, 3, 3,  0,  1 },
/* EX */ { 1, 0, 1, 1, 1,  1, 1,  0, 0, 0,  1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1,  0,  1 },
/* FO */ { 3, 3, 1, 3, 3,  3, 3,  0, 3, 0,  3, 3, 3, 1, 3, 3, 3, 1, 3, 1, 3, 3,  0,  1 },
/* GAZ*/ { 1, 1, 1, 1, 1,  1, 1,  0, 1, 0,  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  0,  1 },
/* HL */ { 1, 0, 1, 7, 1,  1, 1,  0, 0, 0,  1, 0, 1, 1, 9, 1, 9, 1, 0, 1, 1, 8,  0,  1 },
/* KA */ { 1, 1, 1, 1, 1,  1, 1,  0, 0, 0,  1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1,  0,  1 },
/* LF */ { 1, 1, 0, 1, 1,  1, 1,  1, 1, 1,  1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1,  1,  1 },
/* ML */ { 1,11, 1, 1, 1,  1, 1,  0, 1, 0,  1,11, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  0,  1 },
/* MN */ { 1, 1, 1, 1, 1,  1, 1,  0, 1, 0,  1, 1, 1, 1, 1, 1, 1, 1,13, 1, 1, 1,  0,  1 },
/* MB */ { 1,11, 1, 1, 1,  1, 1,  0, 1, 0,  1,11, 1, 1, 1, 1, 1, 1,13, 1, 1, 1,  0,  1 },
/* NL */ { 1, 1, 0, 1, 1,  1, 1,  1, 1, 1,  1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1,  1,  1 },
/* NU */ { 1, 0, 1, 1, 1,  1, 1,  0, 0, 0,  1, 0, 1, 1, 1,15,15, 1, 0, 1, 1,15,  0,  1 },
/* hs */ { 1, 1, 0, 1, 1,  1, 1,  0, 1, 0,  1, 1, 1, 0, 1, 1, 1, 0, 1, 2, 1, 1,  0,  1 },
/* RI */ { 1, 1, 1, 1, 1,  1, 1,  0, 1, 0,  1, 1, 1, 1, 1, 1, 1, 1, 1, 1,16, 1,  0,  1 },
/* SQ */ { 1,11, 1, 1, 1,  1, 1,  0, 1, 0,  1,11, 1, 1, 1, 1, 1, 1,13, 1, 1, 1,  0,  1 },
/* ZWJ*/ { 3, 3, 1, 3, 3,  0, 3,  0, 3, 0,  0, 3, 3, 1, 3, 3, 3, 1, 3, 1, 3, 3,  0,  1 },
/* edg*/ { 1, 1, 1, 1, 1,  1, 1,  1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  1,  0 }
};

#endif	/* defined(PERL_IN_REGEXEC_C) */

/* Generated from:
 * 3d90f60be77f44dea803ca765001cbcbe92d7e0044c7ebbbd120893410ce7bd7 lib/Unicode/UCD.pm
 * 47cb62a53beea6d0263e2147331c7e751853c9327225d95bbe2d9e1dc3e1aa44 lib/unicore/ArabicShaping.txt
 * 153f0a100c315f9f3945e78f57137611d36c44b3a975919c499fd403413fede8 lib/unicore/BidiBrackets.txt
 * fbe806975c1bf9fc9960bbaa39ff6290c42c7da8315f9cd459109b024cc1c485 lib/unicore/BidiMirroring.txt
 * 612127d4889032e55d82522e4a0c19793bda8aa8da14ecb3c696d17c83e6be13 lib/unicore/Blocks.txt
 * 37d40cf8c2c35637f4a04e746814e1fc4eb764c272bed9238a87ee96a4866857 lib/unicore/CaseFolding.txt
 * 5623df16856ad4007c60bdfff6f054e087521becd24cb4006be69c3a1d851aee lib/unicore/CompositionExclusions.txt
 * 5cb15b04693c43df16e0d304deca049e93b001445d163184e0ff1b7c8c852146 lib/unicore/DAge.txt
 * 6662c7e30b572df5d948c092692f52bcc79ab36d49a063a73d6435042db6fb3b lib/unicore/DCoreProperties.txt
 * 02d8e12cea7c61a9f3cc5fbf8eaceadf8da23fe83d60c64cf647088830b810cc lib/unicore/DNormalizationProps.txt
 * 3382cb4980e0021e9d4312f2d099315cfab6100ce0ff63a22d6937bfa720bcb7 lib/unicore/EastAsianWidth.txt
 * cb02ff25c375e78c82afffc852180eae7fc17c2f4434e697107720d05c52b26e lib/unicore/HangulSyllableType.txt
 * 850043bc9db08eddb271e43ed9176b865df76745a5fb6e9a4a1d9e54e5036471 lib/unicore/IndicPositionalCategory.txt
 * 90a3a00a5f5810850986c9279849b03c2f3133cfeeefb0554e851dd5ad7324dc lib/unicore/IndicSyllabicCategory.txt
 * bbd36a8d27f76e0fe0f2b0ca2827ac80da4b4a8d5247f0b0670fcaaac68ae6e1 lib/unicore/Jamo.txt
 * e2698584982ccd96e0c688bbcd4d2c48a23805baa0a0084388ef2e50ebd30aad lib/unicore/LineBreak.txt
 * 9d5353b1d318e72a63a49aa6a6a051d350382ea44b2cf31fe925bb94ce7e129b lib/unicore/NameAliases.txt
 * 009477f536f63016d901aeb1a4c820ad291e30e72ee9eb9ac1dfb6294a8a9bcc lib/unicore/NamedSequences.txt
 * f413ea8dbd3858de72f3148b47dd0586019761357d1481e3b65f3a025bc27f82 lib/unicore/PropList.txt
 * 56582d9875c1997876cf09305ccfc83448195de5d84d6374c78efaffeca4dfd7 lib/unicore/PropValueAliases.txt
 * 047ab0f815b6d1b30a863d6fcbd49e6358e168a3bd6c7189e80ab9863957d222 lib/unicore/PropertyAliases.txt
 * cde011921972dffe3e30dcbc7afbf069bf2eb01269097111dd7533b4c524caac lib/unicore/ScriptExtensions.txt
 * fba415952f5654145acad220dc2b878f815c673474d2bb4928934e3ba6ccca1d lib/unicore/Scripts.txt
 * dfc4f159c5c68328114ff17cd520451714a72ff48657287e5fe2f64344980695 lib/unicore/SpecialCasing.txt
 * 68dfc414d28257b9b5d6ddbb8b466c768c00ebdf6cbf7784364a9b6cad55ee8f lib/unicore/UnicodeData.txt
 * c76e7ed44a602f85161758ec54d2bcbcb7baf38f97720605a3e81ce02af47061 lib/unicore/auxiliary/GCBTest.txt
 * 4bb8931857e0a698fd2ec4a51a84c6de33e48a50d8b4bf0b57d960c41d77a191 lib/unicore/auxiliary/GraphemeBreakProperty.txt
 * 0c609ae4a32a89905c004010ef29885df3c60a941311a1a6e16003333d7fcdc8 lib/unicore/auxiliary/LBTest.txt
 * 26afdd5f38561167807b0f191af55b05761029ae6be2b7fe85b88f28effc66b5 lib/unicore/auxiliary/SBTest.txt
 * 53982766eeaf26674e7e2ec956c52846d52992ea9f4f1a202d488fb5cd76dd10 lib/unicore/auxiliary/SentenceBreakProperty.txt
 * 7f60beacc5ec5671edb56db36defc90cfc0016de4fd2bdfde7a3e4ec209840ce lib/unicore/auxiliary/WBTest.txt
 * cb2db065c77287e0f1d35b8c9b473d848b7566a1670439f67c357ca393084043 lib/unicore/auxiliary/WordBreakProperty.txt
 * 73573a4bee7f7377d936bc1710cd578001d00ba516e72ee54e906ae51a8c1fcb lib/unicore/extracted/DBidiClass.txt
 * ae8f55d8bb2bb69a63f70eef38cf5921fc3ba8e06d4d88f37950d32313a14754 lib/unicore/extracted/DBinaryProperties.txt
 * cf263ac4848d47264300139f97ed15c2c22896d2cef411483cbe292098c08bea lib/unicore/extracted/DCombiningClass.txt
 * 5bac89caf8d1fa23fcbbbf4d49ea0699b680ef1a73cbfa0daaac38c993906a74 lib/unicore/extracted/DDecompositionType.txt
 * e17138d36cd91f0e0d16437780a887c5ac637bde0de370b96f08a3424fcc322f lib/unicore/extracted/DEastAsianWidth.txt
 * fad25c183f4d439622c6be33c22ed709cdc4ac19feef0250527cf319a06952fe lib/unicore/extracted/DGeneralCategory.txt
 * c0d080b188fbe67dead10a3897dd2ca23da862c7f0b03508a92dcf27cd7e1f41 lib/unicore/extracted/DJoinGroup.txt
 * 3a8f8642084ed23b0de52af72dea0ff3d0b6928e6c7442832c663899e2b85e9d lib/unicore/extracted/DJoinType.txt
 * 37f6186253da9824bdb27f4ad867bfe8c25d4dc6bdb2f05585e40a034675a348 lib/unicore/extracted/DLineBreak.txt
 * ef24061b5a5dc93d7e90c2e34530ec757180ee75d872cba65ffc946e52624ae8 lib/unicore/extracted/DNumType.txt
 * a197371fec9a1b517058b440841f60f9378d81682084eef8db22a88cb2f96e90 lib/unicore/extracted/DNumValues.txt
 * 4e326cd667e6212b35f1786d22970d19f1e85ec6622156cfd86febd5bb18fe50 lib/unicore/mktables
 * cdecb300baad839a6f62791229f551a4fa33f3cbdca08e378dc976466354e778 lib/unicore/version
 * 913d2f93f3cb6cdf1664db888bf840bc4eb074eef824e082fceda24a9445e60c regen/charset_translations.pl
 * 9534d0cc3914fa1f5d574332c3199605c3d14f8691a0729d68d8498ac2b36280 regen/mk_invlists.pl
 * ex: set ro: */
